/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import priorseditor.engine.protocol.Protocol;

/**
 * This class represents an compound task wrapping several Executable tasks that should be executed in order
 * 
 * @author kjetikl
 */
public class CompoundTask extends ExecutableTask implements PropertyChangeListener {
    protected ArrayList<ExecutableTask> tasklist;
    protected float overallProgress=0;
    protected int currentTaskNumber=1; // the number (in order) of the OperationTask currently executing (starting at 1)
    
    public CompoundTask(String taskName) {
        super(taskName);
        tasklist=new ArrayList<ExecutableTask>();
    }
        
    /**
     * Adds a new task to this compound. New tasks can only be added before the
     * compound task has started execution 
     * @return true if the OperationTask was successfully added or false if not
     */
    public boolean addTask(ExecutableTask task) {
        if (getStatus().equals(PENDING)) {
            return tasklist.add(task);
        } else return false;
    }

    public void propertyChange(PropertyChangeEvent evt) {
             if (evt.getPropertyName().equals(STATUS_MESSAGE)) setStatusMessage((String)evt.getNewValue());
        else if (evt.getPropertyName().equals(STATUS)) {
            if (((String)evt.getNewValue()).equals(ABORTED)) setStatus(ABORTED);
        }
        else if (evt.getPropertyName().equals(PROGRESS)) {
            float segment=100.0f/(tasklist.size()*1.0f);
            int completed=currentTaskNumber-1;
            float operationProgress=(float)((Integer)evt.getNewValue()).intValue();
            overallProgress=(segment*completed)+(segment*operationProgress/100.0f);
            setProgress((int)overallProgress);            
        }
    }   
    
    @Override
    public void run() throws InterruptedException, Exception { // this method will throw all exception back to the caller since none are caught here
        setStatus(RUNNING); 
        overallProgress=0;
        setProgress((int)overallProgress);
        if (undoMonitor!=null) undoMonitor.register();
        int size=tasklist.size();
        ExecutableTask nextTask=null;
        try {
          for (int i=0;i<size;i++) {  
            checkExecutionLock();
            nextTask=tasklist.get(i);
            currentTaskNumber=i+1;
            nextTask.addPropertyChangeListener(this);
            nextTask.setExecutionLock(getExecutionLock());
            nextTask.setPriorsEditorClient(getPriorsEditorClient());
            nextTask.run();
            nextTask.setPriorsEditorClient(null);
            nextTask.setExecutionLock(null);
            nextTask.removePropertyChangeListener(this);
          }
        } finally { // uncaught exception are thrown back to the caller, but first we clean up!
            if (undoMonitor!=null) undoMonitor.deregister(true); // true=> register final state to allow redo
            nextTask.setPriorsEditorClient(null);
            nextTask.removePropertyChangeListener(this); 
            nextTask.setExecutionLock(null);
        }       
        if (undoMonitor!=null) undoMonitor.deregister(true);  
        setStatus(DONE);
    }




    @Override
    public String getCommandString(Protocol protocol) {
        if (protocol==null) return null;
        else return protocol.getCommandString(this);
    }    
    
    /** Returns the list of tasks wrapped by this CompoundTask */
    public ArrayList<ExecutableTask> getTaskList() {
        return tasklist;
    }
    
}


