/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine;

import priorseditor.engine.data.Motif;

/**
 * This compares two motifs using Pearsons Correlation
 * 
 * @author kjetikl
 */
public class MotifComparator_PearsonsCorrelation implements MotifComparator {
    private int minimumOverlap=3; // minimum number of overlapping columns
    private float[] uniform=new float[]{0.25f,0.25f,0.25f,0.25f};

@Override
public double compareMotifs(Motif motifA, Motif motifB, boolean reverseB) {
    float[][] matrixA=motifA.getMatrixAsFrequencyMatrix(); // forces conversion to frequencies between 0 and 1.0
    float[][] matrixB=motifB.getMatrixAsFrequencyMatrix(); // forces conversion to frequencies between 0 and 1.0
    if (reverseB) matrixB=Motif.reverseComplementMatrix(matrixB);
    
    float[][] largermatrix=null;
    float[][] smallermatrix=null;    
    Motif largermotif=null;
    Motif smallermotif=null;
    if (matrixA.length>matrixB.length) {
        largermatrix=matrixA;
        smallermatrix=matrixB;
        largermotif=motifA;
        smallermotif=motifB;
    } else {
        largermatrix=matrixB;
        smallermatrix=matrixA;       
        largermotif=motifB;
        smallermotif=motifA;        
    }
//    System.err.println("----------------  "+smallermotif.getName()+"  vs  "+largermotif.getName()+"  ---------------");
    int start=minimumOverlap-smallermatrix.length;
    if (start>=0) return -10000; // overlap is smaller than minimumOverlap (return 'default' bad value, large negative but NOT -Inf)
    int end=largermatrix.length-minimumOverlap;
    double result=-Double.MAX_VALUE; // 
    for (int i=start;i<=end;i++) {
        double alignmentresult=compareAtAlignment(largermatrix,smallermatrix,i);
        if (alignmentresult>result) result=alignmentresult;       
    }
    return result;
}    


/**
 * 
 * @param largematrix
 * @param smallermatrix
 * @param startA The position int the large matrix where the start of the smaller matrix has been aligned (can be negative)
 * @return
 */
private double compareAtAlignment(float[][] largematrix,float[][] smallermatrix, int startA) {
    double result=0;
    int endsmallalignment=startA+smallermatrix.length-1;
    int end=(largematrix.length>endsmallalignment)?largematrix.length:endsmallalignment;
    int start=(startA<0)?startA:0;    
    for (int i=start;i<end;i++) { //
        int largermatrixpos=i;
        int smallermatrixpos=i-startA;
        // get correct column from each matrix based on alignment (use uniform distr. column, if position is outside respective matrices)
        float[] largermatrixColumn=(largermatrixpos>=0 && largermatrixpos<largematrix.length)?largematrix[largermatrixpos]:uniform;
        float[] smallermatrixColumn=(smallermatrixpos>=0 && smallermatrixpos<smallermatrix.length)?smallermatrix[smallermatrixpos]:uniform;        
        //double colres=PearsonsCorrelationCoefficent(largermatrixColumn,smallermatrixColumn);      
        double colres=PearsonsCorrelationCoefficent(largermatrixColumn,smallermatrixColumn);
        double IClarger=Motif.calculateColumnIC(largermatrixColumn[0], largermatrixColumn[1], largermatrixColumn[2], largermatrixColumn[3], false);
        double ICsmaller=Motif.calculateColumnIC(smallermatrixColumn[0], smallermatrixColumn[1], smallermatrixColumn[2], smallermatrixColumn[3], false);
        //System.err.println("["+largermatrixpos+"]("+largermatrixColumn[0]+","+largermatrixColumn[1]+","+largermatrixColumn[2]+","+largermatrixColumn[3]+") vs ["+smallermatrixpos+"]("+smallermatrixColumn[0]+","+smallermatrixColumn[1]+","+smallermatrixColumn[2]+","+smallermatrixColumn[3]+") res="+colres);
        //result+=colres;
        result+=colres*(IClarger*ICsmaller/4);
        //result+=PearsonsCorrelationCoefficent(largermatrixColumn,smallermatrixColumn);       
    }
    return result;
}


/** Returns the Pearson's correlation coefficient for the two given columns */
private double PearsonsCorrelationCoefficent(float[] columnA, float[] columnB) {
    double meanA=(columnA[0]+columnA[1]+columnA[2]+columnA[3])/4.0;
    double meanB=(columnB[0]+columnB[1]+columnB[2]+columnB[3])/4.0;
    double above=(columnA[0]-meanA)*(columnB[0]-meanB) + (columnA[1]-meanA)*(columnB[1]-meanB) + (columnA[2]-meanA)*(columnB[2]-meanB) + (columnA[3]-meanA)*(columnB[3]-meanB);
    double below=(Math.pow((columnA[0]-meanA),2)+Math.pow((columnA[1]-meanA),2)+Math.pow((columnA[2]-meanA),2)+Math.pow((columnA[3]-meanA),2)) * (Math.pow((columnB[0]-meanB),2)+Math.pow((columnB[1]-meanB),2)+Math.pow((columnB[2]-meanB),2)+Math.pow((columnB[3]-meanB),2));
    //System.err.print("above="+above+" below="+below);
    if (below==0) return 0;
    else return above/Math.sqrt(below);
}


}