/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine;

/**
 * This class represents the specification of a parameter, describing its name, its type, default value
 * and allowed values. A description of the parameter and its implication can also be provided
 * The class only represents a "template" for a parameter. The value itself is not stored in objects of 
 * this class.
 * Object of this class can be used to "export" information about the set of Parameters available for 
 * for instance DataFormats or external programs. 
 * @author kjetikl
 */
public class Parameter {
    private String name;
    private Class type;
    private Object defaultValue;
    private Object allowedValues;
    private String description;
    private boolean required=true;
    private boolean hidden=false;
    
    
    public Parameter(String name, Class type, Object defaultValue, Object allowedValues, String description) {
        this.name=name;
        this.type=type;
        this.defaultValue=defaultValue;
        this.allowedValues=allowedValues;
        this.description=description;        
    }
    
    public Parameter(String name, Class type, Object defaultValue, Object allowedValues, String description, boolean required, boolean hidden) {
        this.name=name;
        this.type=type;
        this.defaultValue=defaultValue;
        this.allowedValues=allowedValues;
        this.description=description;    
        this.required=required;
        this.hidden=hidden;
    }
    
    public String getName() {return name;}
    public Class getType() {return type;}
    public Object getDefaultValue() {return defaultValue;}
    public Object getAllowedValues() {return allowedValues;}
    public String getDescription() {return description;}
    public boolean isOptional() {return !required;}
    public boolean isRequired() {return required;}
    public boolean isHidden() {return hidden;}
}
