/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine;

import priorseditor.engine.data.Data;

/**
 * 
 * 
 * @author kjetikl
 */
public interface PriorsEditorClient extends MessageListener{

    /**
     * This method can be used to ask the client to provide a value for the specified data item
     * The value returned should be an altered copy of the original (a double buffer) not the 
     * original itself!  
     * @param item The data item (NOT null!)
     * @param message An optional message that the client can pass on to the user
     * @return
     */
    public Data promptValue(Data item, String message) throws ExecutionError;

    
    /**
     * This function should return true if an task should attempt
     * to retry executions that have failed because of unforseen errors
     * (such as network timeouts) or false if the task should abort the
     * attempt and throw an Exception instead.
     * @param The task that encountered the error
     * @param e The Exception which caused the execution to fail
     * @return true if an attempt should be made at reexecuting the task 
     * or false if the task should be aborted by throwing an exception
     */
    public boolean shouldRetry(ExecutableTask task, Exception e);
    
    /**
     * This function should return true if a task should rollback
     * all changes made during its execution upon encountering an error
     * or false if the operation should just abort without performing
     * cleanup.
     * @return true if rollback should be performed
     */
    public boolean shouldRollback();
    
    
    /**
     * This method is called by the engine if an uncaught exception happens
     * The client is asked to deal with the problem. If the problem is non-fatal
     * and the client is able to recover, a TRUE value should be returned. 
     * If the client is not able to recover, necessary clean-up should be performed
     * before returning FALSE. After which the engine will shut down (exit). 
     * 
     * @param e
     */
    public boolean handleUncaughtException(java.lang.Throwable e);
    
    /**
     * This method is called automatically from the engine to initalize the client 
     * after it has been installed!
     */
    public void initializeClient(PriorsEditorEngine engine);
    
    /**
     * This is called by the engine if the engine thinks it is time to exit
     * (for instance after catching an exception which can not be handled)
     */
    public void shutdown();
}
