/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * This class represents an executable task corresponding to a Protocol script.
 * 
 * @author kjetikl
 */
public class ProtocolTask extends CompoundTask implements PropertyChangeListener {
    
    public ProtocolTask(String taskName) {
        super("execution of "+taskName);
    }
    

    public void propertyChange(PropertyChangeEvent evt) {
             if (evt.getPropertyName().equals(STATUS_MESSAGE)) setStatusMessage((String)evt.getNewValue());
        else if (evt.getPropertyName().equals(STATUS)) {
            if (((String)evt.getNewValue()).equals(ABORTED)) setStatus(ABORTED);
            if (((String)evt.getNewValue()).equals(WAITING)) setStatus(WAITING);
        }
        else if (evt.getPropertyName().equals(PROGRESS)) {
            float segment=100.0f/(tasklist.size()*1.0f);
            int completed=currentTaskNumber-1;
            float operationProgress=(float)((Integer)evt.getNewValue()).intValue();
            overallProgress=(segment*completed)+(segment*operationProgress/100.0f);
            setProgress((int)overallProgress); 
        }
    }  

    @Override
    public void run() throws InterruptedException, Exception { // this method will throw all exception back to the caller since none are caught here
        setStatus(RUNNING); 
        overallProgress=0;
        setProgress((int)overallProgress);
        if (undoMonitor!=null) undoMonitor.register();
        int size=tasklist.size();
        ExecutableTask nextTask=null;
        try {
          for (int i=0;i<size;i++) {  
            if (Thread.interrupted() || getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            checkExecutionLock();
            nextTask=tasklist.get(i);
            currentTaskNumber=i+1;
            nextTask.addPropertyChangeListener(this);
            nextTask.setExecutionLock(getExecutionLock());
            nextTask.setPriorsEditorClient(getPriorsEditorClient());
            nextTask.run();
            nextTask.setPriorsEditorClient(null);
            nextTask.setExecutionLock(null);
            nextTask.removePropertyChangeListener(this);
          }
        } finally { // uncaught exception are thrown back to the caller, but first we clean up!
            if (undoMonitor!=null) undoMonitor.deregister(true); // true=> register final state to allow redo
            nextTask.setPriorsEditorClient(null);
            nextTask.removePropertyChangeListener(this); 
            nextTask.setExecutionLock(null);
        }       
        if (undoMonitor!=null) undoMonitor.deregister(true);  
        setProgress(100);
        setStatus(DONE);
        setStatusMessage("Protocol execution finished");
    }


}


