/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.data;

import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;

/**
 *
 * @author kjetikl
 */
public class MotifClassification {

private static String[] classStrings=null;
private static HashMap<String,String> classToNameMap=new HashMap<String,String>();
private static DefaultComboBoxModel comboboxModel=null;
private static DefaultTreeModel treeModel=null;

 public static String UNKNOWN_CLASS_LABEL="Unknown class";


/** Returns the class name for a given class string
 * E.g. the argument "1.2" will return "Helix-loop-helix factors (bHLH)"
 */
public static String getNameForClass(String classString) {
    return classToNameMap.get(classString);
}

/** Returns the level (as integer) for the class (as string)
 *  1 = Superclass (top level)
 *  2 = Class
 *  3 = Family
 *  4 = Subfamily
 */
public static int getClassLevel(String classString) {
    return classString.split("\\.").length;
}

/** Returns a list of all available classStrings*/
public static String[] getClassStrings() {
    return classStrings;
}

/** Returns a class string representing the parent level of the given string */
public static String getParentLevel(String classString) {
    int index=classString.lastIndexOf(".");
    if (index>=0) return classString.substring(0,index);
    else return null;

}

/**
 * Returns a list containing the incremental path for the given class string
 * starting from the top level. E.g. if the method is called with argument "2.1.1.12"
 * the return list will be ["2","2.1","2.1.1","2.1.1.12"]
 * @param classString
 * @return
 */
public static String[] getClassPath(String classString) {
    String[] split=classString.split("\\.");
    String[] result=new String[split.length];
    result[0]=split[0];
    for (int i=1;i<result.length;i++) {
        result[i]=result[i-1]+"."+split[i];
    }
    return result;
}

/**
 * Returns the levels for a given classString as a list of integers
 * (or null if the string could not be parsed properly)
 * E.g. if the method is called with argument "2.1.1.12"
 * the return list will be [2,1,1,12]
 * @param classString
 * @return
 */
public static int[] getClassPathAsNumbers(String classString) {
    String[] split=classString.split("\\.");
    int[] result=new int[split.length];
    try {
        for (int i=0;i<result.length;i++) {
            result[i]=Integer.parseInt(split[i]);
        }
    } catch (NumberFormatException e) {return null;}
    return result;
}

/** Returns a ComboboxModel containing the classStrings
 * The first item in the model is NULL which signifies "unknown" class
 */
public static DefaultComboBoxModel getClassComboBoxModel() {
    if (comboboxModel==null) {
           String[] combostring=new String[classes.length+1];
           for (int i=0;i<classes.length;i++) {
               combostring[i+1]=classes[i][0];
           }
           combostring[0]=null; // this signifies the unknown class
           comboboxModel=new DefaultComboBoxModel(combostring);
    }
    return comboboxModel;
}

/** Returns a TreeModel containing the classStrings
 * The first item in the model is NULL which signifies "unknown" class
 */
public static DefaultTreeModel getClassTreeModel() {
    if (treeModel==null) {
           DefaultMutableTreeNode root=new DefaultMutableTreeNode("root");
           treeModel=new DefaultTreeModel(root);
           root.add(new DefaultMutableTreeNode(null));
           int currentlevel=1;
           DefaultMutableTreeNode currentparent=root;
           for (String classString:classStrings) {
               DefaultMutableTreeNode newnode=new DefaultMutableTreeNode(classString);
               DefaultMutableTreeNode parent=findParentNode(root, classString);
               parent.add(newnode);
           }
    }
    return treeModel;
}


private static DefaultMutableTreeNode findParentNode(DefaultMutableTreeNode rootNode, String classString) {
   DefaultMutableTreeNode newnode=new DefaultMutableTreeNode(classString);
   int nodelevel=getClassLevel(classString);
   if (nodelevel==1) return rootNode;
   else {
       DefaultMutableTreeNode current=rootNode;
       String[] path=getClassPath(classString);
       for (int i=0;i<path.length-1;i++) {
           current=getChild(current,path[i]);
       }
       return current;
   }
}

private static DefaultMutableTreeNode getChild(DefaultMutableTreeNode parent, String value) {
    if (parent.isLeaf()) return null;
    for (int i=0;i<parent.getChildCount();i++) {
       DefaultMutableTreeNode child = (DefaultMutableTreeNode)parent.getChildAt(i);
       String childval=(String)child.getUserObject();
       if (childval!=null && childval.equals(value)) return child;
    }
    return null;
}

public static TreeNode[] getPathToClass(DefaultMutableTreeNode root, String classString) {
    String[] classpath = getClassPath(classString);
    DefaultMutableTreeNode current=root;
    for (int i=0;i<classpath.length;i++) {
        current=getChild(current, classpath[i]);
    }
    return current.getPath();
}

/** Returns true if the argument string represents a known class
 */
public static boolean isKnownClassString(String classString) {
    for (String s:classStrings) {
        if (classString.equals(s)) return true;
    }
    return false;
}


/**
 * Return a full description of all levels (also parent levels)
 * for a  given class string
 */
public static String getFullLevelsString(String classString) {
    if (classString==null || !isKnownClassString(classString)) return "Unknown class";
    String result="";
    String[] split=classString.split("\\.");
    if (split.length==4) {
        result=classString+"\t"+getNameForClass(classString);
    }
    if (split.length>=3) {
        String levelString=split[0]+"."+split[1]+"."+split[2];
        result=levelString+"\t"+getNameForClass(levelString)+"\n"+result;
    }
    if (split.length>=2) {
        String levelString=split[0]+"."+split[1];
        result=levelString+"\t"+getNameForClass(levelString)+"\n"+result;
    }
    result=split[0]+"\t"+getNameForClass(split[0])+"\n"+result;
    return result.trim();
}
/**
 * Return a full description of all levels (also parent levels)
 * for a  given class string
 */
public static String getFullLevelsStringAsHTML(String classString) {
    if (classString==null || !isKnownClassString(classString)) return "Unknown class";
    String result="";
    String[] split=classString.split("\\.");
    if (split.length==4) {
        result=classString+" &rarr; "+getNameForClass(classString);
    }
    if (split.length>=3) {
        String levelString=split[0]+"."+split[1]+"."+split[2];
        result=levelString+" &rarr; "+getNameForClass(levelString)+"<br>"+result;
    }
    if (split.length>=2) {
        String levelString=split[0]+"."+split[1];
        result=levelString+" &rarr; "+getNameForClass(levelString)+"<br>"+result;
    }
    result=split[0]+" &rarr; "+getNameForClass(split[0])+"<br>"+result;
    return "<html>"+result+"</html>";
}


/**
 * Returns a list containing level strings for all sublevels (at all levels) of the given classString (incuding the string itself)
 * In other words, it will return all those levels whose level string start with the given classString
 */
public static ArrayList<String> getAllSubLevels(String classString) {
    ArrayList<String> result=new ArrayList<String>();
    for (int i=0;i<classes.length;i++) {
        if (classes[i][0].startsWith(classString)) result.add(classes[i][0]);
    }
    return result;
}

/**
 * Returns a list containing level strings for all sublevels (at all levels) of the given classString (incuding the string itself)
 * In other words, it will return all those levels whose level string start with the given classString
 */
public static ArrayList<String> getAllSuperLevels(String classString) {
    ArrayList<String> result=new ArrayList<String>();
    for (int i=0;i<classes.length;i++) {
        if (classString.startsWith(classes[i][0])) result.add(classes[i][0]);
    }
    return result;
}

/**
 * Compares two classStrings and returns 0 if they are equal, -1 if classString1 should be sorted
 * before classString2 or +1 if classString2 should be sorted before classString1
 * @param classString1
 * @param classString2
 */
public static int compareClassStrings(String classString1, String classString2) {
    if (classString1==null && classString2==null) return 0;
    else if (classString1==null && classString2!=null) return 1;
    else if (classString1!=null && classString2==null) return -1;
    if (classString1.equals(classString2)) return 0;
    int[] path1=getClassPathAsNumbers(classString1);
    int[] path2=getClassPathAsNumbers(classString2);
    if (path1==null && path2==null) return 0;
    else if (path1==null && path2!=null) return 1;
    else if (path1!=null && path2==null) return -1;
    int levels=(path1.length<path2.length)?path1.length:path2.length;
    // they should have at least 1 level each at this point
    if (path1[0]==0 && path2[0]!=0) return 1;  // sort top-level 0 at the bottom
    if (path1[0]!=0 && path2[0]==0) return -1; // sort top-level 0 at the bottom
    for (int i=0;i<levels;i++) {
        if (path1[i]==path2[i]) continue;
        if (path1[i]<path2[i]) return -1; else return 1;
    }
    // similar levels up to length of shortest
         if (path1.length==path2.length) return 0;
    else if (path1.length<path2.length) return -1;
    else return 1;

}


private static String[][] classes = new String[][] {
{"1","Basic Domains"},
{"1.1","Leucine zipper factors (bZIP)"},
{"1.1.1","AP-1(-like) components"},
{"1.1.1.1","Jun"},
{"1.1.1.2","Fos"},
{"1.1.1.3","Maf"},
{"1.1.1.4","NF-E2"},
{"1.1.1.5","fungal AP-1-like factors"},
{"1.1.1.6","CRE-BP/ATF"},
{"1.1.1.0","Others"},
{"1.1.2","CREB"},
{"1.1.2.0","CREB"},
{"1.1.3","C/EBP-like factors"},
{"1.1.3.0","C/EBP-like factors"},
{"1.1.4","bZIP / PAR"},
{"1.1.4.0","bZIP / PAR"},
{"1.1.5","Plant G-box binding factors"},
{"1.1.5.1","CPRF-2 (\"V\")"},
{"1.1.5.2","EmBP-1 (\"E\")"},
{"1.1.5.3","HBP-1a (\"Q\")"},
{"1.1.5.4","TGA1a (\"L/M\")"},
{"1.1.5.5","TGA1b (\"R\")"},
{"1.1.6","ZIP only"},
{"1.1.6.0","ZIP only"},
{"1.1.0","Other bZIP factors"},
{"1.1.0.0","Other bZIP factors"},
{"1.2","Helix-loop-helix factors (bHLH)"},
{"1.2.1","Ubiquitous (class A) factors"},
{"1.2.1.0","Ubiquitous (class A) factors"},
{"1.2.2","Myogenic transcription factors"},
{"1.2.2.0","Myogenic transcription factors"},
{"1.2.3","Achaete-Scute"},
{"1.2.3.0","Achaete-Scute"},
{"1.2.4","Tal/Twist/Atonal/Hen"},
{"1.2.4.1","Lymphoid factors"},
{"1.2.4.2","Mesodermal Twist-like factors"},
{"1.2.4.3","HEN"},
{"1.2.4.4","Atonal"},
{"1.2.4.5","Pancreatic factors"},
{"1.2.5","Hairy"},
{"1.2.5.1","Hairy"},
{"1.2.5.2","Esp"},
{"1.2.5.3","Fungal regulators"},
{"1.2.6","Factors with PAS domain"},
{"1.2.6.0","Factors with PAS domain"},
{"1.2.7","INO"},
{"1.2.7.0","INO"},
{"1.2.8","HLH domain only"},
{"1.2.8.0","HLH domain only"},
{"1.2.0","Other bHLH factors"},
{"1.2.0.0","Other bHLH factors"},
{"1.3","Helix-loop-helix / leucine zipper factors (bHLH-ZIP)"},
{"1.3.1","Ubiquitous bHLH-ZIP factors"},
{"1.3.1.1","TFE3"},
{"1.3.1.2","USF"},
{"1.3.1.3","SREBP"},
{"1.3.1.4","AP-4"},
{"1.3.2","Cell-cycle controlling factors"},
{"1.3.2.1","Myc"},
{"1.3.2.2","Mad/Max"},
{"1.6","bHSH"},
{"1.6.1","AP-2"},
{"1.6.1.0","AP-2"},
{"2","Zinc-coordinating DNA-binding domains"},
{"2.1","Cys4 zinc finger of nuclear receptor type"},
{"2.1.1","Steroid hormone receptors (NR3)"},
{"2.1.1.1","Corticoid receptors (NR3C)"},
{"2.1.1.2","Progesterone receptor (NR3C)"},
{"2.1.1.3","Androgen receptor (NR3C)"},
{"2.1.1.4","Estrogen receptor (NR3A)"},
{"2.1.1.5","Estrogen related receptor (NR3B)"},
{"2.1.2","Thyroid hormone receptor-like factors (NR0, NR1, NR2, NR4, NR5, NR6)"},
{"2.1.2.1","Thyroid hormone receptors (NR1A)"},
{"2.1.2.2","Retinoic acid receptors (NR1B)"},
{"2.1.2.3","PPAR (NR1C)"},
{"2.1.2.4","Rev-Erb receptors (NR1D)"},
{"2.1.2.5","NR1E"},
{"2.1.2.6","ROR (NR1F)"},
{"2.1.2.8","EcR/LXR/FXR (NR1H)"},
{"2.1.2.9","VDR/PXR/CAR (NR1I)"},
{"2.1.2.10","NR1J"},
{"2.1.2.11","HNF-4 (NR2A)"},
{"2.1.2.12","Retinoid X receptors (NR2B)"},
{"2.1.2.13","NR2C"},
{"2.1.2.14","NR2D"},
{"2.1.2.15","NR2E"},
{"2.1.2.16","COUP (NR2F)"},
{"2.1.2.17","Neuron-derived orphan receptors (NR4A)"},
{"2.1.2.18","FTZ-F1 related factors (NR5A)"},
{"2.1.2.19","NR5B"},
{"2.1.2.20","germ cell derived nuclear receptors (NR6A)"},
{"2.1.2.21","Knirps (NR0A)"},
{"2.1.2.22","NR0B"},
{"2.2","diverse Cys4 zinc fingers"},
{"2.2.1","GATA-Factors"},
{"2.2.1.1","vertebral GATA-Factors"},
{"2.2.1.2","fungal metabolic regulators"},
{"2.2.2","Trithorax"},
{"2.2.2.0","Trithorax"},
{"2.2.0","Other factors"},
{"2.2.0.0","Other factors"},
{"2.3","Cys2His2 zinc finger domain"},
{"2.3.1","Ubiquitous factors"},
{"2.3.1.0","Ubiquitous factors"},
{"2.3.2","Developmental / cell cycle regulators"},
{"2.3.2.1","Egr/Krox"},
{"2.3.2.2","Krueppel-like"},
{"2.3.2.3","GLI-like"},
{"2.3.2.4","Gfi-like"},
{"2.3.2.0","Others"},
{"2.3.3","Metabolic regulators in fungi"},
{"2.3.3.0","Metabolic regulators in fungi"},
{"2.3.4","Large factors with NF-6B-like binding properties"},
{"2.3.5","Viral regulators"},
{"2.4","Cys6 cysteine-zinc cluster"},
{"2.4.1","Metabolic regulators in fungi"},
{"2.4.1.0","Metabolic regulators in fungi"},
{"2.5","Zinc fingers of alternating composition"},
{"2.5.1","Cx7Hx8Cx4C zinc fingers"},
{"2.5.1.0","Cx7Hx8Cx4C zinc fingers"},
{"2.5.2","Cx2Hx4Hx4C zinc fingers"},
{"2.5.2.0","Cx2Hx4Hx4C zinc fingers"},
{"3","Helix-turn-helix"},
{"3.1","Homeo domain"},
{"3.1.1","Homeo domain only"},
{"3.1.1.1","AbdB"},
{"3.1.1.2","Antp"},
{"3.1.1.3","Cad"},
{"3.1.1.4","Cut"},
{"3.1.1.5","Dll"},
{"3.1.1.6","Ems"},
{"3.1.1.7","En"},
{"3.1.1.8","Eve"},
{"3.1.1.9","Prd"},
{"3.1.1.10","HD-ZIP"},
{"3.1.1.11","H2.0"},
{"3.1.1.12","HNF1"},
{"3.1.1.13","Lab"},
{"3.1.1.14","Msh"},
{"3.1.1.15","NK-2/Nkx"},
{"3.1.1.16","Bcd"},
{"3.1.1.17","XANF"},
{"3.1.1.18","PBC"},
{"3.1.1.0","not assigned"},
{"3.1.2","POU domain factors"},
{"3.1.2.1","I"},
{"3.1.2.2","II"},
{"3.1.2.3","III"},
{"3.1.2.4","IV"},
{"3.1.2.5","V"},
{"3.1.2.6","VI"},
{"3.1.2.0","other POU factors"},
{"3.1.3","Homeo domain with LIM region"},
{"3.1.3.1","Homeo domain with LIM region"},
{"3.1.3.2","LIM-only transcription (co-)factors"},
{"3.1.4","homeo domain plus zinc finger motifs"},
{"3.1.4.0","homeo domain plus zinc finger motifs"},
{"3.2","Paired box"},
{"3.2.1","Paired plus homeo domain"},
{"3.2.1.0","Paired plus homeo domain"},
{"3.2.2","Paired domain only"},
{"3.2.2.1","Pax-1 and homologs"},
{"3.2.2.2","Pax-2 and homologs"},
{"3.2.2.3","Pax-5"},
{"3.2.2.4","Pax-8"},
{"3.2.2.5","Pax-9"},
{"3.3","Fork head / winged helix"},
{"3.3.1","Developmental regulators"},
{"3.3.1.0","Developmental regulators"},
{"3.3.2","Tissue-specific regulators"},
{"3.3.2.0","Tissue-specific regulators"},
{"3.3.3","Cell-cycle controlling factors"},
{"3.3.3.1","vertebrate E2F"},
{"3.3.3.2","DP"},
{"3.3.3.3","insect E2F"},
{"3.3.3.4","fungal E2F"},
{"3.3.3.5","plant E2F"},
{"3.3.4","Regulators with a DNA recognition wing"},
{"3.3.4.1","RFX"},
{"3.3.0","Other regulators"},
{"3.3.0.0","Other regulators"},
{"3.4","Heat shock factors"},
{"3.4.1","HSF"},
{"3.4.1.0","HSF"},
{"3.5","Tryptophan clusters"},
{"3.5.1","Myb"},
{"3.5.1.1","Myb-factors"},
{"3.5.1.2","Myb-like factors"},
{"3.5.2","Ets-type"},
{"3.5.2.0","Ets-type"},
{"3.5.3","Interferon-regulating factors"},
{"3.5.3.0","Interferon-regulating factors"},
{"3.6","TEA domain"},
{"3.6.1","TEA"},
{"3.6.1.0","TEA"},
{"3.6.1.1","vertebrate TEA/ATTS, muscle-spesific"},
{"4","beta-Scaffold Factors with Minor Groove Contacts"},
{"4.1","RHR (Rel homology region)"},
{"4.1.1","Rel/ankyrin"},
{"4.1.1.0","Rel/ankyrin"},
{"4.1.2","ankyrin only"},
{"4.1.2.0","ankyrin only"},
{"4.1.3","NF-AT"},
{"4.1.3.0","NF-AT"},
{"4.2","STAT"},
{"4.2.1","STAT"},
{"4.2.1.0","STAT"},
{"4.3","p53"},
{"4.3.1","p53-like"},
{"4.3.1.1","p53"},
{"4.3.1.2","p63"},
{"4.3.1.3","p73"},
{"4.4","MADS box"},
{"4.4.1","Regulators of differentiation"},
{"4.4.1.1","MEF-2"},
{"4.4.1.2","homeotic genes"},
{"4.4.1.3","yeast regulators"},
{"4.4.2","Responders to external signals"},
{"4.4.2.0","Responders to external signals"},
{"4.4.3","Metabolic regulators"},
{"4.4.3.0","Metabolic regulators"},
{"4.5","beta-Barrel alpha-helix transcription factors"},
{"4.5.1","E2"},
{"4.5.1.0","E2"},
{"4.6","TATA-binding proteins"},
{"4.6.1","TBP"},
{"4.6.1.0","TBP"},
{"4.7","HMG"},
{"4.7.1","SOX"},
{"4.7.1.0","SOX"},
{"4.7.2","TCF-1"},
{"4.7.2.0","TCF-1"},
{"4.7.3","HMG2-related"},
{"4.7.3.0","HMG2-related"},
{"4.7.4","UBF"},
{"4.7.4.0","UBF"},
{"4.7.5","MATA"},
{"4.7.5.0","MATA"},
{"4.7.0","Other HMG box factors"},
{"4.7.0.0","Other HMG box factors"},
{"4.8","Histone fold"},
{"4.8.1","Heteromeric CCAAT factors"},
{"4.8.1.0","Heteromeric CCAAT factors"},
{"4.8.2","General transcription factors with histone fold"},
{"4.8.2.0","General transcription factors with histone fold"},
{"4.9","Grainyhead"},
{"4.9.1","LSF/CP2"},
{"4.9.1.0","LSF/CP2"},
{"4.9.2","Grainyhead"},
{"4.9.2.0","Grainyhead"},
{"4.10","Cold-shock domain factors"},
{"4.10.1","csd"},
{"4.10.1.1","A (DbpA-like)"},
{"4.10.1.2","B (YB-1/DbpB-like)"},
{"4.10.1.3","C (FRG Y2-like)"},
{"4.11","Runt"},
{"4.11.1","Runt"},
{"4.11.1.1","PEBP2alphaA"},
{"4.11.1.2","PEBP2alphaB"},
{"4.11.1.3","PEBP2alphaC"},
{"4.11.1.4","Runt"},
{"4.11.1.5","Lozenge"},
{"4.12","SMAD/NF-1"},
{"4.12.1","SMAD"},
{"4.12.1.1","receptor-regulated SMADs"},
{"4.12.1.2","common mediator SMADs"},
{"4.12.1.3","inhibitory SMADs"},
{"4.12.2","NF-1"},
{"4.12.2.0","NF-1"},
{"0","Other Transcription Factors"},
{"0.1","Copper fist proteins"},
{"0.1.1","Fungal regulators"},
{"0.1.1.0","Fungal regulators"},
{"0.2","HMGI(Y)"},
{"0.2.1","HMGI(Y)"},
{"0.2.1.0","HMGI(Y)"},
{"0.3","Pocket domain"},
{"0.3.1","Rb"},
{"0.3.1.0","Rb"},
{"0.3.2","CBP"},
{"0.3.2.0","CBP"},
{"0.4","E1A-like factors"},
{"0.4.1","E1A"},
{"0.4.1.0","E1A"},
{"0.5","AP2/EREBP-related factors"},
{"0.5.1","AP2"},
{"0.5.1.0","AP2"},
{"0.5.2","EREBP"},
{"0.5.2.0","EREBP"},
{"0.5.3","AP2/B3"},
{"0.5.3.0","AP2/B3"}
};


static {
   classStrings=new String[classes.length];
   for (int i=0;i<classes.length;i++) {
       classToNameMap.put(classes[i][0], classes[i][1]);
       classStrings[i]=classes[i][0];
   }
}


}
