/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.data;


/**
 *
 * @author kjetikl
 */
public class RegionDataset extends FeatureDataset implements Cloneable {
    private static String typedescription="Region dataset";
    private float maxScore=0;
    private boolean ismotiftrack=false;
    private boolean ismoduletrack=false;



    public RegionDataset(String datasetName) {
        super(datasetName);
    }

   /**
     * Returns the maximum score value for this dataset
     * @return the maximum score value
     */
    public Float getMaxScoreValue() {
        return maxScore;
    }

    /**
     * Specifies whether this RegionDataset represents a "motif track" where the region typenames
     * corresponds to motifs registered with the engine
     * @param ismotiftrack
     */
    public void setMotifTrack(boolean ismotiftrack) {
        this.ismotiftrack=ismotiftrack;
    }

    /** Returns TRUE if this RegionDataset contains motifs (or transcription factors binding sites),
     *  in which case the typenames of the regions should match the names of motifs registered with the engine
     */
    public boolean isMotifTrack() {
        return ismotiftrack;
    }


    /**
     * Specifies whether this RegionDataset represents a "module track" where the region typenames
     * corresponds to modules registered with the engine
     * @param ismoduletrack
     */
    public void setModuleTrack(boolean ismoduletrack) {
        this.ismoduletrack=ismoduletrack;
    }

    /** Returns TRUE if this RegionDataset contains modules (or composite TF binding sites)
     *  in which case the typenames of the regions should match the names of modules registered with the engine
     */
    public boolean isModuleTrack() {
        return ismoduletrack;
    }


   /**
     * Sets the maximum score value for this dataset
     * @param max The maximum score value for this dataset
     */
    public void setMaxScoreValue(float max) {
        this.maxScore=max;
    }

    /**
     * Returns a deep copy of this object
     */
    @Override
    public RegionDataset clone() {
        RegionDataset newdataset=new RegionDataset(datasetName); // creates a new dataset with the same name and empty storage
        newdataset.datasetDescription=this.datasetDescription;
        int size=getNumberofSequences();
        newdataset.maxScore=this.maxScore;
        newdataset.ismotiftrack=this.ismotiftrack;
        for (int i=0;i<size;i++) {
            RegionSequenceData seq=(RegionSequenceData)getSequenceByIndex(i);
            newdataset.addSequence((RegionSequenceData)seq.clone());
        }
        return newdataset;
    }

   @Override
    public void importData(Data source) throws ClassCastException {
        super.importData(source);
        this.maxScore=((RegionDataset)source).maxScore;
        this.ismotiftrack=((RegionDataset)source).ismotiftrack;
        //notifyListenersOfDataUpdate();
    }

    /**
     * clears all regions in this dataset
     *
     */
    public void clearRegions() {
        for (int i=0;i<getNumberofSequences();i++) {
            RegionSequenceData seq=(RegionSequenceData)getSequenceByIndex(i);
            seq.clearRegions();
        }
    }


    public static String getType() {return typedescription;}

    @Override
    public String getTypeDescription() {return typedescription;}

    @Override
    public boolean containsSameData(Data data) {
        if (data==null || !(data instanceof RegionDataset)) return false;
        RegionDataset other=(RegionDataset)data;
        if (this.ismotiftrack!=other.ismotiftrack) return false;
        if (this.ismoduletrack!=other.ismotiftrack) return false;
        for (FeatureSequenceData sequence:getAllSequences()) {
            if (!sequence.containsSameData(other.getSequenceByName(sequence.getName()))) return false;
        }
        return false;
    }

}
