/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.data;

import java.util.ArrayList;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.protocol.ParseError;

/**
 *
 * @author kjetikl
 */
public class StringVariable extends VariableData {
    private String name; 
    private String value="";
    private static String typedescription="String Variable";
    private static char[] illegal=new char[]{'\"'}; // list of characters that are not allowed in StringVariables
    
    public StringVariable(String name, String value) {
        this.name=name;
        this.value=value;
    }
    
    @Override
    public String getName() {return name;}

    @Override
    public void rename(String newname) {
        this.name=newname;
    }     
    
    @Override
    public String getValue() {return value;}
    
    @Override
    public String getValueAsParameterString() {return value;}   
    

    public void setValue(String newvalue) {
        value=newvalue;
        notifyListenersOfDataUpdate();
    }
    
    @Override
    public void importData(Data source) throws ClassCastException {
        StringVariable datasource=(StringVariable)source;
        this.name=datasource.name;
        this.value=datasource.value;      
        //notifyListenersOfDataUpdate(); 
    }
    
    @Override
    public StringVariable clone() {      
        StringVariable newdata=new StringVariable(name, value);
        return newdata;
    }      
    
    @Override
    public String getTypeDescription() {return typedescription+" = "+value;}
    

    public static String getType() {return typedescription;}

    @Override
    public boolean containsSameData(Data data) {
        if (data==null || !(data instanceof StringVariable)) return false;
        return value.equals(((StringVariable)data).value);
    }

    @Override
    public String output() {
        return value;
    }    
    
    @Override
    public void inputFromPlain(ArrayList<String> input, PriorsEditorEngine engine) throws ParseError {
        if (input==null || input.isEmpty()) throw new ParseError("PLAIN format input for '"+getName()+"' is empty");
        setValue(input.get(0));
    }  
    
    /** 
     * This method checks whether the supplied string object is 
     * contains a value that is valid for StringVariables.
     * For instance, StringVariables can not contain NULL or empty
     * values and are not allowed to contain certain special character 
     * like for instance a comma.
     */
    public static boolean isValidValue(String string) {
         // if (string==null || string.isEmpty()) return false;
         for (int i=0;i<illegal.length;i++) {
             if (string.contains(""+illegal[i])) return false;
         }
         return true;
    }
    
    /** Returns a String containing all characters that are not allowed in StringVariables*/
    public static String illegalCharacters() {
         String result="";
         for (int i=0;i<illegal.length;i++) {
             result+=illegal[i];
         }
         return result;
    }
}

