/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.dataformat;


import java.util.ArrayList;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.OutputData;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.data.DataSegment;
import priorseditor.engine.data.NumericMapVariable;
import priorseditor.engine.data.NumericVariable;
import priorseditor.engine.data.SequenceCollection;
import priorseditor.engine.data.StringCollection;
import priorseditor.engine.data.StringVariable;
import priorseditor.engine.protocol.ParseError;


/**
 * A Simple 'default' DataFormat that can format any Data object in its 'standard plain format'
 * simply by calling the data-objects output() method
 * @author kjetikl
 */
public class DataFormat_Plain extends DataFormat {
    private String name="Plain";
    private Class[] supportedTypes=new Class[]{Data.class};
        
    public DataFormat_Plain() {}
        
    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public boolean canFormatOutput(Data data) {
        return true;
    }
    
    @Override
    public boolean canFormatOutput(Class dataclass) {
        return true;
    }
    
    @Override
    public boolean canParseInput(Data data) {
        return (data instanceof SequenceCollection || data instanceof StringVariable || data instanceof StringCollection || data instanceof NumericVariable || data instanceof NumericMapVariable);
    }
    
    @Override
    public boolean canParseInput(Class dataclass) {
        return (dataclass.equals(SequenceCollection.class) || dataclass.equals(StringVariable.class) || dataclass.equals(StringCollection.class) || dataclass.equals(NumericVariable.class) || dataclass.equals(NumericMapVariable.class));

    }
    
    @Override
    public Class[] getSupportedDataTypes() {
        return supportedTypes;
    }

    @Override
    public String getSuffix() {
        return "txt";
    }
 
    @Override
    public OutputData format(Data dataobject, OutputData outputobject, ParameterSettings settings, ExecutableTask task) throws ExecutionError, InterruptedException {
        if (!canFormatOutput(dataobject)) {
            if (task!=null) throw new ExecutionError("Unable to format object '"+dataobject+"' in "+name+" format",task.getLineNumber());
            else throw new ExecutionError("Unable to format object '"+dataobject+"' in "+name+" format");
        }        
        String text=dataobject.output().trim();
        outputobject.append(text+"\n",getName());
        return outputobject;
    }    
           
    @Override
    public Data parseInput(ArrayList<String> input, Data target, ParameterSettings settings, ExecutableTask task) throws ParseError, InterruptedException {        
         target.inputFromPlain(input, engine);
         return target;
    }
    
    @Override
    public DataSegment parseInput(ArrayList<String> input, DataSegment target, ParameterSettings settings, ExecutableTask task) throws ParseError, InterruptedException {
         throw new ParseError("Unable to parse input to DataSegment in Plain format)");
    }
   
}

        
       
        
        
