/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.dataformat;

import java.util.ArrayList;
import java.util.List;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.OutputData;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.protocol.ParseError;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.MotifCollection;

/**
 *
 * @author kjetikl
 */
public class DataFormat_PriorsEditorMotif extends DataFormat {
    private String name="PriorsEditorMotif";
    
    private Class[] supportedTypes=new Class[]{MotifCollection.class, Motif.class};

    
    public DataFormat_PriorsEditorMotif() {
 
    }
        
    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public boolean canFormatOutput(Data data) {
        return (data instanceof MotifCollection || data instanceof Motif);
    }
    
    @Override
    public boolean canFormatOutput(Class dataclass) {
        return (dataclass.equals(MotifCollection.class) || dataclass.equals(Motif.class));
    }
    
    @Override
    public boolean canParseInput(Data data) {
        return (data instanceof MotifCollection || data instanceof Motif);
    }
    
    @Override
    public boolean canParseInput(Class dataclass) {
        return (dataclass.equals(MotifCollection.class) || dataclass.equals(Motif.class));
    }
    
      
    @Override
    public Class[] getSupportedDataTypes() {
        return supportedTypes;
    }
    
    @Override
    public String getSuffix() {
        return "pmx";
    }
 
    @Override
    public OutputData format(Data dataobject, OutputData outputobject, ParameterSettings settings, ExecutableTask task) throws ExecutionError, InterruptedException {
        if (!canFormatOutput(dataobject)) {
            if (task!=null) throw new ExecutionError("Unable to format object '"+dataobject+"' in "+name+" format",task.getLineNumber());
            else throw new ExecutionError("Unable to format object '"+dataobject+"' in "+name+" format");
        }        
        setProgress(5);
        StringBuilder outputString=new StringBuilder("#PriorsEditorMotif   (inspired by INCLUSive Motif Model v1.0)\n#\n");
        if (dataobject instanceof MotifCollection) {
            ArrayList<Motif> motiflist=((MotifCollection)dataobject).getAllMotifs(engine);
            int size=motiflist.size();
            int i=0;
            for (Motif motif:motiflist) {
                if (task!=null) task.checkExecutionLock(); // checks to see if this task should suspend execution
                if (Thread.interrupted() || (task!=null && task.getStatus().equals(ExecutableTask.ABORTED))) throw new InterruptedException();
                outputMotif(motif, outputString);
                // task.setStatusMessage("Motif "+(i+1)+" of "+size);
                setProgress(i+1, size);
                i++;
                if (i%30==0) Thread.yield();                                 
            } 
        } else if (dataobject instanceof Motif){
             outputMotif((Motif)dataobject, outputString);
        } else throw new ExecutionError("Unable to format object '"+dataobject+"' in "+name+" format");
        outputobject.append(outputString.toString(),getName());
        return outputobject;
    }    
    
    
    /** outputformats a single motif */
    private void outputMotif(Motif motif, StringBuilder outputString) {
            float[][] matrix=motif.getMatrix();
            if (matrix==null) return;
            outputString.append("#ID = ");outputString.append(motif.getName());outputString.append("\n");
            if (motif.getShortName()!=null) {outputString.append("#Short = ");outputString.append(motif.getShortName());outputString.append("\n");}
            if (motif.getLongName()!=null) {outputString.append("#Long = ");outputString.append(motif.getLongName());outputString.append("\n");}
            outputString.append("#W = ");outputString.append(motif.getLength());outputString.append("\n");
            if (motif.getConsensusMotif()!=null) {outputString.append("#Consensus = ");outputString.append(motif.getConsensusMotif());outputString.append("\n");}
            if (motif.getClassification()!=null) {outputString.append("#Class = ");outputString.append(motif.getClassification());outputString.append("\n");}
            outputString.append("#IC = ");outputString.append(motif.getICcontent());outputString.append("\n");
            if (motif.getQuality()!=6) {outputString.append("#Quality = ");outputString.append(motif.getQuality());outputString.append("\n");}
            if (motif.getBindingFactors()!=null) {outputString.append("#Factors = ");outputString.append(motif.getBindingFactors());outputString.append("\n");}
            if (motif.getOrganisms()!=null) {outputString.append("#Organisms = ");outputString.append(motif.getOrganisms());outputString.append("\n");}
            if (motif.isHalfsite()) {outputString.append("#Halfsite\n");}
            for (float[] row:matrix) {
              outputString.append(row[0]);
              outputString.append("\t");
              outputString.append(row[1]);
              outputString.append("\t");
              outputString.append(row[2]);
              outputString.append("\t");
              outputString.append(row[3]);
              outputString.append("\n");
            }
            outputString.append("\n");
    }

      

    @Override
    public Data parseInput(ArrayList<String> input, Data target, ParameterSettings settings, ExecutableTask task) throws ParseError, InterruptedException  {
        if (target==null) throw new ParseError("Unable to parse unknown (null) target Data in DataFormat_PriorsEditorMotif.parseInput(ArrayList<String> input, Data target)");
             if (target instanceof MotifCollection) return parseMotifCollection(input, (MotifCollection)target, task);
        else if (target instanceof Motif) return parseMotif(input, (Motif)target);
        else throw new ParseError("Unable to parse Motif input to target data of type "+target.getTypeDescription());
    }
    

    private Motif parseMotif(List<String> input, Motif target) throws ParseError {
       int expectedsize=0;
       String consensus="";
       String motifID="unknown";
       String motifShortName=null;
       String motifLongName=null;
       int quality=6;
       String classification=null;
       String factors=null;
       String organisms=null;
       boolean halfsite=false;
       if (target==null) target=new Motif("unknown");
       ArrayList<String>matrixlines=new ArrayList<String>();
       for (String line:input) {
           line=line.trim();
           if (line.isEmpty()) continue;
           else if (line.startsWith("#ID")) {
               String[] split=line.split("\\s*=\\s*",2);
               if (split.length==2) {motifID=split[1];}
           } else if (line.startsWith("#W")) {
               String[] split=line.split("\\s*=\\s*",2);
               if (split.length==2) {try {expectedsize=Integer.parseInt(split[1]);} catch(Exception e){ throw new ParseError("Unable to parse expected numeric value for matrix-length in '"+motifID+"':"+line);}}               
           } else if (line.startsWith("#Quality")) {
               String[] split=line.split("\\s*=\\s*",2);
               if (split.length==2) {try {quality=Integer.parseInt(split[1]);} catch(Exception e){ throw new ParseError("Unable to parse expected numeric value for matrix-quality in '"+motifID+"':"+line);}}              
           } else if (line.startsWith("#Consensus") || line.startsWith("#Conensus")) {
               String[] split=line.split("\\s*=\\s*",2);
               if (split.length==2) {consensus=split[1];}
           } else if (line.startsWith("#Short")) {
               String[] split=line.split("\\s*=\\s*",2);
               if (split.length==2) {motifShortName=split[1];}
           } else if (line.startsWith("#Long")) {
               String[] split=line.split("\\s*=\\s*",2);
               if (split.length==2) {motifLongName=split[1];}
           } else if (line.startsWith("#Class")) {
               String[] split=line.split("\\s*=\\s*",2);
               if (split.length==2) {
                   String[] splitSemiColon=split[1].split("\\s*;\\s*");
                   classification=splitSemiColon[0];
               }
           } else if (line.startsWith("#Factors")) {
               String[] split=line.split("\\s*=\\s*",2);
               if (split.length==2) {factors=split[1];}
           } else if (line.startsWith("#Organisms")) {
               String[] split=line.split("\\s*=\\s*",2);
               if (split.length==2) {organisms=split[1];}
           } else if (line.startsWith("#Halfsite") || line.startsWith("#Part = Halfsite")) {
               halfsite=true;
           } else if (line.startsWith("#")) {
               continue;
           } else matrixlines.add(line);
       }
       if (expectedsize>0 && matrixlines.size()!=expectedsize) throw new ParseError("Expected "+expectedsize+" matrix rows in Motif input. Got "+matrixlines.size());
       float[][] matrix=parseValueLines(matrixlines, motifID);
       if (consensus!=null) target.setConsensusMotif(consensus);
       else target.setConsensusMotif(Motif.getConsensusForMatrix(matrix));
       target.setName(motifID);
       target.setShortName(motifShortName);
       target.setLongName(motifLongName);
       target.setQuality(quality);
       target.setClassification(classification);
       target.setHalfsite(halfsite);
       target.setBindingFactors(factors);
       target.setOrganisms(organisms);
       target.setMatrix(matrix);
       return target;
    } 
                        
    
    private MotifCollection parseMotifCollection(List<String> input, MotifCollection target, ExecutableTask task) throws ParseError, InterruptedException {
        if (target==null) target=new MotifCollection("MotifCollection");
        int first=0; int last=0; int size=input.size();
        if (size<1) return target; //throw new ParseError("Empty input for MotifCollection");
        if (!input.get(0).startsWith("#PriorsEditorMotif")) throw new ParseError("Unrecognized header for PriorsEditor Motif format: "+input.get(0));
        int count=0;
        for (int i=1;i<size;i++) {
            String line=input.get(i).trim();
            if (line.startsWith("#ID")) {
                count++;
                if (count%30==0) {
                  if (task!=null) task.checkExecutionLock(); // checks to see if this task should suspend execution
                  if (Thread.interrupted() || (task!=null && task.getStatus().equals(ExecutableTask.ABORTED))) throw new InterruptedException();                
                  Thread.yield();
                }                 
                if (first==0) {
                    first=i;
                } else {
                    last=i;
                    Motif motif=parseMotif(input.subList(first, last), null);
                    target.addMotifToPayload(motif);
                    first=i;
                    last=0;
                }
            }
        }
        if (first>0) {
            Motif motif=parseMotif(input.subList(first, size), null);
            target.addMotifToPayload(motif);
        }
        return target;
    }
            
            
    
    /** */
    private float[][] parseValueLines(List<String> list, String motifID) throws ParseError {
        if (list.size()==0) throw new ParseError("Missing matrix specification values for matrix: "+motifID);
        float[][] result=new float[list.size()][4];
        for (int i=0;i<list.size();i++) {
            String line=list.get(i);
            String[] split=line.split("\\s+");
            if (split.length!=4) throw new ParseError("Expected 4 columns in data matrix '"+motifID+"'. Got "+split.length);
            for (int j=0;j<4;j++) {
                try {
                    result[i][j]=Float.parseFloat(split[j]);
                } catch (NumberFormatException e) {throw new ParseError("Unable to parse expected numeric value in matrix for '"+motifID+"' = " +split[j]);}
            }
        }
        return result;
    }    
    
}

        
       
        
        
