/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.datatracks;

import priorseditor.engine.DataLoader;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.data.DataSegment;
import priorseditor.engine.dataformat.DataFormat;

/**
 *
 * @author Kjetil Klepper 
 */
public abstract class DataSource implements Cloneable {
     public static final String PATTERN_TEMPLATE_CHROMOSOME="$CHROMOSOME";
     public static final String PATTERN_TEMPLATE_START="$START";
     public static final String PATTERN_TEMPLATE_END="$END";
     
     public static final String HTTP_GET="GET";
     public static final String HTTP_POST="POST";
     public static final String DAS_SERVER="DAS";
     public static final String TEST="TEST";
               
     protected String name;
     protected DataTrack dataTrack;
     protected int organism;  
     protected int delay=0;
     protected String genomebuild=null;
     protected String dataformatName=null;
     protected DataFormat dataformat=null;
     protected ParameterSettings dataformatSettings=null; // additional parameters for the DataFormat
     
     protected static DataLoader dataloader=null;
     
     private static int counter=1;
     private int myindex=0;
     
     public DataSource(DataTrack dataTrack, int organism, String genomebuild,String dataformatName) {
         this.dataTrack=dataTrack;
         this.name=dataTrack.getName();
         this.organism=organism;
         this.dataformatName=dataformatName;
         this.genomebuild=genomebuild;
         resolveDataFormat();
         myindex=counter++;
     }
     
     public static void setDataLoader(DataLoader loader) {
         dataloader=loader;
     }
     
     public static String[] getSupportedProtocols() {
         return new String[]{DAS_SERVER,HTTP_GET};
     }
     
     public DataTrack getDataTrack() {
         return dataTrack;
     }
     
     public String getName() {
         return name;
     }
     
     public int getOrganism() {
         return organism;
     }          
     
     public String getGenomeBuild() {
         return genomebuild;
     }
     
     public void setGenomeBuild(String build) {
         genomebuild=build;
     }
     public void setOrganism(int organism) {
         this.organism=organism;
     }
     
     public String getDataFormat() {
         return dataformatName;
     }
     
     public void setDataFormat(String dataformat) {
         this.dataformatName=dataformat;
     }
     
     public void setDataFormatSettings(ParameterSettings dataformatsettings) {
         dataformatSettings=dataformatsettings;
     }

     public ParameterSettings getDataFormatSettings() {
        return dataformatSettings;
     }

     /** Sets the internal DataFormat object reference based on the DataFormat name*/
     protected DataFormat resolveDataFormat() {
         if (dataformatName==null) return null;
         if (dataloader==null || dataloader.getEngine()==null) return null;
         dataformat=dataloader.getEngine().getDataFormat(dataformatName);
         return dataformat;
     }
 
     public abstract String getProtocol();
     
     /** 
      * Returns the server address (domain name or IP) for the internet server
      * associated with this DataSource (or null)
      */
     public abstract String getServerAddress();
     
    /** 
     * This method should be called whenever data is loaded from a DataSource 
     * It will store a timestamp for when the DataSource server was last accessed
     */
    public void notifyDataSourceAccess() {
        String serveraddress=getServerAddress();
        dataloader.notifyDataSourceAccess(serveraddress);
    }
    
    /** 
     * This method will return the number of milliseconds one should wait before accessing the server
     * for this DataSource again. The number is based on the time of last access (as registered by 
     * the last call to notifyDataSourceAccess()) and the "delay" property for this DataSource
     */
    public int getCurrentWaitingPeriod() {
        String serveraddress=getServerAddress();        
        return dataloader.getWaitingPeriodForServer(serveraddress);
    } 

     /**
      * Obtains data from the the genomic region corresponding to the given (empty) DataSegment object. 
      * The obtained data is stored in the DataSegment object (overwriting any previous content).
      * @param segment
      * @return
      * @throws priorseditor.engine.ExecutionError
      */
     public abstract DataSegment loadDataSegment(DataSegment segment, ExecutableTask task) throws Exception;

                          
     /** Returns an XML element for this DataSource that can be included in DataConfiguration XML-documents */
    public org.w3c.dom.Element getXMLrepresentation(org.w3c.dom.Document document) {
        org.w3c.dom.Element element = document.createElement("DataSource");
        element.setAttribute("organism", ""+organism);
        if (genomebuild!=null) element.setAttribute("build", genomebuild);
        return element;
    }

    @Override
    public abstract DataSource clone();
            
    
    public void debug() {
          System.err.println("   ["+myindex+"] "+priorseditor.engine.data.Organism.getCommonName(organism)+"  "+getProtocol()+"   "+getGenomeBuild());        
    }
}
