/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.datatracks;


import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.DNASequenceDataset;
import priorseditor.engine.data.DataSegment;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.NumericDataset;

/**
 * A DataSource objects that retrieves data from a DAS server.
 * The retrieved XML-document is then parsed by a DataFormat object
 * 
 * @author Kjetil Klepper
 */
public class DataSource_DAS extends DataSource {

    private String baseURL;
    private String featurename; // template
 
    /**
     * Creates a new instance of DataSource_http_GET based on the given arguments
     * @param name
     * @param organism
     * @param baseURL
     * @param dataFormatName
     * @param delay
     * @param parameterStringTemplate
     */
    public DataSource_DAS(DataTrack datatrack, int organism, String genomebuild, String baseURL, String dataFormatName, String featurename) {
        super(datatrack,organism,genomebuild,dataFormatName);
        this.baseURL=baseURL;
        this.featurename=featurename;   
    }
    
    @Override
    public String getServerAddress() {
        try {
            URL url=new URL(baseURL);
            String host=url.getHost();
            return host;
        } catch (MalformedURLException e) {return null;}
    }
    
    @Override
    public String getProtocol() {return DAS_SERVER;}
  
    public String getFeature() {
        return featurename;
    }
    public void setFeature(String feature) {
        this.featurename=feature;
    }
    public String getBaseURL() {
        return baseURL;
    }
    public void setBaseURL(String baseURL) {
        this.baseURL=baseURL;
    }
    
    @Override
    public DataSegment loadDataSegment(DataSegment segment, ExecutableTask task) throws Exception {        
        int timeout=dataloader.getEngine().getNetworkTimeout();
        String chromosome=segment.getChromosome();
        String start=""+segment.getSegmentStart();
        String end=""+segment.getSegmentEnd();
        if (!baseURL.startsWith("http://")) baseURL="http://"+baseURL;
        //dataformat=engine.getDataFormat(dataformatName); // this is not used!!!
        int waitperiod=getCurrentWaitingPeriod();
        if (waitperiod>0) {Thread.sleep(waitperiod);} // throws InterruptedException
        
        Object data=null;
        if (dataTrack.getDataType()==NumericDataset.class) {
            // This is not complete!!! Missing parser for DAS Numeric data. I guess that is OK for now since DAS specification 1 doesn't seem to include support for wiggle tracks
            data=null;
            throw new ExecutionError("SLOPPY PROGRAMMING ERROR: DAS protocol has not been implemented for Numeric Data");            
        } else if (dataTrack.getDataType()==RegionDataset.class) {
            String uri=baseURL+"?segment="+chromosome+":"+start+","+end+";type="+featurename;
            DASParser_Region parser=new DASParser_Region();
            ArrayList<Region> list=parser.parse(uri,timeout);
            for (Region reg:list) { // set correct relative coordinates
                reg.setRelativeStart(reg.getRelativeStart()-segment.getSegmentStart());
                reg.setRelativeEnd(reg.getRelativeEnd()-segment.getSegmentStart());
            }
            data=list;
        } else if (dataTrack.getDataType()==DNASequenceDataset.class) { 
            String uri=baseURL+"?segment="+chromosome+":"+start+","+end;
            DASParser_DNA parser=new DASParser_DNA();
            char[] buffer=parser.parse(uri,timeout); 
            data=buffer;
        }
        notifyDataSourceAccess(); 
        segment.setSegmentData(data);
        return segment;
    }    
    
    @Override
    public DataSource clone() {
        DataSource_DAS copy=new DataSource_DAS(dataTrack, organism, genomebuild, baseURL, dataformatName, featurename);
        copy.delay=this.delay;
        copy.dataformat=this.dataformat;
        copy.dataformatSettings=this.dataformatSettings;
        return copy;
    }  

    
    @Override
    public org.w3c.dom.Element getXMLrepresentation(org.w3c.dom.Document document) {
        org.w3c.dom.Element element = super.getXMLrepresentation(document);
        org.w3c.dom.Element protocol=document.createElement("Protocol");
        protocol.setAttribute("type", "DAS");
        org.w3c.dom.Element url=document.createElement("URL");
        url.setTextContent(baseURL);
        protocol.appendChild(url);
        if (featurename!=null) {
            org.w3c.dom.Element feature=document.createElement("Feature");
            feature.setTextContent(featurename);
            protocol.appendChild(feature);
        }
        element.appendChild(protocol);
        return element;
    }    
    
}
