/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import priorseditor.engine.OperationTask;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.data.NumericConstant;
import priorseditor.engine.data.NumericMapVariable;
import priorseditor.engine.data.NumericVariable;
import priorseditor.engine.data.RegionSequenceData;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.NumericSequenceData;
/**
 *
 * @author kjetikl
 */
public class Condition_position extends Condition {
    private boolean isResolved=false;
    
    public Condition_position() {
        super();
    }
    
    public Condition_position(HashMap<String,Object> initialValues) {
        super(initialValues);
    }      
    
    @Override
    public void resolve(PriorsEditorEngine engine, OperationTask task) throws ExecutionError {
        Data operandA=null;
        Data operandB=null;
        String operandAstring=getOperandAString();
        if (operandAstring!=null) {                     
           String operandBstring=getOperandBString();
           operandA=engine.getDataItem(operandAstring);                 
           if (operandA==null || !(operandA instanceof NumericDataset || operandA instanceof RegionDataset)) throw new ExecutionError("First operand '"+operandAstring+"' in where-clause is not a recognized Numeric or Region dataset",task.getLineNumber());
           if (operandBstring!=null) {
              operandB=engine.getDataItem(operandBstring);
              if (operandB==null) {
                    try {
                       float value=Float.parseFloat(operandBstring);
                       operandB=new NumericConstant(operandBstring, (float)value);
                    } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+operandBstring+"' neither data nor numeric constant",task.getLineNumber());}
              } // end operandB==null               
              if (operandA instanceof RegionDataset && !(operandB instanceof RegionDataset)) throw new ExecutionError("Second operand '"+operandBstring+"' in condition should match first operand (region data)",task.getLineNumber());
              if (operandA instanceof NumericDataset && !(operandB instanceof NumericDataset || operandB instanceof NumericVariable || operandB instanceof NumericMapVariable || operandB instanceof NumericConstant)) throw new ExecutionError("Second operand '"+operandBstring+"' in condition should match first operand (numerical)",task.getLineNumber());
           } // end operandBstring!=null 
        } // end whereString!=null        

        setOperandAData(operandA);
        setOperandBData(operandB);  
        //if (operandA!=null) System.err.println("OperandA resolved: "+operandA.getName()+" ["+operandA.getTypeDescription()+"]");
        //if (operandB!=null) System.err.println("OperandB resolved: "+operandB.getName()+" ["+operandB.getTypeDescription()+"]");
        isResolved=true;
    }    

    /**
     * Returns true if this Condition holds at the specified genomic position
     * @param sequencename
     * @param pos the Genomic position! where this condition should be tested
     * @param task
     * @return
     * @throws java.lang.Exception
     */
    protected boolean isConditionSatisfied(String sequencename, int pos, OperationTask task) throws Exception {
        if (!isResolved) {
            task.getEngine().reportError(new ExecutionError("SLOPPY PROGRAMMING ERROR: Condition_position is not 'resolved' before use"));
        }
        String whereString=getOperandAString();
        if (whereString==null) return true; // no condition is set, so the transform should be applied everywhere
        Data operandA=getOperandAData();
        Data operandB=getOperandBData();
        String comparator=getComparator();
        boolean whereNot=false;
        Boolean whereNotBoolean=negateAll();
        if (whereNotBoolean!=null) whereNot=whereNotBoolean.booleanValue();         
        boolean returnValue=true;
        if (operandA instanceof RegionDataset && operandB==null) {// --- single Region comparison
            RegionSequenceData a=(RegionSequenceData)((RegionDataset)operandA).getSequenceByName(sequencename);         
            if (!comparator.equals("inside")) {throw new Exception("Sloppy programming error: comparator is not 'inside' for single region comparison");}
            returnValue=(a.getNumberOfRegionsAtGenomicPosition(pos)>0);                                  
        } // --------------------------------------------------------
        else if (operandA instanceof RegionDataset && operandB!=null && operandB instanceof RegionDataset) { // --- compare Region to Region
            RegionSequenceData a=(RegionSequenceData)((RegionDataset)operandA).getSequenceByName(sequencename);         
            RegionSequenceData b=(RegionSequenceData)((RegionDataset)operandB).getSequenceByName(sequencename);         

            if (comparator.equals("bases overlap")) { 
                   returnValue=(a.getNumberOfRegionsAtGenomicPosition(pos)>0 && b.getNumberOfRegionsAtGenomicPosition(pos)>0);
            } else if (comparator.equals("bases not overlap")) { 
                   returnValue=(a.getNumberOfRegionsAtGenomicPosition(pos)>0 && b.getNumberOfRegionsAtGenomicPosition(pos)==0);
            } else if (comparator.equals("regions overlap")) { 
                   returnValue=false;
                   ArrayList<Region> regionsAtpos=a.getValueAtGenomicPosition(pos);
                   for (Region reg:regionsAtpos) {
                       if (b.getNumberOfRegionsOverlappingGenomicInterval(reg.getGenomicStart(), reg.getGenomicEnd())>0) {returnValue=true;break;}
                   }
            } else if (comparator.equals("regions not overlap")) { 
                   returnValue=false;
                   ArrayList<Region> regionsAtpos=a.getValueAtGenomicPosition(pos);
                   if (regionsAtpos.size()>0) {
                     returnValue=true;
                     for (Region reg:regionsAtpos) {
                       if (b.getNumberOfRegionsOverlappingGenomicInterval(reg.getGenomicStart(), reg.getGenomicEnd())>0) {returnValue=false;break;}
                     }
                   }
            } else throw new ExecutionError("comparator '"+comparator+"' not applicable here",task.getLineNumber());                            
         } // --------------------------------------------------------
         else if (operandA instanceof NumericDataset) { // 
            NumericSequenceData seqdata=(NumericSequenceData)((NumericDataset)operandA).getSequenceByName(sequencename);
            float valueA=seqdata.getValueAtGenomicPosition(pos).floatValue();
            float valueB=0;
                 if (operandB instanceof NumericDataset) {
                     NumericSequenceData seqdataB=(NumericSequenceData)((NumericDataset)operandB).getSequenceByName(sequencename);
                     valueB=seqdataB.getValueAtGenomicPosition(pos).floatValue();
                 }
            else if (operandB instanceof NumericSequenceData) valueB=((NumericSequenceData)operandB).getValueAtGenomicPosition(pos).floatValue();
            else if (operandB instanceof NumericConstant) valueB=((NumericConstant)operandB).getValue().floatValue();
            else if (operandB instanceof NumericMapVariable) valueB=((NumericMapVariable)operandB).getValue(sequencename).floatValue();
            else if (operandB instanceof NumericVariable) valueB=((NumericVariable)operandB).getValue().floatValue();
                 if (comparator.equals("="))  returnValue=(valueA==valueB);
            else if (comparator.equals(">=")) returnValue=(valueA>=valueB);
            else if (comparator.equals(">"))  returnValue=(valueA>valueB);
            else if (comparator.equals("<"))  returnValue=(valueA<valueB);
            else if (comparator.equals("<=")) returnValue=(valueA<=valueB);
            else if (comparator.equals("<>")) returnValue=(valueA!=valueB); 
            //System.err.println("At "+sequencename+"["+pos+"]=> "+valueA+""+comparator+""+valueB+"  => "+returnValue);
        } // --------------------------------------------------------
        else System.err.println("SYSTEM-ERROR in Condition_position: OperandA="+operandA);
        
        if (whereNot) return !returnValue;
        else return returnValue;
    } 
    
    @SuppressWarnings("unchecked")    
    @Override
    public Condition_position clone() {
        Condition_position newdata=new Condition_position((HashMap<String, Object>)storage.clone());
        return newdata;
    }   
}
