/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;
import priorseditor.engine.OperationTask;
import java.util.HashMap;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.MotifCollection;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.data.NumericConstant;
import priorseditor.engine.data.NumericMapVariable;
import priorseditor.engine.data.NumericVariable;
import priorseditor.engine.data.RegionSequenceData;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.StringCollection;
import priorseditor.engine.data.StringVariable;
import priorseditor.engine.data.NumericSequenceData;

/**
 *
 * @author kjetikl
 */
public class Condition_region extends Condition {
    private boolean isResolved=false;
    
    public Condition_region() {
        super();
    }
    public Condition_region(HashMap<String,Object> initialValues) {
        super(initialValues);
    }   
    
    public String getCompareProperty() {return (String)getParameter("compareProperty");}
    public void setCompareProperty(String compareProperty) {setParameter("compareProperty",compareProperty);}
    

    
    @Override
    public void resolve(PriorsEditorEngine engine, OperationTask task) throws ExecutionError {
        Data operandA=null;
        Data operandB=null;
        String operandAstring=getOperandAString();               
        String operandBstring=getOperandBString(); 
        String compareProperty=getCompareProperty();
        if (operandBstring==null) throw new ExecutionError("Missing operand in condition",task.getLineNumber());
        if (compareProperty==null) throw new ExecutionError("Unable to decide type of condition",task.getLineNumber());                     
        else if (compareProperty.equals("")) {}
        else if (compareProperty.equals("type")) {
             if (operandBstring.startsWith("\"") && operandBstring.endsWith("\"")) {
                 operandBstring=operandBstring.substring(1, operandBstring.length()-1); // removes quotes
                 if (operandBstring.isEmpty()) throw new ExecutionError("Empty text expression in comparison",task.getLineNumber());      
                 operandB=new StringVariable("noname", operandBstring);
             } else {
                operandB=engine.getDataItem(operandBstring);
                if (operandB==null) throw new ExecutionError("Unrecognized token '"+operandBstring+"' is not a data object",task.getLineNumber());      
                else if (!(operandB instanceof MotifCollection) && !(operandB instanceof StringCollection)) throw new ExecutionError("Operand '"+operandBstring+"' is not a valid Collection (Motif or String)",task.getLineNumber());      
             }
        } else if (compareProperty.equals("score") || compareProperty.equals("length")) {
              operandB=engine.getDataItem(operandBstring);
              if (operandB==null) {
                  try {
                     float value=Float.parseFloat(operandBstring);
                     operandB=new NumericConstant(operandBstring, (float)value);
                  } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+operandBstring+"' neither data nor numeric constant",task.getLineNumber());}
              } // end operandB==null               
              if (!(operandB instanceof NumericVariable) && !(operandB instanceof NumericMapVariable) && !(operandB instanceof NumericConstant)) throw new ExecutionError("Operand '"+operandBstring+"' should be of numerical type",task.getLineNumber());
        } else if (compareProperty.equals("distance to")) {
             operandA=engine.getDataItem(operandAstring);
             if (!(operandA instanceof RegionDataset)) throw new ExecutionError("Operand '"+operandAstring+"' in distance condition should be a Region Dataset",task.getLineNumber());            
              operandB=engine.getDataItem(operandBstring);
              if (operandB==null) {
                  try {
                     float value=Float.parseFloat(operandBstring);
                     operandB=new NumericConstant(operandBstring, value);
                  } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+operandBstring+"' neither data nor numeric constant",task.getLineNumber());}
              } // end operandB==null               
              if (!(operandB instanceof NumericVariable) && !(operandB instanceof NumericMapVariable) && !(operandB instanceof NumericConstant)) throw new ExecutionError("Operand '"+operandBstring+"' should be of numerical type",task.getLineNumber());            
        } else if (compareProperty.equals("overlaps")) {
              operandB=engine.getDataItem(operandBstring);
              if (operandB==null) throw new ExecutionError("Unrecognized token '"+operandBstring+"' is not a data object",task.getLineNumber());              
              if (!(operandB instanceof RegionDataset)) throw new ExecutionError("Second operand '"+operandBstring+"' in overlaps comparison should be a Region dataset",task.getLineNumber());            
        } else if (operandAstring!=null) { // compare numeric avg|min|max|median
             operandA=engine.getDataItem(operandAstring);
             if (!(operandA instanceof NumericDataset)) throw new ExecutionError("Operand '"+operandAstring+"' in condition should be a Numeric Dataset",task.getLineNumber());            
              operandB=engine.getDataItem(operandBstring);
              if (operandB==null) {
                  try {
                     float value=Float.parseFloat(operandBstring);
                     operandB=new NumericConstant(operandBstring, value);
                  } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+operandBstring+"' neither data nor numeric constant",task.getLineNumber());}
              } // end operandB==null               
              if (!(operandB instanceof NumericVariable) && !(operandB instanceof NumericMapVariable) && !(operandB instanceof NumericConstant)) throw new ExecutionError("Operand '"+operandBstring+"' should be of numerical type",task.getLineNumber());            
        }      
        setOperandAData(operandA);
        setOperandBData(operandB);  
        isResolved=true;
    }  

    
    
    /**
     * Returns true if this Condition holds at the specified genomic position
     * @param sequencename
     * @param pos the Genomic position! where this condition should be tested
     * @param task
     * @return
     * @throws java.lang.Exception
     */
    protected boolean isConditionSatisfied(String sequencename, Region region, OperationTask task) throws Exception {
        if (!isResolved) {
            task.getEngine().reportError(new ExecutionError("SLOPPY PROGRAMMING ERROR: Condition_region is not 'resolved' before use"));
        }
        String compareProperty=getCompareProperty();
        if (compareProperty==null) return true; // no condition is set, so the transform should be applied everywhere
        Data operandA=getOperandAData();
        Data operandB=getOperandBData();
        String comparator=getComparator();
        boolean whereNot=false;
        Boolean whereNotBoolean=negateAll();
        if (whereNotBoolean!=null) whereNot=whereNotBoolean.booleanValue();         
        boolean returnValue=true;
            
        if (compareProperty.equals("type")) {
              String regionType=region.getType();
              if (operandB instanceof StringVariable) {
                  String compareString=((StringVariable)operandB).getValue();
                  if (comparator.equals("equals")) returnValue=regionType.equals(compareString);
                  else if (comparator.equals("matches")) returnValue=regionType.matches(compareString);                 
              } else {
                  if (operandB instanceof MotifCollection) returnValue=((MotifCollection)operandB).contains(regionType);
                  else if (operandB instanceof StringCollection) returnValue=((StringCollection)operandB).contains(regionType);                
              }
        } else if (compareProperty.equals("score") || compareProperty.equals("length")) {
              float valueA=0;
              float valueB=0;
              if (compareProperty.equals("score")) valueA=region.getScore();
              else valueA=region.getLength();
                   if (operandB instanceof NumericMapVariable) valueB=((NumericMapVariable)operandB).getValue(sequencename);
              else if (operandB instanceof NumericVariable) valueB=((NumericVariable)operandB).getValue();
              else if (operandB instanceof NumericConstant) valueB=((NumericConstant)operandB).getValue();
              returnValue=compareNumerical(valueA, valueB, comparator);
        } else if (compareProperty.equals("overlaps")) {
              RegionSequenceData otherSequence=(RegionSequenceData)((RegionDataset)operandB).getSequenceByName(sequencename); // the track we are comparing with
              int overlapping=otherSequence.getNumberOfRegionsOverlappingGenomicInterval(region.getGenomicStart(), region.getGenomicEnd());
              returnValue=(overlapping>0);
        } else if (compareProperty.equals("distance to")) {
              float valueA=0; // this will hold the distance from the region to the target
              float valueB=0; // this is the second operand (the value to which the distance held in valueA is compared)
              RegionSequenceData otherSequence=(RegionSequenceData)((RegionDataset)operandA).getSequenceByName(sequencename); // the track we are comparing with
              valueA=findShortestDistance(region,otherSequence);
                   if (operandB instanceof NumericMapVariable) valueB=((NumericMapVariable)operandB).getValue(sequencename);
              else if (operandB instanceof NumericVariable) valueB=((NumericVariable)operandB).getValue();
              else if (operandB instanceof NumericConstant) valueB=((NumericConstant)operandB).getValue();
              //System.err.println(sequencename+":"+region.toString()+"   => distance="+valueA);
              returnValue=compareNumerical(valueA, valueB, comparator);
        } else if (operandA!=null) { // compare numeric avg|min|max|median
              float valueA=0;
              float valueB=0;
              NumericSequenceData otherSequence=(NumericSequenceData)((NumericDataset)operandA).getSequenceByName(sequencename); // the track we are comparing with
                   if (compareProperty.equals("avg") || compareProperty.equals("average")) valueA=otherSequence.getAverageValueInInterval(region.getRelativeStart(), region.getRelativeEnd());
              else if (compareProperty.equals("min") || compareProperty.equals("minimum")) valueA=otherSequence.getMinValueInInterval(region.getRelativeStart(), region.getRelativeEnd());
              else if (compareProperty.equals("max") || compareProperty.equals("maximum")) valueA=otherSequence.getMaxValueInInterval(region.getRelativeStart(), region.getRelativeEnd());
              else if (compareProperty.equals("sum")) valueA=otherSequence.getSumValueInInterval(region.getRelativeStart(), region.getRelativeEnd());
              else if (compareProperty.equals("median")) valueA=otherSequence.getMedianValueInInterval(region.getRelativeStart(), region.getRelativeEnd());
                   if (operandB instanceof NumericMapVariable) valueB=((NumericMapVariable)operandB).getValue(sequencename);
              else if (operandB instanceof NumericVariable) valueB=((NumericVariable)operandB).getValue();
              else if (operandB instanceof NumericConstant) valueB=((NumericConstant)operandB).getValue();
              returnValue=compareNumerical(valueA, valueB, comparator);
        }        
                
        if (whereNot) return !returnValue;
        else return returnValue;
    } 

    
    
    /** Compares two numerical values with the specified comparator */
    private boolean compareNumerical(float valueA, float valueB, String comparator) {
            boolean returnValue=false;
                 if (comparator.equals("="))  returnValue=(valueA==valueB);
            else if (comparator.equals(">=")) returnValue=(valueA>=valueB);
            else if (comparator.equals(">"))  returnValue=(valueA>valueB);
            else if (comparator.equals("<"))  returnValue=(valueA<valueB);
            else if (comparator.equals("<=")) returnValue=(valueA<=valueB);
            else if (comparator.equals("<>")) returnValue=(valueA!=valueB);      
            return returnValue;
    }
    
    /** Returns the distance between the given region (first argument) and the
     *  region in the (second argument) track that lays closest to the first region.
     *  (If the first region is a member of the second list it will be skipped!)
     */
    private int findShortestDistance(Region region, RegionSequenceData regionTrack) {
        int distance=Integer.MAX_VALUE;
        int startRegion=region.getGenomicStart();
        int endRegion=region.getGenomicEnd();
        for (Region other:regionTrack.getOriginalRegions()) { // these other regions are sorted by position (on direct strand)
               if (region.isSimilarTo(other)) continue; // do not compare to same region (it is probably the same anyway...)
               int startOther=other.getGenomicStart();
               int endOther=other.getGenomicEnd();
               //System.err.print("compare region ["+startRegion+"-"+endRegion+"] to other ["+startOther+"-"+endOther+"] => ");
               if (endOther<startRegion) { // other region lies before
                   int newdist=startRegion-endOther-1;
                   //System.err.println("other before    olddist="+distance+"  newdist="+newdist);
                   if (newdist<distance) distance=newdist; // update shortest distance as long as new distances keep getting smaller
               } else if (endRegion<startOther) { // other region lies after
                   int newdist=startOther-endRegion-1;
                   //System.err.println("other after     olddist="+distance+"  newdist="+newdist);
                   if (newdist<distance) distance=newdist; // update shortest distance as long as new distances keep getting smaller
                   else return distance; // if they are not getting any smaller they will keep getting larger, so just return the smallest until now                  
               } else { // region overlaps (return -1)
                   //System.err.println("overlap");
                   return -1;
               }
        }
        return distance;
    }
    
    
    
    
    @SuppressWarnings("unchecked")  
    @Override
    public Condition_region clone() {
        Condition_region newdata=new Condition_region((HashMap<String, Object>)storage.clone());
        return newdata;
    }      
}










