/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import java.util.ArrayList;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_combine_numeric extends FeatureTransformOperation {
    public static final String SOURCE_DATA="sourceData"; // reference to an array with NumericDatasets
    public static final String OPERATOR="operator"; // 
    private static final String name="combine_numeric";
    private static final String description="Combines several Numeric datasets using either the sum, minimum, maximum or average value at each position";
    private Class[] datasourcePreferences=new Class[]{NumericDataset.class};
    public static final int USE_SUM=0;
    public static final int USE_MIN=1;
    public static final int USE_MAX=2;
    public static final int USE_AVERAGE=3;
    public static final int USE_PRODUCT=4;


    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    
    @Override
    public boolean canHaveMultipleInput() {
        return true; 
    } 
    
    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String sourceDataString=(String)task.getSourceDataName(); // this could now potentially be a comma-separated list!
        String[] sourceNames=sourceDataString.split(",");
        NumericDataset[] sources=new NumericDataset[sourceNames.length];
        for (int i=0;i<sourceNames.length;i++) {
           String sourcename=sourceNames[i].trim();
           if (sourcename.isEmpty()) throw new ExecutionError("Missing name for source data object",task.getLineNumber());
           Data data=engine.getDataItem(sourcename);
           if (data==null) throw new ExecutionError("Unable to locate data object '"+sourcename+"'",task.getLineNumber());
           if (!(data instanceof NumericDataset)) throw new ExecutionError("Data object '"+sourcename+"' is not of required type (Numeric dataset)",task.getLineNumber());
           sources[i]=(NumericDataset)data;
        }
        String operator=(String)task.getParameter(OPERATOR);
        if (operator.equalsIgnoreCase("minimum")) task.setParameter(OPERATOR, "min");
        else if (operator.equalsIgnoreCase("maximum")) task.setParameter(OPERATOR, "max");
        else if (operator.equalsIgnoreCase("avg")) task.setParameter(OPERATOR, "average");
        task.setParameter(SOURCE_DATA, sources);
    }
    
    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        // this method is not used anymore since I override the execute() method in FeatureTransformOperation and use my own transformSequence (below)
    }   
    
    public void transformSequence(NumericSequenceData[] sourceSequences, NumericSequenceData targetSequence, OperationTask task, int operator) throws Exception {
        String seqname=targetSequence.getName();
        float[] values=new float[sourceSequences.length];
        for (int i=targetSequence.getRegionStart();i<=targetSequence.getRegionEnd();i++) {
            if (positionSatisfiesCondition(seqname,i,task)) {
              float newvalue=0;     
              for (int j=0;j<sourceSequences.length;j++) {
                  values[j]=sourceSequences[j].getValueAtGenomicPosition(i);
              }
              switch (operator) {
                  case USE_SUM: newvalue=calculateSum(values); break;
                  case USE_PRODUCT: newvalue=calculateProduct(values); break;
                  case USE_MIN: newvalue=findMinimum(values); break;
                  case USE_MAX: newvalue=findMaximum(values); break;
                  case USE_AVERAGE: newvalue=calculateSum(values)/(float)sourceSequences.length; break;
              }
              targetSequence.setValueAtGenomicPosition(i, newvalue);
           } // end: satisfies 'where'-condition
        }
    }
    
    
    @Override
    public boolean execute(OperationTask task) throws Exception {
        if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");
        resolveParameters(task);
        NumericDataset[] sources=(NumericDataset[])task.getParameter(SOURCE_DATA);
        if (sources.length==0) throw new ExecutionError("Missing source data objects",task.getLineNumber());
        String targetDatasetName=task.getTargetDataName();
        NumericDataset targetDataset=sources[0].clone(); // Double-buffer. Clone first in list!
        targetDataset.setName(targetDatasetName);        
        Condition condition=(Condition)task.getParameter("where");
        if (condition!=null) condition.resolve(engine, task);        
        
        String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
        if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
        Data seqcol=engine.getDataItem(subsetName);
        if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
        if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
        SequenceCollection sequenceCollection=(SequenceCollection)seqcol;
        int operator=USE_SUM;
        String operatorString=(String)task.getParameter(OPERATOR);
             if (operatorString.equalsIgnoreCase("sum")) operator=USE_SUM;
        else if (operatorString.equalsIgnoreCase("product")) operator=USE_PRODUCT;
        else if (operatorString.equalsIgnoreCase("average")) operator=USE_AVERAGE;
        else if (operatorString.equalsIgnoreCase("min")) operator=USE_MIN;
        else if (operatorString.equalsIgnoreCase("max")) operator=USE_MAX;
        else throw new ExecutionError("Unknown operator '"+operatorString+"'",task.getLineNumber());
        int size=sequenceCollection.getNumberofSequences();
        int i=0;
        ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
        NumericSequenceData[] sourceSequences=new NumericSequenceData[sources.length];
        for (Sequence sequence:sequences) { // for each sequence
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            String sequenceName=sequence.getName();
            for (int j=0;j<sources.length;j++) {
                sourceSequences[j]=(NumericSequenceData)sources[j].getSequenceByName(sequenceName);
            }            
            NumericSequenceData targetSequence=(NumericSequenceData)targetDataset.getSequenceByName(sequenceName); // just in case...
            transformSequence(sourceSequences,targetSequence, task, operator);         
            task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
            task.setProgress(i+1, size);
            i++;
            Thread.yield();
        }   
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        targetDataset.setIsDerived(true);
        try {engine.updateDataItem(targetDataset);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        return true;
    }
    
    /** Calculates the sum of an array of float values */
    private float calculateSum(float[] values) {
        float sum=0;
        for (float val:values) sum+=val;
        return sum;
    }
    
    /** Calculates the product of an array of float values */
    private float calculateProduct(float[] values) {
        float product=1f;
        for (float val:values) product*=val;
        return product;
    }
    
    /** Finds the smallest value in an array of values */
    private float findMinimum(float[] values) {
        float min=Float.MAX_VALUE;
        for (float val:values) {
            if (val<min) min=val;
        }
        return min;
    }
    /** Finds the largest value in an array of values */
    private float findMaximum(float[] values) {
        float max=-Float.MAX_VALUE;
        for (float val:values) {
            if (val>max) max=val;
        }
        return max;
    }

}
    