/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import java.util.ArrayList;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_combine_regions extends FeatureTransformOperation {
    public static final String SOURCE_DATA="sourceData"; // reference to an array with RegionDatasets
    public static final String OPERATOR="operator"; // 
    private static final String name="combine_regions";
    private static final String description="Combines several Region datasets into one";
    private Class[] datasourcePreferences=new Class[]{RegionDataset.class};



    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}

    @Override
    public boolean canHaveMultipleInput() {
        return true; 
    }    
    
    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String sourceDataString=(String)task.getSourceDataName(); // this could now potentially be a comma-separated list!
        String[] sourceNames=sourceDataString.split(",");
        RegionDataset[] sources=new RegionDataset[sourceNames.length];
        for (int i=0;i<sourceNames.length;i++) {
           String sourcename=sourceNames[i].trim();
           if (sourcename.isEmpty()) throw new ExecutionError("Missing name for source data object",task.getLineNumber());
           Data data=engine.getDataItem(sourcename);
           if (data==null) throw new ExecutionError("Unable to locate data object '"+sourcename+"'",task.getLineNumber());
           if (!(data instanceof RegionDataset)) throw new ExecutionError("Data object '"+sourcename+"' is not of required type (Region dataset)",task.getLineNumber());
           sources[i]=(RegionDataset)data;
        }
        task.setParameter(SOURCE_DATA, sources);
    }
    
  
    public void transformSequence(RegionSequenceData[] sourceSequences, RegionSequenceData targetSequence, OperationTask task) throws Exception {
        
        String seqname=targetSequence.getName();
        for (int i=0;i<sourceSequences.length;i++) {
             for (Region region:sourceSequences[i].getAllRegions()) {
                if (regionSatisfiesCondition(seqname, region, task)) {
                  targetSequence.addRegion(region);
                }
             }
        } 
    }
    
  @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        // this method is not used anymore since I override the execute() method in FeatureTransformOperation and use my own transformSequence (below)
    }   
    
    @Override
    public boolean execute(OperationTask task) throws Exception {
        if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");
        resolveParameters(task);
        RegionDataset[] sources=(RegionDataset[])task.getParameter(SOURCE_DATA);
        if (sources.length==0) throw new ExecutionError("Missing source data objects",task.getLineNumber());
        String targetDatasetName=task.getTargetDataName();
        RegionDataset targetDataset=sources[0].clone(); // Double-buffer. Clone first in list!
        targetDataset.setName(targetDatasetName);   
        targetDataset.clearRegions();
        Condition condition=(Condition)task.getParameter("where");
        if (condition!=null) condition.resolve(engine, task);        
        
        String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
        if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
        Data seqcol=engine.getDataItem(subsetName);
        if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
        if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
        SequenceCollection sequenceCollection=(SequenceCollection)seqcol;
        int size=sequenceCollection.getNumberofSequences();
        int i=0;
        ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
        RegionSequenceData[] sourceSequences=new RegionSequenceData[sources.length];
        for (Sequence sequence:sequences) { // for each sequence
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            String sequenceName=sequence.getName();
            for (int j=0;j<sources.length;j++) {
                sourceSequences[j]=(RegionSequenceData)sources[j].getSequenceByName(sequenceName);
            }            
            RegionSequenceData targetSequence=(RegionSequenceData)targetDataset.getSequenceByName(sequenceName); // just in case...
            transformSequence(sourceSequences,targetSequence, task);         
            task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
            task.setProgress(i+1, size);
            i++;
            Thread.yield();
        }   
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        targetDataset.setIsDerived(true);
        try {engine.updateDataItem(targetDataset);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        return true;
    }
    

}
    