/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import java.util.ArrayList;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_convert extends Operation {
    private static final String name="convert";
    private static final String description="Converts Numeric datasets to Regions or vice versa";
    private Class[] datasourcePreferences=new Class[]{NumericDataset.class,RegionDataset.class};
    
    public static final String NEW_VALUE="newValue";
    public static final String NEW_VALUE_STRING="newValueString";
    public static final String NEW_VALUE_OPERATOR="newValueOperator";
    public static final String TARGET_TYPE="targetType";
    public static final String REGION="region";
    public static final String NUMERIC="numeric";

    
    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}

    public void parseParameters(OperationTask task) throws Exception {
        String newValueString=(String)task.getParameter(NEW_VALUE_STRING);
        if (newValueString==null) return;
        String newValueOperatorString=(String)task.getParameter(NEW_VALUE_OPERATOR);
        if (newValueOperatorString!=null) { // convert to canonical values
                 if (newValueOperatorString.equals("avg")) task.setParameter(NEW_VALUE_OPERATOR,"average");
            else if (newValueOperatorString.equals("minimum")) task.setParameter(NEW_VALUE_OPERATOR,"min");
            else if (newValueOperatorString.equals("maximum")) task.setParameter(NEW_VALUE_OPERATOR,"max");
        }
        Object newValueObject;
        newValueObject=engine.getDataItem(newValueString);
        if (newValueObject==null) {
            if (newValueString.startsWith("region.")) newValueObject=newValueString;
            else try {
              float value=Float.parseFloat(newValueString);
              newValueObject=new NumericConstant(newValueString, (float)value);
           } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+newValueString+"' does not represent a numerical value",task.getLineNumber());}         
        } else if (!(newValueObject instanceof NumericVariable || newValueObject instanceof NumericMapVariable || newValueObject instanceof NumericDataset)) throw new ExecutionError("Unrecognized token '"+newValueString+"' does not represent a numerical value",task.getLineNumber());         
        task.setParameter(NEW_VALUE, newValueObject);
    }    
    
    
    @Override
    public boolean execute(OperationTask task) throws Exception {
        if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");
        String sourceDatasetName=task.getSourceDataName();
        String targetDatasetName=task.getTargetDataName();
        FeatureDataset sourceDataset=(FeatureDataset)engine.getDataItem(sourceDatasetName);
        if (sourceDataset==null) throw new ExecutionError("Unable to locate data object '"+sourceDatasetName+"'",task.getLineNumber());
        if (!canUseAsSource(sourceDataset)) throw new ExecutionError(sourceDatasetName+"("+sourceDataset.getTypeDescription()+") is of a type not supported by the '"+getName()+"' operation",task.getLineNumber());
        FeatureDataset targetDataset=null; // Double-buffer.
        if (sourceDataset instanceof NumericDataset) targetDataset=new RegionDataset(targetDatasetName);
        else if (sourceDataset instanceof RegionDataset) targetDataset=new NumericDataset(targetDatasetName);
        
        Condition condition=(Condition)task.getParameter("where");
        if (condition!=null) condition.resolve(engine, task);        
        parseParameters(task);
        Object newValueObject=task.getParameter(NEW_VALUE);        
        String newValueOperator=(String)task.getParameter(NEW_VALUE_OPERATOR);        
        SequenceCollection sequenceCollection=engine.getDefaultSequenceCollection();
        int size=sequenceCollection.getNumberofSequences();
        int i=0;
        ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
        for (Sequence sequence:sequences) { // for each sequence
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            String sequenceName=sequence.getName();
            FeatureSequenceData sourceSequence=(FeatureSequenceData)sourceDataset.getSequenceByName(sequenceName);
            FeatureSequenceData targetSequence=null; // just in case...
            if (sourceSequence instanceof NumericSequenceData) {
                targetSequence=new RegionSequenceData(sequence);
                convertNumericToRegion((NumericSequenceData)sourceSequence,(RegionSequenceData)targetSequence, task, newValueObject, newValueOperator);
            } else {
                targetSequence=new NumericSequenceData(sequence,0);
                convertRegionToNumeric((RegionSequenceData)sourceSequence,(NumericSequenceData)targetSequence, task, newValueObject);
            }
            targetDataset.addSequence(targetSequence);            
            //Thread.sleep(200); //  <-- just to test scheduling of slow tasks. Remove this later!!!
            //if (Math.random()<0.05) throw new NullPointerException("Testing!!"); //  <-- DEBUGGING!!
            task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
            task.setProgress(i+1, size);
            i++;
            Thread.yield();
        }   
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        targetDataset.setIsDerived(true);
        try {engine.storeDataItem(targetDataset);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        return true;
    }
       
    
    
    private boolean positionSatisfiesCondition(String sequencename, int pos, OperationTask task) throws Exception {
        Condition_position condition=(Condition_position)task.getParameter("where");
        if (condition==null) return true; // no condition is set, so the transform should be applied everywhere
        else return condition.isConditionSatisfied(sequencename, pos, task);
    }
    
   
    private boolean regionSatisfiesCondition(String sequencename, Region region, OperationTask task) throws Exception {
        Condition_region condition=(Condition_region)task.getParameter("where");
        if (condition==null) return true; // no condition is set, so the transform should be applied everywhere
        else return condition.isConditionSatisfied(sequencename, region, task);
    }
    
   

    public void convertNumericToRegion(NumericSequenceData sourceSequence, RegionSequenceData targetSequence, OperationTask task, Object newValueObject, String newValueOperator) throws Exception {
        String seqname=sourceSequence.getName();
        boolean inside=false;
        int regionStart=0;
        int regionEnd=0;
        for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
            if (positionSatisfiesCondition(seqname,i,task)) {
                if (!inside) {regionStart=i; inside=true;  } 
            } else { // condition not satisfied
                if (inside) {
                 regionEnd=i-1; inside=false;
                 Region newRegion=new Region(targetSequence, sourceSequence.getRelativePositionFromGenomic(regionStart), sourceSequence.getRelativePositionFromGenomic(regionEnd));
                 if (newValueObject!=null) newRegion.setScore(getRegionScore(newRegion.getRelativeStart(),newRegion.getRelativeEnd(),newValueObject,newValueOperator,seqname));
                 targetSequence.addRegion(newRegion);
              }               
           }
        } // end: for each position
        if (inside) { // last region did not close
             regionEnd=sourceSequence.getRegionEnd();
             Region newRegion=new Region(targetSequence, sourceSequence.getRelativePositionFromGenomic(regionStart), sourceSequence.getRelativePositionFromGenomic(regionEnd));
             if (newValueObject!=null) newRegion.setScore(getRegionScore(newRegion.getRelativeStart(),newRegion.getRelativeEnd(),newValueObject,newValueOperator,seqname));
             targetSequence.addRegion(newRegion);          
        }
    }          
            
    public void convertRegionToNumeric(RegionSequenceData sourceSequence, NumericSequenceData targetSequence, OperationTask task, Object newValueObject) throws Exception {
        ArrayList<Region> list = ((RegionSequenceData)sourceSequence).getAllRegions();
        String seqName=targetSequence.getName();
        float newValue=1;
        boolean setValueToRegionCount=false;
        NumericSequenceData wigseq=null;
             if (newValueObject instanceof NumericConstant) newValue=((NumericConstant)newValueObject).getValue().floatValue();
        else if (newValueObject instanceof NumericVariable) newValue=((NumericVariable)newValueObject).getValue().floatValue();
        else if (newValueObject instanceof NumericMapVariable) newValue=((NumericMapVariable)newValueObject).getValue(seqName).floatValue();
        else if (newValueObject instanceof NumericDataset) wigseq=(NumericSequenceData)((NumericDataset)newValueObject).getSequenceByName(seqName);
        else if ((newValueObject instanceof String) && (((String)newValueObject).equals("region.count"))) {
            for (int i=0;i<targetSequence.getSize();i++) {targetSequence.setValueAtRelativePosition(i, 0f);}            
            setValueToRegionCount=true;
        }
        for (Region region:list) {
            if (regionSatisfiesCondition(seqName, region, task)) {
                if (newValueObject instanceof String) {
                    if (((String)newValueObject).equals("region.score")) newValue=region.getScore();
                    else if (((String)newValueObject).equals("region.length")) newValue=region.getLength();
                } 
                int start=region.getRelativeStart();
                int end=region.getRelativeEnd();
                if (start<0) start=0;
                if (end>=sourceSequence.getSize()) end=sourceSequence.getSize()-1;
                if (wigseq==null) {
                    for (int i=start;i<=end;i++) {
                        if (setValueToRegionCount) {
                            float oldValue=targetSequence.getValueAtRelativePosition(i);
                            targetSequence.setValueAtRelativePosition(i, oldValue+1); // use same value all over
                        } else {
                            float oldValue=targetSequence.getValueAtRelativePosition(i);
                            if (newValue>oldValue) targetSequence.setValueAtRelativePosition(i, newValue);
                        } // use same value all over
                    }
                } else {
                    for (int i=start;i<=end;i++) {
                        Float v=wigseq.getValueAtRelativePosition(i);
                        if (v!=null) targetSequence.setValueAtRelativePosition(i, v); // "copy" values from other Numeric dataset
                    }           
                }
            }            
        } // end: for each region
    }          
 
    
    /** Convenience function to retrieve the score function to use for a new region */
    private float getRegionScore(int start, int end, Object newValueObject, String newValueOperator, String seqName) {
        float newValue=0;
             if (newValueObject instanceof NumericConstant) newValue=((NumericConstant)newValueObject).getValue().floatValue();
        else if (newValueObject instanceof NumericVariable) newValue=((NumericVariable)newValueObject).getValue().floatValue();
        else if (newValueObject instanceof NumericMapVariable) newValue=((NumericMapVariable)newValueObject).getValue(seqName).floatValue();
        else if (newValueObject instanceof NumericDataset) {            
            NumericSequenceData wigseq=(NumericSequenceData)((NumericDataset)newValueObject).getSequenceByName(seqName);
            if (newValueOperator==null) newValueOperator="sum";
                 if (newValueOperator.equals("min")) newValue=wigseq.getMinValueInInterval(start, end);
            else if (newValueOperator.equals("max")) newValue=wigseq.getMaxValueInInterval(start, end);
            else if (newValueOperator.equals("average")) newValue=wigseq.getAverageValueInInterval(start, end);
            else if (newValueOperator.equals("median")) newValue=wigseq.getMedianValueInInterval(start, end);
            else if (newValueOperator.equals("sum")) newValue=wigseq.getSumValueInInterval(start, end);
        }
        return newValue;
    }
    
    
}


