/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import java.util.ArrayList;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_count extends FeatureTransformOperation {
    public static final String WINDOW_SIZE="windowSize"; 
    private static final String WINDOW_SIZE_VALUE="windowSizeValue"; 
    public static final String ANCHOR="anchor"; //
    public static final String UPSTREAM="upstream"; //
    public static final String DOWNSTREAM="downstream"; //
    public static final String CENTER="center"; //    
    public static final String COUNT_PROPERTY="countProperty"; //    
    public static final String REGION_NUMBER_COUNT="number"; //
    public static final String REGION_SCORES_COUNT="score"; //
    public static final String OVERLAPPING_OR_WITHIN="overlappingORwithin"; //
    private static final String name="count";
    private static final String description="Counts the number of regions (or sums the scores of regions) that overlap or lie within a sliding window";
    private Class[] datasourcePreferences=new Class[]{RegionDataset.class};
   


    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    
    @Override
    public boolean isSubrangeApplicable() {return true;}
    
    
    @Override
    public boolean execute(OperationTask task) throws Exception {
        if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");
        String sourceDatasetName=task.getSourceDataName();
        String targetDatasetName=task.getTargetDataName();
        FeatureDataset sourceDataset=(FeatureDataset)engine.getDataItem(sourceDatasetName);
        if (sourceDataset==null) throw new ExecutionError("Unable to locate data object '"+sourceDatasetName+"'",task.getLineNumber());
        if (!canUseAsSource(sourceDataset)) throw new ExecutionError(sourceDatasetName+"("+sourceDataset.getTypeDescription()+") is of a type not supported by the '"+getName()+"' operation",task.getLineNumber());
        NumericDataset targetDataset=new NumericDataset(targetDatasetName);
        ArrayList<Data> sequenceList=engine.getAllDataItemsOfType(Sequence.class);
        for (Data seq:sequenceList) {
            NumericSequenceData numericSeq=new NumericSequenceData((Sequence)seq, 0);
            targetDataset.addSequence(numericSeq);
        }
        Condition condition=(Condition)task.getParameter("where");
        if (condition!=null) condition.resolve(engine, task);        
        resolveParameters(task);
        
        String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
        if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
        Data seqcol=engine.getDataItem(subsetName);
        if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
        if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
        SequenceCollection sequenceCollection=(SequenceCollection)seqcol;

        int size=sequenceCollection.getNumberofSequences();
        int i=0;
        ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
        for (Sequence sequence:sequences) { // for each sequence
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            String sequenceName=sequence.getName();
            FeatureSequenceData sourceSequence=(FeatureSequenceData)sourceDataset.getSequenceByName(sequenceName);
            FeatureSequenceData targetSequence=(FeatureSequenceData)targetDataset.getSequenceByName(sequenceName); // just in case...
            transformSequence(sourceSequence,targetSequence, task);         
            task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
            task.setProgress(i+1, size);
            i++;
            Thread.yield();
        }   
        if (targetDataset instanceof NumericDataset) ((NumericDataset)targetDataset).updateAllowedMinMaxValuesFromData();
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        targetDataset.setIsDerived(true);
        try {engine.updateDataItem(targetDataset);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        return true;
    }
    
    
    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String windowSizeString=(String)task.getParameter(WINDOW_SIZE);
        Data windowSizeData=null;
        windowSizeData=engine.getDataItem(windowSizeString);
        if (windowSizeData==null) {
            try {
              float value=Float.parseFloat(windowSizeString);
              windowSizeData=new NumericConstant(windowSizeString, (float)value);
           } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+windowSizeString+"' neither data nor numeric constant",task.getLineNumber());}         
        }
        task.setParameter(WINDOW_SIZE_VALUE, windowSizeData);
    }

    
    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        String countProperty=(String)task.getParameter(COUNT_PROPERTY);
        String overlapping=(String)task.getParameter(OVERLAPPING_OR_WITHIN);
        Boolean strictlyWithin=true;
        if (overlapping.equalsIgnoreCase("overlapping")) strictlyWithin=false;
        Data windowSizeData=(Data)task.getParameter(WINDOW_SIZE_VALUE);
        String anchor=(String)task.getParameter(ANCHOR);
        int windowsize=5;   
        String seqname=sourceSequence.getName();
        if (windowSizeData instanceof NumericMapVariable) windowsize=(int)((NumericMapVariable)windowSizeData).getValue(seqname).floatValue();
        else if (windowSizeData instanceof NumericVariable) windowsize=(int)((NumericVariable)windowSizeData).getValue().floatValue();
        else if (windowSizeData instanceof NumericConstant) windowsize=(int)((NumericConstant)windowSizeData).getValue().floatValue();
        if (windowsize==0) throw new ExecutionError("Window size can not be zero");
                      
        int orientation=sourceSequence.getStrandOrientation();
        for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
            int start=0, end=0;
            if (anchor.equals(UPSTREAM)) {
                start=(orientation==Sequence.DIRECT)?i:i-windowsize+1;
                end=(orientation==Sequence.DIRECT)?i+windowsize-1:i;
            } else if (anchor.equals(DOWNSTREAM)) {
                start=(orientation==Sequence.DIRECT)?i-windowsize+1:i;                
                end=(orientation==Sequence.DIRECT)?i:i+windowsize-1;
            } else {
               if (windowsize%2==0) { // window has even number of bases - anchor left of center
                   int flanksize=(int)(windowsize/2);
                   start=(orientation==Sequence.DIRECT)?i-flanksize+1:i-flanksize;
                   end=(orientation==Sequence.DIRECT)?i+flanksize:i+flanksize-1;                   
               } else { // window has odd number of bases
                   int flanksize=(int)(windowsize/2);
                   start=i-flanksize;
                   end=i+flanksize;
               }
            }
            if (positionSatisfiesCondition(seqname,i,task)) {
              float newvalue=count((RegionSequenceData)sourceSequence,start,end,countProperty,strictlyWithin);
              ((NumericSequenceData)targetSequence).setValueAtGenomicPosition(i, newvalue);
           } // satisfies 'where'-condition
        }
    }
    
    private float count(RegionSequenceData sourceSequence, int start, int end, String countProperty, boolean strictlyWithin) throws ExecutionError {
        ArrayList<Region> list=null;
        if (strictlyWithin) list=sourceSequence.getRegionsWithinGenomicInterval(start, end);
        else list=sourceSequence.getRegionsOverlappingGenomicInterval(start, end);  
        //System.err.println(start+"-"+end+" => "+list.size());
        if (countProperty.equals(REGION_NUMBER_COUNT)) {
            return list.size();
        } else if (countProperty.equals(REGION_SCORES_COUNT)) {
            float scores=0;
            for (Region r:list) scores+=r.getScore();
            return scores;
        } else throw new ExecutionError("Unknown property '"+countProperty+"' for counting");
    }
    
    
}
