/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import java.util.ArrayList;
import java.util.Iterator;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_decrease extends FeatureTransformOperation {
    public static final String DECREASE_STRING="decrease"; // textual representation of the decrease parameter
    public static final String DECREASE_DATA="decreaseData"; // a data object representation of the decrease parameter
    private static final String name="decrease";
    private static final String description="Decreases the values of selected bases in a numeric dataset by a specified number or position-wise by another numeric dataset";
    private Class[] datasourcePreferences=new Class[]{NumericDataset.class, NumericVariable.class, NumericMapVariable.class};



    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String toString() {return name;}

    @Override
    public boolean isSubrangeApplicable() {return true;}


    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String decreaseDataString=(String)task.getParameter(DECREASE_STRING);
        Data decreaseData=null;
        decreaseData=engine.getDataItem(decreaseDataString);
        if (decreaseData==null) {
            try {
              float value=Float.parseFloat(decreaseDataString);
              decreaseData=new NumericVariable(decreaseDataString, (float)value);
           } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+decreaseDataString+"' neither data nor numeric constant",task.getLineNumber());}
        }
        if (!(decreaseData instanceof NumericVariable || decreaseData instanceof NumericMapVariable || decreaseData instanceof NumericDataset)) throw new ExecutionError("'"+decreaseDataString+"' is not a valid numeric value",task.getLineNumber());
        task.setParameter(DECREASE_DATA, decreaseData);
    }



    @Override
    public boolean execute(OperationTask task) throws Exception {
        String sourceDatasetName=task.getSourceDataName();
        String targetDatasetName=task.getTargetDataName();
        Data sourceData=engine.getDataItem(sourceDatasetName);
        if (sourceData==null) throw new ExecutionError("Unable to locate data object '"+sourceDatasetName+"'",task.getLineNumber());
        if (!canUseAsSource(sourceData)) throw new ExecutionError(sourceDatasetName+"("+sourceData.getTypeDescription()+") is of a type not supported by the '"+getName()+"' operation",task.getLineNumber());

        if (sourceData instanceof NumericVariable) {
            task.setStatusMessage("Executing operation: "+task.getOperationName());
            NumericVariable targetData=(NumericVariable)sourceData.clone();
            targetData.rename(targetDatasetName);
            task.setParameter(OperationTask.SOURCE, sourceData);
            task.setParameter(OperationTask.TARGET, targetData);
            resolveParameters(task);
            Data decreaseData=(Data)task.getParameter(DECREASE_DATA);
            if (!(decreaseData instanceof NumericVariable || decreaseData instanceof NumericMapVariable)) throw new ExecutionError((String)task.getParameter(DECREASE_STRING)+" is of a type not applicable in this context",task.getLineNumber());
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            float decreaseValue=0;
            if (decreaseData instanceof NumericVariable) decreaseValue=((NumericVariable)decreaseData).getValue();
            else if (decreaseData instanceof NumericMapVariable) decreaseValue=((NumericMapVariable)decreaseData).getValue(); // use the default value in NumericMapVariable
            targetData.setValue(targetData.getValue()-decreaseValue);
            task.setProgress(100);
            try {engine.updateDataItem(targetData);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        } else if (sourceData instanceof NumericMapVariable) {
            NumericMapVariable targetData=(NumericMapVariable)sourceData.clone();
            targetData.rename(targetDatasetName);
            String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
            if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
            Data seqcol=engine.getDataItem(subsetName);
            if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
            if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
            SequenceCollection sequenceCollection=(SequenceCollection)seqcol;
            task.setParameter(OperationTask.SOURCE, sourceData);
            task.setParameter(OperationTask.TARGET, targetData);
            task.setParameter(OperationTask.SEQUENCE_COLLECTION, sequenceCollection);
            resolveParameters(task);
            ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
            int size=sequences.size();
            Data decreaseData=(Data)task.getParameter(DECREASE_DATA);
            if (!(decreaseData instanceof NumericVariable || decreaseData instanceof NumericMapVariable)) throw new ExecutionError((String)task.getParameter(DECREASE_STRING)+" is of a type not applicable in this context",task.getLineNumber());
            int i=0;
            for (Sequence sequence:sequences) { // for each sequence
                task.checkExecutionLock(); // checks to see if this task should suspend execution
                if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
                String sequenceName=sequence.getName();
                float decreaseValue=0;
                if (decreaseData instanceof NumericMapVariable) decreaseValue=((NumericMapVariable)decreaseData).getValue(sequenceName);
                else decreaseValue=((NumericVariable)decreaseData).getValue();
                targetData.setValue(sequenceName,targetData.getValue(sequenceName)-decreaseValue);
                task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
                task.setProgress(i+1, size);
                i++;
            }
            // decrease value of the NumericMapVariable default also!
            float decreaseValue=0;
            if (decreaseData instanceof NumericMapVariable) decreaseValue=((NumericMapVariable)decreaseData).getValue();
            else decreaseValue=((NumericVariable)decreaseData).getValue();
            targetData.setDefaultValue(targetData.getValue()-decreaseValue);
            try {engine.updateDataItem(targetData);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        } else if (sourceData instanceof FeatureDataset) {
            FeatureDataset sourceDataset=(FeatureDataset)sourceData;
            FeatureDataset targetDataset=(FeatureDataset)sourceDataset.clone(); // Double-buffer.
            targetDataset.setName(targetDatasetName);

            if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");

            Condition condition=(Condition)task.getParameter("where");
            if (condition!=null) condition.resolve(engine, task);

            Condition_within within=(Condition_within)task.getParameter("within");
            if (within!=null) within.resolve(engine, task);

            String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
            if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
            Data seqcol=engine.getDataItem(subsetName);
            if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
            if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
            SequenceCollection sequenceCollection=(SequenceCollection)seqcol;
            task.setParameter(OperationTask.SOURCE, sourceDataset);
            task.setParameter(OperationTask.TARGET, targetDataset);
            task.setParameter(OperationTask.SEQUENCE_COLLECTION, sequenceCollection);
            resolveParameters(task);

            int i=0;
            ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
            if (isSubrangeApplicable() && within!=null) { // remove sequences with no selection windows (if within-condition is used)
                Iterator iter=sequences.iterator();
                while (iter.hasNext()) {
                    Sequence seq = (Sequence) iter.next();
                    if (!within.existsSelectionWithinSequence(seq.getName(), task)) iter.remove();
                }
            }
            int size=sequences.size();
            for (Sequence sequence:sequences) { // for each sequence
                task.checkExecutionLock(); // checks to see if this task should suspend execution
                if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
                String sequenceName=sequence.getName();

                FeatureSequenceData sourceSequence=(FeatureSequenceData)sourceDataset.getSequenceByName(sequenceName);
                FeatureSequenceData targetSequence=(FeatureSequenceData)targetDataset.getSequenceByName(sequenceName); // just in case...
                transformSequence(sourceSequence,targetSequence, task);

                task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
                task.setProgress(i+1, size);
                i++;
                Thread.yield();
            }
            if (targetDataset instanceof NumericDataset) ((NumericDataset)targetDataset).updateAllowedMinMaxValuesFromData();
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            targetDataset.setIsDerived(true);
            try {engine.updateDataItem(targetDataset);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        }
        return true;
    }



    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        Data decreaseData=(Data)task.getParameter(DECREASE_DATA);
        Data decrease;
        if (decreaseData instanceof NumericDataset) decrease=((NumericDataset)decreaseData).getSequenceByName(sourceSequence.getName());
        else decrease=decreaseData;
        String seqname=sourceSequence.getName();
        for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
            if (positionSatisfiesCondition(seqname,i,task)) {
              float decreaseValue=0;
              if (decrease instanceof NumericConstant) {
                  decreaseValue=((NumericConstant)decrease).getValue();
              } else if (decrease instanceof NumericMapVariable) {
                  decreaseValue=((NumericMapVariable)decrease).getValue(seqname);
              } else if (decrease instanceof NumericVariable) {
                  decreaseValue=((NumericVariable)decrease).getValue();
              } else if (decrease instanceof NumericSequenceData) {
                  decreaseValue=((NumericSequenceData)decrease).getValueAtGenomicPosition(i);
              }
              //engine.logMessage("decreaseValue="+decreaseValue+"  decreaseData="+decreaseData.toString()+" (class)="+decreaseData.getClass().toString());
              float oldvalue=((NumericSequenceData)sourceSequence).getValueAtGenomicPosition(i);
              ((NumericSequenceData)targetSequence).setValueAtGenomicPosition(i, oldvalue-decreaseValue);
           } // satisfies 'where'-condition
        }
    }


}
