/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import java.util.ArrayList;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.RegionSequenceData;
import priorseditor.engine.data.Sequence;
import priorseditor.engine.data.SequenceCollection;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.data.NumericSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_distance extends Operation {
    public static final String ANCHOR_POINT="anchorPoint"; // 
    public static final String DIRECTION="DIRECTION"; // 
    public static final String UPSTREAM="upstream"; // 
    public static final String DOWNSTREAM="downstream"; // 
    public static final String BOTH="both"; // 
    private static final String name="distance";
    private static final String description="Creates a new Numeric dataset where the value at each base is determined by its distance from a selected feature";
    private Class[] datasourcePreferences=new Class[]{Region.class};


    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}    
    
    @Override
    public boolean execute(OperationTask task) throws Exception {            
        if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");
        String targetDatasetName=task.getTargetDataName();
        if (targetDatasetName==null || targetDatasetName.isEmpty()) throw new ExecutionError("Missing name for target data object",task.getLineNumber());
        NumericDataset targetDataset=new NumericDataset(targetDatasetName);
        targetDataset.setName(targetDatasetName);
        SequenceCollection sequenceCollection=engine.getDefaultSequenceCollection();
        int size=sequenceCollection.getNumberofSequences();
        int i=0;
        Object anchor=null;
        String direction=(String)task.getParameter(Operation_distance.DIRECTION);
        if (direction==null || direction.isEmpty()) direction=Operation_distance.BOTH;
        String anchorPoint=(String)task.getParameter(Operation_distance.ANCHOR_POINT);
        if (anchorPoint.startsWith("sequence.")) {
            if (anchorPoint.length()==9) throw new ExecutionError("Unrecognized anchor point");
            anchorPoint=anchorPoint.substring(9); 
            if (anchorPoint.equalsIgnoreCase("TSS") || anchorPoint.equalsIgnoreCase("TES") || anchorPoint.equalsIgnoreCase("upstream")|| anchorPoint.equalsIgnoreCase("downstream")) anchor=anchorPoint; 
            else throw new ExecutionError("Unrecognized anchor point: "+anchorPoint);
        } else {
            Object dataobject=engine.getDataItem(anchorPoint);
            if (dataobject==null) throw new ExecutionError("No such data object: "+anchorPoint,task.getLineNumber());
            if (!(dataobject instanceof RegionDataset)) throw new ExecutionError("Anchor point for distance function is not a region dataset",task.getLineNumber());
            anchor=dataobject;
        }
        ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
        for (Sequence sequence:sequences) { // for each sequence
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            NumericSequenceData numericSequence=new NumericSequenceData(sequence,0);
            transformSequence(numericSequence,anchor, direction, task);    
            targetDataset.addSequence(numericSequence);
            //Thread.sleep(200); //  <-- just to test scheduling of slow tasks. Remove this later!!!
            //if (Math.random()<0.05) throw new NullPointerException("Testing!!"); //  <-- DEBUGGING!!
            task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
            task.setProgress(i+1, size);
            i++;
            Thread.yield();
        }   
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        targetDataset.setIsDerived(true);
        try {engine.updateDataItem(targetDataset);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        return true;
    }

    
    public void transformSequence(NumericSequenceData targetSequence, Object anchor, String direction, OperationTask task) throws Exception {
         int orientation=targetSequence.getStrandOrientation();
         
         if (anchor instanceof String && ((String)anchor).equalsIgnoreCase("TSS")) {
             Integer TSSint=targetSequence.getTSS();
             if (TSSint==null) return;
             for (int i=targetSequence.getRegionStart();i<=targetSequence.getRegionEnd();i++) {
                float value=0;
                if (direction.equals(Operation_distance.DOWNSTREAM)) value=(i-TSSint)*orientation;
                else if (direction.equals(Operation_distance.UPSTREAM)) value=(TSSint-i)*orientation;
                else value=Math.abs(i-TSSint);
                targetSequence.setValueAtGenomicPosition(i, value); 
             }
         } else if (anchor instanceof String && ((String)anchor).equalsIgnoreCase("TES")) {
             Integer TESint=targetSequence.getTES();
             if (TESint==null) return;
             for (int i=targetSequence.getRegionStart();i<=targetSequence.getRegionEnd();i++) {
                float value=0;
                if (direction.equals(Operation_distance.DOWNSTREAM)) value=(i-TESint)*orientation;
                else if (direction.equals(Operation_distance.UPSTREAM)) value=(TESint-i)*orientation;
                else value=Math.abs(i-TESint);
                targetSequence.setValueAtGenomicPosition(i, value); 
             }
         }  else if (anchor instanceof String && ((String)anchor).equalsIgnoreCase("upstream")) {
             int upstream=(orientation==Sequence.DIRECT)?targetSequence.getRegionStart():targetSequence.getRegionEnd();
             for (int i=targetSequence.getRegionStart();i<=targetSequence.getRegionEnd();i++) {
                float value=0;
                if (direction.equals(Operation_distance.DOWNSTREAM)) value=(i-upstream)*orientation;
                else if (direction.equals(Operation_distance.UPSTREAM)) value=(upstream-i)*orientation;
                else value=Math.abs(i-upstream);
                targetSequence.setValueAtGenomicPosition(i, value); 
             }
         }  else if (anchor instanceof String && ((String)anchor).equalsIgnoreCase("downstream")) {
             int downstream=(orientation==Sequence.DIRECT)?targetSequence.getRegionEnd():targetSequence.getRegionStart();
             for (int i=targetSequence.getRegionStart();i<=targetSequence.getRegionEnd();i++) {
                float value=0;
                if (direction.equals(Operation_distance.DOWNSTREAM)) value=(i-downstream)*orientation;
                else if (direction.equals(Operation_distance.UPSTREAM)) value=(downstream-i)*orientation;
                else value=Math.abs(i-downstream);
                targetSequence.setValueAtGenomicPosition(i, value); 
             }
         } else if (anchor instanceof RegionDataset) { // anchor is a region dataset
             RegionDataset dataset=(RegionDataset)anchor;
             RegionSequenceData regionSequence=(RegionSequenceData)dataset.getSequenceByName(targetSequence.getName());
             ArrayList<Region> regions=regionSequence.getAllRegions();
             if (regions.isEmpty()) return;
             int pos1=targetSequence.getRegionStart();
             int pos2=regions.get(0).getGenomicStart();
             for (int p=pos1;p<=pos2-1;p++) { // between start and first region
                 float value=0;
                      if (direction.equals(Operation_distance.DOWNSTREAM)) value=(orientation==Sequence.DIRECT)?0:pos2-p;
                 else if (direction.equals(Operation_distance.UPSTREAM))   value=(orientation==Sequence.DIRECT)?pos2-p:0;
                 else value=pos2-p;
                 targetSequence.setValueAtGenomicPosition(p, value);
             }             
             for (int i=1;i<regions.size();i++) { // between regions
                 int newpos1=regions.get(i-1).getGenomicEnd();
                 if (newpos1>pos1) pos1=newpos1; 
                 pos2=regions.get(i).getGenomicStart();               
                 if (pos2-pos1<1) continue; // no 'space' between regions
                 for (int p=pos1+1;p<=pos2-1;p++) {
                     float value=0;
                          if (direction.equals(Operation_distance.DOWNSTREAM)) value=(orientation==Sequence.DIRECT)?p-pos1:pos2-p;
                     else if (direction.equals(Operation_distance.UPSTREAM))   value=(orientation==Sequence.DIRECT)?pos2-p:p-pos1;
                     else value=Math.min(p-pos1, pos2-p);
                     targetSequence.setValueAtGenomicPosition(p, value);
                 }
             }
             pos1=regions.get(regions.size()-1).getGenomicEnd();
             pos2=targetSequence.getRegionEnd();
             for (int p=pos1+1;p<=pos2;p++) { // between end and last region
                 float value=0;
                      if (direction.equals(Operation_distance.DOWNSTREAM)) value=(orientation==Sequence.DIRECT)?p-pos1:0;
                 else if (direction.equals(Operation_distance.UPSTREAM))   value=(orientation==Sequence.DIRECT)?0:p-pos1;
                 else value=p-pos1;
                 targetSequence.setValueAtGenomicPosition(p, value);
             }               
         } else if (anchor==null) throw new ExecutionError("SYSTEM ERROR: Anchor is NULL");
         else throw new ExecutionError("SYSTEM ERROR: Anchor is neither String nor Region dataset but "+anchor.getClass().getSimpleName()+". Anchor="+anchor.toString());

                 
    }

}
