/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import java.util.ArrayList;
import java.util.Iterator;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_divide extends FeatureTransformOperation {
    public static final String DIVIDE_STRING="divide"; // textual representation of the divide parameter
    private static final String DIVIDE_DATA="divideData"; // a data object representation of the divide parameter
    private static final String name="divide";
    private static final String description="Divides the values of selected bases in a numeric dataset by a specified number or position-wise by another numeric dataset";
    private Class[] datasourcePreferences=new Class[]{NumericDataset.class, NumericVariable.class, NumericMapVariable.class};



    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String toString() {return name;}

    @Override
    public boolean isSubrangeApplicable() {return true;}

    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String divideDataString=(String)task.getParameter(DIVIDE_STRING);
        Data divideData=null;
        divideData=engine.getDataItem(divideDataString);
        if (divideData==null) {
            try {
              float value=Float.parseFloat(divideDataString);
              divideData=new NumericVariable(divideDataString, (float)value);
           } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+divideDataString+"' neither data nor numeric constant",task.getLineNumber());}
        }
        if (!(divideData instanceof NumericVariable || divideData instanceof NumericMapVariable || divideData instanceof NumericDataset)) throw new ExecutionError("'"+divideDataString+"' is not a valid numeric value",task.getLineNumber());
        task.setParameter(DIVIDE_DATA, divideData);
    }


    @Override
    public boolean execute(OperationTask task) throws Exception {
        String sourceDatasetName=task.getSourceDataName();
        String targetDatasetName=task.getTargetDataName();
        Data sourceData=engine.getDataItem(sourceDatasetName);
        if (sourceData==null) throw new ExecutionError("Unable to locate data object '"+sourceDatasetName+"'",task.getLineNumber());
        if (!canUseAsSource(sourceData)) throw new ExecutionError(sourceDatasetName+"("+sourceData.getTypeDescription()+") is of a type not supported by the '"+getName()+"' operation",task.getLineNumber());

        if (sourceData instanceof NumericVariable) {
            task.setStatusMessage("Executing operation: "+task.getOperationName());
            NumericVariable targetData=(NumericVariable)sourceData.clone();
            targetData.rename(targetDatasetName);
            task.setParameter(OperationTask.SOURCE, sourceData);
            task.setParameter(OperationTask.TARGET, targetData);
            resolveParameters(task);
            Data divideData=(Data)task.getParameter(DIVIDE_DATA);
            if (!(divideData instanceof NumericVariable || divideData instanceof NumericMapVariable)) throw new ExecutionError((String)task.getParameter(DIVIDE_STRING)+" is of a type not applicable in this context",task.getLineNumber());
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            float divideValue=1;
            if (divideData instanceof NumericVariable) divideValue=((NumericVariable)divideData).getValue();
            else if (divideData instanceof NumericMapVariable) divideValue=((NumericMapVariable)divideData).getValue(); // use default value from NumericMapVariable
            if (divideValue!=0) targetData.setValue(targetData.getValue()/divideValue);
            else engine.logMessage("WARNING: Divide operation skipped to avoid division by zero");
            task.setProgress(100);
            try {engine.updateDataItem(targetData);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        } else if (sourceData instanceof NumericMapVariable) {
            NumericMapVariable targetData=(NumericMapVariable)sourceData.clone();
            targetData.rename(targetDatasetName);
            String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
            if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
            Data seqcol=engine.getDataItem(subsetName);
            if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
            if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
            SequenceCollection sequenceCollection=(SequenceCollection)seqcol;
            task.setParameter(OperationTask.SOURCE, sourceData);
            task.setParameter(OperationTask.TARGET, targetData);
            task.setParameter(OperationTask.SEQUENCE_COLLECTION, sequenceCollection);
            resolveParameters(task);
            ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
            int size=sequences.size();
            Data divideData=(Data)task.getParameter(DIVIDE_DATA);
            if (!(divideData instanceof NumericVariable || divideData instanceof NumericMapVariable)) throw new ExecutionError((String)task.getParameter(DIVIDE_STRING)+" is of a type not applicable in this context",task.getLineNumber());
            int i=0;
            for (Sequence sequence:sequences) { // for each sequence
                task.checkExecutionLock(); // checks to see if this task should suspend execution
                if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
                String sequenceName=sequence.getName();
                float divideValue=1;
                if (divideData instanceof NumericMapVariable) divideValue=((NumericMapVariable)divideData).getValue(sequenceName);
                else divideValue=((NumericVariable)divideData).getValue();
                if (divideValue!=0) targetData.setValue(sequenceName,targetData.getValue(sequenceName)/divideValue);
                else engine.logMessage("WARNING: Divide operation skipped for '"+sequenceName+"' to avoid division by zero");
                task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
                task.setProgress(i+1, size);
                i++;
            }
            // increase value of the NumericMapVariable default also!
            float divideValue=1;
            if (divideData instanceof NumericMapVariable) divideValue=((NumericMapVariable)divideData).getValue();
            else divideValue=((NumericVariable)divideData).getValue();
            if (divideValue!=0) targetData.setDefaultValue(targetData.getValue()/divideValue);
            else engine.logMessage("WARNING: Divide operation skipped for default value to avoid division by zero");
            try {engine.updateDataItem(targetData);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        } else if (sourceData instanceof FeatureDataset) {
            FeatureDataset sourceDataset=(FeatureDataset)sourceData;
            FeatureDataset targetDataset=(FeatureDataset)sourceDataset.clone(); // Double-buffer.
            targetDataset.setName(targetDatasetName);

            if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");

            Condition condition=(Condition)task.getParameter("where");
            if (condition!=null) condition.resolve(engine, task);

            Condition_within within=(Condition_within)task.getParameter("within");
            if (within!=null) within.resolve(engine, task);

            String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
            if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
            Data seqcol=engine.getDataItem(subsetName);
            if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
            if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
            SequenceCollection sequenceCollection=(SequenceCollection)seqcol;
            task.setParameter(OperationTask.SOURCE, sourceDataset);
            task.setParameter(OperationTask.TARGET, targetDataset);
            task.setParameter(OperationTask.SEQUENCE_COLLECTION, sequenceCollection);
            resolveParameters(task);

            int i=0;
            ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
            if (isSubrangeApplicable() && within!=null) { // remove sequences with no selection windows (if within-condition is used)
                Iterator iter=sequences.iterator();
                while (iter.hasNext()) {
                    Sequence seq = (Sequence) iter.next();
                    if (!within.existsSelectionWithinSequence(seq.getName(), task)) iter.remove();
                }
            }
            int size=sequences.size();
            for (Sequence sequence:sequences) { // for each sequence
                task.checkExecutionLock(); // checks to see if this task should suspend execution
                if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
                String sequenceName=sequence.getName();

                FeatureSequenceData sourceSequence=(FeatureSequenceData)sourceDataset.getSequenceByName(sequenceName);
                FeatureSequenceData targetSequence=(FeatureSequenceData)targetDataset.getSequenceByName(sequenceName); // just in case...
                transformSequence(sourceSequence,targetSequence, task);

                task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
                task.setProgress(i+1, size);
                i++;
                Thread.yield();
            }
            if (targetDataset instanceof NumericDataset) ((NumericDataset)targetDataset).updateAllowedMinMaxValuesFromData();
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            targetDataset.setIsDerived(true);
            try {engine.updateDataItem(targetDataset);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        }
        return true;
    }


    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        Data divideData=(Data)task.getParameter(DIVIDE_DATA);
        Data divide;
        int divisionbyzero=0;
        if (divideData instanceof NumericDataset) divide=((NumericDataset)divideData).getSequenceByName(sourceSequence.getName());
        else divide=divideData;
        String seqname=sourceSequence.getName();
        for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
            if (positionSatisfiesCondition(seqname,i,task)) {
              float divideValue=0;
              if (divide instanceof NumericConstant) {
                  divideValue=((NumericConstant)divide).getValue();
              } else if (divide instanceof NumericMapVariable) {
                  divideValue=((NumericMapVariable)divide).getValue(seqname);
              } else if (divide instanceof NumericVariable) {
                  divideValue=((NumericVariable)divide).getValue();
              } else if (divide instanceof NumericSequenceData) {
                  divideValue=((NumericSequenceData)divide).getValueAtGenomicPosition(i);
              }
              //engine.logMessage("divideValue="+divideValue+"  divideData="+divideData.toString()+" (class)="+divideData.getClass().toString());
              float oldvalue=((NumericSequenceData)sourceSequence).getValueAtGenomicPosition(i);
              if (divideValue!=0) ((NumericSequenceData)targetSequence).setValueAtGenomicPosition(i, oldvalue/divideValue);
              else divisionbyzero++;
            } // satisfies 'where'-condition
        }
        if (divisionbyzero>0) engine.logMessage("WARNING: Division by zero in "+seqname+". "+divisionbyzero+((divisionbyzero==1)?" position was skipped":" positions were skipped"));

    }


}
