/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_interpolate extends FeatureTransformOperation {
    public static final String METHOD="method"; //
    public static final String PERIOD="period"; // 
    private static final String PERIOD_DATA="periodData"; // 
    public static final String ZERO_ORDER_HOLD="zero-order-hold"; //
    public static final String LINEAR="linear-interpolation"; // 
    private static final String name="interpolate";
    private static final String description="Fills in missing values between discrete non-zero points in numeric datasets";
    private Class[] datasourcePreferences=new Class[]{NumericDataset.class};



    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    
    @Override
    public boolean isSubrangeApplicable() {return true;}
    
    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String periodString=(String)task.getParameter(PERIOD);
        if (periodString==null) return;
        Data periodData=null;
        periodData=engine.getDataItem(periodString);
        if (periodData==null) {
            try {
              float value=Float.parseFloat(periodString);
              periodData=new NumericVariable(periodString, (float)value);
           } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+periodString+"' neither data nor numeric constant",task.getLineNumber());}         
        }
        if (!(periodData instanceof NumericConstant || periodData instanceof NumericVariable || periodData instanceof NumericMapVariable)) throw new ExecutionError("'"+periodString+"' is not a valid numeric value",task.getLineNumber());          
        task.setParameter(PERIOD_DATA, periodData);
    }

    
    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        String method=(String)task.getParameter(METHOD);
        Data periodData=(Data)task.getParameter(PERIOD_DATA);
        String seqname=sourceSequence.getName();
        float periodfloat=0;
        if (periodData instanceof NumericConstant) {
              periodfloat=((NumericConstant)periodData).getValue();
        } else if (periodData instanceof NumericMapVariable) {
              periodfloat=((NumericMapVariable)periodData).getValue(seqname);
        } else if (periodData instanceof NumericVariable) {
              periodfloat=((NumericVariable)periodData).getValue();
        }
        int period=(int)periodfloat;
        int first=-1;
        for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
            float value=((NumericSequenceData)sourceSequence).getValueAtGenomicPosition(i);
            if (value!=0) {first=i;break;}
        }
        if (first<0) task.getEngine().logMessage("Only zeros in "+seqname);
        if (period==0) transformWithoutPeriod((NumericSequenceData)sourceSequence,(NumericSequenceData)targetSequence,task,method, first);
        else transformWithPeriod((NumericSequenceData)sourceSequence,(NumericSequenceData)targetSequence,task,method,period,first);
        
    }
    
    /** Interpolates between non-zero valued points in the sequence */
    private void transformWithoutPeriod(NumericSequenceData sourceSequence, NumericSequenceData targetSequence, OperationTask task, String method, int first) throws Exception {
        int lastpos=first;
        float lastvalue=((NumericSequenceData)sourceSequence).getValueAtGenomicPosition(lastpos);
        int i=lastpos;
        int end=sourceSequence.getRegionEnd();
        while (i<=end) {
            float value=((NumericSequenceData)sourceSequence).getValueAtGenomicPosition(i);
            if (value!=0) {
                float endvalue=(method.equals(ZERO_ORDER_HOLD))?lastvalue:value;
                fillIn(targetSequence,lastpos,lastvalue,i-1,endvalue,task);
                lastpos=i;
                lastvalue=value;
            }
            i++;
        }
        if (lastpos!=end) {
            fillIn(targetSequence,lastpos,lastvalue,end,lastvalue,task);
        }
    }    
    
    /** Fills in values between two selected points (start/end) in a sequence. The values to use for these endpoints must also 
     *  be provided and the values in between are based on linear interpolation between the two end values. 
     * (thus, zero-order-hold is imposed by using the same value for both end points). 
     *  
     */
    private void fillIn(NumericSequenceData targetSequence, int start, float startvalue, int end, float endvalue, OperationTask task) throws Exception{
        String seqname=targetSequence.getName();
        float increment=(endvalue-startvalue)/(float)(end-start+1);
        float value=startvalue;
        for (int i=start;i<=end;i++) {
            if (positionSatisfiesCondition(seqname,i,task)) {
               targetSequence.setValueAtGenomicPosition(i, value);
            }
            value+=increment;
        }
    }
    
    /** Interpolates between points with a fixed distance */
    private void transformWithPeriod(NumericSequenceData sourceSequence, NumericSequenceData targetSequence, OperationTask task, String method, int period, int first) throws Exception {
        String seqname=sourceSequence.getName();
        int i=first;
        int end=sourceSequence.getRegionEnd();
        while (i<=end) {
            float value=sourceSequence.getValueAtGenomicPosition(i);
            int nextpos=i+period;
            float nextvalue=0;
            if (nextpos>=end) {
                nextpos=end+1;
                nextvalue=sourceSequence.getValueAtGenomicPosition(end);
            } else nextvalue=sourceSequence.getValueAtGenomicPosition(nextpos);
            fillIn(targetSequence,i,value,nextpos-1,(method.equals(ZERO_ORDER_HOLD))?value:nextvalue,task);
            i=nextpos;
            
        }
    }
    
    
    public static String[] getInterpolationMethods() {
        return new String[]{ZERO_ORDER_HOLD,LINEAR};
    }
    
}
