/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_mask extends FeatureTransformOperation {
    public static final int UPPERCASE=0;
    public static final int LOWERCASE=1;
    public static final int LETTER=2;
    public static final int RANDOM_BASES=3;   
    public static final String MASKTYPE="masktype";
    public static final String MASK_STRING="masktype"; // the string describing the masking type to use: 'X' | uppercase | lowercase | Background model name
    public static final String BACKGROUND_MODEL="BackgroundModel";
    public static final String MASK_LETTER="maskletter";
    
    private static final String name="mask";
    private static final String description="Masks selected bases using either lowercase/uppercase, a specified letter or random bases";
    private Class[] datasourcePreferences=new Class[]{DNASequenceDataset.class};



    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    
    @Override
    public boolean isSubrangeApplicable() {return true;}
    
    
    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String maskString=(String)task.getParameter(Operation_mask.MASK_STRING);
        if (maskString.equalsIgnoreCase("lowercase")) task.setParameter(MASKTYPE, LOWERCASE);
        else if (maskString.equalsIgnoreCase("uppercase")) task.setParameter(MASKTYPE, UPPERCASE);
        else if (maskString.startsWith("'")) {
            char maskletter;
            if (maskString.length()!=3 || !maskString.endsWith("'")) throw new ExecutionError("Mask letter should be one character surrounded by single quotes");
            else maskletter=maskString.charAt(1);
            task.setParameter(MASKTYPE, LETTER);
            task.setParameter(MASK_LETTER, new Character(maskletter));
        } else {
            Object backgroundModel=engine.getDataItem(maskString);
            if (backgroundModel==null) throw new ExecutionError("No such data object: "+maskString);
            if (!(backgroundModel instanceof BackgroundModel)) throw new ExecutionError(maskString+" is not a Background model");            
            task.setParameter(MASKTYPE, RANDOM_BASES);
            task.setParameter(BACKGROUND_MODEL, backgroundModel);
        }        
    }

    
    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        char letter='X';
        int masktype=LOWERCASE;
        BackgroundModel backgroundModel=(BackgroundModel)task.getParameter(Operation_mask.BACKGROUND_MODEL);
        
        Integer maskTypeInt=(Integer)task.getParameter(Operation_mask.MASKTYPE);
        if (maskTypeInt==null) throw new ExecutionError("Missing mask type specification");
        else masktype=maskTypeInt.intValue();
        
        Character maskLetter=(Character)task.getParameter(Operation_mask.MASK_LETTER);
        if (maskLetter!=null) letter=maskLetter.charValue();

        String seqname=sourceSequence.getName();
        for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
            if (positionSatisfiesCondition(seqname,i,task)) {
              if (masktype==LETTER) ((DNASequenceData)targetSequence).setValueAtGenomicPosition(i,letter);
              else if (masktype==LOWERCASE) ((DNASequenceData)targetSequence).setValueAtGenomicPosition(i,Character.toLowerCase((Character)sourceSequence.getValueAtGenomicPosition(i)));
              else if (masktype==UPPERCASE) ((DNASequenceData)targetSequence).setValueAtGenomicPosition(i,Character.toUpperCase((Character)sourceSequence.getValueAtGenomicPosition(i)));
              else if (masktype==RANDOM_BASES) {
                  char filler=backgroundModel.getNextBase(getPrefixForPosition(i,backgroundModel.getOrder(),(DNASequenceData)targetSequence)); // the target sequence must be used to obtain the prefix, since this is the sequence being build upon!
                  ((DNASequenceData)targetSequence).setValueAtGenomicPosition(i,filler);
              }
                    
            } // satisfies 'where'-condition
        }
    }
    
    private String getPrefixForPosition(int i, int order, DNASequenceData source) {
        if (order==0 || i==source.getRegionStart()) return "";
        String prefix="";
        int start=i-order;
        int end=i-1;
        if (start<source.getRegionStart()) start=source.getRegionStart();
        for (int j=start;j<=end;j++) prefix+=source.getValueAtGenomicPosition(j).toString();
        //System.err.println(prefix);
        return prefix;
    }
}
