/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import priorseditor.engine.OperationTask;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.data.*;
import priorseditor.engine.dataformat.DataFormat;
import priorseditor.engine.dataformat.DataFormat_INCLUSive_Background;

/**
 *
 * @author kjetikl
 */
public class Operation_new extends Operation {
    private static final String name="new";
    private static final String description="Creates a new feature dataset or variable";
    private Class[] datasourcePreferences=new Class[]{};
    public static final String DATA_TYPE="datatype";
    public static final String PARAMETERS="parameters";
    public static final String DATA_TRACK_PREFIX="DataTrack:";
    public static final String FILE_PREFIX="File:";
    public static final String FILENAME="filename";
    public static final String DATA_FORMAT="dataformat";
    public static final String DATA_FORMAT_SETTINGS="dataformatSettings";
    public static final String MODEL_PREFIX="Model:";        
    public static final String COLLECTION_PREFIX="Collection:";
    public static final String FROMTRACK_PREFIX="Track:";
    public static final String COPY_PREFIX="Copy:";
    public static final String CONSTRUCTOR_PREFIX="Constructor:";
    private static final int UNION=0;
    private static final int INTERSECTION=1;
    private static final int SUBTRACTION=2;
    
    
    /** 
     * Return a list of types for data objects that can be instantiated by the new operator
     * (These should preferably be in sorted order (by type String))
     */
    public static String[] getAvailableTypes() {
        return new String[]{RegionDataset.getType(),DNASequenceDataset.getType(),NumericDataset.getType(),SequenceCollection.getType(), BackgroundModel.getType(),MotifCollection.getType(),Motif.getType(), NumericVariable.getType(), StringVariable.getType(), StringCollection.getType()};
    }
    
    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    

    @Override
    public boolean execute(OperationTask task) throws Exception {
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        String targetName=task.getTargetDataName();        
        //Data existingTarget=engine.getDataItem(targetName);
        if (targetName==null || targetName.isEmpty()) throw new ExecutionError("Missing name for target data object",task.getLineNumber());
        String datatype=(String)task.getParameter(DATA_TYPE);      
        if (datatype==null || datatype.isEmpty()) throw new ExecutionError("Missing data type specification",task.getLineNumber());
        Data newDataItem=null;
        String parameter=(String)task.getParameter(PARAMETERS);
        if ((datatype.equalsIgnoreCase(NumericDataset.getType()) || datatype.equalsIgnoreCase(RegionDataset.getType()) || datatype.equalsIgnoreCase(DNASequenceDataset.getType())) && engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");
        if (parameter!=null && !parameter.isEmpty() && parameter.startsWith(DATA_TRACK_PREFIX)) {
            String trackName=parameter.substring(DATA_TRACK_PREFIX.length());
            newDataItem=engine.getDataLoader().loadDataTrack(trackName,task);
                 if (datatype.equalsIgnoreCase(NumericDataset.getType()) && !(newDataItem instanceof NumericDataset)) throw new ExecutionError("Incompatible assignment to Numeric Dataset for DataTrack="+trackName,task.getLineNumber());
            else if (datatype.equalsIgnoreCase(RegionDataset.getType()) && !(newDataItem instanceof RegionDataset)) throw new ExecutionError("Incompatible assignment to Region Dataset for DataTrack="+trackName,task.getLineNumber());
            else if (datatype.equalsIgnoreCase(DNASequenceDataset.getType()) && !(newDataItem instanceof DNASequenceDataset)) throw new ExecutionError("Incompatible assignment to Sequence Dataset for DataTrack="+trackName,task.getLineNumber());
            newDataItem.rename(targetName);
        }
        else if (parameter!=null && !parameter.isEmpty() && parameter.startsWith(FILE_PREFIX)) {
            String filename=(String)task.getParameter(FILENAME); // would it be better to just parse it here?
            String dataformatname=(String)task.getParameter(DATA_FORMAT);
            ParameterSettings dataformatsettings=(ParameterSettings)task.getParameter(DATA_FORMAT_SETTINGS);
            File file=new File(filename);
            DataFormat formatter;
            if (dataformatname!=null) formatter=engine.getDataFormat(dataformatname);
            else formatter=engine.getDefaultDataFormat(engine.getDataClassForTypeName(datatype));
            if (formatter==null) throw new ExecutionError("Unknown data format: '"+dataformatname+"'");
            newDataItem=engine.getDataLoader().loadData(file, datatype, null, formatter, dataformatsettings, task);
            newDataItem.rename(targetName);
        }
        else if (parameter!=null && !parameter.isEmpty() && parameter.startsWith(COPY_PREFIX)) {
            String originalDataName=parameter.substring(COPY_PREFIX.length());
            Data originalData=engine.getDataItem(originalDataName)      ;
            if (originalData==null) throw new ExecutionError("Unknown data item: '"+originalDataName+"'");
            String originalType=engine.getTypeNameForDataClass(originalData.getClass());
            if (!originalType.equals(datatype)) throw new ExecutionError(originalDataName+" is not of type '"+datatype+"' but of type '"+originalType+"'");
            newDataItem=originalData.clone();
            newDataItem.rename(targetName);
        }
        else if (datatype.equalsIgnoreCase(NumericDataset.getType())) {
            //if (existingTarget!=null && !(existingTarget instanceof NumericDataset)) throw new ExecutionError("Operation 'new' cannot be applied to existing incompatible data object '"+targetName+"'");
            float[] min_max_baseline_default=new float[]{0.0f,1.0f,0.0f,0.0f};
            //String parameter=(String)task.getParameter(Operation_new.PARAMETERS);
            if (parameter!=null && !parameter.isEmpty()) {
                  min_max_baseline_default=parseMinMaxBaselineDefaultParameters(parameter);
            }            
            newDataItem=new NumericDataset(targetName);
            ((NumericDataset)newDataItem).setMinAllowedValue(min_max_baseline_default[0]);
            ((NumericDataset)newDataItem).setMaxAllowedValue(min_max_baseline_default[1]);
            ((NumericDataset)newDataItem).setBaselineValue(min_max_baseline_default[2]);
            SequenceCollection allSequences = engine.getDefaultSequenceCollection();
            ArrayList<Sequence> seqlist=allSequences.getAllSequences(engine);
            for (Sequence sequence:seqlist) {
                NumericSequenceData seq=new NumericSequenceData(sequence.getName(), sequence.getGeneName(), sequence.getChromosome(), sequence.getRegionStart(), sequence.getRegionEnd(), sequence.getTSS(), sequence.getTES(), sequence.getStrandOrientation(),min_max_baseline_default[3]);
                // set some default values also...
                ((NumericDataset)newDataItem).addSequence(seq);
            }
        } else if (datatype.equalsIgnoreCase(RegionDataset.getType())) {
            //if (existingTarget!=null && !(existingTarget instanceof RegionDataset)) throw new ExecutionError("Operation 'new' cannot be applied to existing incompatible data object '"+targetName+"'");
            newDataItem=new RegionDataset(targetName);
            SequenceCollection allSequences = engine.getDefaultSequenceCollection();
            ArrayList<Sequence> seqlist=allSequences.getAllSequences(engine);
            for (Sequence sequence:seqlist) {
                RegionSequenceData seq=new RegionSequenceData(sequence.getName(), sequence.getGeneName(), sequence.getChromosome(), sequence.getRegionStart(), sequence.getRegionEnd(), sequence.getTSS(), sequence.getTES(), sequence.getStrandOrientation());
                // set some default values also...
                ((RegionDataset)newDataItem).addSequence(seq);
            }
        } else if (datatype.equalsIgnoreCase(DNASequenceDataset.getType())) {
            //if (existingTarget!=null && !(existingTarget instanceof DNASequenceDataset)) throw new ExecutionError("Operation 'new' cannot be applied to existing incompatible data object '"+targetName+"'");
            newDataItem=new DNASequenceDataset(targetName);
            Object defaultvalue=new Character('N');
            //String parameter=(String)task.getParameter(Operation_new.PARAMETERS);
            if (parameter!=null && !parameter.isEmpty()) {
                if (parameter.startsWith("'")) {
                    if (parameter.length()!=3 || !parameter.endsWith("'")) throw new ExecutionError("Unable to parse default base:"+parameter);
                    else defaultvalue=parameter.charAt(1);
                }
                else {
                    defaultvalue=engine.getDataItem(parameter);
                    if (defaultvalue==null) throw new ExecutionError("No such data object: "+parameter);
                    if (!(defaultvalue instanceof BackgroundModel)) throw new ExecutionError(parameter+" is not a Background model");                    
                }
            }
            SequenceCollection allSequences = engine.getDefaultSequenceCollection();
            ArrayList<Sequence> seqlist=allSequences.getAllSequences(engine);
            for (Sequence sequence:seqlist) {
                DNASequenceData seq=null;
                if (defaultvalue instanceof BackgroundModel) seq=new DNASequenceData(sequence.getName(), sequence.getGeneName(), sequence.getChromosome(), sequence.getRegionStart(), sequence.getRegionEnd(), sequence.getTSS(), sequence.getTES(), sequence.getStrandOrientation(), (BackgroundModel)defaultvalue);
                else seq=new DNASequenceData(sequence.getName(), sequence.getGeneName(), sequence.getChromosome(), sequence.getRegionStart(), sequence.getRegionEnd(), sequence.getTSS(), sequence.getTES(), sequence.getStrandOrientation(),(Character)defaultvalue);
                // set some default values also...
                ((DNASequenceDataset)newDataItem).addSequence(seq);
            }
        } else if (datatype.equalsIgnoreCase(NumericVariable.getType())) {
            //if (existingTarget!=null && !(existingTarget instanceof DNASequenceDataset)) throw new ExecutionError("Operation 'new' cannot be applied to existing incompatible data object '"+targetName+"'");
            float defaultvalue=0;
            //String parameter=(String)task.getParameter(Operation_new.PARAMETERS);
            if (parameter!=null && !parameter.isEmpty()) {
                try {
                    float val=Float.parseFloat(parameter);
                    defaultvalue=val;
                } catch (Exception e) {throw new ExecutionError("Specified value for "+targetName+" is not numeric");}
            }
            newDataItem=new NumericVariable(targetName,defaultvalue);
        } else if (datatype.equalsIgnoreCase(BackgroundModel.getType())) {
            //if (existingTarget!=null && !(existingTarget instanceof DNASequenceDataset)) throw new ExecutionError("Operation 'new' cannot be applied to existing incompatible data object '"+targetName+"'");
            //String parameter=(String)task.getParameter(Operation_new.PARAMETERS);
            if (parameter!=null && !parameter.isEmpty()) {
                newDataItem=parseMarkovModelParameters(parameter,targetName,task);
            } else newDataItem=new BackgroundModel(targetName); 
        } else if (datatype.equalsIgnoreCase(SequenceCollection.getType())) {
            newDataItem=new SequenceCollection(targetName);
            if (parameter!=null && !parameter.isEmpty()) {
               ArrayList<String> values=parseSequenceCollectionParameters(parameter);
               for (String seqname:values) {
                   int mode=UNION;
                   if (seqname.startsWith("+") && seqname.length()>1) {mode=UNION;seqname=seqname.substring(1);}
                   if (seqname.startsWith("-") && seqname.length()>1) {mode=SUBTRACTION;seqname=seqname.substring(1);}
                   if (seqname.startsWith("&") && seqname.length()>1) {mode=INTERSECTION;seqname=seqname.substring(1);}
                   Data seq=engine.getDataItem(seqname);
                   if (seq==null) throw new ExecutionError("Unknown data item: "+seqname);
                   else if (!(seq instanceof Sequence || seq instanceof SequenceCollection)) throw new ExecutionError("Data item '"+seqname+"' is not a sequence or sequence collection");
                   else {
                            if (mode==SUBTRACTION) subtractFromSequenceCollection((SequenceCollection)newDataItem,seq);
                       else if (mode==INTERSECTION) intersectWithSequenceCollection((SequenceCollection)newDataItem,seq);
                       else addToSequenceCollection((SequenceCollection)newDataItem,seq);
                   }
               }
            } 
        } else if (datatype.equalsIgnoreCase(MotifCollection.getType())) {  
            if (parameter!=null && !parameter.isEmpty()) {
                  newDataItem=parseMotifCollectionParameters(parameter,targetName,task);      
            } else newDataItem=new MotifCollection(targetName);
        } else if (datatype.equalsIgnoreCase(Motif.getType())) {
            if (parameter!=null && !parameter.isEmpty()) {
                 newDataItem=parseMotifParameters(parameter,targetName);
            } else throw new ExecutionError("Missing parameters for motif "+targetName);
        } else if (datatype.equalsIgnoreCase(StringVariable.getType())) {
            //if (existingTarget!=null && !(existingTarget instanceof DNASequenceDataset)) throw new ExecutionError("Operation 'new' cannot be applied to existing incompatible data object '"+targetName+"'");
            if (parameter!=null) {
                if (!StringVariable.isValidValue(parameter)) throw new ExecutionError("Specified value for "+targetName+" is not valid value (might contain illegal characters: "+StringVariable.illegalCharacters()+")");
            }
            newDataItem=new StringVariable(targetName,parameter);
        } else if (datatype.equalsIgnoreCase(StringCollection.getType())) {
            //if (existingTarget!=null && !(existingTarget instanceof DNASequenceDataset)) throw new ExecutionError("Operation 'new' cannot be applied to existing incompatible data object '"+targetName+"'");
            if (parameter!=null && !parameter.isEmpty()) {
               ArrayList<String> values=parseStringCollectionParameters(parameter);
               newDataItem=new StringCollection(targetName,values);
            } else newDataItem=new StringCollection(targetName);
            // I should probably parse parameters here...
        } else {System.err.println("SYSTEM ERROR: Unknown data type for operation NEW: "+datatype);}
        if (newDataItem==null) throw new ExecutionError("Unable to create new data object",task.getLineNumber());
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        engine.storeDataItem(newDataItem);
        task.setProgress(100);
        return true;
    }
    
    
    private Motif parseMotifParameters(String text, String targetName) throws ExecutionError {
        Motif motif=new Motif(targetName);
        float[][] matrix=null;
        boolean[] gotAll=new boolean[]{false,false,false,false};
        String[] segments=text.split("\\s*;\\s*");        
            for (String segment:segments) {
                if (segment.trim().isEmpty()) continue;
                if (segment.startsWith("SHORTNAME:") && segment.length()>10) {
                    motif.setShortName(segment.substring(10));
                } else if (segment.startsWith("LONGNAME:") && segment.length()>9) {
                    motif.setLongName(segment.substring(9));
                } else if (segment.startsWith("CONSENSUS:") && segment.length()>10) {
                    motif.setConsensusMotif(segment.substring(10));
                } else if (segment.startsWith("CLASS:") && segment.length()>6) {
                    motif.setClassification(segment.substring(6));
                } else if (segment.startsWith("ORGANISMS:") && segment.length()>10) {
                    motif.setOrganisms(segment.substring(10));
                } else if (segment.startsWith("QUALITY:") && segment.length()>8) {
                    try {
                        motif.setQuality(Integer.parseInt(segment.substring(8)));
                    } catch(Exception e) {throw new ExecutionError("Unable to parse expected numeric value for "+segment);}
                } else if (segment.startsWith("FACTORS:") && segment.length()>8) {
                    motif.setBindingFactors(segment.substring(8));
                } else if (segment.startsWith("HALFSITE")) {
                    motif.setHalfsite(true);
                } else if ((segment.startsWith("A:") || segment.startsWith("C:") || segment.startsWith("G:") || segment.startsWith("T:"))  && segment.length()>2) {
                    String[] values=segment.substring(2).split("\\s*,\\s*");
                    int index=0;
                         if (segment.startsWith("C")) index=1;
                    else if (segment.startsWith("G")) index=2;
                    else if (segment.startsWith("T")) index=3;
                    if (matrix==null) matrix=new float[values.length][4];
                    else if (matrix.length!=values.length) throw new ExecutionError("Expected "+matrix.length+" values for base '"+segment.substring(0, 1)+"'. Got "+values.length);
                    for (int i=0;i<values.length;i++) {
                         try {
                            matrix[i][index]=Float.parseFloat(values[i]);                           
                        } catch (Exception e) {throw new ExecutionError("Unable to parse expected numeric value for base '"+segment.substring(0, 1)+"': "+values[i]);}
                    }
                    gotAll[index]=true;
                } else throw new ExecutionError("Unknown parameter for Motif: "+segment);
            }
        if (!gotAll[0]) throw new ExecutionError("Missing matrix values for base A");    
        if (!gotAll[1]) throw new ExecutionError("Missing matrix values for base C");    
        if (!gotAll[2]) throw new ExecutionError("Missing matrix values for base G");    
        if (!gotAll[3]) throw new ExecutionError("Missing matrix values for base T");
        motif.setMatrix(matrix);    
        return motif;
    }


    public static float[] parseMinMaxBaselineDefaultParameters(String text) throws ExecutionError {
        float[] min_max_baseline_default=new float[]{0.0f,1.0f,0.0f,0.0f};
        boolean[] isDefined=new boolean[]{false,false,false,false};
        String[] parts=text.split(",");
        if (parts.length==1) { // only a single default value given
            try {
              min_max_baseline_default[3]=Float.parseFloat(parts[0]);
            } catch(Exception e) {throw new ExecutionError("Unable to parse parameter: "+text);}
        } else {
           for(String part:parts) {
               Pattern pattern=Pattern.compile("(min|max|baseline|value|default)\\s*=\\s*(\\S+)");
               Matcher matcher=pattern.matcher(part);
               if (matcher.find()) {
               String paramname=matcher.group(1);
               String valuestring=matcher.group(2);
                 try {
                   int index=0;
                        if (paramname.equals("min")) {index=0;isDefined[0]=true;}
                   else if (paramname.equals("max")) {index=1;isDefined[1]=true;}
                   else if (paramname.equals("baseline")) {index=2;isDefined[2]=true;}
                   else if (paramname.equals("value") || paramname.equals("default")) {index=3;isDefined[3]=true;}
                   float val=Float.parseFloat(valuestring);
                   min_max_baseline_default[index]=val;
                 } catch (Exception e) {throw new ExecutionError("Not a valid number: "+valuestring);}
               } else throw new ExecutionError("Unable to parse: "+part);
           }
        }
        if (min_max_baseline_default[0]>min_max_baseline_default[1]) throw new ExecutionError("Minimum value must be smaller than maximum value");
        if ((min_max_baseline_default[2]>min_max_baseline_default[1] || min_max_baseline_default[2]<min_max_baseline_default[0]) && isDefined[2] && isDefined[0] && isDefined[1]) throw new ExecutionError("Baseline value is outside min-max range");
        if ((min_max_baseline_default[3]>min_max_baseline_default[1] || min_max_baseline_default[3]<min_max_baseline_default[0]) && isDefined[3] && isDefined[0] && isDefined[1]) throw new ExecutionError("Default value is outside min-max range");
        return min_max_baseline_default;
    }
    
    private BackgroundModel parseMarkovModelParameters(String text, String targetName, OperationTask task) throws ExecutionError, InterruptedException {
        if (text.startsWith(MODEL_PREFIX)) {
            String modelName=text.substring(MODEL_PREFIX.length());
            String filename=BackgroundModel.getFilenameForModel(modelName);
            if (filename==null) throw new ExecutionError("Unknown Background Model: "+modelName);
            BufferedReader inputStream=null;
            ArrayList<String> input=new ArrayList<String>();
            try {
                inputStream=new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/priorseditor/engine/resources/"+filename)));
                String line;
                while((line=inputStream.readLine())!=null) {input.add(line);}
            } catch (IOException e) { 
                throw new ExecutionError("An error occurred when loading predefined Background model: ["+e.getClass().getSimpleName()+"] "+e.getMessage(),0);
            } finally {
                try {
                    if (inputStream!=null) inputStream.close();
                } catch (IOException ioe) {System.err.println("SYSTEM ERROR: An error occurred when closing BufferedReader Operation_new.parseMarkovModelParameters(): "+ioe.getMessage());}
            }           
            DataFormat_INCLUSive_Background format=new DataFormat_INCLUSive_Background();   
            BackgroundModel data=new BackgroundModel(targetName);
            try {data=(BackgroundModel)format.parseInput(input, data, null, task);}
            catch (InterruptedException ie) {throw ie;} 
            catch (Exception e) {throw new ExecutionError(e.getMessage());} 
            data.setPredefinedModel(modelName);
            return data;
        } else {
            float[] snf=null;
            float[] matrix=null;
            float[] oligos=null;
            text=text.toUpperCase();
            String[] segments=text.split("\\s*;\\s*");
            int organism=0;
            for (String segment:segments) {
                if (segment.trim().isEmpty()) continue;
                if (segment.startsWith("SNF:") && segment.length()>4) {
                    snf=parseMarkovModelParameters_SNF(segment.substring(4));
                } else if (segment.startsWith("OLIGOS:") && segment.length()>7) {
                    oligos=parseMarkovModelParameters_OligosAndMatrix(segment.substring(7));
                } else if (segment.startsWith("MATRIX:") && segment.length()>7) {
                    matrix=parseMarkovModelParameters_OligosAndMatrix(segment.substring(7));
                } else if (segment.startsWith("ORGANISM:") && segment.length()>9) {
                    try {
                        organism=Integer.parseInt(segment.substring(9));
                    } catch (Exception e) {throw new ExecutionError("Unable to parse expected numeric Organism taxonomy ID: "+segment);}
                } else throw new ExecutionError("Background model parameters should start with either SNF:, OLIGOS: or MATRIX:");
            }
            if (matrix!=null) {
                double log4=Math.log(matrix.length)/Math.log(4);
                int order=(int)log4;
                if (oligos!=null) {
                    int rows=matrix.length/4;
                    if (order<1 && oligos.length!=4) throw new ExecutionError("Expected 4 oligo frequency values for Background Model of order "+order);
                    else if (order>1 && oligos.length!=matrix.length/4) throw new ExecutionError("Expected "+(matrix.length/4)+" oligo frequency values for Background Model of order "+order);
                }
            }
            BackgroundModel newmodel=new BackgroundModel(targetName,snf,oligos,matrix); 
            if (organism!=0) newmodel.setOrganism(organism);
            return newmodel;
        }    
    } 
    
    private float[] parseMarkovModelParameters_SNF(String text) throws ExecutionError {   
        String[] list=text.split("\\s*,\\s*");
        if (list.length!=4) throw new ExecutionError("The number of Background Model SNF values must be 4");
        float values[] = new float[list.length];
        for (int i=0;i<list.length;i++) {
            try {
              values[i]=Float.parseFloat(list[i]);             
            } catch(Exception e) {throw new ExecutionError("Not a valid numeric value: "+list[i]);}
        }
        return values;
    }
    
    private float[] parseMarkovModelParameters_OligosAndMatrix(String text) throws ExecutionError {   
        String[] list=text.split("\\s*,\\s*");
        int matrixSize=list.length;
        double log4=Math.log(matrixSize)/Math.log(4);
        int order=(int)log4;
        if (order!=log4 && matrixSize!=0) throw new ExecutionError("Number of values in Background Model oligo frequencies or transition matrix must be a power of 4");
        float values[] = new float[list.length];
        for (int i=0;i<list.length;i++) {
            try {
              values[i]=Float.parseFloat(list[i]);             
            } catch(Exception e) {throw new ExecutionError("Not a valid numeric value: "+list[i]);}
        }
        return values;
    }

    
    
    
    
    
    private ArrayList<String> parseSequenceCollectionParameters(String text) throws ExecutionError {
        ArrayList<String> list=splitOnComma(text);
        return list;
    }
    
    private MotifCollection parseMotifCollectionParameters(String text, String targetName, OperationTask task) throws ExecutionError, InterruptedException {
       if (text.startsWith(COLLECTION_PREFIX)) {
            String collectionName=text.substring(COLLECTION_PREFIX.length());
            String filename=engine.getFilenameForMotifCollection(collectionName);
            if (filename==null) throw new ExecutionError("Unknown Motif Collection: "+collectionName);
            BufferedReader inputStream=null;
            ArrayList<String> input=new ArrayList<String>();
            try {
                //inputStream=new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/priorseditor/engine/resources/"+filename)));
                inputStream=new BufferedReader(new InputStreamReader(new FileInputStream(new File(filename))));
                String line;
                while((line=inputStream.readLine())!=null) {input.add(line);}
            } catch (IOException e) { 
                throw new ExecutionError("An error occurred when loading predefined Motif Collection: ["+e.getClass().getSimpleName()+"] "+e.getMessage(),0);
            } finally {
                try {
                    if (inputStream!=null) inputStream.close();
                } catch (IOException ioe) {System.err.println("SYSTEM ERROR: An error occurred when closing BufferedReader Operation_new.parseMotifCollectionParameters(): "+ioe.getMessage());}
            }  
            DataFormat format = engine.getDataFormat("PriorsEditorMotif");
            if (format==null) throw new ExecutionError("Unknown Dataformat: PriorsEditorMotif");  
            MotifCollection data=new MotifCollection(targetName);
            try {data=(MotifCollection)format.parseInput(input, data,null, task);}
            catch (InterruptedException ie) {throw ie;} 
            catch (Exception e) {throw new ExecutionError(e.getMessage());} 
            data.setPredefinedCollectionName(collectionName);
            return data;
        } else if (text.startsWith(FROMTRACK_PREFIX)) {
           MotifCollection newDataItem=new MotifCollection(targetName); // empty collection
           String configstring=text.substring(FROMTRACK_PREFIX.length());
           int quorum=1;
           boolean exactly=false;
           boolean percentage=false;
           String trackName="";
           Pattern pattern=Pattern.compile("\\s*(\\S+)\\s*,\\s*support\\s*(>=|=)\\s*(\\d+)(%)?");
           Matcher matcher=pattern.matcher(configstring);
           if (matcher.find()) {
               trackName=matcher.group(1);
               if (matcher.group(2).equals("=")) exactly=true;
               try {
                   quorum=(int)(Float.parseFloat(matcher.group(3)));                   
               } catch (Exception e) {
                   throw new ExecutionError("Unable to parse expected numeric value '"+matcher.group(3)+"'"); 
               }
               if (matcher.group(4)!=null && matcher.group(4).equals("%")) percentage=true;
               //for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
           } else throw new ExecutionError("Unable to parse 'Track' parameter for new Motif Collection");           
           Data trackitem=engine.getDataItem(trackName);
           if (trackitem==null) throw new ExecutionError("Unknown data object '"+trackName+"'");   
           if (!(trackitem instanceof RegionDataset)) throw new ExecutionError("'"+trackName+"' is not a Region dataset");   
           newDataItem.createCollectionFromTrack((RegionDataset)trackitem, quorum, exactly, percentage, engine);
           return newDataItem;           
        } else {
           MotifCollection newDataItem=new MotifCollection(targetName); // empty collection
           ArrayList<String> list=splitOnComma(text);
           for (String motifname:list) {
               int mode=UNION;
               if (motifname.startsWith("+") && motifname.length()>1) {mode=UNION;motifname=motifname.substring(1);}
               if (motifname.startsWith("-") && motifname.length()>1) {mode=SUBTRACTION;motifname=motifname.substring(1);}
               if (motifname.startsWith("&") && motifname.length()>1) {mode=INTERSECTION;motifname=motifname.substring(1);}
               Data dataobject=engine.getDataItem(motifname);
               if (dataobject==null) throw new ExecutionError("Unknown data item: "+motifname);
               else if (!(dataobject instanceof Motif || dataobject instanceof MotifCollection)) throw new ExecutionError("Data item '"+motifname+"' is not a motif or motif collection");
               else {
                        if (mode==SUBTRACTION) subtractFromMotifCollection((MotifCollection)newDataItem,dataobject);
                   else if (mode==INTERSECTION) intersectWithMotifCollection((MotifCollection)newDataItem,dataobject);
                   else addToMotifCollection((MotifCollection)newDataItem,dataobject);
               }
           }
           return newDataItem;
        }
    }
    
    
    private ArrayList<String> parseStringCollectionParameters(String text) throws ExecutionError {
        if (text.trim().isEmpty()) return new ArrayList<String>(0);
        ArrayList<String> list=splitOnComma(text);
        for (int i=0;i<list.size();i++) {
            String element=list.get(i);
            if (element.isEmpty()) throw new ExecutionError("String Collection contains an empty string value (index="+i+")");
            else if (!StringVariable.isValidValue(element)) throw new ExecutionError("The String '"+element+"' is not valid value (might contain illegal characters: "+StringVariable.illegalCharacters()+")");
        }
        return list;
    }
  
  
    
    /** 
     * This method takes a string with comma separated values and returns a list of the values
     * Note that the values themselves can contain commas as long as the full value is within quotes
     * E.g. the string:  CACGTG,"CAC.{2,4}TAT","TATAA",TATAA
     * will return a list with the 4 values:
     *     CACGTG
     *     "CAC.{2,4}TAT"
     *     "TATAA"
     *     TATAA
     * 
     * quotes are kept where applicable
     * 
     */
    private ArrayList<String> splitOnComma(CharSequence text) throws ExecutionError {
        int openedindex=-1;
        int laststart=0;
        while (text.charAt(text.length()-1)==',') text=text.subSequence(0, text.length()-1);
        ArrayList<String> result=new ArrayList<String>();
        for (int i=0;i<text.length()-1;i++) {
            if (text.charAt(i)==',') {
                if (openedindex<0) {
                    String element=(""+text.subSequence(laststart, i)).trim();
                    if (!element.isEmpty()) result.add(element);
                    laststart=i+1;
                }
            }
            else if (text.charAt(i)=='\"') {
                if (openedindex<0) {
                    if (i>0 && text.charAt(i-1)!=',') throw new ExecutionError("Misplaced quote within word after: "+text.subSequence(0, i)); 
                    openedindex=i; // open-quote
                }
                else {
                    if (text.charAt(i+1)!=',') throw new ExecutionError("Misplaced quote near "+text.subSequence(laststart, i+1)+" is not followed by delimiting comma"); 
                    openedindex=-1;
                }
            }
        }
        if (text.charAt(text.length()-1)=='\"' && openedindex<0) throw new ExecutionError("Unmatched quote at end of string");
        else if (text.charAt(text.length()-1)=='\"') openedindex=-1;
        if (openedindex>=0) throw new ExecutionError("Unclosed quote beginning at "+text.subSequence(laststart, text.length()));
        String element=(""+text.subSequence(laststart, text.length())).trim();
        if (!element.isEmpty()) result.add(element);            
        for (int i=0;i<result.size();i++) { // remove quotes now
            String string=result.get(i);
            if (string.startsWith("\"") && string.endsWith("\"")) result.set(i, string.substring(1, string.length()-1));
        }
        return result;
    }
    
    /** Adds a single Sequence or Sequence collection (other) to a target collection */
    private void addToSequenceCollection(SequenceCollection target, Object other) {
        if (other==null) return;
        if (other instanceof Sequence) {
            target.addSequence((Sequence)other);
        } else if (other instanceof SequenceCollection) {
            for (Sequence seq:((SequenceCollection)other).getAllSequences(engine)) {
                target.addSequence(seq);
            }
        } else {
            System.err.println("SYSTEM ERROR: In Operation_new.addToSequenceCollection. Parameter is neither Sequence nor Sequence collection but rather: "+other.getClass().getSimpleName());
        }
    }
    
    /** Removes all sequences from the target collection which is not present in the other Sequence collection (or single sequence) */
    private void intersectWithSequenceCollection(SequenceCollection target, Object other) {
        if (other==null) return;
        if (other instanceof Sequence) {            
            if (target.contains((Sequence)other)) {
                target.clearAll(engine);
                target.addSequence((Sequence)other);
            } else target.clearAll(engine);
        } else if (other instanceof SequenceCollection) {            
            for (Sequence seq:target.getAllSequences(engine)) {
                if (!((SequenceCollection)other).contains(seq)) target.removeSequence(seq);
            }            
        } else {
            System.err.println("SYSTEM ERROR: In Operation_new.intersectWithSequenceCollection. Parameter is neither Sequence nor Sequence collection but rather: "+other.getClass().getSimpleName());
        }
        
    }
     /** Subtracts a single Sequence or Sequence collection (other) from a target collection */
    private void subtractFromSequenceCollection(SequenceCollection target, Object other) {
         if (other==null) return;
        if (other instanceof Sequence) {
            target.removeSequence((Sequence)other);
        } else if (other instanceof SequenceCollection) {
            for (Sequence seq:((SequenceCollection)other).getAllSequences(engine)) {
                target.removeSequence(seq);
            }            
        } else {
            System.err.println("SYSTEM ERROR: In Operation_new.subtractFromSequenceCollection. Parameter is neither Sequence nor Sequence collection but rather: "+other.getClass().getSimpleName());
        }       
    }
    
  /** Adds a single Motif or Motif collection (other) to a target collection */
    private void addToMotifCollection(MotifCollection target, Object other) {
        if (other==null) return;
        if (other instanceof Motif) {
            target.addMotif((Motif)other);
        } else if (other instanceof MotifCollection) {
            for (Motif motif:((MotifCollection)other).getAllMotifs(engine)) {
                target.addMotif(motif);
            }
        } else {
            System.err.println("SYSTEM ERROR: In Operation_new.addToMotifCollection. Parameter is neither Motif nor Motif collection but rather: "+other.getClass().getSimpleName());
        }
    }
    
    /** Removes all motifs from the target collection which is not present in the other Motif collection (or single motif) */
    private void intersectWithMotifCollection(MotifCollection target, Object other) {
        if (other==null) return;
        if (other instanceof Motif) {            
            if (target.contains((Motif)other)) {
                target.clearAll(engine);
                target.addMotif((Motif)other);
            } else target.clearAll(engine);
        } else if (other instanceof MotifCollection) {            
            for (Motif motif:target.getAllMotifs(engine)) {
                if (!((MotifCollection)other).contains(motif)) target.removeMotif(motif);
            }            
        } else {
            System.err.println("SYSTEM ERROR: In Operation_new.intersectWithMotifCollection. Parameter is neither Motif nor Motif collection but rather: "+other.getClass().getSimpleName());
        }
        
    }
     /** Subtracts a single Motif or Motif collection (other) from a target collection */
    private void subtractFromMotifCollection(MotifCollection target, Object other) {
         if (other==null) return;
        if (other instanceof Motif) {
            target.removeMotif((Motif)other);
        } else if (other instanceof MotifCollection) {
            for (Motif motif:((MotifCollection)other).getAllMotifs(engine)) {
                target.removeMotif(motif);
            }            
        } else {
            System.err.println("SYSTEM ERROR: In Operation_new.subtractFromMotifCollection. Parameter is neither Motif nor Motif collection but rather: "+other.getClass().getSimpleName());
        }       
    }    
}
    