/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import java.util.ArrayList;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;
import priorseditor.engine.data.NumericSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_normalize extends FeatureTransformOperation {
    public static final String SUM_TO_ONE="sumToOne";
    public static final String NORMALIZE_TO_RANGE="normalizeToRange";
    public static final String MODE="mode";
    public static final String OLD_MIN="oldMinimum";
    public static final String OLD_MAX="oldMaximum"; //
    public static final String NEW_MIN="newMinimum"; //
    public static final String NEW_MAX="newMaximum"; //
    private static final String OLD_MIN_VALUE="oldMinimumValue";
    private static final String OLD_MAX_VALUE="oldMaximumValue"; //
    private static final String NEW_MIN_VALUE="newMinimumValue"; //
    private static final String NEW_MAX_VALUE="newMaximumValue"; //
    private static final String SEQUENCE_MIN="sequenceMin";
    private static final String SEQUENCE_MAX="sequenceMax";
    private static final String DATASET_MIN="datasetMin";
    private static final String DATASET_MAX="datasetMax";
    private static final String name="normalize";
    private static final String description="Normalizes a dataset by transforming values from one range to another. For region datasets the operation works on region scores";
    private Class[] datasourcePreferences=new Class[]{NumericDataset.class,RegionDataset.class};



    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String toString() {return name;}

    @Override
    public boolean isSubrangeApplicable() {return true;}

    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String mode=(String)task.getParameter(MODE);
        if (mode.equals(NORMALIZE_TO_RANGE)) {
            String oldMinString=(String)task.getParameter(OLD_MIN);
            if (!oldMinString.equals("sequence.min") && !oldMinString.equals("dataset.min") && !oldMinString.equals("sequence.max") && !oldMinString.equals("dataset.max")) {
                Data oldMinData=null;
                oldMinData=engine.getDataItem(oldMinString);
                if (oldMinData==null) {
                    try {
                      float value=Float.parseFloat(oldMinString);
                      oldMinData=new NumericConstant(oldMinString, (float)value);
                   } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+oldMinString+"' neither data nor numeric constant",task.getLineNumber());}
                }
                if (!(oldMinData instanceof NumericConstant || oldMinData instanceof NumericVariable || oldMinData instanceof NumericMapVariable)) throw new ExecutionError("'"+oldMinString+"' is not a valid numeric data object",task.getLineNumber());
                task.setParameter(OLD_MIN_VALUE, oldMinData);
            } else task.setParameter(OLD_MIN_VALUE, oldMinString);
            // -- Old max --
            String oldMaxString=(String)task.getParameter(OLD_MAX);
            //engine.logMessage("Resolve: oldMaxString="+oldMaxString);
            if (!oldMaxString.equals("sequence.min") && !oldMaxString.equals("dataset.min") && !oldMaxString.equals("sequence.max") && !oldMaxString.equals("dataset.max")) {
                Data oldMaxData=null;
                oldMaxData=engine.getDataItem(oldMaxString);
                if (oldMaxData==null) {
                    try {
                      float value=Float.parseFloat(oldMaxString);
                      oldMaxData=new NumericConstant(oldMaxString, (float)value);
                   } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+oldMaxString+"' neither data nor numeric constant",task.getLineNumber());}
                }
                if (!(oldMaxData instanceof NumericConstant || oldMaxData instanceof NumericVariable || oldMaxData instanceof NumericMapVariable)) throw new ExecutionError("'"+oldMaxString+"' is not a valid numeric data object",task.getLineNumber());
                task.setParameter(OLD_MAX_VALUE, oldMaxData);
            } else task.setParameter(OLD_MAX_VALUE, oldMaxString);
            // -- New min --
            String newMinString=(String)task.getParameter(NEW_MIN);
            //engine.logMessage("Resolve: newMinString="+newMinString);
            if (!newMinString.equals("sequence.min") && !newMinString.equals("dataset.min") && !newMinString.equals("sequence.max") && !newMinString.equals("dataset.max")) {
                Data newMinData=null;
                newMinData=engine.getDataItem(newMinString);
                if (newMinData==null) {
                    try {
                      float value=Float.parseFloat(newMinString);
                      newMinData=new NumericConstant(newMinString, (float)value);
                   } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+newMinString+"' neither data nor numeric constant",task.getLineNumber());}
                }
                if (!(newMinData instanceof NumericConstant || newMinData instanceof NumericVariable || newMinData instanceof NumericMapVariable)) throw new ExecutionError("'"+newMinString+"' is not a valid numeric data object",task.getLineNumber());
                task.setParameter(NEW_MIN_VALUE, newMinData);
            } else task.setParameter(NEW_MIN_VALUE, newMinString);
            // -- New max --
            String newMaxString=(String)task.getParameter(NEW_MAX);
            //engine.logMessage("Resolve: newMaxString="+newMaxString);
            if (!newMaxString.equals("sequence.min") && !newMaxString.equals("dataset.min") && !newMaxString.equals("sequence.max") && !newMaxString.equals("dataset.max")) {
                Data newMaxData=null;
                newMaxData=engine.getDataItem(newMaxString);
                if (newMaxData==null) {
                    try {
                      float value=Float.parseFloat(newMaxString);
                      newMaxData=new NumericConstant(newMaxString, (float)value);
                   } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+newMaxString+"' neither data nor numeric constant",task.getLineNumber());}
                }
                if (!(newMaxData instanceof NumericConstant || newMaxData instanceof NumericVariable || newMaxData instanceof NumericMapVariable)) throw new ExecutionError("'"+newMaxString+"' is not a valid numeric data object",task.getLineNumber());
                task.setParameter(NEW_MAX_VALUE, newMaxData);
            } else task.setParameter(NEW_MAX_VALUE, newMaxString);
            FeatureDataset sourcedataset=(FeatureDataset)task.getSourceData();
            int numSequences=sourcedataset.getSize();
            float sequenceMin[]=new float[numSequences];
            float sequenceMax[]=new float[numSequences];
            float datasetMin=Float.MAX_VALUE;
            float datasetMax=-Float.MAX_VALUE;
            for (int i=0;i<numSequences;i++) {
                sequenceMin[i]=Float.MAX_VALUE;
                sequenceMax[i]=-Float.MAX_VALUE;
            }
            for (int i=0;i<sourcedataset.getSize();i++) {
                FeatureSequenceData sequenceData = (FeatureSequenceData)sourcedataset.getSequenceByIndex(i);
                float[] values=new float[]{0,0};
                if (sequenceData instanceof NumericSequenceData) values=((NumericSequenceData)sequenceData).getMinMaxFromData();
                else if (sequenceData instanceof RegionSequenceData) values=((RegionSequenceData)sequenceData).getMinMaxFromData();
                sequenceMin[i]=values[0];
                sequenceMax[i]=values[1];
                if (values[0]<datasetMin) datasetMin=values[0];
                if (values[1]>datasetMax) datasetMax=values[1];
            }
            task.setParameter(SEQUENCE_MIN, sequenceMin);
            task.setParameter(SEQUENCE_MAX, sequenceMax);
            task.setParameter(DATASET_MIN, datasetMin);
            task.setParameter(DATASET_MAX, datasetMax);
        }
    }


    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        String seqname=sourceSequence.getName();
        FeatureDataset dataset=(FeatureDataset)task.getSourceData();
        int index=dataset.getIndexForSequence(seqname);
        float oldMinValue=0;
        float oldMaxValue=0;
        float newMinValue=0;
        float newMaxValue=0;

        String mode=(String)task.getParameter(MODE);
        if (mode==null) mode=Operation_normalize.SUM_TO_ONE;
        if (mode.equals(SUM_TO_ONE)) {
            float sum=0;
            if (targetSequence instanceof NumericSequenceData) {
                for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
                        float oldValue=((NumericSequenceData)targetSequence).getValueAtGenomicPosition(i);
                        sum+=oldValue;
                }
                if (sum==0) return; // nothing to normalize
                for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
                        float oldValue=((NumericSequenceData)targetSequence).getValueAtGenomicPosition(i);
                        float normalizedValue=oldValue/sum;
                        ((NumericSequenceData)targetSequence).setValueAtGenomicPosition(i, normalizedValue);
                }
            } else {
               // targetSequence is not Numeric data -> just ignore?
            }
        } else if (mode.equals(NORMALIZE_TO_RANGE)) {
            // -- Old min --
            Object oldMinObject=task.getParameter(OLD_MIN_VALUE);
            if (oldMinObject instanceof String) {
                     if (((String)oldMinObject).equals("sequence.min")) oldMinValue=((float[])task.getParameter(SEQUENCE_MIN))[index];
                else if (((String)oldMinObject).equals("sequence.max")) oldMinValue=((float[])task.getParameter(SEQUENCE_MAX))[index];
                else if (((String)oldMinObject).equals("dataset.min")) oldMinValue=(Float)task.getParameter(DATASET_MIN);
                else if (((String)oldMinObject).equals("dataset.max")) oldMinValue=(Float)task.getParameter(DATASET_MAX);
            }
            else if (oldMinObject instanceof NumericMapVariable) oldMinValue=(float)((NumericMapVariable)oldMinObject).getValue(seqname).floatValue();
            else if (oldMinObject instanceof NumericVariable) oldMinValue=(float)((NumericVariable)oldMinObject).getValue().floatValue();
            else if (oldMinObject instanceof NumericConstant) oldMinValue=(float)((NumericConstant)oldMinObject).getValue().floatValue();
            //engine.logMessage("Oldmin="+oldMinObject.getClass()+"   OldMinString='"+oldMinObject.toString()+"'    oldMinValue="+oldMinValue);
            // -- Old max --
            Object oldMaxObject=task.getParameter(OLD_MAX_VALUE);
            if (oldMaxObject instanceof String) {
                     if (((String)oldMaxObject).equals("sequence.min")) oldMaxValue=((float[])task.getParameter(SEQUENCE_MIN))[index];
                else if (((String)oldMaxObject).equals("sequence.max")) oldMaxValue=((float[])task.getParameter(SEQUENCE_MAX))[index];
                else if (((String)oldMaxObject).equals("dataset.min")) oldMaxValue=(Float)task.getParameter(DATASET_MIN);
                else if (((String)oldMaxObject).equals("dataset.max")) oldMaxValue=(Float)task.getParameter(DATASET_MAX);
            }
            else if (oldMaxObject instanceof NumericMapVariable) oldMaxValue=(float)((NumericMapVariable)oldMaxObject).getValue(seqname).floatValue();
            else if (oldMaxObject instanceof NumericVariable) oldMaxValue=(float)((NumericVariable)oldMaxObject).getValue().floatValue();
            else if (oldMaxObject instanceof NumericConstant) oldMaxValue=(float)((NumericConstant)oldMaxObject).getValue().floatValue();
            //engine.logMessage("Oldmax="+oldMaxObject.getClass()+"   OldMaxString='"+oldMaxObject.toString()+"'    oldMaxValue="+oldMaxValue);
            // -- New mix --
            Object newMinObject=task.getParameter(NEW_MIN_VALUE);
            if (newMinObject instanceof String) {
                     if (((String)newMinObject).equals("sequence.min")) newMinValue=((float[])task.getParameter(SEQUENCE_MIN))[index];
                else if (((String)newMinObject).equals("sequence.max")) newMinValue=((float[])task.getParameter(SEQUENCE_MAX))[index];
                else if (((String)newMinObject).equals("dataset.min")) newMinValue=(Float)task.getParameter(DATASET_MIN);
                else if (((String)newMinObject).equals("dataset.max")) newMinValue=(Float)task.getParameter(DATASET_MAX);
            }
            else if (newMinObject instanceof NumericMapVariable) newMinValue=(float)((NumericMapVariable)newMinObject).getValue(seqname).floatValue();
            else if (newMinObject instanceof NumericVariable) newMinValue=(float)((NumericVariable)newMinObject).getValue().floatValue();
            else if (newMinObject instanceof NumericConstant) newMinValue=(float)((NumericConstant)newMinObject).getValue().floatValue();

            // -- New max --
            Object newMaxObject=task.getParameter(NEW_MAX_VALUE);
            if (newMaxObject instanceof String) {
                     if (((String)newMaxObject).equals("sequence.min")) newMaxValue=((float[])task.getParameter(SEQUENCE_MIN))[index];
                else if (((String)newMaxObject).equals("sequence.max")) newMaxValue=((float[])task.getParameter(SEQUENCE_MAX))[index];
                else if (((String)newMaxObject).equals("dataset.min")) newMaxValue=(Float)task.getParameter(DATASET_MIN);
                else if (((String)newMaxObject).equals("dataset.max")) newMaxValue=(Float)task.getParameter(DATASET_MAX);
            }
            else if (newMaxObject instanceof NumericMapVariable) newMaxValue=(float)((NumericMapVariable)newMaxObject).getValue(seqname).floatValue();
            else if (newMaxObject instanceof NumericVariable) newMaxValue=(float)((NumericVariable)newMaxObject).getValue().floatValue();
            else if (newMaxObject instanceof NumericConstant) newMaxValue=(float)((NumericConstant)newMaxObject).getValue().floatValue();

            //engine.logMessage("Sequence["+seqname+"]:  calculated seqMin="+((float[])task.getParameter(SEQUENCE_MIN))[index]+",  seqMax="+((float[])task.getParameter(SEQUENCE_MAX))[index]+",  datasetMin="+(Float)task.getParameter(DATASET_MIN)+",  datasetMax="+(Float)task.getParameter(DATASET_MAX));
            //engine.logMessage("Sequence["+seqname+"]:  oldMin="+oldMinValue+",  oldMax="+oldMaxValue+",  newMin="+newMinValue+",  newMax="+newMaxValue);
            float oldrange=oldMaxValue-oldMinValue;
            if (targetSequence instanceof NumericSequenceData) {
                for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
                    if (positionSatisfiesCondition(seqname,i,task)) {
                        float oldValue=((NumericSequenceData)targetSequence).getValueAtGenomicPosition(i);
                        float normalizedValue=newMinValue;
                        if (oldrange!=0) { // if (oldrange==0) throw new ExecutionError("Can not normalize data from zero range");
                            float oldScaleOffset=(oldValue-oldMinValue)/(oldrange);
                            float newScaledOffset=oldScaleOffset*(newMaxValue-newMinValue);
                            normalizedValue+=newScaledOffset;
                        }
                        ((NumericSequenceData)targetSequence).setValueAtGenomicPosition(i, normalizedValue);
                   } // satisfies 'where'-condition
                }
            } else if (targetSequence instanceof RegionSequenceData) {
                ArrayList<Region> regions=((RegionSequenceData)targetSequence).getAllRegions();
                for (Region region:regions) {
                    if (regionSatisfiesCondition(seqname,region,task)) {
                        float oldValue=region.getScore();
                        float normalizedValue=newMinValue;
                        if (oldrange!=0) { // if (oldrange==0) throw new ExecutionError("Can not normalize data from zero range");
                            float oldScaleOffset=(oldValue-oldMinValue)/(oldrange);
                            float newScaledOffset=oldScaleOffset*(newMaxValue-newMinValue);
                            normalizedValue+=newScaledOffset;
                        }
                        region.setScore(normalizedValue);
                   } // satisfies 'where'-condition
                }
            }
        }// end normalize to range
      } // end transformSequence



    }
