/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import priorseditor.engine.OperationTask;
import java.util.ArrayList;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.ProgressListener;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Sequence;
import priorseditor.engine.data.FeatureDataset;
import priorseditor.engine.data.FeatureSequenceData;
import priorseditor.engine.data.BackgroundModel;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.MotifCollection;
import priorseditor.engine.data.NumericMapVariable;
import priorseditor.engine.data.NumericVariable;
import priorseditor.engine.data.OutputData;
import priorseditor.engine.data.SequenceCollection;
import priorseditor.engine.data.StringCollection;
import priorseditor.engine.data.StringVariable;
import priorseditor.engine.dataformat.DataFormat;

/**
 *
 * @author kjetikl
 */
public class Operation_output extends Operation {
    public static final String OUTPUT_FORMAT="format";
    public static final String OUTPUT_FORMAT_PARAMETERS="formatParameters";
    private static final String name="output";
    private static final String description="Outputs data items in selected formats";
    private Class[] datasourcePreferences=new Class[]{FeatureDataset.class,FeatureSequenceData.class,Motif.class,MotifCollection.class,BackgroundModel.class,NumericVariable.class, NumericMapVariable.class,Sequence.class,SequenceCollection.class,StringVariable.class,StringCollection.class};



    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String toString() {return name;}


    @Override
    public boolean execute(final OperationTask task) throws Exception {
        String sourceName=task.getSourceDataName();
        String targetName=task.getTargetDataName();
        if (targetName==null || targetName.isEmpty() || targetName.equals(sourceName)) targetName=engine.getDefaultOutputObjectName();
        String outputformatName=(String)task.getParameter(OUTPUT_FORMAT);
        Data sourceData=engine.getDataItem(sourceName);
        Data targetData=engine.getDataItem(targetName);
        task.setParameter(OperationTask.ENGINE, engine);
        OutputData doubleBuffer;
        if (targetData==null) doubleBuffer=new OutputData(targetName);
        else {
            if (!(targetData instanceof OutputData)) throw new ExecutionError("Incompatible assignment: "+targetData.getName()+" already exists and is not an output object",task.getLineNumber());
            else {
                if (((OutputData)targetData).isHTMLformatted()) throw new ExecutionError("The selected output document is HTML formatted and can not be appended to.\nPlease select a different target name");
                doubleBuffer=(OutputData)targetData.clone();
            }
        }
        if (sourceData==null) throw new ExecutionError("Unable to locate data object '"+sourceName+"'",task.getLineNumber());
        if (!canUseAsSource(sourceData)) throw new ExecutionError("Operation 'output' can not work on '"+sourceName+"'",task.getLineNumber());

        DataFormat formatter=null;
        if (outputformatName==null || outputformatName.isEmpty()) formatter=engine.getDefaultDataFormat(sourceData);
        else formatter=engine.getDataFormat(outputformatName);
        // if (formatter==null) throw new ExecutionError("Unknown output format '"+outputformatName+"'",task.getLineNumber());
        if (formatter!=null && !formatter.canFormatOutput(sourceData)) throw new ExecutionError("Unsupported output format '"+outputformatName+"' for data object '"+sourceName+"'",task.getLineNumber());
        ParameterSettings settings=(ParameterSettings)task.getParameter(Operation_output.OUTPUT_FORMAT_PARAMETERS);
        ProgressListener listener=new ProgressListener() {
            public void processProgressEvent(Object source, int progress) {
                task.setProgress(progress);
            }
        };
// ---> NOTE: A previous design used the deprecated method below to loop through sequences in FeatureDatasets.
//            Now the looping is done by the DataFormat itself
//        if (sourceData instanceof FeatureDataset) {
//            outputFeatureDataset((FeatureDataset)sourceData, doubleBuffer, task, formatter, settings, listener);
//        } else {
            task.setStatusMessage("Executing operation: output");
            if (formatter!=null) {
                if (!formatter.isAppendable() && doubleBuffer.getDocument().getLength()>0) throw new ExecutionError("Text in format '"+formatter.getName()+"' can not be appended to existing output.\nPlease select a different target name");
                try {
                   formatter.addProgressListener(listener);
                   formatter.format(sourceData, doubleBuffer, settings, task);
                }
                catch (ExecutionError ee) {ee.setLineNumber(task.getLineNumber());throw ee;}
                catch (Exception e) {throw e;}
                finally {formatter.removeProgressListener(listener);}
            }
            else doubleBuffer.append(sourceData.output()+"\n","text");
//        }
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        try {engine.updateDataItem(doubleBuffer);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment: "+ce.getMessage(),task.getLineNumber());}
        task.setProgress(100);
        return true;
    }

    @Deprecated
    private void outputFeatureDataset(FeatureDataset sourceData, OutputData targetData, OperationTask task, DataFormat formatter, ParameterSettings settings, ProgressListener listener) throws Exception {
        String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
        if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
        Data seqcol=engine.getDataItem(subsetName);
        if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
        if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
        SequenceCollection sequenceCollection=(SequenceCollection)seqcol;

        int size=sequenceCollection.getNumberofSequences();
        int i=0;
        ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
        for (Sequence sequence:sequences) { // for each sequence
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            String sequenceName=sequence.getName();
            FeatureSequenceData sourceSequence=(FeatureSequenceData)sourceData.getSequenceByName(sequenceName);
            if (formatter!=null) {
                try {
                   formatter.addProgressListener(listener);
                   formatter.format(sourceSequence, targetData, settings, task);
                } catch (Exception e) {throw e;}
                finally {formatter.removeProgressListener(listener);}
            }
            else targetData.append(sourceSequence.output(),"text");
            task.setStatusMessage("Executing operation: output ("+(i+1)+"/"+size+")");
            task.setProgress(i+1, size);
            i++;
            Thread.yield();
            //Thread.sleep(300); // for debug only!! Remove this line later!
        }
    }


}

