/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_physical extends FeatureTransformOperation {
    public static final String WINDOW_SIZE="windowSize"; 
    private static final String WINDOW_SIZE_VALUE="windowSizeValue"; 
    private static final String OLIGO="oligo"; 
    public static final String PHYSICAL_PROPERTY="physhicalProperty"; //
    public static final String ANCHOR="anchor"; //
    public static final String UPSTREAM="upstream"; //
    public static final String DOWNSTREAM="downstream"; //
    public static final String CENTER="center"; //
    private static final String name="physical";
    private static final String description="Creates numeric datasets based on physical properties of DNA sequences";
    private Class[] datasourcePreferences=new Class[]{DNASequenceDataset.class};
   
    private static final String GC_CONTENT="GC-content";
    private static final String AT_CONTENT="AT-content";
    private static final String GC_SKEW="GC-skew";
    private static final String AT_SKEW="AT-skew";
    private static final String STACKING_ENERGY="stacking-energy";
    private static final String PROPELLER_TWIST="propeller-twist";
    private static final String A_PHILICITY="A-philicity";
    private static final String PROTEIN_INDUCED_DEFORMABILITY="protein-induced-deformability";
    private static final String DUPLEX_FREE_ENERGY="duplex-free-energy";
    private static final String DUPLEX_DISRUPT_ENERGY="duplex-disrupt-energy";
    private static final String DNA_DENATURATION="DNA-denaturation";
    private static final String DNA_BENDING_STIFFNESS="DNA-bending-stiffness";
    private static final String B_DNA_TWIST="B-DNA-twist";
    private static final String PROTEIN_DNA_TWIST="protein-DNA-twist";
    private static final String Z_DNA_STABILIZING_ENERGY="Z-DNA-stabilizing-energy";
    private static final String NUCLEOSOME_POSITIONING="nucleosome-position-preference";
    private static final String BENDABILITY="bendability";
    public static final String FREQUENCY="frequency";

 
    
    private HashMap<String,Table> tables;
            
    public Operation_physical() {
        tables=new HashMap<String,Table>();
        Table stackingEnergyTable=new Table();
        stackingEnergyTable.addOligoAndReverse("AA", -5.47f);
        stackingEnergyTable.addOligoAndReverse("AC", -10.51f);
        stackingEnergyTable.addOligoAndReverse("AG", -6.78f);
        stackingEnergyTable.addOligoAndReverse("AT", -6.57f);
        stackingEnergyTable.addOligoAndReverse("CA", -6.57f);
        stackingEnergyTable.addOligoAndReverse("CC", -8.26f);
        stackingEnergyTable.addOligoAndReverse("CG", -9.69f);
        stackingEnergyTable.addOligoAndReverse("GA", -9.81f);
        stackingEnergyTable.addOligoAndReverse("GC", -14.59f);
        stackingEnergyTable.addOligoAndReverse("TA", -3.821f);
        tables.put(STACKING_ENERGY,stackingEnergyTable);

        Table propellerTwistTable=new Table();
        propellerTwistTable.addOligoAndReverse("AA", -18.66f);
        propellerTwistTable.addOligoAndReverse("AC", -13.10f);
        propellerTwistTable.addOligoAndReverse("AG", -14.00f);
        propellerTwistTable.addOligoAndReverse("AT", -15.01f);
        propellerTwistTable.addOligoAndReverse("CA", -9.45f);
        propellerTwistTable.addOligoAndReverse("CC", -8.11f);
        propellerTwistTable.addOligoAndReverse("CG", -10.03f);
        propellerTwistTable.addOligoAndReverse("GA", -13.48f);
        propellerTwistTable.addOligoAndReverse("GC", -11.08f);
        propellerTwistTable.addOligoAndReverse("TA", -11.851f);
        tables.put(PROPELLER_TWIST,propellerTwistTable);        

        Table proteinIndDeformTable=new Table();
        proteinIndDeformTable.addOligoAndReverse("AA", 2.9f);
        proteinIndDeformTable.addOligoAndReverse("AC", 2.3f);
        proteinIndDeformTable.addOligoAndReverse("AG", 2.1f);
        proteinIndDeformTable.addOligoAndReverse("AT", 1.6f);
        proteinIndDeformTable.addOligoAndReverse("CA", 9.8f);
        proteinIndDeformTable.addOligoAndReverse("CC", 6.1f);
        proteinIndDeformTable.addOligoAndReverse("CG", 12.1f);
        proteinIndDeformTable.addOligoAndReverse("GA", 4.5f);
        proteinIndDeformTable.addOligoAndReverse("GC", 4.0f);
        proteinIndDeformTable.addOligoAndReverse("TA", 6.3f);
        tables.put(PROTEIN_INDUCED_DEFORMABILITY,proteinIndDeformTable);        
        
        Table duplexfreenergyTable=new Table();
        duplexfreenergyTable.addOligoAndReverse("AA", -1.2f);
        duplexfreenergyTable.addOligoAndReverse("AC", -1.5f);
        duplexfreenergyTable.addOligoAndReverse("AG", -1.5f);
        duplexfreenergyTable.addOligoAndReverse("AT", -0.9f);
        duplexfreenergyTable.addOligoAndReverse("CA", -1.7f);
        duplexfreenergyTable.addOligoAndReverse("CC", -2.3f);
        duplexfreenergyTable.addOligoAndReverse("CG", -2.8f);
        duplexfreenergyTable.addOligoAndReverse("GA", -1.5f);
        duplexfreenergyTable.addOligoAndReverse("GC", -2.3f);
        duplexfreenergyTable.addOligoAndReverse("TA", -0.9f);
        tables.put(DUPLEX_FREE_ENERGY,duplexfreenergyTable);  
        
        Table duplexdisruptenergyTable=new Table();
        duplexdisruptenergyTable.addOligoAndReverse("AA", 1.9f);
        duplexdisruptenergyTable.addOligoAndReverse("AC", 1.3f);
        duplexdisruptenergyTable.addOligoAndReverse("AG", 1.6f);
        duplexdisruptenergyTable.addOligoAndReverse("AT", 0.9f);
        duplexdisruptenergyTable.addOligoAndReverse("CA", 1.9f);
        duplexdisruptenergyTable.addOligoAndReverse("CC", 3.1f);
        duplexdisruptenergyTable.addOligoAndReverse("CG", 3.6f);
        duplexdisruptenergyTable.addOligoAndReverse("GA", 1.6f);
        duplexdisruptenergyTable.addOligoAndReverse("GC", 3.1f);
        duplexdisruptenergyTable.addOligoAndReverse("TA", 1.5f);
        tables.put(DUPLEX_DISRUPT_ENERGY,duplexdisruptenergyTable);        
        
        Table denaturationTable=new Table();
        denaturationTable.addOligoAndReverse("AA", 66.51f);
        denaturationTable.addOligoAndReverse("AC", 108.80f);
        denaturationTable.addOligoAndReverse("AG", 85.12f);
        denaturationTable.addOligoAndReverse("AT", 72.29f);
        denaturationTable.addOligoAndReverse("CA", 64.92f);
        denaturationTable.addOligoAndReverse("CC", 99.31f);
        denaturationTable.addOligoAndReverse("CG", 88.84f);
        denaturationTable.addOligoAndReverse("GA", 80.03f);
        denaturationTable.addOligoAndReverse("GC", 153.83f);
        denaturationTable.addOligoAndReverse("TA", 50.11f);
        tables.put(DNA_DENATURATION,denaturationTable);        
        
        Table bendingstiffnessTable=new Table();
        bendingstiffnessTable.addOligoAndReverse("AA", 35f);
        bendingstiffnessTable.addOligoAndReverse("AC", 60f);
        bendingstiffnessTable.addOligoAndReverse("AG", 60f);
        bendingstiffnessTable.addOligoAndReverse("AT", 20f);
        bendingstiffnessTable.addOligoAndReverse("CA", 60f);
        bendingstiffnessTable.addOligoAndReverse("CC", 130f);
        bendingstiffnessTable.addOligoAndReverse("CG", 85f);
        bendingstiffnessTable.addOligoAndReverse("GA", 60f);
        bendingstiffnessTable.addOligoAndReverse("GC", 85f);
        bendingstiffnessTable.addOligoAndReverse("TA", 20f);
        tables.put(DNA_BENDING_STIFFNESS,bendingstiffnessTable);        

        Table bdnatwist=new Table();
        bdnatwist.addOligoAndReverse("AA", 35.5f);
        bdnatwist.addOligoAndReverse("AC", 33.1f);
        bdnatwist.addOligoAndReverse("AG", 30.6f);
        bdnatwist.addOligoAndReverse("AT", 43.2f);
        bdnatwist.addOligoAndReverse("CA", 37.7f);
        bdnatwist.addOligoAndReverse("CC", 35.3f);
        bdnatwist.addOligoAndReverse("CG", 31.3f);
        bdnatwist.addOligoAndReverse("GA", 39.6f);
        bdnatwist.addOligoAndReverse("GC", 38.4f);
        bdnatwist.addOligoAndReverse("TA", 31.6f);
        tables.put(B_DNA_TWIST,bdnatwist);        

        Table proteintwist=new Table();
        proteintwist.addOligoAndReverse("AA", 35.1f);
        proteintwist.addOligoAndReverse("AC", 31.5f);
        proteintwist.addOligoAndReverse("AG", 31.9f);
        proteintwist.addOligoAndReverse("AT", 29.3f);
        proteintwist.addOligoAndReverse("CA", 37.3f);
        proteintwist.addOligoAndReverse("CC", 32.9f);
        proteintwist.addOligoAndReverse("CG", 36.1f);
        proteintwist.addOligoAndReverse("GA", 36.3f);
        proteintwist.addOligoAndReverse("GC", 33.6f);
        proteintwist.addOligoAndReverse("TA", 37.8f);
        tables.put(PROTEIN_DNA_TWIST,proteintwist);        
    
        Table zdnastabilizingenergy=new Table();
        zdnastabilizingenergy.addOligoAndReverse("AA", 3.9f);
        zdnastabilizingenergy.addOligoAndReverse("AC", 4.6f);
        zdnastabilizingenergy.addOligoAndReverse("AG", 3.4f);
        zdnastabilizingenergy.addOligoAndReverse("AT", 5.9f);
        zdnastabilizingenergy.addOligoAndReverse("CA", 1.3f);
        zdnastabilizingenergy.addOligoAndReverse("CC", 2.4f);
        zdnastabilizingenergy.addOligoAndReverse("CG", 0.7f);
        zdnastabilizingenergy.addOligoAndReverse("GA", 3.4f);
        zdnastabilizingenergy.addOligoAndReverse("GC", 4.0f);
        zdnastabilizingenergy.addOligoAndReverse("TA", 2.5f);
        tables.put(Z_DNA_STABILIZING_ENERGY,zdnastabilizingenergy); 
        
        Table nucleosome_position=new Table();
        nucleosome_position.addOligoAndReverse("AAA", -36f);
        nucleosome_position.addOligoAndReverse("AAC", -6f);
        nucleosome_position.addOligoAndReverse("AAG", 6f);
        nucleosome_position.addOligoAndReverse("AAT", -30f);
        nucleosome_position.addOligoAndReverse("ACA", 6f);
        nucleosome_position.addOligoAndReverse("ACC", 8f);
        nucleosome_position.addOligoAndReverse("ACG", 8f);
        nucleosome_position.addOligoAndReverse("ACT", 11f);
        nucleosome_position.addOligoAndReverse("AGA", -9f);
        nucleosome_position.addOligoAndReverse("AGC", 25f);
        nucleosome_position.addOligoAndReverse("AGG", 8f);
        nucleosome_position.addOligoAndReverse("ATA", -13f);
        nucleosome_position.addOligoAndReverse("ATC", 7f);
        nucleosome_position.addOligoAndReverse("ATG", 18f);
        nucleosome_position.addOligoAndReverse("CAA", -9f);
        nucleosome_position.addOligoAndReverse("CAC", 17f);
        nucleosome_position.addOligoAndReverse("CAG", -2f);
        nucleosome_position.addOligoAndReverse("CCA", 8f);
        nucleosome_position.addOligoAndReverse("CCC", 13f);
        nucleosome_position.addOligoAndReverse("CCG", 2f);
        nucleosome_position.addOligoAndReverse("CGA", 31f);
        nucleosome_position.addOligoAndReverse("CGC", 25f);
        nucleosome_position.addOligoAndReverse("CTA", -18f);
        nucleosome_position.addOligoAndReverse("CTC", 8f);
        nucleosome_position.addOligoAndReverse("GAA", -12f);
        nucleosome_position.addOligoAndReverse("GAC", 8f);
        nucleosome_position.addOligoAndReverse("GCA", 13f);
        nucleosome_position.addOligoAndReverse("GCC", 45f);
        nucleosome_position.addOligoAndReverse("GGA", -5f);
        nucleosome_position.addOligoAndReverse("GTA", -6f);
        nucleosome_position.addOligoAndReverse("TAA", -20f);
        nucleosome_position.addOligoAndReverse("TCA", 8f);
        tables.put(NUCLEOSOME_POSITIONING,nucleosome_position);        
   
        Table bendability=new Table();
        bendability.addOligoAndReverse("AAA", -0.274f);
        bendability.addOligoAndReverse("AAC", -0.204f);
        bendability.addOligoAndReverse("AAG", -0.081f);
        bendability.addOligoAndReverse("AAT", -0.28f);
        bendability.addOligoAndReverse("ACA", -0.006f);
        bendability.addOligoAndReverse("ACC", -0.032f);
        bendability.addOligoAndReverse("ACG", -0.033f);
        bendability.addOligoAndReverse("ACT", -0.183f);
        bendability.addOligoAndReverse("AGA", 0.027f);
        bendability.addOligoAndReverse("AGC", 0.017f);
        bendability.addOligoAndReverse("AGG", -0.057f);
        bendability.addOligoAndReverse("ATA", 0.182f);
        bendability.addOligoAndReverse("ATC", -0.110f);
        bendability.addOligoAndReverse("ATG", 0.134f);
        bendability.addOligoAndReverse("CAA", 0.015f);
        bendability.addOligoAndReverse("CAC", 0.040f);
        bendability.addOligoAndReverse("CAG", 0.175f);
        bendability.addOligoAndReverse("CCA", -0.246f);
        bendability.addOligoAndReverse("CCC", -0.012f);
        bendability.addOligoAndReverse("CCG", -0.136f);
        bendability.addOligoAndReverse("CGA", -0.003f);
        bendability.addOligoAndReverse("CGC", -0.077f);
        bendability.addOligoAndReverse("CTA", 0.09f);
        bendability.addOligoAndReverse("CTC", 0.031f);
        bendability.addOligoAndReverse("GAA", -0.037f);
        bendability.addOligoAndReverse("GAC", -0.013f);
        bendability.addOligoAndReverse("GCA", 0.076f);
        bendability.addOligoAndReverse("GCC", 0.107f);
        bendability.addOligoAndReverse("GGA", 0.0135f);
        bendability.addOligoAndReverse("GTA", 0.025f);
        bendability.addOligoAndReverse("TAA", 0.068f);
        bendability.addOligoAndReverse("TCA", 0.194f);
        tables.put(BENDABILITY,bendability);        
        
    }            

    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    
    @Override
    public boolean isSubrangeApplicable() {return true;}
    
    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String windowSizeString=(String)task.getParameter(WINDOW_SIZE);
        Data windowSizeData=null;
        windowSizeData=engine.getDataItem(windowSizeString);
        if (windowSizeData==null) {
            try {
              float value=Float.parseFloat(windowSizeString);
              windowSizeData=new NumericConstant(windowSizeString, (float)value);
           } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+windowSizeString+"' neither data nor numeric constant",task.getLineNumber());}         
        }
        task.setParameter(WINDOW_SIZE_VALUE, windowSizeData);
        String property=(String)task.getParameter(PHYSICAL_PROPERTY); 
        if (property.startsWith(FREQUENCY+"-")) {
            if (property.length()<FREQUENCY.length()+2) throw new ExecutionError("Missing nucleotide pattern specification for frequency proprety");
            task.setParameter(OLIGO, property.substring(FREQUENCY.length()+1).toUpperCase()); 
        }
    }
    
    
    @Override
    public boolean execute(OperationTask task) throws Exception {
        if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");
        String sourceDatasetName=task.getSourceDataName();
        String targetDatasetName=task.getTargetDataName();
        FeatureDataset sourceDataset=(FeatureDataset)engine.getDataItem(sourceDatasetName);
        if (sourceDataset==null) throw new ExecutionError("Unable to locate data object '"+sourceDatasetName+"'",task.getLineNumber());
        if (!canUseAsSource(sourceDataset)) throw new ExecutionError(sourceDatasetName+"("+sourceDataset.getTypeDescription()+") is of a type not supported by the '"+getName()+"' operation",task.getLineNumber());
        NumericDataset targetDataset=new NumericDataset(targetDatasetName);
        ArrayList<Data> sequenceList=engine.getAllDataItemsOfType(Sequence.class);
        for (Data seq:sequenceList) {
            NumericSequenceData numericSeq=new NumericSequenceData((Sequence)seq, 0);
            targetDataset.addSequence(numericSeq);
        }
        Condition condition=(Condition)task.getParameter("where");
        if (condition!=null) condition.resolve(engine, task);        
        
        Condition_within within=(Condition_within)task.getParameter("within");
        if (within!=null) within.resolve(engine, task);
        
        resolveParameters(task);
        
        String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
        if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
        Data seqcol=engine.getDataItem(subsetName);
        if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
        if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
        SequenceCollection sequenceCollection=(SequenceCollection)seqcol;

        int i=0;
        ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
        if (isSubrangeApplicable() && within!=null) { // remove sequences with no selection windows (if within-condition is used)
            Iterator iter=sequences.iterator();
            while (iter.hasNext()) {
                Sequence seq = (Sequence) iter.next();
                if (!within.existsSelectionWithinSequence(seq.getName(), task)) iter.remove();
            }           
        }
        int size=sequences.size();
        for (Sequence sequence:sequences) { // for each sequence
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            String sequenceName=sequence.getName();

            FeatureSequenceData sourceSequence=(FeatureSequenceData)sourceDataset.getSequenceByName(sequenceName);
            FeatureSequenceData targetSequence=(FeatureSequenceData)targetDataset.getSequenceByName(sequenceName); // just in case...
            transformSequence(sourceSequence,targetSequence, task);         

            task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
            task.setProgress(i+1, size);
            i++;
            Thread.yield();
        }   
        if (targetDataset instanceof NumericDataset) ((NumericDataset)targetDataset).updateAllowedMinMaxValuesFromData();
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        targetDataset.setIsDerived(true);
        try {engine.updateDataItem(targetDataset);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        return true;
    }
    
    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        String physicalProperty=(String)task.getParameter(PHYSICAL_PROPERTY);
        Data windowSizeData=(Data)task.getParameter(WINDOW_SIZE_VALUE);
        String anchor=(String)task.getParameter(ANCHOR);
        int windowsize=5;   
        String seqname=sourceSequence.getName();
        if (windowSizeData instanceof NumericMapVariable) windowsize=(int)((NumericMapVariable)windowSizeData).getValue(seqname).floatValue();
        else if (windowSizeData instanceof NumericVariable) windowsize=(int)((NumericVariable)windowSizeData).getValue().floatValue();
        else if (windowSizeData instanceof NumericConstant) windowsize=(int)((NumericConstant)windowSizeData).getValue().floatValue();
        if (windowsize==0) throw new ExecutionError("Window size can not be zero");            
        int orientation=sourceSequence.getStrandOrientation();
        String oligo=(String)task.getParameter(OLIGO);
        int reverseOffset=0; // might be set to a larger value to offset window on reverse strand
        if (orientation==Sequence.REVERSE) {
           if (oligo!=null) {
               oligo=PriorsEditorEngine.reverseSequence(oligo); // for reverse strand search
               reverseOffset=oligo.length()-1;
           } else if (tables.containsKey(physicalProperty)) {
               reverseOffset=getOligoLength(physicalProperty)-1;
           }
        }
        for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
            int start=0, end=0;
            if (anchor.equals(UPSTREAM)) {
                start=(orientation==Sequence.DIRECT)?i:i-windowsize+1;
                end=(orientation==Sequence.DIRECT)?i+windowsize-1:i;
            } else if (anchor.equals(DOWNSTREAM)) {
                start=(orientation==Sequence.DIRECT)?i-windowsize+1:i;                
                end=(orientation==Sequence.DIRECT)?i:i+windowsize-1;
            } else {
               if (windowsize%2==0) { // window has even number of bases - anchor left of center
                   int flanksize=(int)(windowsize/2);
                   start=(orientation==Sequence.DIRECT)?i-flanksize+1:i-flanksize;
                   end=(orientation==Sequence.DIRECT)?i+flanksize:i+flanksize-1;                   
               } else { // window has odd number of bases
                   int flanksize=(int)(windowsize/2);
                   start=i-flanksize;
                   end=i+flanksize;
               }
            }
            start+=reverseOffset;
            end+=reverseOffset;
            if (positionSatisfiesCondition(seqname,i,task)) {
              float newvalue=getNewValue((DNASequenceData)sourceSequence,start,end,physicalProperty, orientation, oligo);
              ((NumericSequenceData)targetSequence).setValueAtGenomicPosition(i, newvalue);
           } // satisfies 'where'-condition
        }
    }
    
    /** Start and end are genomic positions */
    private float getNewValue(DNASequenceData sourceSequence, int start, int end, String physicalProperty, int orientation, String oligo) throws ExecutionError {
        if (physicalProperty==null) physicalProperty=GC_CONTENT;
        if (physicalProperty.equalsIgnoreCase(GC_CONTENT)) {
            float GCs=0;
            float counted=0;
            for (int j=start;j<=end;j++) {
                if (j<sourceSequence.getRegionStart() || j>sourceSequence.getRegionEnd()) continue; // outside sequence. just skip
                counted++; // How many bases are included
                char base=sourceSequence.getValueAtGenomicPosition(j);
                if (base=='C' || base=='G' || base=='c' || base=='g') GCs++;
            }
            if (counted>0) return GCs/counted;
            else return 0;          
        } else if (physicalProperty.equalsIgnoreCase(GC_SKEW)) {
            float GCs=0;
            float Gs=0;
            float Cs=0;
            float counted=0;
            for (int j=start;j<=end;j++) {
                if (j<sourceSequence.getRegionStart() || j>sourceSequence.getRegionEnd()) continue; // outside sequence. just skip
                counted++; // How many bases are included
                char base=sourceSequence.getValueAtGenomicPosition(j);
                if (base=='G' || base=='g') {Gs++;GCs++;}
                else if (base=='C' || base=='c') {Cs++;GCs++;}
            }
            float GCdiff=(orientation==Sequence.DIRECT)?(Gs-Cs):(Cs-Gs);
            if (counted>0) return GCdiff/counted;
            else return 0;          
        } else if (physicalProperty.equalsIgnoreCase(AT_CONTENT)) {
            float ATs=0;
            float counted=0;
            for (int j=start;j<=end;j++) {
                if (j<sourceSequence.getRegionStart() || j>sourceSequence.getRegionEnd()) continue; // outside sequence. just skip
                counted++; // How many bases are included
                char base=sourceSequence.getValueAtGenomicPosition(j);
                if (base=='A' || base=='T' || base=='a' || base=='t') ATs++;
            }
            if (counted>0) return ATs/counted;
            else return 0;          
        } else if (physicalProperty.equalsIgnoreCase(AT_SKEW)) {
            float ATs=0;
            float As=0;
            float Ts=0;
            float counted=0;
            for (int j=start;j<=end;j++) {
                if (j<sourceSequence.getRegionStart() || j>sourceSequence.getRegionEnd()) continue; // outside sequence. just skip
                counted++; // How many bases are included
                char base=sourceSequence.getValueAtGenomicPosition(j);
                if (base=='A' || base=='a') {As++;ATs++;}
                else if (base=='T' || base=='t') {Ts++;ATs++;}
            }
            float ATdiff=(orientation==Sequence.DIRECT)?(As-Ts):(Ts-As);
            if (counted>0) return ATdiff/counted;
            else return 0;                    
        } else if (physicalProperty.startsWith(FREQUENCY)) {
            float oligomatch=0;
            float counted=0;
            int oligolength=oligo.length();
            for (int j=start;j<=end;j++) {
                if (j<sourceSequence.getRegionStart() || j>sourceSequence.getRegionEnd()-oligolength+1) continue; // outside sequence. just skip
                counted++; // How many positions are included in the window
                if (oligolength==1) {
                    char c=sourceSequence.getValueAtGenomicPosition(j);
                    if (Character.toUpperCase(c)==oligo.charAt(0)) oligomatch++;
                } else {
                   if (sourceSequence.isDirectMatchAtPosition(j,oligo)) oligomatch++; 
                }                     
            }
            if (counted>0) return oligomatch/counted;
            else return 0;          
        } else {
            Table table=tables.get(physicalProperty);
            int oligolength=getOligoLength(physicalProperty);
            if (table==null) throw new ExecutionError("Unknown physical property function: "+physicalProperty);
            char[] sequence=(char[])sourceSequence.getValueInGenomicInterval(start, end+1); // +1 for dinucleotide. +2 for trinucleotides
            if (sequence==null) return 0; // window wholly or partially outside of sequence
            float result=0;
            for (int j=0;j<sequence.length-oligolength+1;j++) { // -1 for dinucleotides, -2 for trinucleotides
                 Float value=table.get(new String(sequence,j,oligolength).toUpperCase());
                 // no need to reverse sequence since all properties in tables are "symmetric"
                 if (value!=null) result+=value;
            }
            return result;             
        }

    }
    
    
    /** Returns the length of the n-nucleotide words that are used for calculating the given physical property 
     *  This is usually either a di- or trinucleotide (or single nucleotide for frequencies).
     *  Note that this value is different from the window-length
     */
    private int getOligoLength(String property) {
             if (property.equals(BENDABILITY) || property.equals(NUCLEOSOME_POSITIONING)) return 3;
        else return 2;
    }
    
    
    public static String[] getPhysicalProperties() {return new String[]{GC_CONTENT,GC_SKEW,AT_CONTENT,AT_SKEW,FREQUENCY,STACKING_ENERGY,PROPELLER_TWIST,NUCLEOSOME_POSITIONING,BENDABILITY,PROTEIN_INDUCED_DEFORMABILITY,DUPLEX_FREE_ENERGY,DUPLEX_DISRUPT_ENERGY,DNA_DENATURATION,DNA_BENDING_STIFFNESS,B_DNA_TWIST,PROTEIN_DNA_TWIST,Z_DNA_STABILIZING_ENERGY};}
    
    private class Table extends HashMap<String,Float> {
         private void addOligoAndReverse(String oligo, Float value) {
             put(oligo, value);
             put(PriorsEditorEngine.reverseSequence(oligo),value);                    
         }
         
    }

}
