/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import priorseditor.engine.OperationTask;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.FeatureSequenceData;
import priorseditor.engine.data.NumericVariable;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.RegionSequenceData;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.data.NumericSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_rescore extends FeatureTransformOperation {
    public static final String RESCORE_TYPE="rescoreType"; // The RESCORE_TYPE property should be set to WEIGHT_BY or SET_TO
    public static final String WEIGHT_BY="weight by"; // value for RESCORE_TYPE
    public static final String SET_TO="set to";  // value for RESCORE_TYPE
    
    private static final String name="rescore";
    private static final String description="Adjusts the score of regions based on the values of a numeric value track within the region";
    private Class[] datasourcePreferences=new Class[]{RegionDataset.class};

    public static final String RESCORE_DATA_NAME="rescoreDataName"; // textual representation of the numeric dataset or numerical data/value used for weighting by the rescore operation
    public static final String RESCORE_DATA="rescoreData"; // a data object representation of the numeric dataset or numerical data/value used for weighting by the rescore operation
    public static final String RESCORE_OPERATOR="operator"; // textual representation of the weight operator (min,max,average)


    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    
    @Override
    public boolean isSubrangeApplicable() {return true;}
    
    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String rescoreDataName=(String)task.getParameter(RESCORE_DATA_NAME);
        Data rescoreData=null;
        rescoreData=engine.getDataItem(rescoreDataName);
        if (rescoreData==null) {
            try {
               float value=Float.parseFloat(rescoreDataName);
               rescoreData=new NumericVariable(rescoreDataName, value);
            } catch (NumberFormatException e) {throw new ExecutionError("Unrecognized token '"+rescoreDataName+"' is neither a data object nor a numeric constant",task.getLineNumber());}            
        }       
        if (!(rescoreData instanceof NumericDataset || rescoreData instanceof NumericVariable)) throw new ExecutionError("'"+rescoreDataName+"' is not a Numeric Dataset or scalar Numeric value",task.getLineNumber());               
        task.setParameter(RESCORE_DATA, rescoreData);        
    }

    
    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        ArrayList<Region> list = ((RegionSequenceData)targetSequence).getAllRegions();
        String seqName=targetSequence.getName();
        String rescoreType=(String)task.getParameter(RESCORE_TYPE);
        if (rescoreType==null || rescoreType.isEmpty()) rescoreType=WEIGHT_BY;
        String operator=(String)task.getParameter(RESCORE_OPERATOR);
        Object rescoreData=task.getParameter(RESCORE_DATA);
        NumericSequenceData numericSequence=null;
        float numericWeightValue=0;
        if (rescoreData instanceof NumericDataset) {
            NumericDataset numericDataset=(NumericDataset)rescoreData;
            numericSequence=(NumericSequenceData)numericDataset.getSequenceByName(sourceSequence.getName());
        } else if (rescoreData instanceof NumericVariable) numericWeightValue=((NumericVariable)rescoreData).getValue();
        for (Region region:list) {
            if (regionSatisfiesCondition(seqName, region, task)) {
                float weightValue=1;
                int relativeStart=region.getRelativeStart();
                int relativeEnd=region.getRelativeEnd();
                     if (operator.equals("value")) weightValue=numericWeightValue;
                else if ((operator.equals("min") || operator.equals("minimum")) && numericSequence!=null) weightValue=numericSequence.getMinValueInInterval(relativeStart, relativeEnd);
                else if ((operator.equals("max") || operator.equals("maximum")) && numericSequence!=null) weightValue=numericSequence.getMaxValueInInterval(relativeStart, relativeEnd);
                else if ((operator.equals("avg") || operator.equals("average")) && numericSequence!=null) weightValue=numericSequence.getAverageValueInInterval(relativeStart, relativeEnd);
                else if (operator.equals("median") && numericSequence!=null) weightValue=numericSequence.getMedianValueInInterval(relativeStart, relativeEnd);
                else if (operator.equals("sum") && numericSequence!=null) weightValue=numericSequence.getSumValueInInterval(relativeStart, relativeEnd);
                if (rescoreType.equals(SET_TO)) region.setScore(weightValue);     
                else region.setScore(region.getScore()*weightValue);
            }            
        } // end: for each region
    }
    

    private void transformSequenceBasedOnRank(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        ArrayList<Region> list = ((RegionSequenceData)targetSequence).getAllRegions();
        String seqName=targetSequence.getName();
        RegionComparator comparator=new RegionComparator();
        Collections.sort(list, comparator);
        float score=0;
        for (Region region:list) {
            score+=1; 
            region.setScore(score);                                 
        } // end: for each region
    }
    
    
    private class RegionComparator implements Comparator<Region> {
        @Override
        public int compare(Region o1, Region o2) {
             float score1=o1.getScore();
             float score2=o2.getScore();
             if (score1<score2) return -1;
             else if (score1>score2) return 1;
             else return 0;             
        } // end compare              
    } // end RegionComparator    
    
}
