/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_set extends FeatureTransformOperation {
    public static final String VALUE_STRING="value"; // 
    private static final String VALUE_DATA="valueData";
    private static final String name="set";
    private static final String description="Sets the value of selected bases to a specified number";
    private Class[] datasourcePreferences=new Class[]{NumericDataset.class};



    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    
    @Override
    public boolean isSubrangeApplicable() {return true;}
    
    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String valueDataString=(String)task.getParameter(Operation_set.VALUE_STRING);
        Data valueData=null;
        valueData=engine.getDataItem(valueDataString);
        if (valueData==null) {
            try {
              float value=Float.parseFloat(valueDataString);
              valueData=new NumericConstant(valueDataString, (float)value);
           } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+valueDataString+"' neither data nor numeric constant",task.getLineNumber());}         
        }
        if (!(valueData instanceof NumericConstant || valueData instanceof NumericVariable || valueData instanceof NumericMapVariable || valueData instanceof NumericDataset)) throw new ExecutionError("'"+valueDataString+"' is not a valid numeric value",task.getLineNumber());          
        task.setParameter(VALUE_DATA, valueData);
    }
           
    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        Data valueData=(Data)task.getParameter(VALUE_DATA);
        Data value;   
        if (valueData instanceof NumericDataset) value=((NumericDataset)valueData).getSequenceByName(sourceSequence.getName());
        else value=valueData;
        String seqname=sourceSequence.getName();
        for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
            if (positionSatisfiesCondition(seqname,i,task)) {
              float newValue=0;
              if (value instanceof NumericConstant) {
                  newValue=((NumericConstant)value).getValue();
              } else if (value instanceof NumericMapVariable) {
                  newValue=((NumericMapVariable)value).getValue(seqname);
              } else if (value instanceof NumericVariable) {
                  newValue=((NumericVariable)value).getValue();
              } else if (value instanceof NumericSequenceData) {
                  newValue=((NumericSequenceData)value).getValueAtGenomicPosition(i);
              }
              ((NumericSequenceData)targetSequence).setValueAtGenomicPosition(i, newValue);
           } // satisfies 'where'-condition
        }
    }
    
    
}
