/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import priorseditor.engine.OperationTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;
import priorseditor.engine.data.FeatureSequenceData;

/**
 *
 * @author kjetikl
 */
public class Operation_smooth extends FeatureTransformOperation {
    public static final String WINDOW_SIZE="windowSize"; 
    private static final String WINDOW_SIZE_VALUE="windowSizeValue"; 
    public static final String WINDOW_TYPE="windowType"; //
    public static final String ANCHOR="anchor"; //
    public static final String UPSTREAM="upstream"; //
    public static final String DOWNSTREAM="downstream"; //
    public static final String CENTER="center"; //
    private static final String name="smooth";
    private static final String description="Smooths Numeric Datasets with a sliding window function";
    private Class[] datasourcePreferences=new Class[]{NumericDataset.class};
   


    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    
    @Override
    public boolean isSubrangeApplicable() {return true;}
    
    @Override
    public void resolveParameters(OperationTask task) throws Exception {
        String windowSizeString=(String)task.getParameter(WINDOW_SIZE);
        Data windowSizeData=null;
        windowSizeData=engine.getDataItem(windowSizeString);
        if (windowSizeData==null) {
            try {
              float value=Float.parseFloat(windowSizeString);
              windowSizeData=new NumericConstant(windowSizeString, (float)value);
           } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+windowSizeString+"' neither data nor numeric constant",task.getLineNumber());}         
        }
        if (!(windowSizeData instanceof NumericConstant || windowSizeData instanceof NumericVariable || windowSizeData instanceof NumericMapVariable)) throw new ExecutionError("'"+windowSizeString+"' is not a valid numeric value",task.getLineNumber());                  
        task.setParameter(WINDOW_SIZE_VALUE, windowSizeData);
    }

    
    @Override
    public void transformSequence(FeatureSequenceData sourceSequence, FeatureSequenceData targetSequence, OperationTask task) throws Exception {
        String windowType=(String)task.getParameter(WINDOW_TYPE);
        Data windowSizeData=(Data)task.getParameter(WINDOW_SIZE_VALUE);
        String anchor=(String)task.getParameter(ANCHOR);
        int windowsize=5;   
        String seqname=sourceSequence.getName();
        if (windowSizeData instanceof NumericMapVariable) windowsize=(int)((NumericMapVariable)windowSizeData).getValue(seqname).floatValue();
        else if (windowSizeData instanceof NumericVariable) windowsize=(int)((NumericVariable)windowSizeData).getValue().floatValue();
        else if (windowSizeData instanceof NumericConstant) windowsize=(int)((NumericConstant)windowSizeData).getValue().floatValue();
        float[] window=null;
        if (windowsize==0) throw new ExecutionError("Window size can not be zero");
        if (windowType.equalsIgnoreCase("gaussian")) {
            // the gaussian window function is based on code from Matlab 7 which is further based on the paper:  fredric j. harris "On the Use of Windows for Harmonic Analysis with the Discrete Fourier Transform" Proceedings of the IEEE, Vol. 66, No. 1, January 1978
            window=new float[windowsize];
            float alpha=2.5f; //
            float total=0;
            for (int i=0;i<windowsize;i++) {
                float k=(-((float)windowsize-1f)/2f)+i; 
                float halfwindow=((float)windowsize)/2f;                
                float powarg=alpha*k/halfwindow;
                float pow=(float)(-0.5*Math.pow(powarg,2));
                float value=(float)Math.exp(pow); 
                //System.err.println("k=>"+k+"  powarg="+powarg+"  pow="+pow+"  value="+value);
                window[i]=value;
                total+=window[i];
            }    
            for (int i=0;i<windowsize;i++) { // normalize weights so they sum to 1
                window[i]=window[i]/total;
            }            
        } else if (windowType.equalsIgnoreCase("bartlett")) {
            window=new float[windowsize];    
            float half=((int)(windowsize/2))+1;            
            for (int i=0;i<half;i++) {
                window[i]=(1f/half)*(i+1);
                window[windowsize-(i+1)]=(1f/half)*(i+1);
            }
            float total=0;
            for (int i=0;i<windowsize;i++) {
                total+=window[i];
            }             
            for (int i=0;i<windowsize;i++) { // normalize weights so they sum to 1
                window[i]=window[i]/total;
            }             
        } 
            
        int orientation=sourceSequence.getStrandOrientation();
        for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
            int start=0, end=0;
            if (anchor.equals(UPSTREAM)) {
                start=(orientation==Sequence.DIRECT)?i:i-windowsize+1;
                end=(orientation==Sequence.DIRECT)?i+windowsize-1:i;
            } else if (anchor.equals(DOWNSTREAM)) {
                start=(orientation==Sequence.DIRECT)?i-windowsize+1:i;                
                end=(orientation==Sequence.DIRECT)?i:i+windowsize-1;
            } else {
               if (windowsize%2==0) { // window has even number of bases - anchor left of center
                   int flanksize=(int)(windowsize/2);
                   start=(orientation==Sequence.DIRECT)?i-flanksize+1:i-flanksize;
                   end=(orientation==Sequence.DIRECT)?i+flanksize:i+flanksize-1;                   
               } else { // window has odd number of bases
                   int flanksize=(int)(windowsize/2);
                   start=i-flanksize;
                   end=i+flanksize;
               }
            }
            if (windowType.equalsIgnoreCase("shift")) {
                if (anchor.equals(DOWNSTREAM)) {
                    if (orientation==Sequence.REVERSE) start=end;
                } else {
                    if (orientation==Sequence.DIRECT) start=end;
                }
            }
            if (positionSatisfiesCondition(seqname,i,task)) {
              float newvalue=getNewValue((NumericSequenceData)sourceSequence,sourceSequence.getRelativePositionFromGenomic(start),sourceSequence.getRelativePositionFromGenomic(end),windowType,window);
              ((NumericSequenceData)targetSequence).setValueAtGenomicPosition(i, newvalue);
           } // satisfies 'where'-condition
        }
    }
    
    private float getNewValue(NumericSequenceData sourceSequence, int start, int end, String windowType, float[] window) throws ExecutionError {
        if (windowType==null) windowType="uniform";
        if (windowType.equalsIgnoreCase("uniform")) {
            float result=0;
            float counted=0;
            for (int j=start;j<=end;j++) {
                if (j<0 || j>=sourceSequence.getSize()) continue; // outside sequence. just skip
                counted++; // How many bases are included
                result+=sourceSequence.getValueAtRelativePosition(j);
            }
            if (counted>0) result=result/counted;
            return result;            
        } else if (windowType.equalsIgnoreCase("shift")) {
            if (start<0 || start>=sourceSequence.getSize()) return 0; // outside sequence. just skip
            else return sourceSequence.getValueAtRelativePosition(start);
                      
        } else if (windowType.equalsIgnoreCase("sum")) {
            float result=0;
            for (int j=start;j<=end;j++) {
                if (j<0 || j>=sourceSequence.getSize()) continue; // outside sequence. just skip
                result+=sourceSequence.getValueAtRelativePosition(j);
            }
            return result;            
        } else if (windowType.equalsIgnoreCase("minimum")) {
            float minimum=Float.MAX_VALUE;
            for (int j=start;j<=end;j++) {
                if (j<0 || j>=sourceSequence.getSize()) continue; // outside sequence. just skip
                float value=sourceSequence.getValueAtRelativePosition(j);
                if (value<minimum) minimum=value;
            }
            return minimum;             
        } else if (windowType.equalsIgnoreCase("maximum")) {
            float maximum=-Float.MAX_VALUE;
            for (int j=start;j<=end;j++) {
                if (j<0 || j>=sourceSequence.getSize()) continue; // outside sequence. just skip
                float value=sourceSequence.getValueAtRelativePosition(j);
                if (value>maximum) maximum=value;
            }
            return maximum;              
        } else if (windowType.equalsIgnoreCase("gaussian") || windowType.equalsIgnoreCase("bartlett")) {
            float result=0;
            int index=0;
            for (int j=start;j<=end;j++) {
                float value=0;
                if (j>=0 && j<sourceSequence.getSize()) value=sourceSequence.getValueAtRelativePosition(j); // outside sequence set to 0
                result+=value*window[index];
                index++;
            }
            return result;
        } else throw new ExecutionError("Unknown window function: "+windowType);

    }
    
    
    public static String[] getWindowTypes() {return new String[]{"Uniform","Bartlett","Gaussian","Sum","Minimum","Maximum","Shift"};}
}
