/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.operations;

import priorseditor.engine.OperationTask;
import java.util.ArrayList;
import java.util.Iterator;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;


/**
 *
 * @author kjetikl
 */
public class Operation_statistic extends Operation {
    public static final String STATISTIC_FUNCTION="statistic"; //
    private static final String name="statistic";
    private static final String description="Calculates a simple statistic (such as maximum or average value) for each sequence in a dataset";
    private Class[] datasourcePreferences=new Class[]{NumericDataset.class, RegionDataset.class};


    @Override
    public Class[] getDataSourcePreferences() {
        return datasourcePreferences;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getName() {
        return name;
    }
    
    @Override
    public String toString() {return name;}
    
    @Override
    public boolean isSubrangeApplicable() {return true;}
    
    public void resolveParameters(OperationTask task) throws Exception {
        String statistic=(String)task.getParameter(STATISTIC_FUNCTION);
        if (statistic==null) throw new ExecutionError("Missing statistic function");
        if (!isKnownFunction(statistic)) throw new ExecutionError("Unknown statistic function");        
    }
    
    public void transformSequence(FeatureSequenceData sourceSequence, NumericMapVariable targetdata, OperationTask task) throws Exception {
        String statistic=(String)task.getParameter(STATISTIC_FUNCTION);        
        String sequenceName=sourceSequence.getName();
        if (sourceSequence instanceof NumericSequenceData) {
            if (statistic.equalsIgnoreCase("minimum")) {
                float min=Float.MAX_VALUE;
                for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
                    if (!positionSatisfiesCondition(sequenceName, i, task)) continue;
                    float value=((NumericSequenceData)sourceSequence).getValueAtGenomicPosition(i);
                    if (value<min) min=value;
                }
                targetdata.setValue(sequenceName, min);
                if (min<targetdata.getValue()) targetdata.setDefaultValue(min);
            } else if (statistic.equalsIgnoreCase("maximum")) {
                float max=-Float.MAX_VALUE;
                for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
                    if (!positionSatisfiesCondition(sequenceName, i, task)) continue;
                    float value=((NumericSequenceData)sourceSequence).getValueAtGenomicPosition(i);
                    if (value>max) max=value;
                }
                targetdata.setValue(sequenceName, max);
                if (max>targetdata.getValue()) targetdata.setDefaultValue(max);
            } else if (statistic.equalsIgnoreCase("average")) {
                float sum=0;
                int basecount=0;
                for (int i=sourceSequence.getRegionStart();i<=sourceSequence.getRegionEnd();i++) {
                    if (!positionSatisfiesCondition(sequenceName, i, task)) continue;
                    float value=((NumericSequenceData)sourceSequence).getValueAtGenomicPosition(i);
                    sum+=value;
                    basecount++;
                }
                float average=sum/(float)basecount;
                targetdata.setValue(sequenceName, average);
                float globalsum=(Float)task.getParameter("GlobalSum");
                task.setParameter("GlobalSum", globalsum+sum);
                int globalbasecount=(Integer)task.getParameter("GlobalBaseCount");
                task.setParameter("GlobalBaseCount", globalbasecount+basecount);                 
            } 
        } else { // sourceSequence instanceof RegionSequenceData
            ArrayList<Region> regions=((RegionSequenceData)sourceSequence).getAllRegions();
            Iterator iter=regions.iterator();
            while (iter.hasNext()) {
                Region region=(Region)iter.next();
                if (!regionSatisfiesCondition(sequenceName, region, task)) iter.remove();
            }
            if (statistic.equalsIgnoreCase("minimum")) {
                float min=Float.MAX_VALUE;
                for (Region region:regions) {
                    float value=region.getScore();
                    if (value<min) min=value;
                }
                targetdata.setValue(sequenceName, min);
                if (min<targetdata.getValue()) targetdata.setDefaultValue(min);
            } else if (statistic.equalsIgnoreCase("maximum")) {
                float max=-Float.MAX_VALUE;
                for (Region region:regions) {
                    float value=region.getScore();
                    if (value>max) max=value;
                }
                targetdata.setValue(sequenceName, max);
                if (max>targetdata.getValue()) targetdata.setDefaultValue(max);
            } else if (statistic.equalsIgnoreCase("average")) {
                float sum=0;
                int regioncount=0;
                for (Region region:regions) {
                    float value=region.getScore();
                    sum+=value;
                    regioncount++;
                }
                float average=sum/(float)regioncount;
                targetdata.setValue(sequenceName, average);
                float globalsum=(Float)task.getParameter("GlobalSum");
                task.setParameter("GlobalSum", globalsum+sum);
                int globalbasecount=(Integer)task.getParameter("GlobalBaseCount");
                task.setParameter("GlobalBaseCount", globalbasecount+regioncount);               
            }            
        }
    }   

    
    @Override
    public boolean execute(OperationTask task) throws Exception {
        if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");
        String sourceDatasetName=task.getSourceDataName();
        String targetDataName=task.getTargetDataName();
        FeatureDataset sourceDataset=(FeatureDataset)engine.getDataItem(sourceDatasetName);
        if (sourceDataset==null) throw new ExecutionError("Unable to locate data object '"+sourceDatasetName+"'",task.getLineNumber());
        if (!canUseAsSource(sourceDataset)) throw new ExecutionError(sourceDatasetName+"("+sourceDataset.getTypeDescription()+") is of a type not supported by the '"+getName()+"' operation",task.getLineNumber());

        NumericMapVariable targetData=new NumericMapVariable(targetDataName, 0);
        ArrayList<Data>allsequences=engine.getAllDataItemsOfType(Sequence.class);
        for (Data sequence:allsequences) {
            targetData.setValue(sequence.getName(),0); // initialize all sequences to 0. Or should it be NaN?
        }
                                
        Condition condition=(Condition)task.getParameter("where");
        if (condition!=null) condition.resolve(engine, task);  
        
        Condition_within within=(Condition_within)task.getParameter("within");
        if (within!=null) within.resolve(engine, task);        
        
        String subsetName=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
        if (subsetName==null || subsetName.isEmpty()) subsetName=engine.getDefaultSequenceCollectionName();
        Data seqcol=engine.getDataItem(subsetName);
        if (seqcol==null) throw new ExecutionError("No such collection: '"+subsetName+"'",task.getLineNumber());
        if (!(seqcol instanceof SequenceCollection)) throw new ExecutionError(subsetName+" is not a sequence collection",task.getLineNumber());
        SequenceCollection sequenceCollection=(SequenceCollection)seqcol;
        task.setParameter(OperationTask.SOURCE, sourceDataset);
        task.setParameter(OperationTask.TARGET, targetData);
        task.setParameter(OperationTask.SEQUENCE_COLLECTION_NAME, sequenceCollection);
        resolveParameters(task);
        
        int i=0;
        ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
        if (isSubrangeApplicable() && within!=null) { // remove sequences with no selection windows (if within-condition is used)
            Iterator iter=sequences.iterator();
            while (iter.hasNext()) {
                Sequence seq = (Sequence) iter.next();
                if (!within.existsSelectionWithinSequence(seq.getName(), task)) iter.remove();
            }           
        }

        int size=sequences.size();
        if (size>0) {
            String statistic=(String)task.getParameter(STATISTIC_FUNCTION);
                 if (statistic.equalsIgnoreCase("minimum")) targetData.setDefaultValue(Float.MAX_VALUE);
            else if (statistic.equalsIgnoreCase("maximum")) targetData.setDefaultValue(-Float.MAX_VALUE);
            else if (statistic.equalsIgnoreCase("average")) {
                task.setParameter("GlobalSum", new Float(0));
                task.setParameter("GlobalBaseCount", new Integer(0));
            }
        }
        for (Sequence sequence:sequences) { // for each sequence
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            String sequenceName=sequence.getName();

            FeatureSequenceData sourceSequence=(FeatureSequenceData)sourceDataset.getSequenceByName(sequenceName);
            transformSequence(sourceSequence,targetData, task);         

            task.setStatusMessage("Executing operation: "+task.getOperationName()+" ("+(i+1)+"/"+size+")");
            task.setProgress(i+1, size);
            i++;
            Thread.yield();
        }   
        if (((String)task.getParameter(STATISTIC_FUNCTION)).equalsIgnoreCase("average")) {
             float sum=(Float)task.getParameter("GlobalSum");
             int count=(Integer)task.getParameter("GlobalBaseCount");
             float average=sum/(float)count;
             targetData.setDefaultValue(average);
        }
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        try {engine.updateDataItem(targetData);} catch (ClassCastException ce) {throw new ExecutionError("Incompatible assignment:"+ce.getMessage(),task.getLineNumber());}
        return true;
    }
       
    
    
    protected boolean positionSatisfiesCondition(String sequencename, int pos, OperationTask task) throws Exception {
        Condition_position condition=(Condition_position)task.getParameter("where");
        Condition_within within=(Condition_within)task.getParameter("within");  
        if (within!=null) {
            if (!within.isConditionSatisfied(sequencename, pos, task)) return false;
        }
        if (condition==null) return true; // no where condition is set, so the transform should be applied everywhere
        else return condition.isConditionSatisfied(sequencename, pos, task);
    }
    
   
    protected boolean regionSatisfiesCondition(String sequencename, Region region, OperationTask task) throws Exception {
        Condition_region condition=(Condition_region)task.getParameter("where");
        Condition_within within=(Condition_within)task.getParameter("within");  
        if (within!=null) {
            if (!within.isConditionSatisfied(sequencename, region.getGenomicStart(),region.getGenomicEnd(), task)) return false;
        }
        if (condition==null) return true; // no where condition is set, so the transform should be applied everywhere
        else return condition.isConditionSatisfied(sequencename, region, task);
    }    

    
    public static boolean isKnownFunction(String statistic) {
        for (String string:getStatisticFunctions()) {
            if (statistic.equalsIgnoreCase(string)) return true;
        }
        return false;
    }
            
    public static String[] getStatisticFunctions() {return new String[]{"minimum","maximum","average"};}

}
