/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.protocol;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import priorseditor.engine.operations.Condition;
import priorseditor.engine.operations.Condition_region;
import priorseditor.engine.operations.Condition_position;
import priorseditor.engine.OperationTask;
import priorseditor.engine.operations.Condition_within;

/**
 *
 * @author kjetikl
 */
public abstract class StandardOperationParser extends OperationParser {
    
    protected String getCommandString_condition(Condition condition) {
        if (condition==null) return "";
        if (condition instanceof Condition_position) return getCommandString_condition((Condition_position)condition);
        else if (condition instanceof Condition_region) return getCommandString_condition((Condition_region)condition);
        else if (condition instanceof Condition_within) return getCommandString_condition((Condition_within)condition);
        else return "<unknown condition type="+condition.getClass().toString()+">";
    }
    
    private String getCommandString_condition(Condition_position condition) { // 
        String operandAstring=condition.getOperandAString();
        if (operandAstring==null) return "";
        boolean whereNot=condition.negateAll();
        String comparator=condition.getComparator();  
        String operandBstring=condition.getOperandBString();               
        String notString=(whereNot) ? "not " : "";
        String msg=notString;
        if (comparator.equals("inside")) msg+="inside "+operandAstring;
        else msg+=operandAstring+" "+comparator+" "+operandBstring;
        return msg;
    }
    
    private String getCommandString_condition(Condition_region condition) { // 
        String compareProperty=condition.getCompareProperty();
        if (compareProperty==null) return "";
        boolean whereNot=condition.negateAll();
        String comparator=condition.getComparator();  
        String operandAstring=condition.getOperandAString();               
        String operandBstring=condition.getOperandBString();               
        String notString=(whereNot) ? "not " : "";
        String msg=notString;
             if (compareProperty.equals("type")) msg+="region type "+comparator+" "+operandBstring;
        else if (compareProperty.equals("score") || compareProperty.equals("length")) msg+="region "+compareProperty+" "+comparator+" "+operandBstring;
        else if (compareProperty.equals("overlaps")) msg+="region overlaps "+operandBstring;
        else if (compareProperty.equals("distance to")) msg+="region distance to "+operandAstring+" "+comparator+" "+operandBstring;
        else if (operandAstring!=null) msg+="region "+compareProperty+" "+operandAstring+" "+comparator+" "+operandBstring;
        return msg;
    }    
    
    private String getCommandString_condition(Condition_within condition) { // 
        String operandAstring=condition.getOperandAString();
        if (operandAstring==null) return "";
        //boolean whereNot=condition.negateAll();
        String msg=" within ["+operandAstring+"]";
        return msg;
    }    
    
    
    
    
    
    protected String getCommandString_inCollection(OperationTask task) {
        String subset=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);  
        if (subset==null || subset.isEmpty()) return "";
        else return " in collection "+subset;
    }   
    
    /** Parses and sets the where-condition parameter in the OperationTask */
    protected void parseWherePositionCondition(String text, OperationTask task) throws ParseError {
          Condition_position condition=parsePositionCondition(text);
          task.setParameter("where", condition);
    }
    
    /** 
     * Parses the given text string containing a position condition clause and returns a Condition_position object
     * to represent the condition
     */
    protected Condition_position parsePositionCondition(String text) throws ParseError {
          text=text.trim();
          if (text.isEmpty()) throw new ParseError("Empty condition");
          Boolean whereNot;
          String comparator=null;
          String operandA=null;
          String operandB=null;
          Pattern pattern=Pattern.compile("^(not )?(inside) ([a-zA-Z_0-9-]+)");
          Matcher matcher=pattern.matcher(text);
          if (matcher.find()) {
               if (matcher.group(1)!=null && matcher.group(1).startsWith("not")) whereNot=Boolean.TRUE;else whereNot=Boolean.FALSE;
               comparator=matcher.group(2);
               operandA=matcher.group(3);
               // for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
          } else {
              pattern=Pattern.compile("^(not )?([a-zA-Z_0-9-]+)\\s*(bases overlap|bases not overlap|regions overlap|regions not overlap|<|<=|=|>|>=|<>)\\s*([a-zA-Z_0-9-\\.\\-]+)");
              matcher=pattern.matcher(text);
              if (matcher.find()) {
                  if (matcher.group(1)!=null && matcher.group(1).startsWith("not")) whereNot=Boolean.TRUE;else whereNot=Boolean.FALSE;
                  operandA=matcher.group(2);
                  comparator=matcher.group(3);
                  operandB=matcher.group(4);
                  // for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
              } else throw new ParseError("Unable to parse position-condition: '"+text+"'");          
          }
          //System.err.println("whereNot="+whereNot+", comparator="+comparator+",  opA="+operandA+", opB="+operandB);
          Condition_position condition=new Condition_position();         
          condition.setNegateAll(whereNot);
          condition.setComparator(comparator);
          condition.setOperandAString(operandA);
          condition.setOperandBString(operandB);     
          return condition;
    }
    
    
    /** Parses and sets the where-condition parameter in the OperationTask */    
    protected void parseWhereRegionCondition(String text, OperationTask task) throws ParseError {
          Condition_region condition=parseRegionCondition(text);           
          task.setParameter("where", condition);
    }  
    
    /** 
     * Parses the given text string containing a region condition clause and returns a Condition_region object
     * to represent the condition
     */    
    protected Condition_region parseRegionCondition(String text) throws ParseError {
          text=text.trim();
          if (text.isEmpty()) throw new ParseError("Empty condition");
          Boolean whereNot;
          String compareProperty=null;
          String comparator=null;
          String operandA=null;
          String operandB=null;
          Pattern pattern=Pattern.compile("^(not )?region type (equals|matches|in set) (\\S+)");
          Matcher matcher=pattern.matcher(text);
          if (matcher.find()) { // region type condition
               if (matcher.group(1)!=null && matcher.group(1).startsWith("not")) whereNot=Boolean.TRUE;else whereNot=Boolean.FALSE;
               comparator=matcher.group(2);
               operandB=matcher.group(3);
               compareProperty="type";
               if (comparator.equals("in set")) {
                   if (!operandB.matches("^[a-zA-Z_0-9-]+$")) throw new ParseError("Data item name '"+operandB+"' contains illegal characters");
               } else { // comparator is 'equals' or 'matches'
                   if (!(operandB.startsWith("\"") && operandB.endsWith("\""))) throw new ParseError("Literal text expression '"+operandB+"' should be enclosed in double quotation marks");         
               }
               // for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
          } else {
              pattern=Pattern.compile("^(not )?region (score|length)\\s*(<|<=|=|>|>=|<>)\\s*([a-zA-Z_0-9-\\.\\-]+)");
              matcher=pattern.matcher(text);
              if (matcher.find()) { // region score or length condition
                  if (matcher.group(1)!=null && matcher.group(1).startsWith("not")) whereNot=Boolean.TRUE;else whereNot=Boolean.FALSE;
                  compareProperty=matcher.group(2);
                  comparator=matcher.group(3);
                  operandB=matcher.group(4);
                  // for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
              } else {
                  pattern=Pattern.compile("^(not )?region (overlaps) ([a-zA-Z_0-9-\\.\\-]+)");
                  matcher=pattern.matcher(text);
                  if (matcher.find()) { // region overlaps condition
                      if (matcher.group(1)!=null && matcher.group(1).startsWith("not")) whereNot=Boolean.TRUE;else whereNot=Boolean.FALSE;
                      compareProperty=matcher.group(2);
                      operandB=matcher.group(3);
                  // for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
                  } else {
                     pattern=Pattern.compile("^(not )?region (distance to|avg|average|min|max|median|minimum|maximum|sum) ([a-zA-Z_0-9-\\.\\-]+)\\s*(<|<=|=|>|>=|<>)\\s*([a-zA-Z_0-9-\\.\\-]+)");
                     matcher=pattern.matcher(text);
                     if (matcher.find()) { // position-condition
                        if (matcher.group(1)!=null && matcher.group(1).startsWith("not")) whereNot=Boolean.TRUE;else whereNot=Boolean.FALSE;
                        compareProperty=matcher.group(2);
                        operandA=matcher.group(3);
                        comparator=matcher.group(4);
                        operandB=matcher.group(5);
                  // for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
                  } else throw new ParseError("Unable to parse region-condition: '"+text+"'");    
              } // not region overlap condition          
            } // not region score|length condition
          } // not region type condition
          //System.err.println("whereNot="+whereNot+", comparator="+comparator+",  opA="+operandA+", opB="+operandB);

          Condition_region condition=new Condition_region();         
          condition.setNegateAll(whereNot);
          condition.setComparator(comparator);
          condition.setOperandAString(operandA);
          condition.setOperandBString(operandB);        
          condition.setCompareProperty(compareProperty);        
          return condition;
    }

     /** Parses and sets the within-condition parameter in the OperationTask */
    public void parseWithinCondition(String text, OperationTask task) throws ParseError {
          Condition_within condition=parseWithinCondition(text);
          task.setParameter("within", condition);
    }
    
    /** 
     * Parses the given text string containing a position-condition clause and returns a Condition_position object
     * to represent the condition
     */
    protected Condition_within parseWithinCondition(String text) throws ParseError {
          text=text.trim();
          if (text.isEmpty()) throw new ParseError("Empty within condition");
          //Boolean whereNot;
          Pattern pattern=Pattern.compile("\\s*[a-zA-Z_0-9-]+:\\d+-\\d\\s*");
          String[] ranges=text.split(",");
          for (String range:ranges) {
             Matcher matcher=pattern.matcher(range);
             if (!matcher.find()) throw new ParseError("Unable to parse within-condition range: '"+text+"'");              
          }          
          //System.err.println("whereNot="+whereNot+", comparator="+comparator+",  opA="+operandA+", opB="+operandB);
          Condition_within condition=new Condition_within();         
          condition.setOperandAString(text);     
          return condition;
    }   
    
}
