/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.protocol;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.dataformat.DataFormat;
import priorseditor.engine.operations.Operation_new;
/**
 *
 * @author kjetikl
 */
public class StandardOperationParser_new extends StandardOperationParser {
    

    
    @Override
    public String getCommandString(OperationTask task) {
        String targetName=task.getTargetDataName();
        String datatype=(String)task.getParameter(Operation_new.DATA_TYPE);
        String parameters=(String)task.getParameter(Operation_new.PARAMETERS);
        String msg=targetName+" = new "+datatype;   
        if (parameters!=null && parameters.startsWith(Operation_new.FILE_PREFIX)) {
            String filename=(String)task.getParameter(Operation_new.FILENAME);    
            parameters=Operation_new.FILE_PREFIX+"\""+filename+"\"";
            String dataformatname=(String)task.getParameter(Operation_new.DATA_FORMAT);
            if (dataformatname!=null) {
                parameters+=", format="+dataformatname;
                DataFormat formatter=engine.getDataFormat(dataformatname);
                StandardParametersParser spp=(StandardParametersParser)protocol.getParametersParser();
                String settings=spp.getParameterSettingsAsString(formatter,task,Operation_new.DATA_FORMAT_SETTINGS);
                if (!settings.isEmpty()) parameters+=" {"+settings+"}";
            }
          
        }
        if (parameters!=null && !parameters.isEmpty()) msg+="("+parameters+")";
        return msg;
    }

    @Override
    public OperationTask parse(String command) throws ParseError {
           DataTypeTable lookup=protocol.getDataTypeLookupTable();
           String operationName="new";
           String targetName=null;
           String datatype=null;
           String parameters=null;
           String datatypenames="";
           String[] list = Operation_new.getAvailableTypes();
           for (String typename:list) datatypenames+="(?i)"+typename+"|";
           datatypenames=datatypenames.substring(0, datatypenames.length()-1);
           Pattern pattern=Pattern.compile("^(([a-zA-Z_0-9-]+)\\s*=\\s*)?new ("+datatypenames+")?\\s*(\\((.*)\\))?(\\s*\\S.*)?");
           Matcher matcher=pattern.matcher(command);
           if (matcher.find()) {
               targetName=matcher.group(2);
               datatype=matcher.group(3);
               parameters=matcher.group(5);
               String unknown=matcher.group(6);
               if (unknown!=null && !unknown.isEmpty()) {
                   if (datatype==null || datatype.isEmpty()) throw new ParseError("Unknown data type: "+unknown);
                   else throw new ParseError("Unrecognized clause (or wrong order): "+unknown);
               }
               // for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
           } else throw new ParseError("Unable to parse "+operationName+" command: "+command);
           OperationTask task=new OperationTask(operationName);
           Operation operation=engine.getOperation(operationName);
           if (operation==null) throw new ParseError("SYSTEM ERROR: Missing operation '"+operationName+"'");
           if (targetName==null || targetName.isEmpty()) throw new ParseError("Missing name for target data object");
           if (datatype==null || datatype.isEmpty()) throw new ParseError("Missing data type specification for 'new' operation");
           task.setParameter(OperationTask.OPERATION, operation);
           task.setParameter(OperationTask.OPERATION_NAME, operationName);
           task.setParameter(OperationTask.TARGET_NAME, targetName);
           task.setParameter(OperationTask.ENGINE, engine);
           task.setParameter(Operation_new.DATA_TYPE, datatype);
           Class dataclass=engine.getDataClassForTypeName(datatype);
           if (dataclass==null) throw new ParseError("Unknown data type: "+datatype);
           lookup.register(targetName, dataclass);
           if (parameters!=null) {
               parameters=parameters.trim();
               if (!parameters.isEmpty()) task.setParameter(Operation_new.PARAMETERS, parameters);
               if (parameters.startsWith(Operation_new.FILE_PREFIX)) {
                   String[] elements=parseLoadFromFileParameter(parameters.substring(Operation_new.FILE_PREFIX.length()));
                   String filename=elements[0];
                   String dataformatname=elements[1];
                   String dataformatsettings=elements[2];
                   if (filename==null || filename.isEmpty()) throw new ParseError("Missing filename specification for 'load from file' directive");
                   DataFormat formatter=null;
                   if (dataformatname==null || dataformatname.isEmpty()) {
                       formatter=engine.getDefaultDataFormat(lookup.getClassFor(targetName));
                       if (formatter==null) throw new ParseError("Missing data format specification for 'load from file' directive. No default found.");
                       else dataformatname=formatter.getName();
                   } else formatter=engine.getDataFormat(dataformatname);                   
                   if (formatter==null) throw new ParseError("Unrecognized data format '"+dataformatname+"'");
                   task.setParameter(Operation_new.FILENAME, filename);                   
                   task.setParameter(Operation_new.DATA_FORMAT, dataformatname);                   
                   if (dataformatsettings!=null && !dataformatsettings.isEmpty()) {               
                       StandardParametersParser spp=(StandardParametersParser)protocol.getParametersParser();
                       spp.parseFormatParameterSettings(dataformatsettings,formatter,task,Operation_new.DATA_FORMAT_SETTINGS);
                   }                   
               }
           }
           return task;
    }
    
    
     /** The returned Object consists of 3 elements:
      * 1) Name of file (String)
      * 2) Name of data format (String)
      * 3) Parameter settings for data format (String) -optional-
      */
     private String[] parseLoadFromFileParameter(String text) throws ParseError {
         if (!text.startsWith("\"")) throw new ParseError("Filename must be enclosed in double quotes");
           Pattern pattern=Pattern.compile("^\"(.+?)\"(\\s*,\\s*format\\s*=\\s*(\\S+)(\\s*\\{(.+)\\})?)?");
           Matcher matcher=pattern.matcher(text);
           String filename=null;
           String dataformatname=null;
           String formatsettings=null;
           if (matcher.find()) {
               filename=matcher.group(1);
               dataformatname=matcher.group(3);
               formatsettings=matcher.group(5);
           } else throw new ParseError("Unable to parse parameters for 'load from file' directive: "+text);
           return new String[]{filename,dataformatname,formatsettings};
    }
}
