/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.protocol;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.operations.Condition;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.operations.Operation_normalize;

/**
 *
 * @author kjetikl
 */
public class StandardOperationParser_normalize extends StandardOperationParser{


    @Override
    public String getCommandString(OperationTask task) {
        String sourceName=task.getSourceDataName();
        String targetName=task.getTargetDataName();
        String subset=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);        
        String msg="";         
        if (!sourceName.equalsIgnoreCase(targetName))  msg=targetName+" = ";
        msg+="normalize "+sourceName;
        String mode=(String)task.getParameter(Operation_normalize.MODE);
        if (mode==null) mode=Operation_normalize.SUM_TO_ONE;
        if (mode.equals(Operation_normalize.SUM_TO_ONE)) {
            msg+=" sum to one";            
        } else {
            String oldMinString=(String)task.getParameter(Operation_normalize.OLD_MIN);        
            String oldMaxString=(String)task.getParameter(Operation_normalize.OLD_MAX);        
            String newMinString=(String)task.getParameter(Operation_normalize.NEW_MIN);        
            String newMaxString=(String)task.getParameter(Operation_normalize.NEW_MAX);        

            String parstring="from range ["+oldMinString+","+oldMaxString+"] to range ["+newMinString+","+newMaxString+"]";
            msg+=" "+parstring;
            Condition condition=(Condition)task.getParameter("where");
            if (condition!=null) msg+=" where "+getCommandString_condition(condition);
        }
        if (subset!=null && !subset.equals(engine.getDefaultSequenceCollectionName())) msg+=" in collection "+subset;
        return msg;
    }

    @Override
    public OperationTask parse(String command) throws ParseError {
           DataTypeTable lookup=protocol.getDataTypeLookupTable();
           String operationName="normalize";
           String sourceName=null;
           String targetName=null;
           String sequenceCollection=null;
           String rangeString=null;
           String oldMinString=null;
           String oldMaxString=null;
           String newMinString=null;
           String newMaxString=null;
           String whereString=null;
           String mode=null;
           String[] splitOn=command.split(" in collection ");
           if (splitOn.length==2 && !splitOn[1].isEmpty()) sequenceCollection=splitOn[1].trim();       
           if (sequenceCollection!=null) {
             String[] splitOn2=sequenceCollection.split(" ");
             if (splitOn2.length>1) throw new ParseError("Unrecognized clause (or wrong order): "+splitOn2[1]);
           }
           
           Pattern pattern=Pattern.compile("^(([a-zA-Z_0-9-]+)\\s*=\\s*)?normalize ([a-zA-Z_0-9-]+)? sum to one");
           Matcher matcher=pattern.matcher(splitOn[0]);
           if (matcher.find()) {      
               targetName=matcher.group(2);
               sourceName=matcher.group(3);    
               mode=Operation_normalize.SUM_TO_ONE;
           } else {
               pattern=Pattern.compile("^(([a-zA-Z_0-9-]+)\\s*=\\s*)?normalize ([a-zA-Z_0-9-]+)?( from range\\s*\\[\\s*([a-zA-Z_0-9-\\.]+)\\s*,\\s*([a-zA-Z_0-9-\\.]+)\\s*\\] to range\\s*\\[\\s*([a-zA-Z_0-9-\\.]+)\\s*,\\s*([a-zA-Z_0-9-\\.]+)\\s*\\])?( where (.+))?(\\s*\\S.*)?");
               matcher=pattern.matcher(splitOn[0]);
               if (matcher.find()) {
                   targetName=matcher.group(2);
                   sourceName=matcher.group(3);
                   rangeString=matcher.group(4);
                   oldMinString=matcher.group(5);
                   oldMaxString=matcher.group(6);
                   newMinString=matcher.group(7);
                   newMaxString=matcher.group(8);
                   whereString=matcher.group(10);
                   String unknown = matcher.group(11);
                   mode=Operation_normalize.NORMALIZE_TO_RANGE;               
                   if (unknown!=null && !unknown.isEmpty()) throw new ParseError("Unrecognized clause (or wrong order): "+unknown);
                   //for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
               }        
               else throw new ParseError("Unable to parse "+operationName+" command: "+command);
           }
           OperationTask task=new OperationTask(operationName);
           Operation operation=engine.getOperation(operationName);
           if (operation==null) throw new ParseError("SYSTEM ERROR: Missing operation '"+operationName+"'");
           if (sourceName==null || sourceName.isEmpty()) throw new ParseError("Missing source data for "+operationName+" operation");
           if (mode.equals(Operation_normalize.NORMALIZE_TO_RANGE) && (rangeString==null || rangeString.isEmpty())) throw new ParseError("Missing correct range specification");
           if (targetName==null || targetName.isEmpty()) targetName=sourceName;           

           Class sourceclass=lookup.getClassFor(sourceName);
           if (sourceclass==null) throw new ParseError("Unrecognized source object: "+sourceName);
           if (!operation.canUseAsSource(sourceclass)) throw new ParseError("'"+sourceName+"' is of a type not supported by operation '"+operationName+"'");
           
           if (whereString!=null && !whereString.isEmpty()) {
               if (sourceclass==NumericDataset.class) parseWherePositionCondition(whereString,task);
               else if (sourceclass==RegionDataset.class) parseWhereRegionCondition(whereString,task);
           }

           if (sourceclass==NumericDataset.class) {
               lookup.register(targetName, NumericDataset.class);
           }  else if (sourceclass==RegionDataset.class) {
               lookup.register(targetName, RegionDataset.class);
           }
           if (mode==null) mode=Operation_normalize.SUM_TO_ONE;
           task.setParameter(OperationTask.OPERATION, operation);
           task.setParameter(OperationTask.OPERATION_NAME, operationName);
           task.setParameter(OperationTask.SOURCE_NAME, sourceName);
           task.setParameter(Operation_normalize.MODE, mode);
           task.setParameter(Operation_normalize.OLD_MIN, oldMinString);
           task.setParameter(Operation_normalize.OLD_MAX, oldMaxString);
           task.setParameter(Operation_normalize.NEW_MIN, newMinString);
           task.setParameter(Operation_normalize.NEW_MAX, newMaxString);
           task.setParameter(OperationTask.TARGET_NAME, targetName);           
           if (sequenceCollection!=null && !sequenceCollection.isEmpty()) task.setParameter(OperationTask.SEQUENCE_COLLECTION_NAME, sequenceCollection);
           return task;
    }
}
