/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.protocol;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import priorseditor.engine.data.OutputData;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.operations.Operation_output;
import priorseditor.engine.dataformat.DataFormat;
/**
 *
 * @author kjetikl
 */
public class StandardOperationParser_output extends StandardOperationParser {
    

    
    @Override
    public String getCommandString(OperationTask task) {
        String sourceName=task.getSourceDataName();
        String targetName=task.getTargetDataName();
        String outputformatName=(String)task.getParameter(Operation_output.OUTPUT_FORMAT);
        // Data sourceData=engine.getDataItem(sourceName); 
        String subset=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);       
        DataFormat formatter=engine.getDataFormat(outputformatName);
        if (formatter==null) return ("ERROR: Unknown output format '"+outputformatName+"'");
        //if (!formatter.canFormat(sourceData)) return ("Unsupported output format '"+outputformatName+"' for data object '"+sourceName+"'");
        Class sourceClass=protocol.getDataTypeLookupTable().getClassFor(sourceName);
        String msg="";
        if (targetName!=null && !targetName.equals(sourceName) && !targetName.equals(engine.getDefaultOutputObjectName())) msg+=targetName+" = ";
        msg+="output "+sourceName+" in "+outputformatName+" format";        
        StandardParametersParser spp=(StandardParametersParser)protocol.getParametersParser();
        String settings=spp.getParameterSettingsAsString(formatter,task,Operation_output.OUTPUT_FORMAT_PARAMETERS);
        if (!settings.isEmpty()) msg+=" {"+settings+"}";
        if (subset!=null && !subset.equals(engine.getDefaultSequenceCollectionName())) msg+=" in collection "+subset;        
        return msg;
    }

    @Override
    public OperationTask parse(String command) throws ParseError {
           DataTypeTable lookup=protocol.getDataTypeLookupTable();
           String operationName="output";
           String sourceName=null;
           String targetName=null;
           String sequenceCollection=null;
           String outputFormatName=null;
           String outputFormatSettings=null;
           Pattern pattern=Pattern.compile("^(([a-zA-Z_0-9-]+)\\s*=\\s*)?output ([a-zA-Z_0-9-]+)?( in (\\w+) format(\\s*\\{(.*)\\})?)?( in collection ([a-zA-Z_0-9-]+))?( \\S.*)?");
           Matcher matcher=pattern.matcher(command);
           if (matcher.find()) {
               targetName=matcher.group(2);
               sourceName=matcher.group(3);
               outputFormatName=matcher.group(5);
               outputFormatSettings=matcher.group(7);
               sequenceCollection=matcher.group(9);
               String unknown=matcher.group(10);
               if (unknown!=null && !unknown.isEmpty()) throw new ParseError("Unrecognized clause (or wrong order): "+unknown);
               // for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
           } else throw new ParseError("Unable to parse "+operationName+" command: "+command);
           OperationTask task=new OperationTask(operationName);
           Operation operation=engine.getOperation(operationName);
           if (operation==null) throw new ParseError("SYSTEM ERROR: Missing operation '"+operationName+"'");
           if (sourceName==null || sourceName.isEmpty()) throw new ParseError("Missing source data for output operation");
           

           Class sourceclass=lookup.getClassFor(sourceName);
           if (sourceclass==null) throw new ParseError("Unrecognized source object: "+sourceName);
           if (!operation.canUseAsSource(sourceclass)) throw new ParseError("'"+sourceName+"' is of a type not supported by operation '"+operationName+"'");

           DataFormat formatter=null;
           if (outputFormatName!=null) formatter=engine.getDataFormat(outputFormatName);
           if (outputFormatName!=null && formatter==null) throw new ParseError("Unrecognized output format '"+outputFormatName+"'");
           if (outputFormatSettings!=null && !outputFormatSettings.isEmpty()) {   
               StandardParametersParser spp=(StandardParametersParser)protocol.getParametersParser();
               spp.parseFormatParameterSettings(outputFormatSettings,formatter,task,Operation_output.OUTPUT_FORMAT_PARAMETERS);
           }

           
           task.setParameter(OperationTask.OPERATION, operation);
           task.setParameter(OperationTask.OPERATION_NAME, operationName);
           task.setParameter(OperationTask.SOURCE_NAME, sourceName);
           task.setParameter(Operation_output.OUTPUT_FORMAT, outputFormatName);
           if (targetName!=null && !targetName.isEmpty()) {
               task.setParameter(OperationTask.TARGET_NAME, targetName);
               Class oldclass=lookup.getClassFor(targetName);
               if (oldclass!=null && oldclass!=OutputData.class) throw new ParseError("Unable to output to "+targetName+". Target data object has wrong type!");
               lookup.register(targetName, OutputData.class);
           } else {
               lookup.register(engine.getDefaultOutputObjectName(), OutputData.class);
           }
           if (sequenceCollection!=null && !sequenceCollection.isEmpty()) task.setParameter(OperationTask.SEQUENCE_COLLECTION_NAME, sequenceCollection);
           return task;
    }
    
    

}
