/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.engine.protocol;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.operations.Condition;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.NumericVariable;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.operations.Operation_rescore;
/**
 *
 * @author kjetikl
 */
public class StandardOperationParser_rescore extends StandardOperationParser {

    @Override
    public String getCommandString(OperationTask task) {
        String sourceName=task.getSourceDataName();
        String targetName=task.getTargetDataName();
         
        String rescoreDataString=(String)task.getParameter(Operation_rescore.RESCORE_DATA_NAME);        
        String rescoreOperator=(String)task.getParameter(Operation_rescore.RESCORE_OPERATOR);        
        String rescoreType=(String)task.getParameter(Operation_rescore.RESCORE_TYPE);
        if (rescoreType==null) rescoreType=Operation_rescore.WEIGHT_BY;
        String subset=(String)task.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);        
        String msg="";
        String parstring=rescoreType+" "+rescoreOperator+" "+rescoreDataString;
        if (!sourceName.equalsIgnoreCase(targetName))  msg=targetName+" = ";
        msg+="rescore "+sourceName;
        msg+=" "+parstring;
        Condition condition=(Condition)task.getParameter("where");
        if (condition!=null) msg+=" where "+getCommandString_condition(condition);
        if (subset!=null && !subset.equals(engine.getDefaultSequenceCollectionName())) msg+=" in collection "+subset;
        return msg;
    }

    @Override
    public OperationTask parse(String command) throws ParseError {
           DataTypeTable lookup=protocol.getDataTypeLookupTable();
           String operationName="rescore";
           String sourceName=null;
           String targetName=null;
           String sequenceCollection=null;
           String rescoreDataString=null;
           String rescoreOperator=null;
           String rescoreType=null; // "set to" or "weight by"
           String whereString=null;
           String[] splitOn=command.split(" in collection ");
           if (splitOn.length==2 && !splitOn[1].isEmpty()) sequenceCollection=splitOn[1].trim();       
           if (sequenceCollection!=null) {
             String[] splitOn2=sequenceCollection.split(" ");
             if (splitOn2.length>1) throw new ParseError("Unrecognized clause (or wrong order): "+splitOn2[1]);
           }
           String operators="min|minimum|max|maximum|average|median|sum|value";
           Pattern pattern=Pattern.compile("^(([a-zA-Z_0-9-]+)\\s*=\\s*)?rescore ([a-zA-Z_0-9-]+)?( (weight by|set to) ("+operators+") (\\S+))?( where (.+))?(\\s*\\S.*)?");
           Matcher matcher=pattern.matcher(splitOn[0]);
           if (matcher.find()) {
               targetName=matcher.group(2);
               sourceName=matcher.group(3);
               rescoreType=matcher.group(5);
               rescoreOperator=matcher.group(6);
               rescoreDataString=matcher.group(7);
               whereString=matcher.group(9);
               String unknown = matcher.group(10);
               if (unknown!=null && !unknown.isEmpty()) throw new ParseError("Unrecognized clause (or wrong order): "+unknown);
               //for (int i=0;i<=matcher.groupCount();i++) System.err.println("Group["+i+"]=>"+matcher.group(i));
           } else throw new ParseError("Unable to parse "+operationName+" command: "+command);
           OperationTask task=new OperationTask(operationName);
           Operation operation=engine.getOperation(operationName);
           if (operation==null) throw new ParseError("SYSTEM ERROR: Missing operation '"+operationName+"'");
           if (sourceName==null || sourceName.isEmpty()) throw new ParseError("Missing source data for rescore operation");
           if (rescoreType==null || rescoreType.isEmpty()) throw new ParseError("Missing rescore type specification (either 'weight by' or 'set to')");
           if (rescoreOperator==null || rescoreOperator.isEmpty()) throw new ParseError("Missing operator specification for rescore value");
           if (rescoreDataString==null || rescoreDataString.isEmpty()) throw new ParseError("Missing rescore value");
           if (whereString!=null) parseWhereRegionCondition(whereString,task);
           if (targetName==null || targetName.isEmpty()) targetName=sourceName;           

           Class sourceclass=lookup.getClassFor(sourceName);
           if (sourceclass==null) throw new ParseError("Unrecognized source object: "+sourceName);
           if (!operation.canUseAsSource(sourceclass)) throw new ParseError("'"+sourceName+"' is of a type not supported by operation '"+operationName+"'");
           Class datasourceclass=lookup.getClassFor(rescoreDataString);
           if (datasourceclass==null) {
               try {
                  Float.parseFloat(rescoreDataString);   
                  datasourceclass=NumericVariable.class;
               } catch (NumberFormatException e) {throw new ParseError("Unrecognized token '"+rescoreDataString+"' is neither a data object nor a numeric constant");}
           } else {
               if (datasourceclass!=NumericDataset.class && datasourceclass!=NumericVariable.class) throw new ParseError("Rescore data object '"+rescoreDataString+"' is neither a Numeric Dataset nor a Numeric Variable");               
           }
           if (datasourceclass==NumericVariable.class && !rescoreOperator.equals("value")) throw new ParseError("Only 'value' operator is allowed for Numeric Variables or constants");
           else if (datasourceclass==NumericDataset.class && rescoreOperator.equals("value")) throw new ParseError("Operator 'value' is not allowed for Numeric Datasets");
           //Class oldclass=lookup.getClassFor(targetName);
           //if (oldclass!=null && oldclass!=RegionDataset.class) throw new ParseError("Unable to output to "+targetName+". Target data object has wrong type!");
           lookup.register(targetName, RegionDataset.class);

           task.setParameter(OperationTask.OPERATION, operation);
           task.setParameter(OperationTask.OPERATION_NAME, operationName);
           task.setParameter(OperationTask.SOURCE_NAME, sourceName);
           task.setParameter(Operation_rescore.RESCORE_DATA_NAME, rescoreDataString);
           task.setParameter(Operation_rescore.RESCORE_OPERATOR, rescoreOperator);
           task.setParameter(Operation_rescore.RESCORE_TYPE, rescoreType);
           task.setParameter(OperationTask.TARGET_NAME, targetName);           
           if (sequenceCollection!=null && !sequenceCollection.isEmpty()) task.setParameter(OperationTask.SEQUENCE_COLLECTION_NAME, sequenceCollection);
           return task;
    }

}
