/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.external;

import java.util.ArrayList;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.BackgroundModel;
import priorseditor.engine.data.DNASequenceData;
import priorseditor.engine.data.DNASequenceDataset;
import priorseditor.engine.data.MotifCollection;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.NumericConstant;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.RegionSequenceData;
import priorseditor.engine.data.Sequence;
import priorseditor.engine.data.SequenceCollection;

/**
 *
 * @author kjetikl
 */
public class SimpleScanner extends MotifScanning {
    
    public SimpleScanner() {
        this.name="SimpleScanner";
        this.programclass="MotifScanning";
        this.serviceType="bundled";

        addSourceParameter("Sequence", DNASequenceDataset.class, null, null, "input sequences");
        addParameter("Motif Collection",MotifCollection.class, null,new Class[]{MotifCollection.class},null,true,false);
        addParameter("Background", BackgroundModel.class, null, new Class[]{BackgroundModel.class},null,false,false); 
        addParameter("Threshold", Float.class, 7.0f, new Float[]{0.f,20f},null,false,false);
        addResultParameter("Result", RegionDataset.class, null, null, "output track");
    }     
    
    public void parseParameters(OperationTask task) throws Exception {
        String thresholdDataString=(String)task.getParameter("Threshold");
        Data thresholdData=null;
        thresholdData=engine.getDataItem(thresholdDataString);
        if (thresholdData==null) {
            try {
              float value=Float.parseFloat(thresholdDataString);
              thresholdData=new NumericConstant(thresholdDataString, (float)value);
           } catch (Exception e) {throw new ExecutionError("Unrecognized token '"+thresholdDataString+"' neither data nor numeric constant",task.getLineNumber());}         
        }
        task.setParameter("Threshold", thresholdData);  
        
        String backgroundDataString=(String)task.getParameter("Background");
        Data background=engine.getDataItem(backgroundDataString);
        if (background==null) {
            background=new BackgroundModel("temporary");
        }
        if (!(background instanceof BackgroundModel)) throw new ExecutionError(backgroundDataString+"' is not a Background Model",task.getLineNumber());
        task.setParameter("Background", background);  

        String motifsDataString=(String)task.getParameter("Motif Collection");
        Data motifs=engine.getDataItem(motifsDataString);        
        if (motifs==null) throw new ExecutionError("Unrecognized token '"+motifsDataString+"' is not a data object",task.getLineNumber());
        if (!(motifs instanceof MotifCollection)) throw new ExecutionError(motifsDataString+"' is not a Motif Collection",task.getLineNumber());
        task.setParameter("Motif Collection", motifs);  
    } 
    

    @Override
    public void execute(OperationTask task) throws Exception {
        if (engine.getDefaultSequenceCollection().size()==0) throw new ExecutionError("No sequences are selected");
        String targetDatasetName=task.getTargetDataName();
        Data[] sources=(Data[])task.getParameter(SOURCES);
        if (sources==null || sources.length==0) throw new ExecutionError("SYSTEM ERROR: Missing SOURCES for motif scanning with SimpleScanner");
        DNASequenceDataset sourceDataset=(DNASequenceDataset)sources[0];
        RegionDataset targetDataset=new RegionDataset(targetDatasetName);
        ArrayList<Data> allsequences=engine.getAllDataItemsOfType(Sequence.class);
        for (Data sequence:allsequences) {
             RegionSequenceData regionsequence=new RegionSequenceData((Sequence)sequence);
             targetDataset.addSequence(regionsequence);
        }
        SequenceCollection sequenceCollection=(SequenceCollection)task.getParameter(OperationTask.SEQUENCE_COLLECTION);

        int size=sequenceCollection.getNumberofSequences();
        int i=0;
        ArrayList<Sequence> sequences=sequenceCollection.getAllSequences(engine);
        for (Sequence sequence:sequences) { // for each sequence
            task.checkExecutionLock(); // checks to see if this task should suspend execution
            if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
            String sequenceName=sequence.getName();
            DNASequenceData sourceSequence=(DNASequenceData)sourceDataset.getSequenceByName(sequenceName);
            RegionSequenceData targetSequence=(RegionSequenceData)targetDataset.getSequenceByName(sequenceName);
            searchInSequence(sourceSequence,targetSequence, task);                              
            task.setStatusMessage("Executing SimpleScanner:  ("+(i+1)+"/"+size+")");
            task.setProgress(i+1, size);
            i++;
            Thread.yield();
        }   
        //((NumericDataset)targetDataset).updateMinMax();
        task.checkExecutionLock(); // checks to see if this task should suspend execution
        if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();
        task.setParameter("Result", targetDataset);   
    }        
    
    
    
    public void searchInSequence(DNASequenceData sourceSequence, RegionSequenceData targetSequence, OperationTask task) throws Exception {
        String seqname=sourceSequence.getName();        
        MotifCollection motifs=(MotifCollection)task.getParameter("Motif Collection");
        BackgroundModel background=(BackgroundModel)task.getParameter("Background");
        float[] snf=new float[]{0.25f,0.25f,0.25f,0.25f};
        if (background!=null) snf=background.getSNF();
        float threshold=(Float)task.getParameter("Threshold");
        ArrayList<Motif> allMotifs=motifs.getAllMotifs(engine);
        int counter=0;
        for (Motif motif:allMotifs) { // search with each motif in turn
            counter++;
            if (counter%10==0) {
                task.checkExecutionLock(); // checks to see if this task should suspend execution
                if (Thread.interrupted() || task.getStatus().equals(ExecutableTask.ABORTED)) throw new InterruptedException();                
            }
            int motifsize=motif.getLength();
            int startOffset=sourceSequence.getRegionStart();
            for (int i=startOffset;i<=sourceSequence.getRegionEnd();i++) {
                if (i>sourceSequence.getRegionEnd()-motifsize+1) continue; // unable to match near end
                char[] subseq=(char[])sourceSequence.getValueInGenomicInterval(i, i+motifsize-1);
                float score=matrixMatcher(subseq,motif,snf);
                if (score>=threshold) {
                     Region region=new Region(targetSequence,i-startOffset,i+motif.getLength()-(1+startOffset),motif.getName(),score,Region.DIRECT);
                     region.setProperty("consensus motif", motif.getConsensusMotif());
                     //region.setProperty("sequence", new String(subseq));
                     targetSequence.addRegion(region);
                }
                score=matrixMatcher(reverseComplement(subseq),motif,snf);
                if (score>=threshold) {
                     Region region=new Region(targetSequence,i-startOffset,i+motif.getLength()-(1+startOffset),motif.getName(),score,Region.REVERSE);
                     region.setProperty("consensus motif", motif.getConsensusMotif());
                     //region.setProperty("sequence", new String(reverseComplement(subseq)));
                     targetSequence.addRegion(region);
                }
            }
        }
        
    }
    
    
    private float matrixMatcher(char[] subseq, Motif motif, float[] snf) {
         float score=0;         
         for (int i=0;i<subseq.length;i++) {
             char base=subseq[i];
             float basefreq=motif.getBaseFrequency(i,base)+0.01f; // base frequency with pseudocount added
             float backgroundFreq=1;
             switch (base) {
                 case 'A': case 'a': backgroundFreq=snf[0];break;
                 case 'C': case 'c': backgroundFreq=snf[1];break;
                 case 'G': case 'g': backgroundFreq=snf[2];break;
                 case 'T': case 't': backgroundFreq=snf[3];break;
             }
             //System.err.println("["+base+"]  baseFreq="+basefreq+"    backgroundFreq="+backgroundFreq);
             float logodds=(float)Math.log(basefreq/(backgroundFreq+0.04f));
             score+=logodds;
             //System.err.println("["+(i+1)+"/"+subseq.length+"]  logodds="+logodds+"  score="+score+"   basefreq="+basefreq+"   backgroundFreq="+backgroundFreq);
         }
         return score;
    }
    
   private char[] reverseComplement(char[] buffer) {
       int length=buffer.length;
       for (int i=0;i<(int)(length/2);i++) {
           char left=buffer[i];
           char right=buffer[length-1-i];
           buffer[i]=complement(right);
           buffer[length-1-i]=complement(left);        
       }
       if (length%2>0) {
           int middle=((int)(length/2));
           buffer[middle]=complement(buffer[middle]);
       }    
       return buffer;
   }
   
   
   private char complement(char c) {
      switch (c) {
          case 'A': case 'a': return 'T';
          case 'C': case 'c': return 'G';
          case 'G': case 'g': return 'C';
          case 'T': case 't': return 'A';    
      } 
      return c;  
   }    
}