
package priorseditor.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JPopupMenu;
import javax.swing.JList;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.JMenuItem;
import org.jdesktop.application.ResourceMap;

import priorseditor.engine.OperationTask;
import priorseditor.engine.data.*;
import priorseditor.engine.operations.Operation_new;
import priorseditor.gui.prompt.*;


/**
 * This class encapsulates the DataObjects (non-feature data) dataset panel
 * 
 * @author kjetikl
 */
public class DataObjectsPanel extends JPanel {
        private JLabel dataObjectsPanelLabel;
        private JScrollPane dataListScrollPane;
        private JList dataList;
        private JButton addButton;
        private addDataPopupMenu addPopupMenu;
        private PriorsEditorGUI gui;
        private Class[] filter=new Class[]{SequenceCollection.class,NumericVariable.class,NumericMapVariable.class,BackgroundModel.class,StringVariable.class,StringCollection.class}; // list of which class types this panel accepts
    
  public DataObjectsPanel(PriorsEditorGUI gui) {
      this.gui=gui;
      initComponents();
  } 
  
  public JList getDataObjectsList() {
      return dataList;
  }  
  
  
  private void initComponents() {
        ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(DataObjectsPanel.class);

        dataObjectsPanelLabel=new JLabel();
        setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        setName("DataObjectsPanel"); // NOI18N
        setLayout(new BorderLayout());

        dataObjectsPanelLabel.setText(resourceMap.getString("DataObjectsLabel.text")); // NOI18N
        dataObjectsPanelLabel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 3, 1, 1));
        dataObjectsPanelLabel.setName("Data objects"); // NOI18N
        addPopupMenu=new addDataPopupMenu();
        addButton=new JButton(new MiscIcons(MiscIcons.ADD_DATA));
        addButton.setName("addButton");
        addButton.setToolTipText(resourceMap.getString("addButton.toolTipText")); // NOI18N
        addButton.setFont(new Font(Font.SANS_SERIF,Font.PLAIN,12));
        Dimension dim=new Dimension(36,22);
        addButton.setMaximumSize(dim);
        addButton.setPreferredSize(dim);
        addButton.setMinimumSize(dim);
        addButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                addDataObject(evt);
            }

        });
        
        JPanel labelpanel=new JPanel();
        labelpanel.setLayout(new BorderLayout());
        labelpanel.add(dataObjectsPanelLabel,BorderLayout.CENTER);
        labelpanel.add(addButton,BorderLayout.EAST);
        
        add(labelpanel, java.awt.BorderLayout.PAGE_START);
        
        dataListScrollPane = new JScrollPane();
        dataListScrollPane.setName("DataListScrollPane"); // NOI18N

        dataList=new JList();
        dataList.setBackground(resourceMap.getColor("dataList.background")); // NOI18N
        dataList.setModel(new DataObjectsPanelListModel(gui.getEngine(),filter));
        dataList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        dataList.setCellRenderer(new DataPanelCellRenderer(gui.getVisualizationSettings()));
        dataList.setDragEnabled(true);
        dataList.setDropMode(javax.swing.DropMode.ON_OR_INSERT);
        dataList.setName("dataList"); // NOI18N

        dataList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                mousePressedInDataObjectsPanel(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                mouseReleasedInDataObjectsPanel(evt);
            }
        });
        dataList.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                //gui.logMessage("key typed "+e.getKeyCode());
                if (e.getKeyCode()==KeyEvent.VK_DELETE && dataList.getSelectedIndex()>=0) {
                    String dataname=((Data)dataList.getSelectedValue()).getName();
                    if (!dataname.equals(gui.getEngine().getDefaultSequenceCollectionName())) gui.deleteDataItem(((Data)dataList.getSelectedValue()).getName());
                }
            }
        });
        // dataList.setTransferHandler(new FeaturesPanelTransferHandler(dataList,gui)); <-- THIS SHOULD BE UPDATED!!!
        //dataList.getModel().addListDataListener(gui.getVisualizationSettings());
        dataListScrollPane.setViewportView(dataList);

        
        add(dataListScrollPane, java.awt.BorderLayout.CENTER);
  }


private void mousePressedInDataObjectsPanel(java.awt.event.MouseEvent evt) {                                             
    int x=evt.getX(); 
    int y=evt.getY();
    JList list=(JList)evt.getComponent();    
    int index=getElementIndexAtPointInList(list,x,y);
    if (index==-1) {list.clearSelection();return;} 
    else list.setSelectedIndex(index);
    if (evt.getButton()==evt.BUTTON1 && evt.getClickCount()==2) {
         gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
         javax.swing.DefaultListModel model=(javax.swing.DefaultListModel)list.getModel();
         Data selectedData=(Data)model.getElementAt(index);
         showPrompt(selectedData);
    }
    if (evt.isPopupTrigger()) {
        JPopupMenu popup=DataObjectsPanelContextMenu.getInstance(list,index,gui,this);
        if (popup!=null) popup.show(evt.getComponent(), x,y);
    }  
}                                            

private void mouseReleasedInDataObjectsPanel(java.awt.event.MouseEvent evt) {                                              
    int x=evt.getX(); 
    int y=evt.getY();
    JList list=(JList)evt.getComponent();
    int index=getElementIndexAtPointInList(list,x,y);
    if (index==-1) {list.clearSelection();return;} 
    else list.setSelectedIndex(index);
    if (evt.isPopupTrigger()) {;
        JPopupMenu popup=DataObjectsPanelContextMenu.getInstance((JList)evt.getComponent(),index,gui,this);
        if (popup!=null) popup.show(evt.getComponent(), x,y);
    }
}                                             

public void showPrompt(Data selectedData) {
         //gui.debugMessage("Doubled-clicked on "+selectedData.getName());
         if (selectedData instanceof SequenceCollection) {
              Prompt_SequenceCollection prompt=new Prompt_SequenceCollection(gui, null, (SequenceCollection)selectedData.clone());
              prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
              prompt.disableNameEdit();
              prompt.setVisible(true);
              if (prompt.isOKPressed()) {
                 SequenceCollection collection=(SequenceCollection)prompt.getData();
                 if (collection!=null && !collection.containsSameData((SequenceCollection)selectedData)) gui.updateDataItem(collection);
              }
              prompt.dispose();
         } else if (selectedData instanceof NumericVariable) {
              Prompt_NumericVariable prompt=new Prompt_NumericVariable(gui, null, (NumericVariable)selectedData.clone());
              prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
              prompt.disableNameEdit();
              prompt.setVisible(true);
              if (prompt.isOKPressed()) {
                 NumericVariable variable=(NumericVariable)prompt.getData();
                 if (variable!=null && variable.getValue().floatValue()!=((NumericVariable)selectedData).getValue().floatValue()) gui.updateDataItem(variable);
              }
              prompt.dispose();
         } else if (selectedData instanceof NumericMapVariable) {
              Prompt_NumericMapVariable prompt=new Prompt_NumericMapVariable(gui, null, (NumericMapVariable)selectedData.clone());
              prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
              prompt.disableNameEdit();
              prompt.setVisible(true);
              if (prompt.isOKPressed()) {
                 NumericMapVariable variable=(NumericMapVariable)prompt.getData();
                 if (variable!=null && !variable.containsSameData((NumericMapVariable)selectedData)) gui.updateDataItem(variable);
              }
              prompt.dispose();
         } else if (selectedData instanceof BackgroundModel) {
              Prompt_BackgroundModel prompt=new Prompt_BackgroundModel(gui, null, (BackgroundModel)selectedData.clone());
              prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
              prompt.disableNameEdit();
              prompt.setVisible(true);
              if (prompt.isOKPressed()) {
                 BackgroundModel variable=(BackgroundModel)prompt.getData();
                 if (variable!=null && !variable.containsSameData((BackgroundModel)selectedData)) gui.updateDataItem(variable);
              }
              prompt.dispose();
         } else if (selectedData instanceof StringVariable) {
              Prompt_StringVariable prompt=new Prompt_StringVariable(gui, null, (StringVariable)selectedData.clone());
              prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
              prompt.disableNameEdit();
              prompt.setVisible(true);
              if (prompt.isOKPressed()) {
                 StringVariable variable=(StringVariable)prompt.getData();
                 if (variable!=null && !variable.containsSameData((StringVariable)selectedData)) gui.updateDataItem(variable);
              }
              prompt.dispose();
         } else if (selectedData instanceof StringCollection) {
              Prompt_StringCollection prompt=new Prompt_StringCollection(gui, null, (StringCollection)selectedData.clone());
              prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
              prompt.disableNameEdit();
              prompt.setVisible(true);
              if (prompt.isOKPressed()) {
                 StringCollection variable=(StringCollection)prompt.getData();
                 if (variable!=null && !variable.containsSameData((StringCollection)selectedData)) gui.updateDataItem(variable);
              }
              prompt.dispose();
         } 
         gui.getFrame().setCursor(Cursor.getDefaultCursor());    
}

 /**
  * Given a reference to a JList and a point (x,y), this function returns the index 
  * of the element corresponding to that point (between 0 and model.getSize()-1).
  * If the coordinate does not correspond to an element in the list, the method returns the value -1;
  * @param list The JList (fitted with a DefaultListModel)
  * @param x
  * @param y
  * @return
  */
 private int getElementIndexAtPointInList(JList list, int x, int y) {
         int index=list.locationToIndex(new java.awt.Point(x,y));
         if (index>=0) {
            javax.swing.DefaultListModel model=(javax.swing.DefaultListModel)list.getModel();
            java.awt.Rectangle bounds = list.getCellBounds(0, model.getSize()-1);
            if (y>bounds.getHeight() || y<0 || x>bounds.getWidth() || x<0) index=-1; // 
         }
         return index;
 }

 /**
  * This method is called when the "add data object" button (next to the panel label) is pressed
  */
 private void addDataObject(java.awt.event.MouseEvent evt) {
    Component invoker=evt.getComponent();
    int x=evt.getX();
    int y=evt.getY();
    addPopupMenu.show(invoker, x,y);
 }
 

 private class addDataPopupMenu extends JPopupMenu {
     public addDataPopupMenu() {
         JMenuItem seqColItem=new JMenuItem(SequenceCollection.getType());
         JMenuItem numericItem=new JMenuItem(NumericVariable.getType());
         JMenuItem backgroundModelItem=new JMenuItem(BackgroundModel.getType());
         JMenuItem stringVariableItem=new JMenuItem(StringVariable.getType());
         JMenuItem stringCollectionItem=new JMenuItem(StringCollection.getType());
         addDataPopupMenuListener listener=new addDataPopupMenuListener();
         seqColItem.addActionListener(listener);
         numericItem.addActionListener(listener);
         backgroundModelItem.addActionListener(listener);
         stringVariableItem.addActionListener(listener);
         stringCollectionItem.addActionListener(listener);
         add(seqColItem);
         add(numericItem);
         add(backgroundModelItem);
         add(stringVariableItem);
         add(stringCollectionItem);
     }
 }
 
 private class addDataPopupMenuListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
         gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
         Prompt prompt=null;
         String type=e.getActionCommand();
              if (type.equals(SequenceCollection.getType())) prompt=new Prompt_SequenceCollection(gui, null, null);
         else if (type.equals(NumericVariable.getType())) prompt=new Prompt_NumericVariable(gui, null, null);
         else if (type.equals(NumericMapVariable.getType())) prompt=new Prompt_NumericMapVariable(gui, null, null);
         else if (type.equals(BackgroundModel.getType())) prompt=new Prompt_BackgroundModel(gui, null, null);
         else if (type.equals(StringVariable.getType())) prompt=new Prompt_StringVariable(gui, null, null);
         else if (type.equals(StringCollection.getType())) prompt=new Prompt_StringCollection(gui, null, null);
         prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
         prompt.setVisible(true);
         gui.getFrame().setCursor(Cursor.getDefaultCursor());
         if (prompt.isOKPressed()) {
            Data variable=prompt.getData();          
            if (variable==null) {prompt.dispose();return;}
            OperationTask task=new OperationTask("new "+type);
            task.setParameter(Operation_new.DATA_TYPE, type);
            task.setParameter(OperationTask.OPERATION_NAME, "new");
            task.setParameter(OperationTask.ENGINE,gui.getEngine());
            task.setParameter(OperationTask.TARGET_NAME,variable.getName());
            String parameterString=variable.getValueAsParameterString();
            task.setParameter(Operation_new.PARAMETERS, parameterString);
            variable.setAdditionalOperationNewTaskParameters(task);
            prompt.dispose();
            gui.launchOperationTask(task,gui.isRecording());
         } else {prompt.dispose();}
    } 
}
 
 
}
