/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.Data;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JList;
import java.awt.event.ActionListener;
import javax.swing.DefaultListModel;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Collections;
import javax.swing.JColorChooser;
import javax.swing.JMenu;
import javax.swing.JSeparator;
import priorseditor.engine.data.SequenceCollection;



/**
 *
 * @author kjetikl
 */
public class DataObjectsPanelContextMenu extends JPopupMenu  {
    private static String EDIT_DATA="Edit";
    private static String RENAME_DATA="Rename";
    private static String DELETE_DATA="Delete";

    private static String OPERATION_SUBMENU_HEADER="Perform Operation";
    private static String COLOR_SUBMENU_HEADER="Set label color";
    private static String COLOR_CLUSTER_SUBMENU_HEADER="Set label color from clusters";
    private static String ORDER_BY_CLUSTER="Group sequences by clusters";
    private static String SHOW_SEQUENCES="Show sequences";
    private static String HIDE_SEQUENCES="Hide sequences";
    
    private PriorsEditorEngine engine;
    private Data dataitem;
    private PriorsEditorGUI gui;
    private JMenu operationsMenu;
    private DataObjectsPanel datapanel;
    private Object[] colors=new Object[] {
         new Object[]{"BLACK", Color.BLACK},  
         new Object[]{"WHITE", Color.WHITE},  
         new Object[]{"RED", Color.RED},  
         new Object[]{"ORANGE", new Color(255,204,0)},
         new Object[]{"YELLOW", Color.YELLOW},  
         new Object[]{"GREEN", Color.GREEN},  
         new Object[]{"CYAN", Color.CYAN},  
         new Object[]{"LIGHT BLUE", new Color(46,153,255)},  
         new Object[]{"BLUE", Color.BLUE},  
         new Object[]{"VIOLET", new Color(153,51,255)},
         new Object[]{"MAGENTA", Color.MAGENTA}          
    };
    
    public static DataObjectsPanelContextMenu getInstance(JList list, int index, PriorsEditorGUI gui, DataObjectsPanel datapanel) {
         if (index==-1) return null;
         DefaultListModel model=(DefaultListModel)list.getModel();
         return new DataObjectsPanelContextMenu((Data)model.getElementAt(index),gui, datapanel);
    }
    
        private DataObjectsPanelContextMenu(Data data, PriorsEditorGUI gui,DataObjectsPanel datapanel) {
            this.engine=gui.getEngine();
            this.dataitem=data;
            this.gui=gui;
            this.datapanel=datapanel;
            DisplayMenuItemListener menuItemListener=new DisplayMenuItemListener();
            ArrayList<Operation> operationslist=engine.getAllOperations();
            Collections.sort(operationslist, new NameComparator());
            // HeaderMenuItem header=new HeaderMenuItem(dataset.getName(), new java.awt.Color(255,255,255), new java.awt.Color(150,150,255));
            // this.add(header);
            JMenuItem editDataItem=new JMenuItem(EDIT_DATA+" \""+dataitem.getName()+"\"");
            JMenuItem renameDataItem=new JMenuItem(RENAME_DATA+" \""+dataitem.getName()+"\"");
            JMenuItem deleteDataItem=new JMenuItem(DELETE_DATA+" \""+dataitem.getName()+"\"");
            editDataItem.addActionListener(menuItemListener);
            renameDataItem.addActionListener(menuItemListener);
            deleteDataItem.addActionListener(menuItemListener);
            if (dataitem!=engine.getDefaultSequenceCollection()) this.add(editDataItem);
            if (dataitem!=engine.getDefaultSequenceCollection()) this.add(renameDataItem);
            if (dataitem!=engine.getDefaultSequenceCollection()) this.add(deleteDataItem);
            if (!gui.getScheduler().isIdle()) {
                editDataItem.setEnabled(false);
                deleteDataItem.setEnabled(false);
                renameDataItem.setEnabled(false);
            }
            if (dataitem instanceof SequenceCollection) {
                JMenuItem showitem=new JMenuItem(SHOW_SEQUENCES);
                showitem.addActionListener(menuItemListener);           
                this.add(showitem);  
                JMenuItem hideitem=new JMenuItem(HIDE_SEQUENCES);
                hideitem.addActionListener(menuItemListener);              
                this.add(hideitem);
                ColorMenuListener colormenulistener=new ColorMenuListener();
                JMenu colorMenu=new JMenu(COLOR_SUBMENU_HEADER);
                for (Object entry:colors) {
                      String colorname=(String)((Object[])entry)[0];
                      Color color=(Color)((Object[])entry)[1];
                      JMenuItem menuitem=new JMenuItem(colorname);
                      menuitem.setIcon(new SimpleDataPanelIcon(12,12,color));
                      menuitem.addActionListener(colormenulistener);
                      colorMenu.add(menuitem);
                }
                JMenuItem menuitem=new JMenuItem("Other...");
                menuitem.addActionListener(colormenulistener);
                colorMenu.add(menuitem);                
                this.add(colorMenu);     
            }         
            this.add(new JSeparator());
            OperationsMenuListener menulistener=new OperationsMenuListener();
            operationsMenu=new JMenu(OPERATION_SUBMENU_HEADER);
            for (Operation operation:operationslist) {
                  if (operation.canUseAsSource(dataitem)) {
                  JMenuItem menuitem=new JMenuItem(operation.getName());
                  menuitem.addActionListener(menulistener);
                  operationsMenu.add(menuitem);
                }
            }
            this.add(operationsMenu);
        }
        
        
        /**
         * An inner class that listens to popup-menu events related to the operations submenu and notifies the gui events
         */
        private class OperationsMenuListener implements ActionListener {
           public void actionPerformed(ActionEvent e) {
                Operation operation=engine.getOperation(e.getActionCommand());
                OperationTask parameters=new OperationTask(operation.getName());
                parameters.setParameter(OperationTask.OPERATION, operation);
                parameters.setParameter(OperationTask.SOURCE, dataitem);
                gui.launchOperationEditor(parameters);
           }            
        }
        
        /**
         * An inner class that listens to popup-menu events related to the colors submenu
         */
        private class ColorMenuListener implements ActionListener {
           public void actionPerformed(ActionEvent e) {
                String colorname=e.getActionCommand();
                Color color=null;
                if (colorname.equals("Other...")) {
                    color = JColorChooser.showDialog(gui.getFrame(), "Select sequence label color", color);
                } else {
                   for (Object entry:colors) {
                      String name=(String)((Object[])entry)[0];
                      if (name.equals(colorname)) {color=(Color)((Object[])entry)[1];break;}
                   }
                }                     
                if (color!=null) setSequenceLabelColors(gui.getVisualizationSettings(), (SequenceCollection)dataitem, color);
           }            
        }
        
         /**
         * An inner class that listens to popup-menu events NOT related to the operations submenu and notifies the gui events
         */       
        private class DisplayMenuItemListener implements ActionListener {
           public void actionPerformed(ActionEvent e) {
               String dataitemName=dataitem.getName();
              // gui.debugMessage("Selected "+e.getActionCommand()+" on "+dataitemName);

               if (e.getActionCommand().startsWith(RENAME_DATA)) {
                    RenameDataDialog dialog=new RenameDataDialog(gui.getFrame(), gui, dataitemName);
                    dialog.setLocation(100, 250);    
                    dialog.setVisible(true);
                    dialog.dispose();
               } else if (e.getActionCommand().startsWith(DELETE_DATA)) {
                    gui.deleteDataItem(dataitemName);
               } else if (e.getActionCommand().startsWith(EDIT_DATA)) {
                    datapanel.showPrompt(dataitem);
               } else if (e.getActionCommand().equals(SHOW_SEQUENCES)) {
                    setSequenceVisibility(gui.getVisualizationSettings(), (SequenceCollection)dataitem, true);
               } else if (e.getActionCommand().equals(HIDE_SEQUENCES)) {
                    setSequenceVisibility(gui.getVisualizationSettings(), (SequenceCollection)dataitem, false);
               }
           }            
        }
        
   
        /**
         * Just an inner class to sort available operations alphabetically in the popup menu
         */
        private class NameComparator implements Comparator<Operation> {        
            public int compare(Operation o1, Operation o2) {
                return o1.getName().compareTo(o2.getName());
            }
            public boolean equals(Operation o1, Operation o2) {
                return o1.getName().equals(o2.getName());
            }           
        }
        
        /** sets the visibility status of all sequences in the collection to the selected boolean value */
        private void setSequenceVisibility(VisualizationSettings settings, SequenceCollection collection, boolean visibility) {
            String[] sequenceNames=new String[collection.size()];
            sequenceNames=collection.getAllSequenceNames().toArray(sequenceNames);
            settings.setSequenceVisible(sequenceNames, visibility);
        }

        /** Sets the label color of all sequences in the collection to the given color */
        private void setSequenceLabelColors(VisualizationSettings settings, SequenceCollection collection, Color color) {
            for (String sequence:collection.getAllSequenceNames()) {
                settings.setSequenceLabelColor(sequence,color);
            }
            settings.redraw();
        }
             
                            
}
