/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import priorseditor.engine.data.Data;
import priorseditor.engine.DataListener;
import javax.swing.DefaultListModel;
import priorseditor.engine.PriorsEditorEngine;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.SwingUtilities;


/**
 * This class implements a list model that backs up the lists of available
 * datasets (sequences and features) displayed in the left panels of the editor
 * 
 * The listmodel reflects the data that is currently available through the engine
 * and keeps its own ordering for visualization purposes
 * 
 * @author kjetikl
 */
public class DataObjectsPanelListModel extends DefaultListModel implements DataListener {
    PriorsEditorEngine engine=null;
    Class classfilter[]=null;
    
    public DataObjectsPanelListModel(PriorsEditorEngine engine, Class[] classfilter) {
        this.engine=engine;
        this.classfilter=classfilter;
        engine.addDataListener(this);
        // add all applicable dataitems currently in store (if any)
        for (int i=0;i<classfilter.length;i++) {
              ArrayList<Data> list=engine.getAllDataItemsOfType(classfilter[i]);
              for (Data item:list) {
                  if (item==engine.getDefaultSequenceCollection() && engine.getDefaultSequenceCollection().isEmpty()) continue;
                  dataAdded(item);
              }
        }
    }
    
    public PriorsEditorEngine getEngine() {return engine;}

    /**
     * This method is called by the engine when a new data item has been added to the pool
     * @param data
     */
    @Override
    public void dataAdded(final Data data) {
        if (!isAccepted(data)) return;
        if (PriorsEditorEngine.isTemporary(data)) return; // do not show temporary data items
        if (indexOf(data)>=0) return; // data item exists in list? 
        Runnable runner=new Runnable() {
            public void run() {
                if (indexOf(data)<0) addElement(data);
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);                
    }
    
    /**
     * This method is called by the engine when a data item has been removed from the pool
     * @param data
     */
    @Override
    public void dataRemoved(final Data data) {
        if (!isAccepted(data)) return;
        Runnable runner=new Runnable() {
            public void run() {
                removeElement(data);
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);                 
    }
    
    /**
     * This method is called by the engine when a data item has been updated
     * @param data
     */
    @Override
    public void dataUpdated(final Data data) {
        if (!isAccepted(data)) return;
        Runnable runner=new Runnable() {
            public void run() {
                int index=indexOf(data);
                fireContentsChanged(this, index, index);
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);                
    }

    // The following three methods complete  the DataListener interface, but we are not really interested in these events here
    @Override
    public void dataAddedToSet(final Data parent, Data child) {
        if (parent!=engine.getDefaultSequenceCollection()) return;
        if (indexOf(parent)>=0) return; // DefaultSequenceCollection item exists in list
        Runnable runner=new Runnable() {
            public void run() {
                if (indexOf(parent)<0) addElement(parent);
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);                
    }
    
    @Override
    public void dataRemovedFromSet(final Data parent, Data child) {
        if (parent!=engine.getDefaultSequenceCollection()) return;
        if (engine.getDefaultSequenceCollection().isEmpty()) {
            Runnable runner=new Runnable() {
                public void run() {
                    removeElement(parent);
                }
            };
            if (SwingUtilities.isEventDispatchThread()) runner.run();
            else SwingUtilities.invokeLater(runner);             
        }        
    }
    
    @Override    
    public void dataUpdate(Data oldvalue, Data newvalue) {}     
    
    /**
     * Relocates the specified dataset to the new index in the list
     * @param data The dataset to be relocated
     * @param index The new position index that the data should be relocated to
     */
    public void relocate(final Data data,final int index) {
        Runnable runner=new Runnable() {
            public void run() {
                removeElement(data);
                add(index, data);
           }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);                   
    }
    
    /**
     * This method searches through the dataitems in the list to see if any
     * has a name that matches the given name. If found the data item will be
     * return, if not the method will return null.
     * @param name The name of the data item to be retrieved
     * @return Data The data item with the given name (if found) else null
     */
    public Data getDataByName(String name) {    
        for (Enumeration e=elements();e.hasMoreElements();) {
            Data data=(Data)e.nextElement();
            if (data.getName().equals(name)) return data;
        }
        return null;
        
    }
    
    /**
     * This method returns TRUE if the supplied Data argument
     * is an instance of one of the classes accepted by this model
     */
    private boolean isAccepted(Data data) {
        for (int i=0;i<classfilter.length;i++) {
            if (classfilter[i].isInstance(data)) return true;
        }
        return false;
    }
    
}
