/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;


import priorseditor.engine.data.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.ListCellRenderer;

/**
 * Implements List cell renderers to display loaded datasets in the left
 * panels (dataset panels "Sequences" and "Features") of the priorsEditor
 * 
 * @author kjetikl
 */
public class DataPanelCellRenderer extends JPanel implements ListCellRenderer  {
    JLabel typeiconlabel; // a JLabel which is a placeholder for the typeicon
    JLabel name; // a JLabel which displays the dataset name
    SimpleDataPanelIcon typeicon; // the icon to the left of the dataset name which shows the type of dataset
    Color dragOntoColor = new Color(220,220,220);
    VisualizationSettings visualizationsettings;
            
    public DataPanelCellRenderer(VisualizationSettings visualizationsettings) {
        this.visualizationsettings=visualizationsettings;
        setOpaque(false);
        setLayout(new BorderLayout());
        typeicon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.UNKNOWN_ICON,visualizationsettings);
        typeicon.drawBorder(false);
        typeicon.setBackgroundColor(this.getBackground());
        typeiconlabel=new JLabel(typeicon);
        name=new JLabel("");
        name.setMinimumSize(new Dimension(30,20));

        add(typeiconlabel,BorderLayout.WEST);
        add(name,BorderLayout.CENTER);
        setPreferredSize(new Dimension(40,21));
        setMinimumSize(new Dimension(40,21));
        name.setOpaque(true);
        //typeiconlabel.setBorder(BorderFactory.createLineBorder(java.awt.Color.cyan));
        typeiconlabel.setBorder(BorderFactory.createRaisedBevelBorder());
        //typeiconlabel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED, java.awt.Color.gray,java.awt.Color.darkGray));
  
    }
    
    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
        Data dataobject=(Data)value;
        if (isSelected) {
            name.setBackground(list.getSelectionBackground());
            name.setForeground(list.getSelectionForeground());
        } else {
            name.setBackground(list.getBackground());
            //name.setBackground(java.awt.Color.white);
            name.setForeground(list.getForeground());      
        }
        if (list.getDropLocation()!=null && !list.getDropLocation().isInsert() && list.getDropLocation().getIndex()==index) {
            name.setBackground(dragOntoColor);
        }
        if (dataobject instanceof NumericDataset || dataobject instanceof RegionDataset) {
          Color iconforeground=visualizationsettings.getForeGroundColor(dataobject.getName());
          Color iconbackground=visualizationsettings.getBackGroundColor(dataobject.getName());
          typeicon.setForegroundColor(iconforeground);
          typeicon.setBackgroundColor(iconbackground);
        } else {
          typeicon.setForegroundColor(Color.black);
          typeicon.setBackgroundColor(Color.white);            
        }
        

        if (!visualizationsettings.isVisible(dataobject.getName())){
            typeicon.setIconType(SimpleDataPanelIcon.HIDDEN_ICON);
            if (dataobject instanceof FeatureDataset) name.setForeground(Color.LIGHT_GRAY);
        }
        else if (dataobject instanceof DNASequenceDataset) {typeicon.setIconType(SimpleDataPanelIcon.SEQUENCE_ICON_BASES);}
        else if (dataobject instanceof SequenceCollection) {typeicon.setIconType(SimpleDataPanelIcon.SEQUENCE_COLLECTION_ICON);}
        else if (dataobject instanceof RegionDataset) {
            int graphType=visualizationsettings.getGraphType(dataobject.getName());
                 if (graphType==VisualizationSettings.REGION_GRAPH) typeicon.setIconType(SimpleDataPanelIcon.REGION_ICON);
            else if (graphType==VisualizationSettings.REGION_MULTICOLOR_GRAPH) typeicon.setIconType(SimpleDataPanelIcon.REGION_MULTICOLOR_ICON);
            else if (graphType==VisualizationSettings.GRADIENT_REGION_GRAPH) typeicon.setIconType(SimpleDataPanelIcon.REGION_GRADIENT_ICON);
            else { // no correct graph type has been selected. Select default region graph type
              visualizationsettings.setGraphType(dataobject.getName(), VisualizationSettings.REGION_GRAPH);
              typeicon.setIconType(SimpleDataPanelIcon.REGION_ICON);
            }         
        } else if (dataobject instanceof NumericDataset) {
            int graphType=visualizationsettings.getGraphType(dataobject.getName());               
                 if (graphType==VisualizationSettings.BAR_GRAPH) typeicon.setIconType(SimpleDataPanelIcon.NUMERIC_TRACK_GRAPH_ICON);
            else if (graphType==VisualizationSettings.GRADIENT_GRAPH) typeicon.setIconType(SimpleDataPanelIcon.NUMERIC_TRACK_GRADIENT_ICON);
            else {// no correct graph type has been selected. Select default numeric track graph type
              visualizationsettings.setGraphType(dataobject.getName(), VisualizationSettings.BAR_GRAPH);
              typeicon.setIconType(SimpleDataPanelIcon.NUMERIC_TRACK_GRAPH_ICON);                
            }
        } 
        else if (dataobject instanceof NumericVariable) {typeicon.setIconType(SimpleDataPanelIcon.NUMERIC_VARIABLE_ICON); } 
        else if (dataobject instanceof NumericMapVariable) {typeicon.setIconType(SimpleDataPanelIcon.NUMERIC_MAP_VARIABLE_ICON); } 
        else if (dataobject instanceof BackgroundModel) {typeicon.setIconType(SimpleDataPanelIcon.MARKOV_MODEL_ICON); } 
        else if (dataobject instanceof StringVariable) {typeicon.setIconType(SimpleDataPanelIcon.STRING_VARIABLE_ICON); } 
        else if (dataobject instanceof StringCollection) {typeicon.setIconType(SimpleDataPanelIcon.STRING_COLLECTION_ICON); } 
        else {typeicon.setIconType(SimpleDataPanelIcon.UNKNOWN_ICON); }
        String typedescription=dataobject.getTypeDescription();
        if (dataobject instanceof RegionDataset && ((RegionDataset)dataobject).isModuleTrack()) {
            typedescription+=", Module track";
        }
        else if (dataobject instanceof RegionDataset && ((RegionDataset)dataobject).isMotifTrack()) {
            typedescription+=", Motif track";
        }
        this.setToolTipText(dataobject.getName()+"    [ "+typedescription+" ]");
        name.setText("   "+dataobject.getName());
        return this;
    }



}
