/*
 * DataTrackConfigurationDialog.java
 *
 * Created on 18. juni 2009, 11:00
 */

package priorseditor.gui;

import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.SwingWorker;
import org.jdesktop.application.Action;
import priorseditor.engine.data.DNASequenceDataset;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.Organism;
import priorseditor.engine.datatracks.DataTrack;
import priorseditor.engine.datatracks.DataSource;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import priorseditor.engine.DataConfiguration;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.dataformat.DataFormat;
import priorseditor.engine.datatracks.DataSource_DAS;
import priorseditor.engine.datatracks.DataSource_http_GET;
import priorseditor.engine.datatracks.Server;

/**
 *
 * @author  kjetikl
 */
public class DataTrackConfigurationDialog extends javax.swing.JDialog {
    private final String TABLECOLUMN_NAME="Name";    
    private final String TABLECOLUMN_TYPE="Type";
    private final String TABLECOLUMN_SOURCE="Source";
    private final String TABLECOLUMN_ORGANISM="Organism";
    private final String TABLECOLUMN_BUILD="Build";
    private final String TABLECOLUMN_SUPPORTED_ORGANISMS="Supported organism";
    private final String TABLECOLUMN_PROTOCOL="Protocol";
    private final String TABLECOLUMN_DATAFORMAT="Format";
    private final String TABLECOLUMN_SERVER="Server";
    private final String TABLECOLUMN_SERVERADDRESS="Server address";
    private final String TABLECOLUMN_DELAY="Delay (ms)";
    private final String TABLECOLUMN_MIRRORS="Mirror sites";

    private PriorsEditorGUI gui;
    private PriorsEditorEngine engine;
    
    private DefaultTableModel datatrackTableModel;
    private DefaultTableModel datasourceTableModel;
    private DefaultTableModel serverTableModel;
    private JTable datatrackTable;
    private JTable datasourceTable;
    private JTable serverTable;
    private ParametersPanel dataFormatParametersPanel=null;
    private SupportedOrganismsRendered organismRenderer=new SupportedOrganismsRendered();
    private DataConfiguration dataconfiguration=null;
    private HashMap<String,DataTrack> availableTracks=null;
    private HashMap<String,Server> availableServers=null;
    private DataTrack currentDataTrack=null;
    private DataSource currentDataSource=null;
    private boolean addDataTrack=false; // TRUE when adding, FALSE when editing
    private boolean addDataSource=false;// TRUE when adding, FALSE when editing
    
    
    /** Creates new form DataTrackDialog */
    @SuppressWarnings("unchecked")
    public DataTrackConfigurationDialog(PriorsEditorGUI gui) {
        super(gui.getFrame(), true);
        this.gui=gui;
        engine=gui.getEngine();
        initComponents();
        dataconfiguration=(DataConfiguration)gui.getEngine().getDataLoader().getCurrentDataConfiguration().clone();
        availableTracks=dataconfiguration.getAvailableTracks(); // shortcuts
        availableServers=dataconfiguration.getServers();        // shortcuts
        
        datatrackTableModel=new DefaultTableModel(new String[]{TABLECOLUMN_NAME,TABLECOLUMN_TYPE,TABLECOLUMN_SOURCE,TABLECOLUMN_SUPPORTED_ORGANISMS},0);
        datatrackTable=new JTable(datatrackTableModel) {
            @Override
            public boolean isCellEditable(int row, int col) {return false;}
        };

        datatrackTable.setFillsViewportHeight(true);
        datatrackTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        datatrackTable.setRowSelectionAllowed(true);
        datatrackTable.getTableHeader().setReorderingAllowed(false);
        datatrackTable.getColumn(TABLECOLUMN_TYPE).setMaxWidth(50);
        datatrackTable.getColumn(TABLECOLUMN_TYPE).setMaxWidth(50);
        datatrackTable.getColumn(TABLECOLUMN_TYPE).setCellRenderer(new DataTypeRenderer());
        datatrackTable.getColumn(TABLECOLUMN_SUPPORTED_ORGANISMS).setCellRenderer(organismRenderer);               
        datatrackTable.setRowHeight(18);
        cardpaneAllTracksScrollPane.setViewportView(datatrackTable);  
      
        datatrackTable.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
               super.mouseClicked(e);
               if (e.getButton()==MouseEvent.BUTTON1 && e.getClickCount()==2) {
                   allTracksEditButtonClicked();
               }
            }
            
        });
        datatrackTable.setAutoCreateRowSorter(true);
        datatrackTable.getRowSorter().toggleSortOrder(datatrackTable.getColumn(TABLECOLUMN_NAME).getModelIndex());

        datasourceTableModel=new DefaultTableModel(new String[]{TABLECOLUMN_ORGANISM,TABLECOLUMN_BUILD,TABLECOLUMN_PROTOCOL,TABLECOLUMN_DATAFORMAT,TABLECOLUMN_SERVER},0);      
        datasourceTable=new JTable(datasourceTableModel) {
            @Override
            public boolean isCellEditable(int row, int col) {return false;}
        };          
        datasourceTable.setFillsViewportHeight(true);
        datasourceTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        datasourceTable.setRowSelectionAllowed(true);
        datasourceTable.getTableHeader().setReorderingAllowed(false);
        datasourceTable.setRowHeight(18);
        datasourceTable.getColumn(TABLECOLUMN_ORGANISM).setCellRenderer(organismRenderer);
        datasourceTable.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
               super.mouseClicked(e);
               if (e.getButton()==MouseEvent.BUTTON1 && e.getClickCount()==2) {
                   editDataSource();
               }
            }
            
        });
        cardpaneEditTracksScrollPane.setViewportView(datasourceTable);      

        
        serverTableModel=new DefaultTableModel(new String[]{TABLECOLUMN_SERVERADDRESS,TABLECOLUMN_DELAY,TABLECOLUMN_MIRRORS},0);      
        serverTable=new JTable(serverTableModel); 
        ExcelAdapter adapter=new ExcelAdapter(serverTable,true, ExcelAdapter.CONVERT_TO_INTEGER); // enables copy/paste capabilities in the table
        serverTable.setFillsViewportHeight(true);
        serverTable.getTableHeader().setReorderingAllowed(false);
        serverTable.setRowHeight(18);
        serversScrollPane.setViewportView(serverTable);      
        serverTable.setAutoCreateRowSorter(true);
        serverTable.setCellSelectionEnabled(true);
        serverTable.getRowSorter().toggleSortOrder(serverTable.getColumn(TABLECOLUMN_SERVERADDRESS).getModelIndex());
        
        setupTablesFromConfiguration();
        editSourceProtocolCombobox.setModel(new DefaultComboBoxModel(DataSource.getSupportedProtocols()));

        ArrayList<DataFormat> inputformatsList=gui.getEngine().getFeatureDataInputFormats();
        String[] list=new String[inputformatsList.size()];
        int i=0;
        for (DataFormat formatter:inputformatsList) {
            list[i]=formatter.getName();
            i++;
        }
        editSourceDataFormatCombobox.setModel(new DefaultComboBoxModel(list));
        editSourceOrganismCombobox.setModel(new DefaultComboBoxModel(Organism.getSupportedOrganismIDs()));
        OrganismComboboxRenderer organismrenderer=new OrganismComboboxRenderer();
        editSourceOrganismCombobox.setRenderer(organismrenderer);
        editSourceProtocolCombobox.setMinimumSize(new Dimension(50, 20));        
        editSourceProtocolCombobox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (editSourceProtocolCombobox.getSelectedItem().toString().equals("DAS")) {
                    editSourceDataFormatCombobox.setEnabled(false);
                    additionalDataFormatSettingsPanel.setVisible(false);
                } else {
                    editSourceDataFormatCombobox.setEnabled(true);
                    additionalDataFormatSettingsPanel.setVisible(true);
                }
            }
        });       
        editSourceDataFormatCombobox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object selected=editSourceDataFormatCombobox.getSelectedItem();
                if (selected!=null) showDataFormatSettingsPanel(getDataFormatSettingsPanel((String)selected)); 
                else additionalDataFormatSettingsPanel.removeAll();
                pack();
            }
        });                
        showTopMenuPanel();
        pack();
        gui.getFrame().setCursor(Cursor.getDefaultCursor());
    }
    
    /** */
    private void showDataFormatSettingsPanel(JPanel panel) {
        additionalDataFormatSettingsPanel.removeAll();
        additionalDataFormatSettingsPanel.add(panel);  
        additionalDataFormatSettingsPanel.invalidate();
        additionalDataFormatSettingsPanel.repaint();
        
    } 
    
    /** */
    private JPanel getDataFormatSettingsPanel(String outputformatName) {
        DataFormat outputFormat=engine.getDataFormat(outputformatName);
        if (outputFormat==null) {
            dataFormatParametersPanel=null;
            return new JPanel();
        }
        //ParametersPanel panel=new ParametersPanel(outputFormat.getParameters(),parameterSettings, this);
        ParameterSettings settings=null;
        if (currentDataSource!=null) settings=currentDataSource.getDataFormatSettings();
        ParametersPanel panel=new ParametersPanel(outputFormat.getParameters(),settings, null);
        dataFormatParametersPanel=panel;
        return panel;
    }
    
    
    /** Fills in data in the DataTracks and DataSources JTables based on the current config */
    private void setupTablesFromConfiguration() {  
         int tracknumber=datatrackTableModel.getRowCount();
         if (tracknumber>0) {
             for (int i=0;i<tracknumber;i++) datatrackTableModel.removeRow(0);
         }
         int servernumber=serverTableModel.getRowCount();
         if (servernumber>0) {
             for (int i=0;i<servernumber;i++) serverTableModel.removeRow(0);
         }
         for (DataTrack track:availableTracks.values()) {
            int[] supportedOrganisms=track.getSupportedOrganisms();
            Object[] values=new Object[]{track.getName(),track.getDataType(),track.getSourceSite(),supportedOrganisms};
            datatrackTableModel.addRow(values);
        }           
        for (Server server:availableServers.values()) {
            String mirrors="";
            ArrayList<String> mirrorsites=server.getMirrorSites();
            if (mirrorsites!=null) {
              for (int i=0;i<mirrorsites.size();i++) {
                if (i<mirrorsites.size()-1) mirrors+=mirrorsites.get(i)+",";
                else mirrors+=mirrorsites.get(i);
              }
            }
            Object[] values=new Object[]{server.getServerAddress(), new Integer(server.getServerDelay()),mirrors};
            serverTableModel.addRow(values);
        }          
    }
    

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        cardpaneTopMenu = new javax.swing.JPanel();
        cardpaneTopMenuTopPanel = new javax.swing.JPanel();
        cardpaneTopMenuMainPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        configureDatatracksButton = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        configureServerSettingsButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        importSettingsButton = new javax.swing.JButton();
        jPanel4 = new javax.swing.JPanel();
        exportSettingsButton = new javax.swing.JButton();
        jPanel5 = new javax.swing.JPanel();
        revertSettingsButton = new javax.swing.JButton();
        cardpaneTopMenuButtonPanel = new javax.swing.JPanel();
        saveChangesButton = new javax.swing.JButton();
        exitWithoutSaveButton = new javax.swing.JButton();
        cardpaneAllTracks = new javax.swing.JPanel();
        cardpaneAllTracksMainPanel = new javax.swing.JPanel();
        cardpaneAllTracksScrollPane = new javax.swing.JScrollPane();
        datatracksLabel = new javax.swing.JLabel();
        cardpaneAllTracksTopPanel = new javax.swing.JPanel();
        cardpaneAllTracksButtonPanel = new javax.swing.JPanel();
        allTracksEditButton = new javax.swing.JButton();
        allTracksAddButton = new javax.swing.JButton();
        allTracksRemoveButton = new javax.swing.JButton();
        allTracksOKButton = new javax.swing.JButton();
        allTracksAddFromUCSCButton = new javax.swing.JButton();
        cardpaneEditTrack = new javax.swing.JPanel();
        cardpaneEditTrackTopPanel = new javax.swing.JPanel();
        editTrackPropertiesPanel = new javax.swing.JPanel();
        dataTrackPropertiesNameLabel = new javax.swing.JLabel();
        dataTrackPropertiesNameTextfield = new javax.swing.JTextField();
        dataTrackPropertiesSourceLabel = new javax.swing.JLabel();
        dataTrackPropertiesSourceTextfield = new javax.swing.JTextField();
        dataTrackPropertiesDescriptionLabel = new javax.swing.JLabel();
        dataTrackPropertiesDescriptionTextfield = new javax.swing.JTextField();
        dataTrackPropertiesTypeLabel = new javax.swing.JLabel();
        dataTrackPropertiesTypeCombobox = new javax.swing.JComboBox();
        cardpaneEditTrackMainPanel = new javax.swing.JPanel();
        cardpaneEditTracksScrollPane = new javax.swing.JScrollPane();
        datasourcesLabel = new javax.swing.JLabel();
        cardpaneEditTrackButtonPanel = new javax.swing.JPanel();
        editTrackOKButton = new javax.swing.JButton();
        editTrackCancelButton = new javax.swing.JButton();
        editTrackEditButton = new javax.swing.JButton();
        editTrackAddButton = new javax.swing.JButton();
        editTrackRemoveButton = new javax.swing.JButton();
        editTrackAddSourceFromUCSCButton = new javax.swing.JButton();
        cardpaneEditSource = new javax.swing.JPanel();
        cardpaneEditSourceTopPanel = new javax.swing.JPanel();
        editSourcePropertiesPanel = new javax.swing.JPanel();
        editSourceTrackNameLabel = new javax.swing.JLabel();
        editSourceTrackNameTextfield = new javax.swing.JTextField();
        editSourceOrganismLabel = new javax.swing.JLabel();
        editSourceOrganismCombobox = new javax.swing.JComboBox();
        editSourceBuildLabel = new javax.swing.JLabel();
        editSourceBuildTextfield = new javax.swing.JTextField();
        editSourceProtocoLabel = new javax.swing.JLabel();
        editSourceProtocolCombobox = new javax.swing.JComboBox();
        editSourceDataFormatLabel = new javax.swing.JLabel();
        editSourceDataFormatCombobox = new javax.swing.JComboBox();
        cardpaneEditSourceButtonsPanel = new javax.swing.JPanel();
        editSourceOKButton = new javax.swing.JButton();
        editSourceCancelButton = new javax.swing.JButton();
        cardpaneEditSourceMainCard = new javax.swing.JPanel();
        internalWrapper1 = new javax.swing.JPanel();
        cardpaneEditSourceMainCardPanel = new javax.swing.JPanel();
        editGETsourcePanel = new javax.swing.JPanel();
        editGETsourceBaseURLlabel = new javax.swing.JLabel();
        editGETsourceBaseURLtextfield = new javax.swing.JTextField();
        editGETsourceParametersLabel = new javax.swing.JLabel();
        editGETsourceParametersTextfield = new javax.swing.JTextField();
        editDASsourcePanel = new javax.swing.JPanel();
        editDASsourceBaseURLlabel = new javax.swing.JLabel();
        editDASsourceBaseURLtextfield = new javax.swing.JTextField();
        editDASsourceFeatureLabel = new javax.swing.JLabel();
        editDASsourceFeatureTextfield = new javax.swing.JTextField();
        DataFormatSettingsTopPanel = new javax.swing.JPanel();
        additionalDataFormatSettingsPanel = new javax.swing.JPanel();
        cardpaneServers = new javax.swing.JPanel();
        cardpaneServersTopPanel = new javax.swing.JPanel();
        cardpaneServersMainPanel = new javax.swing.JPanel();
        serversLabel = new javax.swing.JLabel();
        serversScrollPane = new javax.swing.JScrollPane();
        cardpaneServersButtonPanel = new javax.swing.JPanel();
        serversOKButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(DataTrackConfigurationDialog.class);
        setTitle(resourceMap.getString("Form.title")); // NOI18N
        setName("Form"); // NOI18N
        getContentPane().setLayout(new java.awt.CardLayout());

        cardpaneTopMenu.setName("cardpaneTopMenu"); // NOI18N
        cardpaneTopMenu.setLayout(new java.awt.BorderLayout());

        cardpaneTopMenuTopPanel.setName("cardpaneTopMenuTopPanel"); // NOI18N
        cardpaneTopMenuTopPanel.setPreferredSize(new java.awt.Dimension(100, 10));

        javax.swing.GroupLayout cardpaneTopMenuTopPanelLayout = new javax.swing.GroupLayout(cardpaneTopMenuTopPanel);
        cardpaneTopMenuTopPanel.setLayout(cardpaneTopMenuTopPanelLayout);
        cardpaneTopMenuTopPanelLayout.setHorizontalGroup(
            cardpaneTopMenuTopPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 670, Short.MAX_VALUE)
        );
        cardpaneTopMenuTopPanelLayout.setVerticalGroup(
            cardpaneTopMenuTopPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 10, Short.MAX_VALUE)
        );

        cardpaneTopMenu.add(cardpaneTopMenuTopPanel, java.awt.BorderLayout.PAGE_START);

        cardpaneTopMenuMainPanel.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createEmptyBorder(10, 10, 10, 10), javax.swing.BorderFactory.createEtchedBorder()));
        cardpaneTopMenuMainPanel.setName("cardpaneTopMenuMainPanel"); // NOI18N
        cardpaneTopMenuMainPanel.setLayout(new javax.swing.BoxLayout(cardpaneTopMenuMainPanel, javax.swing.BoxLayout.Y_AXIS));

        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 5, 25));

        configureDatatracksButton.setText(resourceMap.getString("configureDatatracksButton.text")); // NOI18N
        configureDatatracksButton.setMaximumSize(new java.awt.Dimension(250, 31));
        configureDatatracksButton.setMinimumSize(new java.awt.Dimension(250, 31));
        configureDatatracksButton.setName("configureDatatracksButton"); // NOI18N
        configureDatatracksButton.setPreferredSize(new java.awt.Dimension(250, 31));
        configureDatatracksButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                configureDatatracksButtonClicked(evt);
            }
        });
        jPanel1.add(configureDatatracksButton);

        cardpaneTopMenuMainPanel.add(jPanel1);

        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 5, 25));

        configureServerSettingsButton.setText(resourceMap.getString("configureServerSettingsButton.text")); // NOI18N
        configureServerSettingsButton.setMaximumSize(new java.awt.Dimension(250, 31));
        configureServerSettingsButton.setMinimumSize(new java.awt.Dimension(250, 31));
        configureServerSettingsButton.setName("configureServerSettingsButton"); // NOI18N
        configureServerSettingsButton.setPreferredSize(new java.awt.Dimension(250, 31));
        configureServerSettingsButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                configureServersSettingsButtonClicked(evt);
            }
        });
        jPanel2.add(configureServerSettingsButton);

        cardpaneTopMenuMainPanel.add(jPanel2);

        jPanel3.setName("jPanel3"); // NOI18N
        jPanel3.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 5, 25));

        importSettingsButton.setText(resourceMap.getString("importSettingsButton.text")); // NOI18N
        importSettingsButton.setMaximumSize(new java.awt.Dimension(250, 31));
        importSettingsButton.setMinimumSize(new java.awt.Dimension(250, 31));
        importSettingsButton.setName("importSettingsButton"); // NOI18N
        importSettingsButton.setPreferredSize(new java.awt.Dimension(250, 31));
        importSettingsButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                importSettings(evt);
            }
        });
        jPanel3.add(importSettingsButton);

        cardpaneTopMenuMainPanel.add(jPanel3);

        jPanel4.setName("jPanel4"); // NOI18N
        jPanel4.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 5, 25));

        exportSettingsButton.setText(resourceMap.getString("exportSettingsButton.text")); // NOI18N
        exportSettingsButton.setMaximumSize(new java.awt.Dimension(250, 31));
        exportSettingsButton.setMinimumSize(new java.awt.Dimension(250, 31));
        exportSettingsButton.setName("exportSettingsButton"); // NOI18N
        exportSettingsButton.setPreferredSize(new java.awt.Dimension(250, 31));
        exportSettingsButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                exportSettings(evt);
            }
        });
        jPanel4.add(exportSettingsButton);

        cardpaneTopMenuMainPanel.add(jPanel4);

        jPanel5.setMinimumSize(new java.awt.Dimension(235, 41));
        jPanel5.setName("jPanel5"); // NOI18N
        jPanel5.setPreferredSize(new java.awt.Dimension(235, 91));
        jPanel5.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 5, 25));

        revertSettingsButton.setText(resourceMap.getString("revertSettingsButton.text")); // NOI18N
        revertSettingsButton.setMaximumSize(new java.awt.Dimension(250, 31));
        revertSettingsButton.setMinimumSize(new java.awt.Dimension(250, 31));
        revertSettingsButton.setName("revertSettingsButton"); // NOI18N
        revertSettingsButton.setPreferredSize(new java.awt.Dimension(250, 31));
        revertSettingsButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                revertSettings(evt);
            }
        });
        jPanel5.add(revertSettingsButton);

        cardpaneTopMenuMainPanel.add(jPanel5);

        cardpaneTopMenu.add(cardpaneTopMenuMainPanel, java.awt.BorderLayout.CENTER);

        cardpaneTopMenuButtonPanel.setMinimumSize(new java.awt.Dimension(0, 46));
        cardpaneTopMenuButtonPanel.setName("cardpaneTopMenuButtonPanel"); // NOI18N
        cardpaneTopMenuButtonPanel.setPreferredSize(new java.awt.Dimension(100, 46));
        cardpaneTopMenuButtonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 25, 5));

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(DataTrackConfigurationDialog.class, this);
        saveChangesButton.setAction(actionMap.get("saveChangesAndExitAction")); // NOI18N
        saveChangesButton.setMaximumSize(new java.awt.Dimension(190, 29));
        saveChangesButton.setMinimumSize(new java.awt.Dimension(190, 29));
        saveChangesButton.setName("saveChangesButton"); // NOI18N
        saveChangesButton.setPreferredSize(new java.awt.Dimension(190, 29));
        cardpaneTopMenuButtonPanel.add(saveChangesButton);

        exitWithoutSaveButton.setText(resourceMap.getString("exitWithoutSaveButton.text")); // NOI18N
        exitWithoutSaveButton.setMaximumSize(new java.awt.Dimension(170, 29));
        exitWithoutSaveButton.setMinimumSize(new java.awt.Dimension(170, 29));
        exitWithoutSaveButton.setName("exitWithoutSaveButton"); // NOI18N
        exitWithoutSaveButton.setPreferredSize(new java.awt.Dimension(170, 29));
        exitWithoutSaveButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                exitWithoutSaving(evt);
            }
        });
        cardpaneTopMenuButtonPanel.add(exitWithoutSaveButton);

        cardpaneTopMenu.add(cardpaneTopMenuButtonPanel, java.awt.BorderLayout.PAGE_END);

        getContentPane().add(cardpaneTopMenu, "cardpaneTopMenu");

        cardpaneAllTracks.setName("cardpaneAllTracks"); // NOI18N
        cardpaneAllTracks.setLayout(new java.awt.BorderLayout());

        cardpaneAllTracksMainPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 10, 1, 10));
        cardpaneAllTracksMainPanel.setName("cardpaneAllTracksMainPanel"); // NOI18N
        cardpaneAllTracksMainPanel.setLayout(new java.awt.BorderLayout());

        cardpaneAllTracksScrollPane.setName("cardpaneAllTracksScrollPane"); // NOI18N
        cardpaneAllTracksMainPanel.add(cardpaneAllTracksScrollPane, java.awt.BorderLayout.CENTER);

        datatracksLabel.setFont(resourceMap.getFont("datatracksLabel.font")); // NOI18N
        datatracksLabel.setText(resourceMap.getString("datatracksLabel.text")); // NOI18N
        datatracksLabel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 5, 1));
        datatracksLabel.setName("datatracksLabel"); // NOI18N
        cardpaneAllTracksMainPanel.add(datatracksLabel, java.awt.BorderLayout.PAGE_START);

        cardpaneAllTracks.add(cardpaneAllTracksMainPanel, java.awt.BorderLayout.CENTER);

        cardpaneAllTracksTopPanel.setMaximumSize(new java.awt.Dimension(32767, 40));
        cardpaneAllTracksTopPanel.setName("cardpaneAllTracksTopPanel"); // NOI18N
        cardpaneAllTracksTopPanel.setPreferredSize(new java.awt.Dimension(100, 40));

        javax.swing.GroupLayout cardpaneAllTracksTopPanelLayout = new javax.swing.GroupLayout(cardpaneAllTracksTopPanel);
        cardpaneAllTracksTopPanel.setLayout(cardpaneAllTracksTopPanelLayout);
        cardpaneAllTracksTopPanelLayout.setHorizontalGroup(
            cardpaneAllTracksTopPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 670, Short.MAX_VALUE)
        );
        cardpaneAllTracksTopPanelLayout.setVerticalGroup(
            cardpaneAllTracksTopPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 40, Short.MAX_VALUE)
        );

        cardpaneAllTracks.add(cardpaneAllTracksTopPanel, java.awt.BorderLayout.PAGE_START);

        cardpaneAllTracksButtonPanel.setMaximumSize(new java.awt.Dimension(32767, 90));
        cardpaneAllTracksButtonPanel.setMinimumSize(new java.awt.Dimension(0, 46));
        cardpaneAllTracksButtonPanel.setName("cardpaneAllTracksButtonPanel"); // NOI18N
        cardpaneAllTracksButtonPanel.setPreferredSize(new java.awt.Dimension(100, 46));

        allTracksEditButton.setAction(actionMap.get("allTracksEditButtonClicked")); // NOI18N
        allTracksEditButton.setMaximumSize(new java.awt.Dimension(90, 27));
        allTracksEditButton.setMinimumSize(new java.awt.Dimension(90, 27));
        allTracksEditButton.setName("allTracksEditButton"); // NOI18N
        allTracksEditButton.setPreferredSize(new java.awt.Dimension(90, 27));

        allTracksAddButton.setAction(actionMap.get("allTrackAddButtonClicked")); // NOI18N
        allTracksAddButton.setText(resourceMap.getString("allTracksAddButton.text")); // NOI18N
        allTracksAddButton.setMaximumSize(new java.awt.Dimension(90, 27));
        allTracksAddButton.setMinimumSize(new java.awt.Dimension(90, 27));
        allTracksAddButton.setName("allTracksAddButton"); // NOI18N
        allTracksAddButton.setPreferredSize(new java.awt.Dimension(90, 27));

        allTracksRemoveButton.setAction(actionMap.get("allTracksRemoveButtonClicked")); // NOI18N
        allTracksRemoveButton.setText(resourceMap.getString("allTracksRemoveButton.text")); // NOI18N
        allTracksRemoveButton.setMaximumSize(new java.awt.Dimension(126, 27));
        allTracksRemoveButton.setMinimumSize(new java.awt.Dimension(126, 27));
        allTracksRemoveButton.setName("allTracksRemoveButton"); // NOI18N
        allTracksRemoveButton.setPreferredSize(new java.awt.Dimension(126, 27));

        allTracksOKButton.setAction(actionMap.get("allTracksOKButtonClickedAction")); // NOI18N
        allTracksOKButton.setText(resourceMap.getString("allTracksOKButton.text")); // NOI18N
        allTracksOKButton.setMaximumSize(new java.awt.Dimension(82, 27));
        allTracksOKButton.setMinimumSize(new java.awt.Dimension(82, 27));
        allTracksOKButton.setName("allTracksOKButton"); // NOI18N
        allTracksOKButton.setPreferredSize(new java.awt.Dimension(82, 27));

        allTracksAddFromUCSCButton.setAction(actionMap.get("allTracksAddFromUCSCButtonClicked")); // NOI18N
        allTracksAddFromUCSCButton.setMaximumSize(new java.awt.Dimension(216, 27));
        allTracksAddFromUCSCButton.setMinimumSize(new java.awt.Dimension(216, 27));
        allTracksAddFromUCSCButton.setName("allTracksAddFromUCSCButton"); // NOI18N
        allTracksAddFromUCSCButton.setPreferredSize(new java.awt.Dimension(216, 27));

        javax.swing.GroupLayout cardpaneAllTracksButtonPanelLayout = new javax.swing.GroupLayout(cardpaneAllTracksButtonPanel);
        cardpaneAllTracksButtonPanel.setLayout(cardpaneAllTracksButtonPanelLayout);
        cardpaneAllTracksButtonPanelLayout.setHorizontalGroup(
            cardpaneAllTracksButtonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(cardpaneAllTracksButtonPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(allTracksEditButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(allTracksAddButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(allTracksAddFromUCSCButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(allTracksRemoveButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 24, Short.MAX_VALUE)
                .addComponent(allTracksOKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        cardpaneAllTracksButtonPanelLayout.setVerticalGroup(
            cardpaneAllTracksButtonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, cardpaneAllTracksButtonPanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(cardpaneAllTracksButtonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(allTracksEditButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(allTracksAddButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(allTracksRemoveButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(allTracksOKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(allTracksAddFromUCSCButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        cardpaneAllTracks.add(cardpaneAllTracksButtonPanel, java.awt.BorderLayout.PAGE_END);

        getContentPane().add(cardpaneAllTracks, "cardpaneAllTracks");

        cardpaneEditTrack.setName("cardpaneEditTrack"); // NOI18N
        cardpaneEditTrack.setLayout(new java.awt.BorderLayout());

        cardpaneEditTrackTopPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(10, 10, 10, 10));
        cardpaneEditTrackTopPanel.setMinimumSize(new java.awt.Dimension(20, 150));
        cardpaneEditTrackTopPanel.setName("cardpaneEditTrackTopPanel"); // NOI18N
        cardpaneEditTrackTopPanel.setPreferredSize(new java.awt.Dimension(100, 150));
        cardpaneEditTrackTopPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 0, 0));

        editTrackPropertiesPanel.setMinimumSize(new java.awt.Dimension(400, 20));
        editTrackPropertiesPanel.setName("editTrackPropertiesPanel"); // NOI18N
        editTrackPropertiesPanel.setLayout(new java.awt.GridBagLayout());

        dataTrackPropertiesNameLabel.setText(resourceMap.getString("dataTrackPropertiesNameLabel.text")); // NOI18N
        dataTrackPropertiesNameLabel.setName("dataTrackPropertiesNameLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editTrackPropertiesPanel.add(dataTrackPropertiesNameLabel, gridBagConstraints);

        dataTrackPropertiesNameTextfield.setColumns(20);
        dataTrackPropertiesNameTextfield.setText(resourceMap.getString("dataTrackPropertiesNameTextfield.text")); // NOI18N
        dataTrackPropertiesNameTextfield.setName("dataTrackPropertiesNameTextfield"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editTrackPropertiesPanel.add(dataTrackPropertiesNameTextfield, gridBagConstraints);

        dataTrackPropertiesSourceLabel.setText(resourceMap.getString("dataTrackPropertiesSourceLabel.text")); // NOI18N
        dataTrackPropertiesSourceLabel.setName("dataTrackPropertiesSourceLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editTrackPropertiesPanel.add(dataTrackPropertiesSourceLabel, gridBagConstraints);

        dataTrackPropertiesSourceTextfield.setColumns(20);
        dataTrackPropertiesSourceTextfield.setText(resourceMap.getString("dataTrackPropertiesSourceTextfield.text")); // NOI18N
        dataTrackPropertiesSourceTextfield.setName("dataTrackPropertiesSourceTextfield"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editTrackPropertiesPanel.add(dataTrackPropertiesSourceTextfield, gridBagConstraints);

        dataTrackPropertiesDescriptionLabel.setText(resourceMap.getString("dataTrackPropertiesDescriptionLabel.text")); // NOI18N
        dataTrackPropertiesDescriptionLabel.setName("dataTrackPropertiesDescriptionLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editTrackPropertiesPanel.add(dataTrackPropertiesDescriptionLabel, gridBagConstraints);

        dataTrackPropertiesDescriptionTextfield.setColumns(50);
        dataTrackPropertiesDescriptionTextfield.setText(resourceMap.getString("dataTrackPropertiesDescriptionTextfield.text")); // NOI18N
        dataTrackPropertiesDescriptionTextfield.setName("dataTrackPropertiesDescriptionTextfield"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editTrackPropertiesPanel.add(dataTrackPropertiesDescriptionTextfield, gridBagConstraints);

        dataTrackPropertiesTypeLabel.setText(resourceMap.getString("dataTrackPropertiesTypeLabel.text")); // NOI18N
        dataTrackPropertiesTypeLabel.setName("dataTrackPropertiesTypeLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editTrackPropertiesPanel.add(dataTrackPropertiesTypeLabel, gridBagConstraints);

        dataTrackPropertiesTypeCombobox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Sequence", "Region", "Numeric" }));
        dataTrackPropertiesTypeCombobox.setName("dataTrackPropertiesTypeCombobox"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editTrackPropertiesPanel.add(dataTrackPropertiesTypeCombobox, gridBagConstraints);

        cardpaneEditTrackTopPanel.add(editTrackPropertiesPanel);

        cardpaneEditTrack.add(cardpaneEditTrackTopPanel, java.awt.BorderLayout.PAGE_START);

        cardpaneEditTrackMainPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 10, 1, 10));
        cardpaneEditTrackMainPanel.setName("cardpaneEditTrackMainPanel"); // NOI18N
        cardpaneEditTrackMainPanel.setLayout(new java.awt.BorderLayout());

        cardpaneEditTracksScrollPane.setName("cardpaneEditTracksScrollPane"); // NOI18N
        cardpaneEditTracksScrollPane.setPreferredSize(new java.awt.Dimension(20, 100));
        cardpaneEditTrackMainPanel.add(cardpaneEditTracksScrollPane, java.awt.BorderLayout.CENTER);

        datasourcesLabel.setFont(resourceMap.getFont("datasourcesLabel.font")); // NOI18N
        datasourcesLabel.setText(resourceMap.getString("datasourcesLabel.text")); // NOI18N
        datasourcesLabel.setBorder(javax.swing.BorderFactory.createEmptyBorder(18, 1, 5, 1));
        datasourcesLabel.setName("datasourcesLabel"); // NOI18N
        cardpaneEditTrackMainPanel.add(datasourcesLabel, java.awt.BorderLayout.PAGE_START);

        cardpaneEditTrack.add(cardpaneEditTrackMainPanel, java.awt.BorderLayout.CENTER);

        cardpaneEditTrackButtonPanel.setMaximumSize(new java.awt.Dimension(32767, 46));
        cardpaneEditTrackButtonPanel.setMinimumSize(new java.awt.Dimension(0, 46));
        cardpaneEditTrackButtonPanel.setName("cardpaneEditTrackButtonPanel"); // NOI18N
        cardpaneEditTrackButtonPanel.setPreferredSize(new java.awt.Dimension(100, 46));

        editTrackOKButton.setAction(actionMap.get("editTrackOKButtonClickedAction")); // NOI18N
        editTrackOKButton.setText(resourceMap.getString("editTrackOKButton.text")); // NOI18N
        editTrackOKButton.setMaximumSize(new java.awt.Dimension(68, 27));
        editTrackOKButton.setMinimumSize(new java.awt.Dimension(68, 27));
        editTrackOKButton.setName("editTrackOKButton"); // NOI18N
        editTrackOKButton.setPreferredSize(new java.awt.Dimension(68, 27));

        editTrackCancelButton.setText(resourceMap.getString("editTrackCancelButton.text")); // NOI18N
        editTrackCancelButton.setMaximumSize(new java.awt.Dimension(75, 27));
        editTrackCancelButton.setMinimumSize(new java.awt.Dimension(75, 27));
        editTrackCancelButton.setName("editTrackCancelButton"); // NOI18N
        editTrackCancelButton.setPreferredSize(new java.awt.Dimension(75, 27));
        editTrackCancelButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                editTrackCancelButtonPressed(evt);
            }
        });

        editTrackEditButton.setAction(actionMap.get("editDataSource")); // NOI18N
        editTrackEditButton.setText(resourceMap.getString("editTrackEditButton.text")); // NOI18N
        editTrackEditButton.setMaximumSize(new java.awt.Dimension(100, 27));
        editTrackEditButton.setMinimumSize(new java.awt.Dimension(100, 27));
        editTrackEditButton.setName("editTrackEditButton"); // NOI18N
        editTrackEditButton.setOpaque(false);
        editTrackEditButton.setPreferredSize(new java.awt.Dimension(100, 27));

        editTrackAddButton.setAction(actionMap.get("addDataSource")); // NOI18N
        editTrackAddButton.setText(resourceMap.getString("editTrackAddButton.text")); // NOI18N
        editTrackAddButton.setMaximumSize(new java.awt.Dimension(100, 27));
        editTrackAddButton.setMinimumSize(new java.awt.Dimension(100, 27));
        editTrackAddButton.setName("editTrackAddButton"); // NOI18N
        editTrackAddButton.setPreferredSize(new java.awt.Dimension(100, 27));

        editTrackRemoveButton.setAction(actionMap.get("removeDataSource")); // NOI18N
        editTrackRemoveButton.setText(resourceMap.getString("editTrackRemoveButton.text")); // NOI18N
        editTrackRemoveButton.setMaximumSize(new java.awt.Dimension(130, 27));
        editTrackRemoveButton.setMinimumSize(new java.awt.Dimension(130, 27));
        editTrackRemoveButton.setName("editTrackRemoveButton"); // NOI18N
        editTrackRemoveButton.setPreferredSize(new java.awt.Dimension(130, 27));

        editTrackAddSourceFromUCSCButton.setAction(actionMap.get("addDataSourceFromUCSC")); // NOI18N
        editTrackAddSourceFromUCSCButton.setMaximumSize(new java.awt.Dimension(216, 27));
        editTrackAddSourceFromUCSCButton.setMinimumSize(new java.awt.Dimension(216, 27));
        editTrackAddSourceFromUCSCButton.setName("editTrackAddSourceFromUCSCButton"); // NOI18N
        editTrackAddSourceFromUCSCButton.setPreferredSize(new java.awt.Dimension(216, 27));

        javax.swing.GroupLayout cardpaneEditTrackButtonPanelLayout = new javax.swing.GroupLayout(cardpaneEditTrackButtonPanel);
        cardpaneEditTrackButtonPanel.setLayout(cardpaneEditTrackButtonPanelLayout);
        cardpaneEditTrackButtonPanelLayout.setHorizontalGroup(
            cardpaneEditTrackButtonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(cardpaneEditTrackButtonPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(editTrackEditButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(editTrackAddButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(10, 10, 10)
                .addComponent(editTrackAddSourceFromUCSCButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(editTrackRemoveButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(editTrackOKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(editTrackCancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        cardpaneEditTrackButtonPanelLayout.setVerticalGroup(
            cardpaneEditTrackButtonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, cardpaneEditTrackButtonPanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(cardpaneEditTrackButtonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(editTrackEditButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(editTrackAddButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(editTrackRemoveButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(editTrackCancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(editTrackOKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(editTrackAddSourceFromUCSCButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        cardpaneEditTrack.add(cardpaneEditTrackButtonPanel, java.awt.BorderLayout.PAGE_END);

        getContentPane().add(cardpaneEditTrack, "cardpaneEditTrack");

        cardpaneEditSource.setName("cardpaneEditSource"); // NOI18N
        cardpaneEditSource.setLayout(new java.awt.BorderLayout());

        cardpaneEditSourceTopPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(10, 10, 10, 10));
        cardpaneEditSourceTopPanel.setName("cardpaneEditSourceTopPanel"); // NOI18N
        cardpaneEditSourceTopPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 0, 0));

        editSourcePropertiesPanel.setName("editSourcePropertiesPanel"); // NOI18N
        editSourcePropertiesPanel.setLayout(new java.awt.GridBagLayout());

        editSourceTrackNameLabel.setText(resourceMap.getString("editSourceTrackNameLabel.text")); // NOI18N
        editSourceTrackNameLabel.setName("editSourceTrackNameLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editSourcePropertiesPanel.add(editSourceTrackNameLabel, gridBagConstraints);

        editSourceTrackNameTextfield.setColumns(20);
        editSourceTrackNameTextfield.setEditable(false);
        editSourceTrackNameTextfield.setText(resourceMap.getString("editSourceTrackNameTextfield.text")); // NOI18N
        editSourceTrackNameTextfield.setName("editSourceTrackNameTextfield"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editSourcePropertiesPanel.add(editSourceTrackNameTextfield, gridBagConstraints);

        editSourceOrganismLabel.setText(resourceMap.getString("editSourceOrganismLabel.text")); // NOI18N
        editSourceOrganismLabel.setName("editSourceOrganismLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editSourcePropertiesPanel.add(editSourceOrganismLabel, gridBagConstraints);

        editSourceOrganismCombobox.setName("editSourceOrganismCombobox"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editSourcePropertiesPanel.add(editSourceOrganismCombobox, gridBagConstraints);

        editSourceBuildLabel.setText(resourceMap.getString("editSourceBuildLabel.text")); // NOI18N
        editSourceBuildLabel.setName("editSourceBuildLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(11, 24, 11, 0);
        editSourcePropertiesPanel.add(editSourceBuildLabel, gridBagConstraints);

        editSourceBuildTextfield.setColumns(8);
        editSourceBuildTextfield.setText(resourceMap.getString("editSourceBuildTextfield.text")); // NOI18N
        editSourceBuildTextfield.setName("editSourceBuildTextfield"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editSourcePropertiesPanel.add(editSourceBuildTextfield, gridBagConstraints);

        editSourceProtocoLabel.setText(resourceMap.getString("editSourceProtocoLabel.text")); // NOI18N
        editSourceProtocoLabel.setName("editSourceProtocoLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editSourcePropertiesPanel.add(editSourceProtocoLabel, gridBagConstraints);

        editSourceProtocolCombobox.setName("editSourceProtocolCombobox"); // NOI18N
        editSourceProtocolCombobox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                editSourceProtocolChanged(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editSourcePropertiesPanel.add(editSourceProtocolCombobox, gridBagConstraints);

        editSourceDataFormatLabel.setText(resourceMap.getString("editSourceDataFormatLabel.text")); // NOI18N
        editSourceDataFormatLabel.setName("editSourceDataFormatLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 24, 0, 0);
        editSourcePropertiesPanel.add(editSourceDataFormatLabel, gridBagConstraints);

        editSourceDataFormatCombobox.setName("editSourceDataFormatCombobox"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editSourcePropertiesPanel.add(editSourceDataFormatCombobox, gridBagConstraints);

        cardpaneEditSourceTopPanel.add(editSourcePropertiesPanel);

        cardpaneEditSource.add(cardpaneEditSourceTopPanel, java.awt.BorderLayout.PAGE_START);

        cardpaneEditSourceButtonsPanel.setMaximumSize(new java.awt.Dimension(32767, 46));
        cardpaneEditSourceButtonsPanel.setMinimumSize(new java.awt.Dimension(100, 46));
        cardpaneEditSourceButtonsPanel.setName("cardpaneEditSourceButtonsPanel"); // NOI18N
        cardpaneEditSourceButtonsPanel.setPreferredSize(new java.awt.Dimension(100, 46));

        editSourceOKButton.setAction(actionMap.get("editSourceOKButtonClickedAction")); // NOI18N
        editSourceOKButton.setText(resourceMap.getString("editSourceOKButton.text")); // NOI18N
        editSourceOKButton.setMaximumSize(new java.awt.Dimension(68, 27));
        editSourceOKButton.setMinimumSize(new java.awt.Dimension(68, 27));
        editSourceOKButton.setName("editSourceOKButton"); // NOI18N
        editSourceOKButton.setPreferredSize(new java.awt.Dimension(68, 27));

        editSourceCancelButton.setAction(actionMap.get("editSourceCancelButtonClickedAction")); // NOI18N
        editSourceCancelButton.setText(resourceMap.getString("editSourceCancelButton.text")); // NOI18N
        editSourceCancelButton.setMaximumSize(new java.awt.Dimension(75, 27));
        editSourceCancelButton.setMinimumSize(new java.awt.Dimension(75, 27));
        editSourceCancelButton.setName("editSourceCancelButton"); // NOI18N
        editSourceCancelButton.setOpaque(false);
        editSourceCancelButton.setPreferredSize(new java.awt.Dimension(75, 27));

        javax.swing.GroupLayout cardpaneEditSourceButtonsPanelLayout = new javax.swing.GroupLayout(cardpaneEditSourceButtonsPanel);
        cardpaneEditSourceButtonsPanel.setLayout(cardpaneEditSourceButtonsPanelLayout);
        cardpaneEditSourceButtonsPanelLayout.setHorizontalGroup(
            cardpaneEditSourceButtonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, cardpaneEditSourceButtonsPanelLayout.createSequentialGroup()
                .addContainerGap(507, Short.MAX_VALUE)
                .addComponent(editSourceOKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(editSourceCancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        cardpaneEditSourceButtonsPanelLayout.setVerticalGroup(
            cardpaneEditSourceButtonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(cardpaneEditSourceButtonsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(cardpaneEditSourceButtonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(editSourceOKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(editSourceCancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        cardpaneEditSource.add(cardpaneEditSourceButtonsPanel, java.awt.BorderLayout.PAGE_END);

        cardpaneEditSourceMainCard.setName("cardpaneEditSourceMainCard"); // NOI18N
        cardpaneEditSourceMainCard.setLayout(new java.awt.BorderLayout());

        internalWrapper1.setName("internalWrapper1"); // NOI18N
        internalWrapper1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        cardpaneEditSourceMainCardPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 10, 1, 10));
        cardpaneEditSourceMainCardPanel.setName("cardpaneEditSourceMainCardPanel"); // NOI18N
        cardpaneEditSourceMainCardPanel.setLayout(new java.awt.CardLayout());

        editGETsourcePanel.setName("editGETsourcePanel"); // NOI18N
        editGETsourcePanel.setLayout(new java.awt.GridBagLayout());

        editGETsourceBaseURLlabel.setText(resourceMap.getString("editGETsourceBaseURLlabel.text")); // NOI18N
        editGETsourceBaseURLlabel.setName("editGETsourceBaseURLlabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editGETsourcePanel.add(editGETsourceBaseURLlabel, gridBagConstraints);

        editGETsourceBaseURLtextfield.setColumns(60);
        editGETsourceBaseURLtextfield.setText(resourceMap.getString("editGETsourceBaseURLtextfield.text")); // NOI18N
        editGETsourceBaseURLtextfield.setName("editGETsourceBaseURLtextfield"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editGETsourcePanel.add(editGETsourceBaseURLtextfield, gridBagConstraints);

        editGETsourceParametersLabel.setText(resourceMap.getString("editGETsourceParametersLabel.text")); // NOI18N
        editGETsourceParametersLabel.setName("editGETsourceParametersLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editGETsourcePanel.add(editGETsourceParametersLabel, gridBagConstraints);

        editGETsourceParametersTextfield.setColumns(60);
        editGETsourceParametersTextfield.setText(resourceMap.getString("editGETsourceParametersTextfield.text")); // NOI18N
        editGETsourceParametersTextfield.setName("editGETsourceParametersTextfield"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editGETsourcePanel.add(editGETsourceParametersTextfield, gridBagConstraints);

        cardpaneEditSourceMainCardPanel.add(editGETsourcePanel, "GETprotocol");

        editDASsourcePanel.setName("editDASsourcePanel"); // NOI18N
        editDASsourcePanel.setLayout(new java.awt.GridBagLayout());

        editDASsourceBaseURLlabel.setText(resourceMap.getString("editDASsourceBaseURLlabel.text")); // NOI18N
        editDASsourceBaseURLlabel.setName("editDASsourceBaseURLlabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editDASsourcePanel.add(editDASsourceBaseURLlabel, gridBagConstraints);

        editDASsourceBaseURLtextfield.setColumns(60);
        editDASsourceBaseURLtextfield.setText(resourceMap.getString("editDASsourceBaseURLtextfield.text")); // NOI18N
        editDASsourceBaseURLtextfield.setName("editDASsourceBaseURLtextfield"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editDASsourcePanel.add(editDASsourceBaseURLtextfield, gridBagConstraints);

        editDASsourceFeatureLabel.setText(resourceMap.getString("editDASsourceFeatureLabel.text")); // NOI18N
        editDASsourceFeatureLabel.setName("editDASsourceFeatureLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        editDASsourcePanel.add(editDASsourceFeatureLabel, gridBagConstraints);

        editDASsourceFeatureTextfield.setColumns(15);
        editDASsourceFeatureTextfield.setText(resourceMap.getString("editDASsourceFeatureTextfield.text")); // NOI18N
        editDASsourceFeatureTextfield.setName("editDASsourceFeatureTextfield"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        editDASsourcePanel.add(editDASsourceFeatureTextfield, gridBagConstraints);

        cardpaneEditSourceMainCardPanel.add(editDASsourcePanel, "DASprotocol");

        internalWrapper1.add(cardpaneEditSourceMainCardPanel);

        cardpaneEditSourceMainCard.add(internalWrapper1, java.awt.BorderLayout.NORTH);

        DataFormatSettingsTopPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(10, 8, 5, 10));
        DataFormatSettingsTopPanel.setName("DataFormatSettingsTopPanel"); // NOI18N
        DataFormatSettingsTopPanel.setLayout(new java.awt.BorderLayout());

        additionalDataFormatSettingsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("additionalDataFormatSettingsPanel.border.title"))); // NOI18N
        additionalDataFormatSettingsPanel.setName("additionalDataFormatSettingsPanel"); // NOI18N
        additionalDataFormatSettingsPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
        DataFormatSettingsTopPanel.add(additionalDataFormatSettingsPanel, java.awt.BorderLayout.CENTER);

        cardpaneEditSourceMainCard.add(DataFormatSettingsTopPanel, java.awt.BorderLayout.CENTER);

        cardpaneEditSource.add(cardpaneEditSourceMainCard, java.awt.BorderLayout.CENTER);

        getContentPane().add(cardpaneEditSource, "cardpaneEditSource");

        cardpaneServers.setName("cardpaneServers"); // NOI18N
        cardpaneServers.setLayout(new java.awt.BorderLayout());

        cardpaneServersTopPanel.setMaximumSize(new java.awt.Dimension(32767, 40));
        cardpaneServersTopPanel.setName("cardpaneServersTopPanel"); // NOI18N
        cardpaneServersTopPanel.setPreferredSize(new java.awt.Dimension(100, 40));
        cardpaneServersTopPanel.setRequestFocusEnabled(false);

        javax.swing.GroupLayout cardpaneServersTopPanelLayout = new javax.swing.GroupLayout(cardpaneServersTopPanel);
        cardpaneServersTopPanel.setLayout(cardpaneServersTopPanelLayout);
        cardpaneServersTopPanelLayout.setHorizontalGroup(
            cardpaneServersTopPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 670, Short.MAX_VALUE)
        );
        cardpaneServersTopPanelLayout.setVerticalGroup(
            cardpaneServersTopPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 40, Short.MAX_VALUE)
        );

        cardpaneServers.add(cardpaneServersTopPanel, java.awt.BorderLayout.PAGE_START);

        cardpaneServersMainPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 10, 1, 10));
        cardpaneServersMainPanel.setName("cardpaneServersMainPanel"); // NOI18N
        cardpaneServersMainPanel.setLayout(new java.awt.BorderLayout());

        serversLabel.setFont(resourceMap.getFont("serversLabel.font")); // NOI18N
        serversLabel.setText(resourceMap.getString("serversLabel.text")); // NOI18N
        serversLabel.setName("serversLabel"); // NOI18N
        cardpaneServersMainPanel.add(serversLabel, java.awt.BorderLayout.PAGE_START);

        serversScrollPane.setName("serversScrollPane"); // NOI18N
        cardpaneServersMainPanel.add(serversScrollPane, java.awt.BorderLayout.CENTER);

        cardpaneServers.add(cardpaneServersMainPanel, java.awt.BorderLayout.CENTER);

        cardpaneServersButtonPanel.setName("cardpaneServersButtonPanel"); // NOI18N

        serversOKButton.setAction(actionMap.get("serversOKButtonClickedAction")); // NOI18N
        serversOKButton.setText(resourceMap.getString("serversOKButton.text")); // NOI18N
        serversOKButton.setMaximumSize(new java.awt.Dimension(68, 27));
        serversOKButton.setMinimumSize(new java.awt.Dimension(68, 27));
        serversOKButton.setName("serversOKButton"); // NOI18N
        serversOKButton.setOpaque(false);
        serversOKButton.setPreferredSize(new java.awt.Dimension(68, 27));

        javax.swing.GroupLayout cardpaneServersButtonPanelLayout = new javax.swing.GroupLayout(cardpaneServersButtonPanel);
        cardpaneServersButtonPanel.setLayout(cardpaneServersButtonPanelLayout);
        cardpaneServersButtonPanelLayout.setHorizontalGroup(
            cardpaneServersButtonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, cardpaneServersButtonPanelLayout.createSequentialGroup()
                .addContainerGap(592, Short.MAX_VALUE)
                .addComponent(serversOKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        cardpaneServersButtonPanelLayout.setVerticalGroup(
            cardpaneServersButtonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, cardpaneServersButtonPanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(serversOKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        cardpaneServers.add(cardpaneServersButtonPanel, java.awt.BorderLayout.PAGE_END);

        getContentPane().add(cardpaneServers, "cardpaneServers");

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void exitWithoutSaving(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_exitWithoutSaving
     setVisible(false);
}//GEN-LAST:event_exitWithoutSaving

private void configureDatatracksButtonClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_configureDatatracksButtonClicked
   showAllTracksPanel();
}//GEN-LAST:event_configureDatatracksButtonClicked

private void configureServersSettingsButtonClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_configureServersSettingsButtonClicked
    showServersPanel();
}//GEN-LAST:event_configureServersSettingsButtonClicked

private void importSettings(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_importSettings
        File file=null;//GEN-LAST:event_importSettings
        gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        final JFileChooser fc = new JFileChooser(gui.getLastUsedDirectory());
        fc.setDialogTitle("Import Settings"); 
        int returnValue=fc.showOpenDialog(this);
        if (returnValue==JFileChooser.APPROVE_OPTION) {
            file=fc.getSelectedFile();            
        } else {
            gui.getFrame().setCursor(Cursor.getDefaultCursor());
            return;
        }
        final File configurationfile=file;
        gui.setLastUsedDirectory(file.getParentFile());
        SwingWorker worker=new SwingWorker<Boolean, Void>() {
            Exception ex=null;
            @Override 
            public Boolean doInBackground() {
                configureDatatracksButton.setEnabled(false);
                configureServerSettingsButton.setEnabled(false);
                importSettingsButton.setEnabled(false);
                exportSettingsButton.setEnabled(false);
                revertSettingsButton.setEnabled(false);
                exitWithoutSaveButton.setEnabled(false);
                saveChangesButton.setEnabled(false);
                try {
                    dataconfiguration.loadConfigurationFromFile(configurationfile);
                } catch (Exception e) {ex=e;}
                return Boolean.TRUE;
            }
            @Override
            public void done() { // this method is invoked on the EDT!
                gui.getFrame().setCursor(Cursor.getDefaultCursor());
                configureDatatracksButton.setEnabled(true);
                configureServerSettingsButton.setEnabled(true);
                importSettingsButton.setEnabled(true);
                exportSettingsButton.setEnabled(true);
                revertSettingsButton.setEnabled(true);
                exitWithoutSaveButton.setEnabled(true);
                saveChangesButton.setEnabled(true);
                if (ex==null) {
                  availableTracks=dataconfiguration.getAvailableTracks();
                  availableServers=dataconfiguration.getServers();
                  //setupTablesFromConfiguration();
                } else {
                    JOptionPane.showMessageDialog(gui.getFrame(), ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        }; // end of SwingWorker class
        worker.execute();
}

private void exportSettings(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_exportSettings
        File file=null;//GEN-LAST:event_exportSettings
        gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        final JFileChooser fc = new JFileChooser(gui.getLastUsedDirectory());
        fc.setDialogTitle("Export Settings"); 
        fc.setSelectedFile(new File("DataTracks.xml"));
        int returnValue=fc.showSaveDialog(this);
        if (returnValue==JFileChooser.APPROVE_OPTION) {
            file=fc.getSelectedFile();            
        } else return;                   
        if (file.exists()) {
            int choice=JOptionPane.showConfirmDialog(this, "Overwrite existing file \""+file.getName()+"\" ?","Export Settings",JOptionPane.OK_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
            if (choice!=JOptionPane.OK_OPTION) return;
        }
        saveSettingsToFile(file);   
        gui.setLastUsedDirectory(file.getParentFile());
        gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        
}

private void revertSettings(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_revertSettings
     int choice=JOptionPane.showConfirmDialog(this, "Do you really want to revert settings?\nThis will delete any alterations you have made to the configuration", "Revert Settings", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
     if (choice==JOptionPane.CANCEL_OPTION) return;//GEN-LAST:event_revertSettings
     SwingWorker worker=new SwingWorker<Boolean, Void>() {
            Exception ex=null;
            @Override 
            public Boolean doInBackground() {
                configureDatatracksButton.setEnabled(false);
                configureServerSettingsButton.setEnabled(false);
                importSettingsButton.setEnabled(false);
                exportSettingsButton.setEnabled(false);
                revertSettingsButton.setEnabled(false);
                exitWithoutSaveButton.setEnabled(false);
                saveChangesButton.setEnabled(false);
                try {
                    dataconfiguration.loadDefaultConfiguration();
                } catch (Exception e) {ex=e;}
                return Boolean.TRUE;
            }
            @Override
            public void done() { // this method is invoked on the EDT!
                configureDatatracksButton.setEnabled(true);
                configureServerSettingsButton.setEnabled(true);
                importSettingsButton.setEnabled(true);
                exportSettingsButton.setEnabled(true);
                revertSettingsButton.setEnabled(true);
                exitWithoutSaveButton.setEnabled(true);
                saveChangesButton.setEnabled(true);
                if (ex==null) {
                  availableTracks=dataconfiguration.getAvailableTracks();
                  availableServers=dataconfiguration.getServers();
                  setupTablesFromConfiguration();
                } else {
                    JOptionPane.showMessageDialog(gui.getFrame(), ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        }; // end of SwingWorker class
        worker.execute();      
}

private void editTrackCancelButtonPressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_editTrackCancelButtonPressed
       showAllTracksPanel();
}//GEN-LAST:event_editTrackCancelButtonPressed

private void editSourceProtocolChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_editSourceProtocolChanged
      String protocol=(String)editSourceProtocolCombobox.getSelectedItem();  
           if (protocol.equals("DAS")) ((CardLayout)cardpaneEditSourceMainCardPanel.getLayout()).show(cardpaneEditSourceMainCardPanel, "DASprotocol");
      else if (protocol.equals("GET")) ((CardLayout)cardpaneEditSourceMainCardPanel.getLayout()).show(cardpaneEditSourceMainCardPanel, "GETprotocol");
}//GEN-LAST:event_editSourceProtocolChanged


private void saveSettingsToFile(final File configurationfile) {
       SwingWorker worker=new SwingWorker<Boolean, Void>() {
            Exception ex=null;
            @Override 
            public Boolean doInBackground() {
                try {
                    dataconfiguration.saveConfigurationToFile(configurationfile);
                } catch (Exception e) {
                    ex=e;
                }
                return Boolean.TRUE;
            }
            @Override
            public void done() { // this method is invoked on the EDT!
                if (ex!=null) {
                     JOptionPane.showMessageDialog(gui.getFrame(), ex.getClass().getSimpleName()+"\n"+ex.getMessage(),"File error" ,JOptionPane.ERROR_MESSAGE);
                     //ex.printStackTrace();
                     return;
                }
            }
        }; // end of SwingWorker class
        worker.execute();       
}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel DataFormatSettingsTopPanel;
    private javax.swing.JPanel additionalDataFormatSettingsPanel;
    private javax.swing.JButton allTracksAddButton;
    private javax.swing.JButton allTracksAddFromUCSCButton;
    private javax.swing.JButton allTracksEditButton;
    private javax.swing.JButton allTracksOKButton;
    private javax.swing.JButton allTracksRemoveButton;
    private javax.swing.JPanel cardpaneAllTracks;
    private javax.swing.JPanel cardpaneAllTracksButtonPanel;
    private javax.swing.JPanel cardpaneAllTracksMainPanel;
    private javax.swing.JScrollPane cardpaneAllTracksScrollPane;
    private javax.swing.JPanel cardpaneAllTracksTopPanel;
    private javax.swing.JPanel cardpaneEditSource;
    private javax.swing.JPanel cardpaneEditSourceButtonsPanel;
    private javax.swing.JPanel cardpaneEditSourceMainCard;
    private javax.swing.JPanel cardpaneEditSourceMainCardPanel;
    private javax.swing.JPanel cardpaneEditSourceTopPanel;
    private javax.swing.JPanel cardpaneEditTrack;
    private javax.swing.JPanel cardpaneEditTrackButtonPanel;
    private javax.swing.JPanel cardpaneEditTrackMainPanel;
    private javax.swing.JPanel cardpaneEditTrackTopPanel;
    private javax.swing.JScrollPane cardpaneEditTracksScrollPane;
    private javax.swing.JPanel cardpaneServers;
    private javax.swing.JPanel cardpaneServersButtonPanel;
    private javax.swing.JPanel cardpaneServersMainPanel;
    private javax.swing.JPanel cardpaneServersTopPanel;
    private javax.swing.JPanel cardpaneTopMenu;
    private javax.swing.JPanel cardpaneTopMenuButtonPanel;
    private javax.swing.JPanel cardpaneTopMenuMainPanel;
    private javax.swing.JPanel cardpaneTopMenuTopPanel;
    private javax.swing.JButton configureDatatracksButton;
    private javax.swing.JButton configureServerSettingsButton;
    private javax.swing.JLabel dataTrackPropertiesDescriptionLabel;
    private javax.swing.JTextField dataTrackPropertiesDescriptionTextfield;
    private javax.swing.JLabel dataTrackPropertiesNameLabel;
    private javax.swing.JTextField dataTrackPropertiesNameTextfield;
    private javax.swing.JLabel dataTrackPropertiesSourceLabel;
    private javax.swing.JTextField dataTrackPropertiesSourceTextfield;
    private javax.swing.JComboBox dataTrackPropertiesTypeCombobox;
    private javax.swing.JLabel dataTrackPropertiesTypeLabel;
    private javax.swing.JLabel datasourcesLabel;
    private javax.swing.JLabel datatracksLabel;
    private javax.swing.JLabel editDASsourceBaseURLlabel;
    private javax.swing.JTextField editDASsourceBaseURLtextfield;
    private javax.swing.JLabel editDASsourceFeatureLabel;
    private javax.swing.JTextField editDASsourceFeatureTextfield;
    private javax.swing.JPanel editDASsourcePanel;
    private javax.swing.JLabel editGETsourceBaseURLlabel;
    private javax.swing.JTextField editGETsourceBaseURLtextfield;
    private javax.swing.JPanel editGETsourcePanel;
    private javax.swing.JLabel editGETsourceParametersLabel;
    private javax.swing.JTextField editGETsourceParametersTextfield;
    private javax.swing.JLabel editSourceBuildLabel;
    private javax.swing.JTextField editSourceBuildTextfield;
    private javax.swing.JButton editSourceCancelButton;
    private javax.swing.JComboBox editSourceDataFormatCombobox;
    private javax.swing.JLabel editSourceDataFormatLabel;
    private javax.swing.JButton editSourceOKButton;
    private javax.swing.JComboBox editSourceOrganismCombobox;
    private javax.swing.JLabel editSourceOrganismLabel;
    private javax.swing.JPanel editSourcePropertiesPanel;
    private javax.swing.JLabel editSourceProtocoLabel;
    private javax.swing.JComboBox editSourceProtocolCombobox;
    private javax.swing.JLabel editSourceTrackNameLabel;
    private javax.swing.JTextField editSourceTrackNameTextfield;
    private javax.swing.JButton editTrackAddButton;
    private javax.swing.JButton editTrackAddSourceFromUCSCButton;
    private javax.swing.JButton editTrackCancelButton;
    private javax.swing.JButton editTrackEditButton;
    private javax.swing.JButton editTrackOKButton;
    private javax.swing.JPanel editTrackPropertiesPanel;
    private javax.swing.JButton editTrackRemoveButton;
    private javax.swing.JButton exitWithoutSaveButton;
    private javax.swing.JButton exportSettingsButton;
    private javax.swing.JButton importSettingsButton;
    private javax.swing.JPanel internalWrapper1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JButton revertSettingsButton;
    private javax.swing.JButton saveChangesButton;
    private javax.swing.JLabel serversLabel;
    private javax.swing.JButton serversOKButton;
    private javax.swing.JScrollPane serversScrollPane;
    // End of variables declaration//GEN-END:variables


private void showTopMenuPanel() {
         currentDataTrack=null;
        ((CardLayout)getContentPane().getLayout()).show(getContentPane(), "cardpaneTopMenu");
         getRootPane().setDefaultButton(saveChangesButton);       
}  
private void showServersPanel() {
        ((CardLayout)getContentPane().getLayout()).show(getContentPane(), "cardpaneServers");
         getRootPane().setDefaultButton(serversOKButton);       
}  

private void showAllTracksPanel() {
        setupTablesFromConfiguration();
         currentDataTrack=null;
        ((CardLayout)getContentPane().getLayout()).show(getContentPane(), "cardpaneAllTracks");
         getRootPane().setDefaultButton(allTracksOKButton);       
}    
    
private void showEditDataTrackPanel(DataTrack datatrack) {
         if (datatrack==null) addDataTrack=true; else addDataTrack=false;
         currentDataTrack=datatrack;
         String datatrackName=(addDataTrack)?"":datatrack.getName();
         String description=(addDataTrack)?"":datatrack.getDescription();
         String sourceSite=(addDataTrack)?"":datatrack.getSourceSite();
         Class datatype=(addDataTrack)?null:datatrack.getDataType();
         dataTrackPropertiesNameTextfield.setText(datatrackName);
         dataTrackPropertiesDescriptionTextfield.setText(description);
         dataTrackPropertiesDescriptionTextfield.setCaretPosition(0);
         dataTrackPropertiesSourceTextfield.setText(sourceSite);
              if (datatype==null) dataTrackPropertiesTypeCombobox.setSelectedItem("Numeric");
         else if (datatype==DNASequenceDataset.class) dataTrackPropertiesTypeCombobox.setSelectedItem("Sequence");
         else if (datatype==NumericDataset.class) dataTrackPropertiesTypeCombobox.setSelectedItem("Numeric");
         else if (datatype==RegionDataset.class) dataTrackPropertiesTypeCombobox.setSelectedItem("Region");
         ArrayList<DataSource> sources=(addDataTrack)?new ArrayList<DataSource>():datatrack.getDatasources();
         String[] columnNames=new String[]{TABLECOLUMN_ORGANISM,TABLECOLUMN_BUILD,TABLECOLUMN_PROTOCOL,TABLECOLUMN_DATAFORMAT,TABLECOLUMN_SERVER};
         Object[][] data=new Object[sources.size()][columnNames.length];
         for (int i=0;i<sources.size();i++) {
             DataSource source=sources.get(i);
             data[i]=new Object[]{new Integer(source.getOrganism()),source.getGenomeBuild(),source.getProtocol(),source.getDataFormat(),source.getServerAddress()};
         }
         datasourceTableModel=new DefaultTableModel(data,columnNames);
         datasourceTable.setModel(datasourceTableModel);
         datasourceTable.getColumn(TABLECOLUMN_ORGANISM).setCellRenderer(organismRenderer);
         ((CardLayout)getContentPane().getLayout()).show(getContentPane(), "cardpaneEditTrack");        
         getRootPane().setDefaultButton(editTrackOKButton);
}    
    
private void showEditDataSourcePanel(DataSource datasource) {
         if (datasource==null) addDataSource=true; else addDataSource=false; 
         currentDataSource=datasource;         
         String datatrackName=currentDataTrack.getName();
         String build=(addDataSource)?"":datasource.getGenomeBuild();
         if (build==null) build="";
         int organism=(addDataSource)?Organism.HUMAN:datasource.getOrganism();
         String protocol=(addDataSource)?DataSource.DAS_SERVER:datasource.getProtocol();
         String dataformat=(addDataSource)?null:datasource.getDataFormat();
         if (dataformat!=null) {
            showDataFormatSettingsPanel(getDataFormatSettingsPanel(dataformat)); 
         }
         editSourceTrackNameTextfield.setText(datatrackName);
         editSourceBuildTextfield.setText(build);
         editSourceOrganismCombobox.setSelectedItem(organism);
         if (addDataSource) editSourceDataFormatCombobox.setSelectedIndex(-1);else editSourceDataFormatCombobox.setSelectedItem(dataformat);
         if (datasource==null) {
             editDASsourceBaseURLtextfield.setText("");
             editDASsourceFeatureTextfield.setText("");
             editGETsourceBaseURLtextfield.setText("");
             editGETsourceParametersTextfield.setText("");
             ((CardLayout)cardpaneEditSourceMainCardPanel.getLayout()).show(cardpaneEditSourceMainCardPanel, "DASprotocol");
         } else if (datasource instanceof DataSource_DAS) {
             editDASsourceBaseURLtextfield.setText(((DataSource_DAS)datasource).getBaseURL());
             editDASsourceFeatureTextfield.setText(((DataSource_DAS)datasource).getFeature());     
             editGETsourceBaseURLtextfield.setText("");
             editGETsourceParametersTextfield.setText("");
             ((CardLayout)cardpaneEditSourceMainCardPanel.getLayout()).show(cardpaneEditSourceMainCardPanel, "DASprotocol");
         } else if (datasource instanceof DataSource_http_GET) {
             editGETsourceBaseURLtextfield.setText(((DataSource_http_GET)datasource).getBaseURL());
             editGETsourceParametersTextfield.setText(((DataSource_http_GET)datasource).getParameter());
             editGETsourceParametersTextfield.setCaretPosition(0);
             editDASsourceBaseURLtextfield.setText("");
             editDASsourceFeatureTextfield.setText("");
             ((CardLayout)cardpaneEditSourceMainCardPanel.getLayout()).show(cardpaneEditSourceMainCardPanel, "GETprotocol");
         }
         if (addDataSource) editSourceProtocolCombobox.setSelectedItem("DAS"); else editSourceProtocolCombobox.setSelectedItem(protocol);
         ((CardLayout)getContentPane().getLayout()).show(getContentPane(), "cardpaneEditSource"); 
         getRootPane().setDefaultButton(editSourceOKButton);
}    
    
      

    @Action
    public void allTracksEditButtonClicked() {
       int row=datatrackTable.getSelectedRow();
       if (row<0) return;
       row=datatrackTable.convertRowIndexToModel(row);
       String trackname=(String)datatrackTable.getModel().getValueAt(row, 0);
       DataTrack track=availableTracks.get(trackname);
       showEditDataTrackPanel(track.clone());
    }

    @Action
    public void allTracksRemoveButtonClicked() {
       int row=datatrackTable.getSelectedRow();
       if (row<0) return;
       row=datatrackTable.convertRowIndexToModel(row);
       String trackname=(String)datatrackTable.getModel().getValueAt(row, 0);
       availableTracks.remove(trackname);
       datatrackTableModel.removeRow(row);
    }

    @Action
    public void editDataSource() {
       int row=datasourceTable.getSelectedRow();
       if (row<0) return;
       row=datasourceTable.convertRowIndexToModel(row);
       DataSource source=currentDataTrack.getDatasources().get(row);
       showEditDataSourcePanel(source);
    }
    
    @Action
    public void addDataSource() {
        if (currentDataTrack==null) {
            String editname=dataTrackPropertiesNameTextfield.getText().trim();
            if (editname.isEmpty()) {
                JOptionPane.showMessageDialog(gui.getFrame(), "You must supply a name for the Datatrack","Missing track name error", JOptionPane.ERROR_MESSAGE);
                return;
            }
            String typename=(String)dataTrackPropertiesTypeCombobox.getSelectedItem();
            Class newtype=null;
                 if (typename.equals("Sequence")) newtype=DNASequenceDataset.class;
            else if (typename.equals("Numeric")) newtype=NumericDataset.class;
            else if (typename.equals("Region")) newtype=RegionDataset.class;
            String sourcesite=dataTrackPropertiesSourceTextfield.getText().trim();
            String description=dataTrackPropertiesDescriptionTextfield.getText().trim(); 
            currentDataTrack=new DataTrack(editname, newtype, sourcesite, description);            
            //System.err.println("Creating temp datatrack");
        }
        showEditDataSourcePanel(null);
    }
    
    @Action
    public void addDataSourceFromUCSC() {
        if (currentDataTrack==null) {
            String editname=dataTrackPropertiesNameTextfield.getText().trim();
            if (editname.isEmpty()) {
                JOptionPane.showMessageDialog(gui.getFrame(), "You must supply a name for the Datatrack","Missing track name error", JOptionPane.ERROR_MESSAGE);
                return;
            }
            String typename=(String)dataTrackPropertiesTypeCombobox.getSelectedItem();
            Class newtype=null;
                 if (typename.equals("Sequence")) newtype=DNASequenceDataset.class;
            else if (typename.equals("Numeric")) newtype=NumericDataset.class;
            else if (typename.equals("Region")) newtype=RegionDataset.class;
            String sourcesite=dataTrackPropertiesSourceTextfield.getText().trim();
            String description=dataTrackPropertiesDescriptionTextfield.getText().trim(); 
            currentDataTrack=new DataTrack(editname, newtype, sourcesite, description);            
            //System.err.println("Creating temp datatrack");
        }
        UCSCtrackDialog ucscdialog = new UCSCtrackDialog(gui.getFrame(),currentDataTrack);
        ucscdialog.setLocation(gui.getFrame().getWidth()/2-ucscdialog.getWidth()/2, gui.getFrame().getHeight()/2-ucscdialog.getHeight()/2);
        ucscdialog.setVisible(true);
        if (ucscdialog.isOKPressed()) {
            DataSource newsource=ucscdialog.getDataSource(currentDataTrack);
            ucscdialog.dispose();
            if (newsource!=null) {
               currentDataTrack.addDataSource(newsource);
               showEditDataTrackPanel(currentDataTrack);
            }            
        } else {
           ucscdialog.dispose(); 
        }                        
    }    
        
    @Action
    public void removeDataSource() {
       int row=datasourceTable.getSelectedRow();
       if (row<0) return;
       row=datasourceTable.convertRowIndexToModel(row);
       DataSource source=currentDataTrack.getDatasources().remove(row);
       datasourceTableModel.removeRow(row);
       for (int i=0;i<datatrackTableModel.getRowCount();i++) {
           if (datatrackTableModel.getValueAt(i, 0).equals(currentDataTrack.getName())) {
               int[] supported=currentDataTrack.getSupportedOrganisms();
               datatrackTableModel.setValueAt(supported, i, 3);break;
           }
       }
    }

    @Action
    public void allTrackAddButtonClicked() {
         showEditDataTrackPanel(null);
    }

    @Action
    public void allTracksOKButtonClickedAction() {
        showTopMenuPanel();
    }

    @Action
    public void editTrackOKButtonClickedAction() {
         // store new values in track!!
        if (addDataTrack) { // adding
            String editname=dataTrackPropertiesNameTextfield.getText().trim();
            if (editname.isEmpty()) {
                JOptionPane.showMessageDialog(gui.getFrame(), "You must supply a name for the Datatrack","Missing track name error", JOptionPane.ERROR_MESSAGE);
                return;
            }
            String typename=(String)dataTrackPropertiesTypeCombobox.getSelectedItem();
            Class newtype=null;
                 if (typename.equals("Sequence")) newtype=DNASequenceDataset.class;
            else if (typename.equals("Numeric")) newtype=NumericDataset.class;
            else if (typename.equals("Region")) newtype=RegionDataset.class;
            String sourcesite=dataTrackPropertiesSourceTextfield.getText().trim();
            String description=dataTrackPropertiesDescriptionTextfield.getText().trim();
            if (availableTracks.containsKey(editname)) {
                int choice=JOptionPane.showConfirmDialog(gui.getFrame(), "There is already a track named '"+editname+"'\nWould you like to replace it?","Replace existing track", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
                if (choice==JOptionPane.CANCEL_OPTION) return;
            }  
            currentDataTrack=new DataTrack(editname, newtype, sourcesite, description);
            availableTracks.put(currentDataTrack.getName(), currentDataTrack); 
        } else { // editing
            String currenttrackname=currentDataTrack.getName();
            String editname=dataTrackPropertiesNameTextfield.getText().trim();
            String typename=(String)dataTrackPropertiesTypeCombobox.getSelectedItem();
            Class newtype=null;
                 if (typename.equals("Sequence")) newtype=DNASequenceDataset.class;
            else if (typename.equals("Numeric")) newtype=NumericDataset.class;
            else if (typename.equals("Region")) newtype=RegionDataset.class;
            if (!editname.equals(currenttrackname) && availableTracks.containsKey(editname)) {
                int choice=JOptionPane.showConfirmDialog(gui.getFrame(), "There is already a track named '"+editname+"'\nWould you like to replace it?","Replace existing track", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
                if (choice==JOptionPane.CANCEL_OPTION) return;
            } 
            currentDataTrack.setName(editname);
            currentDataTrack.setDataType(newtype);
            currentDataTrack.setSourceSite(dataTrackPropertiesSourceTextfield.getText().trim());
            currentDataTrack.setDescription(dataTrackPropertiesDescriptionTextfield.getText().trim());  
            if (!editname.equals(currenttrackname)) {
                availableTracks.remove(currenttrackname);
            }  
            availableTracks.put(editname, currentDataTrack);
        }
        //setupTablesFromConfiguration();
        showAllTracksPanel();
    }


    @Action
    public void saveChangesAndExitAction() {
        File configurationfile=gui.getEngine().getDataLoader().getDataConfigurationFile();
        saveSettingsToFile(configurationfile);  
        gui.getEngine().getDataLoader().installConfiguration(dataconfiguration);
        setVisible(false);
    }

    @Action
    public void serversOKButtonClickedAction() {
        availableServers.clear();
        int rows=serverTableModel.getRowCount();
        for (int i=0;i<rows;i++) {
            Object addressobject=serverTableModel.getValueAt(i, 0);
            if (addressobject==null) continue;            
            String address=addressobject.toString();
            Object delayObject=serverTableModel.getValueAt(i, 1);
            String delayasstring=(delayObject!=null)?delayObject.toString():"";
            Object mirrorbject=serverTableModel.getValueAt(i, 2);  
            String mirrorstring=(mirrorbject==null)?"":mirrorbject.toString().trim();

            address=address.trim();
            if (address.isEmpty()) continue;
            if (availableServers.containsKey(address)) {
                int viewrow=serverTable.convertRowIndexToView(i);
                JOptionPane.showMessageDialog(cardpaneTopMenu, "The address '"+delayasstring+"' at row="+(viewrow+1)+",col=1 has been used before", "Duplicate server error", JOptionPane.ERROR_MESSAGE);
                return;                
            }
            mirrorstring=mirrorstring.trim();
            int delay=0;
            if (!delayasstring.isEmpty()) {
              try {delay=Integer.parseInt(delayasstring);} 
              catch (NumberFormatException e) {
                int viewrow=serverTable.convertRowIndexToView(i);
                JOptionPane.showMessageDialog(cardpaneTopMenu, "The value '"+delayasstring+"' at row="+(viewrow+1)+",col=2 is not an integer number", "Number format error", JOptionPane.ERROR_MESSAGE);
                return;
              }
            }
            Server newserver=new Server(address,delay);
            if (!mirrorstring.isEmpty()) {
                String[] mirrors=mirrorstring.split(",");
                for (String mirroraddress:mirrors) newserver.addMirror(mirroraddress);
            }    
            availableServers.put(address, newserver);
        }
        serverTable.clearSelection();
        setupTablesFromConfiguration();
        showTopMenuPanel();
    }

    @Action
    public void editSourceOKButtonClickedAction() {
        int organism=Organism.HUMAN;
        Object organismObject=editSourceOrganismCombobox.getSelectedItem();
        if (organismObject!=null) organism=((Integer)organismObject).intValue();
        String build=editSourceBuildTextfield.getText().trim();
        String dataformat=(String)editSourceDataFormatCombobox.getSelectedItem();
        ParameterSettings dataformatSettings=null;
        if (dataFormatParametersPanel!=null) {
            dataFormatParametersPanel.setParameters();
            dataformatSettings=dataFormatParametersPanel.getParameterSettings();
        }           
        String protocol=(String)editSourceProtocolCombobox.getSelectedItem();  
        if (addDataSource) { // adding          
            if (protocol.equals(DataSource.DAS_SERVER)) {
                String baseURL=editDASsourceBaseURLtextfield.getText().trim();
                String feature=editDASsourceFeatureTextfield.getText().trim();
                currentDataSource=new DataSource_DAS(currentDataTrack,organism,build,baseURL,dataformat,feature);
            }
            else if (protocol.equals(DataSource.HTTP_GET)) {
                String baseURL=editGETsourceBaseURLtextfield.getText().trim();
                String parameters=editGETsourceParametersTextfield.getText().trim();
                currentDataSource=new DataSource_http_GET(currentDataTrack,organism,build,baseURL,dataformat,parameters);
                currentDataSource.setDataFormatSettings(dataformatSettings);
            }
            if (currentDataSource!=null) currentDataTrack.addDataSource(currentDataSource);            
        } else { // editing
            String currentprotocol=currentDataSource.getProtocol();            
            if (protocol.equals(currentprotocol)) { // just replace the current values
                 currentDataSource.setDataFormat(dataformat);
                 currentDataSource.setGenomeBuild(build);
                 currentDataSource.setOrganism(organism);
                 currentDataSource.setDataFormatSettings(dataformatSettings);
                 if (currentDataSource instanceof DataSource_DAS) {
                    String baseURL=editDASsourceBaseURLtextfield.getText().trim();
                    String feature=editDASsourceFeatureTextfield.getText().trim();
                    ((DataSource_DAS)currentDataSource).setBaseURL(baseURL);
                    ((DataSource_DAS)currentDataSource).setFeature(feature);
                 } else if (currentDataSource instanceof DataSource_http_GET) {
                    String baseURL=editGETsourceBaseURLtextfield.getText().trim();
                    String parameter=editGETsourceParametersTextfield.getText().trim();
                    ((DataSource_http_GET)currentDataSource).setBaseURL(baseURL);
                    ((DataSource_http_GET)currentDataSource).setParameter(parameter);
                 }
            } else { // protocol has been changed. We need to replace the object 
                 DataSource newsource=null;
                 if (protocol.equals(DataSource.DAS_SERVER)) {
                     String baseURL=editDASsourceBaseURLtextfield.getText().trim();
                     String feature=editDASsourceFeatureTextfield.getText().trim();
                     newsource=new DataSource_DAS(currentDataTrack,organism,build,baseURL,dataformat,feature);
                 }
                 else if (protocol.equals(DataSource.HTTP_GET)) {
                     String baseURL=editGETsourceBaseURLtextfield.getText().trim();
                     String parameter=editGETsourceParametersTextfield.getText().trim();
                     newsource=new DataSource_http_GET(currentDataTrack,organism,build,baseURL,dataformat,parameter);
                 }     
                 newsource.setDataFormatSettings(dataformatSettings);
                 currentDataTrack.replaceDataSource(currentDataSource, newsource);
                 currentDataSource=newsource;   
                 
            }
        }
        showEditDataTrackPanel(currentDataTrack);
    }

    @Action
    public void editSourceCancelButtonClickedAction() {
        showEditDataTrackPanel(currentDataTrack);
    }
    
// ---------------------   PRIVATE CLASSES BELOW THIS LINE   ---------------------------
    
    
private class DataTypeRenderer extends DefaultTableCellRenderer {
    private SimpleDataPanelIcon numericIcon;
    private SimpleDataPanelIcon regionIcon;
    private SimpleDataPanelIcon DNAIcon;
    public DataTypeRenderer() {
           super();
           this.setHorizontalAlignment(DefaultTableCellRenderer.CENTER);   
           numericIcon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.NUMERIC_TRACK_GRAPH_ICON,null);
           numericIcon.setForegroundColor(java.awt.Color.BLUE);
           numericIcon.setBackgroundColor(null);
           regionIcon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.REGION_ICON,null);
           regionIcon.setForegroundColor(java.awt.Color.GREEN);
           regionIcon.setBackgroundColor(null);
           DNAIcon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.SEQUENCE_ICON_BASES,null);
           DNAIcon.setForegroundColor(java.awt.Color.BLACK);
           
           setOpaque(true);
    }
    
   @Override
   public void setValue(Object value) {
       if (value instanceof String) {
           String val=(String)value;
                if (val.equalsIgnoreCase(NumericDataset.getType())) {setIcon(numericIcon);setToolTipText("Numeric");}
           else if (val.equalsIgnoreCase(RegionDataset.getType())) {setIcon(regionIcon);setToolTipText("Region");}
           else if (val.equalsIgnoreCase(DNASequenceDataset.getType())) {setIcon(DNAIcon);setToolTipText("Sequence");}
       } else if (value instanceof Class) {
           Class cl=(Class)value;
                if (cl==NumericDataset.class) {setIcon(numericIcon);setToolTipText("Numeric");}
           else if (cl==RegionDataset.class) {setIcon(regionIcon);setToolTipText("Region");}
           else if (cl==DNASequenceDataset.class) {setIcon(DNAIcon);setToolTipText("Sequence");}
       }

   }
    } // end class DataTypeRenderer

private class SupportedOrganismsRendered extends DefaultTableCellRenderer {    
       public SupportedOrganismsRendered() {
           super();
       }
       @Override
       public void setValue(Object value) {
           if (value instanceof Integer) {
               setText(Organism.getCommonName(((Integer)value).intValue()));
           } else if (value instanceof int[]) {
               int[] organisms=(int[]) value;
               String text="";
               for (int i=0;i<organisms.length;i++) {
                   text+=Organism.getCommonName(organisms[i]);
                   if (i<organisms.length-1) text+=",";
               }
               setText(text); 
           } else setText("<error>");
       }
}

private class OrganismComboboxRenderer extends JLabel implements ListCellRenderer {
        public OrganismComboboxRenderer () {
            super();
            setBorder(BorderFactory.createEmptyBorder(2, 4, 2, 4));
        }
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(java.awt.Color.WHITE);
                setForeground(list.getForeground());                
            }
            if (value==null) setText("***"); 
            if (value instanceof String) setText((String)value); 
            else {
               int organism=((Integer)value).intValue();
               String name=Organism.getCommonName(organism);
               setText(name);
            }
            return this;
        }
    
}

    @Action
    public void allTracksAddFromUCSCButtonClicked() {
        UCSCtrackDialog ucscdialog = new UCSCtrackDialog(gui.getFrame(),null);
        ucscdialog.setLocation(gui.getFrame().getWidth()/2-ucscdialog.getWidth()/2, gui.getFrame().getHeight()/2-ucscdialog.getHeight()/2);
        ucscdialog.setVisible(true);
        if (ucscdialog.isOKPressed()) {
            DataTrack newtrack=ucscdialog.getDataTrack();
            ucscdialog.dispose();
            if (newtrack!=null) {
                String newtrackname=newtrack.getName();
                if (availableTracks.containsKey(newtrackname)) {
                    int choice=JOptionPane.showConfirmDialog(this, "A data track named \""+newtrackname+"\" already exists.\nWould you like to replace this track?","Replace data track",JOptionPane.OK_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
                    if (choice!=JOptionPane.OK_OPTION) return;                       
                }
                availableTracks.put(newtrack.getName(), newtrack);
                showAllTracksPanel(); // this will setup the table from the configuration
            }            
        } else {
           ucscdialog.dispose(); 
        }     
    }



}
