/*
 * DataTrackDialog.java
 *
 * Created on 6. april 2009, 13:32
 */

package priorseditor.gui;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;
import javax.swing.RowFilter.Entry;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableRowSorter;
import priorseditor.engine.data.*;
import priorseditor.engine.datatracks.*;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.operations.Operation_new;

/**
 *
 * @author  kjetikl
 */
public class DataTrackDialog extends javax.swing.JDialog {
    private final String TABLECOLUMN_NAME="Name";    
    private final String TABLECOLUMN_TYPE="Type";
    private final String TABLECOLUMN_SOURCE="Source";
    private final String TABLECOLUMN_ORGANISM="Supported organism";
    private final String TABLECOLUMN_SUPPORTED=".";
    private JTable datatrackTable;
    private DefaultTableModel datatrackTableModel;
    private PriorsEditorGUI gui;
    private PriorsEditorEngine engine;
    private RowFilter<Object,Object> showSupportedRowFilter=null;
    private RowFilter<Object,Object> expressionRowFilter=null;
    private RowFilter<Object,Object> combinedRowFilter=null;
    private ArrayList<RowFilter<Object,Object>> filters=new ArrayList<RowFilter<Object,Object>>(2);
    private SupportedFilter supportedFilter=null;
    
    /** Creates new form DataTrackDialog */
    public DataTrackDialog(PriorsEditorGUI gui) {
        super(gui.getFrame(), true);
        this.gui=gui;
        engine=gui.getEngine();
        initComponents();
 
        datatrackTableModel=new DefaultTableModel(new String[]{TABLECOLUMN_NAME,TABLECOLUMN_TYPE,TABLECOLUMN_SOURCE,TABLECOLUMN_ORGANISM,TABLECOLUMN_SUPPORTED},0);
        datatrackTable=new JTable(datatrackTableModel) {
            @Override
            public boolean isCellEditable(int row, int col) {return false;}
        };
        
        datatrackTable.setFillsViewportHeight(true);
        datatrackTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        datatrackTable.setRowSelectionAllowed(true);
        datatrackTable.getTableHeader().setReorderingAllowed(false);
        datatrackTable.getColumn(TABLECOLUMN_SUPPORTED).setMinWidth(20);
        datatrackTable.getColumn(TABLECOLUMN_SUPPORTED).setMaxWidth(20);
        datatrackTable.getColumn(TABLECOLUMN_TYPE).setMaxWidth(50);
        datatrackTable.getColumn(TABLECOLUMN_TYPE).setMaxWidth(50);
        datatrackTable.getColumn(TABLECOLUMN_SUPPORTED).setCellRenderer(new SupportedDatatrackRenderer());
        datatrackTable.getColumn(TABLECOLUMN_TYPE).setCellRenderer(new DataTypeRenderer());
        datatrackTable.getColumn(TABLECOLUMN_ORGANISM).setCellRenderer(new SupportedOrganismsRendered());
        
        
        datatrackTable.setRowHeight(18);

        scrollPane.setViewportView(datatrackTable);  
        getRootPane().setDefaultButton(okButton);
        
        DataTrack[] availableTracks=gui.getEngine().getDataLoader().getAvailableDatatracks();
        //System.err.println("Got "+availableTracks.length+" tracks");
        for (DataTrack track:availableTracks) {
            int[] supportedOrganisms=track.getSupportedOrganisms();
            //int[] presentOrganisms=engine.getDefaultSequenceCollection().getOrganisms(engine);
            String[] presentBuilds=engine.getDefaultSequenceCollection().getGenomeBuilds(engine);
            Boolean isSupported=new Boolean(track.isSupported(presentBuilds));
            Object[] values=new Object[]{track.getName(),track.getDataType(),track.getSourceSite(),supportedOrganisms,isSupported};
            datatrackTableModel.addRow(values);
        }
        
        datatrackTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                int row=datatrackTable.getSelectedRow();
                if (row>=0) {
                  String trackname=(String)datatrackTable.getValueAt(row, datatrackTable.getColumn(TABLECOLUMN_NAME).getModelIndex());
                  trackname=trackname.replace("-", "_");
                  datanameTextfield.setText(trackname);       
                  Boolean issupported=(Boolean)datatrackTable.getValueAt(row, datatrackTable.getColumn(TABLECOLUMN_SUPPORTED).getModelIndex());
                  okButton.setEnabled(issupported);
                } else {
                   datanameTextfield.setText("");                   
                }
            }
        });
        datatrackTable.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
               super.mouseClicked(e);
               if (e.getButton()==MouseEvent.BUTTON1 && e.getClickCount()==2) {
                   okButton.doClick();
               }
            }
            
        });
        // Disable OK-button if there are currently no sequences 
        if (gui.getEngine().getDefaultSequenceCollection().size()==0) okButton.setEnabled(false);
        // do not show supported column?
        //datatrackTable.removeColumn(datatrackTable.getColumn(datatrackTable.getColumnName(TABLECOLUMN_SUPPORTED)));
        //datatrackTable.moveColumn(1, 2);
        datatrackTable.setAutoCreateRowSorter(true);
        datatrackTable.getRowSorter().toggleSortOrder(datatrackTable.getColumn(TABLECOLUMN_NAME).getModelIndex());
        filters.add(showSupportedRowFilter);
        filters.add(expressionRowFilter);
        int col=datatrackTableModel.findColumn(TABLECOLUMN_SUPPORTED);
        supportedFilter=new SupportedFilter(col);
        filterTextfield.requestFocus();
    }

    /** This method is called from within the constructor to
     * init((TableRowSorter)datatrackTable.getRowSorter()).setRowFilter(null);
    }ialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        topPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        trackSelectionCombobox = new javax.swing.JComboBox();
        jPanel2 = new javax.swing.JPanel();
        dataNameLabel = new javax.swing.JLabel();
        datanameTextfield = new javax.swing.JTextField();
        buttonsPanel = new javax.swing.JPanel();
        cancelButton = new javax.swing.JButton();
        okButton = new javax.swing.JButton();
        filterLabel = new javax.swing.JLabel();
        filterTextfield = new javax.swing.JTextField();
        mainPanel = new javax.swing.JPanel();
        scrollPane = new javax.swing.JScrollPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(DataTrackDialog.class);
        setTitle(resourceMap.getString("DatatrackDialog.title")); // NOI18N
        setName("DatatrackDialog"); // NOI18N

        topPanel.setMinimumSize(new java.awt.Dimension(100, 46));
        topPanel.setName("topPanel"); // NOI18N
        topPanel.setPreferredSize(new java.awt.Dimension(100, 46));
        topPanel.setLayout(new java.awt.BorderLayout());

        jPanel1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 5));
        jPanel1.setMaximumSize(new java.awt.Dimension(180, 32767));
        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setPreferredSize(new java.awt.Dimension(180, 100));

        trackSelectionCombobox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Show all tracks", "Show supported tracks" }));
        trackSelectionCombobox.setName("trackSelectionCombobox"); // NOI18N
        trackSelectionCombobox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                tracksSelectionItemStateChanged(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(trackSelectionCombobox, 0, 155, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap(13, Short.MAX_VALUE)
                .addComponent(trackSelectionCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        topPanel.add(jPanel1, java.awt.BorderLayout.LINE_END);

        jPanel2.setName("jPanel2"); // NOI18N

        dataNameLabel.setText(resourceMap.getString("dataNameLabel.text")); // NOI18N
        dataNameLabel.setName("dataNameLabel"); // NOI18N

        datanameTextfield.setText(resourceMap.getString("datanameTextfield.text")); // NOI18N
        datanameTextfield.setName("datanameTextfield"); // NOI18N
        datanameTextfield.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                datanameTextfieldActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(dataNameLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(datanameTextfield, javax.swing.GroupLayout.DEFAULT_SIZE, 312, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap(15, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(dataNameLabel)
                    .addComponent(datanameTextfield, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        topPanel.add(jPanel2, java.awt.BorderLayout.CENTER);

        getContentPane().add(topPanel, java.awt.BorderLayout.PAGE_START);

        buttonsPanel.setMinimumSize(new java.awt.Dimension(100, 90));
        buttonsPanel.setName("buttonsPanel"); // NOI18N
        buttonsPanel.setPreferredSize(new java.awt.Dimension(100, 42));

        cancelButton.setText(resourceMap.getString("cancelButton.text")); // NOI18N
        cancelButton.setMaximumSize(new java.awt.Dimension(75, 27));
        cancelButton.setMinimumSize(new java.awt.Dimension(75, 27));
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.setPreferredSize(new java.awt.Dimension(75, 27));
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonPressed(evt);
            }
        });

        okButton.setText(resourceMap.getString("okButton.text")); // NOI18N
        okButton.setMaximumSize(new java.awt.Dimension(75, 27));
        okButton.setMinimumSize(new java.awt.Dimension(75, 27));
        okButton.setName("okButton"); // NOI18N
        okButton.setPreferredSize(new java.awt.Dimension(75, 27));
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonPressed(evt);
            }
        });

        filterLabel.setText(resourceMap.getString("filterLabel.text")); // NOI18N
        filterLabel.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 5, 0, 0));
        filterLabel.setName("filterLabel"); // NOI18N

        filterTextfield.setColumns(30);
        filterTextfield.setText(resourceMap.getString("filterTextfield.text")); // NOI18N
        filterTextfield.setName("filterTextfield"); // NOI18N
        filterTextfield.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                keyReleasedInFilterTextfield(evt);
            }
        });

        javax.swing.GroupLayout buttonsPanelLayout = new javax.swing.GroupLayout(buttonsPanel);
        buttonsPanel.setLayout(buttonsPanelLayout);
        buttonsPanelLayout.setHorizontalGroup(
            buttonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, buttonsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(filterLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(filterTextfield, javax.swing.GroupLayout.PREFERRED_SIZE, 175, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 204, Short.MAX_VALUE)
                .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        buttonsPanelLayout.setVerticalGroup(
            buttonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, buttonsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(buttonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(filterLabel)
                    .addComponent(filterTextfield, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        getContentPane().add(buttonsPanel, java.awt.BorderLayout.PAGE_END);

        mainPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 10, 1, 10));
        mainPanel.setMinimumSize(new java.awt.Dimension(500, 400));
        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setPreferredSize(new java.awt.Dimension(700, 400));
        mainPanel.setLayout(new java.awt.BorderLayout());

        scrollPane.setName("scrollPane"); // NOI18N
        mainPanel.add(scrollPane, java.awt.BorderLayout.CENTER);

        getContentPane().add(mainPanel, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void okButtonPressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonPressed
    int row=datatrackTable.getSelectedRow();
    if (row<0) {setVisible(false);return;} // no datatrack selected    
    String targetName=datanameTextfield.getText().trim();
    String nameError=engine.checkNameValidity(targetName,false);
    if (nameError!=null) {
        JOptionPane.showMessageDialog(this, nameError+":\n"+targetName, "Illegal datatrack name", JOptionPane.ERROR_MESSAGE);
        return;
    }
    setVisible(false);
    Operation operation=gui.getEngine().getOperation("new");   
    Object type=datatrackTable.getValueAt(row, datatrackTable.getColumn(TABLECOLUMN_TYPE).getModelIndex());
    String datatype=null;
    if (type instanceof String) datatype=(String)type;
    else if (type instanceof Class) datatype=getTypeStringForClass((Class)type);
    String dnaTrackSource=(String)datatrackTable.getValueAt(row, datatrackTable.getColumn(TABLECOLUMN_NAME).getModelIndex());
    OperationTask newDNATrackTask=new OperationTask("new "+datatype);
    newDNATrackTask.setParameter(OperationTask.OPERATION, operation);
    newDNATrackTask.setParameter(OperationTask.OPERATION_NAME, operation.getName());
    newDNATrackTask.setParameter(OperationTask.TARGET_NAME, targetName);
    newDNATrackTask.setParameter(OperationTask.SOURCE_NAME, targetName);
    newDNATrackTask.setParameter(Operation_new.DATA_TYPE, datatype);    
    newDNATrackTask.setParameter(Operation_new.PARAMETERS,Operation_new.DATA_TRACK_PREFIX+dnaTrackSource);
    gui.launchOperationTask(newDNATrackTask, gui.isRecording());        
    
}//GEN-LAST:event_okButtonPressed

private void cancelButtonPressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonPressed
   setVisible(false);
}//GEN-LAST:event_cancelButtonPressed

private void datanameTextfieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_datanameTextfieldActionPerformed
// TODO add your handling code here:
}//GEN-LAST:event_datanameTextfieldActionPerformed

@SuppressWarnings("unchecked")
private void keyReleasedInFilterTextfield(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_keyReleasedInFilterTextfield
    String text=filterTextfield.getText();//GEN-LAST:event_keyReleasedInFilterTextfield
    int namecol=datatrackTableModel.findColumn(TABLECOLUMN_NAME);
    if (text!=null && text.isEmpty()) expressionRowFilter=null;
    else expressionRowFilter=RowFilter.regexFilter("(?i)"+text,namecol);
    installFilters();
}

private void tracksSelectionItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_tracksSelectionItemStateChanged
    String selection=(String)trackSelectionCombobox.getSelectedItem();
    if (selection.equals("Show all tracks")) showSupportedRowFilter=null;
    else {   
        int col=datatrackTableModel.findColumn(TABLECOLUMN_SUPPORTED);
        showSupportedRowFilter=supportedFilter;
    }
    installFilters();
}//GEN-LAST:event_tracksSelectionItemStateChanged

@SuppressWarnings("unchecked")
private void installFilters() {
   // RowFilter.regexFilter(null, );
   filters.clear();
   if (showSupportedRowFilter!=null) filters.add(showSupportedRowFilter);
   if (expressionRowFilter!=null) filters.add(expressionRowFilter);
   combinedRowFilter=RowFilter.andFilter(filters);
   ((TableRowSorter)datatrackTable.getRowSorter()).setRowFilter(combinedRowFilter);
}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel buttonsPanel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel dataNameLabel;
    private javax.swing.JTextField datanameTextfield;
    private javax.swing.JLabel filterLabel;
    private javax.swing.JTextField filterTextfield;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JScrollPane scrollPane;
    private javax.swing.JPanel topPanel;
    private javax.swing.JComboBox trackSelectionCombobox;
    // End of variables declaration//GEN-END:variables


private class SupportedDatatrackRenderer extends DefaultTableCellRenderer {
    private ImageIcon redbullet;
    private ImageIcon greenbullet;
    public SupportedDatatrackRenderer() {
           super();
           this.setHorizontalAlignment(CENTER);
           java.net.URL redBulletURL=getClass().getResource("resources/icons/redbullet.png");
           java.net.URL greenBulletURL=getClass().getResource("resources/icons/greenbullet.png");
           redbullet=new ImageIcon(redBulletURL); 
           greenbullet=new ImageIcon(greenBulletURL);       
       }
    @Override
    public void setValue(Object value) {
       if (!(value instanceof Boolean)) setIcon(null);
       else {
           Boolean bool=(Boolean)value;
           if (bool.booleanValue())  setIcon(greenbullet);
           else setIcon(redbullet);
       }
    }
}

private class DataTypeRenderer extends DefaultTableCellRenderer {
    private SimpleDataPanelIcon numericIcon;
    private SimpleDataPanelIcon regionIcon;
    private SimpleDataPanelIcon DNAIcon;
    public DataTypeRenderer() {
           super();
           this.setHorizontalAlignment(DefaultTableCellRenderer.CENTER);   
           numericIcon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.NUMERIC_TRACK_GRAPH_ICON,null);
           numericIcon.setForegroundColor(java.awt.Color.BLUE);
           numericIcon.setBackgroundColor(null);
           regionIcon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.REGION_ICON,null);
           regionIcon.setForegroundColor(java.awt.Color.GREEN);
           regionIcon.setBackgroundColor(null);
           DNAIcon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.SEQUENCE_ICON_BASES,null);
           DNAIcon.setForegroundColor(java.awt.Color.BLACK);
           
           setOpaque(true);
    }
    
   @Override
   public void setValue(Object value) {
       if (value instanceof String) {
           String val=(String)value;
                if (val.equalsIgnoreCase(NumericDataset.getType())) setIcon(numericIcon);
           else if (val.equalsIgnoreCase(RegionDataset.getType())) setIcon(regionIcon);
           else if (val.equalsIgnoreCase(DNASequenceDataset.getType())) setIcon(DNAIcon);
       } else if (value instanceof Class) {
           Class cl=(Class)value;
                if (cl==NumericDataset.class) setIcon(numericIcon);
           else if (cl==RegionDataset.class) setIcon(regionIcon);
           else if (cl==DNASequenceDataset.class) setIcon(DNAIcon);
       }

   }
    }

private String getTypeStringForClass(Class type) {
       if (type==NumericDataset.class) return NumericDataset.getType();
  else if (type==RegionDataset.class) return RegionDataset.getType();
  else if (type==DNASequenceDataset.class) return DNASequenceDataset.getType();
  else return null;
}


private class SupportedOrganismsRendered extends DefaultTableCellRenderer {
    
       public SupportedOrganismsRendered() {
           super();
       }
       @Override
       public void setValue(Object value) {
           if (!(value instanceof int[])) setText("<error>");
           int[] organisms=(int[]) value;
           String text="";
           for (int i=0;i<organisms.length;i++) {
               text+=Organism.getCommonName(organisms[i]);
               if (i<organisms.length-1) text+=",";
           }
           setText(text);

       }
}

private class SupportedFilter extends RowFilter<Object,Object> {
        int col=0;
        public SupportedFilter(int column) {
            col=column;
        }
        
        @Override
        public boolean include(Entry<? extends Object, ? extends Object> entry) {
            return ((Boolean)entry.getValue(col)).booleanValue();
        }             
}


}
