/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import priorseditor.engine.data.FeatureSequenceData;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Graphics2D;
import javax.swing.border.Border;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import javax.swing.event.MouseInputListener;
import javax.swing.JComponent;
import java.util.ArrayList;
import priorseditor.engine.data.FeatureDataset;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.RegionSequenceData;
import priorseditor.engine.data.NumericSequenceData;

/**
 * This is the superclass for all datatrack visualizers. 
 * A DataTrackVisualizer is responsible for visualizing one type of data (for one sequence)
 * @author kjetikl
 */
public abstract class DataTrackVisualizer extends JComponent implements MouseInputListener {
     FeatureSequenceData sequencedata; // this is the single DNA sequence from which this datatrack stems
     VisualizationSettings settings;
     String sequenceName;
     String featureName;
     PriorsEditorGUI gui;
     int bordersize=1; // size of black border surrounding track;
     int geneOrientation;
     int sequenceStart;
     int sequenceEnd;
     int TSS=-1;
     int TES=-1;
     int currentmouseposition=-1; // stores the genomic coordinate corresponding to the current mouse position

     
  public DataTrackVisualizer (String sequenceName, String featureName, VisualizationSettings settings) {
      this.sequenceName=sequenceName;
      this.featureName=featureName;
      this.settings=settings;
      this.gui=settings.getGUI();
      FeatureDataset dataset=(FeatureDataset)settings.getEngine().getDataItem(featureName);          
      sequencedata=dataset.getSequenceByName(sequenceName);
      if (sequencedata!=null) {
        geneOrientation=sequencedata.getStrandOrientation();
        sequenceStart=sequencedata.getRegionStart();
        sequenceEnd=sequencedata.getRegionEnd();  
        Integer TSSint=sequencedata.getTSS();
        if (TSSint!=null) TSS=TSSint.intValue();
        Integer TESint=sequencedata.getTES();
        if (TESint!=null) TES=TESint.intValue();
      }
      setBackground(Color.GRAY);
      setOpaque(true);
      addMouseListener(this);
      addMouseMotionListener(this);
      setFocusable(false);
      //setAutoscrolls(true);
      //gui.debugMessage("Added track for "+sequenceName+" track="+featureName+" dataset="+sequencedata);
  }
    
 @Override
   public Dimension getPreferredSize() {
     int height=settings.getDataTrackHeight(featureName);
     int width=settings.getSequenceWindowSize();
     Border border=getBorder();
      if (border==null) {return new Dimension(width,height);}
      else {
          Insets insets=getBorder().getBorderInsets(this);
          return new Dimension(width+insets.left+insets.right,height+insets.top+insets.bottom);
      }
   }

  @Override
   public int getHeight() {
      int height=settings.getDataTrackHeight(featureName);
      Border border=getBorder();
      if (border==null) {return height;}
      else {
        Insets insets=getBorder().getBorderInsets(this);
        return height+insets.top+insets.bottom;
      }
   }
  
  @Override
   public int getWidth() {
      int width=settings.getSequenceWindowSize();
      Border border=getBorder();
      if (border==null) {return width;}
      else {
        Insets insets=getBorder().getBorderInsets(this);
        return width+insets.left+insets.right;
      }
   }

  /** Tries to locate the sequence data if it could not be found at construction time */
  protected void installSequenceData() {
      FeatureDataset dataset=(FeatureDataset)settings.getEngine().getDataItem(featureName);          
      sequencedata=dataset.getSequenceByName(sequenceName);     
      if (sequencedata==null) return;
      geneOrientation=sequencedata.getStrandOrientation();
      sequenceStart=sequencedata.getRegionStart();
      sequenceEnd=sequencedata.getRegionEnd();      
      Integer TSSint=sequencedata.getTSS();
      if (TSSint!=null) TSS=TSSint.intValue();
      Integer TESint=sequencedata.getTES();
      if (TESint!=null) TES=TESint.intValue();
  }

  /** renames the feature dataset for this DataTrackVisualizer */
  public void rename(String newname) {
      featureName=newname;
  }
    
    @Override
    public void paintComponent (Graphics g) {
              super.paintComponent(g);  
              if (sequencedata==null) {
                  installSequenceData();
                  if (sequencedata==null) {
                      setToolTipText(featureName+" data missing");
                      return; // data track is still missing
                  }
              }
              int height=settings.getDataTrackHeight(featureName);
              int width=settings.getSequenceWindowSize();
              int sequenceGenomicStart=sequencedata.getRegionStart();
              int sequenceGenomicEnd=sequencedata.getRegionEnd();
              int yoffset=0;
              int xoffset=0;
              Border border=getBorder();
              if (border!=null) {
                  Insets insets=getBorder().getBorderInsets(this);
                  yoffset=insets.top;
                  xoffset=insets.left;
              }    

              int viewPortStart=settings.getSequenceViewPortStart(sequenceName);
              int viewPortEnd=settings.getSequenceViewPortEnd(sequenceName);
              double scaleFactor=settings.getScale(sequenceName);
              int orientation=settings.getSequenceOrientation(sequenceName);

              int startPosWithinSequence=(viewPortStart>sequenceGenomicStart)?viewPortStart:sequenceGenomicStart; // start at highest
              int endPosWithinSequence=(viewPortEnd>sequenceGenomicEnd)?sequenceGenomicEnd:viewPortEnd; // end at lowest
              int drawnSequenceRegionWidth=endPosWithinSequence-startPosWithinSequence+1;
              int relativeStart=startPosWithinSequence-sequenceGenomicStart;
              if (drawnSequenceRegionWidth<=0) return;
              int translation=sequenceGenomicStart-viewPortStart+relativeStart;
              Graphics2D buffer=(Graphics2D)g;
              if (isPaintingForPrint()) {
                  buffer.setColor(Color.GRAY);
                  buffer.drawRect(0,0,width,height);
                  buffer.fillRect(0,0,width,height);
              }
              AffineTransform saveAt=buffer.getTransform();
              if (optimizeForSpeed(scaleFactor)) scaleFactor=1.0;
              if (orientation==VisualizationSettings.REVERSE) translation-=(int)(Math.ceil((float)width/(float)scaleFactor))+1; // reverse oriented sequences should be "right aligned"
              if (scaleFactor>2) {
                  // I'm not proud of this and I am not sure why it is needed, but it is! 
                  // Or else there will be a small "gap" at the end of the window when scale>2
                  buffer.translate(1-(scaleFactor/2),0);
              }
              scaleFactor*=orientation;
              buffer.scale(scaleFactor,1.0);
              buffer.translate(translation,0); //   
              //System.err.println("translation="+translation);
              //gui.debugMessage(sequenceName+"["+orientation+"]: Drawing "+drawnSequenceRegionWidth+" bases from relative start "+relativeStart+"   translation="+translation+"  viewPort=["+viewPortStart+","+viewPortEnd+"]");
              // - - - - - 
              drawVisibleRegion(buffer,relativeStart,height,width, drawnSequenceRegionWidth,xoffset,yoffset, settings.getSequenceOrientation(sequenceName));
              // - - - - -   
 
              buffer.setTransform(saveAt);
      
     } // end paintComponent
              

/**
 * This function renders the actual datatrack. Subclasses that implement graphtype specific rendering should override this method
 * and render the track according to the parameters given (and the data fields in this class)
 * 
 * @param graphics An instance of the graphics object where the datatrack should be rendered
 * @param start The relative start of viewPort within the sequence. The region of the sequence that should be rendered is by decided by "start" and "width"
 * @param height The height to be used for visualization of the datatrack
 * @param width The width of the whole track (in pixels)
 * @param bases The number of bases to be rendered
 * @param graphicsXoffset The x-coordinate within the graphics object where rendering should placed
 * @param graphicsYoffset The y-coordinate within the graphics object where rendering should placed
 * @param orientation The strand and orientation that should be used when rendering (DIRECT=1, REVERSE=-1).
 */
    public abstract void drawVisibleRegion(Graphics2D graphics, int start, int height, int width, int bases, int graphicsXoffset, int graphicsYoffset, int orientation);
    
    
    // ****************** mouse event listeners ********************
    
    @Override
    public void mouseClicked(MouseEvent e) {}  // unneeded interface implementation
    @Override
    public void mouseEntered(MouseEvent e) {}  // unneeded interface implementation
    @Override
    public void mouseExited(MouseEvent e) {}   // unneeded interface implementation
    @Override   
    public void mousePressed(MouseEvent e) { } // unneeded interface implementation
    @Override    
    public void mouseReleased(MouseEvent e) {} // unneeded interface implementation
    @Override    
    public void mouseDragged(MouseEvent e) {}  // unneeded interface implementation
    @Override
    public void mouseMoved(MouseEvent e) { // update tooltip based on position under mouse pointer
          //if (!gui.getSelectedTool().equals(PriorsEditorGUI.SELECTION_TOOL)) return;
          if (sequencedata==null) return;
          if (SequenceContextMenu.isPopupShowing()) {setToolTipText(null); return;} // Do not display tooltips on top of ContextMenus!
          int shownOrientation=settings.getSequenceOrientation(sequenceName);
          int x=e.getPoint().x;
          if (x<=0 || x>settings.getSequenceWindowSize() || gui.getSelectedTool().equals(PriorsEditorGUI.DRAW_TOOL)) {setToolTipText(null);return;}
          String description="";
          x=x-bordersize;
          int[] range=getGenomicCoordinateFromScreenOffset(x);
//          if (shownOrientation==VisualizationSettings.DIRECT) {
//              for (int i=range[0];i<=range[1];i++) {description+=describeBase(i,shownOrientation)+"<br>";}
//          } else {
//              for (int i=range[1];i>=range[0];i--) {description+=describeBase(i,shownOrientation)+"<br>";}              
//          }
          
          int i=(int)((range[0]+range[1])/2); // use midpoint
          currentmouseposition=i;
          description=describeBase(i,shownOrientation);
          setToolTipText("<html>position = "+description+"</html>");          
    }
  
    /** This method provides a tooltip to be displayed when the mousepointer hoovers over a base*/
    public String describeBase(int genomicCoordinate,int shownOrientation) {
          Object value=sequencedata.getValueAtGenomicPosition(genomicCoordinate);
          String description=""+genomicCoordinate;
          if (TSS>=0) {
              int relative=(geneOrientation==VisualizationSettings.DIRECT)?genomicCoordinate-TSS:TSS-genomicCoordinate;
              if (relative>=0) relative++; // because there is no 0
              if (relative==1) description+=" [+1 TSS]";
              else if (relative>1) description+=" [+"+relative+"]";
              else description+=" ["+relative+"]";
          }
          //description+=": ";
          description+="<br>";
          if (sequencedata instanceof RegionSequenceData) {
              if (((RegionSequenceData)sequencedata).isMotifTrack()) return description; 
          }
          if (value==null) {
                   if (genomicCoordinate>=sequenceStart && genomicCoordinate<=sequenceEnd) description+="NULL!";
                   else description+="Outside sequence";
          }
          else if (value instanceof Character) { 
              char base=((Character)value).charValue();              
              if (shownOrientation==VisualizationSettings.REVERSE) description+="DNA-base = "+reverseBase(base)+" (reverse)";
              else description+="DNA-base = "+base+" (direct)";
          }
          else if (value instanceof Float) { 
              description+=featureName+" = "+value.toString();
              description+=",&nbsp;&nbsp;&nbsp;&nbsp;shown range=["+((NumericSequenceData)sequencedata).getMinAllowedValue()+","+((NumericSequenceData)sequencedata).getMaxAllowedValue()+"]";
          }
          else if (value instanceof ArrayList) {
              ArrayList list=(ArrayList)value;
              int size=list.size();
              if (size==0) return description;
              for (int i=0;i<size;i++) {
                  Region region=(Region)list.get(i);
                  String orientation="*";
                  int ro=region.getOrientation();
                  if (ro==Region.DIRECT) orientation="+";
                  else if (ro==Region.REVERSE) orientation="-";
                  description+=region.getType()+"  ("+region.getLength()+" bp "+orientation+")"+",  score="+region.getScore();
                  String otherProperties=region.getPropertiesAsString();
                  if (otherProperties!=null && !otherProperties.isEmpty()) description+=", "+otherProperties;
                  if (i<size-1) description+="<br>";
              }
          }      
          return description;
    }
    
    
    
    /** Returns the Watson-Crick complementary base for the specified base. Case is conserved */
    public char reverseBase(char base) {
        char reversebase;
        switch (base) {
            case 'a':reversebase='t';break;
            case 'A':reversebase='T';break;
            case 'c':reversebase='g';break;
            case 'C':reversebase='G';break;
            case 'g':reversebase='c';break;
            case 'G':reversebase='C';break;
            case 't':reversebase='a';break;
            case 'T':reversebase='A';break;
            default:reversebase=base;break;
        }
        return reversebase;    
    }
    
    
    // ****************** End of mouse event listeners ********************
  

    public int[] getGenomicCoordinateFromScreenOffset(int x) {
        return settings.getGenomicCoordinateFromScreenOffset(sequenceName,x);   
    }
    
    public int[] getScreenCoordinateFromGenomic(int x) {
        return settings.getScreenCoordinateFromGenomic(sequenceName,x);       
    } 
    
    /** 
     * This method should return true if visualization should be optimized for speed
     * at the specified scale. When visualization is optimized for speed, the bases from
     * the viewport are sampled at equidistance and rendered at 1.0 scale rather than
     * rendering all the bases and using an affine transform to scale it to fit the 
     * sequence window
     */
    public boolean optimizeForSpeed(double scale) {
        return false;
    }
    
    /** Returns the genomic coordinate corresponding to the current mouse position */
    public int getCurrentGenomicMousePosition() {
        return currentmouseposition;
    }
    
    /** Returns the FeatureSequenceData visualized by this class */
    public FeatureSequenceData getTrack() {
        return sequencedata;
    }
}
