/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import javax.swing.border.Border;
import priorseditor.engine.data.FeatureDataset;
import priorseditor.engine.data.NumericSequenceData;

/**
 *
 * @author kjetikl
 */
public class DataTrackVisualizer_Numeric extends DataTrackVisualizer {
    
    /** Constructs a new DefaultDataTrackVisualizer */
    public DataTrackVisualizer_Numeric(String sequenceName, String featureName, VisualizationSettings settings) {
        super(sequenceName, featureName, settings);
        if (sequencedata!=null && !(sequencedata instanceof NumericSequenceData)) {System.err.println("WARNING: DataTrackVisualizer_Sequence is used to render data of type:"+sequencedata.getClass().toString());}
    }
    
    
/**
 * This function renders the actual datatrack. 
 * 
 * @param graphics An instance of the graphics object where the datatrack should be rendered
 * @param start The relative start of viewPort within the sequence. The region of the sequence that should be rendered is by decided by "start" and "width"
 * @param height The height to be used for visualization of the datatrack
 * @param width The width of the whole track
 * @param bases The number of bases to be rendered
 * @param graphicsXoffset The x-coordinate within the graphics object where rendering should placed
 * @param graphicsYoffset The y-coordinate within the graphics object where rendering should placed
 * @param orientation The strand and orientation that should be used when rendering (DIRECT=1, REVERSE=-1).
 */
    @Override
    public void drawVisibleRegion(Graphics2D graphics, int start, int height,int width, int bases, int graphicsXoffset, int graphicsYoffset, int orientation) {
        Color background=Color.GRAY;
        Color foreground=Color.BLACK;
        ColorGradient gradient=null;
        float maxvalue=1;
        float minvalue=0;
        float baselinevalue=0;
        int aboveHeight=0; // number of pixels for displaying values above baseline
        int belowHeight=0; // number of pixels for displaying values below baseline
        boolean drawBaseLine=false;
        float directionalMaximum=0; // 
        float aboveMaximum=0; // 
        float belowMaximum=0; // 
        Color baselineColor=Color.lightGray;
        double scale=settings.getScale(sequenceName);
        boolean safeScale=(scale==(int)scale||scale==0.5||scale==0.25); // deprecated. It might not be as safe as I thought :|
        boolean optimize=optimizeForSpeed(scale);
        background=settings.getBackGroundColor(featureName);
        foreground=settings.getForeGroundColor(featureName);        
        gradient  =settings.getColorGradient(featureName);


        maxvalue=((NumericSequenceData)sequencedata).getMaxAllowedValue();
        minvalue=((NumericSequenceData)sequencedata).getMinAllowedValue();
        baselinevalue=((NumericSequenceData)sequencedata).getBaselineValue();
        if (baselinevalue>minvalue) {
            float range=Math.abs(maxvalue-minvalue);
            aboveMaximum=Math.abs(maxvalue-baselinevalue);
            belowMaximum=Math.abs(baselinevalue-minvalue);
            directionalMaximum=Math.max(aboveMaximum, belowMaximum);
            if (settings.getGraphType(featureName)==VisualizationSettings.BAR_GRAPH) 
                 aboveHeight=(int)Math.ceil((aboveMaximum/range)*(height-1)); 
            else aboveHeight=(int)Math.ceil((height-1)/2f);
            belowHeight=height-(aboveHeight+1); 
            drawBaseLine=true;
        }

        int pixels=(optimize)?width:bases; // the number of pixels to draw in the graphics object (before affine transform)
        for (int i=0;i<pixels;i++) { // draw visible region
                  int pos=start+i; // relative position within sequence
                  // boolean isSafeScale=(safeScale || i==bases-1); // because of a bug (fixed or introduced) in the latest Java VMs this is never safe
                  boolean isSafeScale=false;  
                  if (optimize) pos=start+(int)(i*((float)bases/(float)width)); // this is still buggy!!
                  Object value=sequencedata.getValueAtRelativePosition(pos);
                  if (value==null) { // this probably should not happen...
                      graphics.setColor(Color.GRAY);
                      graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                  }
                  else if (value instanceof Float) { // bargraph or gradient. Instead of drawing a single solid line, finetune the "tip" of the bar with a gradient color to get more precision
                     if (settings.getGraphType(featureName)==VisualizationSettings.BAR_GRAPH) {
                        if (drawBaseLine) {
                           graphics.setColor(background);
                           if (isSafeScale) graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                           else {
                              graphics.drawRect(graphicsXoffset+i, graphicsYoffset, 1, height-1);   
                              graphics.fillRect(graphicsXoffset+i, graphicsYoffset, 1, height-1);   
                           }
                           float val=((Float)value).floatValue();
                           if (val==baselinevalue) continue;
                           else if (val>baselinevalue) {
                               float relativeValue=Math.abs(val/aboveMaximum);
                               int solidbarheight=(int)Math.floor(aboveHeight*relativeValue); // round down to nearest pixel
                               float overshoot=(aboveHeight*relativeValue)-solidbarheight; // amount overshooting the nearest pixel. Use this value to get a gradient color and place one pixel with this color on top of the bar (simulates higher precision)
                               if (solidbarheight<aboveHeight) {
                                   graphics.setColor(gradient.getColor(overshoot));
                                   if (isSafeScale) graphics.drawLine(graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight+1), graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight+1));
                                   else {
                                      graphics.drawRect(graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight+1), 1, 1);
                                      graphics.fillRect(graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight+1), 1, 1);
                                   }
                               }                          
                               if (solidbarheight>0) {
                                   graphics.setColor(foreground);                                     
                                   if (isSafeScale) graphics.drawLine(graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight), graphicsXoffset+i, graphicsYoffset+aboveHeight-1);
                                   else {
                                      graphics.drawRect(graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight), 1, solidbarheight-1);
                                      graphics.fillRect(graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight), 1, solidbarheight-1);
                                   }
                               }  
                          } else { // value below baseline
                               float relativeValue=Math.abs(val/belowMaximum);
                               int solidbarheight=(int)Math.floor(belowHeight*relativeValue); // round down to nearest pixel
                               float overshoot=(belowHeight*relativeValue)-solidbarheight; // amount overshooting the nearest pixel. Use this value to get a gradient color and place one pixel with this color on top of the bar (simulates higher precision)
                               if (solidbarheight<belowHeight && overshoot>0) { // draw overshoot
                                   graphics.setColor(gradient.getColor(overshoot));
                                   if (isSafeScale) graphics.drawLine(graphicsXoffset+i, (graphicsYoffset+aboveHeight+1)+solidbarheight, graphicsXoffset+i, (graphicsYoffset+aboveHeight+1)+solidbarheight);
                                   else { 
                                      graphics.drawRect(graphicsXoffset+i, (graphicsYoffset+aboveHeight)+solidbarheight, 1, 1);
                                      graphics.fillRect(graphicsXoffset+i, (graphicsYoffset+aboveHeight)+solidbarheight, 1, 1);
                                   }
                               }                          
                               if (solidbarheight>0) { // draw the solid bar itself
                                   graphics.setColor(foreground);                                            
                                   if (isSafeScale) graphics.drawLine(graphicsXoffset+i, (graphicsYoffset+aboveHeight+1), graphicsXoffset+i, (graphicsYoffset+aboveHeight)+solidbarheight);
                                   else {
                                      graphics.drawRect(graphicsXoffset+i, (graphicsYoffset+aboveHeight), 1, solidbarheight);
                                      graphics.fillRect(graphicsXoffset+i, (graphicsYoffset+aboveHeight), 1, solidbarheight);
                                   }
                               }                                 
                          }
                        } else { // Bar graph without baseline
                           float relativeValue=((Float)value).floatValue()/maxvalue; 
                           int solidbarheight=(int)Math.floor(height*relativeValue); // round down to nearest pixel
                           float overshoot=(height*relativeValue)-solidbarheight; // amount overshooting the nearest pixel. Use this value to get a gradient color and place one pixel with this color on top of the bar (simulates higher precision)
                           graphics.setColor(background);
                           if (isSafeScale) graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                           else {
                              graphics.drawRect(graphicsXoffset+i, graphicsYoffset, 1, height-1);   
                              graphics.fillRect(graphicsXoffset+i, graphicsYoffset, 1, height-1);   
                           }
                           if (solidbarheight<height) {
                               graphics.setColor(gradient.getColor(overshoot));
                               if (isSafeScale) graphics.drawLine(graphicsXoffset+i, graphicsYoffset+height-(solidbarheight+1), graphicsXoffset+i, graphicsYoffset+height-(solidbarheight+1));
                               else {
                                   graphics.drawRect(graphicsXoffset+i, graphicsYoffset+height-(solidbarheight+1), 1, 1);
                                   graphics.fillRect(graphicsXoffset+i, graphicsYoffset+height-(solidbarheight+1), 1, 1);
                               }
                           }                                                
                           if (solidbarheight>0) {
                               graphics.setColor(foreground); 
                               if (isSafeScale) graphics.drawLine(graphicsXoffset+i, graphicsYoffset+height-(solidbarheight), graphicsXoffset+i, graphicsYoffset+height-1);
                               else {
                                   graphics.drawRect(graphicsXoffset+i, graphicsYoffset+height-(solidbarheight), 1, solidbarheight-1);
                                   graphics.fillRect(graphicsXoffset+i, graphicsYoffset+height-(solidbarheight), 1, solidbarheight-1);
                               }
                           }                            
                        }                     
                     } else { // ++++++++++++++++  Gradient graph  +++++++++++++++++
                        if (drawBaseLine) {
                           graphics.setColor(background);
                           if (isSafeScale) {
                              graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                           } else {
                              graphics.drawRect(graphicsXoffset+i, graphicsYoffset, 1, height-1);  
                              graphics.fillRect(graphicsXoffset+i, graphicsYoffset, 1, height-1);  
                           }
                           float val=((Float)value).floatValue();
                           if (val==baselinevalue) continue;
                           else if (val>baselinevalue) {
                               float relativeValue=val/directionalMaximum;
                               graphics.setColor(gradient.getColor(relativeValue)); 
                               if (isSafeScale) graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+aboveHeight-1);                                                           
                               else {
                                  graphics.drawRect(graphicsXoffset+i, graphicsYoffset, 1, aboveHeight-1);                  
                                  graphics.fillRect(graphicsXoffset+i, graphicsYoffset, 1, aboveHeight-1);                  
                               }
                           } else {
                               float relativeValue=Math.abs(val/directionalMaximum);
                               graphics.setColor(gradient.getColor(relativeValue)); 
                               if (isSafeScale) graphics.drawLine(graphicsXoffset+i, graphicsYoffset+aboveHeight+1, graphicsXoffset+i, graphicsYoffset+height-1);                                                             
                               else {
                                  graphics.drawRect(graphicsXoffset+i, graphicsYoffset+aboveHeight+1, 1, height-aboveHeight-2);                  
                                  graphics.fillRect(graphicsXoffset+i, graphicsYoffset+aboveHeight+1, 1, height-aboveHeight-2);                  
                               }
                           }
                        } else { // gradient graph without baseline
                           float relativeValue=((Float)value).floatValue()/(maxvalue-baselinevalue);
                           graphics.setColor(gradient.getColor(relativeValue)); 
                           if (isSafeScale)  graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);                              
                           else {
                               graphics.drawRect(graphicsXoffset+i, graphicsYoffset, 1, height-1);                  
                               graphics.fillRect(graphicsXoffset+i, graphicsYoffset, 1, height-1);                  
                           }
                        }       
                     }
                  }
              } // end for each position
        if (drawBaseLine) {
             graphics.setColor(baselineColor);
             graphics.drawLine(graphicsXoffset-1, graphicsYoffset+aboveHeight, graphicsXoffset+pixels, graphicsYoffset+aboveHeight);   
             //graphics.drawRect(graphicsXoffset, graphicsYoffset+aboveHeight, bases-1, 0); 
        }
          
    } // end of drawVisibleRegion   
    
    
    @Override
    public boolean optimizeForSpeed(double scale) {
        return false;
    }    
    
//    ---------------   Functionality for the "Draw tool"   ------------------------------
            
    private Float oldvalue=null;
    private Integer oldX=null;
    private NumericSequenceData buffer;
    private NumericSequenceData original;
    @Override
    public void mousePressed(MouseEvent e) {
       if (gui.getSelectedTool().equals(PriorsEditorGUI.DRAW_TOOL)) {
             //oldcolor=settings.getForeGroundColor(featureName);
             //settings.setForeGroundColor(featureName,java.awt.Color.MAGENTA);
             original=(NumericSequenceData)sequencedata;
             buffer=(NumericSequenceData)sequencedata.clone();
             sequencedata=buffer; // 'buffer' is not referenced in the engine. We switch the current 'reference' (sequencedata) to point to this buffer
             mouseDragged(e); // this is just to paint at the first pixel (before dragging has commenced)
       }
    }

    @Override
    public void mouseReleased(MouseEvent e) {
       if (gui.getSelectedTool().equals(PriorsEditorGUI.DRAW_TOOL)) {
            //settings.setForeGroundColor(featureName,oldcolor);
            // commit changes to engine. using the gui's "updateDataItem" item
            oldvalue=null;
            oldX=null;
            FeatureDataset dataset=(FeatureDataset)gui.getEngine().getDataItem(featureName);
            dataset=(FeatureDataset)dataset.clone();
            dataset.replaceSequence(sequencedata); // copy contents of the buffer (referenced by sequencedata) into the new clone
            gui.updateDataItem(dataset); // update the registered dataset with the new data from the buffer (this will update the whole dataset not just the single sequence).
            sequencedata=original; // and point back to the (new) data now registered with the engine instead of the temporary buffer which is not registered
            this.repaint();
       }
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        if (!gui.getSelectedTool().equals(PriorsEditorGUI.DRAW_TOOL)) return;
        float maxvalue=((NumericSequenceData)sequencedata).getMaxAllowedValue();
        float minvalue=((NumericSequenceData)sequencedata).getMinAllowedValue();
        int y=e.getPoint().y;
        int[] range=getGenomicCoordinateFromScreenOffset(e.getPoint().x-bordersize); // remember to subtract the border when calculating relative X-coordinate
        float value=getValueForY(y);
        if (value>maxvalue && !e.isShiftDown()) value=maxvalue;
        else if (value<minvalue && !e.isShiftDown()) value=minvalue;
        if (oldX==null) {
            oldX=range[0];
            int newX=range[1];
            oldvalue=value;
            for (int i=oldX;i<=newX;i++) ((NumericSequenceData)sequencedata).setValueAtGenomicPosition(i, value);
        } else {
            int newX=(oldX<range[1])?range[1]:range[0];
            if (newX==oldX) {
                ((NumericSequenceData)sequencedata).setValueAtGenomicPosition(newX, value);
            } else {
                float increment=(value-oldvalue)/Math.abs(newX-oldX);
                float current=oldvalue;
                int inc=(newX<oldX)?-1:1;
                for (int i=oldX;i!=newX;i+=inc) {
                    ((NumericSequenceData)sequencedata).setValueAtGenomicPosition(i, current);
                    current+=increment;
                }
            }
            //gui.logMessage(featureName+":"+e.getPoint().x+","+y+" old:"+oldX+"="+oldvalue+"   new:"+newX+"="+value+"  INC="+inc);
            oldX=newX;
            oldvalue=value;
            gui.statusMessage(featureName+" = "+value);
        }
        this.repaint();
    }


    @Override
    public void mouseMoved(MouseEvent e) {
        super.mouseMoved(e);
        if (!gui.getSelectedTool().equals(PriorsEditorGUI.DRAW_TOOL)) return;
        int y=e.getPoint().y;
        int height=settings.getDataTrackHeight(featureName);
        float value=getValueForY(y);
        if (y<=height) gui.statusMessage(featureName+" = "+value);  else gui.statusMessage("");
    }


    @Override
    public void mouseExited(MouseEvent e) {
        super.mouseExited(e);
        if (gui.getSelectedTool().equals(PriorsEditorGUI.DRAW_TOOL)) gui.statusMessage("");
    }

   /** Returns the track value corresponding to the given mouse Y position within the track
     * An y value of 0 should correspond to
     */
    private float getValueForY(int y) {
        Border border=getBorder();
        if (border!=null) {
          y-=getBorder().getBorderInsets(this).top;
        }
        float maxvalue=((NumericSequenceData)sequencedata).getMaxAllowedValue();
        float minvalue=((NumericSequenceData)sequencedata).getMinAllowedValue();
        float baselinevalue=((NumericSequenceData)sequencedata).getBaselineValue();
        int height=settings.getDataTrackHeight(featureName);
        if (baselinevalue>minvalue) { // baseline is used
            float range=Math.abs(maxvalue-minvalue);
            float aboveRange=Math.abs(maxvalue-baselinevalue); // magnitude above baseline
            float belowRange=Math.abs(baselinevalue-minvalue); // magnitude above baseline
            //float directionalMaximum=Math.max(aboveMaximum, belowMaximum);
            float aboveHeight=0;
            if (settings.getGraphType(featureName)==VisualizationSettings.BAR_GRAPH)
                 aboveHeight=(int)Math.ceil((aboveRange/range)*(height-1));
            else aboveHeight=(int)Math.ceil((height-1)/2f);
            float belowHeight=height-(aboveHeight+1);
            //gui.logMessage("belowHeight="+belowHeight+"    y="+y+"  height="+height+"  fraction="+((y-aboveHeight)/belowHeight)+"  line="+(y-aboveHeight));
            if (y==aboveHeight) return baselinevalue; //baselinevalue;
            else if (y<=aboveHeight) return ((aboveHeight-y)/(float)aboveHeight)*aboveRange+baselinevalue;
            else return (belowHeight>0)?(minvalue+(height-(y+1))*(belowRange/belowHeight)):(minvalue+(height-(y+1))*(range/height));

        } else { // baseline not used
            return (maxvalue-minvalue)*((height-y)/(float)height);
        }
    }
    
}
