/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JToolTip;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.FeatureSequenceData;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.RegionSequenceData;
import priorseditor.engine.data.Region;

/**
 *
 * @author kjetikl
 */
public class DataTrackVisualizer_Region extends DataTrackVisualizer {
    private MotifTooltip customTooltip;
    
    /** Constructs a new DefaultDataTrackVisualizer */
    public DataTrackVisualizer_Region(String sequenceName, String featureName, VisualizationSettings settings) {
        super(sequenceName, featureName, settings);
        customTooltip=new MotifTooltip(this,settings);
        customTooltip.setComponent(this);
        if (sequencedata!=null && !(sequencedata instanceof RegionSequenceData)) {System.err.println("WARNING: DataTrackVisualizer_Region is used to render data of type:"+sequencedata.getClass().toString());}

    }
    
    
/**
 * This function renders the actual datatrack. 
 * 
 * @param graphics An instance of the graphics object where the datatrack should be rendered
 * @param start The relative start of viewPort within the sequence. The region of the sequence that should be rendered is by decided by "start" and "width"
 * @param height The height to be used for visualization of the datatrack
 * @param width The width of the whole track
 * @param numbases The number of bases to be rendered
 * @param graphicsXoffset The x-coordinate within the graphics object where rendering should placed
 * @param graphicsYoffset The y-coordinate within the graphics object where rendering should placed
 * @param orientation The strand and orientation that should be used when rendering (DIRECT=1, REVERSE=-1).
 */
    @Override
    public void drawVisibleRegion(Graphics2D graphics, int start, int height, int width, int numbases, int graphicsXoffset, int graphicsYoffset, int orientation) {
        Color background=Color.GRAY;
        Color foreground=Color.BLACK;
        ColorGradient gradient=null;
        float maxvalue=1;
        
        int aboveHeight=0; // number of pixels for displaying values above baseline
        int belowHeight=0; // number of pixels for displaying values below baseline

        //if (sequenceName.equals("NTNG1")) System.err.println("Drawing track"); // debug
        Color baselineColor=Color.lightGray;
        double scale=settings.getScale(sequenceName);
        boolean safeScale=(scale==(int)scale||scale==0.5||scale==0.25);
        int graphType=settings.getGraphType(featureName);
        boolean isMotifTrack=((RegionSequenceData)sequencedata).isMotifTrack();
        boolean isModuleTrack=((RegionSequenceData)sequencedata).isModuleTrack();
        boolean useMulticolor=(graphType==VisualizationSettings.REGION_MULTICOLOR_GRAPH);// || isMotifTrack;
        boolean showStrand=settings.shouldVisualizeRegionStrand(featureName);
        boolean showScore=settings.shouldVisualizeRegionScore(featureName);
        
        int genomicStart=sequencedata.getRegionStart()+start;
        int genomicEnd=sequencedata.getRegionEnd()+start;
        
        background=settings.getBackGroundColor(featureName);
        foreground=settings.getForeGroundColor(featureName);        
        gradient  =settings.getColorGradient(featureName);

        maxvalue=((RegionSequenceData)sequencedata).getMaxScoreValue();

        aboveHeight=(int)Math.ceil((height-1)/2f);
        belowHeight=height-(aboveHeight+1); 

        //System.err.println(sequenceName+"["+sequencedata.getStrandOrientation()+"] shown="+orientation+"  from Gene="+orientationFromGene);
        graphics.setColor(background);
        graphics.drawRect(graphicsXoffset, graphicsYoffset, numbases-1, height);
        graphics.fillRect(graphicsXoffset, graphicsYoffset, numbases-1, height);        
        RegionVisualizationFilter filter=gui.getRegionVisualizationFilter();
        if (!useMulticolor) graphics.setColor(foreground);
        ArrayList<Region>regionList=((RegionSequenceData)sequencedata).getRegionsOverlappingGenomicInterval(genomicStart, genomicEnd);
        for (Region region:regionList) {
            if (isMotifTrack){
                if (!settings.isVisible(region.getType())) continue;
                if (filter!=null && !filter.shouldVisualize(region)) continue;
            }
            drawRegion(graphics, region, (RegionSequenceData)sequencedata, showScore, showStrand, (useMulticolor)?settings.getFeatureColor(region.getType()):foreground, start, graphicsXoffset, graphicsYoffset, aboveHeight, belowHeight, maxvalue, height, orientation);           
                   
        } // end: for each region
        // draw "baseline"
        if (showStrand) {
             graphics.setColor(baselineColor);
             graphics.drawLine(graphicsXoffset-1, graphicsYoffset+aboveHeight, graphicsXoffset+numbases, graphicsYoffset+aboveHeight);   
        } 
    } // end of drawVisibleRegion    


    private void drawRegion(Graphics2D graphics, Region region, RegionSequenceData sequencedata, boolean showScore, boolean showStrand, Color usecolor, int start, int graphicsXoffset, int graphicsYoffset, int aboveHeight, int belowHeight, float maxvalue, int height, int orientation) {
            int regionwidth=region.getLength()-1;
            int regionStart=region.getRelativeStart();
            if (regionStart>sequencedata.getSize()) return;
            if (regionStart<0) {regionwidth+=regionStart;regionStart=0;}
            int regionEnd=region.getRelativeEnd();
            if (regionEnd<0) return;
            if (regionEnd>sequencedata.getSize()) {regionwidth-=(regionEnd-sequencedata.getSize()+1);}
            regionStart+=(graphicsXoffset-start);

            graphics.setColor(usecolor);
       
            float unit=height/maxvalue;
            float aboveunit=aboveHeight/maxvalue;
            float belowunit=belowHeight/maxvalue;
            //System.err.println("height="+height+"  AboveHeight="+aboveHeight+"   belowHeight="+belowHeight+"  AboveUnit="+aboveunit+"   belowUnit="+belowunit);

            int regionOrientation=region.getOrientation();
            if (orientation==FeatureSequenceData.REVERSE) {
                if (regionOrientation==Region.DIRECT) regionOrientation=Region.REVERSE;
                else if (regionOrientation==Region.REVERSE) regionOrientation=Region.DIRECT;
            }
            if (showScore && showStrand) { // show both score and strand
                   float score=region.getScore();
                   int aboveboxheight=(int)Math.ceil(score*aboveunit);
                   int belowboxheight=(int)Math.ceil(score*belowunit);
                   //System.err.println("score="+score+"   aboveunit="+aboveunit+"  aboveboxheight="+aboveboxheight+"   belowunit="+belowunit+"    belowboxheight="+belowboxheight);
                   if (region.getScore()!=0) {
                       if (regionOrientation==Region.DIRECT  || regionOrientation==Region.INDETERMINED) drawBox(graphics, regionStart, graphicsYoffset+(aboveHeight-aboveboxheight), regionwidth, aboveboxheight);
                       if (regionOrientation==Region.REVERSE || regionOrientation==Region.INDETERMINED) drawBox(graphics, regionStart, graphicsYoffset+aboveHeight+1, regionwidth, belowboxheight);
                   }
            } else if (showScore && !showStrand) { // show score but not strand
                int boxheight=(int)Math.ceil(region.getScore()*unit);
                //System.err.println("Score="+region.getScore()+"  relative="+relativescore+"  unit="+unit+"   ["+height+","+maxvalue+"]   boxheight="+boxheight);
                if (region.getScore()!=0) drawBox(graphics, regionStart, graphicsYoffset+(height-boxheight), regionwidth, boxheight);
            } else if (!showScore && showStrand) { // show strand but not score
                if (regionOrientation==Region.DIRECT  || regionOrientation==Region.INDETERMINED) drawBox(graphics, regionStart, graphicsYoffset, regionwidth, aboveHeight-1);
                if (regionOrientation==Region.REVERSE || regionOrientation==Region.INDETERMINED) drawBox(graphics, regionStart, graphicsYoffset+aboveHeight+1, regionwidth, belowHeight);
            } else if (!showScore && !showStrand) { // show neither strand nor score
                drawBox(graphics, regionStart, graphicsYoffset, regionwidth, height);
            }
    } // end of drawRegion


    private void drawBox(Graphics2D g, int x, int y, int width, int height) {
           g.fillRect(x, y, width, height);     
           g.drawRect(x, y, width, height);
    }

    @Override
    public void mouseExited(MouseEvent e) {
        super.mouseExited(e);
        customTooltip.clearData();
    }
       
    @Override
    public void mouseClicked(MouseEvent e) {
       if (e.getClickCount()<2 || sequencedata==null || !(((RegionSequenceData)sequencedata).isMotifTrack() || ((RegionSequenceData)sequencedata).isModuleTrack()) || !gui.getSelectedTool().equals(PriorsEditorGUI.SELECTION_TOOL)) return;
//       setToolTipText(null);
//       repaint();
       gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
       int x=e.getPoint().x;
       x=x-bordersize;
       int[] range=getGenomicCoordinateFromScreenOffset(x);
//       final ArrayList<Region> regions=((RegionSequenceData)sequencedata).getRegionsOverlappingGenomicInterval(range[0], range[1]);
       final ArrayList<Region> regions=getVisibleRegions((range[0]+range[1])/2);
       if (regions.size()==0) {
           gui.getFrame().setCursor(Cursor.getDefaultCursor());
           return;
       }
       if (regions.size()==1) {
              Region region=regions.get(0);
              String type=region.getType();
              Data motifdata=gui.getEngine().getDataItem(type);
              if (motifdata!=null && motifdata instanceof Motif) showPrompt(motifdata);
       } else {
           final JPopupMenu menu=new JPopupMenu();
           ActionListener menulistener=new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent ev) {
                    Object obj=ev.getSource();
                    for (int i=0;i<menu.getComponentCount();i++) {
                        if (obj==menu.getComponent(i)) {
                            Region region=regions.get(i);
                            String type=region.getType();
                            Data data=gui.getEngine().getDataItem(type);
                            if (data!=null) showPrompt(data);
                        }
                    }
                }
            };
           for (Region region:regions) {
              String type=region.getType();
              Data motifdata=gui.getEngine().getDataItem(type);
              if (motifdata!=null && motifdata instanceof Motif) {
                  String command=((Motif)motifdata).getPresentationName();
                  if (menuContainsItem(menu, command)) continue;
                  JMenuItem menuitem=new JMenuItem(command);
                  menuitem.addActionListener(menulistener);
                  menu.add(menuitem);
              }              
           }
           if (menu.getComponentCount()==1) { // The multiple overlapping motifs were all of the same type so just show the prompt
                Region region=regions.get(0);
                String type=region.getType();
                Data data=gui.getEngine().getDataItem(type);
                if (data!=null) showPrompt(data);
           } else {
               gui.getFrame().setCursor(Cursor.getDefaultCursor());
               menu.show(e.getComponent(), e.getX(),e.getY());
           }
       }
    }
    
    
    private boolean menuContainsItem(JPopupMenu menu, String text) {
        for (int i=0;i<menu.getComponentCount();i++) {
            Component item=menu.getComponent(i);
            if (item instanceof JMenuItem && ((JMenuItem)item).getActionCommand().equals(text)) return true;
        }
        return false;
    }

    private void showPrompt(Data data) {
            gui.getMotifsPanel().showPrompt(data);
    }


    @Override
    public JToolTip createToolTip() {
        return customTooltip;
    }

    /** Returns TRUE if this DataTrackVisualizer is coupled to a motif datatrack*/
    public boolean isMotifTrack() {
        if (sequencedata==null || !(sequencedata instanceof RegionSequenceData)) return false;
        return ((RegionSequenceData)sequencedata).isMotifTrack();
    }

    /** Returns TRUE if this DataTrackVisualizer is coupled to a motif datatrack*/
    public boolean isModuleTrack() {
        if (sequencedata==null || !(sequencedata instanceof RegionSequenceData)) return false;
        return ((RegionSequenceData)sequencedata).isModuleTrack();
    }


    /** 
     * Returns a list of the currently visible regions (as determined by visibility status and dynamic filters)
     * that overlap with the given genomic position
     */
    public ArrayList<Region> getVisibleRegions(int genomicpos) {
           ArrayList<Region> regions=((RegionSequenceData)sequencedata).getValueAtGenomicPosition(genomicpos);
           if (regions==null) return new ArrayList<Region>();
           if (regions.isEmpty()) return regions;
           RegionVisualizationFilter regionfilter=settings.getGUI().getRegionVisualizationFilter();           
           Iterator<Region> iter=regions.iterator();
           while (iter.hasNext()) {
               Region region=iter.next();
               String motifname=region.getType();
               if (!settings.isVisible(motifname)) iter.remove();
               if (regionfilter!=null && !regionfilter.shouldVisualize(region)) iter.remove();
           }
           return regions;
    }
    
        
    
}

