/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.geom.AffineTransform;
import javax.swing.border.Border;
import priorseditor.engine.data.DNASequenceData;
import priorseditor.engine.data.FeatureSequenceData;


/**
 *
 * @author kjetikl
 */
public class DataTrackVisualizer_Sequence extends DataTrackVisualizer {
    private static Font baseFont = new Font(Font.SANS_SERIF, Font.PLAIN, 10);
    
    /** Constructs a new DefaultDataTrackVisualizer */
    public DataTrackVisualizer_Sequence(String sequenceName, String featureName, VisualizationSettings settings) {
        super(sequenceName, featureName, settings);
        if (sequencedata!=null && !(sequencedata instanceof DNASequenceData)) {System.err.println("WARNING: DataTrackVisualizer_Sequence is used to render data of type:"+sequencedata.getClass().toString());}
    }
    
    @Override
    public void paintComponent (Graphics g) {
              super.paintComponent(g);   
              if (sequencedata==null) {
                  installSequenceData();
                  if (sequencedata==null) {
                      setToolTipText(featureName+" data missing");
                      return; // data track is still missing
                  }
              }
              int height=settings.getDataTrackHeight(featureName);
              int width=settings.getSequenceWindowSize();
              int sequenceGenomicStart=sequencedata.getRegionStart();
              int sequenceGenomicEnd=sequencedata.getRegionEnd();
              int yoffset=0;
              int xoffset=0;
              Border border=getBorder();
              if (border!=null) {
                  Insets insets=getBorder().getBorderInsets(this);
                  yoffset=insets.top;
                  xoffset=insets.left;
              }    

              int viewPortStart=settings.getSequenceViewPortStart(sequenceName);
              int viewPortEnd=settings.getSequenceViewPortEnd(sequenceName);
              double scaleFactor=settings.getScale(sequenceName);
              int orientation=settings.getSequenceOrientation(sequenceName);

              int startPosWithinSequence=(viewPortStart>sequenceGenomicStart)?viewPortStart:sequenceGenomicStart; // start at highest
              int endPosWithinSequence=(viewPortEnd>sequenceGenomicEnd)?sequenceGenomicEnd:viewPortEnd; // end at lowest
              int drawnSequenceRegionWidth=endPosWithinSequence-startPosWithinSequence+1;
              int relativeStart=startPosWithinSequence-sequenceGenomicStart;
              if (drawnSequenceRegionWidth<=0) return;
              int translation=sequenceGenomicStart-viewPortStart+relativeStart;
              Graphics2D buffer=(Graphics2D)g;
              if (isPaintingForPrint()) {
                  buffer.setColor(Color.GRAY);
                  buffer.drawRect(0,0,width,height);
                  buffer.fillRect(0,0,width,height);
              }
              AffineTransform saveAt=buffer.getTransform();
              if (orientation==VisualizationSettings.REVERSE) translation-=(int)(Math.ceil((float)width/(float)scaleFactor))+1; // reverse oriented sequences should be "right aligned"
              if (scaleFactor>2) {
                  // I'm not proud of this and I am not sure why it is needed, but it is! 
                  // Or else there will be a small "gap" at the end of the window when scale>2
                  buffer.translate(1-(scaleFactor/2),0);
              }
              scaleFactor*=orientation;
              buffer.scale(scaleFactor,1.0);
              buffer.translate(translation,0); //       
              //gui.debugMessage(sequenceName+"["+orientation+"]: Drawing "+drawnSequenceRegionWidth+" bases from relative start "+relativeStart+"   translation="+translation+"  viewPort=["+viewPortStart+","+viewPortEnd+"]");
              // - - - - - 
              drawVisibleRegion(buffer,relativeStart,height,width, drawnSequenceRegionWidth,xoffset,yoffset, settings.getSequenceOrientation(sequenceName));
              // - - - - -    
              buffer.setTransform(saveAt);
              float pixelsprbase=(float)settings.getScale(sequenceName);
              float letterwidth=g.getFontMetrics(baseFont).charWidth('G');
              if (height>=10 && pixelsprbase>=letterwidth+2) {
                  g.setFont(baseFont);
                  g.setColor(java.awt.Color.BLACK);
                  int pos=0;
                  int vpwidth=viewPortEnd-viewPortStart+1;
                  Character[] characterbuffer=new Character[vpwidth];                 
                  for (int i=0;i<vpwidth;i++) {
                      pos=viewPortStart+i;
                      Object value=sequencedata.getValueAtGenomicPosition(pos);
                      if (!(value instanceof Character)) continue;
                      char base=((Character)value).charValue();
                      if (orientation==FeatureSequenceData.REVERSE) characterbuffer[vpwidth-(i+1)]=reverseBase(base);
                      else characterbuffer[i]=base;
                  }   
                  float letteroffset=(pixelsprbase-letterwidth)/2;
                  for (int i=0;i<vpwidth;i++) {                      
                     if (characterbuffer[i]!=null) g.drawString(""+characterbuffer[i], (int)(xoffset+pixelsprbase*i+letteroffset), (int)(yoffset+(height+10)/2)-1);
                  }                  
              }
      
     } // end paintComponent
    
/**
 * This function renders the actual datatrack. 
 * 
 * @param graphics An instance of the graphics object where the datatrack should be rendered
 * @param start The relative start of viewPort within the sequence. The region of the sequence that should be rendered is by decided by "start" and "width"
 * @param height The height to be used for visualization of the datatrack
 * @param width The total width of the track (in pixels) 
 * @param bases The number of bases to be rendered
 * @param graphicsXoffset The x-coordinate within the graphics object where rendering should placed
 * @param graphicsYoffset The y-coordinate within the graphics object where rendering should placed
 * @param orientation The strand and orientation that should be used when rendering (DIRECT=1, REVERSE=-1).
 */
    @Override
    public void drawVisibleRegion(Graphics2D graphics, int start, int height,int width, int bases, int graphicsXoffset, int graphicsYoffset, int orientation) {
        Color colorA=settings.getBaseColor('A');
        Color colorC=settings.getBaseColor('C');
        Color colorG=settings.getBaseColor('G');
        Color colorT=settings.getBaseColor('T');
        Color colorX=settings.getBaseColor('X');

        double scale=settings.getScale(sequenceName);
        boolean safeScale=(scale==(int)scale||scale==0.5||scale==0.25);
        for (int i=0;i<bases;i++) { // draw visible region
                  char base;
                  int pos=start+i; // relative position within sequence
                  //if (orientation==FeatureSequenceData.REVERSE) pos=start+(bases-1)-i; // <= I use (-1)*scale instead in Affine transform
                  Object value=sequencedata.getValueAtRelativePosition(pos);
                  if (value==null) { // this probably should not happen...
                      graphics.setColor(Color.GRAY);
                      graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                  }
                  else if (value instanceof Character) { // DNA track
                      base=((Character)value).charValue();
                      if (orientation==FeatureSequenceData.REVERSE) { base=reverseBase(base); }
                      switch(base) {
                          case 'A': graphics.setColor(colorA);break;
                          case 'a': graphics.setColor(colorA);break;
                          case 'C': graphics.setColor(colorC);break;
                          case 'c': graphics.setColor(colorC);break;
                          case 'G': graphics.setColor(colorG);break;
                          case 'g': graphics.setColor(colorG);break;
                          case 'T': graphics.setColor(colorT);break;
                          case 't': graphics.setColor(colorT);break;
                          default : graphics.setColor(colorX);break;
                      }
                      //graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                       if (safeScale || i==bases-1) graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                       else {
                          // NOTE: both the following lines (drawRect AND fillRect) are needed for correct functioning!!
                          // (I don't know why but this is true. At least on my VM)
                          // if only drawRect is used there will still be streaking (gray lines)
                          // if only fillRect is used there will be a translation offset error of half a base (small gray segment at end of window)
                          graphics.drawRect(graphicsXoffset+i, graphicsYoffset, 1, height);
                          graphics.fillRect(graphicsXoffset+i, graphicsYoffset, 1, height);                          
                       }  
                  }
              }  
    } // end of drawVisibleRegion    
}
