/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import priorseditor.engine.data.*;
import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;

/**
 *
 * @author kjetikl
 */
public class DefaultDataTrackVisualizer extends DataTrackVisualizer {
    
    /** Constructs a new DefaultDataTrackVisualizer */
    public DefaultDataTrackVisualizer(String sequenceName, String featureName, VisualizationSettings settings) {
        super(sequenceName, featureName, settings);
    }
    
    
/**
 * This function renders the actual datatrack. Subclasses that implement graphtype specific rendering should override this method
 * and render the track according to the parameters given (and the data fields in this class)
 * 
 * @param graphics An instance of the graphics object where the datatrack should be rendered
 * @param start The relative start of viewPort within the sequence. The region of the sequence that should be rendered is by decided by "start" and "width"
 * @param height The height to be used for visualization of the datatrack
 * @param width The width of the track (in pixels)
 * @param bases The number of bases to be rendered
 * @param graphicsXoffset The x-coordinate within the graphics object where rendering should placed
 * @param graphicsYoffset The y-coordinate within the graphics object where rendering should placed
 * @param orientation The strand and orientation that should be used when rendering (DIRECT=1, REVERSE=-1).
 */
    @Override
    public void drawVisibleRegion(Graphics2D graphics, int start, int height, int width, int bases, int graphicsXoffset, int graphicsYoffset, int orientation) {
        Color background=Color.GRAY;
        Color foreground=Color.BLACK;
        ColorGradient gradient=null;
        float maxvalue=1;
        float minvalue=0;
        float baselinevalue=0;
        int aboveHeight=0; // number of pixels for displaying values above baseline
        int belowHeight=0; // number of pixels for displaying values below baseline
        boolean drawBaseLine=false;
        float directionalMaximum=0; // 
        float aboveMaximum=0; // 
        float belowMaximum=0; // 
        Color baselineColor=Color.lightGray;
        double scale=settings.getScale(sequenceName);
        boolean safeScale=(scale==(int)scale||scale==0.5||scale==0.25);
        
        if (!(sequencedata instanceof DNASequenceData)) {
              background=settings.getBackGroundColor(featureName);
              foreground=settings.getForeGroundColor(featureName);        
              gradient  =settings.getColorGradient(featureName);
         }
        if (sequencedata instanceof NumericSequenceData) {
            maxvalue=((NumericSequenceData)sequencedata).getMaxAllowedValue();
            minvalue=((NumericSequenceData)sequencedata).getMinAllowedValue();
            baselinevalue=((NumericSequenceData)sequencedata).getBaselineValue();
            if (baselinevalue>minvalue) {
                float range=Math.abs(maxvalue-minvalue);
                aboveMaximum=Math.abs(maxvalue-baselinevalue);
                belowMaximum=Math.abs(baselinevalue-minvalue);
                directionalMaximum=Math.max(aboveMaximum, belowMaximum);
                if (settings.getGraphType(featureName)==VisualizationSettings.BAR_GRAPH) 
                     aboveHeight=(int)Math.ceil((aboveMaximum/range)*(height-1)); 
                else aboveHeight=(int)Math.ceil((height-1)/2);
                belowHeight=height-(aboveHeight+1); 
                drawBaseLine=true;
            }
        }
        for (int i=0;i<bases;i++) { // draw visible region
                  int pos=start+i; // relative position within sequence
                  //if (orientation==FeatureSequenceData.REVERSE) pos=start+(bases-1)-i; // <= I use (-1)*scale instead in Affine transform
                  Object value=sequencedata.getValueAtRelativePosition(pos);
                  if (value==null) { // this probably should not happen...
                      graphics.setColor(Color.GRAY);
                      graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                  }
                  else if (value instanceof Character) { // DNA track
                      char base=((Character)value).charValue();
                      if (orientation==FeatureSequenceData.REVERSE) { base=reverseBase(base); }
                      graphics.setColor(settings.getBaseColor(base));
                      graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                  }
                  else if (value instanceof Float) { // bargraph or gradient. Instead of drawing a single solid line, finetune the "tip" of the bar with a gradient color to get more precision
                     if (settings.getGraphType(featureName)==VisualizationSettings.BAR_GRAPH) {
                        if (drawBaseLine) {
                           graphics.setColor(background);
                           graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                           graphics.setColor(baselineColor);
                           graphics.drawLine(graphicsXoffset+i, graphicsYoffset+aboveHeight, graphicsXoffset+i, graphicsYoffset+aboveHeight);  
                           float val=((Float)value).floatValue();
                           if (val==baselinevalue) continue;
                           else if (val>baselinevalue) {
                               float relativeValue=Math.abs(val/aboveMaximum);
                               int solidbarheight=(int)Math.floor(aboveHeight*relativeValue); // round down to nearest pixel
                               float overshoot=(aboveHeight*relativeValue)-solidbarheight; // amount overshooting the nearest pixel. Use this value to get a gradient color and place one pixel with this color on top of the bar (simulates higher precision)
                               graphics.setColor(gradient.getColor(overshoot));
                               if (solidbarheight<aboveHeight) graphics.drawLine(graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight+1), graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight+1));                          
                               graphics.setColor(foreground);                       
                               if (solidbarheight>0) graphics.drawLine(graphicsXoffset+i, graphicsYoffset+aboveHeight-(solidbarheight), graphicsXoffset+i, graphicsYoffset+aboveHeight-1);  
                          } else {
                               float relativeValue=Math.abs(val/belowMaximum);
                               int solidbarheight=(int)Math.floor(belowHeight*relativeValue); // round down to nearest pixel
                               float overshoot=(belowHeight*relativeValue)-solidbarheight; // amount overshooting the nearest pixel. Use this value to get a gradient color and place one pixel with this color on top of the bar (simulates higher precision)
                               graphics.setColor(gradient.getColor(overshoot));
                               if (solidbarheight<belowHeight) graphics.drawLine(graphicsXoffset+i, (graphicsYoffset+aboveHeight+1)+solidbarheight, graphicsXoffset+i, (graphicsYoffset+aboveHeight+1)+solidbarheight);                          
                               graphics.setColor(foreground);                                            
                               if (solidbarheight>0) graphics.drawLine(graphicsXoffset+i, (graphicsYoffset+aboveHeight+1), graphicsXoffset+i, (graphicsYoffset+aboveHeight)+solidbarheight);                                 
                          }
                        } else { // Bar graph without baseline
                           float relativeValue=((Float)value).floatValue()/maxvalue; 
                           int solidbarheight=(int)Math.floor(height*relativeValue); // round down to nearest pixel
                           float overshoot=(height*relativeValue)-solidbarheight; // amount overshooting the nearest pixel. Use this value to get a gradient color and place one pixel with this color on top of the bar (simulates higher precision)
                           graphics.setColor(background);
                           graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                           graphics.setColor(gradient.getColor(overshoot));
                           if (solidbarheight<height) graphics.drawLine(graphicsXoffset+i, graphicsYoffset+height-(solidbarheight+1), graphicsXoffset+i, graphicsYoffset+height-(solidbarheight+1));                          
                           graphics.setColor(foreground);                       
                           if (solidbarheight>0) graphics.drawLine(graphicsXoffset+i, graphicsYoffset+height-(solidbarheight), graphicsXoffset+i, graphicsYoffset+height-1);                            
                        }                     
                     } else { // Gradient graph
                        if (drawBaseLine) {
                           graphics.setColor(background);
                           graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                           graphics.setColor(baselineColor);
                           graphics.drawLine(graphicsXoffset+i, graphicsYoffset+aboveHeight, graphicsXoffset+i, graphicsYoffset+aboveHeight);  
                           float val=((Float)value).floatValue();
                           if (val==baselinevalue) continue;
                           else if (val>baselinevalue) {
                               float relativeValue=val/directionalMaximum;
                               graphics.setColor(gradient.getColor(relativeValue)); 
                               graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+aboveHeight-1);                                                           
                           } else {
                               float relativeValue=Math.abs(val/directionalMaximum);
                               graphics.setColor(gradient.getColor(relativeValue)); 
                               graphics.drawLine(graphicsXoffset+i, graphicsYoffset+aboveHeight+1, graphicsXoffset+i, graphicsYoffset+height-1);                                                             
                           }
                        } else { // gradient graph without baseline
                           float relativeValue=((Float)value).floatValue()/(maxvalue-baselinevalue);
                           graphics.setColor(gradient.getColor(relativeValue)); 
                           graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);                              
                        }       
                     }
                  }
                  else if (value instanceof ArrayList) { // region data
                      if (((ArrayList)value).size()>0) graphics.setColor(foreground);
                      else graphics.setColor(background);
                      graphics.drawLine(graphicsXoffset+i, graphicsYoffset, graphicsXoffset+i, graphicsYoffset+height-1);  
                  }
              }          
    }    
}
