/*
 * ExternalProgramsDialog.java
 *
 * Created on 31. august 2009, 12:42
 */

package priorseditor.gui;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;
import priorseditor.engine.SystemError;
import priorseditor.external.ExternalProgram;

/**
 *
 * @author  kjetikl
 */
public class ExternalProgramsDialog extends javax.swing.JDialog {
    private static final String TABLECOLUMN_NAME="Name";
    private static final String TABLECOLUMN_TYPE="Type";
    private static final String TABLECOLUMN_LOCATION="Location";
    
    
    private PriorsEditorGUI gui;
    private DefaultTableModel externalProgramsModel;
    private JTable externalProgramsTable;
    private Collection<ExternalProgram> externalProgramsList=null;
    
    /** Creates new form ExternalProgramsDialog */
    public ExternalProgramsDialog(PriorsEditorGUI GUI) {
        super(GUI.getFrame(), true);
        this.gui=GUI;
        initComponents();
        getRootPane().setDefaultButton(okButton);
        externalProgramsModel=new DefaultTableModel(new String[]{TABLECOLUMN_NAME,TABLECOLUMN_TYPE,TABLECOLUMN_LOCATION},0);
        externalProgramsTable=new JTable(externalProgramsModel) {
            @Override
            public boolean isCellEditable(int row, int col) {return false;}
        };
        externalProgramsList=gui.getEngine().getAllExternalPrograms();
         for (ExternalProgram program:externalProgramsList) {
            String type=program.getProgramClass();
            String location=program.getServiceType();
            if (!location.equalsIgnoreCase("bundled")) location+=" = "+program.getLocation();
            Object[] values=new Object[]{program.getName(),type,location};
            externalProgramsModel.addRow(values);
        }       
        externalProgramsTable.setFillsViewportHeight(true);
        externalProgramsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        externalProgramsTable.setRowSelectionAllowed(true);
        externalProgramsTable.getTableHeader().setReorderingAllowed(false);             
        externalProgramsTable.setRowHeight(18);
        scrollPane.setViewportView(externalProgramsTable);  
        externalProgramsTable.getColumn(TABLECOLUMN_TYPE).setPreferredWidth(50);

        externalProgramsTable.setAutoCreateRowSorter(true);
        externalProgramsTable.getRowSorter().toggleSortOrder(externalProgramsTable.getColumn(TABLECOLUMN_NAME).getModelIndex());
        externalProgramsTable.getRowSorter().toggleSortOrder(externalProgramsTable.getColumn(TABLECOLUMN_TYPE).getModelIndex());
        externalProgramsTable.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
               super.mouseClicked(e);
               if (e.getClickCount()<2) return;
               int row=externalProgramsTable.getSelectedRow();
               String programname=(String)externalProgramsTable.getValueAt(row, 0);
               ExternalProgram program=gui.getEngine().getExternalProgram(programname);
               showProgramInfo(program);
            }           
        });
        this.setPreferredSize(new Dimension(600,400));
        pack();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        addProgramButton = new javax.swing.JButton();
        addProgramFromRepositoryButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        scrollPane = new javax.swing.JScrollPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(ExternalProgramsDialog.class);
        setTitle(resourceMap.getString("Form.title")); // NOI18N
        setMinimumSize(new java.awt.Dimension(200, 150));
        setName("Form"); // NOI18N

        jPanel1.setMaximumSize(new java.awt.Dimension(32767, 36));
        jPanel1.setMinimumSize(new java.awt.Dimension(100, 20));
        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setPreferredSize(new java.awt.Dimension(100, 20));

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 530, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 20, Short.MAX_VALUE)
        );

        getContentPane().add(jPanel1, java.awt.BorderLayout.PAGE_START);

        jPanel2.setMaximumSize(new java.awt.Dimension(32767, 46));
        jPanel2.setMinimumSize(new java.awt.Dimension(100, 46));
        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setPreferredSize(new java.awt.Dimension(100, 46));

        okButton.setText(resourceMap.getString("okButton.text")); // NOI18N
        okButton.setMaximumSize(new java.awt.Dimension(68, 27));
        okButton.setMinimumSize(new java.awt.Dimension(68, 27));
        okButton.setName("okButton"); // NOI18N
        okButton.setPreferredSize(new java.awt.Dimension(68, 27));
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonPressed(evt);
            }
        });

        addProgramButton.setText(resourceMap.getString("addProgramButton.text")); // NOI18N
        addProgramButton.setMaximumSize(new java.awt.Dimension(176, 27));
        addProgramButton.setMinimumSize(new java.awt.Dimension(176, 27));
        addProgramButton.setName("addProgramButton"); // NOI18N
        addProgramButton.setPreferredSize(new java.awt.Dimension(176, 27));
        addProgramButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addProgramFromFile(evt);
            }
        });

        addProgramFromRepositoryButton.setText(resourceMap.getString("addProgramFromRepositoryButton.text")); // NOI18N
        addProgramFromRepositoryButton.setMaximumSize(new java.awt.Dimension(210, 27));
        addProgramFromRepositoryButton.setMinimumSize(new java.awt.Dimension(210, 27));
        addProgramFromRepositoryButton.setName("addProgramFromRepositoryButton"); // NOI18N
        addProgramFromRepositoryButton.setPreferredSize(new java.awt.Dimension(210, 27));
        addProgramFromRepositoryButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addProgramFromRepository(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(addProgramButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(addProgramFromRepositoryButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 46, Short.MAX_VALUE)
                .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(addProgramButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(addProgramFromRepositoryButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        getContentPane().add(jPanel2, java.awt.BorderLayout.PAGE_END);

        jPanel3.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 10, 1, 10));
        jPanel3.setName("jPanel3"); // NOI18N
        jPanel3.setLayout(new java.awt.BorderLayout());

        scrollPane.setName("scrollPane"); // NOI18N
        jPanel3.add(scrollPane, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel3, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void okButtonPressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonPressed
    this.setVisible(false);
    
}//GEN-LAST:event_okButtonPressed

private void addProgramFromFile(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addProgramFromFile
    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    final JFileChooser fc = new JFileChooser(gui.getLastUsedDirectory());
    fc.setDialogTitle("Import external program definition file");
    fc.addChoosableFileFilter(new FileNameExtensionFilter("XML-files", "XML","xml"));
    int returnValue=fc.showOpenDialog(this);
    setCursor(Cursor.getDefaultCursor());
    if (returnValue!=JFileChooser.APPROVE_OPTION) return; // user cancelled
    File file=fc.getSelectedFile();
    ExternalProgram program=null;
    try {
        program = ExternalProgram.initializeExternalProgramFromFile(file,true);
    } catch (SystemError ex) {
        JOptionPane.showMessageDialog(rootPane, ex.getMessage(), "Import Error", JOptionPane.ERROR_MESSAGE);
        return;
    }
    gui.setLastUsedDirectory(file.getParentFile());
    // the following line should not happen (but just in case...)
    if (program==null) {
        JOptionPane.showMessageDialog(rootPane, "Unable to load program definition file", "Import Error", JOptionPane.ERROR_MESSAGE);
        return;
    }
    addProgram(program);
}//GEN-LAST:event_addProgramFromFile

private void addProgramFromRepository(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addProgramFromRepository
    // TODO add your handling code here:
    ExternalProgramAddFromRepositoryDialog repositoryDialog=new ExternalProgramAddFromRepositoryDialog(gui);
    repositoryDialog.setLocation(gui.getFrame().getWidth()/2-repositoryDialog.getWidth()/2, gui.getFrame().getHeight()/2-repositoryDialog.getHeight()/2);
    repositoryDialog.setVisible(true);
    String programName=repositoryDialog.getProgramName();
    String urlString=repositoryDialog.getURL();
    repositoryDialog.dispose();
    if (urlString==null) return;
    ExternalProgram program=null;
    try {
        URL url=new URL(urlString);
        InputStream stream=url.openStream();
        program = ExternalProgram.initializeExternalProgramFromStream(stream,programName+".xml", true);
    } catch (Exception ex) {
        JOptionPane.showMessageDialog(rootPane, ex.getMessage(), "Import Error", JOptionPane.ERROR_MESSAGE);
        return;
    }
    // the following line should not happen (but just in case...)
    if (program==null) {
        JOptionPane.showMessageDialog(rootPane, "Unable to load program definition file", "Import Error", JOptionPane.ERROR_MESSAGE);
        return;
    }
    addProgram(program);
}//GEN-LAST:event_addProgramFromRepository



/** Adds a new program to the list. This method handles the common code for addProgramFromFile() and addProgramFromRepository()
 * @param program The program to be installed (already initiated from file)
 */
public void addProgram(ExternalProgram program) {
        String programName=program.getName();
        String license=(String)program.getProperty("license");
        if (license!=null && !license.trim().isEmpty()) {
            license="<html><body>"+license+"</body></html>";
            LicenseAgreementDialog dialog=new LicenseAgreementDialog(gui.getFrame(), programName+" license", license);
            dialog.setVisible(true);
            boolean isAccepted=dialog.isLicenseAccepted();
            dialog.dispose();
            if (!isAccepted) {
                JOptionPane.showMessageDialog(rootPane, "Installation aborted because license was not accepted", "Installation Aborted", JOptionPane.WARNING_MESSAGE);
                return;
             }
        }
        ExternalProgram removedProgram=null;
        if (gui.getEngine().getExternalProgram(programName)!=null) {
            int answer=JOptionPane.showConfirmDialog(rootPane, "There is already a program registered with the name '"+programName+"'.\nWould you like to replace the existing program?", "Warning", JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
            if (answer==JOptionPane.NO_OPTION) return;
            else {
                for (int i=0;i<externalProgramsModel.getRowCount();i++) {
                   if (externalProgramsModel.getValueAt(i,0).equals(programName)) {
                       externalProgramsModel.removeRow(i);
                       removedProgram=gui.getEngine().removeExternalProgram(programName);
                       break;
                   }
                }
            }
        }
        program.setEngine(gui.getEngine());
        // --- check that the location is known. If not prompt the user ---
        if (program.getServiceType().equalsIgnoreCase("local")) {
            boolean promptForLocation=true;
            if (program.getLocation()!=null && !program.getLocation().isEmpty()) {
                File executableFile=new File(program.getLocation());
                if (executableFile.exists()) promptForLocation=false;
            }
            if (promptForLocation) {
                String location=null;
                PromptLocationDialog promptLocationDialog=new PromptLocationDialog(gui, program);
                int height=promptLocationDialog.getHeight();
                int width=promptLocationDialog.getWidth();
                java.awt.Dimension size=gui.getFrame().getSize();
                promptLocationDialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));
                promptLocationDialog.setVisible(true);
                location=promptLocationDialog.getExecutableLocation();
                promptLocationDialog.dispose();
                if (location==null) {
                    if (removedProgram!=null) installProgramInListAndEngine(removedProgram); // put it back
                    return; // cancel 'Add program'
                }
                else program.setLocation(location);
            } // end prompt for location
        } // serviceType == 'local'
        // first remove configuration file of the program that is being replaced (if any)
        if (removedProgram!=null && removedProgram.getConfigurationFile()!=null) {
             File oldfile=new File(removedProgram.getConfigurationFile());
             oldfile.delete();
        }
        File externalfile=new File(gui.getEngine().getWorkDirectory()+File.separator+"external"+File.separator+program.getName()+".xml");
        try {
            //gui.logMessage("saving to: "+externalfile.getAbsolutePath());
            program.saveConfigurationToFile(externalfile);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(rootPane, e.getClass().getSimpleName()+":"+e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            if (e instanceof NullPointerException) e.printStackTrace(System.err);
            return;
        }
        program.setConfigurationFile(externalfile.getAbsolutePath());
        installProgramInListAndEngine(program);
        JOptionPane.showMessageDialog(rootPane,"Program '"+program.getName()+"' installed successfully", "Installation complete", JOptionPane.INFORMATION_MESSAGE);

}



private void installProgramInListAndEngine(ExternalProgram program) {
        String type=program.getProgramClass();
        String locationDescription=program.getServiceType();
        if (!locationDescription.equalsIgnoreCase("bundled")) locationDescription+=" = "+program.getLocation();
        Object[] values=new Object[]{program.getName(),type,locationDescription};
        externalProgramsModel.addRow(values);  
        program.clearProperties();
        gui.getEngine().registerExternalProgram(program);     
}

private void showProgramInfo(ExternalProgram externalprogram) {
    gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    String externaldir = gui.getEngine().getWorkDirectory()+java.io.File.separator+"external"+java.io.File.separator;
    ExternalProgram program=externalprogram;
    File externalprogramfile=new File(externaldir+externalprogram.getName()+".xml");
    if (!externalprogramfile.exists()) externalprogramfile=new File(externaldir+externalprogram.getName()+".XML");
    if (externalprogramfile.exists()) {
        try {
           program=ExternalProgram.initializeExternalProgramFromFile(externalprogramfile,true);
        } catch (SystemError e) {}
    }    
    String authors="";
    Object authorsObject=program.getProperty("author");
    if (authorsObject instanceof ArrayList) {
        int i=((ArrayList)authorsObject).size();
        for (Object obj:(ArrayList)authorsObject) {
            authors+=obj.toString();
            i--;
            if (i>0) authors+=", "; 
        }
    } else if (authorsObject!=null)  authors=authorsObject.toString();
    String citations="";
    Object citationsObject=program.getProperty("citation");
    if (citationsObject instanceof ArrayList) {
        int i=((ArrayList)citationsObject).size();
        for (Object obj:(ArrayList)citationsObject) {
            citations+="&nbsp;&nbsp;&nbsp;&nbsp;"+obj.toString();
            i--;
            if (i>0) citations+="<br><br>"; 
        }
    } else if (citationsObject!=null) citations=citationsObject.toString();
    String programClass=program.getProgramClass();
    if (programClass.equals("MotifDiscovery")) programClass="Motif discovery (<i>de novo</i>)";
    else if (programClass.equals("MotifScanning")) programClass="Motif scanning"; 
    String service=program.getServiceType();
    if (service.equals("local")) service="Locally installed program";
    else if (service.equals("soap")) service="Web Service";
    Object contact=program.getProperty("contact");
    Object homepage=program.getProperty("homepage");
    if (program.getLocation()!=null && !service.equals("bundled")) service=service+"<br><nobr>"+program.getLocation()+"</nobr>";
    String document="<html><body><h1>"+program.getName()+"</h1><br>"
                    +"<table border=0><tr><td><b>Class: </b></td><td>"+programClass+"</td></tr>"
                    +"<tr><td valign=top><b>Service: </b></td><td valign=top>"+service+"</td></tr>";
    if (!authors.isEmpty()) {
                    document+="<tr><td valign=top><b>Authors: </b></td><td valign=top>"+authors+"</td></tr>";
    }
    if (contact!=null) document+="<tr><td valign=top><b>Contact: </b></td><td valign=top>"+contact+"</td></tr>";
    if (homepage!=null) document+="<tr><td valign=top><b>Homepage: </b></td><td valign=top><nobr><a href=\""+homepage+"\">"+homepage+"</a></nobr></td></tr>";
    document+="</table>";
    String description=program.getDescription();
    if (description!=null && !description.isEmpty()) document+="<br><hr><br>"+description+"<br><br>";
    if (!citations.isEmpty()) {
       document+="<hr><br>If you use "+program.getName()+" in your own research, please cite one of the following publications:<br><br>"+citations+"<br>";   
    }    
    String license=(String)program.getProperty("license");
    if (license!=null && !license.trim().isEmpty()) document+="<br><hr><br>"+license;
    document+="<br><br></body></html>";
    InfoDialog dialog=new InfoDialog(gui.getFrame(), "External Program", document);
    gui.getFrame().setCursor(Cursor.getDefaultCursor());
    dialog.setVisible(true);
    dialog.dispose();   
}



    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addProgramButton;
    private javax.swing.JButton addProgramFromRepositoryButton;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JButton okButton;
    private javax.swing.JScrollPane scrollPane;
    // End of variables declaration//GEN-END:variables
         
                
}
