
package priorseditor.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JPopupMenu;
import javax.swing.JList;
import java.awt.Font;
import java.awt.Color;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseWheelEvent;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListDataListener;
import org.jdesktop.application.ResourceMap;

import priorseditor.engine.data.*;

/**
 * This class encapsulates the Features dataset panel
 * 
 * @author kjetikl
 */
public class FeaturesPanel extends JPanel {
        private JLabel featuresPanelLabel;
        private JScrollPane featuresetListScrollPane;
        private JList featuresetList;
        private JButton addButton;
        private PriorsEditorGUI gui;
        private VisualizationSettings visualizationSettings;
        private Class filter=priorseditor.engine.data.FeatureDataset.class; // 

        
  public FeaturesPanel(PriorsEditorGUI gui) {
      this.gui=gui;
      visualizationSettings=gui.getVisualizationSettings();
      initComponents();
  } 
  
  public JList getFeaturesList() {
      return featuresetList;
  }  
  
  
  private void initComponents() {
        ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(FeaturesPanel.class);

        featuresPanelLabel=new JLabel();
        setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        setName("FeaturesPanel"); // NOI18N
        setLayout(new BorderLayout());

        featuresPanelLabel.setText(resourceMap.getString("FeaturesLabel.text")); // NOI18N
        featuresPanelLabel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 3, 1, 1));
        featuresPanelLabel.setName("FeaturesLabel"); // NOI18N
        
        addButton=new JButton(new MiscIcons(MiscIcons.ADD_DATA));
        addButton.setName("addButton");
        addButton.setToolTipText(resourceMap.getString("addButton.toolTipText")); // NOI18N
        addButton.setFont(new Font(Font.SANS_SERIF,Font.PLAIN,12));
        Dimension dim=new Dimension(36,22);
        addButton.setMaximumSize(dim);
        addButton.setPreferredSize(dim);
        addButton.setMinimumSize(dim);
        addButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                addFeatureDataset();
            }

        });
    
        JPanel labelpanel=new JPanel();
        labelpanel.setLayout(new BorderLayout());
        labelpanel.add(featuresPanelLabel,BorderLayout.CENTER);
        labelpanel.add(addButton,BorderLayout.EAST);
        
        add(labelpanel, java.awt.BorderLayout.PAGE_START);
        
        featuresetListScrollPane = new JScrollPane();
        featuresetListScrollPane.setName("FeaturesetListScrollPane"); // NOI18N

        featuresetList=new JList();
        featuresetList.setBackground(resourceMap.getColor("featuresetList.background")); // NOI18N
        featuresetList.setModel(new FeaturesPanelListModel(gui.getEngine(),filter,visualizationSettings));
        featuresetList.setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        featuresetList.setCellRenderer(new DataPanelCellRenderer(gui.getVisualizationSettings()));
        featuresetList.setDragEnabled(true);
        featuresetList.setDropMode(javax.swing.DropMode.ON_OR_INSERT);
        featuresetList.setName("featuresetList"); // NOI18N
        featuresetList.addMouseWheelListener(new java.awt.event.MouseWheelListener() {
            public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) {
                mouseWheelMovedInFeaturesPanel(evt);
            }
        });
        featuresetList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                mousePressedInFeaturesPanel(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                mouseReleasedInFeaturesPanel(evt);
            }
        });
        featuresetList.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                //gui.logMessage("key typed "+e.getKeyCode());              
                int[] selected=featuresetList.getSelectedIndices();
                if (selected.length==0) return;
                if (e.getKeyCode()==KeyEvent.VK_SPACE) {
                    if (selected.length==1) toggleVisibility((FeatureDataset)featuresetList.getSelectedValue());
                    else {
                        int visible=0;
                        FeaturesPanelListModel model=(FeaturesPanelListModel)featuresetList.getModel();
                        String[] names=new String[selected.length];
                        for (int i=0;i<selected.length;i++) {
                            names[i]=((FeatureDataset)model.getElementAt(selected[i])).getName();
                            if (visualizationSettings.isVisible(names[i])) visible++;
                        }
                        boolean show=true;
                        if (visible==selected.length) show=false; // all selected datatracks are currently visible so hide all
                        visualizationSettings.setVisible(names, show);
                    }
                }
                else if (e.getKeyCode()==KeyEvent.VK_DELETE) {
                    String[] names=new String[selected.length];
                    FeaturesPanelListModel model=(FeaturesPanelListModel)featuresetList.getModel();
                    for (int i=0;i<selected.length;i++) {
                        names[i]=((FeatureDataset)model.getElementAt(selected[i])).getName();
                        //gui.logMessage("Selected: "+names[i]+" of "+selected.length);
                    }
                    gui.deleteDataItems(names); // queues and undoable event
                }
                else if (e.getKeyCode()==KeyEvent.VK_PLUS || e.getKeyCode()==KeyEvent.VK_ADD) {
                    MouseWheelEvent mwe=new MouseWheelEvent(getFeaturesList(), e.getID(), e.getWhen(), e.getModifiers(), 0, 0, 0, false, MouseWheelEvent.WHEEL_UNIT_SCROLL, 1, 1);
                    mouseWheelMovedInFeaturesPanel(mwe);
                }                
                else if (e.getKeyCode()==KeyEvent.VK_MINUS || e.getKeyCode()==KeyEvent.VK_SUBTRACT) {
                    MouseWheelEvent mwe=new MouseWheelEvent(getFeaturesList(), e.getID(), e.getWhen(), e.getModifiers(), 0, 0, 0, false, MouseWheelEvent.WHEEL_UNIT_SCROLL, 1, -1);
                    mouseWheelMovedInFeaturesPanel(mwe);
                }                
            }
        });
        featuresetList.setTransferHandler(new FeaturesPanelTransferHandler(featuresetList,gui));
        featuresetList.getModel().addListDataListener(gui.getVisualizationSettings());
        featuresetListScrollPane.setViewportView(featuresetList);        
        add(featuresetListScrollPane, java.awt.BorderLayout.CENTER);
  }


private void mousePressedInFeaturesPanel(java.awt.event.MouseEvent evt) {                                             
    int x=evt.getX(); 
    int y=evt.getY();
    JList list=(JList)evt.getComponent();    
    int index=getElementIndexAtPointInList(list,x,y);
    if (index==-1) {list.clearSelection();return;} 
    if (evt.isPopupTrigger()) {
        JPopupMenu popup=FeaturesPanelContextMenu.getInstance(list,index,gui);
        if (popup!=null) popup.show(evt.getComponent(), x,y);
    }  
}                                            

private void mouseReleasedInFeaturesPanel(java.awt.event.MouseEvent evt) {                                              
    int x=evt.getX(); 
    int y=evt.getY();
    JList list=(JList)evt.getComponent();
    int index=getElementIndexAtPointInList(list,x,y);
    if (index==-1) {list.clearSelection();return;} 
    if (evt.isPopupTrigger()) {
        JPopupMenu popup=FeaturesPanelContextMenu.getInstance((JList)evt.getComponent(),index,gui);
        if (popup!=null) popup.show(evt.getComponent(), x,y);
    }
}                                             

private void mouseWheelMovedInFeaturesPanel(java.awt.event.MouseWheelEvent evt) {
    int x=evt.getX(); 
    int y=evt.getY();
    JList list=(JList)evt.getComponent();
    int modifiers=evt.getModifiersEx();
    if ((modifiers & (evt.SHIFT_DOWN_MASK | evt.CTRL_DOWN_MASK))==evt.SHIFT_DOWN_MASK) { // SHIFT modifier -> adjust track color
      int[] selected=list.getSelectedIndices();
      if (selected.length==0) return;
      FeatureDataset dataset=(FeatureDataset)list.getModel().getElementAt(selected[0]);  // only apply to first selected
      String datasetname=dataset.getName();
      int dir=evt.getWheelRotation();
      Color fg=visualizationSettings.getForeGroundColor(datasetname);
      Color bg=visualizationSettings.getBackGroundColor(datasetname);
      Color[] newColors;
      if (dir>=0) newColors=visualizationSettings.getNextPresetColorPair(fg, bg);
      else newColors=visualizationSettings.getPreviousPresetColorPair(fg, bg);
      visualizationSettings.setForeGroundColor(datasetname, newColors[0]);
      visualizationSettings.setBackGroundColor(datasetname, newColors[1]);
    } else if ((modifiers & (evt.SHIFT_DOWN_MASK | evt.CTRL_DOWN_MASK))==evt.CTRL_DOWN_MASK) { // CTRL modifier -> adjust sequence margin
        int margin=visualizationSettings.getSequenceMargin();
        int newmargin=margin+evt.getWheelRotation();
        visualizationSettings.setSequenceMargin(newmargin);       
    }    
    else { // no keyboard modifiers -> adjust track height       
       int wheelupdate=evt.getWheelRotation();
       int[] selected=featuresetList.getSelectedIndices();
       if (selected.length==0) return;
       FeaturesPanelListModel model=(FeaturesPanelListModel)featuresetList.getModel();
            for (int i=0;i<selected.length;i++) {
                String datasetname=((FeatureDataset)model.getElementAt(selected[i])).getName();
                int currentTrackHeight=visualizationSettings.getDataTrackHeight(datasetname);
                int newheight=currentTrackHeight+wheelupdate;
                if (newheight>VisualizationSettings.DATATRACK_MAX_HEIGHT) newheight=VisualizationSettings.DATATRACK_MAX_HEIGHT;
                else if (newheight<VisualizationSettings.DATATRACK_MIN_HEIGHT) newheight=VisualizationSettings.DATATRACK_MIN_HEIGHT;
                visualizationSettings.setDataTrackHeight(datasetname, newheight,(i==selected.length-1));
            }       
    }
  }

 /**
  * Given a reference to a JList and a point (x,y), this function returns the index 
  * of the element corresponding to that point (between 0 and model.getSize()-1).
  * If the coordinate does not correspond to an element in the list, the method returns the value -1;
  * @param list The JList (fitted with a DefaultListModel)
  * @param x
  * @param y
  * @return
  */
 private int getElementIndexAtPointInList(JList list, int x, int y) {
         int index=list.locationToIndex(new java.awt.Point(x,y));
         if (index>=0) {
            javax.swing.DefaultListModel model=(javax.swing.DefaultListModel)list.getModel();
            java.awt.Rectangle bounds = list.getCellBounds(0, model.getSize()-1);
            if (y>bounds.getHeight() || y<0 || x>bounds.getWidth() || x<0) index=-1; // 
         }
         return index;
    }

 /**
  * This method is called when the "add feature dataset" button (next to the panel label) is pressed
  */
 private void addFeatureDataset() {
     //gui.debugMessage("Add Feature dataset (this should trigger a 'load feature' dialog or, if there are no sequences, the 'define sequences' dialog)");
     gui.showDatatrackDialog();
 }
 
 /**
  * This method toggles the visibility of a selected feature datatrack
  * (The method is called when the user presses the spacebar while a feature dataset is selected and has focus)
  */
 private void toggleVisibility(FeatureDataset dataset) {
     String datasetname=dataset.getName();
     boolean visible=visualizationSettings.isVisible(datasetname);
     //gui.logMessage("toggle visibility for "+datasetname+" old="+visible+" new="+(!visible));
     visualizationSettings.setVisible(datasetname, !visible);
 }
 
 
 
}
