/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.FeatureDataset;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JList;
import java.awt.event.ActionListener;
import javax.swing.DefaultListModel;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Collections;
import java.util.HashMap;
import javax.swing.JColorChooser;
import javax.swing.JSeparator;
import javax.swing.JMenu;
import priorseditor.engine.data.DNASequenceData;
import priorseditor.engine.data.DNASequenceDataset;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.RegionSequenceData;
import priorseditor.engine.data.NumericDataset;



/**
 *
 * @author kjetikl
 */
public class FeaturesPanelContextMenu extends JPopupMenu  {
    private static String SHOW_DATATRACK="Show Track";
    private static String HIDE_DATATRACK="Hide Track";
    private static String RENAME_DATA ="Rename";
    private static String DELETE_DATA ="Delete";
    private static String CONVERT_TO_MOTIF_TRACK="Convert to Motif Track";
    private static String DISPLAY_SETTINGS_DIALOG="Change Display Settings";
    private static String SET_COLOR="Set Color";
    private static String SET_RANGE ="Set Range";
    private static String OPERATION_SUBMENU_HEADER="Perform Operation";
    
    private PriorsEditorEngine engine;
    private FeatureDataset dataset;
    private FeatureDataset[] datasets;
    private PriorsEditorGUI gui;
    private JMenu operationsMenu;
    private JList list=null;
    
    public static FeaturesPanelContextMenu getInstance(JList list, int index, PriorsEditorGUI gui) {
         if (index==-1) return null;
         DefaultListModel model=(DefaultListModel)list.getModel();
         Object[] values=list.getSelectedValues();
         if (values==null || values.length<=1) return new FeaturesPanelContextMenu(list,(FeatureDataset)model.getElementAt(index),gui);
         else {
             FeatureDataset[] newsets=new FeatureDataset[values.length];
             for (int i=0;i<newsets.length;i++) newsets[i]=(FeatureDataset)values[i];
             return new FeaturesPanelContextMenu(list,newsets,gui);
         }
         
    }
    
    
    private FeaturesPanelContextMenu(JList list, FeatureDataset dataset, PriorsEditorGUI gui) {
            this.engine=gui.getEngine();
            this.dataset=dataset;
            this.datasets=null;
            this.gui=gui;
            this.list=list;
            DisplayMenuItemListener menuItemListener=new DisplayMenuItemListener();
            ArrayList<Operation> operationslist=engine.getAllOperations();
            Collections.sort(operationslist, new NameComparator());
            // HeaderMenuItem header=new HeaderMenuItem(dataset.getName(), new java.awt.Color(255,255,255), new java.awt.Color(150,150,255));
            // this.add(header);
            JMenuItem displayoptions=new JMenuItem(DISPLAY_SETTINGS_DIALOG);
            JMenuItem renameDataItem=new JMenuItem(RENAME_DATA+" \""+dataset.getName()+"\"");
            JMenuItem deleteDataItem=new JMenuItem(DELETE_DATA+" \""+dataset.getName()+"\"");

            String hideorshowString=(gui.getVisualizationSettings().isVisible(dataset.getName()))?HIDE_DATATRACK:SHOW_DATATRACK;
            JMenuItem showorhide=new JMenuItem(hideorshowString);
            showorhide.setSelected(true);
            displayoptions.addActionListener(menuItemListener);
            showorhide.addActionListener(menuItemListener);
            renameDataItem.addActionListener(menuItemListener);
            deleteDataItem.addActionListener(menuItemListener);
            if (!gui.getScheduler().isIdle()) {
                deleteDataItem.setEnabled(false);
                renameDataItem.setEnabled(false);
            }
          
            this.add(displayoptions);
            this.add(showorhide);
            if (dataset instanceof NumericDataset) {
               JMenuItem setRangeItem=new JMenuItem(SET_RANGE); 
               setRangeItem.addActionListener(menuItemListener);
               this.add(setRangeItem);
            }
            if (dataset instanceof RegionDataset && !((RegionDataset)dataset).isMotifTrack()) {
                JMenuItem convertToMotifTrackItem=new JMenuItem(CONVERT_TO_MOTIF_TRACK);
                convertToMotifTrackItem.addActionListener(menuItemListener);
                this.add(convertToMotifTrackItem);
            }              
            this.add(renameDataItem);
            this.add(deleteDataItem);               
            this.add(new JSeparator());
            OperationsMenuListener menulistener=new OperationsMenuListener();
            operationsMenu=new JMenu(OPERATION_SUBMENU_HEADER);
            for (Operation operation:operationslist) {
                if (operation.canUseAsSource(dataset)) {
                  JMenuItem menuitem=new JMenuItem(operation.getName());
                  menuitem.addActionListener(menulistener);
                  operationsMenu.add(menuitem);
                }
            }
            this.add(operationsMenu);
        }
        

    private FeaturesPanelContextMenu(JList list,FeatureDataset[] datasets, PriorsEditorGUI gui) {
            this.engine=gui.getEngine();
            this.datasets=datasets;
            this.dataset=null;
            this.gui=gui;
            this.list=list;
            int countRegionDatasets=0;
            int countNumericDatasets=0;
            for (FeatureDataset data:datasets) {
                     if (data instanceof RegionDataset) countRegionDatasets++;
                else if (data instanceof NumericDataset) countNumericDatasets++;
            }
            DisplayMenuItemListener menuItemListener=new DisplayMenuItemListener();
            ArrayList<Operation> operationslist=engine.getAllOperations();
            Collections.sort(operationslist, new NameComparator());
            // HeaderMenuItem header=new HeaderMenuItem(dataset.getName(), new java.awt.Color(255,255,255), new java.awt.Color(150,150,255));
            // this.add(header);
            JMenuItem setColor=new JMenuItem(SET_COLOR);
            JMenuItem deleteDataItems=new JMenuItem(DELETE_DATA);

            JMenuItem showDataitems=new JMenuItem(SHOW_DATATRACK+"s");
            JMenuItem hideDataitems=new JMenuItem(HIDE_DATATRACK+"s");
            setColor.addActionListener(menuItemListener);
            showDataitems.addActionListener(menuItemListener);
            hideDataitems.addActionListener(menuItemListener);
            deleteDataItems.addActionListener(menuItemListener);
            if (!gui.getScheduler().isIdle()) {
                deleteDataItems.setEnabled(false);
            }           
            this.add(setColor);
            this.add(showDataitems);
            this.add(hideDataitems);
            if (countNumericDatasets==datasets.length) { // only NumericDatasets selected
               JMenuItem setRangeItem=new JMenuItem(SET_RANGE); 
               setRangeItem.addActionListener(menuItemListener);
               this.add(setRangeItem);
            }
            if (countRegionDatasets==datasets.length) { // only RegionDatasets selected
                JMenuItem convertToMotifTrackItem=new JMenuItem(CONVERT_TO_MOTIF_TRACK);
                convertToMotifTrackItem.addActionListener(menuItemListener);
                this.add(convertToMotifTrackItem);
            }
            this.add(deleteDataItems);            
            this.add(new JSeparator());
            OperationsMenuListener menulistener=new OperationsMenuListener();
            operationsMenu=new JMenu(OPERATION_SUBMENU_HEADER);
            for (Operation operation:operationslist) {
                if (canUseAsSources(operation,datasets) && operation.canHaveMultipleInput()) {
                  JMenuItem menuitem=new JMenuItem(operation.getName());
                  menuitem.addActionListener(menulistener);
                  operationsMenu.add(menuitem);
                }
            }
            this.add(operationsMenu);
            if (operationsMenu.getItemCount()==0) operationsMenu.setEnabled(false);
        }    
    
        /**
         * An inner class that listens to popup-menu events related to the operations submenu and notifies the gui events
         */
        private class OperationsMenuListener implements ActionListener {
           public void actionPerformed(ActionEvent e) {
                Operation operation=engine.getOperation(e.getActionCommand());
                OperationTask parameters=new OperationTask(operation.getName());
                parameters.setParameter(OperationTask.OPERATION, operation);
                if (dataset!=null) parameters.setParameter(OperationTask.SOURCE, dataset);
                else if (datasets!=null && datasets.length>0) {
                    String names=datasets[0].getName();
                    for (int i=1;i<datasets.length;i++) {
                        names+=","+datasets[i].getName();
                    }
                    parameters.setParameter(OperationTask.SOURCE_NAME, names);
                }
                gui.launchOperationEditor(parameters);
           }            
        }
        
         /**
         * An inner class that listens to popup-menu events NOT related to the operations submenu and notifies the gui events
         */       
        private class DisplayMenuItemListener implements ActionListener {
           public void actionPerformed(ActionEvent e) {
               
               //gui.debugMessage("Selected "+e.getActionCommand()+" on "+datasetName);

                if (e.getActionCommand().startsWith(SHOW_DATATRACK)) {
                    if (dataset!=null) gui.getVisualizationSettings().setVisible(dataset.getName(), true);
                    else if (datasets!=null) {
                        String[] names=new String[datasets.length];
                        for (int i=0;i<datasets.length;i++) names[i]=datasets[i].getName();
                        gui.getVisualizationSettings().setVisible(names, true);
                    }
                } else if (e.getActionCommand().startsWith(HIDE_DATATRACK)) {
                    if (dataset!=null) gui.getVisualizationSettings().setVisible(dataset.getName(), false);
                    else if (datasets!=null) {
                        String[] names=new String[datasets.length];
                        for (int i=0;i<datasets.length;i++) names[i]=datasets[i].getName();
                        gui.getVisualizationSettings().setVisible(names, false);                        
                    }                    
                } else if (e.getActionCommand().startsWith(RENAME_DATA)) {
                    RenameDataDialog dialog=new RenameDataDialog(gui.getFrame(), gui, dataset.getName());
                    dialog.setLocation(100, 250);    
                    dialog.setVisible(true);
                    dialog.dispose();
                } else if (e.getActionCommand().startsWith(DELETE_DATA)) {
                    if (dataset!=null) gui.deleteDataItem(dataset.getName());
                    else if (datasets!=null) {
                        String[] names=new String[datasets.length];
                        for (int i=0;i<datasets.length;i++) names[i]=datasets[i].getName();
                        gui.deleteDataItems(names);                
                    }                      
                } else if (e.getActionCommand().equals(CONVERT_TO_MOTIF_TRACK)) {
                   gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                   convertTracksToMotifTracks();
                   gui.getFrame().setCursor(Cursor.getDefaultCursor());
                } else if (e.getActionCommand().equals(SET_RANGE)) {
                    SetRangeDialog dialog;
                    if (dataset!=null) dialog=new SetRangeDialog(gui, (NumericDataset)dataset);
                    else {
                        NumericDataset[] data=new NumericDataset[datasets.length];
                        for (int i=0;i<datasets.length;i++) data[i]=(NumericDataset)datasets[i];
                        dialog=new SetRangeDialog(gui, data);
                    }
                    dialog.setLocation(100, 250);    
                    dialog.setVisible(true);
                    dialog.dispose();
                } else if (e.getActionCommand().equals(DISPLAY_SETTINGS_DIALOG)) {
                    DatasetDisplaySettingsDialog dialog=new DatasetDisplaySettingsDialog(gui.getFrame(), dataset, gui.getVisualizationSettings());
                    dialog.setLocation(100, 250);    
                    dialog.setVisible(true);
                    dialog.dispose();
                } else if (e.getActionCommand().equals(SET_COLOR)) {
                     Color newcolor = JColorChooser.showDialog(gui.getFrame(), "Select new foreground color",null);
                     if (newcolor != null) {
                           String[] names=new String[datasets.length];
                           for (int i=0;i<datasets.length;i++) names[i]=datasets[i].getName();                         
                           gui.getVisualizationSettings().setForeGroundColor(names, newcolor);
                     }
                }
           }            
        }
        
        
        /**
         * Just an inner class to sort available operations alphabetically in the popup menu
         */
        private class NameComparator implements Comparator<Operation> {        
            public int compare(Operation o1, Operation o2) {
                return o1.getName().compareTo(o2.getName());
            }
            public boolean equals(Operation o1, Operation o2) {
                return o1.getName().equals(o2.getName());
            }           
        }
        
        
        
        private boolean canUseAsSources(Operation operation, FeatureDataset[] sources) {
            for (FeatureDataset source:sources) {
                if (!operation.canUseAsSource(source)) return false;
            }
            return true;            
        }
        

        
        
        private void convertTracksToMotifTracks() {
            HashMap<String,String> namemap=new HashMap<String,String>();
            ArrayList<Data> motifs=engine.getAllDataItemsOfType(Motif.class);
            for (Data motif:motifs) {
                namemap.put(((Motif)motif).getShortName(), motif.getName()); // note that there might be a 1-to-many mapping between shortname and motif ID but I ignore this here and just overwrite previous entries
            }
            DNASequenceDataset dna=null;
            int listsize=list.getModel().getSize();
            for (int i=0;i<listsize;i++) { // try to locate a DNASequenceDataset in the features panel list
                Object element=list.getModel().getElementAt(i);
                if (element instanceof DNASequenceDataset) {dna=(DNASequenceDataset)element;break;}
            }
            if (dataset!=null) convertSingleTrackToMotifTrack((RegionDataset)dataset,namemap,dna);
            else if (datasets!=null) {
                for (int i=0;i<datasets.length;i++) convertSingleTrackToMotifTrack((RegionDataset)datasets[i],namemap,dna);              
            }             
        }
        
        private void convertSingleTrackToMotifTrack(RegionDataset dataset, HashMap<String,String>namemap, DNASequenceDataset dna) {
              int count=0;
              for (int i=0;i<dataset.getNumberofSequences();i++) {
                  RegionSequenceData sequence=(RegionSequenceData)dataset.getSequenceByIndex(i);
                  DNASequenceData dnasequence=null;
                  if (dna!=null) dnasequence=(DNASequenceData) dna.getSequenceByName(sequence.getName());
                  for (Region region:sequence.getOriginalRegions()) {
                      String type=region.getType();
                      if (engine.dataExists(type, Motif.class)) {} // type is already a motif name
                      else if (namemap.containsKey(type)) {
                          region.setType(namemap.get(type));
                      } else continue; // this is not a recognized motif
                      Object bindingsite=region.getProperty("sequence");
                      if (bindingsite==null) {
                          char[] interval=(char[])dnasequence.getValueInGenomicInterval(region.getGenomicStart(), region.getGenomicEnd());
                          String siteseq=new String(interval);
                          if (region.getOrientation()==Region.REVERSE) siteseq=PriorsEditorEngine.reverseSequence(siteseq);
                          region.setProperty("sequence", siteseq);
                      }
                      count++;
                  } // end for each region                
              } // end for each sequence
              if (count==0) gui.logMessage("No known motifs found in "+dataset.getName()+". Conversion failed.");
              else {
                 String datasetName=dataset.getName();
                 dataset.setMotifTrack(true);
                 gui.getVisualizationSettings().setGraphType(datasetName, VisualizationSettings.REGION_MULTICOLOR_GRAPH);
                 gui.getVisualizationSettings().setVisualizeRegionStrand(datasetName,true);
                 gui.getVisualizationSettings().setVisualizeRegionScore(datasetName,true);   
                 gui.getVisualizationSettings().setDataTrackHeight(datasetName, 24, true);
              }
        }
}
