/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import priorseditor.engine.data.Data;
import priorseditor.engine.DataListener;
import javax.swing.DefaultListModel;
import priorseditor.engine.PriorsEditorEngine;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.SwingUtilities;


/**
 * This class implements a list model that backs up the lists of available
 * datasets (sequences and features) displayed in the left panels of the editor
 * 
 * The listmodel reflects the data that is currently available through the engine
 * and keeps its own ordering for visualization purposes
 * 
 * @author kjetikl
 */
public class FeaturesPanelListModel extends DefaultListModel implements DataListener {
    PriorsEditorEngine engine=null;
    Class classfilter=null;
    VisualizationSettings settings=null;
    
    public FeaturesPanelListModel(PriorsEditorEngine engine, Class classfilter, VisualizationSettings settings) {
        this.engine=engine;
        this.classfilter=classfilter;
        this.settings=settings;
        engine.addDataListener(this);
        // add all applicable dataitems currently in store (if any)
        ArrayList<Data> list=engine.getAllDataItemsOfType(classfilter);
        for (Data item:list) {dataAdded(item);}
    }
    
    public PriorsEditorEngine getEngine() {return engine;}

    /**
     * This method is called by the engine when a new data item has been added to the pool
     * @param data
     */
    public void dataAdded(final Data data) {
        if (!classfilter.isInstance(data)) return;
        if (PriorsEditorEngine.isTemporary(data)) return; // do not show temporary data items
        final FeaturesPanelListModel panel=this;
        Runnable runner=new Runnable() {
            public void run() {
                if (indexOf(data)==-1) { // the track is not present already
                    int index=settings.getTrackOrderPosition(data.getName(),panel);
                    if (index<0) addElement(data);
                    else add(index, data);
                    settings.setupMasterTrackorderList(panel);
                }
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);

    }
    
    /**
     * This method is called by the engine when a data item has been removed from the pool
     * @param data
     */
    public void dataRemoved(final Data data) {
        if (!classfilter.isInstance(data)) return;
        Runnable runner=new Runnable() {
            public void run() {
                removeElement(data);
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);        
    }
    
    /**
     * This method is called by the engine when a data item has been updated
     * @param data
     */
    public void dataUpdated(final Data data) {
        if (!classfilter.isInstance(data)) return;
        Runnable runner=new Runnable() {
            public void run() {
                int index=indexOf(data);
                fireContentsChanged(this, index, index);
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);   
    }
    
    public void dataUpdate(Data oldvalue, Data newvalue) {
        // check for data type conversions
    }    

    
    // The following two methods complete the DataListener interface, but we are not really interested in these events here
    public void dataAddedToSet(Data parent, Data child) {}
    public void dataRemovedFromSet(Data parent, Data child) {}
    
    /**
     * Relocates the specified dataset to the new index in the list
     * @param data The dataset to be relocated
     * @param index The new position index that the data should be relocated to
     */
    public void relocate(final Data data,final int index) {
        final FeaturesPanelListModel panel=this;
        Runnable runner=new Runnable() {
            public void run() {
                removeElement(data);
                add(index, data);
                settings.setupMasterTrackorderList(panel);        
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);   
    }
    
    /**
     * This method searches through the dataitems in the list to see if any
     * has a name that matches the given name. If found the data item will be
     * return, if not the method will return null.
     * @param name The name of the data item to be retrieved
     * @return Data The data item with the given name (if found) else null
     */
    public Data getDataByName(String name) {    
        for (Enumeration e=elements();e.hasMoreElements();) {
            Data data=(Data)e.nextElement();
            if (data.getName().equals(name)) return data;
        }
        return null;       
    }
    
    
}
