/*
 * 
 */

package priorseditor.gui;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Color;
import javax.swing.Icon;



/**
 * This class is an implementation of Icon that can draw different
 * simple icons that are used at various places
 * 
 * @author kjetikl
 */
public class MiscIcons implements Icon {
    public static final int CLOSE_ICON=0;
    public static final int ERROR_ICON=1;
    public static final int LEFT_ARROW=2;
    public static final int RIGHT_ARROW=3;
    public static final int TREE_GROUPS=4;
    public static final int ADD_DATA=5;


    private int type=0;
    private Color foregroundcolor=Color.black;

    
    public MiscIcons(int type) {
        this.type=type;
    }
    
    public int getIconHeight() {
        switch(type) {       
          case CLOSE_ICON: return 12;
          case LEFT_ARROW: return 13;
          case RIGHT_ARROW: return 13;
          case TREE_GROUPS: return 10;
          case ADD_DATA: return 10;
          default: return 8;
        }
    }

    public int getIconWidth() {
        switch(type) {       
          case CLOSE_ICON: return 12;
          case LEFT_ARROW: return 13;
          case RIGHT_ARROW: return 13;
          default: return 8;
        }
    }

    public int getIconType() {
        return type;
    }
    
    public void setIconType(int type) {
        this.type=type;
    }
    
    public void setForegroundColor(Color color) {
        this.foregroundcolor=color;
    }
    

    public void paintIcon(Component c, Graphics g, int x, int y) {
      switch(type) {
          case CLOSE_ICON: drawCloseIcon(g, x, y);break;          
          case ERROR_ICON: drawErrorIcon(g, x, y);break;          
          case LEFT_ARROW: drawLeftArrowIcon(g, x, y);break;          
          case RIGHT_ARROW: drawRightArrowIcon(g, x, y);break;          
          case TREE_GROUPS: drawTreeGroupsIcon(g, x, y);break;          
          case ADD_DATA: drawAddDataIcon(g, x, y);break;          
      }
    }
    
    private void drawCloseIcon(Graphics g, int x, int y) {
         g.setColor(foregroundcolor);   
         g.drawLine(x+2,y+2,x+8,y+8); // down left-right
         g.drawLine(x+2,y+3,x+7,y+8); // down left-right 
         g.drawLine(x+3,y+2,x+8,y+7); // down left-right     
         g.drawLine(x+2,y+7,x+7,y+2); // up left-right 
         g.drawLine(x+2,y+8,x+8,y+2); // up left-right 
         g.drawLine(x+3,y+8,x+8,y+3); // up left-right 
         g.drawRect(x+0, x+0, 10, 10);
    }
    
    private void drawErrorIcon(Graphics g, int x, int y) {
         g.setColor(Color.RED);   
         g.fillRect(x+1,y+1,6,6); // 
         g.drawLine(x+2,y+0,x+5,y+0); // 
         g.drawLine(x+2,y+7,x+5,y+7); //      
         g.drawLine(x+0,y+2,x+0,y+5); //
         g.drawLine(x+7,y+2,x+7,y+5); // 
         g.setColor(Color.WHITE);
         g.drawLine(x+3,y+1,x+3,y+4); // up left-right 
         g.drawLine(x+4,y+1,x+4,y+4); // up left-right 
         g.drawLine(x+3,y+6,x+4,y+6); // up left-right 

    }

    private void drawLeftArrowIcon(Graphics g, int x, int y) {
         g.setColor(foregroundcolor);   
         g.drawLine(x+0,y+6,x+0,y+6); // 
         g.drawLine(x+1,y+5,x+1,y+7); // 
         g.drawLine(x+2,y+4,x+2,y+8); // 
         g.drawLine(x+3,y+3,x+3,y+9); // 
         g.drawLine(x+4,y+2,x+4,y+10); // 
         g.drawLine(x+5,y+1,x+5,y+11); //          
         g.fillRect(x+6,y+3,7,7); //      


    }

    private void drawRightArrowIcon(Graphics g, int x, int y) {
         g.setColor(foregroundcolor);   
         g.drawLine(x+12,y+6,x+12,y+6); // 
         g.drawLine(x+11,y+5,x+11,y+7); // 
         g.drawLine(x+10,y+4,x+10,y+8); // 
         g.drawLine(x+9,y+3,x+9,y+9); // 
         g.drawLine(x+8,y+2,x+8,y+10); // 
         g.drawLine(x+7,y+1,x+7,y+11); //          
         g.fillRect(x,y+3,7,7); // 

    }

    private void drawTreeGroupsIcon(Graphics g, int x, int y) {
         g.setColor(foregroundcolor);   
         g.drawLine(x,y,x+1,y); // 
         g.drawLine(x+1,y,x+1,y+8); // 
         g.drawLine(x+1,y+3,x+3,y+3); // 
         g.drawLine(x+1,y+8,x+3,y+8); // 
         g.drawLine(x+5,y+2,x+7,y+2); // 
         g.drawLine(x+5,y+3,x+7,y+3); // 
         g.drawLine(x+5,y+4,x+7,y+4); // 
         g.drawLine(x+5,y+7,x+7,y+7); // 
         g.drawLine(x+5,y+8,x+7,y+8); // 
         g.drawLine(x+5,y+9,x+7,y+9); // 
    }

    private void drawAddDataIcon(Graphics g, int x, int y) {
         g.setColor(foregroundcolor);   
         g.drawLine(x+5,y+3,x+5,y+7); // vertical
         g.drawLine(x+3,y+5,x+7,y+5); // horisontal
    }



  
    
}
