/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.RowFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableRowSorter;
import priorseditor.engine.DataListener;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.MotifClassification;
import priorseditor.engine.data.MotifCollection;
import priorseditor.gui.prompt.Prompt_Motif;

/**
 *
 * @author kjetikl
 */
public class MotifBrowserPanel extends JPanel implements DataListener {
    private JTable manualSelectionTable;
    private DefaultTableColumnModel columnModel;
    private JTextField filterTextfield;
    private JComboBox filterOperator;
    private JLabel numberOfMotifsShowingLabel;
    private Filter filter;
    private ManualSelectionTableModel model;    
    private ManualSelectionContextMenu manualSelectionContextMenu;
    private SequenceLogo logorenderer;
    private PriorsEditorGUI gui;
    private PriorsEditorEngine engine;
    private MotifCollection motifcollection=null;
    private String initialFilterChoice="Factors";
    private String[] columnHeaders=new String[]{" ","Motif","Classification","Consensus","Factors","Halfsite","Information Content","Name","Organisms","Size","Quality","PWM match","Logo"};
    private HashMap<String,Float> PWMsimilarityCache=new HashMap<String,Float>(); // caches PWM similarity values
    private String oldPWMtarget="";



    public MotifBrowserPanel(PriorsEditorGUI gui, MotifCollection motifcollection) {
        super();
        this.gui=gui;
        this.engine=gui.getEngine();
        this.motifcollection=motifcollection;
        setupManualEntryPanel();
        if (motifcollection==null) {   
              columnModel.getColumn(0).setCellRenderer(new FeatureColorCellRenderer(gui));
//            columnModel.removeColumn(columnModel.getColumn(0)); // remove checkbox column
//            columnModel.getColumn(0).setHeaderValue("Motif");
//            columnModel.getColumn(1).setHeaderValue(initialFilterChoice); // this is a hack to get the column title correct
        }
    }
        
    @SuppressWarnings("unchecked")
    private void setupManualEntryPanel() {
        ArrayList<Data> motifs=engine.getAllDataItemsOfType(Motif.class);
        model=new ManualSelectionTableModel(motifs,columnHeaders);
        columnModel = new DefaultTableColumnModel();
        for (int i=0;i<=2;i++) {
            TableColumn col = new TableColumn(i);
            col.setHeaderValue(model.getColumnName(i));
            columnModel.addColumn(col);
        }
        TableColumn col = new TableColumn(columnHeaders.length-1); // last column: LOGO
        col.setHeaderValue(model.getColumnName(columnHeaders.length-1)); // last column: LOGO
        columnModel.addColumn(col);
        manualSelectionTable=new JTable(model,columnModel);
        manualSelectionTable.setAutoCreateRowSorter(true);
        manualSelectionTable.getTableHeader().setReorderingAllowed(false);
        columnModel.getColumn(0).setMinWidth(20);
        columnModel.getColumn(0).setMaxWidth(20);
        JScrollPane manualSelectionScrollPane=new JScrollPane(manualSelectionTable);
        this.setLayout(new BorderLayout());
        this.add(manualSelectionScrollPane,BorderLayout.CENTER);
        JPanel filterPanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        filterPanel.add(new JLabel("Filter "));
        final JComboBox filterCombobox=new JComboBox(new String[]{"Classification","Consensus","Factors","Halfsite","Information Content","Name","Organisms","Size","Quality","PWM match"});
        filterTextfield=new JTextField(12);                  
        filterOperator = new JComboBox(new String[]{"=","<>","<","<=",">=",">"});   
        numberOfMotifsShowingLabel = new JLabel();
        filterPanel.add(filterCombobox);
        filterPanel.add(filterOperator);
        filterPanel.add(filterTextfield);
        filterPanel.add(numberOfMotifsShowingLabel);
        this.add(filterPanel,BorderLayout.SOUTH);
        columnModel.getColumn(1).setCellRenderer(new CellRenderer_Motif());
        columnModel.getColumn(2).setCellRenderer(new CellRenderer_FilterColumn());
        Color [] basecolors=new Color[]{Color.GREEN,Color.BLUE,new Color(220,220,0),Color.RED};   
        int rowheight=manualSelectionTable.getRowHeight();                
        logorenderer=new SequenceLogo(basecolors,(int)(rowheight*1.15),(int)(rowheight/4f),null);
        logorenderer.setUseAntialias(gui.getVisualizationSettings().useMotifAntialiasing());
        logorenderer.setSelectedBackgroundColor(manualSelectionTable.getSelectionBackground());
        columnModel.getColumn(3).setCellRenderer(logorenderer);
        manualSelectionTable.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode()==KeyEvent.VK_SPACE && motifcollection!=null) {                   
                    int[] rows=manualSelectionTable.getSelectedRows();
                    int checkedRows=0;
                    for (int row:rows) {
                        if ((Boolean)model.getValueAt(manualSelectionTable.convertRowIndexToModel(row),0)) checkedRows++;
                    }
                    Boolean doCheck=Boolean.TRUE;
                    if (checkedRows==rows.length) doCheck=Boolean.FALSE;
                    for (int row:rows) {
                        model.setValueAt(doCheck,manualSelectionTable.convertRowIndexToModel(row),0);
                    }
                    updateCountLabelText();
                } else if (e.getKeyCode()==KeyEvent.VK_PLUS || e.getKeyCode()==KeyEvent.VK_ADD) {
                    int newheight=manualSelectionTable.getRowHeight()+1;
                    if (newheight>80) return;
                    logorenderer.setFontHeight((int)(newheight*1.15), (int)(newheight/4f));
                    manualSelectionTable.setRowHeight(newheight);
                } else if (e.getKeyCode()==KeyEvent.VK_MINUS || e.getKeyCode()==KeyEvent.VK_SUBTRACT) {
                    int newheight=manualSelectionTable.getRowHeight()-1;
                    if (newheight<8) return;
                    logorenderer.setFontHeight((int)(newheight*1.15), (int)(newheight/4f));
                    manualSelectionTable.setRowHeight(newheight); 
                }
            }
        });
        filterCombobox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                //int offset=(motifcollection==null)?1:2; // use this if first checkbox column should be hidden when motifcollection==null
                int offset=2;
                int newFilterColumn=filterCombobox.getSelectedIndex()+2;
                model.setFilterColumnIndex(newFilterColumn);
                columnModel.getColumn(offset).setHeaderValue(model.getColumnName(newFilterColumn));
                manualSelectionTable.getTableHeader().repaint();
                manualSelectionTable.repaint();
            }
        });
        if (motifs.size()>0) {
            manualSelectionTable.getRowSorter().toggleSortOrder(1);
            if (motifcollection!=null) {
               for (int i=0;i<model.getRowCount();i++) {
                   Motif motif=(Motif)model.getValueAt(i, 1);
                   model.setValueAt(motifcollection.contains(motif), i, 0);
               }
            manualSelectionTable.getRowSorter().toggleSortOrder(0); // show checked first
            manualSelectionTable.getRowSorter().toggleSortOrder(0); // show checked first - must sort twice to get checked on top               
            }
        }
        filter=new Filter();
        filterTextfield.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                ((TableRowSorter)manualSelectionTable.getRowSorter()).setRowFilter(filter);
                updateCountLabelText();
            }
        }); 
        filterTextfield.addKeyListener(new java.awt.event.KeyAdapter() { // note that this is a direct copy of the one above
            @Override
            public void keyReleased(java.awt.event.KeyEvent evt) {
                ((TableRowSorter)manualSelectionTable.getRowSorter()).setRowFilter(filter);
                updateCountLabelText();
            }
        });        
        filterOperator.setSelectedIndex(0);         
        filterCombobox.setSelectedIndex(0);
        filterCombobox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {                
                ((TableRowSorter)manualSelectionTable.getRowSorter()).setRowFilter(filter);
            }
        });         
        manualSelectionContextMenu=new ManualSelectionContextMenu(motifcollection!=null);
        manualSelectionTable.addMouseListener(new java.awt.event.MouseAdapter() {
            @Override
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                if (evt.getClickCount()==2) {
                    int column = manualSelectionTable.columnAtPoint(evt.getPoint()); //
                    if (column==0) return; // clicked the checkbox                    
                    int modelrow=manualSelectionTable.convertRowIndexToModel(manualSelectionTable.getSelectedRow());
                    Motif motif=(Motif)model.getValueAt(modelrow, 1);
                    if (motif!=null) {
                        gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                        Prompt_Motif prompt=new Prompt_Motif(gui, null, motif.clone(), gui.getVisualizationSettings());
                        prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
                        prompt.disableNameEdit();
                        prompt.setVisible(true);  
                        gui.getFrame().setCursor(Cursor.getDefaultCursor());
                        prompt.dispose();                        
                    }
                }                  
            }
            @Override
            public void mousePressed(java.awt.event.MouseEvent evt) {
                if (evt.isPopupTrigger()) showContextMenu(evt);
            }
            @Override
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                updateCountLabelText();
                if (evt.isPopupTrigger()) showContextMenu(evt);
            }
        });
       updateCountLabelText();
       filterCombobox.setSelectedItem(initialFilterChoice);
    } // end setupManualEntryPanel()

    
//    public void addRowSorterListener(RowSorterListener listener) {
//        manualSelectionTable.getRowSorter().addRowSorterListener(listener);
//    } 
//    
//    public void addTableSelectionListener(ListSelectionListener listener) {
//        manualSelectionTable.getSelectionModel().addListSelectionListener(listener);
//    }    
    
    public JTable getTable() {
        return manualSelectionTable;
    }
    
    /** Updates the current Motif Collection (if set) to reflect the new selection */    
    public void updateMotifCollection() {
        if (motifcollection==null) return;
        motifcollection.setPredefinedCollectionName(null);
        motifcollection.clearAll(engine);
        for (int i=0;i<model.getRowCount();i++) {
            if ((Boolean)model.getValueAt(i,0)) {
                Motif motif=(Motif)model.getValueAt(i, 1);
                motifcollection.addMotif((Motif)motif);
            }
        }      
    }    

    @Override
    public void dataAdded(Data data) {
        if (!(data instanceof Motif)) return;
        model.addMotif((Motif)data);
    }

    @Override
    public void dataRemoved(Data data) {
        if (!(data instanceof Motif)) return;
        model.removeMotif((Motif)data);
    }

    @Override
    public void dataUpdate(Data oldvalue, Data newvalue) {}
    @Override
    public void dataUpdated(Data data) {
         if (!(data instanceof Motif)) return;
         model.fireTableDataChanged();
    }

    @Override
    public void dataAddedToSet(Data parentDataset, Data child) {}

    @Override
    public void dataRemovedFromSet(Data parentDataset, Data child) { }

    /** Shows or hides all motifs depending on the argument, but does not update visualization */
    public void setVisibilityOnAllMotifs(boolean visible) {
         VisualizationSettings settings=gui.getVisualizationSettings();
         for (int i=0;i<model.getRowCount();i++) {
             settings.setFeatureVisible(((Motif)model.getValueAt(i,1)).getName(), visible, false);
         }
    }


    /** Returns the number of motifs currently selected under Manual Entry tab*/
    private int countSelectedMotifs() {
         int selected=0;
         for (int i=0;i<model.getRowCount();i++) {
             if ((Boolean)model.getValueAt(i,0)) selected++;
         }           
         return selected;
    }
    /** Updates the label displaying counts (shown, selected and total) in the manual selection tab*/
    private void updateCountLabelText() {
        int total=model.getRowCount();
        if (motifcollection!=null) numberOfMotifsShowingLabel.setText("<html>Showing: "+manualSelectionTable.getRowCount()+" of "+total+"<br>Selected: "+countSelectedMotifs()+" of "+total+"</html>");
        else numberOfMotifsShowingLabel.setText("<html>Showing: "+manualSelectionTable.getRowCount()+" of "+total+"</html>");
    }


     private void showContextMenu(MouseEvent evt) {
        manualSelectionContextMenu.show(evt.getComponent(), evt.getX(),evt.getY());     
     }

     /**
      *  This method compares the given matrix to the current filter-string (which should be a DNA oligo)
      *  and returns a score value reflecting the similarity of the matrix to the score.
      *  All possible alignments in both orientations are tried, and the best score found is returned.
      */
     private float getPWMSimilarityToFilter(Motif motif) {
         String dnasequence=filterTextfield.getText().trim();
         if (dnasequence.isEmpty()) return 0;
         if (!oldPWMtarget.equals(dnasequence)) {
             PWMsimilarityCache.clear(); // the cache is made invalid since the current filterstring is different from the one used by the cache
             oldPWMtarget=dnasequence;
         }
         String motifname=motif.getName();
         Float cached=PWMsimilarityCache.get(motifname);
         if (cached!=null) return cached.floatValue();
         float scorevalue=0;
         float[][] matrix=motif.getMatrixAsFrequencyMatrix();
         int orientation=0; // 0=use both orientations. 1=only DIRECT, -1=only REVERSE
              if (dnasequence.startsWith("+")) {dnasequence=dnasequence.substring(1);orientation=1;}
         else if (dnasequence.startsWith("-")) {dnasequence=dnasequence.substring(1);orientation=-1;}
         if (dnasequence.length()>=motif.getLength()) {
             float directbest=0;
             float reversebest=0;
             if (orientation>=0) directbest=scorePWMstringAgainstDNA(dnasequence, matrix);
             if (orientation<=0) reversebest=scorePWMstringAgainstDNA(PriorsEditorEngine.reverseSequence(dnasequence), matrix);
             scorevalue=Math.max(directbest, reversebest);
         } else {
             float directbest=0;
             float reversebest=0;
             if (orientation>=0) directbest=scoreDNAstringAgainstPWM(dnasequence, matrix);
             if (orientation<=0) reversebest=scoreDNAstringAgainstPWM(PriorsEditorEngine.reverseSequence(dnasequence), matrix);
             scorevalue=Math.max(directbest, reversebest);
         }
         PWMsimilarityCache.put(motifname,scorevalue);
         return scorevalue;
     }
    
private class ManualSelectionTableModel extends AbstractTableModel { 
    private ArrayList<Boolean> checked=null;
    private ArrayList<Data> motifs=null;
    private String[] columnNames=null;
    private int filterColumnIndex=2;

    public ManualSelectionTableModel(ArrayList<Data> motifs,String[] columnNames) {
        this.motifs=motifs;
        checked=new ArrayList<Boolean>(motifs.size());
        for (int i=0;i<motifs.size();i++) checked.add(Boolean.FALSE);
        this.columnNames=columnNames;
    }
    
    @Override
    public Class getColumnClass(int c) {
        Class cl=null;
        if (c==0) return Boolean.class;
        else if (c==1 || c==12 || c==3) return Motif.class;
        else {
            switch (filterColumnIndex) {
                case 0:return Boolean.class;
                case 1:return Motif.class; 
                case 2:return String.class;
                case 3:return String.class;
                case 4:return String.class; 
                case 5:return Boolean.class;
                case 6:return Float.class;
                case 7:return String.class; 
                case 8:return String.class; 
                case 9:return Integer.class; 
                case 10:return Integer.class; 
                case 11:return Float.class;
                case 12:return Motif.class;
                default:return Object.class;
            }
        }
    }
    
    @Override    
    public boolean isCellEditable(int rowIndex, int columnIndex) {
	return (columnIndex==0 && motifcollection!=null);
    }
    
    private int getFilterColumnIndex() {return filterColumnIndex;}
    private void setFilterColumnIndex(int filterColumnIndex) {this.filterColumnIndex=filterColumnIndex;}
    
    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        checked.set(rowIndex,(Boolean)aValue);
        fireTableCellUpdated(rowIndex, columnIndex);
    }

    public void addMotif(Motif newmotif) {
        int index=motifs.size();
        motifs.add(newmotif);
        checked.add(Boolean.FALSE);
        fireTableRowsInserted(index, index);
    }

    public void removeMotif(Motif motif) {
        int index=motifs.indexOf(motif);
        if (index>=0) {
            motifs.remove(index);
            checked.remove(index);
        }
        fireTableRowsDeleted(index, index);
    }
    
    @Override    
    public int getColumnCount() {
        return columnHeaders.length;
    }
    
    @Override
    public int getRowCount() {
        return motifs.size();
    }
    
    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        if (columnIndex==0) return checked.get(rowIndex);
        else if (columnIndex==1 || columnIndex==12) return (Motif)motifs.get(rowIndex);
        else {
            switch (filterColumnIndex) {
                case 0:return checked.get(rowIndex);
                case 1:return ((Motif)motifs.get(rowIndex)).getName();
                case 2:return ((Motif)motifs.get(rowIndex)).getClassification();
                case 3:return ((Motif)motifs.get(rowIndex)).getConsensusMotif();
                case 4:return ((Motif)motifs.get(rowIndex)).getBindingFactors();
                case 5:return ((Motif)motifs.get(rowIndex)).isHalfsite();
                case 6:return ((Motif)motifs.get(rowIndex)).getICcontent();
                case 7:return getMotifDisplayName((Motif)motifs.get(rowIndex));
                case 8:return ((Motif)motifs.get(rowIndex)).getOrganisms();
                case 9:return ((Motif)motifs.get(rowIndex)).getLength();
                case 10:return ((Motif)motifs.get(rowIndex)).getQuality();
                case 11:return getPWMSimilarityToFilter((Motif)motifs.get(rowIndex));
                default: return "-";
            }
        }
    }

    @Override
    public String getColumnName(int column) {
        if (column==3) column=filterColumnIndex;
        return columnNames[column];
    }

    private String getMotifDisplayName(Motif motif) {
        String shortName=motif.getShortName();
        String longName=motif.getLongName();
        if (shortName!=null && !shortName.isEmpty() && longName!=null && !longName.isEmpty()) return shortName+", "+longName;
        else if (shortName!=null && !shortName.isEmpty() && (longName==null || longName.isEmpty())) return shortName;
        else if (longName!=null && !longName.isEmpty() && (shortName==null || shortName.isEmpty())) return longName;
        else return "";
    }
    
}    
        
   
private class CellRenderer_RightAlign extends DefaultTableCellRenderer {
    public CellRenderer_RightAlign() {
           super();
           this.setHorizontalAlignment(DefaultTableCellRenderer.RIGHT);              
    }
    @Override
    public void setValue(Object value) {
           if (value!=null)setText(value.toString());
           else setText("");
       }
}// end class CellRenderer_RightAlign    

private class CellRenderer_Motif extends DefaultTableCellRenderer {
    public CellRenderer_Motif() {
           super();
           this.setHorizontalAlignment(DefaultTableCellRenderer.LEFT);              
    }
    @Override
    public void setValue(Object value) {
           if (value!=null && value instanceof Motif) {
               setText(((Motif)value).getPresentationName());
           }
           else setText("");
       }
}// end class CellRenderer_Motif 

private class CellRenderer_FilterColumn extends DefaultTableCellRenderer {
    public CellRenderer_FilterColumn() {
           super();
    }
    @Override
    public void setValue(Object value) {
           int filterCol=model.getFilterColumnIndex();
           if (filterCol==6 || filterCol==9 || filterCol==10 || filterCol==11) this.setHorizontalAlignment(DefaultTableCellRenderer.RIGHT);
           else this.setHorizontalAlignment(DefaultTableCellRenderer.LEFT);
           if (filterCol==1 && value instanceof Motif) {
                setText(((Motif)value).getPresentationName());
           } else if (filterCol==2) {
               if (value==null) setText("Unknown");
               else {
                  String name=MotifClassification.getNameForClass((String)value);  
                  String fullPath=MotifClassification.getFullLevelsString((String)value);
                  fullPath=fullPath.replaceAll("\t", ":");
                  fullPath=fullPath.replaceAll("\n", " > ");
                  setText(((String)value)+" : "+name+"  ("+fullPath+")");
               }
           } else if (filterCol==5) {
               if (value instanceof Boolean) {
                   if (((Boolean)value).booleanValue()) setText("YES"); else setText("NO");
               }
           } else if (value!=null) {
               setText(value.toString());
           }
           else setText("");
       }
}// end class CellRenderer_Motif    


private class Filter extends RowFilter<Object,Object> {
        
    @Override
    public boolean include(Entry<? extends Object, ? extends Object> entry) {
     String filterString=filterTextfield.getText();
     if (filterString==null || filterString.trim().isEmpty()) return true;
     filterString=filterString.trim().toLowerCase();
     ManualSelectionTableModel model= (ManualSelectionTableModel) entry.getModel();
     int row = ((Integer)entry.getIdentifier()).intValue();
     String comparator=(String)filterOperator.getSelectedItem();
     Object value=model.getValueAt(row,2);
     int filterCol=model.getFilterColumnIndex();
     if (filterCol==6 || filterCol==9 || filterCol==10) {
         try {
            float numeric=0;
            if (value instanceof Integer) numeric=(float)((Integer)value).intValue();
            else numeric=((Float)value).floatValue();
            Float filterNumber = Float.parseFloat(filterString);
                 if (comparator.equals("="))  return numeric==filterNumber;
            else if (comparator.equals("<>")) return numeric!=filterNumber;
            else if (comparator.equals("<"))  return numeric<filterNumber;
            else if (comparator.equals("<=")) return numeric<=filterNumber;
            else if (comparator.equals(">"))  return numeric>filterNumber;
            else if (comparator.equals(">=")) return numeric>=filterNumber;
         } catch (Exception e) {System.err.println(e.getClass().getSimpleName()+":"+e.getMessage());return false;}
     } else if (filterCol==5) {
         boolean halfsite=(Boolean)value;
         boolean showhalfsites=(filterString.equals("yes") || filterString.equals("true") || filterString.equals("halfsite"));
         if (comparator.equals("<>") || comparator.equals("<") || comparator.equals(">")) showhalfsites=!showhalfsites;
         return ((showhalfsites && halfsite) || (!showhalfsites && !halfsite));
     } else if (filterCol==2) { // Classification
         boolean showContains=(comparator.equals("=") || comparator.equals("<=") || comparator.equals(">="));
         String classification;
         if (value==null) classification="unknown";
         else {
             classification=MotifClassification.getFullLevelsString(value.toString()).toLowerCase();
             classification=classification.replaceAll("\n", " ");
             classification=classification.replaceAll("\t", " ");
             classification=value.toString()+" "+classification;
         }
         filterString=filterString.replace(".", "\\.");
         if (!filterString.matches("\\d.*")) filterString=".*"+filterString;
         filterString+=".*";
         boolean contains=false;
         try {contains=classification.matches(filterString);} catch (Exception e){}
         return ((showContains && contains) || (!showContains && !contains));            
     } else if (filterCol==3) { // Consensus
         boolean convertIUPAC=false;
         boolean showContains=!(comparator.equals("<>"));
         boolean sameOrMore=false; 
         if (comparator.equals(">") || comparator.equals(">=")) {
             sameOrMore=true; convertIUPAC=true;
         } else if (comparator.equals("=") || comparator.equals("<") || comparator.equals("<=")) {
             sameOrMore=false; convertIUPAC=true;
         }        
         if (value==null) return !showContains;
         else value=value.toString().toLowerCase();
         boolean anchorStart=false;
         boolean anchorEnd=false;
         int compareOrientation=0; // 0=both orientations, 1=only direct, -1=only reverse
         if (filterString.startsWith("+")) {filterString=filterString.substring(1);compareOrientation=1;}
         else if (filterString.startsWith("-")) {filterString=filterString.substring(1);compareOrientation=-1;}
         if (filterString.startsWith("_") || filterString.startsWith("^")) {filterString=filterString.substring(1);anchorStart=true;}
         if (filterString.endsWith("_") || filterString.endsWith("$")) {filterString=filterString.substring(0,filterString.length()-1);anchorEnd=true;}
         String filterStringReverse=reverseComplementIUPACString(filterString); // reverse the other strings in stead. So we can use regex?
         if (!anchorStart) {
             filterString=".*"+filterString;
             filterStringReverse=filterStringReverse+".*";
         }
         if (!anchorEnd) {
             filterString=filterString+".*";
             filterStringReverse=".*"+filterStringReverse;
         }
         boolean contains=false;
         if (convertIUPAC) {
             filterString=convertIUPACStringtoRegex(filterString,sameOrMore);
             filterStringReverse=convertIUPACStringtoRegex(filterStringReverse,sameOrMore);
         }
         try {
             //System.err.println("["+compareOrientation+"] compare '"+filterString+"' to "+value.toString());
                  if (compareOrientation==0) contains=(value.toString().matches(filterString) || (value.toString().matches(filterStringReverse)));
             else if (compareOrientation>0)  contains=value.toString().matches(filterString);
             else if (compareOrientation<0)  contains=value.toString().matches(filterStringReverse);                      
         } catch (Exception e){}           

         
         return ((showContains && contains) || (!showContains && !contains));           
     } else if (filterCol==7) { // Name
         boolean showContains=(comparator.equals("=") || comparator.equals("<=") || comparator.equals(">="));
         if (value==null) return !showContains;
         Motif motif=(Motif)model.getValueAt(row,1);
         String motifname=(motif.getShortName()+", "+motif.getLongName()).toLowerCase();
         boolean contains=motifname.contains(filterString);
         return ((showContains && contains) || (!showContains && !contains));
     } else if (filterCol==11) { // PWM
         return true; // show all motifs
     } else {
         boolean showContains=(comparator.equals("=") || comparator.equals("<=") || comparator.equals(">="));
         if (value==null) return !showContains;
         boolean contains=(value.toString().toLowerCase()).contains(filterString);
         return ((showContains && contains) || (!showContains && !contains));       
     }
     return false;
  }   
  
    private String convertIUPACStringtoRegex(String filterString,boolean sameOrMore) {
        String result="";
        for (int i=0;i<filterString.length();i++) {
            result+=(sameOrMore)?convertIUPACLettertoRegexMatchSameorMore(filterString.charAt(i)):convertIUPACLettertoRegexMatchSameorLess(filterString.charAt(i));
        }
        return result;
    }  

    private String reverseComplementIUPACString(String string) {
        String result="";
        for (int i=0;i<string.length();i++) {
            char c=string.charAt(i);
            result=reverseComplementIUPACletter(c)+result;
        }
        return result;
    }
    
    private char reverseComplementIUPACletter(char c) {
        switch (c) {
            case 'n':return 'n';
            case 'a':return 't';
            case 'c':return 'g';
            case 'g':return 'c';
            case 't':return 'a';
            case 'r':return 'y';
            case 'y':return 'r';
            case 'm':return 'k';
            case 'k':return 'm';
            case 'w':return 'w';
            case 's':return 's';
            case 'b':return 'v';
            case 'd':return 'h';
            case 'h':return 'd';
            case 'v':return 'b';
            default: return c;
        }
    } 
     
    private String convertIUPACLettertoRegexMatchSameorMore(char c) {
        switch (c) {
            case 'n':return ".";
            case 'a':return "[armwdhvn]";
            case 'c':return "[cymsbhvn]";
            case 'g':return "[grksbdvn]";
            case 't':return "[tykwbdhn]";
            case 'r':return "[ragdvn]";
            case 'y':return "[yctbhn]";
            case 'm':return "[machvn]";
            case 'k':return "[kgtbdn]";
            case 'w':return "[watdhn]";
            case 's':return "[scgbvn]";
            case 'b':return "[bcgtskyn]";
            case 'd':return "[dagtrkwn]";
            case 'h':return "[hactmywn]";
            case 'v':return "[vacgmrsn]";
            default: return ""+c;
        }
    }  

    private String convertIUPACLettertoRegexMatchSameorLess(char c) {
        switch (c) {
            case 'n':return ".";
            case 'a':return "a";
            case 'c':return "c";
            case 'g':return "g";
            case 't':return "t";
            case 'r':return "[rag]";
            case 'y':return "[yct]";
            case 'm':return "[mac]";
            case 'k':return "[kgt]";
            case 'w':return "[wat]";
            case 's':return "[scg]";
            case 'b':return "[bcgtsky]";
            case 'd':return "[dagtrkw]";
            case 'h':return "[hactmyw]";
            case 'v':return "[vacgmrs]";
            default: return ""+c;
        }

    }     
    
} // end class Filter


/** This private class implements the context menu */
private class ManualSelectionContextMenu extends JPopupMenu implements ActionListener {
     JMenuItem markSelectedItem=new JMenuItem("Include selected motifs");
     JMenuItem unmarkSelectedItem=new JMenuItem("Exclude selected motifs");
     JMenuItem markShownItem=new JMenuItem("Include shown motifs");
     JMenuItem unmarkShownItem=new JMenuItem("Exclude shown motifs");
     JMenuItem markAllItem=new JMenuItem("Include all motifs");
     JMenuItem unmarkAllItem=new JMenuItem("Exclude all motifs");
     JMenuItem invertMarksItem=new JMenuItem("Invert collection");
     JMenuItem showSelectedMotifs=new JMenuItem("Show selected motifs");
     JMenuItem hideSelectedMotifs=new JMenuItem("Hide selected motifs");
     JMenu setColorMenu=new JMenu("Set color on highlighted");
     private Object[] colors=new Object[] {
         new Object[]{"BLACK", Color.BLACK},  
         new Object[]{"WHITE", Color.WHITE},  
         new Object[]{"RED", Color.RED},  
         new Object[]{"ORANGE", new Color(255,204,0)},
         new Object[]{"YELLOW", Color.YELLOW},  
         new Object[]{"GREEN", Color.GREEN},  
         new Object[]{"CYAN", Color.CYAN},  
         new Object[]{"LIGHT BLUE", new Color(46,153,255)},  
         new Object[]{"BLUE", Color.BLUE},  
         new Object[]{"VIOLET", new Color(153,51,255)},
         new Object[]{"MAGENTA", Color.MAGENTA}          
    };
     
    public ManualSelectionContextMenu(boolean isSelectionMenu) {
         if (isSelectionMenu) {
             markSelectedItem.addActionListener(this);
             unmarkSelectedItem.addActionListener(this);
             markShownItem.addActionListener(this);
             unmarkShownItem.addActionListener(this);
             markAllItem.addActionListener(this);
             unmarkAllItem.addActionListener(this);
             invertMarksItem.addActionListener(this);
             add(markSelectedItem);
             add(markShownItem);
             add(markAllItem);
             add(unmarkSelectedItem);
             add(unmarkShownItem);
             add(unmarkAllItem);
             add(invertMarksItem);
             add(new JSeparator());
         }
         ColorMenuListener colormenulistener=new ColorMenuListener();
         for (Object entry:colors) {
              String colorname=(String)((Object[])entry)[0];
              Color color=(Color)((Object[])entry)[1];
              JMenuItem menuitem=new JMenuItem(colorname);
              menuitem.setIcon(new SimpleDataPanelIcon(12,12,color));
              menuitem.addActionListener(colormenulistener);
              setColorMenu.add(menuitem);
         }
         JMenuItem menuitemOther=new JMenuItem("Other...");
         menuitemOther.addActionListener(colormenulistener);
         setColorMenu.add(menuitemOther);        
         add(setColorMenu);
         showSelectedMotifs.addActionListener(this);
         hideSelectedMotifs.addActionListener(this);
         add(showSelectedMotifs);
         add(hideSelectedMotifs);
     }
     
     @Override
     public void actionPerformed(ActionEvent e) {
         String cmd=e.getActionCommand();
         if (cmd.equals(markSelectedItem.getActionCommand())) {
             int[] rows=manualSelectionTable.getSelectedRows();
             for (int row:rows) {
                 int modelrow=manualSelectionTable.convertRowIndexToModel(row);
                 model.setValueAt(true, modelrow, 0);
             }
         } else if (cmd.equals(unmarkSelectedItem.getActionCommand())) {
             int[] rows=manualSelectionTable.getSelectedRows();
             for (int row:rows) {
                 int modelrow=manualSelectionTable.convertRowIndexToModel(row);
                 model.setValueAt(false, modelrow, 0);
             }
         } else if (cmd.equals(markShownItem.getActionCommand())) {
             for (int i=0;i<manualSelectionTable.getRowCount();i++) {
                 int modelrow=manualSelectionTable.convertRowIndexToModel(i);
                 model.setValueAt(true, modelrow, 0);
             }
         } else if (cmd.equals(unmarkShownItem.getActionCommand())) {
             for (int i=0;i<manualSelectionTable.getRowCount();i++) {
                 int modelrow=manualSelectionTable.convertRowIndexToModel(i);
                 model.setValueAt(false, modelrow, 0);
             }
         } else if (cmd.equals(markAllItem.getActionCommand())) {
             for (int i=0;i<model.getRowCount();i++) {
                 model.setValueAt(true, i, 0);
             }
         } else if (cmd.equals(unmarkAllItem.getActionCommand())) {
             for (int i=0;i<model.getRowCount();i++) {
                 model.setValueAt(false, i, 0);
             }
         } else if (cmd.equals(invertMarksItem.getActionCommand())) {
             for (int i=0;i<model.getRowCount();i++) {
                 boolean checked=(Boolean)model.getValueAt(i,0);
                 model.setValueAt(!checked, i, 0);
             }
         } else if (cmd.equals(showSelectedMotifs.getActionCommand())) {
             int[] rows=manualSelectionTable.getSelectedRows();
             VisualizationSettings settings=gui.getVisualizationSettings();
             for (int row:rows) {
                 int modelrow=manualSelectionTable.convertRowIndexToModel(row);
                 settings.setFeatureVisible(((Motif)model.getValueAt(modelrow, 1)).getName(), true, false);
             }
             settings.redraw();
         } else if (cmd.equals(hideSelectedMotifs.getActionCommand())) {
             int[] rows=manualSelectionTable.getSelectedRows();
             VisualizationSettings settings=gui.getVisualizationSettings();
             for (int row:rows) {
                 int modelrow=manualSelectionTable.convertRowIndexToModel(row);
                 settings.setFeatureVisible(((Motif)model.getValueAt(modelrow, 1)).getName(), false, false);
             }
             settings.redraw();
         }
         manualSelectionTable.repaint();
         updateCountLabelText();
     }

    /**
     * An inner class that listens to popup-menu events related to the colors submenu
     */
    private class ColorMenuListener implements ActionListener {
       public void actionPerformed(ActionEvent e) {
            String colorname=e.getActionCommand();
            Color color=null;
             if (colorname.equals("Other...")) {
                color = JColorChooser.showDialog(gui.getFrame(), "Select motif color", color);
            } else {
               for (Object entry:colors) {
                  String name=(String)((Object[])entry)[0];
                  if (name.equals(colorname)) {color=(Color)((Object[])entry)[1];break;}
               }
            }
            if (color==null) return;
            int[] selectedRows = manualSelectionTable.getSelectedRows();
            VisualizationSettings settings=gui.getVisualizationSettings();
            for (int i=0;i<selectedRows.length;i++) {
                int modelrow=manualSelectionTable.convertRowIndexToModel(selectedRows[i]);
                settings.setFeatureColor(((Motif)model.getValueAt(modelrow, 1)).getName(), color, false);
            }
            manualSelectionTable.repaint();
            settings.redraw();
         }
    }
 }


private class FeatureColorCellRenderer extends JCheckBox implements TableCellRenderer {
    
    SimpleDataPanelIcon selectedicon=new SimpleDataPanelIcon(12,12,SimpleDataPanelIcon.COLOR_ICON,SimpleDataPanelIcon.SIMPLE_BORDER, null);
    VisualizationSettings settings;

    public FeatureColorCellRenderer(PriorsEditorGUI gui) {
         super();
         settings=gui.getVisualizationSettings();
         setOpaque(false);
         Dimension d=new Dimension(18,18);
         setMinimumSize(d);
         setPreferredSize(d);
         setMaximumSize(d);
         setIcon(selectedicon);
         setSelectedIcon(selectedicon);   
         setMargin(new Insets(0,3,0,0));
    }

    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        Motif motif=(Motif)table.getValueAt(row, column+1);
        selectedicon.setForegroundColor(settings.getFeatureColor(motif.getName()));
        return this;
    }           
}

    /** scores a short DNA string against a PWM and returns the best matching score. The string should be shorter than the PWM (else use scorePWMstringAgainstDNA() below). */
    private float scoreDNAstringAgainstPWM(String dnasequence, float[][] pwm) {
        float currentbest=0;
        for (int i=0;i<=pwm.length-dnasequence.length();i++) { // i is position in the PWM
           float positionscore=0;
           for (int j=0;j<dnasequence.length();j++) {  // j is position in the sequence
               char base=dnasequence.charAt(j);
               if (base=='A' || base=='a') positionscore+=pwm[i+j][0];
               else if (base=='C' || base=='c') positionscore+=pwm[i+j][1];
               else if (base=='G' || base=='g') positionscore+=pwm[i+j][2];
               else if (base=='T' || base=='t') positionscore+=pwm[i+j][3];
           }
           if (positionscore>currentbest) currentbest=positionscore;
        }
        return currentbest;
    }

    /** scores a PWM against a DNA string and returns the score for the best matching position. The lenght of the DNA sequence must be at least that of the PWM (else use scoreDNAstringAgainstPWM() above). */
    private float scorePWMstringAgainstDNA(String dnasequence, float[][] pwm) {
        float currentbest=0;
        for (int i=0;i<=dnasequence.length()-pwm.length;i++) { // i is position in the sequence
           float positionscore=0;
           for (int j=0;j<pwm.length;j++) {  // j is position in the PWM
               char base=dnasequence.charAt(i+j);
               if (base=='A' || base=='a') positionscore+=pwm[j][0];
               else if (base=='C' || base=='c') positionscore+=pwm[j][1];
               else if (base=='G' || base=='g') positionscore+=pwm[j][2];
               else if (base=='T' || base=='t') positionscore+=pwm[j][3];
           }
           if (positionscore>currentbest) currentbest=positionscore;
        }
        return currentbest;
    }




}
