/*
 * Prompt.java
 *
 * Created on 25. mars 2009, 16:45
 */

package priorseditor.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import priorseditor.engine.MotifComparator;
import priorseditor.engine.MotifComparator_PearsonsCorrelation;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.MotifClassification;
import priorseditor.engine.data.MotifCollection;
import priorseditor.engine.data.Region;
import priorseditor.gui.PriorsEditorGUI;
import priorseditor.gui.prompt.Prompt_Motif;

/**
 * This class is used for the dialog that compares a target motif 
 * (for instance one which is predicted de novo) to all motifs known
 * to the engine and lists those that are closest
 *
 * @author  kjetikl
 */
public class MotifComparisonPanel extends javax.swing.JDialog {
    protected PriorsEditorEngine engine;
    protected PriorsEditorGUI gui;
    private MotifCollection motifcollection=null;
    private Motif targetMotif=null;
    private JPanel tablepanel;
    private JTable motifsTable;
    private DefaultTableModel model;
    private ManualSelectionContextMenu manualSelectionContextMenu;
    private SequenceLogo logorenderer;
    private boolean isprocessing=false;
    private boolean isprocessingInterrupted=false;
    private MotifComparator motifComparator=null;
        
    /** Creates new form Prompt */
    public MotifComparisonPanel(PriorsEditorGUI gui, Motif target) {
        super(gui.getFrame(),"Comparison of "+target.getName()+" to other motifs" , true);
        this.engine=gui.getEngine();
        this.gui=gui;
        targetMotif=target;
        motifComparator=new MotifComparator_PearsonsCorrelation();
        initComponents();
        String targetName="Similar_to_"+((targetMotif==null)?"X":targetMotif.getName());
        dataItemNameTextField.setText(targetName);
        progressbar.setVisible(false);
        getRootPane().setDefaultButton(cancelButton);
        setupManualEntryPanel();
        mainPanel.add(tablepanel,BorderLayout.CENTER);
        cancelButton.setText("Cancel");
        okButton.setVisible(false);
        Dimension d=new Dimension(650,500);
        setPreferredSize(d);
        pack();
        model.addRow(new Object[]{Boolean.TRUE,targetMotif,Double.POSITIVE_INFINITY,Region.DIRECT,targetMotif,targetMotif});
        motifsTable.getRowSorter().toggleSortOrder(2); // sort on score, ascending
        motifsTable.getRowSorter().toggleSortOrder(2); // descending
        compareToKnownMotifs(target);
    }


    /** Returns the MotifCollection (if the user pressed 'keep') or NULL
     *  if the user pressed 'discard'
     */
    public MotifCollection getMotifCollection() {
        return motifcollection;
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        dataItemNameLabel = new javax.swing.JLabel();
        dataItemNameTextField = new javax.swing.JTextField();
        jPanel4 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        cancelButton = new javax.swing.JButton();
        okButton = new javax.swing.JButton();
        progressbar = new javax.swing.JProgressBar();
        mainPanel = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setName("Form"); // NOI18N

        jPanel1.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        jPanel1.setAlignmentX(0.0F);
        jPanel1.setMinimumSize(new java.awt.Dimension(110, 10));
        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setLayout(new java.awt.BorderLayout());

        jPanel3.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 5, 5, 5));
        jPanel3.setMaximumSize(new java.awt.Dimension(2147483647, 40));
        jPanel3.setMinimumSize(new java.awt.Dimension(50, 34));
        jPanel3.setName("jPanel3"); // NOI18N
        jPanel3.setPreferredSize(new java.awt.Dimension(100, 34));
        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.LINE_AXIS));

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(MotifComparisonPanel.class);
        dataItemNameLabel.setText(resourceMap.getString("dataItemNameLabel.text")); // NOI18N
        dataItemNameLabel.setName("dataItemNameLabel"); // NOI18N
        jPanel3.add(dataItemNameLabel);

        dataItemNameTextField.setColumns(30);
        dataItemNameTextField.setText(resourceMap.getString("dataItemNameTextField.text")); // NOI18N
        dataItemNameTextField.setMaximumSize(new java.awt.Dimension(500, 40));
        dataItemNameTextField.setMinimumSize(new java.awt.Dimension(150, 20));
        dataItemNameTextField.setName("dataItemNameTextField"); // NOI18N
        dataItemNameTextField.setPreferredSize(new java.awt.Dimension(250, 20));
        jPanel3.add(dataItemNameTextField);

        jPanel1.add(jPanel3, java.awt.BorderLayout.PAGE_END);

        jPanel4.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        jPanel4.setName("jPanel4"); // NOI18N
        jPanel4.setLayout(new javax.swing.BoxLayout(jPanel4, javax.swing.BoxLayout.LINE_AXIS));
        jPanel1.add(jPanel4, java.awt.BorderLayout.PAGE_START);

        getContentPane().add(jPanel1, java.awt.BorderLayout.PAGE_START);

        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setPreferredSize(new java.awt.Dimension(100, 40));

        cancelButton.setText(resourceMap.getString("cancelButton.text")); // NOI18N
        cancelButton.setMaximumSize(new java.awt.Dimension(75, 27));
        cancelButton.setMinimumSize(new java.awt.Dimension(75, 27));
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.setPreferredSize(new java.awt.Dimension(75, 27));
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonPressed(evt);
            }
        });

        okButton.setText(resourceMap.getString("okButton.text")); // NOI18N
        okButton.setMaximumSize(new java.awt.Dimension(75, 27));
        okButton.setMinimumSize(new java.awt.Dimension(75, 27));
        okButton.setName("okButton"); // NOI18N
        okButton.setPreferredSize(new java.awt.Dimension(75, 27));
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonPressed(evt);
            }
        });

        progressbar.setName("progressbar"); // NOI18N

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(progressbar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 70, Short.MAX_VALUE)
                .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(progressbar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );

        getContentPane().add(jPanel2, java.awt.BorderLayout.PAGE_END);

        mainPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setLayout(new java.awt.BorderLayout());
        getContentPane().add(mainPanel, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void cancelButtonPressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonPressed
    if (isprocessing) {
        isprocessingInterrupted=true;
    } else setVisible(false);
}//GEN-LAST:event_cancelButtonPressed

private void okButtonPressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonPressed
    if (isprocessing) {
        isprocessingInterrupted=true;
        return;
    }    
    String targetName=dataItemNameTextField.getText().trim();
    if (targetName.isEmpty()) targetName="Similar_to_"+((targetMotif==null)?"X":targetMotif.getName());
    motifcollection=new MotifCollection(targetName);
    for (int i=0;i<model.getRowCount();i++) {
        if ((Boolean)model.getValueAt(i,0)) {
            Motif motif=(Motif)model.getValueAt(i, 1);
            motifcollection.addMotif((Motif)motif);
        }
    }      
    setVisible(false);    
}//GEN-LAST:event_okButtonPressed




   @SuppressWarnings("unchecked")
    private void setupManualEntryPanel() {
        tablepanel=new JPanel();
        model=new DefaultTableModel(null,new String[]{" ","Motif","Score","+","Class","Logo"}) {
            @Override
            public Class getColumnClass(int col) {
                switch (col) {
                    case 0: return Boolean.class;
                    case 1: return Motif.class;
                    case 2: return Double.class;
                    case 3: return Integer.class;                            
                    case 4: return Motif.class;                            
                    case 5: return Motif.class;                            
                    default: return Object.class;        
                }
            }
        };
        DefaultTableColumnModel columnModel = new DefaultTableColumnModel();
        for (int i=0;i<model.getColumnCount();i++) {
            TableColumn col = new TableColumn(i);
            col.setHeaderValue(model.getColumnName(i));
            columnModel.addColumn(col);
        }
        motifsTable=new JTable(model,columnModel) {
            @Override    
            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return (columnIndex==0);
            }            
        };
        motifsTable.setAutoCreateRowSorter(true);
        motifsTable.getTableHeader().setReorderingAllowed(false);
        columnModel.getColumn(0).setMinWidth(20); // checkbox
        columnModel.getColumn(0).setMaxWidth(20); // checkbox
        columnModel.getColumn(3).setMinWidth(20); // orientation
        columnModel.getColumn(3).setMaxWidth(20);  // orientation   
        columnModel.getColumn(2).setMaxWidth(100); // score
        columnModel.getColumn(4).setMaxWidth(60); // class
        columnModel.getColumn(5).setPreferredWidth(200); // class
        JScrollPane manualSelectionScrollPane=new JScrollPane(motifsTable);
        tablepanel.setLayout(new BorderLayout());

        tablepanel.add(manualSelectionScrollPane,BorderLayout.CENTER);
        columnModel.getColumn(1).setCellRenderer(new CellRenderer_Motif());
        Color [] basecolors=new Color[]{Color.GREEN,Color.BLUE,new Color(220,220,0),Color.RED};    
        int rowheight=motifsTable.getRowHeight();                
        logorenderer=new SequenceLogo(basecolors,(int)(rowheight*1.15),(int)(rowheight/4f),null);
        logorenderer.setStrandColumn(motifsTable, 3);
        logorenderer.setUseAntialias(gui.getVisualizationSettings().useMotifAntialiasing());
        logorenderer.setSelectedBackgroundColor(motifsTable.getSelectionBackground());
        CellRenderer_misc miscRenderer = new CellRenderer_misc();
        columnModel.getColumn(2).setCellRenderer(new CellRenderer_Score());
        columnModel.getColumn(3).setCellRenderer(miscRenderer);
        columnModel.getColumn(4).setCellRenderer(miscRenderer);
        columnModel.getColumn(5).setCellRenderer(logorenderer);
        motifsTable.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode()==KeyEvent.VK_SPACE) {                   
                    int[] rows=motifsTable.getSelectedRows();
                    int checkedRows=0;
                    for (int row:rows) {
                        if ((Boolean)model.getValueAt(motifsTable.convertRowIndexToModel(row),0)) checkedRows++;
                    }
                    Boolean doCheck=Boolean.TRUE;
                    if (checkedRows==rows.length) doCheck=Boolean.FALSE;
                    for (int row:rows) {
                        model.setValueAt(doCheck,motifsTable.convertRowIndexToModel(row),0);
                    }
                } else if (e.getKeyCode()==KeyEvent.VK_PLUS || e.getKeyCode()==KeyEvent.VK_ADD) {
                    int newheight=motifsTable.getRowHeight()+1;
                    if (newheight>80) return;
                    logorenderer.setFontHeight((int)(newheight*1.15), (int)(newheight/4f));
                    motifsTable.setRowHeight(newheight);
                } else if (e.getKeyCode()==KeyEvent.VK_MINUS || e.getKeyCode()==KeyEvent.VK_SUBTRACT) {
                    int newheight=motifsTable.getRowHeight()-1;
                    if (newheight<8) return;
                    logorenderer.setFontHeight((int)(newheight*1.15), (int)(newheight/4f));
                    motifsTable.setRowHeight(newheight); 
                }
            }
        });
 
        manualSelectionContextMenu=new ManualSelectionContextMenu();
        motifsTable.addMouseListener(new java.awt.event.MouseAdapter() {
            @Override
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                if (evt.getClickCount()==2) {
                    int column = motifsTable.columnAtPoint(evt.getPoint()); //
                    if (column==0) return; 
                    int modelrow=motifsTable.convertRowIndexToModel(motifsTable.getSelectedRow());
                    Motif motif=(Motif)model.getValueAt(modelrow, 1);
                    if (motif!=null) {
                        Prompt_Motif prompt=new Prompt_Motif(gui, null, motif.clone(), gui.getVisualizationSettings());
                        prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
                        prompt.disableNameEdit();
                        prompt.setVisible(true);  
                        prompt.dispose();                        
                    }
                }
            }
            @Override
            public void mousePressed(java.awt.event.MouseEvent evt) {
                if (evt.isPopupTrigger()) showContextMenu(evt);
            }
            @Override
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                if (evt.isPopupTrigger()) showContextMenu(evt);
            }
        });
    }
   
  /** Does the comparison in an off-EDT thread */
  private void compareToKnownMotifs(final Motif target) {
        SwingWorker worker=new SwingWorker<Boolean, Void>() {
            @Override 
            public Boolean doInBackground() {
                ArrayList<Data>motifs=engine.getAllDataItemsOfType(Motif.class);
                int size=motifs.size();
                int counter=0;
                for (Data motif:motifs) {
                    counter++;                    
                    if (isprocessingInterrupted) {
                        isprocessingInterrupted=false;
                        isprocessing=false;
                        return Boolean.TRUE;
                    }
                    if (motif==target) continue;
                    double directscore=motifComparator.compareMotifs(target, (Motif)motif, false);
                    double reversescore=motifComparator.compareMotifs(target, (Motif)motif, true);
                    int orientation=Region.INDETERMINED;
                    double bestscore=0;
                    if (directscore>=reversescore) {
                        bestscore=directscore;
                        orientation=Region.DIRECT;
                    } else {
                        bestscore=reversescore;
                        orientation=Region.REVERSE;                        
                    }                        
                    if (bestscore>-Double.MAX_VALUE) { // 20 is an arbitrary threshold
                        addMotifToList((Motif)motif, bestscore, orientation,(int)((float)counter/(float)size*100f));
                    } else addMotifToList(null, bestscore, orientation,(int)((float)counter/(float)size*100f)); // this is just to update progress. No motif is really added
                }
                return Boolean.TRUE;
            } // end doInBackground
            @Override
            public void done() { // this method is invoked on the EDT!
                isprocessing=false;
                progressbar.setValue(0);
                progressbar.setVisible(false); 
                okButton.setVisible(true);
                cancelButton.setText("Discard");
            }
        }; // end of SwingWorker class
        isprocessing=true;
        progressbar.setValue(0);
        progressbar.setVisible(true);
        worker.execute();              
    }    
   
 
 private void addMotifToList(final Motif motif, final double score,final int orientation, final int progress) {
     Runnable runner=new Runnable() {
            @Override
            public void run() {
                if (motif!=null) model.addRow(new Object[]{Boolean.TRUE,motif,new Double(score),orientation,motif,motif});
                progressbar.setValue(progress);
            }
        };
     SwingUtilities.invokeLater(runner);          
 }
   
   
 private void showContextMenu(MouseEvent evt) {
    manualSelectionContextMenu.show(evt.getComponent(), evt.getX(),evt.getY());     
 }  


 
 
 
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel dataItemNameLabel;
    private javax.swing.JTextField dataItemNameTextField;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JProgressBar progressbar;
    // End of variables declaration//GEN-END:variables

   

private class CellRenderer_Motif extends DefaultTableCellRenderer {
    public CellRenderer_Motif() {
           super();
           this.setHorizontalAlignment(DefaultTableCellRenderer.LEFT);              
    }
    @Override
    public void setValue(Object value) {
           if (value!=null && value instanceof Motif) {
               setText(((Motif)value).getPresentationName());
               setToolTipText(((Motif)value).getLongName());
           }
           else super.setValue(value);
       }
}// end class CellRenderer_Motif     
private class CellRenderer_Score extends DefaultTableCellRenderer {
    public CellRenderer_Score() {
           super();
           this.setHorizontalAlignment(DefaultTableCellRenderer.RIGHT);              
    }
    @Override
    public void setValue(Object value) {
           if (value instanceof Double && ((Double)value).isInfinite()) {
               setText("TARGET");
           }
           else super.setValue(value);
       }
}// end class CellRenderer_Motif   
private class CellRenderer_misc extends DefaultTableCellRenderer {
    public CellRenderer_misc() {
           super();
           this.setHorizontalAlignment(DefaultTableCellRenderer.LEADING);              
    }
    @Override
    public void setValue(Object value) {
           if (value!=null && value instanceof Integer) {
               if (((Integer)value)==Region.DIRECT) {
                   setText("+");
                   setForeground(java.awt.Color.GREEN);
                   setToolTipText("Direct orientation");
               } else {
                  setText("-"); 
                  setForeground(java.awt.Color.RED);
                  setToolTipText("Reverse orientation");
               }
           } else if (value!=null && value instanceof Motif) {
               String classString=((Motif)value).getClassification();
               setText(classString);
               setForeground(java.awt.Color.BLACK);
               setToolTipText(MotifClassification.getFullLevelsStringAsHTML(classString));
           }          
       }
}// end class CellRenderer_Orientation     
    
private class ManualSelectionContextMenu extends JPopupMenu implements ActionListener {
     public ManualSelectionContextMenu() {
         JMenuItem markSelectedItem=new JMenuItem("Selected highlighted");
         JMenuItem unmarkSelectedItem=new JMenuItem("Unselect highlighted");
         markSelectedItem.addActionListener(this);
         unmarkSelectedItem.addActionListener(this);
         add(markSelectedItem);
         add(unmarkSelectedItem);
     }
     
     @Override
     public void actionPerformed(ActionEvent e) {
         String cmd=e.getActionCommand();
         if (cmd.equals("Selected highlighted")) {
             int[] rows=motifsTable.getSelectedRows();
             for (int row:rows) {
                 int modelrow=motifsTable.convertRowIndexToModel(row);
                 model.setValueAt(true, modelrow, 0);
             }
         } else if (cmd.equals("Unselect highlighted")) {
             int[] rows=motifsTable.getSelectedRows();
             for (int row:rows) {
                 int modelrow=motifsTable.convertRowIndexToModel(row);
                 model.setValueAt(false, modelrow, 0);
             }             
         }
         motifsTable.repaint();
     }
 }

 
}
