/*
 * MotifScoreFilterDialog.java
 *
 * Created on 14. april 2010, 14:53
 */

package priorseditor.gui;

import java.awt.Dimension;
import java.util.ArrayList;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import priorseditor.engine.DataListener;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.RegionDataset;

/**
 *
 * @author  kjetikl
 */
public class MotifScoreFilterDialog extends javax.swing.JDialog implements ChangeListener, RegionVisualizationFilter, DataListener {
    float currentScoreThreshold=0;
    PriorsEditorGUI gui;
    float maxvalue=0;
    
    /** Creates new form MotifScoreFilterDialog */
    public MotifScoreFilterDialog(PriorsEditorGUI gui) {
        super(gui.getFrame(),false);
        this.gui=gui;
        initComponents();
        Dimension size=new Dimension(370,47);
        getRootPane().setPreferredSize(size);
        getRootPane().setPreferredSize(size);
        getRootPane().setMaximumSize(new Dimension(500,47));
        scoreSlider.setMinimum(0);
        scoreSlider.setMaximum(100);
        scoreSlider.setValue(0);    
        scoreSlider.addChangeListener(this);
        initRange();
        thresholdLabel.setText(String.format("%6.3f",currentScoreThreshold));        
        gui.setRegionVisualizationFilter(this);
        pack();
    
    }

    private void initRange() {
        float max=0;
        ArrayList<Data> regionTracks=gui.getEngine().getAllDataItemsOfType(RegionDataset.class);
        for (Data data:regionTracks) {
            if (((RegionDataset)data).isMotifTrack()) {
                float currentmax=((RegionDataset)data).getMaxScoreValue();
                if (currentmax>max) max=currentmax;
            }
        }
        maxvalue=max;  
    }
    
    
    @Override
    public void stateChanged(ChangeEvent e) {
       int percentage=scoreSlider.getValue();
       currentScoreThreshold=(maxvalue/100f)*percentage;
       thresholdLabel.setText(String.format("%6.3f",currentScoreThreshold));
       gui.getVisualizationSettings().redraw();
    }

    @Override
    public boolean shouldVisualize(Region region) {
        return region.getScore()>=currentScoreThreshold;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel3 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        thresholdLabel = new javax.swing.JLabel();
        scoreSlider = new javax.swing.JSlider();
        jPanel1 = new javax.swing.JPanel();
        closeButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(MotifScoreFilterDialog.class);
        setTitle(resourceMap.getString("Form.title")); // NOI18N
        setMinimumSize(new java.awt.Dimension(350, 33));
        setName("Form"); // NOI18N
        getContentPane().setLayout(new java.awt.FlowLayout());

        jPanel3.setName("jPanel3"); // NOI18N
        jPanel3.setLayout(new java.awt.BorderLayout());

        jPanel2.setMaximumSize(new java.awt.Dimension(90, 33));
        jPanel2.setMinimumSize(new java.awt.Dimension(90, 33));
        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setPreferredSize(new java.awt.Dimension(90, 33));
        jPanel2.setLayout(new java.awt.BorderLayout());

        thresholdLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        thresholdLabel.setText(resourceMap.getString("thresholdLabel.text")); // NOI18N
        thresholdLabel.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 10));
        thresholdLabel.setFocusable(false);
        thresholdLabel.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
        thresholdLabel.setMaximumSize(new java.awt.Dimension(110, 33));
        thresholdLabel.setMinimumSize(new java.awt.Dimension(110, 33));
        thresholdLabel.setName("thresholdLabel"); // NOI18N
        thresholdLabel.setPreferredSize(new java.awt.Dimension(110, 33));
        jPanel2.add(thresholdLabel, java.awt.BorderLayout.CENTER);

        jPanel3.add(jPanel2, java.awt.BorderLayout.WEST);

        scoreSlider.setAlignmentY(0.0F);
        scoreSlider.setMaximumSize(new java.awt.Dimension(32767, 33));
        scoreSlider.setMinimumSize(new java.awt.Dimension(200, 33));
        scoreSlider.setName("scoreSlider"); // NOI18N
        scoreSlider.setPreferredSize(new java.awt.Dimension(200, 33));
        jPanel3.add(scoreSlider, java.awt.BorderLayout.CENTER);

        jPanel1.setName("jPanel1"); // NOI18N

        closeButton.setText(resourceMap.getString("closeButton.text")); // NOI18N
        closeButton.setName("closeButton"); // NOI18N
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonClicked(evt);
            }
        });
        jPanel1.add(closeButton);

        jPanel3.add(jPanel1, java.awt.BorderLayout.EAST);

        getContentPane().add(jPanel3);

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void closeButtonClicked(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonClicked
// TODO add your handling code here:
    gui.setRegionVisualizationFilter(null);
    gui.getVisualizationSettings().redraw();
    setVisible(false);
    this.dispose();
}//GEN-LAST:event_closeButtonClicked

    @Override
    public void dataAdded(Data data) {
        if (data instanceof RegionDataset) {
            initRange();
            scoreSlider.setValue(0);
        }
    }
    @Override
    public void dataRemoved(Data data) {
        if (data instanceof RegionDataset) {
            initRange();
            scoreSlider.setValue(0);
        }    
    }
    
    @Override
    public void dataUpdated(Data data) {
         if (data instanceof RegionDataset) {
            initRange();
            scoreSlider.setValue(0);
        }   
    }

    @Override
    public void dataAddedToSet(Data parentDataset, Data child) {}
    @Override
    public void dataRemovedFromSet(Data parentDataset, Data child) {}
    @Override
    public void dataUpdate(Data oldvalue, Data newvalue) {}







    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton closeButton;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JSlider scoreSlider;
    private javax.swing.JLabel thresholdLabel;
    // End of variables declaration//GEN-END:variables

}
