/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import javax.swing.JToolTip;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.Region;
import priorseditor.engine.data.RegionSequenceData;
import priorseditor.engine.data.Sequence;

/**
 * This class is used to draw tooltips that appear when hoovering over binding sites in motif tracks
 * @author kjetikl
 */
public class MotifTooltip extends JToolTip implements MouseMotionListener {
    private DataTrackVisualizer_Region track;
    private Font logofont;
    private Font font;
    private Font ttfont;
    private char[] bases;
    private float[] letterXoffset=new float[4];        
    private int fontheight=25; // if you change this size you must also change the ascentCorrection below accordingly (but I am not sure about the relationship...)
    private int ascentCorrection=6; // this is a needed hack because the ascent returned by the FontMetrics is not really the same height as that of a capital letter (even though that was stated in the documentation)
    private PriorsEditorEngine engine;
    private VisualizationSettings settings;
    private int positionTextHeight=20;
    private int lineheight=20;
    private int xoffset=20;
    private int lastposition=Integer.MAX_VALUE; // keeps track of the last genomic position. MAX_VALUE is a flag used for illegal values
    private Dimension lastsize=null; // keeps track of the last tooltip dimension
    private ArrayList<Region> regions=null;
    // ThreadMXBean bean = ManagementFactory.getThreadMXBean( );


    
    public MotifTooltip(DataTrackVisualizer_Region trackvisualizer, VisualizationSettings settings) {
        track=trackvisualizer;
        logofont=new Font(Font.SANS_SERIF,Font.BOLD,fontheight);
        font=new Font(Font.SERIF,Font.PLAIN,13);
        ttfont=new Font(Font.MONOSPACED,Font.PLAIN,13);
        bases=new char[]{'A','C','G','T'};
        this.settings=settings;
        engine=settings.getEngine();
        addMouseMotionListener(this); 
    }
    
    
    
    @Override
    @SuppressWarnings("unchecked")
    protected void paintComponent(Graphics g) {    
        if (!track.isMotifTrack()) {
           super.paintComponent(g);
        } else if (settings.getGUI().useGraphicalTooltips()) {
           super.paintComponent(g);
           paintGraphicalTooltip(g);           
        } else {
           super.paintComponent(g);
           g.setColor(Color.WHITE);
           paintTextualtooltip(g);           
           
        }
    }

    @SuppressWarnings("unchecked")    
    private void paintTextualtooltip(Graphics g) {
           //int pos=track.getCurrentGenomicMousePosition();
           RegionSequenceData regiontrack=(RegionSequenceData)track.getTrack();
           //ArrayList<Region> regions=getVisibleRegions(pos,regiontrack);
           if (regions.isEmpty()) return;   
           int totalsize=0;
           int leftmost=Integer.MAX_VALUE; // position of leftmost
           //Motif[] motifs=new Motif[regions.size()];
           //String[] sequences=new String[regions.size()];
           Object[] list = new Object[regions.size()];
           int shownorientation=settings.getSequenceOrientation(regiontrack.getSequenceName());
           //boolean showGeneStrand=shownorientation!=regiontrack.getStrandOrientation();
           for (int i=0;i<regions.size();i++) { // locate motifs and sequences. Set 'leftmost' variable
               Region region=regions.get(i);
               String sequence=(String)region.getProperty("sequence");
               String motifname=region.getType();
               Data data=engine.getDataItem(motifname);
               if (data!=null && data instanceof Motif && sequence!=null && region.getLength()==((Motif)data).getLength() && region.getLength()==sequence.length()) {
                   if (region.getOrientation()!=shownorientation && sequence!=null) sequence=PriorsEditorEngine.reverseSequence(sequence);               
                   int start=region.getGenomicStart();
                   if (start<leftmost) leftmost=start;
                   list[i]=new Object[]{region,data,sequence};
               } else {
                   list[i]=region;
               }              
           }
           for (int i=0;i<regions.size();i++) { // set 'longest' variable
               if (list[i] instanceof Region) continue; // the region has no motif association
               Region region=(Region)((Object[])list[i])[0];
               int offset=region.getGenomicStart()-leftmost;
               int total=region.getLength()+offset;
               if (total>totalsize) totalsize=total;
           }  
           // sort the array so that all regions with motifs are first and second according to position (which is orientation dependent!)
           RegionComparator comparator=new RegionComparator(shownorientation==Sequence.DIRECT);
           Arrays.sort(list, comparator);  
           int textoffset=getFontMetrics(ttfont).charWidth('G')*totalsize;
           for (int i=0;i<list.length;i++) { // paint info for each region
               Object object=list[i];
               Region region=null;
               String sequence=null;
               if (object instanceof Region) {
                   region=(Region)object;
               } else {
                   region=(Region)((Object[])object)[0];
                   sequence=(String)((Object[])object)[2];
               }
               int offset=region.getGenomicStart()-leftmost;
               if (shownorientation==Sequence.REVERSE) offset=totalsize-(offset+region.getLength());
               Color motifcolor=settings.getFeatureColor(region.getType());
               StringBuffer buffer=new StringBuffer(totalsize);
               int length=region.getLength();
               if (object instanceof Object[]) {
                   //textoffset=paintMatchLogo(g, motif, sequence,xoffset, yoffset, totalsize, offset, shownorientation, region.getOrientation());
                   for (int k=0;k<offset;k++) buffer.append('.');                 
                   for (int k=0;k<length;k++) buffer.append(sequence.charAt(k)); 
                   for (int k=offset+length;k<totalsize;k++) buffer.append('.');           
               } else {
                   for (int k=0;k<totalsize;k++) buffer.append('.');   
               }    
               Data data=engine.getDataItem(region.getType());
               if (data==null || !(data instanceof Motif)) data=null;
               String string=getRegionString(region,(Motif)data);
               g.setColor(Color.WHITE);
               g.fillRect(1, lineheight*i+positionTextHeight, textoffset, lineheight);
               g.setColor(motifcolor);
               g.setFont(ttfont);
               g.drawString(buffer.toString(),1,lineheight*(i+1)+positionTextHeight-2);
               g.setColor(Color.BLACK);
               g.setFont(font);
               g.drawString(string,textoffset+4,lineheight*(i+1)+positionTextHeight-2);
           }    
    }
    
    
    @SuppressWarnings("unchecked")    
    private void paintGraphicalTooltip(Graphics g) {
           int pos=track.getCurrentGenomicMousePosition();
//           if (pos==lastposition) {}
           RegionSequenceData regiontrack=(RegionSequenceData)track.getTrack();
//           regions=regiontrack.getVisibleRegions(pos);
           if (regions.isEmpty()) return;            
           int yoffset=positionTextHeight;
           int totalsize=0;
           int leftmost=Integer.MAX_VALUE; // position of leftmost
           //Motif[] motifs=new Motif[regions.size()];
           //String[] sequences=new String[regions.size()];
           Object[] list = new Object[regions.size()];
           int shownorientation=settings.getSequenceOrientation(regiontrack.getSequenceName());
           //boolean showGeneStrand=shownorientation!=regiontrack.getStrandOrientation();
           int motifspainted=0;
           for (int i=0;i<regions.size();i++) { // locate motifs and sequences. Set 'leftmost' variable
               Region region=regions.get(i);
               String sequence=(String)region.getProperty("sequence");
               String motifname=region.getType();
               Data data=engine.getDataItem(motifname);
               if (data!=null && data instanceof Motif && sequence!=null && region.getLength()==((Motif)data).getLength() && region.getLength()==sequence.length()) {
                   if (region.getOrientation()!=shownorientation && sequence!=null) sequence=PriorsEditorEngine.reverseSequence(sequence);               
                   int start=region.getGenomicStart();
                   if (start<leftmost) leftmost=start;
                   list[i]=new Object[]{region,data,sequence};
               } else {
                   list[i]=region;
               }              
           }
           for (int i=0;i<regions.size();i++) { // set 'longest' variable
               if (list[i] instanceof Region) continue; // the region has no motif association
               Region region=(Region)((Object[])list[i])[0];
               int offset=region.getGenomicStart()-leftmost;
               int total=region.getLength()+offset;
               if (total>totalsize) totalsize=total;
           }  
           // sort the array so that all regions with motifs are first and second according to position (which is orientation dependent!)
           RegionComparator comparator=new RegionComparator(shownorientation==Sequence.DIRECT);
           Arrays.sort(list, comparator);             
           for (int i=0;i<list.length;i++) { // paint info for each region
               Object object=list[i];
               Region region=null;
               Motif motif=null;
               String sequence=null;
               if (object instanceof Region) {
                   region=(Region)object;
               } else {
                   region=(Region)((Object[])object)[0];
                   motif=(Motif)((Object[])object)[1];
                   sequence=(String)((Object[])object)[2];
               }
               int offset=region.getGenomicStart()-leftmost;
               if (shownorientation==Sequence.REVERSE) offset=totalsize-(offset+region.getLength());
               g.setColor(Color.WHITE);
               g.fillRect(1, yoffset, xoffset, lineheight);
               g.setColor(settings.getFeatureColor(region.getType()));
               g.fillRect(5, yoffset+5, 10, 10);
               g.setColor(Color.BLACK);
               g.drawRect(5, yoffset+5, 10, 10);
               g.drawRect(1, yoffset, xoffset, lineheight);
               int textoffset=xoffset;
               if (object instanceof Object[]) {
                   motifspainted++;
                   textoffset=paintMatchLogo(g, motif, sequence,xoffset, yoffset, totalsize, offset, shownorientation, region.getOrientation());
               }    
               Data data=engine.getDataItem(region.getType());
               if (data==null || !(data instanceof Motif)) data=null;
               String string=getRegionString(region,(Motif)data);
               g.setFont(font);
               g.setColor(java.awt.Color.BLACK);
               g.drawString(string, textoffset+5, yoffset+lineheight-5);
               yoffset+=lineheight;
               //engine.logMessage(sequence);
           }
           // draw base cursor 
           if (motifspainted>0) {
               FontMetrics metrics=g.getFontMetrics(logofont); 
               int widthG=metrics.charWidth('G');             
               g.setColor(Color.MAGENTA);
               int currentoffset=pos-leftmost;  
               if (shownorientation==Sequence.REVERSE) currentoffset=totalsize-(currentoffset+1);
               //if (shownorientation==Sequence.REVERSE) currentoffset=
               g.drawRect(xoffset+currentoffset*widthG, positionTextHeight+1, widthG, lineheight*motifspainted-2);     
           }  
    }
    

    
    
    @Override
    public Dimension getPreferredSize() {
        Dimension dim=super.getPreferredSize();
        if (!track.isMotifTrack()) {
           return dim;
        } 
        int pos=track.getCurrentGenomicMousePosition();
        if (pos==lastposition) return lastsize;        
        boolean useGraphical=settings.getGUI().useGraphicalTooltips();
        int width=dim.width;
        //RegionSequenceData regiontrack=(RegionSequenceData)track.getTrack();
        regions=track.getVisibleRegions(pos);
        if (regions.isEmpty()) {lastsize=dim;return dim;}
        FontMetrics metrics=getFontMetrics(font);
        int charwidth=(useGraphical)?getFontMetrics(logofont).charWidth('G'):getFontMetrics(ttfont).charWidth('G');
        int[] stringlengths=new int[regions.size()];
        boolean[] hasmotif=new boolean[regions.size()];
        int leftmost=Integer.MAX_VALUE;
        for (int i=0;i<regions.size();i++) { 
            Region region=regions.get(i);
            String motifname=region.getType();
            Data data=engine.getDataItem(motifname);
            if (data!=null && data instanceof Motif && region.getLength()==((Motif)data).getLength()) {
               int start=region.getGenomicStart();
               if (start<leftmost) leftmost=start;
               hasmotif[i]=true;
               String string=getRegionString(region,(Motif)data);
               stringlengths[i]=metrics.stringWidth(string);

            }  else {
              String string=getRegionString(region,null);
              stringlengths[i]=metrics.stringWidth(string);
            }           
        }
        int totalsize=0;
        for (int i=0;i<regions.size();i++) { // set 'longest' variable
               if (!hasmotif[i]) continue; // the region has no motif association
               Region region=regions.get(i);
               int offset=region.getGenomicStart()-leftmost;
               int total=region.getLength()+offset;
               if (total>totalsize) totalsize=total;
        }   
        int drawingsize=(useGraphical)?xoffset+totalsize*charwidth+10:(totalsize*charwidth)+7; // 
        for (int i=0;i<regions.size();i++) { 
             int linewidth=stringlengths[i]+drawingsize; 
             if (linewidth>width) width=linewidth;
        }    
        Dimension newsize=new Dimension(width,lineheight*regions.size()+positionTextHeight+2);
        lastsize=newsize;
        lastposition=pos;
        return newsize;
    }

    /**
     * Returns a string that describes this region
     * If a corresponding Motif object is supplied, the the returned name of the Region 
     * will be be presentation name of the motif. If motif==null then the returned name
     * will just be the type of the region 
     */
    private String getRegionString(Region region, Motif motif) {
        String orientation;
        if (region.getOrientation()==Region.DIRECT) orientation="+";
        else if (region.getOrientation()==Region.REVERSE) orientation="-";
        else orientation=".";
        String type=(motif!=null)?motif.getPresentationName():region.getType();
        if (type==null) type="UNKNOWN";
        return type+" ["+orientation+"], "+region.getLength()+" bp, score="+region.getScore();
    }
    
    private int paintMatchLogo(Graphics graphics, Motif motif, CharSequence region, int xoffset, int yoffset, int size, int offset, int shownorientation, int motiforientation) {
        //super.paintComponent(graphics);
        float[][] matrix=motif.getMatrix();
        if (matrix==null) return 0;
        if (motif.getMatrixType()==Motif.LOG_ODDS_MATRIX) matrix=motif.getMatrixAsFrequencyMatrix();        
        Graphics2D g = (Graphics2D)graphics;
        FontMetrics metrics=g.getFontMetrics(logofont);
        int ascent=metrics.getAscent()-ascentCorrection;
        int widthA=metrics.charWidth('A');     
        int widthC=metrics.charWidth('C');     
        int widthG=metrics.charWidth('G');     
        int widthT=metrics.charWidth('T');
        letterXoffset[0]=(widthG-widthA)/2;
        letterXoffset[1]=(widthG-widthC)/2;
        letterXoffset[3]=(widthG-widthT)/2;
        int logowidth=matrix.length*widthG;
        int fullwidth=size*widthG;
        if (settings.useMotifAntialiasing()) g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g.setFont(logofont);
        g.setColor(java.awt.Color.RED);
        AffineTransform restore=g.getTransform();
        AffineTransform save=g.getTransform();           
        if (logowidth>getWidth()) save.scale((float)getWidth()/(float)logowidth, 1); // scale X-direction so that logo fits irrespective of size
        AffineTransform scaleY = new AffineTransform();
        g.setColor(Color.DARK_GRAY);
        g.fillRect(xoffset, yoffset, fullwidth, ascent);
        g.setColor(Color.WHITE);
        g.fillRect(xoffset+offset*widthG, yoffset, logowidth, ascent);
        float xpos=xoffset+offset*widthG;
        boolean showDirectMotif=(shownorientation==motiforientation);
        for (int i=0;i<matrix.length;i++) {
            int pos=(showDirectMotif)?i:(matrix.length-(i+1));
            char baseAtPosition=Character.toUpperCase(region.charAt(i)); // note that the matching sequence har already been reversed when needed            
            float[] counts;
            if (showDirectMotif) counts=new float[]{matrix[pos][0],matrix[pos][1],matrix[pos][2],matrix[pos][3]};
            else counts=new float[]{matrix[pos][3],matrix[pos][2],matrix[pos][1],matrix[pos][0]};
            float total=counts[0]+counts[1]+counts[2]+counts[3];
            float ic=Motif.calculateColumnIC(counts[0], counts[1], counts[2], counts[3], false);
            int[] sorted=new int[4]; // sorted in ascending order. Values are base-indices (i.e. 0=>A, 1=>C, 2=>G, 3=>T)
            int indexA=0, indexC=0, indexG=0, indexT=0;
            if (counts[0]>=counts[1]) indexA++;
            if (counts[0]>=counts[2]) indexA++;
            if (counts[0]>=counts[3]) indexA++;                
            if (counts[1]>counts[0]) indexC++;
            if (counts[1]>=counts[2]) indexC++;
            if (counts[1]>=counts[3]) indexC++;               
            if (counts[2]>counts[0]) indexG++;
            if (counts[2]>counts[1]) indexG++;
            if (counts[2]>=counts[3]) indexG++;                
            if (counts[3]>counts[0]) indexT++;
            if (counts[3]>counts[1]) indexT++;
            if (counts[3]>counts[2]) indexT++;
            sorted[indexA]=0;
            sorted[indexC]=1;
            sorted[indexG]=2;
            sorted[indexT]=3;
            float currentYoffset=yoffset+ascent-(ascent*ic/2f);
            for (int j=3;j>=0;j--) { // draws letters from top to bottom (most frequent first)
                int base=sorted[j];
                float fraction=counts[base]/total;
                scaleY.setTransform(save);
                scaleY.translate(letterXoffset[base],currentYoffset); // translated in 1-to-1 scale                    
                scaleY.scale(1,ic/2f); // scale by IC-content in position                   
                scaleY.scale(1, fraction);   
                if (baseAtPosition==bases[base]) {
                    if (showDirectMotif) g.setColor(Color.GREEN);
                    else g.setColor(Color.RED);
                }
                else g.setColor(Color.LIGHT_GRAY);                
                g.setTransform(scaleY);
                g.drawString(""+bases[base], xpos, ascent); // draw all letters at same position and use transform to place them correctly
                currentYoffset+=ascent*fraction*(ic/2f);
            } // end for each base  
            xpos+=widthG; // 
        } // end for each position
        g.setTransform(save);
        g.setColor(Color.BLACK);
        g.drawRect(xoffset,yoffset,fullwidth,ascent);
        g.setTransform(restore);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
        return xoffset+fullwidth+1;
    }     

    /** Clears any cached data for this tooltip */
    public void clearData() {
        lastposition=Integer.MAX_VALUE;
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        lastposition=Integer.MAX_VALUE;
        
    }

    @Override
    public void mouseMoved(MouseEvent e) {
       lastposition=Integer.MAX_VALUE;
       
    }

    
    
    
    
    
    
    
    
    
    public class RegionComparator implements Comparator {
        boolean direct=true;
        public RegionComparator(boolean direct) {
            this.direct=direct;
        }
        @Override
        public int compare(Object o1, Object o2) {
                 if (o1 instanceof Object[] && o2 instanceof Region) return -1;
            else if (o1 instanceof Region && o2 instanceof Object[]) return 1;
            else {
                  Region region1=null;
                  Region region2=null;
                  if (o1 instanceof Region && o2 instanceof Region) { 
                      region1=(Region)o1;
                      region2=(Region)o2;
                  } else if (o1 instanceof Object[] && o2 instanceof Object[]) {
                      region1=(Region)((Object[])o1)[0];
                      region2=(Region)((Object[])o2)[0];                 
                  } else return 0; // this should not happen    
                  if (direct) {
                       int start1=region1.getRelativeStart();
                       int start2=region2.getRelativeStart();
                       if (start1<start2) return -1;
                       else if (start2>start1) return 1;
                       else return 0;
                  } else { // reverse
                       int end1=region1.getRelativeEnd();
                       int end2=region2.getRelativeEnd();
                       if (end1>end2) return -1;
                       else if (end1<end2) return 1;
                       else return 0;                       
                  }
             } // end same type object                
        } // end compare              
    } // end RegionComparator
    
}
