
package priorseditor.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JPopupMenu;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.JToolTip;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import org.jdesktop.application.ResourceMap;

import priorseditor.engine.CompoundTask;
import priorseditor.engine.DataListener;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.*;
import priorseditor.engine.operations.Operation_new;
import priorseditor.gui.prompt.Prompt;
import priorseditor.gui.prompt.Prompt_Motif;
import priorseditor.gui.prompt.Prompt_MotifCollection;


/**
 * This class encapsulates the MotifCollection panel
 * 
 * @author kjetikl
 */
public class MotifsPanel extends JPanel implements DataListener {
        private static final String GROUP_BY_MOTIFS="No groups";
        private static final String GROUP_BY_MOTIF_COLLECTIONS="Group by motif collections";
        private static final String GROUP_BY_CLASS="Group by motif classes";
        private JLabel motifCollectionPanelLabel;
        private JScrollPane motifCollectionScrollPane;
        private JTree motifCollectionTree;
        private JButton addButton;
        private JButton groupButton;
        private addDataPopupMenu addPopupMenu;
        private groupPopupMenu groupPopupMenu;
        private PriorsEditorGUI gui;
        private CheckBoxTreeCellRenderer checkBoxTreeCellRenderer;
        private DefaultTreeModel model=null;
        private SingleMotifTooltip tooltip=null;        
        VisualizationSettings settings=null;
        private TreeListener treelistener;
        
 
        
  public MotifsPanel(PriorsEditorGUI gui) {
      this.gui=gui;      
      this.settings=gui.getVisualizationSettings();      
      CheckBoxTreeNode.setVisualizationSettings(settings); // static property
      treelistener=new TreeListener();
      initComponents();
      motifCollectionTree.setRootVisible(false); // NOTE: hiding the root causes NullPointerExceptions and GUI crash in Nimbus when removing tree nodes (I don't know why and have not been able to create a workaround yet)
      // motifCollectionTree.setShowsRootHandles(true);
      tooltip=new SingleMotifTooltip(settings);
      //tooltip.setComponent(motifCollectionTree);
      ToolTipManager.sharedInstance().registerComponent(motifCollectionTree); // this is necessary 
      gui.getEngine().addDataListener(this);
      
  } 
  
  public JTree getMotifCollectionList() {
      return motifCollectionTree;
  }  
  
    
  private void initComponents() {
        ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(MotifsPanel.class);

        motifCollectionPanelLabel=new JLabel();
        setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        setName("MotifCollectionPanel"); // NOI18N
        setLayout(new BorderLayout());

        motifCollectionPanelLabel.setText(resourceMap.getString("motifCollectionLabel.text")); // NOI18N
        motifCollectionPanelLabel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 3, 1, 1));
        motifCollectionPanelLabel.setName("Motif Collections"); // NOI18N
        Font buttonsFont=new Font(Font.SANS_SERIF,Font.PLAIN,12);
        addPopupMenu=new addDataPopupMenu();
        addButton=new JButton(new MiscIcons(MiscIcons.ADD_DATA));
        addButton.setName("addButton");
        addButton.setToolTipText(resourceMap.getString("addButton.toolTipText")); // NOI18N
        addButton.setFont(buttonsFont);
        Dimension dim=new Dimension(36,22);
        addButton.setMaximumSize(dim);
        addButton.setPreferredSize(dim);
        addButton.setMinimumSize(dim);
        addButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                addDataObject(evt);
            }
        });
        groupPopupMenu=new groupPopupMenu();        
        groupButton=new JButton(new MiscIcons(MiscIcons.TREE_GROUPS));
        groupButton.setName("groupButton");
        groupButton.setToolTipText(resourceMap.getString("groupButton.toolTipText")); // NOI18N
        groupButton.setFont(buttonsFont);
        Dimension dim2=new Dimension(36,22);
        groupButton.setMaximumSize(dim2);
        groupButton.setPreferredSize(dim2);
        groupButton.setMinimumSize(dim2);
        groupButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                setGrouping(evt);
            }
        });
        JPanel buttonspanel=new JPanel();
        buttonspanel.setLayout(new BoxLayout(buttonspanel, BoxLayout.X_AXIS));
        buttonspanel.add(groupButton);
        buttonspanel.add(addButton);
        JPanel labelpanel=new JPanel();
        labelpanel.setLayout(new BorderLayout());
        labelpanel.add(motifCollectionPanelLabel,BorderLayout.CENTER);
        labelpanel.add(buttonspanel,BorderLayout.EAST);
        
        add(labelpanel, java.awt.BorderLayout.PAGE_START);
        
        motifCollectionScrollPane = new JScrollPane();
        motifCollectionScrollPane.setName("MotifCollectionScrollPane"); // NOI18N

        motifCollectionTree=new JTree() {
            @Override
            public JToolTip createToolTip() {
                return tooltip;
            }              
        };       
        motifCollectionTree.setBackground(java.awt.Color.WHITE); // NOI18N        
        setGroupingNone(); //setGroupingByCollection();
        motifCollectionTree.setDragEnabled(false);
        // motifCollectionTree.setDropMode(javax.swing.DropMode.ON_OR_INSERT);
        motifCollectionTree.setName("motifCollectionList"); // NOI18N
        motifCollectionTree.setEditable(false);
        motifCollectionTree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
        motifCollectionTree.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode()==KeyEvent.VK_SPACE) {                   
                    int[] rows=motifCollectionTree.getSelectionModel().getSelectionRows();
                    if (rows==null) return;
                    Arrays.sort(rows);
                    int checkedNodes=0;
                    for (int row:rows) {
                        CheckBoxTreeNode node=(CheckBoxTreeNode)motifCollectionTree.getPathForRow(row).getLastPathComponent();
                        if (node.isChecked()) checkedNodes++;
                    } 
                    Boolean doCheck=Boolean.TRUE;
                    if (checkedNodes==rows.length) doCheck=Boolean.FALSE;
                    setVisibilityOnSelectedRows(doCheck);
                } else if (e.getKeyCode()==KeyEvent.VK_DELETE) {
                    int[] rows=motifCollectionTree.getSelectionModel().getSelectionRows();
                    if (rows==null) return; // this can happen!                   
                    ArrayList<String> deletionList=new ArrayList<String>();
                    for (int row:rows) {
                        CheckBoxTreeNode node=(CheckBoxTreeNode)motifCollectionTree.getPathForRow(row).getLastPathComponent();                     
                        Object object=node.getUserObject();                        
                        if (object instanceof MotifCollection) { // do not allow deletion of single Motifs from the engine
                            deletionList.add(((Data)object).getName());
                        }                          
                    } // end for each selected row
                    String[] names=new String[deletionList.size()];
                    if (!deletionList.isEmpty()) gui.deleteDataItems(deletionList.toArray(names)); 
                } // end key pressed = DELETE
            }
        });
        checkBoxTreeCellRenderer=new CheckBoxTreeCellRenderer();        
        motifCollectionTree.setCellRenderer(checkBoxTreeCellRenderer);        
        motifCollectionScrollPane.setViewportView(motifCollectionTree);       
        add(motifCollectionScrollPane, java.awt.BorderLayout.CENTER);
        motifCollectionTree.addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                Object node=null;
                int x=e.getX();
                int y=e.getY();
                int row = motifCollectionTree.getRowForLocation(x, y);
                if (row<0) return;
                if (!motifCollectionTree.isRowSelected(row)) {
                    motifCollectionTree.setSelectionRow(row);
                }
                Rectangle rect = motifCollectionTree.getRowBounds(row);
	        if (rect == null) return;	  
                TreePath path=motifCollectionTree.getPathForRow(row);
                if (path!=null) node=path.getLastPathComponent();
                if (node!=null && node instanceof CheckBoxTreeNode) {
                   if (checkBoxTreeCellRenderer.isOnHotspot(x - rect.x, y - rect.y)) { // pressed show/hide button
                       CheckBoxTreeNode checknode=(CheckBoxTreeNode)node;
                       checknode.setChecked(!checknode.isChecked()); // reverse Check
                       if (!checknode.isChecked()) { // uncheck parents if necessary
                          Object[] nodepath=path.getPath();
                          for (Object parentnode:nodepath) {
                              ((CheckBoxTreeNode)parentnode).setChecked(false);
                          }
                       }
                       if (!checknode.isLeaf()) {
                           Object userObject=checknode.getUserObject();
                           if (userObject==null) setCheckedRecursively(checknode,checknode.isChecked());
                       }
                       settings.redraw();
                       motifCollectionTree.repaint();
	            } else { // pressed the label
                       Object val=((CheckBoxTreeNode)node).getUserObject();
                       mousePressedInTree(e,val);                  
                    }                   
                }
                //
            }
            @Override
            public void mouseReleased(MouseEvent e) {
                Object node=null;
                int row = motifCollectionTree.getRowForLocation(e.getX(), e.getY());
                if (row<0) return;
                if (!motifCollectionTree.isRowSelected(row)) motifCollectionTree.setSelectionRow(row);                	  
                mouseReleasedInTree(e); 
                //motifCollectionTree.repaint();
            }            
        });

  }

/** Sets the check-status of all children of the specified node (if any) to the specified check value.
 * Note that the check-status of the specified node itself is not updated!
 */  
private void setCheckedRecursively(CheckBoxTreeNode node, boolean check) {
    int size=node.getChildCount();
    for (int i=0;i<size;i++) {
      CheckBoxTreeNode child=(CheckBoxTreeNode)node.getChildAt(i);
      child.setChecked(check);
      if (!child.isLeaf()) setCheckedRecursively(child,check);
    }
}


/** Sets the processed-status of all children of the specified node (if any) to the specified value.
 * The processed status of the node itself is also set!
 */  
private void setProcessedRecursively(CheckBoxTreeNode node, boolean processed) {
    node.setProcessed(processed);
    int size=node.getChildCount();
    for (int i=0;i<size;i++) {
      CheckBoxTreeNode child=(CheckBoxTreeNode)node.getChildAt(i);
      child.setProcessed(processed);
      if (!child.isLeaf()) setProcessedRecursively(child,processed);
    }
}


private void mousePressedInTree(java.awt.event.MouseEvent evt, Object selectedData) {                                             
    if (evt.getButton()==MouseEvent.BUTTON1 && evt.getClickCount()==2) {
         if (selectedData instanceof Data) showPrompt((Data)selectedData);
    }
    if (evt.isPopupTrigger()) {
        JPopupMenu popup=MotifsPanelContextMenu.getInstance(motifCollectionTree,gui,this);     
        if (popup!=null) popup.show(evt.getComponent(), evt.getX(),evt.getY());
    }  
}                                            

private void mouseReleasedInTree(java.awt.event.MouseEvent evt) {                                              
    if (evt.isPopupTrigger()) {
        JPopupMenu popup=MotifsPanelContextMenu.getInstance(motifCollectionTree,gui,this);     
        if (popup!=null) popup.show(evt.getComponent(), evt.getX(),evt.getY());
    }
}                                             
 /**  For some reason, if the new presentation name of a motif is wider than the original
   *  presentation name of the motif, the width of the cellrenderer will not update properly
   *  The only way I know to fix it is to replace the cellrenderer with a new one.
   *  Better solutions are gratefully acknowledged!! (send an email to me!)
   */
public void updateCellRenderer() {
     checkBoxTreeCellRenderer=new CheckBoxTreeCellRenderer();
     motifCollectionTree.setCellRenderer(checkBoxTreeCellRenderer);
}


/** Shows a prompt to edit the selected data object */
public void showPrompt(Data selectedData) {
     //gui.debugMessage("Doubled-clicked on "+selectedData.getName());
    //boolean mustUpdateRenderer=false;
    gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    if (selectedData instanceof Motif) {
          // String oldname=((Motif)selectedData).getPresentationName();
          Prompt_Motif prompt=new Prompt_Motif(gui, null, (Motif)selectedData.clone(),settings);
          prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
          prompt.disableNameEdit();
          prompt.setVisible(true);
          if (prompt.isOKPressed()) {
             Motif motif=(Motif)prompt.getData();
             if (motif!=null && !motif.containsSameData((Motif)selectedData)) gui.updateDataItem(motif);
             // if (!motif.getPresentationName().equals(oldname)) mustUpdateRenderer=true;
          }
          prompt.dispose();
          
     } else if (selectedData instanceof MotifCollection) {
          Prompt_MotifCollection prompt=new Prompt_MotifCollection(gui, null, (MotifCollection)selectedData.clone());
          prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
          prompt.disableNameEdit();
          prompt.setVisible(true);
          if (prompt.isOKPressed()) {
             MotifCollection collection=(MotifCollection)prompt.getData();
             if (collection!=null && !collection.containsSameData((MotifCollection)selectedData)) gui.updateDataItem(collection);
          }
          prompt.dispose();
     } 
     // if (mustUpdateRenderer) updateCellRenderer();
     gui.getFrame().setCursor(Cursor.getDefaultCursor());    
}


/** */
public void setColorOnSelectedRows(Color color) {
    int[] rows=motifCollectionTree.getSelectionModel().getSelectionRows();
    if (rows==null) return;    
    Arrays.sort(rows);
    HashSet<String> affectedClasses=new HashSet<String>();
    for (int row:rows) {
        CheckBoxTreeNode node=(CheckBoxTreeNode)motifCollectionTree.getPathForRow(row).getLastPathComponent();
        if (node.isProcessed()) continue;
        if (node.isLeaf()) { // let us say that classes or groups can be leaf nodes too...
            Object data=node.getUserObject();
            if (data instanceof Motif) {
                settings.setFeatureColor(data.toString(), color, false);
                if (!(model instanceof MotifsPanelTreeModel_GroupByClass) && data instanceof Motif) {
                    String motifClassification=((Motif)data).getClassification();
                    if (motifClassification==null) motifClassification=MotifClassification.UNKNOWN_CLASS_LABEL;
                    affectedClasses.add(motifClassification);
                }
            } else {
                if (model instanceof MotifsPanelTreeModel_GroupByClass) affectedClasses.add(node.getName());
            }                     
        } // end node is leaf
        else { // internal node (class or group (MotifCollection) )
            String nodeName=node.getName();
            settings.setClassLabelColor(nodeName,color);
            TreeNode parent=node.getParent();
            if (parent instanceof CheckBoxTreeNode && !((CheckBoxTreeNode)parent).isProcessed()) clearParentColors(node);
            if (model instanceof MotifsPanelTreeModel_GroupByClass) {
                affectedClasses.add(nodeName);
                setColorsRecursively(node, color,null);
            }
            else setColorsRecursively(node, color,affectedClasses);
            setProcessedRecursively(node, true);
        } 
    }   
    if (model instanceof MotifsPanelTreeModel_GroupByClass) setSubClassColors(affectedClasses,color);
    else clearSuperClassColors(affectedClasses);
    settings.redraw();
    setProcessedRecursively((CheckBoxTreeNode)motifCollectionTree.getModel().getRoot(),false); // clear processed flags                   
}

private void setSubClassColors(HashSet<String> affected, Color newColor) {
    for (String classname:affected) {
        ArrayList<String> list=MotifClassification.getAllSubLevels(classname);
        for (String c:list) {
            settings.setClassLabelColor(c,newColor);
        }
    }
}
private void clearSuperClassColors(HashSet<String> affected) {
    for (String classname:affected) {
        if (classname.equals(MotifClassification.UNKNOWN_CLASS_LABEL)) {
            settings.setClassLabelColor(MotifClassification.UNKNOWN_CLASS_LABEL,null);
        } else {
            ArrayList<String> list=MotifClassification.getAllSuperLevels(classname);
            for (String c:list) {
                settings.setClassLabelColor(c,null);
            }
        }
    }
}

private void clearParentColors(CheckBoxTreeNode node) {
    TreeNode[] pathtoroot=node.getPath();
    for (int i=pathtoroot.length-2;i>0;i--) {
        if (pathtoroot[i] instanceof CheckBoxTreeNode) {
            CheckBoxTreeNode parent=(CheckBoxTreeNode)pathtoroot[i];
            if (parent.isProcessed()) break;
            else {
                settings.setClassLabelColor(parent.getName(),null);
                parent.setProcessed(true);
            }
        }
    }    
}

/** 
 * Sets the color of all motifs below the specified parent node to the given color
 */  
private void setColorsRecursively(CheckBoxTreeNode node, Color color, HashSet<String> labels) {
    int size=node.getChildCount();
    for (int i=0;i<size;i++) {
      CheckBoxTreeNode child=(CheckBoxTreeNode)node.getChildAt(i);
      if (!child.isLeaf()) {
          settings.setClassLabelColor(child.getName(),color);
          setColorsRecursively(child,color, labels);
      }
      else {
          Object data=child.getUserObject();
          if (data instanceof Motif) {
              settings.setFeatureColor(data.toString(), color, false);
              if (labels!=null) {
                  String motifClass=((Motif)data).getClassification();
                  if (motifClass==null) motifClass=MotifClassification.UNKNOWN_CLASS_LABEL;
                  labels.add(motifClass);
              }
          }
      }
    }
}

/** */
public void setVisibilityOnSelectedRows(boolean show) {
    int[] rows=motifCollectionTree.getSelectionModel().getSelectionRows();
    if (rows==null) return;
    Arrays.sort(rows);
    for (int row:rows) {
        CheckBoxTreeNode node=(CheckBoxTreeNode)motifCollectionTree.getPathForRow(row).getLastPathComponent();
        if (node.isProcessed()) continue;
        if (node.isLeaf()) {
            Object data=node.getUserObject();
            if (data instanceof Motif) {settings.setFeatureVisible(data.toString(), show, false);}
        }
        else {
            node.setChecked(show);
            setVisibilityRecursively(node, show);
            setProcessedRecursively(node, true);
        } 
    }                    
    setProcessedRecursively((CheckBoxTreeNode)motifCollectionTree.getModel().getRoot(),false); // clear processed flags                   
    settings.redraw();
}


/** 
 * Sets the visibility of all motifs below the specified parent node to the given boolean value
 */  
private void setVisibilityRecursively(CheckBoxTreeNode node, boolean show) {
    int size=node.getChildCount();
    for (int i=0;i<size;i++) {
      CheckBoxTreeNode child=(CheckBoxTreeNode)node.getChildAt(i);
      if (!child.isLeaf()) {child.setChecked(show);setVisibilityRecursively(child,show);}
      else {
          Object data=child.getUserObject();
          if (data instanceof Motif) {settings.setFeatureVisible(data.toString(), show, false);}
      }
    }
}

 /**
  * This method is called when the "add data object" button (next to the panel label) is pressed
  */
 private void addDataObject(java.awt.event.MouseEvent evt) {
    Component invoker=evt.getComponent();
    int x=evt.getX();
    int y=evt.getY();
    addPopupMenu.show(invoker, x,y);
 }
 
 /**
  * This method is called when the "group motifs" button (next to the panel label) is pressed
  */
 private void setGrouping(java.awt.event.MouseEvent evt) {
    Component invoker=evt.getComponent();
    int x=evt.getX();
    int y=evt.getY();
    groupPopupMenu.checkColorByClassMenuItem(settings.shouldColorMotifsByClass());
    groupPopupMenu.show(invoker, x,y);
 }
 
 
 private void setGroupingNone() {
     if (model!=null) model.removeTreeModelListener(treelistener);
     model=new MotifsPanelTreeModel_NoGroup(gui.getEngine(),this);
     model.addTreeModelListener(treelistener);
     motifCollectionTree.setModel(model);

 }
 private void setGroupingByCollection() {
     if (model!=null) model.removeTreeModelListener(treelistener);
     model=new MotifsPanelTreeModel_GroupByCollection(gui.getEngine(),this);
     model.addTreeModelListener(treelistener);
     motifCollectionTree.setModel(model);     
 }

 private void setGroupingByClass(int levels) {
     if (model!=null) model.removeTreeModelListener(treelistener);
     model=new MotifsPanelTreeModel_GroupByClass(gui.getEngine(),levels,this);
     model.addTreeModelListener(treelistener);
     motifCollectionTree.setModel(model);
 }


 
 // ---- data listener interface ----
    private String updatedMotif=null; // name of motif that is updated
    @Override public void dataAdded(Data data) {}
    @Override public void dataAddedToSet(Data parentDataset, Data child) {}
    @Override public void dataRemoved(Data data) {}
    @Override public void dataRemovedFromSet(Data parentDataset, Data child) {}
    @Override public void dataUpdated(Data data) { // this method is set up by dataUpdate below and used if needed
        if (updatedMotif!=null && data.getName().equals(updatedMotif)) {
            updateCellRenderer();
            updatedMotif=null;
        }
    }

    @Override
    public void dataUpdate(Data oldvalue, Data newvalue) {
        if (!(newvalue instanceof Motif && oldvalue instanceof Motif)) return;
        if (!((Motif)newvalue).getPresentationName().equals(((Motif)oldvalue).getPresentationName())) updatedMotif=newvalue.getName();
    }

 
 
 
 
 
 
 // ------------------------------------------- INNER CLASSES ---------------------------------

 
 private class addDataPopupMenu extends JPopupMenu {
     public addDataPopupMenu() {
         JMenuItem addCollection=new JMenuItem(MotifCollection.getType());
         JMenuItem addMotif=new JMenuItem(Motif.getType());
         addDataPopupMenuListener listener=new addDataPopupMenuListener();
         addCollection.addActionListener(listener);
         addMotif.addActionListener(listener);
         add(addCollection);
         add(addMotif);
     }
 }
 
 private class addDataPopupMenuListener implements ActionListener {
     @Override
     public void actionPerformed(ActionEvent e) {
         gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
         Prompt prompt=null;
         String type=e.getActionCommand();
              if (type.equals(MotifCollection.getType())) prompt=new Prompt_MotifCollection(gui, null, null);
         else if (type.equals(Motif.getType())) prompt=new Prompt_Motif(gui, null, null,settings);
         prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
         prompt.setVisible(true);
         gui.getFrame().setCursor(Cursor.getDefaultCursor());
         if (prompt.isOKPressed()) {
            Data variable=prompt.getData();
            if (variable==null) return;
            OperationTask task=new OperationTask("new "+type);
            task.setParameter(OperationTask.OPERATION_NAME, "new");
            task.setParameter(Operation_new.DATA_TYPE, type);
            task.setParameter(OperationTask.TARGET_NAME,variable.getName());
            task.setParameter(OperationTask.ENGINE,gui.getEngine());
            variable.setAdditionalOperationNewTaskParameters(task);
            
            if (prompt instanceof Prompt_MotifCollection && ((Prompt_MotifCollection)prompt).isImported()) {
                gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                CompoundTask compoundTask=new CompoundTask("new "+MotifCollection.getType());
                MotifCollection collection=(MotifCollection)variable;
                List<Motif> motifslist=collection.getPayload();
                String parameterString="";
                for (int i=0;i<motifslist.size();i++) {
                    Motif motif=motifslist.get(i);
                    String motifname=motif.getName();
                    if (i==motifslist.size()-1) parameterString+=motifname;
                    else parameterString+=motifname+",";
                    OperationTask newmotiftask=new OperationTask("new");
                    newmotiftask.setParameter(OperationTask.OPERATION_NAME, "new");
                    newmotiftask.setParameter(Operation_new.DATA_TYPE, Motif.getType());
                    newmotiftask.setParameter(OperationTask.TARGET_NAME,motifname);
                    newmotiftask.setParameter(OperationTask.ENGINE,gui.getEngine());   
                    newmotiftask.setParameter(Operation_new.PARAMETERS, motif.getValueAsParameterString());
                    compoundTask.addTask(newmotiftask);
                }
                task.setParameter(Operation_new.PARAMETERS, parameterString); // new MotifCollection task
                compoundTask.addTask(task);
                gui.launchOperationTask(compoundTask,gui.isRecording());
                gui.getFrame().setCursor(Cursor.getDefaultCursor());
            } else {
                task.setParameter(Operation_new.PARAMETERS, variable.getValueAsParameterString());
                gui.launchOperationTask(task,gui.isRecording());
            }
         }
    } 
}

 private class groupPopupMenu extends JPopupMenu implements ActionListener {
     private JMenuItem colorByClassItem;
     public groupPopupMenu() {
         JMenuItem noGroupsItem=new JCheckBoxMenuItem(GROUP_BY_MOTIFS);
         JMenuItem groupByCollectionsItem=new JCheckBoxMenuItem(GROUP_BY_MOTIF_COLLECTIONS);
         JMenu groupByClassesItem=new JMenu(GROUP_BY_CLASS);
         JMenuItem level1Item=new JCheckBoxMenuItem("1 class level");
         JMenuItem level2Item=new JCheckBoxMenuItem("2 class levels");
         JMenuItem level3Item=new JCheckBoxMenuItem("3 class levels");
         JMenuItem level4Item=new JCheckBoxMenuItem("4 class levels");
         groupByClassesItem.add(level1Item);
         groupByClassesItem.add(level2Item);
         groupByClassesItem.add(level3Item);
         groupByClassesItem.add(level4Item);
         colorByClassItem=new JCheckBoxMenuItem("Color by class");
         //colorByClassItem=new JMenuItem("Color by class");
         noGroupsItem.addActionListener(this);
         groupByCollectionsItem.addActionListener(this);
         level1Item.addActionListener(this);
         level2Item.addActionListener(this);
         level3Item.addActionListener(this);
         level4Item.addActionListener(this);
         colorByClassItem.addActionListener(this);
         add(noGroupsItem);
         add(groupByCollectionsItem);
         add(groupByClassesItem);
         add(new JSeparator());
         add(colorByClassItem);
         ButtonGroup buttonsgroup=new ButtonGroup();
         buttonsgroup.add(noGroupsItem);
         buttonsgroup.add(groupByCollectionsItem);
         buttonsgroup.add(level1Item);      
         buttonsgroup.add(level2Item);         
         buttonsgroup.add(level3Item);         
         buttonsgroup.add(level4Item);         
         noGroupsItem.setSelected(true);//groupByCollectionsItem.setSelected(true);
     }
     
    @Override
    public void actionPerformed(ActionEvent e) {
        String cmd=e.getActionCommand();
        if (cmd.equals(GROUP_BY_MOTIFS)) setGroupingNone();
        else if (cmd.equals(GROUP_BY_MOTIF_COLLECTIONS)) setGroupingByCollection();
        else if (cmd.startsWith("1"))  setGroupingByClass(1);
        else if (cmd.startsWith("2"))  setGroupingByClass(2);
        else if (cmd.startsWith("3"))  setGroupingByClass(3);
        else if (cmd.startsWith("4"))  setGroupingByClass(4);    
        else if (cmd.equals("Color by class")) {
            settings.setColorMotifsByClass(!settings.shouldColorMotifsByClass()); // going from not color by class to do color by class -> set colors again!
            motifCollectionTree.repaint();
        }
    }
    public void checkColorByClassMenuItem(boolean flag) {colorByClassItem.setSelected(flag);}
 }
 
private class CheckBoxTreeCellRenderer extends JPanel implements TreeCellRenderer {
   DefaultTreeCellRenderer tcr;
   JCheckBox button = new JCheckBox();
   JLabel label = new JLabel();
   SimpleDataPanelIcon unselectedIcon=new SimpleDataPanelIcon(12,12,SimpleDataPanelIcon.COLOR_ICON,SimpleDataPanelIcon.SIMPLE_BORDER, null);
   SimpleDataPanelIcon selectedicon=new SimpleDataPanelIcon(12,12,SimpleDataPanelIcon.COLOR_ICON,SimpleDataPanelIcon.SIMPLE_BORDER, null);
   
   public boolean isOnHotspot(int x, int y) {
     return (button.getBounds().contains(x, y));
   }

   public CheckBoxTreeCellRenderer() {
     super();
     tcr=new DefaultTreeCellRenderer();
     label.setOpaque(false);
     button.setOpaque(false);
     setOpaque(false);
     setLayout(new FlowLayout(FlowLayout.LEADING,0,0));
     add(button);
     add(label);
     Dimension d=new Dimension(18,18);
     button.setMinimumSize(d);
     button.setPreferredSize(d);
     button.setMaximumSize(d);
     unselectedIcon.setForegroundColor(java.awt.Color.WHITE);
     unselectedIcon.setBorderColor(java.awt.Color.LIGHT_GRAY);
   }
    
    
    @Override
    public Component getTreeCellRendererComponent(JTree tree, Object value, boolean isSelected,boolean expanded,boolean leaf, int row,boolean hasFocus) {
        if (isSelected) label.setForeground(tcr.getTextSelectionColor());
        else label.setForeground(tcr.getTextNonSelectionColor());
        button.setIcon(null);
        button.setSelectedIcon(null);   
        button.setSelected(((CheckBoxTreeNode)value).isChecked());
//        if (leaf) label.setIcon(null);     
//        else if (expanded) label.setIcon(getOpenIcon());           
//        else label.setIcon(getClosedIcon());      
        boolean doColorByClass=settings.shouldColorMotifsByClass();
        Object valueAsObject=((CheckBoxTreeNode)value).getUserObject();  
        if (valueAsObject instanceof String) {
           String stringValue=(String)valueAsObject;
           if (stringValue.matches("^[0-9\\.]+")) { // node represents a motif class               
               if (doColorByClass) {
                  Color labelColor=settings.getClassLabelColor(stringValue);
                  if (labelColor!=null) {
                      selectedicon.setForegroundColor(labelColor);
                      button.setSelectedIcon(selectedicon); 
                      button.setIcon(unselectedIcon);      
                  }
               }
               stringValue=stringValue+":"+MotifClassification.getNameForClass(stringValue);
           } else if (stringValue.equals(MotifClassification.UNKNOWN_CLASS_LABEL) && doColorByClass) {
                  Color labelColor=settings.getClassLabelColor(stringValue);
                  if (labelColor!=null) {
                      selectedicon.setForegroundColor(labelColor);
                      button.setSelectedIcon(selectedicon);                 
                      button.setIcon(unselectedIcon);  
                  }
           }
           setToolTipText("Transcription factor class");
           label.setText(stringValue);
           return this; 
        } 

        Data data=null;
        if (valueAsObject instanceof Data) data=(Data)valueAsObject;
	String stringValue=null;
        if (data instanceof Motif) stringValue=((Motif)data).getPresentationName();
        else stringValue=valueAsObject.toString();
        label.setText(stringValue);

        if (data instanceof Motif) {
            //icon.setForegroundColor(settings.getFeatureColor(data.getName()));
            selectedicon.setForegroundColor(settings.getFeatureColor(data.getName()));
            button.setSelectedIcon(selectedicon);
            button.setIcon(unselectedIcon);
            setToolTipText(data.getName()); // use only the motif name here, since this will be used as a key to look up the motif itself          
        } else if (data instanceof MotifCollection) {
            setToolTipText(data.getName()+"   [ "+data.getTypeDescription()+" ]");
        } else if (data==null) {
            System.err.println("Got NULL data in TreeNodeRenderer");
            setToolTipText("Nothing here...");    
        }      
        setComponentOrientation(tree.getComponentOrientation());	    
	//selected = isSelected;
        return this;
    }
    
}

private class TreeListener implements TreeModelListener {
    @Override public void treeNodesChanged(TreeModelEvent e) {}
    @Override public void treeNodesRemoved(TreeModelEvent e) {}
    @Override public void treeStructureChanged(TreeModelEvent e) {}
    @Override public void treeNodesInserted(TreeModelEvent e) {motifCollectionTree.expandPath(e.getTreePath());}
}

}
