/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.Data;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Collections;
import javax.swing.JColorChooser;
import javax.swing.JMenu;
import javax.swing.JSeparator;
import javax.swing.JTree;
import javax.swing.tree.TreeNode;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.MotifClassification;
import priorseditor.engine.data.MotifCollection;



/**
 *
 * @author kjetikl
 */
public class MotifsPanelContextMenu extends JPopupMenu  {
    private static String EDIT_DATA="Edit";
    private static String RENAME_DATA="Rename";
    private static String DELETE_DATA="Delete";
    private static String SHOW="Show";
    private static String HIDE="Hide";
    private static String COLOR_SUBMENU_HEADER="Set color";
    private static String COMPARE_TO_KNOWN_MOTIFS="Compare to other motifs";

    private static String OPERATION_SUBMENU_HEADER="Perform Operation";
    private Object[] colors=new Object[] {
         new Object[]{"BLACK", Color.BLACK},  
         new Object[]{"WHITE", Color.WHITE},  
         new Object[]{"RED", Color.RED},  
         new Object[]{"ORANGE", new Color(255,204,0)},
         new Object[]{"YELLOW", Color.YELLOW},  
         new Object[]{"GREEN", Color.GREEN},  
         new Object[]{"CYAN", Color.CYAN},  
         new Object[]{"LIGHT BLUE", new Color(46,153,255)},  
         new Object[]{"BLUE", Color.BLUE},  
         new Object[]{"VIOLET", new Color(153,51,255)},
         new Object[]{"MAGENTA", Color.MAGENTA}          
    };


    private PriorsEditorEngine engine;
    private JTree tree;
    private MotifsPanel panel;
    private PriorsEditorGUI gui;
    private JMenu operationsMenu;
    private Data dataitem=null;
    private CheckBoxTreeNode node;
    
    public static MotifsPanelContextMenu getInstance(JTree tree, PriorsEditorGUI gui, MotifsPanel panel) {
         if (tree.isSelectionEmpty()) return null;
         return new MotifsPanelContextMenu(tree,gui,panel);
    }
    
        private MotifsPanelContextMenu(JTree tree, PriorsEditorGUI gui,MotifsPanel panel) {
            this.engine=gui.getEngine();
            this.tree=tree;            
            this.gui=gui;
            this.panel=panel;            
            int[] selection=tree.getSelectionRows();
            if (selection!=null && selection.length==1) {
                node=(CheckBoxTreeNode)tree.getPathForRow(selection[0]).getLastPathComponent();
                Object userObject=node.getUserObject();
                if (userObject instanceof Data) dataitem=(Data)userObject;
            } else node=null;
            DisplayMenuItemListener menuItemListener=new DisplayMenuItemListener();
            JMenuItem showItem=new JMenuItem(SHOW);
            JMenuItem hideItem=new JMenuItem(HIDE);
            showItem.addActionListener(menuItemListener);
            hideItem.addActionListener(menuItemListener);
            ColorMenuListener colormenulistener=new ColorMenuListener();
            JMenu colorMenu=new JMenu(COLOR_SUBMENU_HEADER);
            for (Object entry:colors) {
                  String colorname=(String)((Object[])entry)[0];
                  Color color=(Color)((Object[])entry)[1];
                  JMenuItem menuitem=new JMenuItem(colorname);
                  menuitem.setIcon(new SimpleDataPanelIcon(12,12,color));
                  menuitem.addActionListener(colormenulistener);
                  colorMenu.add(menuitem);
            }
            JMenuItem menuitemOther=new JMenuItem("Other...");
            menuitemOther.addActionListener(colormenulistener);
            colorMenu.add(menuitemOther);
            this.add(showItem);
            this.add(hideItem);
            this.add(colorMenu);
            if (dataitem!=null) {
                JMenuItem editDataItem=new JMenuItem(EDIT_DATA+" \""+dataitem.getName()+"\"");
                editDataItem.addActionListener(menuItemListener);
                this.add(editDataItem);
                if (!gui.getScheduler().isIdle()) editDataItem.setEnabled(false);  
                if (dataitem instanceof Motif) {
                    JMenuItem compareMotifItem=new JMenuItem(COMPARE_TO_KNOWN_MOTIFS);
                    compareMotifItem.addActionListener(menuItemListener);
                    this.add(compareMotifItem);                
                }
                if (dataitem instanceof MotifCollection) {
                    JMenuItem renameDataItem=new JMenuItem(RENAME_DATA+" \""+dataitem.getName()+"\"");
                    JMenuItem deleteDataItem=new JMenuItem(DELETE_DATA+" \""+dataitem.getName()+"\"");
                    renameDataItem.addActionListener(menuItemListener);
                    deleteDataItem.addActionListener(menuItemListener);
                    this.add(renameDataItem);
                    this.add(deleteDataItem);
                    if (!gui.getScheduler().isIdle()) {
                        deleteDataItem.setEnabled(false);
                        renameDataItem.setEnabled(false);
                    }
                }               
                ArrayList<Operation> operationslist=engine.getAllOperations();
                Collections.sort(operationslist, new NameComparator());                
                OperationsMenuListener menulistener=new OperationsMenuListener();
                operationsMenu=new JMenu(OPERATION_SUBMENU_HEADER);
                boolean hasOperations=false;
                for (Operation operation:operationslist) {
                      if (operation.canUseAsSource(dataitem)) {
                      JMenuItem menuitem=new JMenuItem(operation.getName());
                      menuitem.addActionListener(menulistener);
                      operationsMenu.add(menuitem);
                      hasOperations=true;
                    }
                }
                if (hasOperations) {
                    this.add(new JSeparator());
                    this.add(operationsMenu);
                }
            }
        }
        
        
        /**
         * An inner class that listens to popup-menu events related to the operations submenu and notifies the gui events
         */
        private class OperationsMenuListener implements ActionListener {
           public void actionPerformed(ActionEvent e) {
                Operation operation=engine.getOperation(e.getActionCommand());
                OperationTask parameters=new OperationTask(operation.getName());
                parameters.setParameter(OperationTask.OPERATION, operation);
                parameters.setParameter(OperationTask.SOURCE, dataitem);
                gui.launchOperationEditor(parameters);
           }            
        }
        
         /**
         * An inner class that listens to popup-menu events NOT related to the operations submenu and notifies the gui events
         */       
        private class DisplayMenuItemListener implements ActionListener {
           public void actionPerformed(ActionEvent e) {
               if (e.getActionCommand().startsWith(RENAME_DATA)) {
                    RenameDataDialog dialog=new RenameDataDialog(gui.getFrame(), gui, dataitem.getName());
                    dialog.setLocation(100, 250);    
                    dialog.setVisible(true);
                    dialog.dispose();
               } else if (e.getActionCommand().startsWith(DELETE_DATA)) {
                    gui.deleteDataItem(dataitem.getName());
               } else if (e.getActionCommand().startsWith(EDIT_DATA)) {
                    panel.showPrompt(dataitem);
               } else if (e.getActionCommand().startsWith(SHOW)) {
                    if (dataitem instanceof Motif) {gui.getVisualizationSettings().setFeatureVisible(dataitem.getName(),true,true);}
                    else {panel.setVisibilityOnSelectedRows(true);}
               } else if (e.getActionCommand().startsWith(HIDE)) {
                    if (dataitem instanceof Motif) {gui.getVisualizationSettings().setFeatureVisible(dataitem.getName(),false,true);}
                    else {panel.setVisibilityOnSelectedRows(false);}
               } else if (e.getActionCommand().startsWith(COMPARE_TO_KNOWN_MOTIFS)) {
                    MotifComparisonPanel motifcomparisonPanel=new MotifComparisonPanel(gui, (Motif)dataitem);
                    motifcomparisonPanel.setLocation(gui.getFrame().getWidth()/2-motifcomparisonPanel.getWidth()/2, gui.getFrame().getHeight()/2-motifcomparisonPanel.getHeight()/2);
                    motifcomparisonPanel.setVisible(true);
                    MotifCollection newCollection=motifcomparisonPanel.getMotifCollection();                    
                    motifcomparisonPanel.dispose();
                    if (newCollection!=null) {
                        gui.updateDataItem(newCollection);
                    }
               } 
           }            
        }
        
        
        /**
         * Just an inner class to sort available operations alphabetically in the popup menu
         */
        private class NameComparator implements Comparator<Operation> {        
            public int compare(Operation o1, Operation o2) {
                return o1.getName().compareTo(o2.getName());
            }
            public boolean equals(Operation o1, Operation o2) {
                return o1.getName().equals(o2.getName());
            }           
        }


        private void setColorForMotif(Motif motif, Color newColor) {
            VisualizationSettings settings=gui.getVisualizationSettings();
            settings.setFeatureColor(motif.getName(), newColor, true);
            if (node!=null) { // clear parent nodes colors
                TreeNode[] pathtoroot=node.getPath();
                for (int i=pathtoroot.length-2;i>0;i--) {
                    if (pathtoroot[i] instanceof CheckBoxTreeNode) {
                        CheckBoxTreeNode parent=(CheckBoxTreeNode)pathtoroot[i];
                        settings.setClassLabelColor(parent.getName(),null);                     
                    }
                } 
            }
             // clear also colors from parent class labels
            String motifclass=((Motif)motif).getClassification();
            if (motifclass==null) {
                settings.setClassLabelColor(MotifClassification.UNKNOWN_CLASS_LABEL, null);
            } else {
                String[] path=MotifClassification.getClassPath(motifclass);
                for (int i=0;i<path.length;i++) settings.setClassLabelColor(path[i], null);
            }
        }
        
        /**
         * An inner class that listens to popup-menu events related to the colors submenu
         */
        private class ColorMenuListener implements ActionListener {
           public void actionPerformed(ActionEvent e) {
                String colorname=e.getActionCommand();
                Color color=null;
                if (dataitem instanceof Motif) color=gui.getVisualizationSettings().getFeatureColor(dataitem.getName());

                if (colorname.equals("Other...")) {
                    color = JColorChooser.showDialog(gui.getFrame(), "Select motif color", color);
                } else {
                   for (Object entry:colors) {
                      String name=(String)((Object[])entry)[0];
                      if (name.equals(colorname)) {color=(Color)((Object[])entry)[1];break;}
                   }
                }
                if (dataitem instanceof Motif) {setColorForMotif((Motif)dataitem,color);}
                else {panel.setColorOnSelectedRows(color);}
             }
        }
        
}
