/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.data.Data;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;

import java.awt.datatransfer.*;
import javax.swing.TransferHandler;
import javax.swing.JList;
import javax.swing.JComponent;


/**
 * This class implements support for DataTransfer (Cut/Copy/Paste and Drag'n'Drop)
 * from and to the operations panel that lists all available operations. 
 * Specifically, if an Operation is dragged onto the Procotol Editor, 
 * the name of the operation should be copied to the caret position. 
 * If an Operation is dropped (or pasted) onto a dataset in the one of the 
 * dataset panels(or vice versa) the operation-editor should pop up with the 
 * dataset preselected as source and target
 * 
 * @author kjetikl
 */
public class OperationsPanelTransferHandler extends TransferHandler {
    private JList list;
    private PriorsEditorGUI gui;
    
          public OperationsPanelTransferHandler(JList list, PriorsEditorGUI gui) {
              this.list=list;
              this.gui=gui;
          }
    
    
          @Override
          /**
           * This method is called repeatedly during a drag gesture to confirm
           * whether the area below the cursor can accept the datatransfer.
           * If it can accept the transfer at the current drop location the function
           * should return true, if not it should return false
           */
          public boolean canImport(TransferHandler.TransferSupport info) {
                // we only import Strings
                if (!info.isDataFlavorSupported(DataFlavor.stringFlavor)) {
                    return false;
                }
                // OK, the import supports String transfer but is the location right?
                if (info.isDrop()) {
                  JList.DropLocation dl = (JList.DropLocation)info.getDropLocation();
                  if (dl.getIndex() == -1) {
                    return false;
                  }
                }
                return true;
            }
          
            /**
             * The following method is called automatically on a successful drop
             * and should handle the actual import processing. The method should
             * return true if the import of the dataobject went 
             * @param info
             * @return
             */
            @Override
            public boolean importData(TransferHandler.TransferSupport info) {
                // just to be sure, check one more time if we can import data of this type
                if (!canImport(info)) {
                    return false;
                }
                // Since TransferHandlers and TransferSupport can also be used by Cut/Copy/Paste
                // the following method call checks whether importData was called as the 
                // result of a drag'n'drop gesture or the result of a paste
                if (!info.isDrop()) { // if this is not the result of drag'n'drop we don't bother
                    return false;
                }
                JList.DropLocation droplocation = (JList.DropLocation)info.getDropLocation();
                OperationsPanelListModel listModel = (OperationsPanelListModel)list.getModel();
                PriorsEditorEngine engine=listModel.getEngine();               
                int index = droplocation.getIndex();
                boolean insert = droplocation.isInsert();              
                // Get the string that is being dropped.
                Transferable t = info.getTransferable();
                String dragSourceName="<unknown>";
                try {
                    dragSourceName = (String)t.getTransferData(DataFlavor.stringFlavor);
                } 
                catch (Exception e) { 
                        ((FeaturesPanelListModel)list.getModel()).getEngine().errorMessage(e.getMessage(),0);
                        return false;
                } // unable to get data as string. Import is cancelled!
                
               
                // Display a dialog with the drop information.
            
                if (droplocation.isInsert()) {
                    if (droplocation.getIndex() == 0) {
                        //gui.debugMessage(dragSourceName + " dropped at beginning of list");
                    } else if (droplocation.getIndex() >= listModel.getSize()) {
                        //gui.debugMessage(dragSourceName + " dropped at end of list");
                    } else {
                        String value1 = ((Operation)listModel.get(droplocation.getIndex()-1)).getName();  
                        String value2 = ((Operation)listModel.get(droplocation.getIndex())).getName();
                        //gui.debugMessage(dragSourceName + " dropped between \"" + value1 + "\" and \"" + value2 + "\"");
                    }
                } else {
                    String dropTargetName = ((Operation)listModel.get(droplocation.getIndex())).getName();
                    //gui.debugMessage(dragSourceName + " dropped on top of " + "\"" + dropTargetName + "\"");
                    Data dataset=gui.getEngine().getDataItem(dragSourceName);
                    if (dataset!=null) {
                        Operation operation=gui.getEngine().getOperation(dropTargetName);
                        OperationTask parameters=new OperationTask(operation.getName());
                        parameters.setParameter(OperationTask.OPERATION, operation);
                        parameters.setParameter(OperationTask.SOURCE, dataset);
                        gui.launchOperationEditor(parameters);
                    }
                }
                
		return true;
            }
            
            /**
             * This method declares which kinds of transfer actions are allowed
             * when the "DataPanel" (JList) component acts as a source for datatransfer.
             * In this case we only allow that datasets can be 'copied' out of
             * the list but never removed from it (at least not with DnD).
             * 
             * @param c 
             * @return
             */
            public int getSourceActions(JComponent c) {
                return COPY;
            }
            
            
            /* The createTransferable() method bundles up the data that should
             * be transferred when the component acts as a source for datatransfer
             * (a drag gesture is initiated on the component).   
             * 
             */
            @Override
            protected Transferable createTransferable(JComponent c) {
                JList list = (JList)c;
                Object selectedOperation = list.getSelectedValue();
                if (selectedOperation==null) return null;
                // ((OperationsPanelListModel)list.getModel()).getEngine().logMessage("Creating transferable of "+selectedOperation.toString());
                return new StringSelection(selectedOperation.toString()); // for now I only return a string representation of the selected item
            }
            



}
