/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.HashMap;
import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.Parameter;
import priorseditor.engine.data.NumericVariable;
import priorseditor.gui.operationdialog.OperationDialog;

/**
 *
 * @author kjetikl
 */
public class ParametersPanel extends JPanel {
    private HashMap<String,JComponent> additionalParameters; // this stores the input widgets for all parameters
    private Parameter[] parameterTemplates;
    private ParameterSettings parameters;
    private Font smallfont=new Font(Font.SERIF,Font.PLAIN,10);

    Icon informationIcon=null;
    OperationDialog dialog;

    /** */
    public ParametersPanel(Parameter[] parameterTemplates, ParameterSettings parameterValues, OperationDialog dialog) {
        this.parameterTemplates=parameterTemplates;
        this.parameters=parameterValues;
        this.dialog=dialog;
        if (dialog!=null) this.informationIcon=dialog.getInformationIcon();
        additionalParameters=new HashMap<String, JComponent>();
        this.setBorder(BorderFactory.createEmptyBorder(0,40,0,0));
        int visibleParameters=parameterTemplates.length;
        for (Parameter par:parameterTemplates) {
          if (par.isHidden()) visibleParameters--;
        }
        if (parameterTemplates==null || visibleParameters==0) return;
        //GridLayout layout=new GridLayout(visibleParameters,3   ,6,6); // 3 columns: label, widget and help-icon. The two last numbers "6,6" are padding
        this.setLayout(new GridBagLayout());
        int row=-1;
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.anchor=GridBagConstraints.LINE_START;
        constraints.ipadx=10;
        constraints.ipady=5;
        //constraints.insets=new Insets(6,20,6,20);
        for (Parameter par:parameterTemplates) {
            if (par.isHidden()) continue;
            row++;
            constraints.gridy=row;
            constraints.gridx=0;
            String parametername=par.getName();
            JLabel namelabel=new JLabel(parametername);
            JLabel helplabel;
            if (par.getDescription()!=null) {
                helplabel=new JLabel(informationIcon);
                helplabel.setToolTipText(par.getDescription());
            } else helplabel=new JLabel("");
            this.add(namelabel,constraints);
            Object userSelected=null;
            if (parameters!=null) userSelected=parameters.getParameterAsString(parametername,parameterTemplates);
            else userSelected=par.getDefaultValue();
            JComponent newcomponent=getWidgetFor(par.getType(),par.getDefaultValue(),userSelected, par.getAllowedValues(), par.isOptional());
            additionalParameters.put(parametername,newcomponent);
            constraints.fill=GridBagConstraints.HORIZONTAL;
            constraints.gridx=1;
            this.add(newcomponent,constraints);
            constraints.fill=GridBagConstraints.NONE;
            JPanel additionals=new JPanel(new FlowLayout(FlowLayout.LEADING));
            additionals.add(helplabel);
            if (par.isRequired()) {
                JLabel isRequiredLabel=new JLabel("Required");
                isRequiredLabel.setForeground(java.awt.Color.RED);
                isRequiredLabel.setFont(smallfont);
                additionals.add(isRequiredLabel);
            } else additionals.add(new JLabel(" "));
            constraints.gridx=2;
            this.add(additionals,constraints);
        }
    }

    /** Copies the values of the widgets into the dialogs ParameterSettings object which can be retrieved with getParameterSettings() */
    public void setParameters() {
            if (parameters==null) parameters=new ParameterSettings();
            if (parameterTemplates!=null && parameterTemplates.length>0) {
                for (Parameter par:parameterTemplates) {
                   Object value;
                   String parametername=par.getName();
                   Class parameterclass=par.getType();
                   Object factoryDefault=par.getDefaultValue();
//                   if (factoryDefault==null) { // just in case
//                            if (parameterclass==Boolean.class) factoryDefault=Boolean.FALSE;
//                       else if (parameterclass==Integer.class) factoryDefault=new Integer(0);
//                       else if (parameterclass==Float.class) factoryDefault=new Float(0);
//                       else if (parameterclass==String.class) factoryDefault="";
//                   }
                   if (par.isHidden()) {
                       parameters.setParameter(parametername, factoryDefault);
                       continue;
                   }
                   JComponent widget=additionalParameters.get(parametername);
                   if (parameterclass==Boolean.class) {
                       if (((JCheckBox)widget).isSelected()) value=Boolean.TRUE; else value=Boolean.FALSE;
                   } else if (Number.class.isAssignableFrom(parameterclass)) {
                        if (widget instanceof JSpinner) {
                            value=((JSpinner)widget).getValue();
                            if (value instanceof Double) value=new Float(((Double)value).floatValue());
                        }
                        else if (widget instanceof JComboBox) value=(String)((JComboBox)widget).getSelectedItem();
                        else if (widget instanceof JTextField) value=((JTextField)widget).getText().trim();
                        else value=null; // this should preferably not happen
                   } else { // value should be a string?
                       if (widget instanceof JComboBox) value=(String)((JComboBox)widget).getSelectedItem();
                       else if (widget instanceof JTextField) value=postprocess(((JTextField)widget).getText().trim());
                       else value=null; // this should preferably not happen
                   }
                   parameters.setParameter(parametername, value);
                } // end for each parameter
            } // end additional parameters > 0
    }

    /**
     *
     * @return
     */
    public ParameterSettings getParameterSettings() {
        return parameters;
    }

    private JComponent getWidgetFor(Class type, Object factoryDefaultValue, Object userSelected, Object values, boolean optional) {
        //if (dialog==null) type=String.class; // just a simple hack to avoid displaying "data" in widgets when dialog is NULL
        if (type==Integer.class && dialog!=null) {
            DefaultComboBoxModel boxmodel;
            boxmodel=dialog.getDataCandidates(NumericVariable.class);
            JComboBox box=new JComboBox(boxmodel);
            box.setEditable(true);
            //if (userSelected==null) userSelected=(factoryDefaultValue!=null)?factoryDefaultValue:new Integer(0);
            if (userSelected==null) userSelected=factoryDefaultValue;
            box.setSelectedItem(userSelected);
            return box;
        } else if (type==Float.class && dialog!=null) {
            DefaultComboBoxModel boxmodel;
            boxmodel=dialog.getDataCandidates(NumericVariable.class);
            JComboBox box=new JComboBox(boxmodel);
            box.setEditable(true);
            // if (userSelected==null) userSelected=(factoryDefaultValue!=null)?factoryDefaultValue:new Float(0);
            if (userSelected==null) userSelected=factoryDefaultValue;
            box.setSelectedItem(userSelected);
            return box;
        } if (type==Integer.class && dialog==null) {
            //if (userSelected==null) userSelected=(factoryDefaultValue!=null)?factoryDefaultValue:new Integer(0);
            if (userSelected==null) userSelected=factoryDefaultValue;
            int selected=1;
            if (userSelected!=null) {
                if (userSelected instanceof Integer) selected=(Integer)userSelected;
                else if (userSelected instanceof String) {
                    try {selected=Integer.parseInt((String)userSelected);} catch (NumberFormatException e) {}
                }
            }
            int step=1;
            int min=Integer.MIN_VALUE;
            int max=Integer.MAX_VALUE;
            if (values!=null && values instanceof Integer[]) {
                min=((Integer[])values)[0];
                max=((Integer[])values)[1];
            }
            JSpinner widget=new JSpinner(new SpinnerNumberModel(selected, min, max, step));
            return widget;
        } if (type==Float.class && dialog==null) {
            //if (userSelected==null) userSelected=(factoryDefaultValue!=null)?factoryDefaultValue:new Float(0);
            if (userSelected==null) userSelected=factoryDefaultValue;
            float selected=1;
            if (userSelected!=null) {
                if (userSelected instanceof Float) selected=(Float)userSelected;
                else if (userSelected instanceof String) {
                    try {selected=Float.parseFloat((String)userSelected);} catch (NumberFormatException e) {}
                }
            }
            float step=0.01f;
            float min=-Float.MAX_VALUE;
            float max=Float.MAX_VALUE;
            if (values!=null && values instanceof Float[]) {
                min=((Float[])values)[0];
                max=((Float[])values)[1];
            }
            JSpinner widget=new JSpinner(new SpinnerNumberModel(selected, min, max, step));
            return widget;
        } else if (type==Boolean.class) {
            boolean isSel=false;
            if (userSelected!=null && userSelected instanceof Boolean) isSel=((Boolean)userSelected).booleanValue();
            else if (userSelected!=null && userSelected instanceof String) isSel=Boolean.parseBoolean((String)userSelected);
            else isSel=(factoryDefaultValue!=null)?((Boolean)factoryDefaultValue).booleanValue():false;
            return new JCheckBox("", isSel);
        } else if (values!=null && (values instanceof Class || values instanceof Class[])) {
            DefaultComboBoxModel boxmodel;
            if (values instanceof Class) boxmodel=dialog.getDataCandidates((Class)values,optional);
            else boxmodel=dialog.getDataCandidates((Class[])values,optional);
            JComboBox box=new JComboBox(boxmodel);
            if (inBox(boxmodel,userSelected)) box.setSelectedItem(userSelected);
            else if (inBox(boxmodel,factoryDefaultValue)) box.setSelectedItem(factoryDefaultValue);
            else if (boxmodel.getSize()>0) box.setSelectedIndex(0);
            return box;
        } else { // I don't know what values this parameter takes. Use strings or drop-down boxes
            if (values==null || !(values instanceof Object[])) {
                JTextField textfield=new JTextField(16);
                if (userSelected!=null) textfield.setText(preprocess(userSelected.toString()));
                else if (factoryDefaultValue!=null) textfield.setText(factoryDefaultValue.toString());
                else textfield.setText("");
                textfield.setCaretPosition(0);
                return textfield;
            } else {
               JComboBox box=new JComboBox((Object[])values);
               if (inArray((Object[])values,userSelected)) box.setSelectedItem(userSelected);
               else if (inBox(box.getModel(),factoryDefaultValue)) box.setSelectedItem(factoryDefaultValue);
               else if (box.getModel().getSize()>0) box.setSelectedIndex(0);
               return box;
            }
        }
    }


    private boolean inArray(Object[] list, Object element) {
        for (Object e:list) {if (e.equals(element)) return true;}
        return false;
    }

    /** Returns true if the specified element is present in the ComboBoxModel*/
    private boolean inBox(ComboBoxModel model, Object element) {
        if (element==null) return false;
        int size=model.getSize();
        for (int i=0;i<size;i++) {
            Object item=model.getElementAt(i);
            if (item.equals(element)) return true;
        }
        return false;
    }

    /** Performs necessary escaping of special characters before displaying strings in textfields */
    private String preprocess(String string) {
        return string.replace("\t", "\\t");
    }

    /** Performs necessary unescaping of special characters in string retrieved from textfields */
    private String postprocess(String string) {
        return string.replace("\\t", "\t");
    }
}
