/*
 * PreferencesDialog.java
 *
 * Created on 12. juni 2009, 12:56
 */

package priorseditor.gui;

import java.awt.Color;
import org.jdesktop.application.Action;
import java.util.prefs.Preferences;
import javax.swing.JColorChooser;
import priorseditor.engine.PriorsEditorEngine;

/**
 *
 * @author  kjetikl
 */
public class PreferencesDialog extends javax.swing.JDialog {
     VisualizationSettings settings;
     private Preferences guiPreferences;
     private Preferences enginePreferences;
     
    /** Creates new form PreferencesDialog */
    public PreferencesDialog(PriorsEditorGUI gui) {
        super(gui.getFrame(), true);
        this.settings=gui.getVisualizationSettings();
        
        enginePreferences=Preferences.userNodeForPackage(PriorsEditorEngine.class);
        guiPreferences=Preferences.userNodeForPackage(PriorsEditorGUI.class);
        initComponents();
        int windowsize=settings.getSequenceWindowSize();
        int labelsize=settings.getSequenceLabelFixedWidth();
        sequenceWindowSizeSelector.setValue(new Integer(windowsize));
        sequenceLabelSizeSelector.setValue(new Integer(labelsize));
        antialias_checkbox.setSelected(settings.useTextAntialiasing());
        antialiasSequencelogos_checkbox.setSelected(settings.useMotifAntialiasing());
        
        boolean useFeatureDataCache=enginePreferences.getBoolean(PriorsEditorEngine.PREFERENCES_USE_CACHE_FEATUREDATA, true);
        boolean useGeneIDMappingCache=enginePreferences.getBoolean(PriorsEditorEngine.PREFERENCES_USE_CACHE_GENE_ID_MAPPING, true);
        int networkTimeout=enginePreferences.getInt(PriorsEditorEngine.PREFERENCES_NETWORK_TIMEOUT, 25000);
        int maxSequenceLength=enginePreferences.getInt(PriorsEditorEngine.MAX_SEQUENCE_LENGTH, 10000);

        boolean promptBeforeDiscard=guiPreferences.getBoolean(PriorsEditorGUI.PREFERENCES_PROMPT_BEFORE_DISCARD, true);
        boolean scaleLabelsToFit=settings.getScaleSequenceLabelsToFit();
        cacheFeatureDataCheckbox.setSelected(useFeatureDataCache);
        cacheGeneIDCheckbox.setSelected(useGeneIDMappingCache);
        promptBeforeDiscardingCheckbox.setSelected(promptBeforeDiscard);
        networkTimeoutSpinner.setValue(networkTimeout);
        maxSequenceLengthSpinner.setValue(maxSequenceLength);
        scaleToFitLabelCheckbox.setSelected(scaleLabelsToFit);
        sequenceLabelSizeSelector.setEnabled(!scaleLabelsToFit);
        this.getRootPane().setDefaultButton(OKButton);
        Color bgcolor=settings.getVisualizationPanelBackgroundColor();
        int red=guiPreferences.getInt(VisualizationSettings.MAINPANEL_BACKGROUND+"RED",bgcolor.getRed());
        int green=guiPreferences.getInt(VisualizationSettings.MAINPANEL_BACKGROUND+"GREEN",bgcolor.getGreen());
        int blue=guiPreferences.getInt(VisualizationSettings.MAINPANEL_BACKGROUND+"BLUE",bgcolor.getBlue());
        backgroundcolorbutton.setBackground(new Color(red,green,blue));
        backgroundcolorbutton.setOpaque(true);
        
    }

    public void applyChanges() {
        int windowsize=((Integer)sequenceWindowSizeSelector.getValue()).intValue();
        int labelsize=((Integer)sequenceLabelSizeSelector.getValue()).intValue();
        boolean useAntialias=antialias_checkbox.isSelected();
        boolean useMotifAntialias=antialiasSequencelogos_checkbox.isSelected();
        boolean scaleLabelsToFit=scaleToFitLabelCheckbox.isSelected();
        
        settings.setSequenceLabelFixedWidth(labelsize);
        guiPreferences.putInt(VisualizationSettings.SEQUENCE_LABEL_FIXED_WIDTH, labelsize);
        settings.setScaleSequenceLabelsToFit(scaleLabelsToFit); // this is persistant in VisualizationSettings
        guiPreferences.putBoolean(VisualizationSettings.SCALE_SEQUENCE_LABELS_TO_FIT, scaleLabelsToFit);        
        settings.setUseTextAntialiasing(useAntialias);
        guiPreferences.putBoolean(VisualizationSettings.ANTIALIAS_TEXT, useAntialias);        
        settings.setUseMotifAntialiasing(useMotifAntialias);
        guiPreferences.putBoolean(VisualizationSettings.ANTIALIAS_MOTIFS, useMotifAntialias);        
        guiPreferences.putInt(VisualizationSettings.SEQUENCE_WINDOW_SIZE, windowsize);
        if (windowsize!=settings.getSequenceWindowSize()) {
            settings.setSequenceWindowSize(windowsize);
            settings.getGUI().getVisualizationPanel().realignSequences();
        }    
        enginePreferences.putBoolean(PriorsEditorEngine.PREFERENCES_USE_CACHE_FEATUREDATA, cacheFeatureDataCheckbox.isSelected());
        settings.getEngine().getDataLoader().setUseCache(cacheFeatureDataCheckbox.isSelected());
        enginePreferences.putBoolean(PriorsEditorEngine.PREFERENCES_USE_CACHE_GENE_ID_MAPPING, cacheGeneIDCheckbox.isSelected());        
        settings.getEngine().getGeneIDResolver().setUseCache(cacheGeneIDCheckbox.isSelected());
        
        enginePreferences.putInt(PriorsEditorEngine.PREFERENCES_NETWORK_TIMEOUT, (Integer)networkTimeoutSpinner.getValue());
        settings.getEngine().setNetworkTimeout((Integer)networkTimeoutSpinner.getValue());
        enginePreferences.putInt(PriorsEditorEngine.MAX_SEQUENCE_LENGTH, (Integer)maxSequenceLengthSpinner.getValue());
        settings.getEngine().setMaxSequenceLength((Integer)maxSequenceLengthSpinner.getValue());

        guiPreferences.putBoolean(PriorsEditorGUI.PREFERENCES_PROMPT_BEFORE_DISCARD, promptBeforeDiscardingCheckbox.isSelected());        
        settings.getGUI().setPromptBeforeDiscard(promptBeforeDiscardingCheckbox.isSelected());
        
        Color newColor=backgroundcolorbutton.getBackground();
        guiPreferences.putInt(VisualizationSettings.MAINPANEL_BACKGROUND+"RED",newColor.getRed());
        guiPreferences.putInt(VisualizationSettings.MAINPANEL_BACKGROUND+"GREEN",newColor.getGreen());
        guiPreferences.putInt(VisualizationSettings.MAINPANEL_BACKGROUND+"BLUE",newColor.getBlue());     
        settings.setVisualizationPanelBackgroundColor(new Color(newColor.getRed(),newColor.getGreen(),newColor.getBlue()));
    }

    /** Restores the visualization settings that can be set in Preferences to the chosen values (in the preferences dialog) 
     *  This method is used on "session restore", to change the settings back to the values currently chosen in Preferences
     *  rather than the settings saved with the session
     */
    public static void restoreVisualizationSettings(PriorsEditorGUI gui) {
        VisualizationSettings settings=gui.getVisualizationSettings();
        Preferences guiPreferences=Preferences.userNodeForPackage(PriorsEditorGUI.class);
        int windowsize=guiPreferences.getInt(VisualizationSettings.SEQUENCE_WINDOW_SIZE, settings.getSequenceWindowSize());
        if (windowsize!=settings.getSequenceWindowSize()) {
            settings.setSequenceWindowSize(windowsize);
            settings.getGUI().getVisualizationPanel().realignSequences();
        }   
        settings.setSequenceLabelFixedWidth(guiPreferences.getInt(VisualizationSettings.SEQUENCE_LABEL_FIXED_WIDTH, settings.getSequenceLabelFixedWidth()));
        settings.setScaleSequenceLabelsToFit(guiPreferences.getBoolean(VisualizationSettings.SCALE_SEQUENCE_LABELS_TO_FIT, settings.getScaleSequenceLabelsToFit()));
        settings.setUseTextAntialiasing(guiPreferences.getBoolean(VisualizationSettings.ANTIALIAS_TEXT, settings.useTextAntialiasing()));
        settings.setUseMotifAntialiasing(guiPreferences.getBoolean(VisualizationSettings.ANTIALIAS_MOTIFS, settings.useMotifAntialiasing()));
        Color bgcolor=settings.getVisualizationPanelBackgroundColor();
        int red=guiPreferences.getInt(VisualizationSettings.MAINPANEL_BACKGROUND+"RED",bgcolor.getRed());
        int green=guiPreferences.getInt(VisualizationSettings.MAINPANEL_BACKGROUND+"GREEN",bgcolor.getGreen());
        int blue=guiPreferences.getInt(VisualizationSettings.MAINPANEL_BACKGROUND+"BLUE",bgcolor.getBlue());
        settings.setVisualizationPanelBackgroundColor(new Color(red,green,blue));
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonsPanel = new javax.swing.JPanel();
        cancelButton = new javax.swing.JButton();
        applyButton = new javax.swing.JButton();
        OKButton = new javax.swing.JButton();
        tabbedPane = new javax.swing.JTabbedPane();
        generalSettingsPanel = new javax.swing.JPanel();
        generalSettingsInternalPanel = new javax.swing.JPanel();
        sequenceLengthLabel = new javax.swing.JLabel();
        maxSequenceLengthSpinner = new javax.swing.JSpinner();
        networkTimeoutLabel = new javax.swing.JLabel();
        networkTimeoutSpinner = new javax.swing.JSpinner();
        promptSaveLabel = new javax.swing.JLabel();
        promptBeforeDiscardingCheckbox = new javax.swing.JCheckBox();
        visualizationSettingsPanel = new javax.swing.JPanel();
        vizualizationSettingsInternalPanel = new javax.swing.JPanel();
        sequenceWindowSizeLabel = new javax.swing.JLabel();
        sequenceWindowSizeSelector = new javax.swing.JSpinner();
        jLabel1 = new javax.swing.JLabel();
        sequenceLabelSizeLabel = new javax.swing.JLabel();
        sequenceLabelSizeSelector = new javax.swing.JSpinner();
        scaleToFitLabelCheckbox = new javax.swing.JCheckBox();
        antialias_label = new javax.swing.JLabel();
        antialias_checkbox = new javax.swing.JCheckBox();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        antialiasSequencelogos_checkbox = new javax.swing.JCheckBox();
        jLabel4 = new javax.swing.JLabel();
        backgroundcolorlabel = new javax.swing.JLabel();
        backgroundcolorbutton = new javax.swing.JButton();
        resetbgcolorbutton = new javax.swing.JButton();
        cacheSettingsPanel = new javax.swing.JPanel();
        cacheSettingsInternalPanel = new javax.swing.JPanel();
        cacheFeatureDataLabel = new javax.swing.JLabel();
        cacheFeatureDataCheckbox = new javax.swing.JCheckBox();
        clearFeatureDataCacheButton = new javax.swing.JButton();
        cacheGeneIdDataLabel = new javax.swing.JLabel();
        cacheGeneIDCheckbox = new javax.swing.JCheckBox();
        clearGeneIDCacheButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(PreferencesDialog.class);
        setTitle(resourceMap.getString("Form.title")); // NOI18N
        setName("Form"); // NOI18N

        buttonsPanel.setMaximumSize(new java.awt.Dimension(32767, 44));
        buttonsPanel.setMinimumSize(new java.awt.Dimension(100, 44));
        buttonsPanel.setName("buttonsPanel"); // NOI18N
        buttonsPanel.setPreferredSize(new java.awt.Dimension(100, 44));

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PreferencesDialog.class, this);
        cancelButton.setAction(actionMap.get("cancelButtonPressed")); // NOI18N
        cancelButton.setMaximumSize(new java.awt.Dimension(75, 27));
        cancelButton.setMinimumSize(new java.awt.Dimension(75, 27));
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.setPreferredSize(new java.awt.Dimension(75, 27));

        applyButton.setAction(actionMap.get("applyButtonPressed")); // NOI18N
        applyButton.setText(resourceMap.getString("applyButton.text")); // NOI18N
        applyButton.setMaximumSize(new java.awt.Dimension(75, 27));
        applyButton.setMinimumSize(new java.awt.Dimension(75, 27));
        applyButton.setName("applyButton"); // NOI18N
        applyButton.setPreferredSize(new java.awt.Dimension(75, 27));

        OKButton.setAction(actionMap.get("OKButtonPressed")); // NOI18N
        OKButton.setText(resourceMap.getString("OKButton.text")); // NOI18N
        OKButton.setMaximumSize(new java.awt.Dimension(75, 27));
        OKButton.setMinimumSize(new java.awt.Dimension(75, 27));
        OKButton.setName("OKButton"); // NOI18N
        OKButton.setOpaque(false);
        OKButton.setPreferredSize(new java.awt.Dimension(75, 27));

        javax.swing.GroupLayout buttonsPanelLayout = new javax.swing.GroupLayout(buttonsPanel);
        buttonsPanel.setLayout(buttonsPanelLayout);
        buttonsPanelLayout.setHorizontalGroup(
            buttonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, buttonsPanelLayout.createSequentialGroup()
                .addContainerGap(198, Short.MAX_VALUE)
                .addComponent(OKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(applyButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        buttonsPanelLayout.setVerticalGroup(
            buttonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, buttonsPanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(buttonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(applyButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(OKButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        getContentPane().add(buttonsPanel, java.awt.BorderLayout.PAGE_END);

        tabbedPane.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        tabbedPane.setName("tabbedPane"); // NOI18N

        generalSettingsPanel.setName("generalSettingsPanel"); // NOI18N
        generalSettingsPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        generalSettingsInternalPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(20, 10, 10, 10));
        generalSettingsInternalPanel.setName("generalSettingsInternalPanel"); // NOI18N
        generalSettingsInternalPanel.setLayout(new java.awt.GridLayout(3, 2, 20, 16));

        sequenceLengthLabel.setText(resourceMap.getString("sequenceLengthLabel.text")); // NOI18N
        sequenceLengthLabel.setName("sequenceLengthLabel"); // NOI18N
        generalSettingsInternalPanel.add(sequenceLengthLabel);

        maxSequenceLengthSpinner.setModel(new javax.swing.SpinnerNumberModel(Integer.valueOf(10000), null, null, Integer.valueOf(1)));
        maxSequenceLengthSpinner.setName("maxSequenceLengthSpinner"); // NOI18N
        generalSettingsInternalPanel.add(maxSequenceLengthSpinner);

        networkTimeoutLabel.setText(resourceMap.getString("networkTimeoutLabel.text")); // NOI18N
        networkTimeoutLabel.setName("networkTimeoutLabel"); // NOI18N
        generalSettingsInternalPanel.add(networkTimeoutLabel);

        networkTimeoutSpinner.setModel(new javax.swing.SpinnerNumberModel(Integer.valueOf(25000), Integer.valueOf(0), null, Integer.valueOf(100)));
        networkTimeoutSpinner.setName("networkTimeoutSpinner"); // NOI18N
        generalSettingsInternalPanel.add(networkTimeoutSpinner);

        promptSaveLabel.setText(resourceMap.getString("promptSaveLabel.text")); // NOI18N
        promptSaveLabel.setName("promptSaveLabel"); // NOI18N
        generalSettingsInternalPanel.add(promptSaveLabel);

        promptBeforeDiscardingCheckbox.setText(resourceMap.getString("promptBeforeDiscardingCheckbox.text")); // NOI18N
        promptBeforeDiscardingCheckbox.setName("promptBeforeDiscardingCheckbox"); // NOI18N
        generalSettingsInternalPanel.add(promptBeforeDiscardingCheckbox);

        generalSettingsPanel.add(generalSettingsInternalPanel);

        tabbedPane.addTab(resourceMap.getString("generalSettingsPanel.TabConstraints.tabTitle"), generalSettingsPanel); // NOI18N

        visualizationSettingsPanel.setName("visualizationSettingsPanel"); // NOI18N
        visualizationSettingsPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        vizualizationSettingsInternalPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(20, 10, 10, 10));
        vizualizationSettingsInternalPanel.setName("vizualizationSettingsInternalPanel"); // NOI18N
        vizualizationSettingsInternalPanel.setLayout(new java.awt.GridLayout(5, 3, 20, 16));

        sequenceWindowSizeLabel.setText(resourceMap.getString("sequenceWindowSizeLabel.text")); // NOI18N
        sequenceWindowSizeLabel.setName("sequenceWindowSizeLabel"); // NOI18N
        vizualizationSettingsInternalPanel.add(sequenceWindowSizeLabel);

        sequenceWindowSizeSelector.setName("sequenceWindowSizeSelector"); // NOI18N
        vizualizationSettingsInternalPanel.add(sequenceWindowSizeSelector);

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N
        vizualizationSettingsInternalPanel.add(jLabel1);

        sequenceLabelSizeLabel.setText(resourceMap.getString("sequenceLabelSizeLabel.text")); // NOI18N
        sequenceLabelSizeLabel.setName("sequenceLabelSizeLabel"); // NOI18N
        vizualizationSettingsInternalPanel.add(sequenceLabelSizeLabel);

        sequenceLabelSizeSelector.setName("sequenceLabelSizeSelector"); // NOI18N
        vizualizationSettingsInternalPanel.add(sequenceLabelSizeSelector);

        scaleToFitLabelCheckbox.setSelected(true);
        scaleToFitLabelCheckbox.setText(resourceMap.getString("scaleToFitLabelCheckbox.text")); // NOI18N
        scaleToFitLabelCheckbox.setName("scaleToFitLabelCheckbox"); // NOI18N
        scaleToFitLabelCheckbox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                scaleToFitComboboxAction(evt);
            }
        });
        vizualizationSettingsInternalPanel.add(scaleToFitLabelCheckbox);

        antialias_label.setText(resourceMap.getString("antialias_label.text")); // NOI18N
        antialias_label.setName("antialias_label"); // NOI18N
        vizualizationSettingsInternalPanel.add(antialias_label);

        antialias_checkbox.setName("antialias_checkbox"); // NOI18N
        vizualizationSettingsInternalPanel.add(antialias_checkbox);

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N
        vizualizationSettingsInternalPanel.add(jLabel2);

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N
        vizualizationSettingsInternalPanel.add(jLabel3);

        antialiasSequencelogos_checkbox.setText(resourceMap.getString("antialiasSequencelogos_checkbox.text")); // NOI18N
        antialiasSequencelogos_checkbox.setName("antialiasSequencelogos_checkbox"); // NOI18N
        vizualizationSettingsInternalPanel.add(antialiasSequencelogos_checkbox);

        jLabel4.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        jLabel4.setName("jLabel4"); // NOI18N
        vizualizationSettingsInternalPanel.add(jLabel4);

        backgroundcolorlabel.setText(resourceMap.getString("backgroundcolorlabel.text")); // NOI18N
        backgroundcolorlabel.setName("backgroundcolorlabel"); // NOI18N
        vizualizationSettingsInternalPanel.add(backgroundcolorlabel);

        backgroundcolorbutton.setText(resourceMap.getString("backgroundcolorbutton.text")); // NOI18N
        backgroundcolorbutton.setName("backgroundcolorbutton"); // NOI18N
        backgroundcolorbutton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backgroundcolorbuttonpressed(evt);
            }
        });
        vizualizationSettingsInternalPanel.add(backgroundcolorbutton);

        resetbgcolorbutton.setText(resourceMap.getString("resetbgcolorbutton.text")); // NOI18N
        resetbgcolorbutton.setName("resetbgcolorbutton"); // NOI18N
        resetbgcolorbutton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetbgcolorbuttonclicked(evt);
            }
        });
        vizualizationSettingsInternalPanel.add(resetbgcolorbutton);

        visualizationSettingsPanel.add(vizualizationSettingsInternalPanel);

        tabbedPane.addTab(resourceMap.getString("visualizationSettingsPanel.TabConstraints.tabTitle"), visualizationSettingsPanel); // NOI18N

        cacheSettingsPanel.setName("cacheSettingsPanel"); // NOI18N
        cacheSettingsPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        cacheSettingsInternalPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(20, 10, 10, 10));
        cacheSettingsInternalPanel.setName("cacheSettingsInternalPanel"); // NOI18N
        cacheSettingsInternalPanel.setLayout(new java.awt.GridLayout(2, 3, 20, 16));

        cacheFeatureDataLabel.setText(resourceMap.getString("cacheFeatureDataLabel.text")); // NOI18N
        cacheFeatureDataLabel.setName("cacheFeatureDataLabel"); // NOI18N
        cacheSettingsInternalPanel.add(cacheFeatureDataLabel);

        cacheFeatureDataCheckbox.setText(resourceMap.getString("cacheFeatureDataCheckbox.text")); // NOI18N
        cacheFeatureDataCheckbox.setName("cacheFeatureDataCheckbox"); // NOI18N
        cacheSettingsInternalPanel.add(cacheFeatureDataCheckbox);

        clearFeatureDataCacheButton.setAction(actionMap.get("clearFeatureData")); // NOI18N
        clearFeatureDataCacheButton.setToolTipText(null);
        clearFeatureDataCacheButton.setMaximumSize(new java.awt.Dimension(39, 23));
        clearFeatureDataCacheButton.setMinimumSize(new java.awt.Dimension(39, 23));
        clearFeatureDataCacheButton.setName("clearFeatureDataCacheButton"); // NOI18N
        clearFeatureDataCacheButton.setPreferredSize(new java.awt.Dimension(39, 23));
        cacheSettingsInternalPanel.add(clearFeatureDataCacheButton);

        cacheGeneIdDataLabel.setText(resourceMap.getString("cacheGeneIdDataLabel.text")); // NOI18N
        cacheGeneIdDataLabel.setName("cacheGeneIdDataLabel"); // NOI18N
        cacheSettingsInternalPanel.add(cacheGeneIdDataLabel);

        cacheGeneIDCheckbox.setText(resourceMap.getString("cacheGeneIDCheckbox.text")); // NOI18N
        cacheGeneIDCheckbox.setName("cacheGeneIDCheckbox"); // NOI18N
        cacheSettingsInternalPanel.add(cacheGeneIDCheckbox);

        clearGeneIDCacheButton.setAction(actionMap.get("clearGeneIDMappings")); // NOI18N
        clearGeneIDCacheButton.setToolTipText(null);
        clearGeneIDCacheButton.setName("clearGeneIDCacheButton"); // NOI18N
        cacheSettingsInternalPanel.add(clearGeneIDCacheButton);

        cacheSettingsPanel.add(cacheSettingsInternalPanel);

        tabbedPane.addTab(resourceMap.getString("cacheSettingsPanel.TabConstraints.tabTitle"), cacheSettingsPanel); // NOI18N

        getContentPane().add(tabbedPane, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void scaleToFitComboboxAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_scaleToFitComboboxAction
// TODO add your handling code here:
       sequenceLabelSizeSelector.setEnabled(!scaleToFitLabelCheckbox.isSelected());//GEN-LAST:event_scaleToFitComboboxAction
}

private void backgroundcolorbuttonpressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backgroundcolorbuttonpressed
// TODO add your handling code here:
      Color color=JColorChooser.showDialog(settings.getGUI().getFrame(),"Select new background color",backgroundcolorbutton.getBackground());//GEN-LAST:event_backgroundcolorbuttonpressed
      if (color!=null) backgroundcolorbutton.setBackground(color);
}

private void resetbgcolorbuttonclicked(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resetbgcolorbuttonclicked
// TODO add your handling code here:
    backgroundcolorbutton.setBackground(new javax.swing.JPanel().getBackground());//GEN-LAST:event_resetbgcolorbuttonclicked
}


    @Action
    public void cancelButtonPressed() {
       setVisible(false);
    }

    @Action
    public void applyButtonPressed() {
       applyChanges();
    }

    @Action
    public void OKButtonPressed() {
       applyChanges();
       setVisible(false);
    }

    @Action
    public void clearFeatureData() {
        settings.getEngine().getDataLoader().clearCache();
        settings.getGUI().logMessage("Clearing feature data");
    }

    @Action
    public void clearGeneIDMappings() {
        settings.getEngine().getGeneIDResolver().clearCache();
        settings.getGUI().logMessage("Clearing gene mappings");
    }


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton OKButton;
    private javax.swing.JCheckBox antialiasSequencelogos_checkbox;
    private javax.swing.JCheckBox antialias_checkbox;
    private javax.swing.JLabel antialias_label;
    private javax.swing.JButton applyButton;
    private javax.swing.JButton backgroundcolorbutton;
    private javax.swing.JLabel backgroundcolorlabel;
    private javax.swing.JPanel buttonsPanel;
    private javax.swing.JCheckBox cacheFeatureDataCheckbox;
    private javax.swing.JLabel cacheFeatureDataLabel;
    private javax.swing.JCheckBox cacheGeneIDCheckbox;
    private javax.swing.JLabel cacheGeneIdDataLabel;
    private javax.swing.JPanel cacheSettingsInternalPanel;
    private javax.swing.JPanel cacheSettingsPanel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JButton clearFeatureDataCacheButton;
    private javax.swing.JButton clearGeneIDCacheButton;
    private javax.swing.JPanel generalSettingsInternalPanel;
    private javax.swing.JPanel generalSettingsPanel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JSpinner maxSequenceLengthSpinner;
    private javax.swing.JLabel networkTimeoutLabel;
    private javax.swing.JSpinner networkTimeoutSpinner;
    private javax.swing.JCheckBox promptBeforeDiscardingCheckbox;
    private javax.swing.JLabel promptSaveLabel;
    private javax.swing.JButton resetbgcolorbutton;
    private javax.swing.JCheckBox scaleToFitLabelCheckbox;
    private javax.swing.JLabel sequenceLabelSizeLabel;
    private javax.swing.JSpinner sequenceLabelSizeSelector;
    private javax.swing.JLabel sequenceLengthLabel;
    private javax.swing.JLabel sequenceWindowSizeLabel;
    private javax.swing.JSpinner sequenceWindowSizeSelector;
    private javax.swing.JTabbedPane tabbedPane;
    private javax.swing.JPanel visualizationSettingsPanel;
    private javax.swing.JPanel vizualizationSettingsInternalPanel;
    // End of variables declaration//GEN-END:variables

}
