/*
 * PriorsEditorApp.java
 */

package priorseditor.gui;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.SplashScreen;
import javax.swing.UIManager;
import org.jdesktop.application.Application;
import org.jdesktop.application.SingleFrameApplication;
import priorseditor.engine.MessageListener;
import priorseditor.engine.PriorsEditorEngine;

/**
 * The main class of the application.
 */
public class PriorsEditorApp extends SingleFrameApplication implements MessageListener {
    PriorsEditorGUI gui;
    Graphics2D graphics=null;
    SplashScreen splash=null;
    int i=1;
    /**
     * At startup create and show the main frame of the application.
     */
    @Override protected void startup() {
        
       splash = SplashScreen.getSplashScreen();
       if (splash != null) {
            graphics = (Graphics2D)splash.createGraphics();
        }
        try {
            //UIManager.setLookAndFeel( new com.nilo.plaf.nimrod.NimRODLookAndFeel());
            UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
        }
        catch (Exception e) {}
        //try {Thread.sleep(10000);} catch(InterruptedException e) {}
        PriorsEditorEngine engine=new PriorsEditorEngine();
        engine.setWorkDirectory(getContext().getLocalStorage().getDirectory().getAbsolutePath());
        engine.addMessageListener(this);
        engine.initialize();      
        logMessage("Launching GUI");
        gui=new PriorsEditorGUI(this,engine);
        engine.removeMessageListener(this);
        if (splash!=null)  splash.close();
        show(gui);
        gui.firstTimeStartup();
    }

    /**
     * This method is automatically called on shutdown of the Application
     */
    @Override protected void shutdown() {
        gui.cleanup();
        super.shutdown(); // don't forget this or session state will not be saved!!!
    }

   /**
     * This method is to initialize the specified window by injecting resources.
     * Windows shown in our application come fully initialized from the GUI
     * builder, so this additional configuration is not needed.
     */
    @Override protected void configureWindow(java.awt.Window root) {
    }

    /**
     * A convenient static getter for the application instance.
     * @return the instance of PriorsEditorApp
     */
    public static PriorsEditorApp getApplication() {
        return Application.getInstance(PriorsEditorApp.class);
    }

    /**
     * Main method launching the application.
     */
    public static void main(String[] args) {
        launch(PriorsEditorApp.class, args);
    }
   
    
    static void renderSplashFrame(Graphics2D g, int frame, String message) {
        g.setComposite(AlphaComposite.Clear);
        g.fillRect(25,180,500,40);
        g.setPaintMode();
        g.setColor(Color.ORANGE);
        g.drawString(message, 25, 212);
        int length=frame*20;
        if (length>280) length=280;
        g.fillRect(25,216,length,5);
    }


         @Override
        public void logMessage(String msg) {
            if (splash!=null) {
                renderSplashFrame(graphics,i++,msg);
                splash.update();
            } else {
               System.err.println(msg); 
            }
        }

        @Override
        public void statusMessage(String msg) {}               
        @Override
        public void errorMessage(String msg, int errortype) {}   
    
}
