/*
 * PriorsEditorView.java
 */

package priorseditor.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.EventObject;
import javax.swing.event.DocumentEvent;
import javax.swing.event.ListDataEvent;
import javax.swing.event.UndoableEditEvent;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.DataListener;
import priorseditor.engine.data.*;

import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.awt.print.PrinterException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryUsage;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.prefs.Preferences;
import javax.imageio.ImageIO;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.ToolTipManager;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListDataListener;
import javax.swing.event.UndoableEditListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.text.StyledDocument;
import javax.swing.undo.UndoableEdit;
import org.jdesktop.application.Application;
import org.jdesktop.application.ApplicationContext;
import org.jdesktop.application.LocalStorage;
import priorseditor.engine.AddSequencesTask;
import priorseditor.engine.GuiScheduler;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.PriorsEditorClient;
import priorseditor.engine.ProtocolTask;
import priorseditor.engine.SystemError;
import priorseditor.engine.operations.Condition_within;
import priorseditor.engine.operations.Operation_new;
import priorseditor.engine.protocol.DataTypeTable;
import priorseditor.engine.protocol.ParseError;
import priorseditor.engine.protocol.Protocol;
import priorseditor.external.ExternalProgram;
import priorseditor.gui.operationdialog.*;
import priorseditor.gui.prompt.*;



/**
 * The application's main frame.
 */
public class PriorsEditorGUI extends FrameView implements PriorsEditorClient, DataListener {

    public static final String PREFERENCES_PROMPT_BEFORE_DISCARD="promptBeforeDiscard";
    
    public static final String SELECTION_TOOL="Selection tool";
    public static final String HAND_TOOL="Hand tool";
    public static final String ZOOM_TOOL="Zoom tool";
    public static final String DRAW_TOOL="Draw tool";
    
    private PriorsEditorEngine engine=null;
    private VisualizationSettings visualizationSettings=null;
    private RegionVisualizationFilter regionVisualizationFilter=null;
    private String selectedToolName=null;
    private VisualizationPanel visualizationPanel;
    private FeaturesPanel featuresPanel;
    private DataObjectsPanel dataObjectsPanel;
    private MotifsPanel motifCollectionPanel;
    private ProtocolEditor protocolEditor;
    private GUIUndoManager undoManager;
    private GuiScheduler scheduler=null;
    private MainPanel mainWindow;
    private UndoRedoDocumentListener undoRedoDocumentListener;
    private ProgressMonitor progressMonitor;
    private ArrayList<String> recentsessions=new ArrayList<String>();
    private OpenRecentSessionMenuListener openrecentsessionmenulistener;
    private String recentsessionsfilename="recentsession.ser";
    private int maxRecentSessions=6; // max number of recent sessions to show in menu
    private boolean promptBeforeDiscard=true;
    private DecimalFormat motifcounterformat = new DecimalFormat("0000"); 
    private Preferences preferences;
    private MemoryMonitor memorymonitor;
    private File lastUsedDirectory=null;
    private PriorsEditorGUI gui;
    private HashSet<String> reservedDataNames=new HashSet<String>(); // this is used by GUI-dialogs to reserve names of new data items until they are either stored properly or the task is aborted
    private MotifBrowser motifbrowser=null;
    
    /**
     * Creates a new instance of PriorsEditor
     * @param app
     */
    public PriorsEditorGUI(SingleFrameApplication app, PriorsEditorEngine priorsEditorEngine) {
        super(app); // initialize some GUI framework
        gui=this; 
        try {
            ArrayList<Image>iconlist=new ArrayList<Image>(3);
            iconlist.add(Toolkit.getDefaultToolkit().getImage(getClass().getResource("resources/icons/PriorsEditorIcon_16.png")));
            iconlist.add(Toolkit.getDefaultToolkit().getImage(getClass().getResource("resources/icons/DNAhelix24.png")));
            iconlist.add(Toolkit.getDefaultToolkit().getImage(getClass().getResource("resources/icons/DNAhelix32.png")));
            this.getFrame().setIconImages(iconlist);
        } catch (Exception e) {}
        setEngine(priorsEditorEngine); // initialize the PriorsEditorEngine
        engine.addDataListener(this);
        preferences = Preferences.userNodeForPackage(this.getClass());
        setPromptBeforeDiscard(preferences.getBoolean(PREFERENCES_PROMPT_BEFORE_DISCARD, promptBeforeDiscard));               
        openrecentsessionmenulistener=new OpenRecentSessionMenuListener();
        visualizationSettings=VisualizationSettings.getInstance(this);
        getApplication().addExitListener(new ConfirmExit());
        ApplicationContext context=getApplication().getContext();
        undoManager=new GUIUndoManager(context,this);
        scheduler=new GuiScheduler(this);
        
        initComponents(); // lay out the components
        
        viewInMainWindowMenu=new JMenu("Panels");
        viewMenu.add(viewInMainWindowMenu);
        viewMenu.add(new JSeparator());
        setupCondensedModeMenuItem();
        setupBaseCursorMenuItem();       
        setupGraphicalTooltipsMenuItem();
        setupOperationsMenu();
        
        progressMonitor=new ProgressMonitor();
        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);
        progressBar.setStringPainted(true);
        scheduler.addPropertyChangeListener(progressMonitor);
               
        undoManager.addUndoableEditListener(new UndoRedoButtonUpdater());
        setSelectedTool(SELECTION_TOOL);
        selectionToolButton.setSelected(true);
        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this);
        actionMap.get("stopAction").setEnabled(false);
        mainWindow=new MainPanel(this,viewInMainWindowMenu);
        mainWindow.setMinimumSize(new java.awt.Dimension(200,200));
        mainWindow.setPreferredSize(new java.awt.Dimension(800,600));
        verticalSplitPane.setRightComponent(mainWindow);
        featuresPanel=new FeaturesPanel(this);
        dataObjectsPanel=new DataObjectsPanel(this);
        motifCollectionPanel=new MotifsPanel(this);
        allListsSplitPane.setTopComponent(featuresPanel);
        allListsSplitPane.setBottomComponent(firstSplitPane);
        firstSplitPane.setTopComponent(motifCollectionPanel);
        firstSplitPane.setBottomComponent(dataObjectsPanel); 

        
        visualizationPanel=new VisualizationPanel(getVisualizationSettings());
        protocolEditor = new ProtocolEditor(this);
        mainWindow.addTab("Visualization",visualizationPanel,false);
        mainWindow.addTab("Protocol",protocolEditor,false);
        ProtocolManagerListener pml=new ProtocolManagerListener();        
        protocolEditor.getProtocolManager().addListDataListener(pml);
        pml.populateProtocolList();
        updateRecentSessionsMenu(true);
        memorymonitor=new MemoryMonitor(this,8);
        logLeftPanel.setLayout(new BorderLayout());
        logLeftPanel.setBorder(BorderFactory.createEmptyBorder(8,5,8,5));
        logLeftPanel.add(memorymonitor);
        ToolTipManager tpm=ToolTipManager.sharedInstance();
        tpm.setDismissDelay(Integer.MAX_VALUE);
        //  tpm.setInitialDelay(0);       
    }
    
    
    /**
     * This method should be called when the system exits.
     * It is called by PriorsEditorApp on shutdown
     */
    public void cleanup() {
        scheduler.shutdown(true);
        protocolEditor.shutdown();
        LocalStorage localStorage=getApplication().getContext().getLocalStorage();
        try {
           localStorage.save(recentsessions, recentsessionsfilename);
        } catch (Exception e) {}
        //try {
           undoManager.clearCache();
        //} catch (IOException e) {System.err.println("An error occurred during Undo cache cleanup: "+e.getMessage());}      
    }
    
    
    /**
     * Specifies an instance of PriorsEditor Engine to use by this GUI
     * 
     * @param engine
     */
    private void setEngine(PriorsEditorEngine engine) {
        this.engine=engine;
        engine.setClient(this);       
    } 
    
    /**
     * Returns and instance of PriorsEditorEngine which is used by this GUI
     * @return an instance of PriorsEditorEngine
     */
    public PriorsEditorEngine getEngine() {
        return engine; 
    }
    
    /**
     * Returns an instance of the top-level panel in the GUI (area between the menu and the statusbar)
     * @return the top level graphics panel
     */
    public JPanel getTopPanel() {
        return topPanel;
    }
    
    /**
     * Returns an instance of the main panel in the GUI (the tabbed pane that shows Visualization, Protocols and outputs)
     * @return the main graphics panel
     */
    public MainPanel getMainWindow() {
        return mainWindow;
    }
    
    /**
     * Returns the current ProtocolEditor
     * @return the current ProtocolEditor
     */
    public ProtocolEditor getProtocolEditor() {
        return protocolEditor;
    }
    
    /** Returns the Motif Panel*/
    public MotifsPanel getMotifsPanel() {
        return motifCollectionPanel;
    }
    
    /**
     * Returns an instance of the VisualizationSettings used by this GUI
     * @return the VisualizationSettings
     */
    public VisualizationSettings getVisualizationSettings() {
        return visualizationSettings;
    }
    
    public DocumentListener getUndoRedoDocumentListener() {
        if (undoRedoDocumentListener==null) undoRedoDocumentListener=new UndoRedoDocumentListener();
        return undoRedoDocumentListener;
    }
    
    /**
     * Fetches the ListModel underlying the FeaturesPanel (containing Feature Datasets) and returns it
     * @return the ListModel for the DataPanel
     */
    public FeaturesPanelListModel getFeaturesPanelListModel() {
        if (featuresPanel==null || featuresPanel.getFeaturesList()==null) return null;
        return (FeaturesPanelListModel)featuresPanel.getFeaturesList().getModel();     
    }
    
    /**
     * Fetches the ListModel underlying the DataObjectsPanel (containing non-feature data) and returns it
     * @return the ListModel for the DataObjectsPanel
     */
    public DataObjectsPanelListModel getDataObjectsPanelListModel() {
        if (dataObjectsPanel==null || dataObjectsPanel.getDataObjectsList()==null) return null;
        return (DataObjectsPanelListModel)dataObjectsPanel.getDataObjectsList().getModel();     
    }
    
    /**
     * Sets the name of the currently selected GUI tool (usually selected by pressing a radio-button on the toolbar)
     * @param toolname The name of the current tool. This class defines useful constants such as SELECTION_TOOL, HAND_TOOL and ZOOM_TOOL
     */
    public void setSelectedTool(String toolname) {
        selectedToolName=toolname;
        //debugMessage("Selected "+toolname);
    }
    
    /**
     * Returns the name of the currently selected GUI tool (usually selected by pressing a radio-button on the toolbar)
     * @return The name of the currently selected GUI tool. This class defines useful constants such as SELECTION_TOOL, HAND_TOOL and ZOOM_TOOL
     */
    public String getSelectedTool() {
        return selectedToolName;
    }
    
    /** 
     * Sets the 'Prompt before discard' flag. If this flag is TRUE the GUI will always prompt the user
     * for confirmation before discarding data that is not saved
     */
    public void setPromptBeforeDiscard(boolean doPrompt) {
        promptBeforeDiscard=doPrompt;
    }
    
    /** 
     * If this method returns TRUE the GUI should always prompt the user
     * for confirmation before discarding data that is not saved
     */    
    public boolean doPromptBeforeDiscard() {
        return promptBeforeDiscard;
    }
    
    /** Adds the "Show base cursor" menu item to the View menu*/
    private void setupBaseCursorMenuItem() {
        SequenceVisualizer.setVisualizeBaseCursor(true);
        final JCheckBoxMenuItem item=new JCheckBoxMenuItem("Show Base Cursor");
        item.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                SequenceVisualizer.setVisualizeBaseCursor(item.isSelected());
            }
        });
        item.setSelected(true);
        viewMenu.add(item);
    }
    
    /** Adds the "Show condensed" menu item to the View menu*/    
    private void setupCondensedModeMenuItem() {
        boolean useCondensedmode=SequenceVisualizer.getCondensedMode();
        final JCheckBoxMenuItem item=new JCheckBoxMenuItem("Show condensed");
        item.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                SequenceVisualizer.setCondensedMode(item.isSelected());
                getVisualizationPanel().condensedModeChanged(item.isSelected()); // revalidates and repaints
            }
        });
        item.setSelected(useCondensedmode);
        viewMenu.add(item);
    }
    
    private boolean useGraphicalTooltipsSetting=true;
    /** Adds the "Show Graphical tooltips" menu item to the View menu*/    
    private void setupGraphicalTooltipsMenuItem() {
        SequenceVisualizer.setVisualizeBaseCursor(true);
        final JCheckBoxMenuItem item=new JCheckBoxMenuItem("Show Graphical Tooltips");
        item.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                useGraphicalTooltipsSetting=item.isSelected();
            }
        });
        item.setSelected(useGraphicalTooltipsSetting);
        viewMenu.add(item);
    }
    
    
    /** Returns true if the GUI should use graphical tooltips (e.g. showing sequence logos for motif tracks) */
    public boolean useGraphicalTooltips() {
        return useGraphicalTooltipsSetting;
    }
            
    
    
    
    /**
     * A call to this method launches the modal operations dialog where the
     * user can choose parameters for an operation
     */
    public void launchOperationEditor(OperationTask operationtask) {
        String opstring=null;
        String sourceDatastring=null;
        String targetDatastring=null;
        Object operation=operationtask.getParameter(OperationTask.OPERATION);
        Object source=operationtask.getParameter(OperationTask.SOURCE);
        Object target=operationtask.getParameter(OperationTask.TARGET);
        opstring=operationtask.getOperationName();
        if (source!=null) sourceDatastring=((Data)source).getName();
        if (target!=null) targetDatastring=((Data)target).getName();
        if (source!=null) operationtask.setParameter(OperationTask.SOURCE_NAME,((Data)source).getName());
        if (target!=null) operationtask.setParameter(OperationTask.TARGET_NAME,((Data)target).getName());   
        operationtask.setParameter(OperationTask.ENGINE, getEngine());
        //logMessage("Launcing Operations Editor: Operation="+opstring+", source="+sourceDatastring+", target="+targetDatastring);
        ArrayList<SelectionWindow>selections=visualizationSettings.getSelectionWindows();
        if (selections!=null && ((Operation)operation).isSubrangeApplicable()) {
            Condition_within within=new Condition_within(selections);
            operationtask.setParameter("within", within);
        }
        if (source!=null && !((Operation)operation).canUseAsSource((Data)source)) {
            String message=sourceDatastring+" is not a valid datatype for operation "+opstring;
            JOptionPane.showMessageDialog(this.getFrame(), message,"Operations editor", JOptionPane.ERROR_MESSAGE);
            return;
        } else {
            try {
               Class dialogclass = Class.forName("priorseditor.gui.operationdialog.OperationDialog_"+opstring);
               OperationDialog dialog=(OperationDialog)dialogclass.newInstance();
               dialog.initialize(operationtask,null,this);
               dialog.setLocation(getFrame().getWidth()/2-dialog.getWidth()/2, getFrame().getHeight()/2-dialog.getHeight()/2);
               dialog.setVisible(true);               
               if (dialog.okPressed()) launchOperationTask(dialog.getOperationTask(),isRecording());
               dialog.dispose();
            } catch (Exception e) {
                 JOptionPane.showMessageDialog(this.getFrame(), e.getClass().toString()+"\n"+e.getMessage(),"Operations editor" ,JOptionPane.ERROR_MESSAGE);
                 engine.reportError(e);
                 //e.printStackTrace(System.err);
//                 StackTraceElement[] stackTrace=e.getStackTrace();
//                 debugMessage("An error occurred: "+e.getClass().toString()+" => "+e.getMessage());
//                 for (StackTraceElement stack:stackTrace) {
//                    debugMessage("   "+stack.getClassName()+"."+stack.getMethodName()+"()   line "+stack.getLineNumber());
//                 }
            }
                    
        
        }
  }
    
    /**
     * Launches an OperationTask as an undoable event and makes a record in the protocol script (if in record mode)
     */
    public void launchOperationTask(ExecutableTask task, boolean recordInProtocol) {
        if (task instanceof OperationTask) {
            reserveDataName(((OperationTask)task).getReserveDataNames());
        }
        UndoMonitor monitor=undoManager.getUndoMonitor(task.getPresentationName());
        task.setUndoMonitor(monitor);
        RecordingCompoundEdit protocolEdit=new RecordingCompoundEdit();
        undoManager.forwardUndoEvents(protocolEdit);

        if (recordInProtocol) {
           int pos=protocolEditor.getCaretPosition();
           protocolEditor.insertOperationAt(task,pos);               
        }        

        protocolEdit.end();
        undoManager.forwardUndoEvents(null);
        task.setProtocolEdit(protocolEdit);
        undoManager.addEdit(task);
        scheduler.submit(task);
    }    
        
    /**
     * Launches an AddSequencesTask as an undoable event
     */
    public void launchAddSequencesTask(AddSequencesTask task) {      
        if (task.getNumberofSequencesToAdd()==0) return; // no sequences specified
        //System.err.println("LaunchAddSequencesTask");        
        UndoMonitor monitor=undoManager.getUndoMonitor(task.getPresentationName());
        task.setUndoMonitor(monitor);
        undoManager.addEdit(task);
        scheduler.submit(task);
        //System.err.println("submitted");
    }    
        
    
    
      
    /**
     * Returns a reference to the GUIUndoManager used by this GUI
     * @return
     */
    public GUIUndoManager getUndoManager() {
        return undoManager;
    }
    
    /**
     * Returns a reference to the Scheduler used by this gui
     * @return
     */
    public GuiScheduler getScheduler() {
        return scheduler;
    }
    

    
    /**
     * This method returns the VisualizationPanel
     */
    public VisualizationPanel getVisualizationPanel() {
        return visualizationPanel;
    }
    
    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = PriorsEditorApp.getApplication().getMainFrame();
            aboutBox = new PriorsEditorAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        PriorsEditorApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        topPanel = new javax.swing.JPanel();
        Toolbar = new javax.swing.JToolBar();
        newButton = new javax.swing.JButton();
        openButton = new javax.swing.JButton();
        saveButton = new javax.swing.JButton();
        printButton = new javax.swing.JButton();
        jSeparator9 = new javax.swing.JToolBar.Separator();
        sequenceButton = new javax.swing.JButton();
        databaseButton = new javax.swing.JButton();
        jSeparator3 = new javax.swing.JToolBar.Separator();
        cutbutton = new javax.swing.JButton();
        copybutton = new javax.swing.JButton();
        pasteButton = new javax.swing.JButton();
        undoButton = new javax.swing.JButton();
        redoButton = new javax.swing.JButton();
        jSeparator2 = new javax.swing.JToolBar.Separator();
        selectButton = new javax.swing.JButton();
        selectionToolButton = new javax.swing.JToggleButton();
        dragToolButton = new javax.swing.JToggleButton();
        drawToolButton = new javax.swing.JToggleButton();
        zoomToolButton = new javax.swing.JToggleButton();
        zoomLevelCombobox = new javax.swing.JComboBox();
        jSeparator4 = new javax.swing.JToolBar.Separator();
        recordButton = new javax.swing.JToggleButton();
        executeButton = new javax.swing.JToggleButton();
        stopButton = new javax.swing.JToggleButton();
        jSeparator1 = new javax.swing.JToolBar.Separator();
        jLabel2 = new javax.swing.JLabel();
        centerArea = new javax.swing.JPanel();
        horisontalSplitPane = new javax.swing.JSplitPane();
        verticalSplitPane = new javax.swing.JSplitPane();
        allListsPane = new javax.swing.JPanel();
        aboveListsPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        allListsSplitPane = new javax.swing.JSplitPane();
        firstSplitPane = new javax.swing.JSplitPane();
        logTopPanel = new javax.swing.JPanel();
        logLeftPanel = new javax.swing.JPanel();
        logRightPanel = new javax.swing.JPanel();
        logScrollPane = new javax.swing.JScrollPane();
        logPanel = new javax.swing.JTextPane();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        fileNewMenuItem = new javax.swing.JMenuItem();
        fileOpenMenuItem = new javax.swing.JMenuItem();
        fileOpenRecentSubmenu = new javax.swing.JMenu();
        fileCloseMenuItem = new javax.swing.JMenuItem();
        jSeparator11 = new javax.swing.JSeparator();
        fileSaveMenuItem = new javax.swing.JMenuItem();
        fileSaveAsMenuItem = new javax.swing.JMenuItem();
        fileSaveAllMenuItem = new javax.swing.JMenuItem();
        jSeparator12 = new javax.swing.JSeparator();
        fileSaveSessionMenuItem = new javax.swing.JMenuItem();
        fileRestoreSessionMenuItem = new javax.swing.JMenuItem();
        fileRestoreRecentSessionSubmenu = new javax.swing.JMenu();
        jSeparator10 = new javax.swing.JSeparator();
        filePrintMenuItem = new javax.swing.JMenuItem();
        jMenuItem1 = new javax.swing.JMenuItem();
        jSeparator8 = new javax.swing.JSeparator();
        javax.swing.JMenuItem fileExitMenuItem = new javax.swing.JMenuItem();
        editMenu = new javax.swing.JMenu();
        cutMenuItem = new javax.swing.JMenuItem();
        copyMenuItem = new javax.swing.JMenuItem();
        pasteMenuItem = new javax.swing.JMenuItem();
        jSeparator5 = new javax.swing.JSeparator();
        undoMenuItem = new javax.swing.JMenuItem();
        redoMenuItem = new javax.swing.JMenuItem();
        viewMenu = new javax.swing.JMenu();
        dataMenu = new javax.swing.JMenu();
        addSequencesMenuItem = new javax.swing.JMenuItem();
        jMenuItem4 = new javax.swing.JMenuItem();
        addDataTracksMenuItem = new javax.swing.JMenuItem();
        addNewDataItemMenu = new javax.swing.JMenu();
        addBackgroundMenuItem = new javax.swing.JMenuItem();
        addMotifMenuItem = new javax.swing.JMenuItem();
        addMotifCollectionMenuItem = new javax.swing.JMenuItem();
        addNumericVariableMenuItem = new javax.swing.JMenuItem();
        addSequenceCollectionMenuItem = new javax.swing.JMenuItem();
        addStringMenuItem = new javax.swing.JMenuItem();
        addStringCollectionMenuItem = new javax.swing.JMenuItem();
        jMenuItem3 = new javax.swing.JMenuItem();
        jSeparator14 = new javax.swing.JSeparator();
        jMenuItem2 = new javax.swing.JMenuItem();
        jMenuItem5 = new javax.swing.JMenuItem();
        fileClearAllDataMenuItem = new javax.swing.JMenuItem();
        operationsMenu = new javax.swing.JMenu();
        protocolMenu = new javax.swing.JMenu();
        recordMenuItem = new javax.swing.JMenuItem();
        executeMenuItem = new javax.swing.JMenuItem();
        stopMenuItem = new javax.swing.JMenuItem();
        jSeparator7 = new javax.swing.JSeparator();
        colorKeywordsMenuItem = new javax.swing.JCheckBoxMenuItem();
        changeProtocolSubmenu = new javax.swing.JMenu();
        toolsMenu = new javax.swing.JMenu();
        selectionToolMenuItem = new javax.swing.JMenuItem();
        handToolMenuItem = new javax.swing.JMenuItem();
        drawToolMenuItem = new javax.swing.JMenuItem();
        zoomToolMenuItem = new javax.swing.JMenuItem();
        jSeparator6 = new javax.swing.JSeparator();
        jMenuItem7 = new javax.swing.JMenuItem();
        jMenuItem8 = new javax.swing.JMenuItem();
        jSeparator13 = new javax.swing.JSeparator();
        configureDatatracksMenuItem = new javax.swing.JMenuItem();
        externalProgramMenuItem = new javax.swing.JMenuItem();
        optionsMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        helpMenuItem = new javax.swing.JMenuItem();
        jMenuItem9 = new javax.swing.JMenuItem();
        jMenuItem6 = new javax.swing.JMenuItem();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        selectedToolButtonGroup = new javax.swing.ButtonGroup();
        recplaystopButtonGroup = new javax.swing.ButtonGroup();

        topPanel.setName("topPanel"); // NOI18N
        topPanel.setLayout(new java.awt.BorderLayout());

        Toolbar.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        Toolbar.setFloatable(false);
        Toolbar.setRollover(true);
        Toolbar.setMinimumSize(new java.awt.Dimension(1224, 39));
        Toolbar.setName("Toolbar"); // NOI18N
        Toolbar.setPreferredSize(new java.awt.Dimension(15, 39));

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this);
        newButton.setAction(actionMap.get("newProtocol")); // NOI18N
        newButton.setFocusable(false);
        newButton.setHideActionText(true);
        newButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        newButton.setName("newButton"); // NOI18N
        newButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(newButton);

        openButton.setAction(actionMap.get("openProtocol")); // NOI18N
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(PriorsEditorGUI.class);
        openButton.setText(resourceMap.getString("openButton.text")); // NOI18N
        openButton.setFocusable(false);
        openButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        openButton.setName("openButton"); // NOI18N
        openButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(openButton);

        saveButton.setAction(actionMap.get("saveFile")); // NOI18N
        saveButton.setText(resourceMap.getString("saveButton.text")); // NOI18N
        saveButton.setFocusable(false);
        saveButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        saveButton.setName("saveButton"); // NOI18N
        saveButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(saveButton);

        printButton.setAction(actionMap.get("printActionMethod")); // NOI18N
        printButton.setText(resourceMap.getString("printButton.text")); // NOI18N
        printButton.setFocusable(false);
        printButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        printButton.setName("printButton"); // NOI18N
        printButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(printButton);

        jSeparator9.setName("jSeparator9"); // NOI18N
        Toolbar.add(jSeparator9);

        sequenceButton.setAction(actionMap.get("showSequenceInputDialog")); // NOI18N
        sequenceButton.setText(resourceMap.getString("")); // NOI18N
        sequenceButton.setFocusable(false);
        sequenceButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        sequenceButton.setName("sequenceButton"); // NOI18N
        sequenceButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(sequenceButton);

        databaseButton.setAction(actionMap.get("showDatatrackDialog")); // NOI18N
        databaseButton.setText(resourceMap.getString("")); // NOI18N
        databaseButton.setFocusable(false);
        databaseButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        databaseButton.setName("databaseButton"); // NOI18N
        databaseButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(databaseButton);

        jSeparator3.setName("jSeparator3"); // NOI18N
        Toolbar.add(jSeparator3);

        cutbutton.setAction(getApplication().getContext().getActionMap().get("cut"));
        cutbutton.setIcon(resourceMap.getIcon("cutbutton.icon")); // NOI18N
        cutbutton.setFocusable(false);
        cutbutton.setHideActionText(true);
        cutbutton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        cutbutton.setName("cutbutton"); // NOI18N
        cutbutton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(cutbutton);

        copybutton.setAction(getApplication().getContext().getActionMap().get("copy"));
        copybutton.setIcon(resourceMap.getIcon("copybutton.icon")); // NOI18N
        copybutton.setFocusable(false);
        copybutton.setHideActionText(true);
        copybutton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        copybutton.setName("copybutton"); // NOI18N
        copybutton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(copybutton);

        pasteButton.setAction(getApplication().getContext().getActionMap().get("paste"));
        pasteButton.setIcon(resourceMap.getIcon("pasteButton.icon")); // NOI18N
        pasteButton.setFocusable(false);
        pasteButton.setHideActionText(true);
        pasteButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        pasteButton.setName("pasteButton"); // NOI18N
        pasteButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(pasteButton);

        undoButton.setAction(actionMap.get("undo")); // NOI18N
        undoButton.setFocusable(false);
        undoButton.setHideActionText(true);
        undoButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        undoButton.setName("undoButton"); // NOI18N
        undoButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(undoButton);

        redoButton.setAction(actionMap.get("redo")); // NOI18N
        redoButton.setFocusable(false);
        redoButton.setHideActionText(true);
        redoButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        redoButton.setName("redoButton"); // NOI18N
        redoButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(redoButton);

        jSeparator2.setName("jSeparator2"); // NOI18N
        Toolbar.add(jSeparator2);

        selectButton.setAction(actionMap.get("selectButtonAction")); // NOI18N
        selectButton.setFocusable(false);
        selectButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        selectButton.setName("selectButton"); // NOI18N
        selectButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(selectButton);

        selectionToolButton.setAction(actionMap.get("selectionToolAction")); // NOI18N
        selectedToolButtonGroup.add(selectionToolButton);
        selectionToolButton.setFocusable(false);
        selectionToolButton.setHideActionText(true);
        selectionToolButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        selectionToolButton.setName("selectionToolButton"); // NOI18N
        selectionToolButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(selectionToolButton);

        dragToolButton.setAction(actionMap.get("handToolAction")); // NOI18N
        selectedToolButtonGroup.add(dragToolButton);
        dragToolButton.setToolTipText(resourceMap.getString("dragToolButton.toolTipText")); // NOI18N
        dragToolButton.setFocusable(false);
        dragToolButton.setHideActionText(true);
        dragToolButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        dragToolButton.setName("dragToolButton"); // NOI18N
        dragToolButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(dragToolButton);

        drawToolButton.setAction(actionMap.get("drawToolAction")); // NOI18N
        selectedToolButtonGroup.add(drawToolButton);
        drawToolButton.setText(resourceMap.getString("drawToolButton.text")); // NOI18N
        drawToolButton.setFocusable(false);
        drawToolButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        drawToolButton.setName("drawToolButton"); // NOI18N
        drawToolButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(drawToolButton);

        zoomToolButton.setAction(actionMap.get("zoomToolAction")); // NOI18N
        selectedToolButtonGroup.add(zoomToolButton);
        zoomToolButton.setFocusable(false);
        zoomToolButton.setHideActionText(true);
        zoomToolButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        zoomToolButton.setName("zoomToolButton"); // NOI18N
        zoomToolButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(zoomToolButton);

        zoomLevelCombobox.setModel(getModelForZoomLevelCombobox());
        zoomLevelCombobox.setFocusable(false);
        zoomLevelCombobox.setName("zoomLevelCombobox"); // NOI18N
        zoomLevelCombobox.setRenderer(new ZoomLevelComboboxCellRenderer());
        Toolbar.add(zoomLevelCombobox);
        zoomLevelCombobox.addActionListener(new ZoomLevelComboboxActionListener());

        jSeparator4.setName("jSeparator4"); // NOI18N
        Toolbar.add(jSeparator4);

        recordButton.setAction(actionMap.get("recordMode")); // NOI18N
        recplaystopButtonGroup.add(recordButton);
        recordButton.setFocusPainted(false);
        recordButton.setFocusable(false);
        recordButton.setHideActionText(true);
        recordButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        recordButton.setName("recordButton"); // NOI18N
        recordButton.setRolloverEnabled(true);
        recordButton.setSelectedIcon(resourceMap.getIcon("recordButton.selectedIcon")); // NOI18N
        recordButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(recordButton);

        executeButton.setAction(actionMap.get("executeProtocol")); // NOI18N
        recplaystopButtonGroup.add(executeButton);
        executeButton.setFocusPainted(false);
        executeButton.setFocusable(false);
        executeButton.setHideActionText(true);
        executeButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        executeButton.setName("executeButton"); // NOI18N
        executeButton.setRolloverEnabled(true);
        executeButton.setSelectedIcon(resourceMap.getIcon("executeButton.selectedIcon")); // NOI18N
        executeButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(executeButton);

        stopButton.setAction(actionMap.get("stopAction")); // NOI18N
        recplaystopButtonGroup.add(stopButton);
        stopButton.setFocusPainted(false);
        stopButton.setFocusable(false);
        stopButton.setHideActionText(true);
        stopButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        stopButton.setName("stopButton"); // NOI18N
        stopButton.setRolloverEnabled(true);
        stopButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        Toolbar.add(stopButton);

        jSeparator1.setName("jSeparator1"); // NOI18N
        Toolbar.add(jSeparator1);

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setMaximumSize(new java.awt.Dimension(450, 10));
        jLabel2.setMinimumSize(new java.awt.Dimension(450, 10));
        jLabel2.setName("jLabel2"); // NOI18N
        jLabel2.setPreferredSize(new java.awt.Dimension(450, 10));
        Toolbar.add(jLabel2);

        topPanel.add(Toolbar, java.awt.BorderLayout.PAGE_START);

        centerArea.setBorder(javax.swing.BorderFactory.createEmptyBorder(3, 3, 3, 3));
        centerArea.setName("centerArea"); // NOI18N
        centerArea.setLayout(new java.awt.BorderLayout());

        horisontalSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        horisontalSplitPane.setName("horisontalSplitPane"); // NOI18N
        horisontalSplitPane.setOneTouchExpandable(true);

        verticalSplitPane.setName("verticalSplitPane"); // NOI18N
        verticalSplitPane.setOneTouchExpandable(true);

        allListsPane.setName("allListsPane"); // NOI18N
        allListsPane.setLayout(new java.awt.BorderLayout());

        aboveListsPanel.setName("aboveListsPanel"); // NOI18N
        aboveListsPanel.setLayout(new java.awt.BorderLayout());

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setBorder(javax.swing.BorderFactory.createEmptyBorder(10, 1, 13, 1));
        jLabel1.setName("jLabel1"); // NOI18N
        aboveListsPanel.add(jLabel1, java.awt.BorderLayout.CENTER);

        allListsPane.add(aboveListsPanel, java.awt.BorderLayout.PAGE_START);

        allListsSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        allListsSplitPane.setName("allListsSplitPane"); // NOI18N

        firstSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        firstSplitPane.setName("firstSplitPane"); // NOI18N
        allListsSplitPane.setTopComponent(firstSplitPane);

        allListsPane.add(allListsSplitPane, java.awt.BorderLayout.CENTER);

        verticalSplitPane.setLeftComponent(allListsPane);

        horisontalSplitPane.setLeftComponent(verticalSplitPane);

        logTopPanel.setName("logTopPanel"); // NOI18N
        logTopPanel.setPreferredSize(new java.awt.Dimension(70, 52));
        logTopPanel.setLayout(new java.awt.BorderLayout());

        logLeftPanel.setMaximumSize(new java.awt.Dimension(30, 50));
        logLeftPanel.setMinimumSize(new java.awt.Dimension(30, 50));
        logLeftPanel.setName("logLeftPanel"); // NOI18N
        logLeftPanel.setPreferredSize(new java.awt.Dimension(30, 50));

        javax.swing.GroupLayout logLeftPanelLayout = new javax.swing.GroupLayout(logLeftPanel);
        logLeftPanel.setLayout(logLeftPanelLayout);
        logLeftPanelLayout.setHorizontalGroup(
            logLeftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 30, Short.MAX_VALUE)
        );
        logLeftPanelLayout.setVerticalGroup(
            logLeftPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 196, Short.MAX_VALUE)
        );

        logTopPanel.add(logLeftPanel, java.awt.BorderLayout.LINE_START);

        logRightPanel.setMaximumSize(new java.awt.Dimension(30, 50));
        logRightPanel.setMinimumSize(new java.awt.Dimension(30, 50));
        logRightPanel.setName("logRightPanel"); // NOI18N

        javax.swing.GroupLayout logRightPanelLayout = new javax.swing.GroupLayout(logRightPanel);
        logRightPanel.setLayout(logRightPanelLayout);
        logRightPanelLayout.setHorizontalGroup(
            logRightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );
        logRightPanelLayout.setVerticalGroup(
            logRightPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 196, Short.MAX_VALUE)
        );

        logTopPanel.add(logRightPanel, java.awt.BorderLayout.LINE_END);

        logScrollPane.setName("logScrollPane"); // NOI18N

        logPanel.setEditable(false);
        logPanel.setName("logPanel"); // NOI18N
        logPanel.setPreferredSize(new java.awt.Dimension(6, 30));
        logScrollPane.setViewportView(logPanel);

        logTopPanel.add(logScrollPane, java.awt.BorderLayout.CENTER);

        horisontalSplitPane.setRightComponent(logTopPanel);

        centerArea.add(horisontalSplitPane, java.awt.BorderLayout.CENTER);

        topPanel.add(centerArea, java.awt.BorderLayout.CENTER);

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        fileNewMenuItem.setAction(actionMap.get("newProtocol")); // NOI18N
        fileNewMenuItem.setName("fileNewMenuItem"); // NOI18N
        fileMenu.add(fileNewMenuItem);

        fileOpenMenuItem.setAction(actionMap.get("openProtocol")); // NOI18N
        fileOpenMenuItem.setName("fileOpenMenuItem"); // NOI18N
        fileMenu.add(fileOpenMenuItem);

        fileOpenRecentSubmenu.setText(resourceMap.getString("fileOpenRecentSubmenu.text")); // NOI18N
        fileOpenRecentSubmenu.setEnabled(false);
        fileOpenRecentSubmenu.setName("fileOpenRecentSubmenu"); // NOI18N
        fileMenu.add(fileOpenRecentSubmenu);

        fileCloseMenuItem.setAction(actionMap.get("closefileActionMethod")); // NOI18N
        fileCloseMenuItem.setName("fileCloseMenuItem"); // NOI18N
        fileMenu.add(fileCloseMenuItem);

        jSeparator11.setName("jSeparator11"); // NOI18N
        fileMenu.add(jSeparator11);

        fileSaveMenuItem.setAction(actionMap.get("saveFile")); // NOI18N
        fileSaveMenuItem.setName("fileSaveMenuItem"); // NOI18N
        fileMenu.add(fileSaveMenuItem);

        fileSaveAsMenuItem.setAction(actionMap.get("saveAsActionMethod")); // NOI18N
        fileSaveAsMenuItem.setName("fileSaveAsMenuItem"); // NOI18N
        fileMenu.add(fileSaveAsMenuItem);

        fileSaveAllMenuItem.setAction(actionMap.get("saveAllActionMethod")); // NOI18N
        fileSaveAllMenuItem.setName("fileSaveAllMenuItem"); // NOI18N
        fileMenu.add(fileSaveAllMenuItem);

        jSeparator12.setName("jSeparator12"); // NOI18N
        fileMenu.add(jSeparator12);

        fileSaveSessionMenuItem.setAction(actionMap.get("saveSessionActionMethod")); // NOI18N
        fileSaveSessionMenuItem.setName("fileSaveSessionMenuItem"); // NOI18N
        fileMenu.add(fileSaveSessionMenuItem);

        fileRestoreSessionMenuItem.setAction(actionMap.get("restoreSessionActionMethod")); // NOI18N
        fileRestoreSessionMenuItem.setName("fileRestoreSessionMenuItem"); // NOI18N
        fileMenu.add(fileRestoreSessionMenuItem);

        fileRestoreRecentSessionSubmenu.setText(resourceMap.getString("fileRestoreRecentSessionSubmenu.text")); // NOI18N
        fileRestoreRecentSessionSubmenu.setEnabled(false);
        fileRestoreRecentSessionSubmenu.setName("fileRestoreRecentSessionSubmenu"); // NOI18N
        fileMenu.add(fileRestoreRecentSessionSubmenu);

        jSeparator10.setName("jSeparator10"); // NOI18N
        fileMenu.add(jSeparator10);

        filePrintMenuItem.setAction(actionMap.get("printActionMethod")); // NOI18N
        filePrintMenuItem.setName("filePrintMenuItem"); // NOI18N
        fileMenu.add(filePrintMenuItem);

        jMenuItem1.setAction(actionMap.get("saveAsImageMethod")); // NOI18N
        jMenuItem1.setName("jMenuItem1"); // NOI18N
        fileMenu.add(jMenuItem1);

        jSeparator8.setName("jSeparator8"); // NOI18N
        fileMenu.add(jSeparator8);

        fileExitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        fileExitMenuItem.setName("fileExitMenuItem"); // NOI18N
        fileMenu.add(fileExitMenuItem);

        menuBar.add(fileMenu);

        editMenu.setText(resourceMap.getString("editMenu.text")); // NOI18N
        editMenu.setActionCommand(resourceMap.getString("editMenu.actionCommand")); // NOI18N
        editMenu.setName("editMenu"); // NOI18N

        cutMenuItem.setAction(getApplication().getContext().getActionMap().get("cut"));
        cutMenuItem.setName("cutMenuItem"); // NOI18N
        editMenu.add(cutMenuItem);

        copyMenuItem.setAction(getApplication().getContext().getActionMap().get("copy"));
        copyMenuItem.setName("copyMenuItem"); // NOI18N
        editMenu.add(copyMenuItem);

        pasteMenuItem.setAction(getApplication().getContext().getActionMap().get("paste"));
        pasteMenuItem.setName("pasteMenuItem"); // NOI18N
        editMenu.add(pasteMenuItem);

        jSeparator5.setName("jSeparator5"); // NOI18N
        editMenu.add(jSeparator5);

        undoMenuItem.setAction(actionMap.get("undo")); // NOI18N
        undoMenuItem.setName("undoMenuItem"); // NOI18N
        editMenu.add(undoMenuItem);

        redoMenuItem.setAction(actionMap.get("redo")); // NOI18N
        redoMenuItem.setName("redoMenuItem"); // NOI18N
        editMenu.add(redoMenuItem);

        menuBar.add(editMenu);

        viewMenu.setText(resourceMap.getString("viewMenu.text")); // NOI18N
        viewMenu.setName("viewMenu"); // NOI18N
        menuBar.add(viewMenu);

        dataMenu.setText(resourceMap.getString("dataMenu.text")); // NOI18N
        dataMenu.setName("dataMenu"); // NOI18N

        addSequencesMenuItem.setAction(actionMap.get("showSequenceInputDialog")); // NOI18N
        addSequencesMenuItem.setName("addSequencesMenuItem"); // NOI18N
        dataMenu.add(addSequencesMenuItem);

        jMenuItem4.setAction(actionMap.get("showAddSequencesFromFileDialog")); // NOI18N
        jMenuItem4.setName("jMenuItem4"); // NOI18N
        dataMenu.add(jMenuItem4);

        addDataTracksMenuItem.setAction(actionMap.get("showDatatrackDialog")); // NOI18N
        addDataTracksMenuItem.setName("addDataTracksMenuItem"); // NOI18N
        dataMenu.add(addDataTracksMenuItem);

        addNewDataItemMenu.setText(resourceMap.getString("addNewDataItemMenu.text")); // NOI18N
        addNewDataItemMenu.setName("addNewDataItemMenu"); // NOI18N

        addBackgroundMenuItem.setText(resourceMap.getString("addBackgroundMenuItem.text")); // NOI18N
        addBackgroundMenuItem.setName("addBackgroundMenuItem"); // NOI18N
        addBackgroundMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addDataItemMenuHandler(evt);
            }
        });
        addNewDataItemMenu.add(addBackgroundMenuItem);

        addMotifMenuItem.setText(resourceMap.getString("addMotifMenuItem.text")); // NOI18N
        addMotifMenuItem.setName("addMotifMenuItem"); // NOI18N
        addMotifMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addDataItemMenuHandler(evt);
            }
        });
        addNewDataItemMenu.add(addMotifMenuItem);

        addMotifCollectionMenuItem.setText(resourceMap.getString("addMotifCollectionMenuItem.text")); // NOI18N
        addMotifCollectionMenuItem.setName("addMotifCollectionMenuItem"); // NOI18N
        addMotifCollectionMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addDataItemMenuHandler(evt);
            }
        });
        addNewDataItemMenu.add(addMotifCollectionMenuItem);

        addNumericVariableMenuItem.setText(resourceMap.getString("addNumericVariableMenuItem.text")); // NOI18N
        addNumericVariableMenuItem.setName("addNumericVariableMenuItem"); // NOI18N
        addNumericVariableMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addDataItemMenuHandler(evt);
            }
        });
        addNewDataItemMenu.add(addNumericVariableMenuItem);

        addSequenceCollectionMenuItem.setText(resourceMap.getString("addSequenceCollectionMenuItem.text")); // NOI18N
        addSequenceCollectionMenuItem.setName("addSequenceCollectionMenuItem"); // NOI18N
        addSequenceCollectionMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addDataItemMenuHandler(evt);
            }
        });
        addNewDataItemMenu.add(addSequenceCollectionMenuItem);

        addStringMenuItem.setText(resourceMap.getString("addStringMenuItem.text")); // NOI18N
        addStringMenuItem.setName("addStringMenuItem"); // NOI18N
        addStringMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addDataItemMenuHandler(evt);
            }
        });
        addNewDataItemMenu.add(addStringMenuItem);

        addStringCollectionMenuItem.setText(resourceMap.getString("addStringCollectionMenuItem.text")); // NOI18N
        addStringCollectionMenuItem.setName("addStringCollectionMenuItem"); // NOI18N
        addStringCollectionMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addDataItemMenuHandler(evt);
            }
        });
        addNewDataItemMenu.add(addStringCollectionMenuItem);

        dataMenu.add(addNewDataItemMenu);

        jMenuItem3.setAction(actionMap.get("showLoadFromFileDialog")); // NOI18N
        jMenuItem3.setName("jMenuItem3"); // NOI18N
        dataMenu.add(jMenuItem3);

        jSeparator14.setName("jSeparator14"); // NOI18N
        dataMenu.add(jSeparator14);

        jMenuItem2.setAction(actionMap.get("clearAllSequencesActionMethod")); // NOI18N
        jMenuItem2.setName("jMenuItem2"); // NOI18N
        dataMenu.add(jMenuItem2);

        jMenuItem5.setAction(actionMap.get("clearAllMotifsActionMethod")); // NOI18N
        jMenuItem5.setName("jMenuItem5"); // NOI18N
        dataMenu.add(jMenuItem5);

        fileClearAllDataMenuItem.setAction(actionMap.get("clearAllDataActionMethod")); // NOI18N
        fileClearAllDataMenuItem.setText(resourceMap.getString("fileClearAllDataMenuItem.text")); // NOI18N
        fileClearAllDataMenuItem.setName("fileClearAllDataMenuItem"); // NOI18N
        dataMenu.add(fileClearAllDataMenuItem);

        menuBar.add(dataMenu);

        operationsMenu.setText(resourceMap.getString("operationsMenu.text")); // NOI18N
        operationsMenu.setName("operationsMenu"); // NOI18N
        menuBar.add(operationsMenu);

        protocolMenu.setText(resourceMap.getString("protocolMenu.text")); // NOI18N
        protocolMenu.setName("protocolMenu"); // NOI18N

        recordMenuItem.setAction(actionMap.get("recordMode")); // NOI18N
        recordMenuItem.setName("recordMenuItem"); // NOI18N
        recordMenuItem.setSelectedIcon(resourceMap.getIcon("recordMenuItem.selectedIcon")); // NOI18N
        protocolMenu.add(recordMenuItem);

        executeMenuItem.setAction(actionMap.get("executeProtocol")); // NOI18N
        executeMenuItem.setName("executeMenuItem"); // NOI18N
        protocolMenu.add(executeMenuItem);

        stopMenuItem.setAction(actionMap.get("stopAction")); // NOI18N
        stopMenuItem.setName("stopMenuItem"); // NOI18N
        protocolMenu.add(stopMenuItem);

        jSeparator7.setName("jSeparator7"); // NOI18N
        protocolMenu.add(jSeparator7);

        colorKeywordsMenuItem.setAction(actionMap.get("selectColorProtocol")); // NOI18N
        colorKeywordsMenuItem.setSelected(true);
        colorKeywordsMenuItem.setName("colorKeywordsMenuItem"); // NOI18N
        protocolMenu.add(colorKeywordsMenuItem);

        changeProtocolSubmenu.setText(resourceMap.getString("changeProtocolSubmenu.text")); // NOI18N
        changeProtocolSubmenu.setName("changeProtocolSubmenu"); // NOI18N
        protocolMenu.add(changeProtocolSubmenu);

        menuBar.add(protocolMenu);

        toolsMenu.setText(resourceMap.getString("toolsMenu.text")); // NOI18N
        toolsMenu.setName("toolsMenu"); // NOI18N

        selectionToolMenuItem.setAction(actionMap.get("selectionToolAction")); // NOI18N
        selectionToolMenuItem.setName("selectionToolMenuItem"); // NOI18N
        toolsMenu.add(selectionToolMenuItem);

        handToolMenuItem.setAction(actionMap.get("handToolAction")); // NOI18N
        handToolMenuItem.setName("handToolMenuItem"); // NOI18N
        toolsMenu.add(handToolMenuItem);

        drawToolMenuItem.setAction(actionMap.get("drawToolAction")); // NOI18N
        drawToolMenuItem.setName("drawToolMenuItem"); // NOI18N
        toolsMenu.add(drawToolMenuItem);

        zoomToolMenuItem.setAction(actionMap.get("zoomToolAction")); // NOI18N
        zoomToolMenuItem.setName("zoomToolMenuItem"); // NOI18N
        toolsMenu.add(zoomToolMenuItem);

        jSeparator6.setName("jSeparator6"); // NOI18N
        toolsMenu.add(jSeparator6);

        jMenuItem7.setAction(actionMap.get("showMotifBrowser")); // NOI18N
        jMenuItem7.setName("jMenuItem7"); // NOI18N
        toolsMenu.add(jMenuItem7);

        jMenuItem8.setAction(actionMap.get("showMotifScoreFilterDialog")); // NOI18N
        jMenuItem8.setName("jMenuItem8"); // NOI18N
        toolsMenu.add(jMenuItem8);

        jSeparator13.setName("jSeparator13"); // NOI18N
        toolsMenu.add(jSeparator13);

        configureDatatracksMenuItem.setAction(actionMap.get("showConfigureDatatracksDialog")); // NOI18N
        configureDatatracksMenuItem.setName("configureDatatracksMenuItem"); // NOI18N
        toolsMenu.add(configureDatatracksMenuItem);

        externalProgramMenuItem.setAction(actionMap.get("showConfigureExternalProgramsDialog")); // NOI18N
        externalProgramMenuItem.setName("externalProgramMenuItem"); // NOI18N
        toolsMenu.add(externalProgramMenuItem);

        optionsMenuItem.setAction(actionMap.get("showOptionsDialog")); // NOI18N
        optionsMenuItem.setName("optionsMenuItem"); // NOI18N
        toolsMenu.add(optionsMenuItem);

        menuBar.add(toolsMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        helpMenuItem.setAction(actionMap.get("helpAction")); // NOI18N
        helpMenuItem.setName("helpMenuItem"); // NOI18N
        helpMenu.add(helpMenuItem);

        jMenuItem9.setName("jMenuItem9"); // NOI18N
        helpMenu.add(jMenuItem9);

        jMenuItem6.setAction(actionMap.get("showCitationDialog")); // NOI18N
        jMenuItem6.setName("jMenuItem6"); // NOI18N
        helpMenu.add(jMenuItem6);

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setMaximumSize(new java.awt.Dimension(32767, 30));
        statusPanel.setMinimumSize(new java.awt.Dimension(0, 30));
        statusPanel.setName("statusPanel"); // NOI18N
        statusPanel.setPreferredSize(new java.awt.Dimension(2, 30));

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 1293, Short.MAX_VALUE)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(statusMessageLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 1119, Short.MAX_VALUE)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 9, Short.MAX_VALUE)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusMessageLabel)
                    .addComponent(statusAnimationLabel)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(3, 3, 3))
        );

        setComponent(topPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
        setToolBar(Toolbar);
    }// </editor-fold>//GEN-END:initComponents

private void addDataItemMenuHandler(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addDataItemMenuHandler
         gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
         Prompt prompt=null;
         String type=evt.getActionCommand();//GEN-LAST:event_addDataItemMenuHandler
              if (type.equals(SequenceCollection.getType())) prompt=new Prompt_SequenceCollection(gui, null, null);
         else if (type.equals(NumericVariable.getType())) prompt=new Prompt_NumericVariable(gui, null, null);
         else if (type.equals(BackgroundModel.getType())) prompt=new Prompt_BackgroundModel(gui, null, null);
         else if (type.equals(StringVariable.getType())) prompt=new Prompt_StringVariable(gui, null, null);
         else if (type.equals(StringCollection.getType())) prompt=new Prompt_StringCollection(gui, null, null);
         else if (type.equals(Motif.getType())) prompt=new Prompt_Motif(gui, null, null,visualizationSettings);
         else if (type.equals(MotifCollection.getType())) prompt=new Prompt_MotifCollection(gui, null, null);
         prompt.setLocation(gui.getFrame().getWidth()/2-prompt.getWidth()/2, gui.getFrame().getHeight()/2-prompt.getHeight()/2);
         prompt.setVisible(true);
         gui.getFrame().setCursor(Cursor.getDefaultCursor());
         if (prompt.isOKPressed()) {
            Data variable=prompt.getData();
            if (variable==null) {
                prompt.dispose();
                return;
            }
            OperationTask task=new OperationTask("new "+type);
            task.setParameter(Operation_new.DATA_TYPE, type);
            task.setParameter(OperationTask.TARGET_NAME,variable.getName());
            String parameterString=variable.getValueAsParameterString();
            task.setParameter(OperationTask.OPERATION_NAME, "new");
            task.setParameter(Operation_new.PARAMETERS, parameterString);
            task.setParameter(OperationTask.ENGINE,gui.getEngine());
            if (prompt.isDataImportedFromFile()) {
                prompt.setImportFromFileSettingsInTask(task); // sets FILENAME, DATA_FORMAT and DATA_FORMAT_SETTINGS parameters
            }
            variable.setAdditionalOperationNewTaskParameters(task);
            prompt.dispose();
            gui.launchOperationTask(task,gui.isRecording());
         } else {
             prompt.dispose();
         }
}


private void setupOperationsMenu() {
    ArrayList<Operation> operations=engine.getAllOperations();
    OperationsComparator comparator=new OperationsComparator();
    Collections.sort(operations,comparator);
    OperationsMenuListener menuListener = new OperationsMenuListener();
    for (Operation operation:operations) {
        JMenuItem item=new JMenuItem(operation.getName());
        item.setToolTipText(operation.getDescription());
        item.addActionListener(menuListener);
        operationsMenu.add(item);        
    }   
}
private class OperationsComparator implements Comparator<Operation> {
        @Override
        public int compare(Operation o1, Operation o2) {
            return o1.getName().compareTo(o2.getName());
        }    
}
    /**
     * An inner class that listens to popup-menu events related to the operations submenu and notifies the gui events
     */
    private class OperationsMenuListener implements ActionListener {
       public void actionPerformed(ActionEvent e) {
            Operation operation=engine.getOperation(e.getActionCommand());
            OperationTask parameters=new OperationTask(operation.getName());
            parameters.setParameter(OperationTask.OPERATION, operation);
            gui.launchOperationEditor(parameters);
       }            
    }


/** 
 * This method searches the engine for the data item with the given name and deletes it if found.
 * An undoableEdit event is registered with the undoManager so that the data item can be recovered again 
 */
public void deleteDataItem(String name) {
    if (engine.getDataItem(name)==null) return;
    getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    UndoMonitor undomonitor=undoManager.getUndoMonitor("Delete "+name);
    undomonitor.register();
    engine.removeDataItem(name);
    undomonitor.deregister(true);
    undoManager.addEdit(undomonitor);
    getFrame().setCursor(Cursor.getDefaultCursor());
}

/** 
 * This method searches the engine for the data items with the given names and deletes them if found.
 * An undoableEdit event is registered with the undoManager so that the data can be recovered again 
 */
public void deleteDataItems(String[] names) {
    getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    UndoMonitor undomonitor=undoManager.getUndoMonitor("Delete data");
    undomonitor.register();
    for (String name:names) {
        if (engine.getDataItem(name)!=null) engine.removeDataItem(name);
    }    
    undomonitor.deregister(true);
    undoManager.addEdit(undomonitor);
    getFrame().setCursor(Cursor.getDefaultCursor());
}

/** 
 * Given a (non-registered) copy of a Data item with the same name as a registered data item,
 * this method will update the registered data item of the same name by importing the values
 * from the provided copy. If there are no registered data items with the same name as the copy
 * the copy will be stored directly as a new data item. 
 * An undoableEdit event is registered with the undoManager so that the data update can be undone
 */
public void updateDataItem(Data data) {
    String undoName;
    boolean dataExists=false;
    if (engine.getDataItem(data.getName())!=null) dataExists=true;
    if (dataExists) undoName="Update "+data.getName();
    else undoName="Add data: "+data.getName();
    UndoMonitor undomonitor=undoManager.getUndoMonitor(undoName);
    undomonitor.register();
    try {
        if (dataExists) engine.updateDataItem(data);
        else engine.storeDataItem(data);
    } catch (ExecutionError e) {JOptionPane.showMessageDialog(getFrame(), "An error occurred during session restore:\n"+e.getMessage(),"Restore Session Error" ,JOptionPane.ERROR_MESSAGE);}
    undomonitor.deregister(true);
    undoManager.addEdit(undomonitor);
}



/**
  * Given a reference to a JList and a point (x,y), this function returns the index 
  * of the element corresponding to that point (between 0 and model.getSize()-1).
  * If the coordinate does not correspond to an element in the list, the method returns the value -1;
  * @param list The JList (fitted with a DefaultListModel)
  * @param x
  * @param y
  * @return
  */
 private int getElementIndexAtPointInList(JList list, int x, int y) {
         int index=list.locationToIndex(new java.awt.Point(x,y));
         if (index>=0) {
            javax.swing.DefaultListModel model=(javax.swing.DefaultListModel)list.getModel();
            java.awt.Rectangle bounds = list.getCellBounds(0, model.getSize()-1);
            if (y>bounds.getHeight() || y<0 || x>bounds.getWidth() || x<0) index=-1; // 
         }
         return index;
    }
 

   


    /**
     * This method is called whenever a print action is invoked
     * (it is the "actionPerformed" method for the print action listener)
     */
    @Action
    public void printActionMethod() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        Component currentWindow=mainWindow.getSelectedComponent();
        if (currentWindow instanceof ProtocolEditor) {
                if (java.awt.Desktop.isDesktopSupported()) {
                    java.awt.Desktop desktop=java.awt.Desktop.getDesktop();
                    try {
                        File tempfile=File.createTempFile(getProtocolEditor().getProtocol().getName(), ".txt");
                        savePlainDocumentToFile((javax.swing.text.PlainDocument)getProtocolEditor().getProtocol().getDocument(),tempfile);
                        desktop.print(tempfile);
                    } catch (Exception e) {
                        JOptionPane.showMessageDialog(this.getFrame(), e.getClass().getSimpleName()+":\n\n"+e.getMessage(),"Print Error" ,JOptionPane.ERROR_MESSAGE);
                    }
                } else {
                    try {
                        ((ProtocolEditor)currentWindow).print();
                    } catch (PrinterException e) {
                        JOptionPane.showMessageDialog(this.getFrame(), e.getMessage(),"Print Error" ,JOptionPane.ERROR_MESSAGE);
                    }
                }
        } else if (currentWindow instanceof OutputPanel) {
                if (java.awt.Desktop.isDesktopSupported()) {
                    java.awt.Desktop desktop=java.awt.Desktop.getDesktop();
                    try {
                        OutputData outputdata=((OutputPanel)currentWindow).getOutputData();
                        String suffix=(outputdata.isHTMLformatted())?".html":".txt";
                        File tempfile=File.createTempFile(outputdata.getName(), suffix);
                        outputdata.saveToFile(tempfile, false);
                        desktop.print(tempfile);
                    } catch (Exception e) {
                        JOptionPane.showMessageDialog(this.getFrame(), e.getClass().getSimpleName()+":\n\n"+e.getMessage(),"Print Error" ,JOptionPane.ERROR_MESSAGE);
                    }
                } else {
                    try {
                        ((OutputPanel)currentWindow).print();
                    } catch (PrinterException e) {
                        JOptionPane.showMessageDialog(this.getFrame(), e.getMessage(),"Print Error" ,JOptionPane.ERROR_MESSAGE);
                    }
                }
        } else if (currentWindow instanceof VisualizationPanel) {
                try {
                    ((VisualizationPanel)currentWindow).print(null, null, true, null, null, true);
                } catch (PrinterException e) {
                    JOptionPane.showMessageDialog(this.getFrame(), e.getMessage(),"Print Error" ,JOptionPane.ERROR_MESSAGE);
                }
        } else logMessage("Unable to print...");
        getFrame().setCursor(Cursor.getDefaultCursor());
    }

    /** Saves a document to file in a quick and dirty way... */
    private void savePlainDocumentToFile(javax.swing.text.PlainDocument document, File file) throws Exception {
        javax.swing.text.Element root=document.getDefaultRootElement();
        int linecount=root.getElementCount();
        java.io.PrintWriter outputStream=new java.io.PrintWriter(new java.io.FileWriter(file));
        for (int i=0;i<linecount;i++) {
              javax.swing.text.Element e=root.getElement(i);
              String line=document.getText(e.getStartOffset(),e.getEndOffset()-e.getStartOffset());
              line=line.trim();
              outputStream.println(line);
        }
        outputStream.close();
    }

    
       
    @Action(enabledProperty = "undoEnabled")
    public void undo() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        visualizationSettings.enableVisualizationSettingsNotifications(false); // disable to avoid redundant visualization updates        
        UndoableEdit edit=undoManager.peek();
        if (edit instanceof ExecutableTask) {
            scheduler.abort((ExecutableTask)edit);
        }
        try {undoManager.undo();}
        catch (Exception e) {logMessage("Undo failed: "+e.getMessage()+" ("+e.getClass().toString()+")");}
        visualizationSettings.enableVisualizationSettingsNotifications(true); // reenable
        visualizationSettings.notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.REORDERED, engine.getDefaultSequenceCollection());
        getFrame().setCursor(Cursor.getDefaultCursor());
    }

    
    @Action(enabledProperty = "redoEnabled")
    public void redo() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        visualizationSettings.enableVisualizationSettingsNotifications(false); // disable to avoid redundant visualization updates        
        try {undoManager.redo();}
        catch (Exception e) {
            logMessage("Redo failed: "+e.getMessage());
            //e.printStackTrace(System.err);
        
        }
        visualizationSettings.enableVisualizationSettingsNotifications(true); // reenable
        visualizationSettings.notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.REORDERED, engine.getDefaultSequenceCollection());
        getFrame().setCursor(Cursor.getDefaultCursor());
    }



    @Action
    public void helpAction() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("priorseditor/gui/resources/PriorsEditorApp"); // NOI18N
        String homepage=bundle.getString("Application.homepage");
        String manualhomepage=homepage+"/manual";
        try {
           java.awt.Desktop.getDesktop().browse(java.net.URI.create(manualhomepage));
        } catch (Exception e) {
            JOptionPane.showMessageDialog(getFrame(), "Unable to display help page in default browser","Help", JOptionPane.ERROR_MESSAGE, null);            
        }
        getFrame().setCursor(Cursor.getDefaultCursor());
    }

    @Action
    public void goToWebSite() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("priorseditor/gui/resources/PriorsEditorApp"); // NOI18N
        String homepage=bundle.getString("Application.homepage");
        try {
           java.awt.Desktop.getDesktop().browse(java.net.URI.create(homepage));
        } catch (Exception e) {
            JOptionPane.showMessageDialog(getFrame(), "Unable to display web site in default browser","Web Site", JOptionPane.ERROR_MESSAGE, null);            
        }
        getFrame().setCursor(Cursor.getDefaultCursor());        
    }    
    
    

    /** Implementation of MessageListener interface method */
    public void errorMessage(final String msg, final int errortype) {
          Runnable runnable = new Runnable() {
                public void run() {
                       try {
                            StyledDocument log=logPanel.getStyledDocument();
                            log.insertString(log.getLength(), msg+"\n", null);
                       } catch (Exception e) {System.err.println("ERROR: Unable to write message in LogPanel: '"+msg+"'");}
                } // end run()
          }; // end Runnable
          if (SwingUtilities.isEventDispatchThread()) runnable.run(); // invoke directly on EDT 
          else {SwingUtilities.invokeLater(runnable); // queue on EDT
        }
    }
    
    /** Implementation of MessageListener interface method */
    public void logMessage(final String msg) {
          Runnable runnable = new Runnable() {
                public void run() {
                       try {
                            StyledDocument log=logPanel.getStyledDocument();
                            log.insertString(log.getLength(), msg+"\n", null);
                       } catch (Exception e) {System.err.println("ERROR: Unable to write message in LogPanel: '"+msg+"'");}
                } // end run()
          }; // end Runnable
          if (SwingUtilities.isEventDispatchThread()) runnable.run(); // invoke directly on EDT 
          else {SwingUtilities.invokeLater(runnable); // queue on EDT
        }
    }
    
    /** Implementation of MessageListener interface method */
    public void outputMessage(final String msg) {
        logMessage(msg);
    }
    
    /** Implementation of MessageListener interface method */
    public void statusMessage(final String msg) {
        PropertyChangeEvent event=new PropertyChangeEvent(engine, ExecutableTask.STATUS_MESSAGE, msg, msg);
        progressMonitor.propertyChange(event);
    }
    
    /** Sets the value of the progressbar 
     *  the value should be between 0 and 100
     *  If provided value is outside this range, the progress bar will be hidden
     */
    public void setProgress(final int progress) {
        String property=(progress>=0 && progress<=100)?ExecutableTask.PROGRESS:"HIDE";
        PropertyChangeEvent event=new PropertyChangeEvent(engine, property, progress, progress);
        progressMonitor.propertyChange(event);
    }
        
    
    /** Implementation of MessageListener interface method */    
    public void debugMessage(final String msg) {
          Runnable runnable = new Runnable() {
                public void run() {
                       try {
                            StyledDocument log=logPanel.getStyledDocument();
                            log.insertString(log.getLength(), msg+"\n", null);
                       } catch (Exception e) {System.err.println("ERROR: Unable to write message in LogPanel: '"+msg+"'");}
                } // end run()
          }; // end Runnable
          if (SwingUtilities.isEventDispatchThread()) runnable.run(); // invoke directly on EDT 
          else {SwingUtilities.invokeLater(runnable); // queue on EDT
        }
    }




    @Action
    public void debugAction() {
        getVisualizationSettings().debug();
        undoManager.debug();
    }


    @Action
    public void selectionToolAction() {
        setSelectedTool(SELECTION_TOOL);
        selectionToolButton.setSelected(true);
    }

    @Action
    public void handToolAction() {
        setSelectedTool(HAND_TOOL);
        dragToolButton.setSelected(true);
    }

    @Action
    public void zoomToolAction() {
        setSelectedTool(ZOOM_TOOL);
        zoomToolButton.setSelected(true);
    }

    @Action
    public void drawToolAction() {
        setSelectedTool(DRAW_TOOL);
        drawToolButton.setSelected(true); 
    }

     
    
    
    private DefaultComboBoxModel getModelForZoomLevelCombobox() {
        double[] zoomlevels=getVisualizationSettings().getZoomChoices();
        Object[] elements=new Object[zoomlevels.length+2];
        Object selected=null;
        for (int i=0;i<zoomlevels.length;i++) {
            Double val=new Double(zoomlevels[i]);
            elements[i]=val;
            if (val==100) selected=val;
            
        }
        elements[elements.length-2]="To Fit";
        elements[elements.length-1]="Custom...";  
        DefaultComboBoxModel model=new DefaultComboBoxModel(elements);
        model.setSelectedItem(selected);
        return model;
        
    }
    
    public void setRecordEnabled(boolean enabled) {
        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this);
        actionMap.get("recordMode").setEnabled(enabled);        
    }
    public void setExecuteEnabled(boolean enabled) {
        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this);
        actionMap.get("executeProtocol").setEnabled(enabled);        
    }
    public void setStopEnabled(boolean enabled) {
        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this);
        actionMap.get("stopAction").setEnabled(enabled);        
    }
    

    @Action
    public void recordMode() {
        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this);
        actionMap.get("executeProtocol").setEnabled(false);
        actionMap.get("stopAction").setEnabled(true);
        recordButton.setSelected(true);
        setRecordMode(true);
                
    }

    @Action
    public void executeProtocol() {
        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this);
        Protocol protocol=protocolEditor.getProtocol();
        if (protocol.getDocument().getLength()==0) {
            recplaystopButtonGroup.clearSelection();
            actionMap.get("executeProtocol").setEnabled(true);
            actionMap.get("recordMode").setEnabled(true);
            actionMap.get("stopAction").setEnabled(false);
            statusMessage("Protocol script is emtpy");
            return;
        }
        if (engine.getDefaultSequenceCollection().size()==0) {
           showSequenceInputDialog(); // ask the user to specify sequences if there is none           
        }
        actionMap.get("recordMode").setEnabled(false);
        actionMap.get("executeProtocol").setEnabled(false);
        actionMap.get("stopAction").setEnabled(true);    
        executeButton.setSelected(true);
        try {
            statusMessage("Parsing protocol script... ");
            ProtocolTask task=protocol.parse();
            statusMessage("Parsing protocol script... Completed!");
            UndoMonitor monitor=undoManager.getUndoMonitor(task.getPresentationName());
            task.setUndoMonitor(monitor);
            undoManager.addEdit(task);
            task.addPropertyChangeListener(new protocolTaskListener(actionMap));
            scheduler.submit(task);
        } catch (ParseError e) {
            String errorpanemessage=e.getMessage();   
            if (e.getLineNumber()!=0) errorpanemessage="Line "+e.getLineNumber()+":\n"+errorpanemessage;
            JOptionPane.showMessageDialog(this.getFrame(), errorpanemessage,"Parse Error" ,JOptionPane.ERROR_MESSAGE);
            String errormessage;   
            if (e.getLineNumber()!=0) errormessage="Parse Error [line "+e.getLineNumber()+"]:"+e.getMessage();
            else errormessage="Parse Error :"+e.getMessage();
            errorMessage(errormessage, 0);
            recplaystopButtonGroup.clearSelection();
            actionMap.get("executeProtocol").setEnabled(true);
            actionMap.get("recordMode").setEnabled(true);
            actionMap.get("stopAction").setEnabled(false);
        }
    }
    
    /** Sets the selected status of the Record, Play and Stop buttons to false */
    public void unselectRecordPlayStop() {
        recplaystopButtonGroup.clearSelection();
    }
    
    private class protocolTaskListener implements PropertyChangeListener {  
        javax.swing.ActionMap actionMap;
        public protocolTaskListener(javax.swing.ActionMap actionMap) {this.actionMap=actionMap;}
        public void propertyChange(PropertyChangeEvent evt) {
            if (!evt.getPropertyName().equals(ExecutableTask.STATUS)) return;
            String value=(String)evt.getNewValue();
            if (value.equals(ExecutableTask.DONE) || value.equals(ExecutableTask.ABORTED) || value.equals(ExecutableTask.ERROR)) {
                recplaystopButtonGroup.clearSelection();
                actionMap.get("executeProtocol").setEnabled(true);
                actionMap.get("recordMode").setEnabled(true);
                actionMap.get("stopAction").setEnabled(false);
            }
        }
        
    }
    
    @Action
    public void stopAction() {
        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this); 
        actionMap.get("stopAction").setEnabled(false);
        actionMap.get("executeProtocol").setEnabled(false);
        actionMap.get("stopAction").setEnabled(false); 
        if (recording) { // record mode
            recplaystopButtonGroup.clearSelection();
            actionMap.get("executeProtocol").setEnabled(true);
            actionMap.get("recordMode").setEnabled(true);
            actionMap.get("stopAction").setEnabled(false); 
            setRecordMode(false);
        }
        else { // play mode (executing script)
            boolean aborted=abortExecution();
            if (!aborted) {
                recplaystopButtonGroup.clearSelection();
                actionMap.get("executeProtocol").setEnabled(false);
                executeButton.setSelected(true);
                actionMap.get("recordMode").setEnabled(false);
                actionMap.get("stopAction").setEnabled(true);
            } else {
                recplaystopButtonGroup.clearSelection();
                actionMap.get("executeProtocol").setEnabled(true);
                actionMap.get("recordMode").setEnabled(true);
                actionMap.get("stopAction").setEnabled(false);                 
            }
        }

    }
    
    
    @Action
    public void showMemoryUsage() {
        MemoryMXBean memoryBean = ManagementFactory.getMemoryMXBean();
        MemoryUsage heapmemory= memoryBean.getHeapMemoryUsage();
        MemoryUsage nonheapmemory= memoryBean.getNonHeapMemoryUsage();        
        debugMessage("Memory usage\n===========\n\nHEAP\n  "+heapmemory.toString()+"\n\nNON-HEAP\n  "+nonheapmemory.toString());
    }

    private boolean recording = false;   
    /**
     * Returns true if the system is currently in "RECORD" mode,
     * meaning that any operations executed should be logged in the protocol script
     * @return
     */ 
    public boolean isRecording() {
        return recording;
    }
    /**
     * Sets the value of the "recording" flag
     * @return
     */
    public void setRecordMode(boolean isRecording) {
        recording=isRecording;
    }
    
    private boolean saveEnabled = false;
    public boolean isSaveEnabled() {
        return saveEnabled;
    }
    public void setSaveEnabled(boolean b) {
        boolean old = isSaveEnabled();
        this.saveEnabled = b;
        firePropertyChange("saveEnabled", old, isSaveEnabled());
    } 
    
    private boolean saveAsEnabled = false;
    public boolean isSaveAsEnabled() {
        return saveAsEnabled;
    }
    public void setSaveAsEnabled(boolean b) {
        boolean old = isSaveAsEnabled();
        this.saveAsEnabled = b;
        firePropertyChange("saveAsEnabled", old, isSaveAsEnabled());
    }   
    private boolean saveAllEnabled = false;
    public boolean isSaveAllEnabled() {
        return saveAllEnabled;
    }
    public void setSaveAllEnabled(boolean b) {
        boolean old = isSaveAllEnabled();
        this.saveAllEnabled = b;
        firePropertyChange("saveAllEnabled", old, isSaveAllEnabled());
    }   
    
    private boolean closeEnabled = false;
    public boolean isCloseEnabled() {
        return closeEnabled;
    }
    public void setCloseEnabled(boolean b) {
        boolean old = isCloseEnabled();
        this.closeEnabled = b;
        firePropertyChange("closeEnabled", old, isCloseEnabled());
    }   

    
    
    private boolean redoEnabled = false;
    public boolean isRedoEnabled() {
        return redoEnabled;
    }

    public void setRedoEnabled(boolean b) {
        boolean old = isRedoEnabled();
        this.redoEnabled = b;
        firePropertyChange("redoEnabled", old, isRedoEnabled());
    }

    
    
    private boolean undoEnabled = false;
    public boolean isUndoEnabled() {
        return undoEnabled;
    }

    public void setUndoEnabled(boolean b) {
        boolean old = isUndoEnabled();
        this.undoEnabled = b;
        firePropertyChange("undoEnabled", old, isUndoEnabled());
    }
    
    /** This should be called whenever undos/redos are made so that actions are updated properly*/
    public void updateUndoRedoStates() {
        if (undoManager.canUndo()) {// System.err.println("Can undo");
                setUndoEnabled(true);
                undoButton.setToolTipText(undoManager.getUndoPresentationName());                
            } else { //System.err.println("Can not undo");
                setUndoEnabled(false);
                undoButton.setToolTipText(null);
            }
            
            if (undoManager.canRedo()) { //System.err.println("Can redo");
                setRedoEnabled(true);
                redoButton.setToolTipText(undoManager.getRedoPresentationName());                
            } else { //System.err.println("Can not redo");
                setRedoEnabled(false);
                redoButton.setToolTipText(null);
            }         
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JToolBar Toolbar;
    private javax.swing.JPanel aboveListsPanel;
    private javax.swing.JMenuItem addBackgroundMenuItem;
    private javax.swing.JMenuItem addDataTracksMenuItem;
    private javax.swing.JMenuItem addMotifCollectionMenuItem;
    private javax.swing.JMenuItem addMotifMenuItem;
    private javax.swing.JMenu addNewDataItemMenu;
    private javax.swing.JMenuItem addNumericVariableMenuItem;
    private javax.swing.JMenuItem addSequenceCollectionMenuItem;
    private javax.swing.JMenuItem addSequencesMenuItem;
    private javax.swing.JMenuItem addStringCollectionMenuItem;
    private javax.swing.JMenuItem addStringMenuItem;
    private javax.swing.JPanel allListsPane;
    private javax.swing.JSplitPane allListsSplitPane;
    private javax.swing.JPanel centerArea;
    private javax.swing.JMenu changeProtocolSubmenu;
    private javax.swing.JCheckBoxMenuItem colorKeywordsMenuItem;
    private javax.swing.JMenuItem configureDatatracksMenuItem;
    private javax.swing.JMenuItem copyMenuItem;
    private javax.swing.JButton copybutton;
    private javax.swing.JMenuItem cutMenuItem;
    private javax.swing.JButton cutbutton;
    private javax.swing.JMenu dataMenu;
    private javax.swing.JButton databaseButton;
    private javax.swing.JToggleButton dragToolButton;
    private javax.swing.JToggleButton drawToolButton;
    private javax.swing.JMenuItem drawToolMenuItem;
    private javax.swing.JMenu editMenu;
    private javax.swing.JToggleButton executeButton;
    private javax.swing.JMenuItem executeMenuItem;
    private javax.swing.JMenuItem externalProgramMenuItem;
    private javax.swing.JMenuItem fileClearAllDataMenuItem;
    private javax.swing.JMenuItem fileCloseMenuItem;
    private javax.swing.JMenuItem fileNewMenuItem;
    private javax.swing.JMenuItem fileOpenMenuItem;
    private javax.swing.JMenu fileOpenRecentSubmenu;
    private javax.swing.JMenuItem filePrintMenuItem;
    private javax.swing.JMenu fileRestoreRecentSessionSubmenu;
    private javax.swing.JMenuItem fileRestoreSessionMenuItem;
    private javax.swing.JMenuItem fileSaveAllMenuItem;
    private javax.swing.JMenuItem fileSaveAsMenuItem;
    private javax.swing.JMenuItem fileSaveMenuItem;
    private javax.swing.JMenuItem fileSaveSessionMenuItem;
    private javax.swing.JSplitPane firstSplitPane;
    private javax.swing.JMenuItem handToolMenuItem;
    private javax.swing.JMenuItem helpMenuItem;
    private javax.swing.JSplitPane horisontalSplitPane;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    private javax.swing.JMenuItem jMenuItem5;
    private javax.swing.JMenuItem jMenuItem6;
    private javax.swing.JMenuItem jMenuItem7;
    private javax.swing.JMenuItem jMenuItem8;
    private javax.swing.JMenuItem jMenuItem9;
    private javax.swing.JToolBar.Separator jSeparator1;
    private javax.swing.JSeparator jSeparator10;
    private javax.swing.JSeparator jSeparator11;
    private javax.swing.JSeparator jSeparator12;
    private javax.swing.JSeparator jSeparator13;
    private javax.swing.JSeparator jSeparator14;
    private javax.swing.JToolBar.Separator jSeparator2;
    private javax.swing.JToolBar.Separator jSeparator3;
    private javax.swing.JToolBar.Separator jSeparator4;
    private javax.swing.JSeparator jSeparator5;
    private javax.swing.JSeparator jSeparator6;
    private javax.swing.JSeparator jSeparator7;
    private javax.swing.JSeparator jSeparator8;
    private javax.swing.JToolBar.Separator jSeparator9;
    private javax.swing.JPanel logLeftPanel;
    private javax.swing.JTextPane logPanel;
    private javax.swing.JPanel logRightPanel;
    private javax.swing.JScrollPane logScrollPane;
    private javax.swing.JPanel logTopPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JButton newButton;
    private javax.swing.JButton openButton;
    private javax.swing.JMenu operationsMenu;
    private javax.swing.JMenuItem optionsMenuItem;
    private javax.swing.JButton pasteButton;
    private javax.swing.JMenuItem pasteMenuItem;
    private javax.swing.JButton printButton;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JMenu protocolMenu;
    private javax.swing.JToggleButton recordButton;
    private javax.swing.JMenuItem recordMenuItem;
    private javax.swing.ButtonGroup recplaystopButtonGroup;
    private javax.swing.JButton redoButton;
    private javax.swing.JMenuItem redoMenuItem;
    private javax.swing.JButton saveButton;
    private javax.swing.JButton selectButton;
    private javax.swing.ButtonGroup selectedToolButtonGroup;
    private javax.swing.JToggleButton selectionToolButton;
    private javax.swing.JMenuItem selectionToolMenuItem;
    private javax.swing.JButton sequenceButton;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JToggleButton stopButton;
    private javax.swing.JMenuItem stopMenuItem;
    private javax.swing.JMenu toolsMenu;
    private javax.swing.JPanel topPanel;
    private javax.swing.JButton undoButton;
    private javax.swing.JMenuItem undoMenuItem;
    private javax.swing.JSplitPane verticalSplitPane;
    private javax.swing.JMenu viewMenu;
    private javax.swing.JComboBox zoomLevelCombobox;
    private javax.swing.JToggleButton zoomToolButton;
    private javax.swing.JMenuItem zoomToolMenuItem;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;
    private javax.swing.JMenu viewInMainWindowMenu; // lists those 
    
    /*
     * Implements the PriorEditorClient interface
     */
    @Override
    public Data promptValue(final Data item, final String message) throws ExecutionError {
        final Data[] buffer=new Data[1];
        buffer[0]=item.clone();
        final Prompt prompt;
             if (item instanceof SequenceCollection) prompt=new Prompt_SequenceCollection(gui, message, (SequenceCollection)buffer[0]);
        else if (item instanceof NumericVariable) prompt=new Prompt_NumericVariable(gui, message, (NumericVariable)buffer[0]);
        else if (item instanceof NumericMapVariable) prompt=new Prompt_NumericMapVariable(gui, message, (NumericMapVariable)buffer[0]);
        else if (item instanceof BackgroundModel) prompt=new Prompt_BackgroundModel(gui, message, (BackgroundModel)buffer[0]);
        else if (item instanceof StringVariable) prompt=new Prompt_StringVariable(gui, message, (StringVariable)buffer[0]);
        else if (item instanceof StringCollection) prompt=new Prompt_StringCollection(gui, message, (StringCollection)buffer[0]);
        else if (item instanceof Motif) prompt=new Prompt_Motif(gui, message, (Motif)buffer[0],visualizationSettings);
        else if (item instanceof MotifCollection) prompt=new Prompt_MotifCollection(gui, message, (MotifCollection)buffer[0]);
        else throw new ExecutionError("Cannot prompt for value for data of type "+item.getTypeDescription());

        Runnable runner = new Runnable() {
            public void run() {
                prompt.setLocation(getFrame().getWidth()/2-prompt.getWidth()/2, getFrame().getHeight()/2-prompt.getHeight()/2);
                prompt.disableNameEdit();
                prompt.setVisible(true);
                if (prompt.isOKPressed()) {
                  buffer[0]=prompt.getData();
                }  
                prompt.dispose();
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else try {
            SwingUtilities.invokeAndWait(runner);
        } catch (Exception ie) {}
        return buffer[0];
    }
    
    
    
    

    // ------------------------------------- PRIVATE CLASSES ---------------------------------------
    
    /**
     * Renderer for values in the Zoom Combobox in the toolbar
     */
    private class ZoomLevelComboboxCellRenderer extends JLabel implements ListCellRenderer {

        public ZoomLevelComboboxCellRenderer() {
            setBorder(BorderFactory.createEmptyBorder(1, 4, 1, 4));
        }
        
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            setOpaque(true);
            if (isSelected) {
                setForeground(list.getSelectionForeground());
                setBackground(list.getSelectionBackground());
            }
            else {
                setForeground(list.getForeground());
                setBackground(java.awt.Color.WHITE);
            }
                     
            if (value instanceof Double) {
                int val=(int)(((Double)value).doubleValue());
                setText(""+val+" %");
                setHorizontalAlignment(JLabel.RIGHT);
            } else {
                setText(value.toString());
                setHorizontalAlignment(JLabel.LEFT);
            }                           
            return this;
        } // end getListCellRendererComponent     
    }
    
    
    /**
     * ActionListener the Zoom Combobox in the toolbar
     */
    private class ZoomLevelComboboxActionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object selection=zoomLevelCombobox.getSelectedItem();
            VisualizationSettings settings=getVisualizationSettings();
            if (selection instanceof Double) {
                 settings.setGlobalZoomLevel((Double)selection);
                 visualizationPanel.zoomToLevelOnAllSequences((Double)selection);
            } else if (selection.toString().equals("To Fit")) {
                visualizationPanel.zoomToFitOnAllSequences();
            } else if (selection.toString().equals("Custom...")) {
              String newZoomString=(String)JOptionPane.showInputDialog(settings.getGUI().getFrame(),"Enter new zoom level","Zoom to custom level",JOptionPane.PLAIN_MESSAGE,null,null,100.0);
              try {
                 double zoomlevel=Double.parseDouble(newZoomString);
                 if (zoomlevel>getVisualizationSettings().getSequenceWindowSize()*100) zoomlevel=settings.getSequenceWindowSize()*100;
                 settings.setGlobalZoomLevel(zoomlevel);
                 visualizationPanel.zoomToLevelOnAllSequences(zoomlevel);
              } catch (Exception ex) {} 
            }
        }     
    }
    
    /**
     * Updates tooltips and enabled status of toolbar undo/redo buttons when edit events happen
     */    
    private class UndoRedoButtonUpdater implements UndoableEditListener {
        public void undoableEditHappened(UndoableEditEvent e) {
            updateUndoRedoStates();     
        }      
    }
    
    /**
     * Listens to Document updates and sets undo/redo action states
     */      
   private class UndoRedoDocumentListener implements DocumentListener {
        public void changedUpdate(DocumentEvent e) {updateUndoRedoStates();}
        public void insertUpdate(DocumentEvent e)  {updateUndoRedoStates();}
        public void removeUpdate(DocumentEvent e)  {updateUndoRedoStates();}       
    }

    /**
     * Listens to changes in the list of currently open Protocols
     */      
   private class ProtocolManagerListener implements ListDataListener {
        ProtocolListMenuListener listener;
        int lastSize=1;

        public ProtocolManagerListener() {
            listener = new ProtocolListMenuListener();
        }       
        public void contentsChanged(ListDataEvent e) {populateProtocolList();}
        public void intervalAdded(ListDataEvent e) {populateProtocolList();}
        public void intervalRemoved(ListDataEvent e) {populateProtocolList();}

        public void populateProtocolList() {
            ProtocolManager manager=protocolEditor.getProtocolManager();
            changeProtocolSubmenu.removeAll();
            //System.err.println("protocolListChanges: "+manager.getSize()+" protocols");
            if (manager.getSize()==0) { // no open protocol scripts. Disable play/rec-buttons
                changeProtocolSubmenu.setEnabled(false);
                setRecordEnabled(false);
                setExecuteEnabled(false);
                setStopEnabled(false);                                          
            } else { // protocol scripts are present
               if (lastSize==0) { // going from zero to one protocol scripts in list. Enable play/rec-buttons
                   setRecordEnabled(true);
                   setExecuteEnabled(true);
                   setStopEnabled(false);
               }
               changeProtocolSubmenu.setEnabled(true);
               Protocol currentProtocol=protocolEditor.getProtocol();
               String currentProtocolName="";
               if (currentProtocol!=null) currentProtocolName=currentProtocol.getName();
               for (int i=0;i<manager.getSize();i++) {
                   Protocol prot=(Protocol)manager.getElementAt(i);
                   JCheckBoxMenuItem item=new JCheckBoxMenuItem(prot.getName());
                   if (currentProtocolName.equals(prot.getName())) item.setSelected(true);
                   else item.setSelected(false);
                   item.addActionListener(listener);
                   changeProtocolSubmenu.add(item);
               }
            }
            lastSize=manager.getSize();
        }
      
        private class ProtocolListMenuListener implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                if (isRecording()) {// notify user that recording is on
                    JOptionPane.showMessageDialog(getFrame(), "You are currently in \"record mode\".\nSubsequent operations will be recorded in the new protocol", "Notification", JOptionPane.INFORMATION_MESSAGE);
                }
                //protocolEditor.changeProtocol(e.getActionCommand(),null,true); // queue undoable event
                protocolEditor.changeProtocol(e.getActionCommand());
            }            
        }
   } // end class ProtocolManagerListener

   

   
 
    @Action
    public void changeDisplaySettings() {
        java.util.ArrayList test=engine.getAllDataItemsOfType(FeatureDataset.class);
        if (test.size()==0) {
            JOptionPane.showMessageDialog(getFrame(), "You can not change display settings for datasets, since no datasets have been loaded", "Error",JOptionPane.ERROR_MESSAGE,null);
            return;
        }
        DatasetDisplaySettingsDialog dialog=new DatasetDisplaySettingsDialog(getFrame(), null, getVisualizationSettings());
        int height=dialog.getHeight();
        int width=dialog.getWidth();
        java.awt.Dimension size=getFrame().getSize();
        dialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));   
        dialog.setVisible(true);
        dialog.dispose();
    }

    
    @Action
    public void showOptionsDialog() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        PreferencesDialog dialog=new PreferencesDialog(this);
        int height=dialog.getHeight();
        int width=dialog.getWidth();
        java.awt.Dimension size=getFrame().getSize();
        dialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));      
        dialog.setVisible(true);
        dialog.dispose();
        getFrame().setCursor(Cursor.getDefaultCursor());
    }
    
    /** Returns a unique generic data item name that can be used as default name in dialogs etc.
     * @param type The class of the new data item. The class will dictate the returned name, however
     * it can also be NULL, in which case a generic type name will be returned
     * @param datatypetable (can be null) The method will return a name that is not already in use by other registered data items
     * if a DataTypeTable is provided the method will also avoid names present in the this table
     */
   
    
    /** Returns a filter that screens regions and selects which should be displayed and not */
    public RegionVisualizationFilter getRegionVisualizationFilter() {
        return regionVisualizationFilter;
    }
    
     /** Sets a RegionVisualizationFilter that screens regions and selects which should be displayed and not
      *  Use value NULL to disable filtering and display all regions
      */
    public void  setRegionVisualizationFilter(RegionVisualizationFilter filter) {
        regionVisualizationFilter=filter;
    }
    
    
     /** Returns a suggested unique name to be used for a new data object of the given type */
    public String getGenericDataitemName(Class type, DataTypeTable datatypetable) {
        int counter=1;
        String prefix="Data";
        //     if (FeatureDataset.class.isAssignableFrom(type)) {prefix="Feature";}
             if (type==DNASequenceDataset.class) {prefix="DNA";}
        else if (type==NumericDataset.class) {prefix="NumericDataset";}
        else if (type==RegionDataset.class) {prefix="RegionDataset";}
        else if (type==Sequence.class) {prefix="Sequence";}
        else if (type==SequenceCollection.class) {prefix="SequenceCollection";}
        else if (type==Motif.class) {prefix="MM";}
        else if (type==MotifCollection.class) {prefix="MotifCollection";}
        else if (type==StringVariable.class) {prefix="String";}
        else if (type==NumericVariable.class) {prefix="Value";}
        else if (type==NumericMapVariable.class) {prefix="MapVariable";}
        else if (type==StringCollection.class) {prefix="Strings";}
        else if (type==BackgroundModel.class) {prefix="Background";}
        else if (type==OutputData.class) {prefix="Output";}
        if (type!=null) {
            counter=engine.countDataItemsOfType(type);
            if (type!=SequenceCollection.class) counter++; // this will start the SequenceCollection counter at 1 even thought there is already a (default) SequenceCollection registered
        }
        String name;
        if (type==Motif.class) name=prefix+motifcounterformat.format(counter);
        else name=prefix+counter;
        while (engine.getDataItem(name)!=null || (datatypetable!=null && datatypetable.contains(name)) || reservedDataNames.contains(name)) {
            counter++; // adjust counter if name is taken already
            if (type==Motif.class) name=prefix+motifcounterformat.format(counter);
            else name=prefix+counter;
        }
        return name;
    }

    /** Returns a list of suggested unique names to be used for a set of new data objects of the given types
     *  The size of the list returned corresponds to the size of the type-array
     */
    public String[] getGenericDataitemNames(Class[] types, DataTypeTable datatypetable) {
        if (types==null || types.length==0) return new String[0];
        String[] resultnames=new String[types.length];
        for (int index=0;index<types.length;index++) {
            Class type=types[index];
            int counter=1;
            String prefix="Data";
            //     if (FeatureDataset.class.isAssignableFrom(type)) {prefix="Feature";}
                 if (type==DNASequenceDataset.class) {prefix="DNA";}
            else if (type==NumericDataset.class) {prefix="NumericDataset";}
            else if (type==RegionDataset.class) {prefix="RegionDataset";}
            else if (type==Sequence.class) {prefix="Sequence";}
            else if (type==SequenceCollection.class) {prefix="SequenceCollection";}
            else if (type==Motif.class) {prefix="MM";}
            else if (type==MotifCollection.class) {prefix="MotifCollection";}
            else if (type==StringVariable.class) {prefix="String";}
            else if (type==NumericVariable.class) {prefix="Value";}
            else if (type==NumericMapVariable.class) {prefix="MapVariable";}
            else if (type==StringCollection.class) {prefix="Strings";}
            else if (type==BackgroundModel.class) {prefix="Background";}
            else if (type==OutputData.class) {prefix="Output";}
            if (type!=null) {
                counter=engine.countDataItemsOfType(type);
                if (type!=SequenceCollection.class) counter++; // this will start the SequenceCollection counter at 1 even thought there is already a (default) SequenceCollection registered
            }
            String name;
            if (type==Motif.class) name=prefix+motifcounterformat.format(counter);
            else name=prefix+counter;
            while (engine.getDataItem(name)!=null || arrayContainsString(resultnames,name) || (datatypetable!=null && datatypetable.contains(name)) || reservedDataNames.contains(name)) {
                counter++; // adjust counter if name is taken already
                if (type==Motif.class) name=prefix+motifcounterformat.format(counter);
                else name=prefix+counter;
            }
            resultnames[index]=name;
        }
        return resultnames;
    }
    private boolean arrayContainsString(String[] list, String target) {
        for (String element:list) {
            if (element!=null && element.equals(target)) return true;
        }
        return false;
    }

    /** Returns a suggested unique name to be used for an new data object
     *  A prefix can be suggested
     */
    public String getGenericDataitemName(String prefix, DataTypeTable datatypetable) {
        int counter=1;
        if (prefix==null || prefix.isEmpty()) prefix="Data";
        String name=prefix+counter;
        while (engine.getDataItem(name)!=null || (datatypetable!=null && datatypetable.contains(name)) || reservedDataNames.contains(name)) {
            counter++; // adjust counter if name is taken already
            name=prefix+counter;
        }
        return name;
    }

    /** 
     * This method can be used to reserve names that might be used in the future by tasks that have been scheduled already
     * If a name is reserved it will not be suggested as a "generic name" by the getGenericDataitemName() methods
     * @param name A single name or a comma-separated list of names to be reserved
     */
    public void reserveDataName(String name) {
        if (name==null || name.isEmpty()) return;
        String[] reserve=name.split("\\s*,\\s*");
        for (String rname:reserve) reservedDataNames.add(rname);
       
    }

    /**
     * This method is used to release names that have been marked as 'reserved' with the reserveDataName() method
      * @param name A single name or a comma-separated list of names to be reserved
     */
    public void releaseReservedDataName(String name) {
        if (name==null || name.isEmpty()) return;
        String[] reserve=name.split("\\s*,\\s*");
        for (String rname:reserve) reservedDataNames.remove(rname);    
    }



  /**
   * This class listens to PropertyChanges and updates progress-bar and status messages
   * in the status field accordingly. It is used as a replacement for TaskMonitor which
   * didnt't quite have the functionality I was looking for
   * The class uses references to widgets defined as fields in the outer class
   */
  private class ProgressMonitor implements PropertyChangeListener {

      @Override
      public void propertyChange(java.beans.PropertyChangeEvent evt) {
          ProgressRunner runner=new ProgressRunner(evt);
          if (SwingUtilities.isEventDispatchThread()) runner.run();
          else SwingUtilities.invokeLater(runner);
      }
      
      private class ProgressRunner implements Runnable {
          private PropertyChangeEvent evt=null;
          public ProgressRunner(PropertyChangeEvent event) {
              evt=event;
          }
      
          public void run() {
                String propertyName = evt.getPropertyName();
                if (propertyName.equals(ExecutableTask.STATUS)) {
                   String newstatus=(String)evt.getNewValue();
                   if (newstatus.equals(ExecutableTask.RUNNING)) {
                       if (!busyIconTimer.isRunning()) {
                          statusAnimationLabel.setIcon(busyIcons[0]);
                          busyIconIndex = 0;
                          busyIconTimer.start();
                       }
                       progressBar.setVisible(true);
                  } else if (newstatus.equals(ExecutableTask.WAITING) || newstatus.equals(ExecutableTask.PENDING)) {
                       progressBar.setVisible(true);
                       progressBar.setIndeterminate(true);
                  } else {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                    progressBar.setString("");
                  } 
                } else if (propertyName.equals(ExecutableTask.STATUS_MESSAGE)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if (propertyName.equals(ExecutableTask.PROGRESS)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                    //int thisNumber=scheduler.getCurrentTaskQueueNumber();
                    //int totalNumber=scheduler.getLastQueueNumber();
                    int pending=scheduler.pendingTasks();
                    if (pending>1) progressBar.setString("+"+pending+" tasks pending");
                    else if (pending==1) progressBar.setString("+"+pending+" task pending");
                    else progressBar.setString("");
                }  else if (propertyName.equals("HIDE")) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                    progressBar.setString("");                
                }       
        }   
      } // of class ProgressRunner
  }  // end of class ProgressMonitor

   
    /**
     * Attempts to abort the execution of the currently running protocol script
     * @return true if the execution was aborted or false it is still running
     */
    public boolean abortExecution() {
        int n=-1;
        synchronized (scheduler.getExecutionLock()) { // this will temporarily suspend task execution as soon as possible
           Object[] options={"OK","Cancel"};
           n=JOptionPane.showOptionDialog(getFrame(), "Abort execution?", "Abort execution", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null,options,options[1]);
        }
        // I will release the execution-lock first before calling abort (because abort() can remove the execution lock from the task and cause deadlock)
        if (n==0) {scheduler.abort(); return true;}
        else return false;
    }

    @Action
    public void showSequenceInputDialog() {
        if (engine.hasDataItemsOfType(FeatureDataset.class)) {
            JOptionPane.showMessageDialog(getFrame(), "New sequences can not be added after feature data has been loaded", "Add Sequences", JOptionPane.INFORMATION_MESSAGE);
            return;
        }
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        SequenceInputDialog dialog=new SequenceInputDialog(this,engine.getGeneIDResolver());
        int height=dialog.getHeight();
        int width=dialog.getWidth();
        java.awt.Dimension size=getFrame().getSize();
        dialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));  
        if (executeButton.isSelected()) dialog.disableAutoFetchDNA();
        dialog.setVisible(true);    
        dialog.dispose();
        getFrame().setCursor(Cursor.getDefaultCursor()); 
    }
    
    @Action
    public void showDatatrackDialog() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        DataTrackDialog dialog=new DataTrackDialog(this);
        int height=dialog.getHeight();
        int width=dialog.getWidth();
        java.awt.Dimension size=getFrame().getSize();
        dialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));      
        dialog.setVisible(true);    
        dialog.dispose();
        getFrame().setCursor(Cursor.getDefaultCursor());  
    }
    
    @Action
    public void showLoadFromFileDialog() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        LoadDataFromFileDialog dialog=new LoadDataFromFileDialog(this);
        int height=dialog.getHeight();
        int width=dialog.getWidth();
        java.awt.Dimension size=getFrame().getSize();
        dialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));      
        dialog.setVisible(true);    
        dialog.dispose();
        getFrame().setCursor(Cursor.getDefaultCursor());  
    }
    
    @Action
    public void showAddSequencesFromFileDialog() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        AddSequencesFromFileDialog dialog=new AddSequencesFromFileDialog(this);
        int height=dialog.getHeight();
        int width=dialog.getWidth();
        java.awt.Dimension size=getFrame().getSize();
        dialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));      
        dialog.setVisible(true);    
        dialog.dispose();
        getFrame().setCursor(Cursor.getDefaultCursor());  
    }
    
    

    @Action
    public void newProtocol() {
        if (isRecording()) {// notify user that recording is on
             JOptionPane.showMessageDialog(getFrame(), "You are currently in \"record mode\".\nSubsequent operations will be recorded in the new protocol", "New Protocol", JOptionPane.INFORMATION_MESSAGE);
        }
        protocolEditor.newProtocol(null);
        mainWindow.showProtocolEditor();
    }
    
    
    @Action
    public void openProtocol() {
        if (isRecording()) {// notify user that recording is on
             JOptionPane.showMessageDialog(getFrame(), "You are currently in \"record mode\".\nSubsequent operations will be recorded in the new protocol", "Open Protocol", JOptionPane.INFORMATION_MESSAGE);
        }
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        protocolEditor.openProtocolFile();
        getFrame().setCursor(Cursor.getDefaultCursor());
    } 

    
    @Action(enabledProperty = "saveEnabled")
    public void saveFile() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        mainWindow.saveCurrentTabToFile(false); // checks which tab is currently used and saves the contents to file
        getFrame().setCursor(Cursor.getDefaultCursor());
    }

    
    @Action
    public void saveAllActionMethod() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        mainWindow.saveAllTabs(); // 
        getFrame().setCursor(Cursor.getDefaultCursor());
    }

    @Action(enabledProperty = "saveAsEnabled")
    public void saveAsActionMethod() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        mainWindow.saveCurrentTabToFile(true); // checks which tab is currently used and saves the contents to file
        getFrame().setCursor(Cursor.getDefaultCursor());
    }
    
    
    @Action(enabledProperty = "closeEnabled")
    public void closefileActionMethod() {
        String currentTab=mainWindow.getSelectedTabName();
        if (currentTab.equals("Visualization") || currentTab.equals("Protocol")) {
            if (isRecording()) stopButton.doClick(); // stop recording when closing protocol
            protocolEditor.closeProtocolFile();
        }
        else mainWindow.closeCurrentTab();
    }


    @Action
    public void saveSessionActionMethod() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        File file=null;
        final JFileChooser fc = new JFileChooser(getLastUsedDirectory());
        fc.setDialogTitle("Save Session"); 
        fc.addChoosableFileFilter(new FileNameExtensionFilter("PriorsEditor Session (*.pes)", "pes"));
        int returnValue=fc.showSaveDialog(protocolEditor);
        if (returnValue==JFileChooser.APPROVE_OPTION) {
            file=fc.getSelectedFile();            
        } else {
            getFrame().setCursor(Cursor.getDefaultCursor());
            return;
        }
        if (!file.getAbsolutePath().endsWith(".pes")) file=new File(file.getAbsolutePath()+".pes");
        if (file.exists()) {
            int choice=JOptionPane.showConfirmDialog(protocolEditor, "Overwrite existing file \""+file.getName()+"\" ?","Save Session",JOptionPane.OK_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
            if (choice!=JOptionPane.OK_OPTION) {
                getFrame().setCursor(Cursor.getDefaultCursor());
                return;
            }
        }
        saveSessionInBackground(file);
        setLastUsedDirectory(file.getParentFile());
    }
    
  /** A convenience method to save images off the EDT */
  private void saveSessionInBackground(final File file) {
        SwingWorker worker=new SwingWorker<Boolean, Void>() {
            Exception ex=null;
            @Override 
            public Boolean doInBackground() {
                ObjectOutputStream outputStream = null;
                //SerializedSession session=new SerializedSession();
                try {
                    outputStream = new ObjectOutputStream(new FileOutputStream(file));
                    outputStream.writeObject(mainWindow.getTabNames());
                    outputStream.writeObject(engine.getAllDataItemsOfType(Data.class));
                    outputStream.writeObject(getVisualizationSettings());
                    outputStream.writeObject(protocolEditor.getProtocolManager().getSerializedProtocolManager());
                    outputStream.writeObject(mainWindow.getSelectedTabName());
                    outputStream.close();                
                } catch (Exception e) {
                    ex=e;
                    return Boolean.FALSE;
                } finally {
                    try {if (outputStream!=null) outputStream.close();} catch (Exception x) {}
                }
                return Boolean.TRUE;
            } // end doInBackground
            @Override
            public void done() { // this method is invoked on the EDT!
                if (ex!=null) {
                     JOptionPane.showMessageDialog(getFrame(), ex.getClass().toString()+":\n"+ex.getMessage(),"File error" ,JOptionPane.ERROR_MESSAGE);
                     logMessage("Unable to save session to \""+file.getAbsolutePath()+"\"  ("+ex.getClass().toString()+":"+ex.getMessage()+")");
                } else { // save went OK                    
                     logMessage("Saved session to \""+file.getAbsolutePath()+"\"");
                     registerRecentSession(file.getAbsolutePath());
                }
                getFrame().setCursor(Cursor.getDefaultCursor());
            }
        }; // end of SwingWorker class
        worker.execute();              
    }    
    
    
   
    
    @Action
    public void restoreSessionActionMethod() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        File file=null;
        final JFileChooser fc = new JFileChooser(getLastUsedDirectory());
        fc.setDialogTitle("Restore Session"); 
        fc.addChoosableFileFilter(new FileNameExtensionFilter("PriorsEditor Session (*.pes)", "pes"));
        int returnValue=fc.showOpenDialog(protocolEditor);
        if (returnValue==JFileChooser.APPROVE_OPTION) {
            file=fc.getSelectedFile();            
        } else {
            getFrame().setCursor(Cursor.getDefaultCursor());
            return;
        }
        //if (!file.getAbsolutePath().endsWith(".pes")) file=new File(file.getAbsolutePath()+".pes");
        restoreSessionInBackground(file);
        setLastUsedDirectory(file.getParentFile());
    }
      
    @SuppressWarnings("unchecked")
    private void restoreSessionInBackground(final File file) {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        statusMessage("Restoring session...");
         SwingWorker worker=new SwingWorker<Boolean, Void>() {
            ArrayList<Data> datalist=null;
            VisualizationSettings settings=null;
            SerializedProtocolManager protManager=null;
            String[] tabNames;
            String selectedTabName=null;
            HashMap<String,Integer> VPstart=new HashMap<String,Integer>();
            Exception ex=null;
            @Override 
            public Boolean doInBackground() {
                ObjectInputStream inputStream = null;        
                try {
                    inputStream = new ObjectInputStream(new FileInputStream(file));
                    tabNames=(String[])inputStream.readObject();
                    datalist=(ArrayList<Data>)inputStream.readObject();
                    settings=(VisualizationSettings)inputStream.readObject();
                    protManager=(SerializedProtocolManager)inputStream.readObject();
                    selectedTabName=(String)inputStream.readObject();
                    inputStream.close();                  
                } catch (Exception e) {
                    ex=e;
                    return Boolean.FALSE;
                } finally {
                    try {if (inputStream!=null) inputStream.close();} catch (Exception x) {}
                }
                return Boolean.TRUE;
            } // end doInBackground
            @Override
            public void done() { // this method is invoked on the EDT!
                if (ex!=null) {
                     if (ex instanceof InvalidClassException) {
                        JOptionPane.showMessageDialog(getFrame(), "An error occurred during session restore:\nThe saved session is not compatible with\nthe current version of PriorsEditor","Restore Session Error" ,JOptionPane.ERROR_MESSAGE);
                        removeSessionFromRecentMenu(file.getAbsolutePath());      
                     }
                     else {
                         JOptionPane.showMessageDialog(getFrame(), ex.getClass().toString()+":\n"+ex.getMessage(),"Restore Session Error" ,JOptionPane.ERROR_MESSAGE);
                         logMessage("Unable to restore session: "+ex.getClass().toString()+"-"+ex.getMessage());                
                     }
                } else { // restore went OK  
                    visualizationSettings.enableVisualizationSettingsNotifications(false); // disable to avoid redundant updates                           
                    engine.clearAllData(); 
                    protocolEditor.getProtocolManager().closeAllWithoutUndo();
                    getVisualizationSettings().importSettings(settings);
                    PreferencesDialog.restoreVisualizationSettings(gui); // iverride with settings chosen in Preferences
                    for (Data element:datalist) { // restore all sequences and motifs first, since these are referred to by collections!
                      try {
                        if (element instanceof Sequence) {
                            VPstart.put(element.getName(), settings.getSequenceViewPortStart(element.getName())); // save VP-start to correctly restore it later...
                            engine.storeDataItem(element);
                        } else if (element instanceof Motif) {
                            engine.storeDataItem(element);
                        }
                      } catch (ExecutionError e) {JOptionPane.showMessageDialog(getFrame(), "An error occurred during session restore:\n"+e.getMessage(),"Restore Session Error" ,JOptionPane.ERROR_MESSAGE);}
                    }
                    visualizationSettings.enableVisualizationSettingsNotifications(true); // reenable
                    //visualizationSettings.notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.REORDERED, engine.getDefaultSequenceCollection());
                    for (String outputDataName:tabNames){
                        Data element=getDataByName(datalist,outputDataName);
                        if (element!=null) try {engine.storeDataItem(element);} catch (ExecutionError e) {JOptionPane.showMessageDialog(getFrame(), "An error occurred during session restore:\n"+e.getMessage(),"Restore Session Error" ,JOptionPane.ERROR_MESSAGE);}
                    }
                    for (Data element:datalist) {
                        if (!(element instanceof Sequence || element instanceof OutputData || element instanceof Motif)) try {engine.storeDataItem(element);} catch (ExecutionError e) {JOptionPane.showMessageDialog(getFrame(), "An error occurred during session restore:\n"+e.getMessage(),"Restore Session Error" ,JOptionPane.ERROR_MESSAGE);}
                    }  
                    visualizationPanel.sequencesReorderEvent(0,null);
                    mainWindow.invalidate();
                    mainWindow.repaint();
                    SequenceCollection col=engine.getDefaultSequenceCollection();
                    for (String name:col.getAllSequenceNames()) visualizationSettings.setSequenceViewPortStart(name, VPstart.get(name));
                    protocolEditor.getProtocolManager().importSettings(protManager);
                    undoManager.discardAllEdits(); // clears the undo queue
                    if (selectedTabName!=null) mainWindow.setSelectedTab(selectedTabName);
                    registerRecentSession(file.getAbsolutePath());            
                    recplaystopButtonGroup.clearSelection();
                    if (isRecording()) {
                        stopAction();
                    }
                    logMessage("Restored session from "+file.getAbsolutePath()); 
                }
                getFrame().setCursor(Cursor.getDefaultCursor());
            }
            private Data getDataByName(ArrayList<Data>list, String name) {
                for (Data element:list) {if (element.getName().equals(name)) return element;}
                return null;
            }
        }; // end of SwingWorker class
        worker.execute();    
    }    
    
    private void registerRecentSession(String filename) {
        if (recentsessions.contains(filename)) recentsessions.remove(filename);
        if (recentsessions.size()>=maxRecentSessions) recentsessions.remove(maxRecentSessions-1);
        recentsessions.add(0,filename);
        updateRecentSessionsMenu(false);
    }   
    
    private void removeSessionFromRecentMenu(String filename) {
        if (!recentsessions.contains(filename)) return;
        recentsessions.remove(filename);
        updateRecentSessionsMenu(false);
    }   
    
    
    
    @SuppressWarnings("unchecked")
    private void updateRecentSessionsMenu(boolean readFromDisc) {
        JMenu menu=fileRestoreRecentSessionSubmenu;
        menu.removeAll();       
        if (readFromDisc) {  
            LocalStorage localStorage=protocolEditor.getGUI().getApplication().getContext().getLocalStorage();
            try {
               Object list=localStorage.load(recentsessionsfilename);
               if (list instanceof ArrayList) recentsessions=(ArrayList<String>)list;
            } catch (Exception e) {}
        } 
        for(String pathname:recentsessions) {
            File file=new File(pathname);
            String shortname=file.getName();
            JMenuItem menuItem=new JMenuItem(shortname);
            menuItem.setActionCommand(pathname);
            menuItem.setToolTipText(pathname);
            menuItem.addActionListener(openrecentsessionmenulistener);
            menu.add(menuItem);
        }
        if (recentsessions.size()==0) menu.setEnabled(false);
        else menu.setEnabled(true);
    }    
    
private class OpenRecentSessionMenuListener implements ActionListener {
    @Override
    public void actionPerformed(ActionEvent e) {
        String fullpath=e.getActionCommand();
        File file=new File(fullpath);
        //String shortname=file.getName();
        restoreSessionInBackground(file);
    }        
} // end private class OpenRecentSessionMenuListener    
    
    
    
    @Action
    public void saveAsImageMethod() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        File file=null;
        final JFileChooser fc = new JFileChooser(getLastUsedDirectory());
        fc.setDialogTitle("Save visualization"); 
        FileNameExtensionFilter GIFfilter=new FileNameExtensionFilter("GIF image", "gif","GIF");
        FileNameExtensionFilter JPGfilter=new FileNameExtensionFilter("JPG image", "jpg","JPG");
        FileNameExtensionFilter PNGfilter=new FileNameExtensionFilter("PNG image", "png","PNG");
        fc.addChoosableFileFilter(GIFfilter);
        fc.addChoosableFileFilter(JPGfilter);
        fc.addChoosableFileFilter(PNGfilter);
        int returnValue=fc.showSaveDialog(protocolEditor);
        if (returnValue==JFileChooser.APPROVE_OPTION) {
            file=fc.getSelectedFile();            
        } else {
            getFrame().setCursor(Cursor.getDefaultCursor());
            return;
        }
        String suffix="";;
        String filename=file.getName();
             if (filename.endsWith(".png") || filename.endsWith(".PNG")) suffix="png";
        else if (filename.endsWith(".gif") || filename.endsWith(".GIF")) suffix="gif";
        else if (filename.endsWith(".jpg") || filename.endsWith(".JPG")) suffix="jpg";
        else if (fc.getFileFilter()==GIFfilter) {suffix="gif";file=new File(file.getPath()+".gif");}
        else if (fc.getFileFilter()==JPGfilter) {suffix="jpg";file=new File(file.getPath()+".jpg");}
        else if (fc.getFileFilter()==PNGfilter) {suffix="png";file=new File(file.getPath()+".png");}
        else {
            JOptionPane.showMessageDialog(getFrame(), "Unknown image format.\nPlease enter a filename that ends with '.png', '.gif', or '.jpg'", "Save Image", JOptionPane.WARNING_MESSAGE);
            getFrame().setCursor(Cursor.getDefaultCursor());
            return;
        }
        if (file.exists()) {
            int choice=JOptionPane.showConfirmDialog(protocolEditor, "Overwrite existing file \""+file.getName()+"\" ?","Save Image",JOptionPane.OK_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
            if (choice!=JOptionPane.OK_OPTION) {
                getFrame().setCursor(Cursor.getDefaultCursor());
                return;
            }
        }
        saveImageInBackground(file,suffix);
        setLastUsedDirectory(file.getParentFile());
    }
    
  /** A convenience method to save images off the EDT */
  private void saveImageInBackground(final File file, final String suffix) {
        //statusMessage("Saving image to \""+file.getName()+"\"");
        SwingWorker worker=new SwingWorker<Boolean, Void>() {
            Exception ex=null;
            @Override 
            public Boolean doInBackground() {
                JPanel panel=visualizationPanel.getMainVisualizationPanel();  
                int width=visualizationSettings.getSequenceLabelWidth()+visualizationSettings.getSequenceWindowSize()+2*VisualizationSettings.DATATRACK_BORDER_SIZE;
                Font trackLabelsFont=SequenceVisualizer.getTrackLabelsFont();
                int maxtracklabelwidth=0;
                ArrayList<FeatureDataset> trackorder=visualizationSettings.getDatatrackOrder();
                for (FeatureDataset track:trackorder) {                 
                   String trackname=track.getName();
                   if (!visualizationSettings.isVisible(trackname)) continue;
                   int tracklabelwidth=visualizationPanel.getFontMetrics(trackLabelsFont).stringWidth(trackname);
                   if (tracklabelwidth>maxtracklabelwidth) maxtracklabelwidth=tracklabelwidth;
                }
                width+=maxtracklabelwidth+16; // 16 is just a padding value
                int height=panel.getHeight();
                BufferedImage image=new BufferedImage(width,height, BufferedImage.TYPE_INT_RGB);
                Graphics2D g=image.createGraphics();
                g.setColor(java.awt.Color.WHITE);
                g.fillRect(0, 0, width, height);
                panel.print(g);
                g.dispose();
                try {
                    ImageIO.write(image, suffix, file);
                    return Boolean.TRUE;
                } catch (Exception e) {
                    ex=e;
                    return Boolean.FALSE;
                }       

            } // end doInBackground
            @Override
            public void done() { // this method is invoked on the EDT!
                if (ex!=null) {
                     JOptionPane.showMessageDialog(getFrame(), ex.getClass().toString()+":\n"+ex.getMessage(),"File error" ,JOptionPane.ERROR_MESSAGE);
                     logMessage("Unable to save to \""+file.getName()+"\"  ("+ex.getMessage()+")");
                } else { // save went OK                    
                     logMessage("Saved image to \""+file.getName()+"\"");
                }
                getFrame().setCursor(Cursor.getDefaultCursor());
            }
        }; // end of SwingWorker class
        worker.execute();              
    }
  
  
    /** Returns the directory last used to load or save files. This can be used to
     *  initialize new FileChoosers with the same location
     */
    public File getLastUsedDirectory() {
        return lastUsedDirectory;
    }
  
    /** Sets the directory last used to load or save files. This directory can be used to
     *  initialize new FileChoosers with the same location
     */
    public void setLastUsedDirectory(File dir) {
        lastUsedDirectory=dir;
    }
  
  
  
  
    
    @Override
    public void initializeClient(PriorsEditorEngine engine) {
        // void callback method
    }
    
    
    /** Returns a reference to the menu containing recent protocols*/
    public JMenu getFileOpenRecentProtocolsSubmenu() {
        return  fileOpenRecentSubmenu;
    }
    
    /** 
     * This class is registered as a listener that is notified when the user tries to exit the application
     * It can veto the exit by returning FALSE in the canExit method
     */
    private class ConfirmExit implements Application.ExitListener {
        @Override
        public boolean canExit(EventObject arg0) {
            if (mainWindow.isAnyDirty() && doPromptBeforeDiscard()) {
                int option=JOptionPane.showConfirmDialog(getFrame(),"Save changes before exit?","Exit PriorsEditor",JOptionPane.YES_NO_CANCEL_OPTION);
                if (option==JOptionPane.CANCEL_OPTION) return false;
                else if (option==JOptionPane.YES_OPTION) mainWindow.saveAllTabs();
            }
            return true;
        }
        @Override
        public void willExit(EventObject arg0) {}
        
    }

    /** The next two methods implement parts of the PriorsEditorClient inferface */
      final boolean[] answer=new boolean[]{false,false}; // just a quick cheat to get around "final" :-)
      @Override
      public boolean shouldRetry(final ExecutableTask t, final Exception e) {
            // answer[0]=true if should retry. If answer[0]==false then answer[1] should be set to true if rollback should be performed
            int linenumber=t.getLineNumber();
            final String linemsg=(linenumber==0)?"":" in line "+linenumber;
            final String msg="An error occurred during execution"+linemsg+":\n"+e.getMessage();
            Runnable runner = new Runnable() {
                public void run() {
                    Object[] options={"Retry","Abort","Abort and Rollback"};
                    int n=JOptionPane.showOptionDialog(getFrame(), msg, "Error", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE, null,options,options[0]);
                    if (n==0) {answer[0]=true;answer[1]=false;}
                    else if (n==1) {answer[0]=false;answer[1]=false;}
                    else if (n==2) {answer[0]=false;answer[1]=true;}
                }
            };
            if (!(e instanceof ExecutionError)) { // ExecutionErrors are caused by the user. Other errors are caused by sloppy programming on my part
              StackTraceElement[] stackTrace=e.getStackTrace();
              System.err.println("An error occurred: "+e.getClass().toString()+" => "+e.getMessage());
              for (StackTraceElement stack:stackTrace) {
                System.err.println("   "+stack.getClassName()+"."+stack.getMethodName()+"()   line "+stack.getLineNumber());
              }
            }
            if (SwingUtilities.isEventDispatchThread()) runner.run();
            else try {SwingUtilities.invokeAndWait(runner);} catch (Exception ex) {return false;}
            return answer[0];
        }
        @Override
        public boolean shouldRollback() {
            return answer[1];
        }

// ---------------- Implementation of the DataListener interface -------------        
    @Override
    public void dataAdded(Data data) {
        releaseReservedDataName(data.getName());
        if (data instanceof FeatureDataset) {
          updateActionButtons(Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this));                           
        }  
    }
    @Override
    public void dataAddedToSet(Data parentDataset, Data child) {
        releaseReservedDataName(child.getName());
        if (parentDataset==engine.getDefaultSequenceCollection()) {
          updateActionButtons(Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this));             
        }    
    }
    @Override
    public void dataRemoved(Data data) {
        if (data instanceof FeatureDataset) {
          updateActionButtons(Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this));                        
        }      
    }
    @Override
    public void dataRemovedFromSet(Data parentDataset, Data child) {
        if (parentDataset==engine.getDefaultSequenceCollection()) {
          updateActionButtons(Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this));                        
        }    
    }
    @Override
    public void dataUpdate(Data oldvalue, Data newvalue) { }
    @Override
    public void dataUpdated(Data data) { }
        
    private void updateActionButtons(final ActionMap actionMap) {
        Runnable runner = new Runnable() {
            public void run() {
                if (engine.hasDataItemsOfType(FeatureDataset.class)) {
                    actionMap.get("showSequenceInputDialog").setEnabled(false);
                    actionMap.get("showAddSequencesFromFileDialog").setEnabled(false);
                }
                else { // no feature data present
                    actionMap.get("showSequenceInputDialog").setEnabled(true);
                    boolean empty=engine.getDefaultSequenceCollection().isEmpty();
                    actionMap.get("showAddSequencesFromFileDialog").setEnabled(empty);
                } 
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);                   
    }
    
    
    @Action
    public void showMotifBrowser() {
        if (motifbrowser!=null && motifbrowser.isVisible()) return;
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        motifbrowser=new MotifBrowser(this);
        motifbrowser.setLocation(getFrame().getWidth()/2-motifbrowser.getWidth()/2, getFrame().getHeight()/2-motifbrowser.getHeight()/2);
        motifbrowser.setVisible(true);
        getFrame().setCursor(Cursor.getDefaultCursor());
    }


    @Action
    public void showMotifScoreFilterDialog() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        final MotifScoreFilterDialog scoredialog=new MotifScoreFilterDialog(this);
        scoredialog.setLocation(getFrame().getWidth()/2-scoredialog.getWidth()/2, getFrame().getHeight()/2-scoredialog.getHeight()/2);
        scoredialog.setVisible(true); 
        getFrame().setCursor(Cursor.getDefaultCursor());           
    }


        
    
    
    @Action
    public void showConfigureDatatracksDialog() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        DataTrackConfigurationDialog dialog=new DataTrackConfigurationDialog(this);
        int height=dialog.getHeight();
        int width=dialog.getWidth();
        java.awt.Dimension size=getFrame().getSize();
        dialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));      
        dialog.setVisible(true);
        dialog.dispose();
        getFrame().setCursor(Cursor.getDefaultCursor());
    }

    @Action
    public void clearAllDataActionMethod() {
        if (engine.isDataStorageEmpty()) return;
        if (!scheduler.isIdle()) {logMessage("Unable to delete data while executing tasks");return;}
        if (doPromptBeforeDiscard()) {
           int option=JOptionPane.showConfirmDialog(getFrame(), "Do you really want to clear all data?", "Priors Editor", JOptionPane.OK_CANCEL_OPTION);
           if (option==JOptionPane.CANCEL_OPTION) return;
        }
        visualizationSettings.enableVisualizationSettingsNotifications(false); // turn off to avoid redundant updates
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        if (motifbrowser!=null) {
            motifbrowser.setVisible(false);
            motifbrowser.dispose();
            motifbrowser=null;
        }
        UndoMonitor undomonitor=undoManager.getUndoMonitor("Clear All Data");
        undomonitor.register();
        undomonitor.dataRemoved(engine.getDefaultSequenceCollection()); // we must tell the undomonitor explicitly to store the default sequence collection since it is not actually removed (only cleared)
        engine.clearAllData();
        visualizationSettings.clearSelectionWindows(); // I am not sure if this activity is registered in the monitor
        undomonitor.deregister(true);
        undoManager.addEdit(undomonitor);
        visualizationSettings.enableVisualizationSettingsNotifications(true); // reenable   
        visualizationSettings.notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.REORDERED, engine.getDefaultSequenceCollection());
        getFrame().setCursor(Cursor.getDefaultCursor());  
    }
    
   @Action
    public void clearAllMotifsActionMethod() {
        if (engine.isDataStorageEmpty()) return;
        if (!scheduler.isIdle()) {logMessage("Unable to delete data while executing tasks");return;}
        if (doPromptBeforeDiscard()) {
           int option=JOptionPane.showConfirmDialog(getFrame(), "Do you really want to clear all motifs?", "Priors Editor", JOptionPane.OK_CANCEL_OPTION);
           if (option==JOptionPane.CANCEL_OPTION) return;
        }
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        if (motifbrowser!=null) {
            motifbrowser.setVisible(false);
            motifbrowser.dispose();
            motifbrowser=null;
        }
        UndoMonitor undomonitor=undoManager.getUndoMonitor("Clear All Data");
        undomonitor.register();
        engine.clearAllMotifData();
        undomonitor.deregister(true);
        undoManager.addEdit(undomonitor);
        getFrame().setCursor(Cursor.getDefaultCursor());  
    }

    @Action
    public void clearAllSequencesActionMethod() {
        if (engine.getDefaultSequenceCollection().isEmpty()) return;
        if (!scheduler.isIdle()) {logMessage("Unable to delete data while executing tasks");return;}
        if (doPromptBeforeDiscard()) {
           int option=JOptionPane.showConfirmDialog(getFrame(), "Do you really want to clear all sequences and feature data?", "Priors Editor", JOptionPane.OK_CANCEL_OPTION);
           if (option==JOptionPane.CANCEL_OPTION) return;
        }
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        visualizationSettings.enableVisualizationSettingsNotifications(false); // turn off to avoid redundant updates
        UndoMonitor undomonitor=undoManager.getUndoMonitor("Clear Sequences");
        undomonitor.register();
        undomonitor.dataRemoved(engine.getDefaultSequenceCollection()); // we must tell the undomonitor explicitly to store the default sequence collection since it is not actually removed (only cleared)
        engine.clearAllSequences();
        visualizationSettings.clearSelectionWindows(); // I am not sure if this activity is registered in the monitor
        undomonitor.deregister(true);
        undoManager.addEdit(undomonitor);
        visualizationSettings.enableVisualizationSettingsNotifications(true); // reenable
        visualizationSettings.notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.REORDERED, engine.getDefaultSequenceCollection());
        getFrame().setCursor(Cursor.getDefaultCursor());          
    }

    /** This is a callback method invoked by GuiScheduler to report that Execution of a task or protocol script has started 
     *  Note that this method is NOT called on the EDT! 
     */
    public void reportExecutionStarted() {
        final javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this);
        // the actionMap can not be retrieved within the runner below, because then the GUI will freeze :-|
        Runnable runner=new Runnable() {
                @Override
                public void run() {        
                    actionMap.get("clearAllDataActionMethod").setEnabled(false);
                    actionMap.get("clearAllMotifsActionMethod").setEnabled(false);
                    actionMap.get("clearAllSequencesActionMethod").setEnabled(false);
                    actionMap.get("restoreSessionActionMethod").setEnabled(false);                
                    actionMap.get("showSequenceInputDialog").setEnabled(false);                
                    fileRestoreRecentSessionSubmenu.setEnabled(false);      
                }
        };
        try {
            if (SwingUtilities.isEventDispatchThread()) runner.run();
            else  SwingUtilities.invokeLater(runner);
        } catch (Exception ie) {}         
    }

    /**
     * This is a callback method invoked by GuiScheduler to report that Execution of a task or protocol script has ended
     */
    public void reportExecutionEnded() {
        reservedDataNames.clear();
        final javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(PriorsEditorGUI.class, this);
        // the actionMap can not be retrieved within the runner below, because then the GUI will freeze :-|
        Runnable runner=new Runnable() {
                @Override
                public void run() {        
                    actionMap.get("clearAllDataActionMethod").setEnabled(true);
                    actionMap.get("clearAllMotifsActionMethod").setEnabled(true);
                    actionMap.get("clearAllSequencesActionMethod").setEnabled(true);        
                    actionMap.get("restoreSessionActionMethod").setEnabled(true);  
                    if (!engine.hasDataItemsOfType(FeatureDataset.class)) {
                       actionMap.get("showSequenceInputDialog").setEnabled(true);   
                    }
                    if (recentsessions.size()==0) fileRestoreRecentSessionSubmenu.setEnabled(false);
                    else fileRestoreRecentSessionSubmenu.setEnabled(true);    
                }
        };
        try {
            if (SwingUtilities.isEventDispatchThread()) runner.run();
            else  SwingUtilities.invokeLater(runner);
        } catch (Exception ie) {}  

    }

    @Action
    public void selectColorProtocol() {
       protocolEditor.setUseColors(colorKeywordsMenuItem.isSelected());
    }

    @Action
    public void showConfigureExternalProgramsDialog() {
        getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        ExternalProgramsDialog dialog=new ExternalProgramsDialog(this);
        int height=dialog.getHeight();
        int width=dialog.getWidth();
        java.awt.Dimension size=getFrame().getSize();
        dialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));      
        dialog.setVisible(true);
        dialog.dispose();
        getFrame().setCursor(Cursor.getDefaultCursor());        
    }

    
    @Override
    public boolean handleUncaughtException(final Throwable e) {
        if (e instanceof IllegalStateException) return true; // do nothing for this type of exception
        Runnable runner=new Runnable() {
            @Override
            public void run() {
                int option=JOptionPane.showOptionDialog(getFrame(), e.getClass().getSimpleName()+":\n"+e.getMessage(), "ERROR: Uncaught Exception", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.ERROR_MESSAGE,null, new String[]{"OK","Send Error Report"},"OK");
                if (option==1 && !(e instanceof OutOfMemoryError)) {
                    engine.reportError(e);
                    //logMessage(e.getClass().getSimpleName()+": "+e.getMessage());
                    //e.printStackTrace(System.err);
                }
                scheduler.abortEmergency(); // in case the error occurred while some task was executing
            }
        };
        if (SwingUtilities.isEventDispatchThread()) runner.run();
        else SwingUtilities.invokeLater(runner);
        return true; // this is to implement the interface
    }

    @Override
    public void shutdown() {
        engine.shutdown();
        getApplication().exit(); 
    }

    @Action
    public void showCitationDialog() {
        String document="<html><body>" +
                "<h2>Citations</h2>" +
                "If you use PriorsEditor for your own research please cite the following publication:<br><br><br>" +
                "Klepper, K. and Drabløs, F. (2010)<br>\"PriorsEditor: a tool for the construction and use of positional priors in motif discovery\"<br><i>Bioinformatics</i> <b>26</b>(17) : 2195-97<br><br><br><br>" +
                "To cite external programs used within PriorsEditor, please see citation " +
                "information for individual programs under \"Tools\" &rarr; \"External Programs...\"" +                
                "</body><html>";        
        InfoDialog dialog=new InfoDialog(gui.getFrame(), "Citations", document,550,350);
        dialog.setVisible(true);
        dialog.dispose();             
    }

    
    /** 
     * This method checks to see if this is the first time the is user
     * starts PriorsEditor (signaled by the absence of a special flag-file in
     * the work directory). On startup the first time there might be some 
     * installations needed and license acceptance and registrations that must
     * be performed. If the method detects that the user has previously started
     * PriorsEditor it just returns prematurely
     */
    public void firstTimeStartup() {
        if (!engine.isFirstTimeStartup()) return; // is it first time!
        gui.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        setProgress(10); // just to have some start value
        String document="<html><body>" +
                "<h1>Welcome to PriorsEditor</h1>" +
                "<br>" +
                "PriorsEditor is a workbench for regulatory region analysis that emphasizes the use of positional priors to guide motif discovery searches" +
                "<br><br>" +
                "The software was developed by Kjetil Klepper (kjetil.klepper@ntnu.no) under the supervision of professor Finn Drabløs (finn.drablos@ntnu.no)<br><br>" +
                "The project was supported by the National Programme for Research in Functional Genomics in Norway (FUGE) " +
                "in the Research Council of Norway" +
                "<br><br>PriorsEditor is open source and free to use \"as is\" for academic and commercial purposes. " +
                "However, no parts of the source code may be reused in its original or modified form as part of other commercial software projects without the consent of the authors. " +
                "Note that PriorsEditor can link to other external programs whose use might be subject to other license restrictions." +
                "<br><br>" +
                "<center>"+
                "Bioinformatics and Gene Regulation Group (BiGR)<br>"+
                "Department of Cancer Research and Molecular Medicine<br>"+
                "Norwegian University of Science and Technology (NTNU)" +
                "<br></center><br>"+
                "</body></html>";
        InfoDialog dialog=new InfoDialog(gui.getFrame(), "PriorsEditor installation", document, 500,400);
        dialog.setVisible(true);
        dialog.dispose();  
        setProgress(20); //
        String priorityMessage=
                "<html>PriorsEditor comes bundled with the <i>de novo</i> motif discovery program <b>PRIORITY</b>, which<br>" +
                "has been developed by the group of professor Alexander Hartemink at Duke University,<br>Durham, North Carolina, USA<br><br>" +
                "In order to use PRIORITY from within PriorsEditor you must accept a separate licence agreement..." +
                "</html>";
        //boolean installPriority=false;
        //int n=JOptionPane.showConfirmDialog(this.getFrame(),priorityMessage,"Install PRIORITY?",JOptionPane.YES_NO_OPTION);
        //if (n==JOptionPane.YES_OPTION) installPriority=true;        
        JOptionPane.showMessageDialog(this.getFrame(),priorityMessage,"Install PRIORITY?",JOptionPane.INFORMATION_MESSAGE);       
        ArrayList<String> bundled=locateBundledFiles("/priorseditor/external",".xml");      
        if (!bundled.isEmpty()) logMessage("Installing bundled programs");
        int count=0;
        int size=bundled.size();
        for (String filename:bundled) {
            try {
                InputStream stream=this.getClass().getResourceAsStream("/priorseditor/external/"+filename);
                installBundledProgram(filename,stream);
                setProgress((int)(20+(30f/(float)size)*count));
                count++;
            } catch (SystemError e) {
               logMessage("Unable to install bundled program '"+filename+"' => "+e.getMessage());
            }
        }
        setProgress(50); //
        ArrayList<String> bundledMotifs=locateBundledFiles("/priorseditor/engine/resources",".pmx");
        logMessage("Installing bundled motif collections");
        installBundledMotifCollections(bundledMotifs,50,100); // note that this is asynchroneous

    }

    /** Installs the program with XML-config file provided in the given stream
     *  @param filename The name of the XML-config file (including XML-suffix)
     *  @param stream An InputStream providing access to the file
     */
    private void installBundledProgram(String filename,InputStream stream) throws SystemError {
        ExternalProgram program=null;
        program = ExternalProgram.initializeExternalProgramFromStream(stream,filename,true);
        // the following line should not happen (but just in case...)
        if (program==null) throw new SystemError("Unable to read program definition file");
        String programName=program.getName();
        String license=(String)program.getProperty("license");
        if (license!=null && !license.trim().isEmpty()) {
            license="<html><body>"+license+"</body></html>";
            JDialog dialog;
            if (programName.equals("PRIORITY")) dialog=new priorseditor.external.priority.PriorityRegistration(gui.getFrame(), license);
            else dialog=new LicenseAgreementDialog(gui.getFrame(), programName+" license", license);
            dialog.setVisible(true);
            boolean isAccepted=(dialog instanceof LicenseAgreementDialog)?((LicenseAgreementDialog)dialog).isLicenseAccepted():((priorseditor.external.priority.PriorityRegistration)dialog).isLicenseAccepted();
            dialog.dispose(); 
            if (!isAccepted) return;
        }    
        File externalfile=new File(engine.getWorkDirectory()+File.separator+"external"+File.separator+filename);
        try {
            program.saveConfigurationToFile(externalfile);
        } catch (Exception e) {
            if (e instanceof SystemError) throw (SystemError)e;
            else throw new SystemError(e.getClass().getSimpleName()+":"+e.getMessage());
        }         
        program.clearProperties();
        engine.registerExternalProgram(program);          
    }
    
   
    private void installBundledMotifCollections(final ArrayList<String> bundledMotifs, final int progressStart, final int progressEnd)  {
        final float range=progressEnd-progressStart;
        SwingWorker worker=new SwingWorker<Boolean, Void>() {
            Exception ex=null;
            @Override 
            public Boolean doInBackground() {
                int count=0;
                int size=bundledMotifs.size();
                for (String filename:bundledMotifs) {
                    try {
                        InputStream stream=this.getClass().getResourceAsStream("/priorseditor/engine/resources/"+filename);
                        gui.setProgress((int)(progressStart+(range/(float)size)*count));
                        gui.logMessage("Installing motif collection: "+filename);
                        engine.registerPredefinedMotifCollectionFromStream(stream, filename); 
                        count++; 
                        try {Thread.sleep(50);} catch (InterruptedException ie) {} // pause for applause :-)               
                    } catch (Exception e) {
                       logMessage("Unable to install bundled motif collection '"+filename+"' => "+e.getClass().getSimpleName()+":"+e.getMessage());
                       //e.printStackTrace(System.err);
                    }
                } 
                setProgress(100);
                return Boolean.TRUE;
            } // end doInBackground
            @Override
            public void done() { // this method is invoked on the EDT!
                gui.getFrame().setCursor(Cursor.getDefaultCursor());
                gui.setProgress(-1); // hides progressbar       
                if (engine.isFirstTimeStartup()) { // this should return FALSE now if everything is registered OK
                    JOptionPane.showMessageDialog(gui.getFrame(), "It seems that something went wrong during installation\nPlease try to restart PriorsEditor","Install error",JOptionPane.ERROR_MESSAGE);
                }
            }
        }; // end of SwingWorker class
        worker.execute();              
    }    
    
    
    
    
    
/** 
 * This method returns a list of files (with the given suffix) that are bundled
 * with PriorsEditor. I would really have liked this list to be determined dynamically
 * based on the files that are actually present in the given directories in the 
 * packaged JAR file, however, since I (after tedious struggle) still have been unable to
 * find ways to accomplish this, I must simply hardcode the names of the files in the list :-(
 */    
private ArrayList<String> locateBundledFiles(String packagedir, String suffix) {
        ArrayList<String> list=new ArrayList<String>();
        if (packagedir.equals("/priorseditor/engine/resources") && suffix.endsWith("pmx")) {
            list.add("Transfac_public.pmx");
            list.add("Jaspar_core.pmx");  
            list.add("Jaspar_FAM.pmx");  
            list.add("Jaspar_phylofacts.pmx");  
            list.add("Jaspar_splice.pmx");  
            list.add("Jaspar_cne.pmx");  
            list.add("Jaspar_polII.pmx");  
            list.add("Jaspar_PBM.pmx");  
            list.add("Jaspar_PBM_HLH.pmx");  
            list.add("Jaspar_PBM_HOMEO.pmx");         
        } else if (packagedir.equals("/priorseditor/external") && suffix.endsWith("xml")) {
            list.add("Priority.xml");
            list.add("SimpleScanner.xml");
        }
        return list;
  }



    
    
}

