/*
 * ProtocolEditor.java
 *
 * Created on 7. oktober 2008, 14:44
 */

package priorseditor.gui;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.event.CaretEvent;
import javax.swing.event.DocumentEvent;
import java.awt.BorderLayout;
import javax.swing.text.*;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.print.PrinterException;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;
import javax.swing.SwingWorker;
import javax.swing.border.EtchedBorder;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentListener;
import javax.swing.plaf.TextUI;
import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import priorseditor.engine.DataListener;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.OperationTask;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.protocol.Protocol;
import priorseditor.engine.data.*;
import priorseditor.engine.operations.Operation_new;
import priorseditor.engine.protocol.DataTypeTable;
import priorseditor.engine.protocol.ParseError;
import priorseditor.gui.operationdialog.OperationDialog;
import priorseditor.engine.dataformat.DataFormat;
import priorseditor.engine.protocol.StandardProtocol;
import priorseditor.external.ExternalProgram;
import priorseditor.gui.ProtocolManager;



/**
 * 
 *
 * @author  kjetikl
 */
public class ProtocolEditor extends javax.swing.JPanel implements DataListener {
    private ProtocolEditorPane protocolEditor;
    private LineNumberPanel leftmargin;
    private JPanel scrollingPanel;
    private JPanel statuspanel;
    private GradientPanel headerpanel;
    private JLabel headerlabel;
    private AbstractDocument document;
    private PriorsEditorGUI gui;
    private Protocol protocol;
    private ProtocolEditorContextMenu contextmenu;
    private JMenuItem editLineMenuItem;
    private int fontSize=12;
    private Font font;
    private JScrollPane protocolPanelScroll;
    private int lineHeight=0;
    private int leading=0;
    private OperationTask editTask; // this represents the task currently being edited
    private javax.swing.Icon errorIcon=new MiscIcons(MiscIcons.ERROR_ICON);
    private javax.swing.Timer repaintTimer;
    private CaretPositionPanel caretPositionPanel;
    private JLabel insertOrOverwriteLabel;
    private ImageIcon redbullet;
    private ImageIcon greenbullet;
    private ImageIcon yellowbullet;
    private ImageIcon blackbullet;
    private JLabel errorBulletLabel;
    private HashMap<String,Color> patternColors; // colors for 'global' identifiers
    private HashSet<String> variablenamepatterns; // Compiled regex patterns for variable names
    private boolean containsErrors; // this flag is set to true if the current Protocol document results in Parse Errors
    private JLabel statusLabel;
    private ProtocolDocumentListener protocolDocumentListener;
    private GUIUndoManager undoManager;
    private DocumentListener undoRedoDocumentListener;   
    private ProtocolManager protocolManager;
    //Pattern numericConstants=Pattern.compile("(?:\\W|^)(-\\d+\\.\\d+|-\\d+|\\d+\\.\\d+|\\d+)",Pattern.MULTILINE);    
    
    /** Creates new form OutputPanel */
    public ProtocolEditor(PriorsEditorGUI gui) {
        initComponents();
        this.gui=gui;
        protocolDocumentListener = new ProtocolDocumentListener();
        undoManager = gui.getUndoManager();
        undoRedoDocumentListener = gui.getUndoRedoDocumentListener(); 
        protocolManager = new ProtocolManager(this);
        patternColors=new HashMap<String,Color>();
        variablenamepatterns=new HashSet<String>();
        setupGlobalPatternColors();
        java.net.URL redBulletURL=getClass().getResource("resources/icons/redbullet.png");
        java.net.URL greenBulletURL=getClass().getResource("resources/icons/greenbullet.png");
        java.net.URL yellowBulletURL=getClass().getResource("resources/icons/yellowbullet.png");
        java.net.URL blackBulletURL=getClass().getResource("resources/icons/blackbullet.png");
        redbullet=new ImageIcon(redBulletURL); 
        greenbullet=new ImageIcon(greenBulletURL); 
        yellowbullet=new ImageIcon(yellowBulletURL); 
        blackbullet=new ImageIcon(blackBulletURL); 
        this.setLayout(new BorderLayout());   
        protocolEditor=new ProtocolEditorPane();
        font=new Font(Font.MONOSPACED,Font.PLAIN,fontSize);
        FontMetrics fontmetrics=protocolEditor.getFontMetrics(font);
        lineHeight=fontmetrics.getHeight();
        leading=fontmetrics.getLeading();
        protocolEditor.setFont(font);
        protocolEditor.setEditorKit(new ColoringEditorKit());
        protocolEditor.setSelectionColor(new Color(200,200,255));
        errorBulletLabel=new JLabel(greenbullet);
        Dimension errorBulletLabelSize=new Dimension(30,20);
        errorBulletLabel.setPreferredSize(errorBulletLabelSize);
        errorBulletLabel.setMinimumSize(errorBulletLabelSize);
        errorBulletLabel.setMaximumSize(errorBulletLabelSize);
        errorBulletLabel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(0, 2, 0, 0), BorderFactory.createEtchedBorder(EtchedBorder.LOWERED)));               
        errorBulletLabel.setHorizontalAlignment(JLabel.CENTER);            
        caretPositionPanel=new CaretPositionPanel();
        Dimension caretPositionPanelSize=new Dimension(60,20);
        caretPositionPanel.setPreferredSize(caretPositionPanelSize);
        caretPositionPanel.setMinimumSize(caretPositionPanelSize);
        caretPositionPanel.setMaximumSize(caretPositionPanelSize);
        caretPositionPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(0, 3, 0, 3), BorderFactory.createEtchedBorder(EtchedBorder.LOWERED)));               
        protocolEditor.addCaretListener(caretPositionPanel);
        leftmargin=new LineNumberPanel();
        Dimension leftmarginsize=new Dimension(40,200);
        leftmargin.setPreferredSize(leftmarginsize);
        leftmargin.setMinimumSize(leftmarginsize);
        statuspanel=new JPanel();
        Dimension statusfieldsize=new Dimension(200,24);
        statuspanel.setPreferredSize(statusfieldsize);
        statuspanel.setMinimumSize(statusfieldsize);
        statuspanel.setLayout(new BorderLayout());
        JPanel insetPanel1=new JPanel();
        insetPanel1.setLayout(new BoxLayout(insetPanel1, BoxLayout.LINE_AXIS));
        insetPanel1.add(errorBulletLabel);
        insetPanel1.add(caretPositionPanel);
        insertOrOverwriteLabel=new JLabel("INS");
        Dimension insertOrOverwriteLabelSize=new Dimension(34,20);
        insertOrOverwriteLabel.setPreferredSize(insertOrOverwriteLabelSize);
        insertOrOverwriteLabel.setMinimumSize(insertOrOverwriteLabelSize);
        insertOrOverwriteLabel.setMaximumSize(insertOrOverwriteLabelSize);
        insetPanel1.add(insertOrOverwriteLabel);
        insertOrOverwriteLabel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
        insertOrOverwriteLabel.setHorizontalAlignment(JLabel.CENTER);
        statuspanel.add(insetPanel1,BorderLayout.WEST);
        statusLabel=new JLabel("");
        statuspanel.add(statusLabel,BorderLayout.CENTER);
        scrollingPanel=new JPanel();
        scrollingPanel.setLayout(new BorderLayout());
        scrollingPanel.add(leftmargin,BorderLayout.WEST);
        scrollingPanel.add(protocolEditor,BorderLayout.CENTER);
        headerpanel=new GradientPanel();
        Dimension headersize=new Dimension(200,20);
        headerpanel.setPreferredSize(headersize);
        headerpanel.setMinimumSize(headersize);      
        headerpanel.setBorder(BorderFactory.createEmptyBorder(2,5,2,2));
        headerpanel.setLayout(new BoxLayout(headerpanel, BoxLayout.X_AXIS));
        headerlabel=new JLabel("");
        headerlabel.setFont(new Font(Font.MONOSPACED,Font.BOLD,12));
        headerlabel.setForeground(java.awt.Color.WHITE);
        headerpanel.add(headerlabel);
        headerpanel.setBackground(java.awt.Color.BLUE);
        headerpanel.setOpaque(true);
        this.add(headerpanel,BorderLayout.NORTH);
        this.add(statuspanel,BorderLayout.SOUTH);
        protocolPanelScroll=new JScrollPane(scrollingPanel);    
        this.add(protocolPanelScroll,java.awt.BorderLayout.CENTER);
        contextmenu=new ProtocolEditorContextMenu();
        
        protocolEditor.addMouseListener(new MouseAdapter(){
            @Override
            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) showContextMenu(e);
            }
            @Override
            public void mouseReleased(MouseEvent e) {
                if (e.isPopupTrigger()) showContextMenu(e);
            }           
        });
        repaintTimer = new javax.swing.Timer(1000, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                reparse();
                scrollingPanel.repaint();
            }
        });
        repaintTimer.setRepeats(false);
        repaintTimer.setCoalesce(true);
        gui.getEngine().addDataListener(this);
        /*
        Protocol useprotocol=protocolManager.createNewProtocol();
        protocolManager.addProtocol(useprotocol);  
        installProtocol(useprotocol);
        */
        installProtocol(null);
    }
        
    /** Returns a reference to the GUI */
    public PriorsEditorGUI getGUI() {
        return gui;
    }
   

    /**
     * Installs a new Protocol in the ProtocolEditor
     * (this will register all required listeners, set the document in the editor and
     * install the name of the protocol in the header, plus update relevant GUI actions and buttons)
     * Note: to change the protocol currently being edited, consider using changeProtocol() instead
     * since that method will also create and queue an undoable event
     */
    private void installProtocol(Protocol newprotocol) {
        if (protocol!=null) protocolManager.setCaretPosition(protocol, protocolEditor.getCaretPosition()); // store caret position in current protocol
        protocol=newprotocol; // link to field variable
        if (protocol!=null) {
            document=(AbstractDocument)newprotocol.getDocument(); // link to field variable
            document.addUndoableEditListener(undoManager);
            document.addDocumentListener(undoRedoDocumentListener); 
            document.addDocumentListener(protocolDocumentListener); 
            protocolEditor.setDocument(document);
            protocolManager.setCurrent(protocol.getName());
            protocolEditor.setVisible(true);
            try {protocolEditor.setCaretPosition(protocolManager.getCaretPosition(protocol));} catch(Exception e){}
            leftmargin.setVisible(true);
            headerpanel.setVisible(true);
            caretPositionPanel.setText("1:1 ");
            gui.setCloseEnabled(true);
            gui.setSaveEnabled(((StandardProtocol)newprotocol).isDirty());
            gui.setSaveAsEnabled(true);
            if (!gui.isRecording()) {
                gui.setRecordEnabled(true);
                gui.setExecuteEnabled(true);
                gui.setStopEnabled(false);
                gui.unselectRecordPlayStop();
            }
        } else {
            protocolManager.setCurrent(null);
            protocolEditor.setVisible(false);
            headerpanel.setVisible(false);
            leftmargin.setVisible(false);
            caretPositionPanel.setText("");
            caretPositionPanel.repaint();
            gui.setCloseEnabled(false);
            gui.setSaveEnabled(false);
            gui.setSaveAsEnabled(false);
            gui.setRecordEnabled(false);
            gui.setRecordMode(false);
            gui.setExecuteEnabled(false);
            gui.setStopEnabled(false);
        }
        leftmargin.removeAll(); // removes old parseerrors 
        leftmargin.repaint();
        reparse(); // this will also set the header!
        scrollingPanel.repaint();
    }
    
    
    /**
     * This method installs a brand new Protocol script in the editor
     * If the user supplies a Protocol script as an argument, this protocol will be
     * installed, else a new Protocol with a generic name will be created and installed instead
     * The call to this method results in an undoable event being registered on the undo queue
     */
    public void newProtocol(Protocol newprotocol) {
        String undoname="Open protocol";
        if (newprotocol==null) {
            undoname="New protocol";
            newprotocol=protocolManager.createNewProtocol();
        } 
        protocolManager.addProtocol(newprotocol);        
        String newProtocolName=newprotocol.getName();
        changeProtocol(newProtocolName, undoname, true);
    }
    
    /**
     * Replaces the contents of the old protocol with the contents of the new protocol
     * The old protocol should already be open (but not necessarily "installed"), while
     * the new protocol should not be installed (it will overwrite the old) 
     * This method is called when a user "opens" a file which is already open
     * @param protocolName The name of the protocol to be reverted
     * @param newProtocol The new protocol to replace the old
     */
    public void revertProtocol(String protocolName, Protocol newProtocol) {
        Protocol oldProtocol = protocolManager.getProtocol(protocolName);
        RecordingCompoundEdit undoableEdit= new RecordingCompoundEdit("Revert protocol");
        gui.getUndoManager().forwardUndoEvents(undoableEdit);
        PlainDocument doc=(PlainDocument)oldProtocol.getDocument();
        PlainDocument newdoc=(PlainDocument)newProtocol.getDocument();
        //changeProtocol(protocolName, null, true);
        changeProtocol(protocolName);
        try {
            doc.replace(0, doc.getLength(), newdoc.getText(0, newdoc.getLength()), null);
        } catch (Exception e) {}
        undoableEdit.end();
        gui.getUndoManager().forwardUndoEvents(null);
        gui.getUndoManager().addEdit(undoableEdit);                
    }
    
    /**
     * This method will install a previously opened Protocol script with the given 
     * name in the ProtocolEditor. If no Protocol with the specified name is currently 
     * registered with the ProtocolManager then nothing will happen
     * @param protocolName The name of the Protocol to change to
     * @param undoableEventName A name for the undoable event. If this is NULL it will default to "Change protocol"
     * @param undoable (if this is set to TRUE an undoable event will be queued)
     */
    public void changeProtocol(String protocolName, String undoableEventName, boolean undoable) {
        if (undoableEventName==null) undoableEventName="Change protocol";
        String oldProtocolName=null;
        if (protocol!=null) oldProtocolName=protocol.getName();       
        installProtocolEdit edit=new installProtocolEdit(undoableEventName,oldProtocolName, protocolName); // this will automatically uninstall the old protocol and install the new
        if (undoable) undoManager.addEdit(edit);
               
    }

    /**
     * This method will install a previously opened Protocol script with the given 
     * name in the ProtocolEditor. If no Protocol with the specified name is currently 
     * registered with the ProtocolManager then nothing will happen
     * A call to this method will not result in the queuing of an undoable event! 
     * @param protocolName The name of the Protocol to change to
     */    
    public void changeProtocol(String protocolName) {
        String oldProtocolName=null;
        if (protocol!=null) oldProtocolName=protocol.getName();    
        changeInstalledProtocol(oldProtocolName,protocolName);
    }    
    
    /** 
     * Parses the protocol script and updates information about ParseErrors and data types 
     * which can be used to "beautify" the rendering of the protocol. This method should be 
     * called intermittently 
     */
    private void reparse() {
        //errorBulletLabel.setIcon(yellowbullet);
        if (protocol!=null) {
            DataTypeTable lookup=protocol.getDataTypeLookupTable();
            lookup.clear();
            lookup.populateFromEngine(); //
            int numlines=getNumberOfLines();
            if (numlines>0) {
                for (int i=0;i<numlines;i++) {
                   int ypos=(i+1)*lineHeight-leading;
                   try {
                       protocol.parseCommand(i+1,false);
                   } catch (ParseError pe) {
                       JLabel errorLabel=new JLabel(errorIcon);
                       leftmargin.add(errorLabel);
                       errorLabel.setLocation(5, ypos-8);
                       errorLabel.setSize(8,8);
                       errorLabel.setToolTipText(pe.getMessage());           
                   }         
                }   
                setupDataPatternColors(); // register the names of data objects in the editors color map
            }
        } // end if protocol != null
        leftmargin.validate();
        int numberoferrors=leftmargin.getComponentCount();
        if (numberoferrors>0) {
            containsErrors=true;
            errorBulletLabel.setIcon(redbullet);
            statusLabel.setText("      "+numberoferrors+" parse error"+((numberoferrors==1)?"":"s")+" detected");
        }
        else {
            containsErrors=false;
            if (protocol==null) errorBulletLabel.setIcon(blackbullet);
            else errorBulletLabel.setIcon(greenbullet);
            statusLabel.setText("");
        }   
        if (protocol!=null) {
            if (((StandardProtocol)protocol).isDirty()) setHeader(protocol.getName()+" *");
            else setHeader(protocol.getName());
        } else {
             setHeader("");
        }
        headerlabel.repaint();
    }
    
    private void setHeader(String headertext) {
        headerlabel.setText(headertext);
    }
    
    private void showContextMenu(MouseEvent e) {   
        // reparse();
        int linenumber=e.getY()/lineHeight+1;
        editTask=null;
        try {
            editTask=protocol.parseCommand(linenumber,true); //
            editTask.setLineNumber(linenumber);
        } catch (Exception ex) {
            String msg=ex.getMessage();
            if (msg!=null && !msg.equals("null")) gui.logMessage(msg);
        }
        if (editTask!=null) editLineMenuItem.setEnabled(true);
        else editLineMenuItem.setEnabled(false);
        //if (editTask!=null) editTask.debug();
        contextmenu.show(e.getComponent(),e.getX(),e.getY());
    }
    

    
    
    
    /** Returns the number of lines (or commands) in the script (including comment lines) */
    public int getNumberOfLines() {
        if (document==null || document.getLength()==0) return 0;
        Element root=document.getDefaultRootElement();
        return root.getElementCount();
    }
    
    /**
     * Returns a reference to the Protocol script currently being edited
     * @return
     */
    public Protocol getProtocol() {
        return protocol;
    }
    
    /**
     * Returns a reference to the ProtocolManager which keeps track of all
     * Protocol scripts that are currently open (but not necessarily "installed" in the ProtocolEditor)
     * @return
     */    
    public ProtocolManager getProtocolManager() {
        return protocolManager;
    }
    
    /**
     * Returns the current position of the caret
     * @return
     */
    public int getCaretPosition() {
        return protocolEditor.getCaretPosition();
    } 

    
   /**
     * Inserts an operation-command into the protocol at the line corresponding to
     * the specified caret position. If the caret position is in the middle of a line
     * already occupied by a command the new command will be inserted before the old
     * command (i.e. the new command will be placed at the specified line and the old
     * command (and subsequent lines) will be bumped down 1 line).
     * @param task
     * @param pos
     */
    public boolean insertOperationAt(ExecutableTask task, int pos) {
        Element lineelement=document.getParagraphElement(pos);
        if (lineelement==null) return appendOperation(task);
        else {
            pos=lineelement.getStartOffset();
            return protocol.insertOperationAt(task, pos);
        }
        
    }
    
    /**
     * Appends an operation-command to the end of the protocol 
     * @param task
     */
    public boolean appendOperation(ExecutableTask task){
        return protocol.appendOperation(task);
    }

    /**
     * Replaces the command currently occupying the given line with a new operation
     * @param task
     */
    public boolean replaceCommandAtLine(ExecutableTask task, int line){
        Element lineelement=document.getDefaultRootElement().getElement(line-1);
        if (lineelement==null) return false;
        RecordingCompoundEdit undoableEdit=new RecordingCompoundEdit("replace");
        gui.getUndoManager().forwardUndoEvents(undoableEdit);
        protocolEditor.select(lineelement.getStartOffset(), lineelement.getEndOffset());
        protocolEditor.replaceSelection(task.getCommandString(protocol)+"\n");
        undoableEdit.end();
        gui.getUndoManager().forwardUndoEvents(null);
        gui.getUndoManager().addEdit(undoableEdit);
        return true;
    }
    
    /**
     * A call to this method launches the modal operations dialog where the
     * user can choose parameters for an operation
     * @param replace Set this to true if the command should replace the one curently under the caret
     * or false it a new command should be inserted before the one currently under the caret
     */
    public void launchOperationEditor(OperationTask operationtask, boolean replace) {
        String opstring=operationtask.getOperationName();
        operationtask.setParameter(OperationTask.ENGINE, gui.getEngine());
        //logMessage("Launcing Operations Editor: Operation="+opstring+", source="+sourceDatastring+", target="+targetDatastring);
        int linenumber=0;
        if (replace) linenumber=operationtask.getLineNumber();
        else {
            linenumber=document.getDefaultRootElement().getElementIndex(protocolEditor.getCaretPosition())+1;         
        }
        DataTypeTable typetable=protocol.getDataTypeStateAtCommand(linenumber-1);
        try {
            Class dialogclass = Class.forName("priorseditor.gui.operationdialog.OperationDialog_"+opstring);
            OperationDialog dialog=(OperationDialog)dialogclass.newInstance();
            dialog.initialize(operationtask,typetable,gui);
            dialog.setLocation(gui.getFrame().getWidth()/2-dialog.getWidth()/2, gui.getFrame().getHeight()/2-dialog.getHeight()/2);
            dialog.setVisible(true);               
            if (dialog.okPressed()) {
               ExecutableTask executable=dialog.getOperationTask(); // because the task might have changed!
               if (replace) replaceCommandAtLine(executable, operationtask.getLineNumber());
               else insertOperationAt(executable, protocolEditor.getCaretPosition());                   
            }
            dialog.dispose();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(gui.getFrame(), e.getClass().toString()+"\n"+e.getMessage(),"Operations editor" ,JOptionPane.ERROR_MESSAGE);
            //e.printStackTrace(System.err);
        }                                
  } 
    
    /**
     * Looks for matches to the regular expression in the document and replaces
     * all found occurrences with a new string
     * @param oldexpression a regular expression to search for
     * @param newString the new string which will replace the old expression
     */
    public void replaceExpression(String oldexpression, String newString) {
        if (document==null) return;
        try {
           String fullText=document.getText(0, document.getLength());
           fullText=fullText.replaceAll(oldexpression, newString);
           document.replace(0, document.getLength(),fullText,null);
        } catch (BadLocationException ble) {System.err.println("SYSTEM ERROR: "+ble.getMessage());}
    }

    
    /** Specifies whether the ProtocolEditor should display language keywords,
     *  data object names etc. in different colors
     */
    public void setUseColors(boolean flag) {
        ColoringEditorKit editorKit=(ColoringEditorKit)protocolEditor.getEditorKit();
        editorKit.setUseColors(flag);
        protocolEditor.revalidate();
        protocolEditor.repaint();
    }
    
    private void setupGlobalPatternColors() {
        Color orange=new Color(230,180,0);
        Color darkgreen=new Color(0,200,0);
        //patternColors.add(new Object[]{Pattern.compile("(^#.*$)"), Color.gray}); // comments
        //patternColors.add(new Object[]{Pattern.compile("(^@.*$)"), Color.CYAN}); // display settigs
        //patternColors.add(new Object[]{Pattern.compile("(\".*?\")"), darkgreen}); // display settigs
        //patternColors.add(new Object[]{Pattern.compile("(\\b-?\\d+\\.\\d+\\b|\\b-?\\d+\\b)"), Color.MAGENTA}); // numeric constants
        String[] list=Operation_new.getAvailableTypes();       
        for (String typename:list) {
            patternColors.put(typename,orange); // data types
        }   
        for (DataFormat dataformat:gui.getEngine().getAllDataFormats()) {
            patternColors.put(dataformat.getName(), orange); // Data formats
        } 
        for (ExternalProgram program:gui.getEngine().getAllExternalPrograms()) {
            patternColors.put(program.getName(), orange); // External programs
        } 
        for (Operation operation:gui.getEngine().getAllOperations()) {
            patternColors.put(operation.getName(),Color.red); // operations
        } 

   }
    
    private void setupDataPatternColors() {
        DataTypeTable lookup=protocol.getDataTypeLookupTable();
        for (String dataname:lookup.getAllDataItemsOfType(Data.class)) {
            variablenamepatterns.add(dataname); // operations
        }        
   }

   /**
    * Loads a new protocol from a specified and adds it to the ProtocolManager
    * This method does not ask for file
    * @param file
    */
   public void openProtocolFile(final File file) {
        final ProtocolEditor parent=this;
        final String filename=file.getName();
        gui.statusMessage("Opening protocol script \""+filename+"\"");
        SwingWorker worker=new SwingWorker<StandardProtocol, Void>() {
            Exception ex=null;
            @Override 
            public StandardProtocol doInBackground() {
                BufferedReader inputStream=null;
                String text="";
                try {
                    inputStream=new BufferedReader(new FileReader(file));
                    String line;
                    while((line=inputStream.readLine())!=null) {text+=line+"\n";}
                } catch (IOException e) { 
                    ex=e;
                    return null;
                } finally {
                    try {if (inputStream!=null) inputStream.close();} catch (IOException ioe) {System.err.println("SYSTEM ERROR: An error occurred when closing BufferedReader: "+ioe.getMessage());}
                }
                StandardProtocol newProtocol=new StandardProtocol(gui.getEngine(),text);
                return newProtocol;
            }
            @Override
            public void done() { // this method is invoked on the EDT!
                if (ex!=null) {
                     JOptionPane.showMessageDialog(parent, ex.getMessage(),"File error" ,JOptionPane.ERROR_MESSAGE);
                     return;
                }
                try {
                   StandardProtocol prot=get();
                   prot.setFileName(file.getCanonicalPath());
                   prot.setName(filename);
                   if (protocolManager.getProtocol(filename)!=null) revertProtocol(filename,prot); // revert existing protocol to saved version
                   else newProtocol(prot); // installs the new protocol script and adds it to the ProtocolManager
                   protocolManager.registerRecent(prot);
                   gui.getMainWindow().setSelectedTab("Protocol");  
                   gui.logMessage("Opened protocol script \""+filename+"\"");
                } catch (Exception e) {
                    gui.logMessage("Unable to open protocol script \""+filename+"\"  ("+e.getMessage()+")");
                }
            }
        }; // end of SwingWorker class
        worker.execute();
    } 
   
   
   /** Displays a file dialog and lets the user choose a protocol to open */
    public void openProtocolFile() {
        final JFileChooser fc = new JFileChooser(gui.getLastUsedDirectory());
        fc.setDialogTitle("Open Protocol");  
        int returnValue=fc.showOpenDialog(this);
        if (returnValue!=JFileChooser.APPROVE_OPTION) return; // user cancelled
        File file=fc.getSelectedFile();
        if (protocolManager.isProtocolOpen(file.getName())) {
            int choice=JOptionPane.showConfirmDialog(protocolEditor, "A protocol named \""+file.getName()+"\" is already open.\nWould you like to revert to the saved version?","Open Protocol",JOptionPane.OK_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
            if (choice!=JOptionPane.OK_OPTION) return;            
        }   
        gui.setLastUsedDirectory(file.getParentFile());
        openProtocolFile(file);
    }  
    
   /** 
    * Tries to open a protocol from a given file. However, if a protocol with the same name
    * is already open it will ask the user if he/she wants to revert to the saved file
    * (Note: "same name" means the last part of the filename is the same)
    */    
    public void openProtocolFileButAskToRevert(File file) {
        if (protocolManager.isProtocolOpen(file.getName())) {
            int choice=JOptionPane.showConfirmDialog(protocolEditor, "A protocol named \""+file.getName()+"\" is already open.\nWould you like to revert to the saved version?","Open Protocol",JOptionPane.OK_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
            if (choice!=JOptionPane.OK_OPTION) return;            
        }       
        openProtocolFile(file);        
    }
    
    
   /** 
    * Saves the current Protocol script to file.
    * If the supplied boolean argument is true, a dialog will be displayed to allow the user to
    * select a filename for the protocol. If the argument is false, the filename currently associated
    * with the script will be used (if the protocol has not been save before a dialog will be displayed)
    * @return FALSE if the user cancelled the save interactively or TRUE if the data was saved (or attempted)
    */
    public boolean saveProtocolFile(boolean askForNewName) {
        boolean dirty=protocolManager.saveProtocolFile((StandardProtocol)protocol,askForNewName);
        setDirtyFlag(dirty);
        return dirty;
    }   
            
    /**
     * Closes the current protocol file (and asks to user if he/she wants to save the protocol first)
     */
    public void closeProtocolFile() {
        if (((StandardProtocol)protocol).isDirty() && gui.doPromptBeforeDiscard()) {
            int response=JOptionPane.showConfirmDialog(this, "Save changes to "+protocol.getName()+"?","Close protocol",JOptionPane.YES_NO_CANCEL_OPTION);
            if (response==JOptionPane.CANCEL_OPTION) return;
            else if (response==JOptionPane.OK_OPTION) saveProtocolFile(false);
        }
        protocolManager.closeProtocol(protocol.getName());       
    }
    
    
    /** This method should be called when the system exits. It will perform cleanup() */
    public void shutdown() {
       protocolManager.shutdown();
    }
   
    /** Mostly called by the protocolManager after saving a file to update dirty flag and header */
    public void repaintProtocolEditor() { 
        leftmargin.removeAll(); // removes old parseerrors 
        leftmargin.repaint();
        reparse(); // this will also set the header!
        scrollingPanel.repaint();        
    }   
    
// ********    DataListener Interface implementation    **********
    
    @Override
    public void dataAdded(Data data) {
        repaintTimer.stop();
        leftmargin.removeAll(); 
        leftmargin.repaint();
        repaintTimer.start();
    }
    @Override
    public void dataRemoved(Data data) {
        repaintTimer.stop();
        leftmargin.removeAll(); 
        leftmargin.repaint();
        repaintTimer.start();
    }
    @Override
    public void dataUpdated(Data data) {
        repaintTimer.stop();
        leftmargin.removeAll(); 
        leftmargin.repaint();
        repaintTimer.start();
    }
    
    public void dataAddedToSet(Data parentDataset, Data child) {}
    public void dataRemovedFromSet(Data parentDataset, Data child) {}
    public void dataUpdate(Data oldvalue, Data newvalue) {}


 
    
    /** Prints the contents of the Protocol Editor to a selected printer */
    public void print() throws PrinterException {
        protocolEditor.print();
    }    
    
    public void setDirtyFlag(boolean dirty) {
       ((StandardProtocol)protocol).setDirtyFlag(dirty);
       gui.setSaveEnabled(dirty);
    }
    
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setName("Form"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables


// *****************************   PRIVATE CLASSES   ***********************************************
   
/** 
 * Listens to updates in the protocol document and calls starts (or resets) the timer which 
 * will eventually (after 1 second) trigger a reparsing of the document
 */    
private class ProtocolDocumentListener implements DocumentListener {
            @Override
            public void insertUpdate(DocumentEvent e) { // resets current timer and starts over              
                setDirtyFlag(true);                
                ((ColoringEditorKit)protocolEditor.getEditorKit()).clearCachedColoringInformation();
                repaintTimer.stop();
                leftmargin.removeAll(); 
                leftmargin.repaint();
                repaintTimer.start();
            }
            @Override
            public void removeUpdate(DocumentEvent e) { // resets current timer and starts over     
                setDirtyFlag(true); 
                ((ColoringEditorKit)protocolEditor.getEditorKit()).clearCachedColoringInformation();
                repaintTimer.stop();
                leftmargin.removeAll(); 
                leftmargin.repaint();
                repaintTimer.start();
            }
            @Override
            public void changedUpdate(DocumentEvent e) { // resets current timer and starts over     
                setDirtyFlag(true); 
                ((ColoringEditorKit)protocolEditor.getEditorKit()).clearCachedColoringInformation();
                repaintTimer.stop();
                leftmargin.removeAll(); 
                leftmargin.repaint();
                repaintTimer.start();
            }       
}    
   
/** Changes the installed protocol from the old to the new
 *  and makes sure all listeners are removed from the old protocol
 */
public void changeInstalledProtocol(String oldProtocolName, String newProtocolName) {   
        Protocol oldProtocol=protocolManager.getProtocol(oldProtocolName);
        Protocol newProtocol=protocolManager.getProtocol(newProtocolName);
        if (oldProtocol!=null) removeListeners(oldProtocol); // removes listeners from "current" protocol document
        installProtocol(newProtocol); 
    
}
 private void removeListeners(Protocol oldProtocol) {
        Document doc=oldProtocol.getDocument();
        doc.removeDocumentListener(protocolDocumentListener);
        doc.removeDocumentListener(undoRedoDocumentListener);
        doc.removeUndoableEditListener(undoManager);
 }
 
/**
 * Changes the currently installed (i.e. being edited) Protocol script with a new one 
 */
private class installProtocolEdit extends AbstractUndoableEdit {
    String oldProtocolName=null;
    String newProtocolName=null;
    String editName="";
    
    public installProtocolEdit(String editName, String oldprotocolName, String newprotocolName) {
        this.editName=editName;
        oldProtocolName=oldprotocolName;
        newProtocolName=newprotocolName;  
        changeInstalledProtocol(oldProtocolName,newProtocolName);
    }
    
    @Override
    public String getPresentationName() {
        return editName;
    }

    @Override
    public void redo() throws CannotRedoException {
        super.redo();
        changeInstalledProtocol(oldProtocolName,newProtocolName);   
    }

    @Override
    public void undo() throws CannotUndoException {
        super.undo();
        changeInstalledProtocol(newProtocolName,oldProtocolName);  
    }

       
    
}    
    
          
    
 private class ProtocolEditorContextMenu extends JPopupMenu {
        public ProtocolEditorContextMenu() {
            PriorsEditorEngine engine=gui.getEngine();
            editLineMenuItem=new JMenuItem("Edit");
            editLineMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (editTask!=null) launchOperationEditor(editTask,true); // 
                }
            });
            ArrayList<Operation> oplist=engine.getAllOperations();
            int numberOfOperations=oplist.size();
            String[] operationNames=new String[numberOfOperations];
            int i=0;
            for (Operation o:oplist) {
                operationNames[i]=o.getName();
                i++;
            }
            Arrays.sort(operationNames);
            ActionListener operationActionListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String operationName=e.getActionCommand();
                    OperationTask insertTask=new OperationTask(operationName);
                    insertTask.setParameter(OperationTask.OPERATION_NAME, operationName);
                    launchOperationEditor(insertTask,false);
                }                
            };

            JMenu operationsSubMenu=new JMenu("Insert operation");
            for (String opname:operationNames) {
                JMenuItem opmenuitem=new JMenuItem(opname);
                opmenuitem.addActionListener(operationActionListener);
                opmenuitem.setToolTipText(engine.getOperation(opname).getDescription());
                operationsSubMenu.add(opmenuitem);
            }
            this.add(editLineMenuItem);
            this.add(operationsSubMenu);
            this.add(new JSeparator());
            JMenuItem menuitem;
            menuitem=new JMenuItem(gui.getApplication().getContext().getActionMap().get("cut"));
            menuitem.setIcon(null);
            this.add(menuitem);
            menuitem=new JMenuItem(gui.getApplication().getContext().getActionMap().get("copy"));
            menuitem.setIcon(null);
            this.add(menuitem);
            menuitem=new JMenuItem(gui.getApplication().getContext().getActionMap().get("paste"));
            menuitem.setIcon(null);
            this.add(menuitem);     
        }   
        
        
}
    
private class LineNumberPanel extends JPanel {
    public LineNumberPanel() {
        super();
        setLayout(null);
        setOpaque(false);
        setBackground(new java.awt.Color(255,245,100));
    }
    @Override
    public void paintComponent(Graphics g) {
        g.setFont(font);
        g.setColor(java.awt.Color.BLACK);
        int height=this.getHeight();
        int width=this.getWidth();
        int numlines=getNumberOfLines();
        if (numlines==0) return;
        for (int i=0;i<numlines;i++) {
           int ypos=(i+1)*lineHeight-leading;
           if (ypos>height) break;
           String lineString=""+(i+1);
           int lnwidth=font.getStringBounds(lineString, ((Graphics2D)g).getFontRenderContext()).getBounds().width;
           int xpos=width-(lnwidth+5); // 5px margin
           g.drawString(""+lineString,xpos,ypos);
        }        
    }
}

private class CaretPositionPanel extends JLabel implements CaretListener {
        
        public CaretPositionPanel() {
            this.setHorizontalTextPosition(JLabel.RIGHT);
            this.setHorizontalAlignment(JLabel.RIGHT);
            this.setText("1:1 ");
        }    
        public void caretUpdate(CaretEvent e) {
            if (protocol==null){
                this.setText(" ");
            } else {
                int pos=getCaretPosition();
                Element root=document.getDefaultRootElement();
                int line=root.getElementIndex(pos);
                Element lineElement=root.getElement(line);
                int linestartsat=lineElement.getStartOffset();
                this.setText((line+1)+":"+(pos-linestartsat+1)+" ");   
            }
        }
    
}


private class ProtocolEditorPane extends JEditorPane {
	private boolean isOvertypeMode=false;
	private Caret defaultCaret;
	private Caret overtypeCaret;
        private Action changeOvertypeModeAction;
        private final String OVERTYPEMODE="OvertypeMode"; 
        private ExecuteCommandAction executeAction;
        
    public ProtocolEditorPane() {
        super();
        executeAction=new ExecuteCommandAction();
        Keymap parentKeyMap=getKeymap();
        Keymap newmap=addKeymap("ProtocolEditorKeymap", parentKeyMap);
        KeyStroke enter=KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,InputEvent.CTRL_DOWN_MASK);
        newmap.addActionForKeyStroke(enter, executeAction);
        this.setKeymap(newmap);
        defaultCaret = getCaret();
	overtypeCaret = new OvertypeCaret();
	overtypeCaret.setBlinkRate(defaultCaret.getBlinkRate());	
        changeOvertypeModeAction=new AbstractAction("OvertypeMode") { 
            public void actionPerformed(ActionEvent e) {
               setOvertypeMode(!isOvertypeMode());
            }
        };
        getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0, true), OVERTYPEMODE);
        getActionMap().put(OVERTYPEMODE, changeOvertypeModeAction);                
    }
    
    /** Returns the overtypemode*/
    public boolean isOvertypeMode(){
	return isOvertypeMode;
    }
    /** Sets the overtype mode */
    public void setOvertypeMode(boolean isOvertypeMode) {
	this.isOvertypeMode = isOvertypeMode;
	int pos = getCaretPosition(); 
	if (isOvertypeMode()){
	   setCaret(overtypeCaret); 
           insertOrOverwriteLabel.setText("OVR");
	} else  {
	   setCaret(defaultCaret);
           insertOrOverwriteLabel.setText("INS");
	}
        setCaretPosition( pos );
    }
 
    @Override
    public void replaceSelection(String text){
	if (isOvertypeMode()){
            int pos = getCaretPosition();
            if (getSelectedText()==null &&  pos<getDocument().getLength()) moveCaretPosition(pos+1);		
	}
        super.replaceSelection(text);
    }


private class ExecuteCommandAction extends AbstractAction {
      public void actionPerformed(ActionEvent e) {
          int pos=getCaretPosition();
          Element root=document.getDefaultRootElement();
          int line=root.getElementIndex(pos)+1;
          OperationTask task=null;
          DataTypeTable lookup=protocol.getDataTypeLookupTable();
          lookup.clear();
          lookup.populateFromEngine(); //
          try {
              task=protocol.parseCommand(line,false);
          } catch (ParseError pe) {gui.logMessage("Parse Error: "+pe.getMessage());}
          if (task!=null) gui.launchOperationTask(task,false); // false => do not record in protocol
      }    
}    
    
    
    
private class OvertypeCaret extends DefaultCaret {	
       @Override
	public void paint(Graphics g) {
	    if (isVisible()) {
	      try {
		   JTextComponent component = getComponent();
		   TextUI mapper = component.getUI();
		   Rectangle r = mapper.modelToView(component, getDot());
		   //g.setColor(component.getCaretColor());
		   g.setColor(Color.BLACK);
                   int width = g.getFontMetrics().charWidth( 'w' );
		   if (x!=r.x||y!=r.y) {
                      repaint(); 
                      x=r.x;y=r.y;height=r.height;
                   }
                   g.setColor(Color.BLACK);
                   g.setXORMode(component.getBackground());
		   g.fillRect(r.x, r.y, width, r.height);
              }
              catch (Exception e) {}
           }
        }
       
	@Override
	protected synchronized void damage(Rectangle r) {
            if (r == null) return;
            JTextComponent component = getComponent();
            x = r.x;
            y = r.y;
            width = component.getFontMetrics( component.getFont() ).charWidth( 'w' );
            height = r.height;
            repaint();           
	}        
    } // end class OvertypeCaret    
    
} // end of class ProtocolEditorPane


    


/** This editor kit can color keywords and special tokens in the text */
private class ColoringEditorKit extends DefaultEditorKit implements ViewFactory {
        private boolean shouldUseColors=true;
        private Object[] datatypenames=null; // each entry is a String[] containing a datatypename split into words e.g. [Motif], [Sequence,Collection] or [DNA,sequence,dataset]
        private HashMap<Integer,SortedMap> colorcache=new HashMap<Integer,SortedMap>(); // caches information about keyword coloring, must be updated each time the document is changed
        
        public void setUseColors(boolean flag) {
            shouldUseColors=flag;
        }
        public void clearCachedColoringInformation() {
            colorcache.clear();
        }
        
        @Override
        public ViewFactory getViewFactory() {return this;}
        @Override
        public View create (Element elem) {
             return new CustomView(elem);
        }
        public ColoringEditorKit() {
            super();
            String[] availabletypes=Operation_new.getAvailableTypes();
            datatypenames=new Object[availabletypes.length];
            int i=0;
            for (String typename:availabletypes) {
                String[] words=typename.split(" ");
                if (words.length<2) continue;
                datatypenames[i]=words; // keep only 
                i++;
            }         
            datatypenames=Arrays.copyOf(datatypenames,i);
        }
 
    private Pattern splitpattern=Pattern.compile("\\W");
    /** This is a slightly modified version of String.split() which also includes the matched expressions */        
    private String[] split(String input) {
        int index = 0;
        ArrayList<String> matchList = new ArrayList<String>();
        Matcher m = splitpattern.matcher(input);
        // Add segments before each match found
        while(m.find()) {
                String match = input.substring(index, m.start());
                matchList.add(match);
                index = m.end();
                matchList.add(input.substring(m.start(),index)); // this will include the matched non-word character in the matchList                              
        }        
        if (index == 0) return new String[] {input}; // If no match was found, return this        
        matchList.add(input.substring(index, input.length())); // Add remaining segment
        // Construct result
        int resultSize = matchList.size();
        while (resultSize > 0 && matchList.get(resultSize-1).equals("")) resultSize--;        
        String[] result = new String[resultSize];
        return matchList.subList(0, resultSize).toArray(result);
    }
    
    
    /** This method splits up a textline into 'tokens' */
    private String[] preprocessLineForColoring(String text) {
        String[] words=split(text);    
        int startQuote=-1; 
        // merge tokens within same quotes
        for (int i=0;i<words.length;i++) {
            if (words[i].trim().equals("\"")) {
                if (startQuote<0) {// opening quote
                    startQuote=i;
                } else {// closing quote
                    words[startQuote]+=words[i];
                    words[i]=null;
                    startQuote=-1;
                }
            } else if (startQuote>0) { // within a running quote
                words[startQuote]+=words[i]; // move this word into the 'quote' array element
                words[i]=null;
            }
        }
        // merge tokens for numbers (e.g. -94.23)
        for (int i=0;i<words.length;i++) {
            if (words[i]==null || !words[i].matches("\\s*\\d+\\s*")) continue;
            // we found a number... check the neighborhood
            if (i>0 && words[i-1]!=null) {
                if (words[i-1].endsWith("-.")) {
                    words[i]="-."+words[i];
                    words[i-1]=words[i-1].substring(0,words[i-1].length()-2);
                } else if (words[i-1].endsWith("-")) {
                    words[i]="-"+words[i];
                    words[i-1]=words[i-1].substring(0,words[i-1].length()-1);                     
                } else if (words[i-1].endsWith(".")) {
                    words[i]="."+words[i];
                    words[i-1]=words[i-1].substring(0,words[i-1].length()-1);                     
                }
            }
            if (i<words.length-1 && words[i+1]!=null && words[i+1].equals(".")) {
                 if (i<words.length-2 && words[i+2]!=null && words[i+2].matches("\\d+\\s*")) {                     
                     words[i]+=words[i+1]+words[i+2]; // contract
                     words[i+1]=null;
                     words[i+2]=null;
                 }
            }
            if (!variablenamepatterns.contains(words[i])) words[i]="\f"+words[i]; // the formfeed marks this a numerical token so that we don't have to parse it again
        }
        // contract data type names that span several words
        for (int i=0;i<words.length;i++) {
            if (words[i]==null || words[i].startsWith("\f")) continue;
            int matchingwordcount=getMatchingDataType(words,i); // returns 0 of no match is found
            if (matchingwordcount>0) {
                for (int j=1;j<matchingwordcount*2-1;j++) {
                    words[i]+=words[i+j];
                    words[i+j]=null;
                }
            }            
        }     
        return words; 
    }
    
    private int getMatchingDataType(String[] words, int pos) {
        for (int i=0;i<datatypenames.length;i++) {
            String[] datatypename=(String[])datatypenames[i];
            if (!words[pos].equals(datatypename[0])) continue; // first word does not match at this position
            if (pos+(datatypename.length-1)*2>=words.length) continue; // the datatype name is too long to fit here anyway
            boolean match=true;
            for (int j=1;j<datatypename.length;j++) {
                if (!words[pos+j*2].equals(datatypename[j])) {match=false;break;}
            }
            if (match) return datatypename.length;
        }
        return 0;
    }
    
    
    private SortedMap<Integer, Object[]> processLineForColoring(String text) {
        SortedMap<Integer, Object[]> colorMap = new TreeMap<Integer, Object[]>(); // holds start position (key) and end position and color info for the same matching regex      
        if (text.matches("\\s*#.*\n?")) {colorMap.put(0,new Object[]{new Integer(text.length()),Color.gray});} 
        else if (text.matches("\\s*@.*\n?")) {colorMap.put(0,new Object[]{new Integer(text.length()),Color.CYAN});}
        else {
            String[] words=preprocessLineForColoring(text.trim());
            int wordstart=0;
            for (String word:words) {
                if (word==null) continue; // this has been merged in preprocessing
                Color color=null;
                if (word.startsWith("\f")) {
                    color=Color.MAGENTA; // the \\f character is a flag to mark numerical tokens introduced in the preprocessing step above
                    word=word.substring(1);
                }
                //gui.logMessage("["+word+"] ("+word.length()+")");
                int wordend=wordstart+word.length();  
                word=word.trim();
                if (color==null) {
                     if (patternColors.containsKey(word)) color=patternColors.get(word);
                     else if (variablenamepatterns.contains(word)) color=Color.BLUE;
                     else if (word.startsWith("\"") && word.endsWith("\"")) color=new Color(0,200,0);
                }
                if (color!=null) {
                    colorMap.put(wordstart, new Object[]{new Integer(wordend),color});      
                }
                wordstart=wordend;               
            }
        }
        return colorMap;
    }   
    
        
  private class CustomView extends PlainView {// Note this is an inner class of ColoringEditorKit
    
   public CustomView(Element elem) {
       super(elem);
   }

    @Override
   protected void drawLine(int lineIndex, Graphics g, int x, int y) {
        Element line = getElement().getElement(lineIndex);
	Element elem;
	
	try {
	    if (line.isLeaf()) {
                drawElement(lineIndex, line, g, x, y);
	    } else {
	        // this line contains the composed text.
	        int count = line.getElementCount();
		for(int i = 0; i < count; i++) {
		    elem = line.getElement(i);
		    x = drawElement(lineIndex, elem, g, x, y);
		}
	    }
        } catch (BadLocationException e) {
            System.err.print("Can't render line: " + lineIndex);
        }
    }
   
    private int drawElement(int lineIndex, Element elem, Graphics g, int x, int y) throws BadLocationException {
	int p0 = elem.getStartOffset();
        int p1 = elem.getEndOffset();
        p1 = Math.min(getDocument().getLength(), p1);
	x = drawUnselectedText(g, x, y, p0, p1);        
        return x;
    }
    
    
    @Override
    @SuppressWarnings("unchecked")
    protected int drawUnselectedText(Graphics graphics, int x, int y, int p0, int p1) throws BadLocationException {

        Document doc = getDocument();
        String text = doc.getText(p0, p1 - p0);
        Segment segment = getLineBuffer();

        if (!shouldUseColors) {
            graphics.setColor(Color.black);
             doc.getText(p0, text.length(), segment);
             x = Utilities.drawTabbedText(segment, x, y, graphics, this, 0);
            return x; // Note early return           
        }
        
        // --- draw colored text below this line ---       
//        SortedMap<Integer, Object[]> colorMap=(SortedMap<Integer, Object[]>)colorcache.get(p0); // check if the line has been processed before and this info is up to date
//        if (colorMap==null) {
//            colorMap = processLineForColoring(text);
//            colorcache.put(p0, colorMap);
//        }
        
          SortedMap<Integer, Object[]> colorMap=processLineForColoring(text);
          
        // Colour the parts
        int i = 0;        
        for (Map.Entry<Integer, Object[]> entry : colorMap.entrySet()) {
            int start = entry.getKey();
            Object[] value = entry.getValue();
            int end = (Integer)value[0];
            Color color=(Color)value[1];

            if (i < start) { // paint black text at the start of the line
                graphics.setColor(Color.black);
                doc.getText(p0 + i, start - i, segment);
                x = Utilities.drawTabbedText(segment, x, y, graphics, this, i);
            }

            graphics.setColor(color);
            i = end;
            doc.getText(p0 + start, i - start, segment);
            x = Utilities.drawTabbedText(segment, x, y, graphics, this, start);       
        }

        // Paint possible remaining text black
        if (i < text.length()) {
            graphics.setColor(Color.black);
            doc.getText(p0 + i, text.length() - i, segment);
            x = Utilities.drawTabbedText(segment, x, y, graphics, this, i);
        }
        return x;
    }                

  } // end of private class CustomView                
    } // end of private class ColoringEditorKit
 



private class GradientPanel extends JPanel {
    private Color leftEnd=new Color(80,80,255);
    private Color rightEnd=new Color(150,230,255);
    private Color leftEndDisabled=new Color(50,50,50);
    private Color rightEndDisabled=new Color(180,180,180);
    public GradientPanel() {
        super();
        this.setOpaque(true);
    }
    
    @Override
    public void paintComponent(Graphics g) {
        Color leftColor=(protocol!=null)?leftEnd:leftEndDisabled;
        Color rightColor=(protocol!=null)?rightEnd:rightEndDisabled;       
        Graphics2D g2=(Graphics2D)g;
        int w=getWidth();
        int h=getHeight();
        GradientPaint gradient=new GradientPaint(0,0,leftColor,w,h,rightColor);
        g2.setPaint(gradient);
        g2.fillRect(0,0,w,h);
    }
    
} // end of private class GradientPanel



} // end of class ProtocolEditor

