/*
 * SequenceInputDialog.java
 *
 * Created on 3. april 2009, 10:21
 */

package priorseditor.gui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Vector;
import javax.swing.AbstractCellEditor;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingWorker;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import priorseditor.engine.AddSequencesTask;
import priorseditor.engine.GeneIDResolver;
import priorseditor.engine.data.Organism;
import priorseditor.engine.data.DNASequenceDataset;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.Sequence;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.operations.Operation_new;
import priorseditor.engine.GeneIDmapping;
import priorseditor.engine.GeneIdentifier;

/**
 *
 * @author  kjetikl
 */
public class SequenceInputDialog extends javax.swing.JDialog {
    private JTable advancedInputTable;
    private PriorsEditorGUI gui;
    private DefaultTableModel advancedTableModel;
    private int MAX_SEQUENCE_LENGTH=10000;
    private GeneIDResolver idResolver;
    private int exampleShown=-1;

    /** Creates new form SequenceInputDialog */
    public SequenceInputDialog(PriorsEditorGUI gui, GeneIDResolver resolver) {
        super(gui.getFrame(), true);
        this.gui=gui;
        this.idResolver=resolver;
        this.MAX_SEQUENCE_LENGTH=gui.getEngine().getMaxSequenceLength();
        initComponents();
        progressbar.setVisible(false);
        Vector<String> columnNames=new Vector<String>(9);
        columnNames.addElement("Name");
        columnNames.addElement("Organism");
        columnNames.addElement("Build");
        columnNames.addElement("Chromosome");
        columnNames.addElement("Start");
        columnNames.addElement("End");
        columnNames.addElement("Gene name");
        columnNames.addElement("Gene TSS");
        columnNames.addElement("Gene TES");
        columnNames.addElement("Orientation");
        advancedTableModel=new DefaultTableModel(columnNames,1);
        advancedInputTable=new JTable(advancedTableModel);
        ExcelAdapter adapter=new ExcelAdapter(advancedInputTable,true, ExcelAdapter.CONVERT_TO_FLOAT_OR_INTEGER); // enables copy/paste capabilities in the table
        advancedInputTable.setFillsViewportHeight(true);
        advancedInputTable.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        advancedInputTable.setCellSelectionEnabled(true);
        advancedInputTable.getTableHeader().setReorderingAllowed(false);
        tableScrollPane.setViewportView(advancedInputTable);
        Object[] dnaSourceList=gui.getEngine().getDataLoader().getAvailableDatatracks(DNASequenceDataset.class);
        dnaSequenceSourceCombobox.setModel(new DefaultComboBoxModel(dnaSourceList));
        DefaultTableCellRenderer organismRenderer=new SequenceTableRenderer_Organism();
        DefaultTableCellRenderer orientationRenderer=new SequenceTableRenderer_Orientation();
        DefaultTableCellRenderer rightRenderer=new SequenceTableRenderer_RightAlign();      
        DefaultTableCellRenderer numberRenderer=new SequenceTableRenderer_number();           
        advancedInputTable.getColumn("Organism").setCellRenderer(organismRenderer);
        advancedInputTable.getColumn("Orientation").setCellRenderer(orientationRenderer);
        advancedInputTable.getColumn("Chromosome").setCellRenderer(rightRenderer);
        advancedInputTable.getColumn("Start").setCellRenderer(numberRenderer);
        advancedInputTable.getColumn("End").setCellRenderer(numberRenderer);
        advancedInputTable.getColumn("Gene TSS").setCellRenderer(numberRenderer);
        advancedInputTable.getColumn("Gene TES").setCellRenderer(numberRenderer);
        advancedInputTable.getModel().addTableModelListener(new TableChangeListener());
        advancedInputTable.getColumn("Organism").setCellEditor(new CellEditor_Organism());
        advancedInputTable.getColumn("Orientation").setCellEditor(new CellEditor_Orientation());
        identifierTypeCombobox.setModel(new DefaultComboBoxModel(idResolver.getSupportedIDFormats()));
        int[] supportedOrganism=idResolver.getSupportedOrganisms();
        String[] organismNames=new String[supportedOrganism.length];
        for (int i=0;i<supportedOrganism.length;i++) {
            organismNames[i]=Organism.getCommonName(supportedOrganism[i]);
        }
        organismCombobox.setModel(new DefaultComboBoxModel(organismNames));   
        organismCombobox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                String organismname=(String)organismCombobox.getSelectedItem();
                int organismID=Organism.getTaxonomyID(organismname);
                String[] supportedBuilds=Organism.getSupportedGenomeBuilds(organismID);
                if (supportedBuilds!=null) {
                    genomeBuildCombobox.setModel(new DefaultComboBoxModel(supportedBuilds));
                    genomeBuildCombobox.setEnabled(true);
                } else {
                    genomeBuildCombobox.setEnabled(false);
                }
            }
        });
        organismCombobox.setSelectedIndex(0);
    }

    
    private ComboBoxModel getIDtypeModel() {
        return new DefaultComboBoxModel(new String[]{"Ensembl"});
    }
    
    public void disableAutoFetchDNA() {
        loadDNAtrackCheckbox.setSelected(false);
        loadDNAtrackCheckbox.setVisible(false);
        dnaSequenceSourceCombobox.setVisible(false);
        dnaTrackNameTextfield.setVisible(false);
        fromLabel.setVisible(false);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        fileOrInput = new javax.swing.ButtonGroup();
        selectMappingDialog = new javax.swing.JDialog();
        smdTopPanel = new javax.swing.JPanel();
        smdMessageLabel = new javax.swing.JLabel();
        smdButtonsPanel = new javax.swing.JPanel();
        jButton2 = new javax.swing.JButton();
        smdMiddlePanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        smdList = new javax.swing.JList();
        jPanel1 = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        loadDNAtrackCheckbox = new javax.swing.JCheckBox();
        dnaTrackNameTextfield = new javax.swing.JTextField();
        errorMessageLabel = new javax.swing.JLabel();
        fromLabel = new javax.swing.JLabel();
        dnaSequenceSourceCombobox = new javax.swing.JComboBox();
        progressbar = new javax.swing.JProgressBar();
        helpIconLabel = new javax.swing.JLabel();
        topPanel = new javax.swing.JPanel();
        tabPanel = new javax.swing.JTabbedPane();
        simpleTab = new javax.swing.JPanel();
        geneIDtypeLabel = new javax.swing.JLabel();
        upstreamLabel = new javax.swing.JLabel();
        downstreamLabel = new javax.swing.JLabel();
        anchorLabel = new javax.swing.JLabel();
        anchorPointComboBox = new javax.swing.JComboBox();
        browseButton = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        geneIdentifiersTextarea = new javax.swing.JTextArea();
        identifierTypeCombobox = new javax.swing.JComboBox();
        organismLabel = new javax.swing.JLabel();
        organismCombobox = new javax.swing.JComboBox();
        upstreamSpinner = new javax.swing.JSpinner();
        downstreamSpinner = new javax.swing.JSpinner();
        enteridslabel = new javax.swing.JLabel();
        genomeBuildLabel = new javax.swing.JLabel();
        genomeBuildCombobox = new javax.swing.JComboBox();
        helpIconButton = new javax.swing.JButton();
        advancedTab = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        advancedExampleButton = new javax.swing.JButton();
        jPanel4 = new javax.swing.JPanel();
        tableScrollPane = new javax.swing.JScrollPane();

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(SequenceInputDialog.class);
        selectMappingDialog.setTitle(resourceMap.getString("selectMappingDialog.title")); // NOI18N
        selectMappingDialog.setModal(true);
        selectMappingDialog.setName("selectMappingDialog"); // NOI18N

        smdTopPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(10, 10, 1, 1));
        smdTopPanel.setName("smdTopPanel"); // NOI18N
        smdTopPanel.setPreferredSize(new java.awt.Dimension(100, 60));
        smdTopPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        smdMessageLabel.setText(resourceMap.getString("smdMessageLabel.text")); // NOI18N
        smdMessageLabel.setName("smdMessageLabel"); // NOI18N
        smdTopPanel.add(smdMessageLabel);

        selectMappingDialog.getContentPane().add(smdTopPanel, java.awt.BorderLayout.PAGE_START);

        smdButtonsPanel.setMinimumSize(new java.awt.Dimension(100, 46));
        smdButtonsPanel.setName("smdButtonsPanel"); // NOI18N
        smdButtonsPanel.setPreferredSize(new java.awt.Dimension(100, 46));

        jButton2.setText(resourceMap.getString("jButton2.text")); // NOI18N
        jButton2.setName("jButton2"); // NOI18N
        jButton2.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                smdOKclicked(evt);
            }
        });

        javax.swing.GroupLayout smdButtonsPanelLayout = new javax.swing.GroupLayout(smdButtonsPanel);
        smdButtonsPanel.setLayout(smdButtonsPanelLayout);
        smdButtonsPanelLayout.setHorizontalGroup(
            smdButtonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, smdButtonsPanelLayout.createSequentialGroup()
                .addContainerGap(204, Short.MAX_VALUE)
                .addComponent(jButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 66, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        smdButtonsPanelLayout.setVerticalGroup(
            smdButtonsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, smdButtonsPanelLayout.createSequentialGroup()
                .addContainerGap(12, Short.MAX_VALUE)
                .addComponent(jButton2)
                .addContainerGap())
        );

        selectMappingDialog.getContentPane().add(smdButtonsPanel, java.awt.BorderLayout.PAGE_END);

        smdMiddlePanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 10, 1, 10));
        smdMiddlePanel.setName("smdMiddlePanel"); // NOI18N
        smdMiddlePanel.setLayout(new java.awt.BorderLayout());

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        smdList.setFont(resourceMap.getFont("smdList.font")); // NOI18N
        smdList.setForeground(resourceMap.getColor("smdList.foreground")); // NOI18N
        smdList.setName("smdList"); // NOI18N
        jScrollPane1.setViewportView(smdList);

        smdMiddlePanel.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        selectMappingDialog.getContentPane().add(smdMiddlePanel, java.awt.BorderLayout.CENTER);

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle(resourceMap.getString("Form.title")); // NOI18N
        setMinimumSize(new java.awt.Dimension(750, 439));
        setName("Form"); // NOI18N

        jPanel1.setMinimumSize(new java.awt.Dimension(100, 9));
        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setPreferredSize(new java.awt.Dimension(100, 80));

        okButton.setText(resourceMap.getString("okButton.text")); // NOI18N
        okButton.setMaximumSize(new java.awt.Dimension(75, 27));
        okButton.setMinimumSize(new java.awt.Dimension(75, 27));
        okButton.setName("okButton"); // NOI18N
        okButton.setPreferredSize(new java.awt.Dimension(75, 27));
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onOKClicked(evt);
            }
        });

        cancelButton.setText(resourceMap.getString("cancelButton.text")); // NOI18N
        cancelButton.setMaximumSize(new java.awt.Dimension(75, 27));
        cancelButton.setMinimumSize(new java.awt.Dimension(75, 27));
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.setPreferredSize(new java.awt.Dimension(75, 27));
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onCancelClicked(evt);
            }
        });

        loadDNAtrackCheckbox.setSelected(true);
        loadDNAtrackCheckbox.setText(resourceMap.getString("loadDNAtrackCheckbox.text")); // NOI18N
        loadDNAtrackCheckbox.setName("loadDNAtrackCheckbox"); // NOI18N

        dnaTrackNameTextfield.setText(resourceMap.getString("dnaTrackNameTextfield.text")); // NOI18N
        dnaTrackNameTextfield.setName("dnaTrackNameTextfield"); // NOI18N

        errorMessageLabel.setFont(resourceMap.getFont("errorMessageLabel.font")); // NOI18N
        errorMessageLabel.setForeground(resourceMap.getColor("errorMessageLabel.foreground")); // NOI18N
        errorMessageLabel.setText(resourceMap.getString("errorMessageLabel.text")); // NOI18N
        errorMessageLabel.setMaximumSize(new java.awt.Dimension(600, 16));
        errorMessageLabel.setMinimumSize(new java.awt.Dimension(100, 16));
        errorMessageLabel.setName("errorMessageLabel"); // NOI18N
        errorMessageLabel.setPreferredSize(new java.awt.Dimension(100, 16));

        fromLabel.setText(resourceMap.getString("fromLabel.text")); // NOI18N
        fromLabel.setName("fromLabel"); // NOI18N

        dnaSequenceSourceCombobox.setName("dnaSequenceSourceCombobox"); // NOI18N

        progressbar.setName("progressbar"); // NOI18N

        helpIconLabel.setText(resourceMap.getString("helpIconLabel.text")); // NOI18N
        helpIconLabel.setToolTipText(resourceMap.getString("helpIconLabel.toolTipText")); // NOI18N
        helpIconLabel.setName("helpIconLabel"); // NOI18N
        helpIconLabel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                showExample(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(loadDNAtrackCheckbox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(dnaTrackNameTextfield, javax.swing.GroupLayout.PREFERRED_SIZE, 219, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(fromLabel)
                        .addGap(18, 18, 18)
                        .addComponent(dnaSequenceSourceCombobox, 0, 256, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                        .addComponent(progressbar, javax.swing.GroupLayout.PREFERRED_SIZE, 43, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(helpIconLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(errorMessageLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 461, Short.MAX_VALUE)
                        .addGap(26, 26, 26)
                        .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(loadDNAtrackCheckbox)
                    .addComponent(dnaTrackNameTextfield, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(fromLabel)
                    .addComponent(dnaSequenceSourceCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(progressbar, javax.swing.GroupLayout.DEFAULT_SIZE, 27, Short.MAX_VALUE)
                            .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(errorMessageLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(helpIconLabel)))
                .addContainerGap())
        );

        getContentPane().add(jPanel1, java.awt.BorderLayout.PAGE_END);

        topPanel.setMinimumSize(new java.awt.Dimension(100, 30));
        topPanel.setName("topPanel"); // NOI18N
        topPanel.setPreferredSize(new java.awt.Dimension(100, 20));

        javax.swing.GroupLayout topPanelLayout = new javax.swing.GroupLayout(topPanel);
        topPanel.setLayout(topPanelLayout);
        topPanelLayout.setHorizontalGroup(
            topPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 720, Short.MAX_VALUE)
        );
        topPanelLayout.setVerticalGroup(
            topPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 20, Short.MAX_VALUE)
        );

        getContentPane().add(topPanel, java.awt.BorderLayout.PAGE_START);

        tabPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 5, 1, 5));
        tabPanel.setMinimumSize(new java.awt.Dimension(700, 400));
        tabPanel.setName("tabPanel"); // NOI18N
        tabPanel.setPreferredSize(new java.awt.Dimension(700, 400));

        simpleTab.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        simpleTab.setToolTipText(resourceMap.getString("simpleTab.toolTipText")); // NOI18N
        simpleTab.setName("simpleTab"); // NOI18N

        geneIDtypeLabel.setText(resourceMap.getString("geneIDtypeLabel.text")); // NOI18N
        geneIDtypeLabel.setName("geneIDtypeLabel"); // NOI18N

        upstreamLabel.setText(resourceMap.getString("upstreamLabel.text")); // NOI18N
        upstreamLabel.setName("upstreamLabel"); // NOI18N

        downstreamLabel.setText(resourceMap.getString("downstreamLabel.text")); // NOI18N
        downstreamLabel.setName("downstreamLabel"); // NOI18N

        anchorLabel.setText(resourceMap.getString("anchorLabel.text")); // NOI18N
        anchorLabel.setName("anchorLabel"); // NOI18N

        anchorPointComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Transcription Start Site", "Transcription End Site" }));
        anchorPointComboBox.setLightWeightPopupEnabled(false);
        anchorPointComboBox.setMinimumSize(new java.awt.Dimension(180, 22));
        anchorPointComboBox.setName("anchorCombobox"); // NOI18N
        anchorPointComboBox.setPreferredSize(new java.awt.Dimension(180, 22));

        browseButton.setText(resourceMap.getString("browseButton.text")); // NOI18N
        browseButton.setName("browseButton"); // NOI18N
        browseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onBrowseButtonPressed(evt);
            }
        });

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        geneIdentifiersTextarea.setColumns(20);
        geneIdentifiersTextarea.setRows(5);
        geneIdentifiersTextarea.setName("geneIdentifiersTextarea"); // NOI18N
        geneIdentifiersTextarea.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                idTextAreaFocusGained(evt);
            }
        });
        jScrollPane2.setViewportView(geneIdentifiersTextarea);

        identifierTypeCombobox.setModel(getIDtypeModel());
        identifierTypeCombobox.setName("identifierTypeCombobox"); // NOI18N

        organismLabel.setText(resourceMap.getString("organismLabel.text")); // NOI18N
        organismLabel.setName("organismLabel"); // NOI18N

        organismCombobox.setName("organismCombobox"); // NOI18N

        upstreamSpinner.setModel(new javax.swing.SpinnerNumberModel(Integer.valueOf(-2000), null, null, Integer.valueOf(1)));
        upstreamSpinner.setMaximumSize(new java.awt.Dimension(98, 22));
        upstreamSpinner.setMinimumSize(new java.awt.Dimension(98, 22));
        upstreamSpinner.setName("upstreamSpinner"); // NOI18N
        upstreamSpinner.setPreferredSize(new java.awt.Dimension(98, 22));

        downstreamSpinner.setModel(new javax.swing.SpinnerNumberModel(Integer.valueOf(200), null, null, Integer.valueOf(1)));
        downstreamSpinner.setMaximumSize(new java.awt.Dimension(98, 22));
        downstreamSpinner.setMinimumSize(new java.awt.Dimension(98, 22));
        downstreamSpinner.setName("downstreamSpinner"); // NOI18N
        downstreamSpinner.setOpaque(false);
        downstreamSpinner.setPreferredSize(new java.awt.Dimension(98, 22));

        enteridslabel.setText(resourceMap.getString("enteridslabel.text")); // NOI18N
        enteridslabel.setName("enteridslabel"); // NOI18N

        genomeBuildLabel.setText(resourceMap.getString("genomeBuildLabel.text")); // NOI18N
        genomeBuildLabel.setName("genomeBuildLabel"); // NOI18N

        genomeBuildCombobox.setName("genomeBuildCombobox"); // NOI18N

        helpIconButton.setIcon(resourceMap.getIcon("helpIconButton.icon")); // NOI18N
        helpIconButton.setText(resourceMap.getString("helpIconButton.text")); // NOI18N
        helpIconButton.setToolTipText(resourceMap.getString("helpIconButton.toolTipText")); // NOI18N
        helpIconButton.setName("helpIconButton"); // NOI18N
        helpIconButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                showExample(evt);
            }
        });

        javax.swing.GroupLayout simpleTabLayout = new javax.swing.GroupLayout(simpleTab);
        simpleTab.setLayout(simpleTabLayout);
        simpleTabLayout.setHorizontalGroup(
            simpleTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(simpleTabLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(simpleTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 681, Short.MAX_VALUE)
                    .addGroup(simpleTabLayout.createSequentialGroup()
                        .addComponent(anchorLabel)
                        .addGap(18, 18, 18)
                        .addComponent(anchorPointComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 148, Short.MAX_VALUE)
                        .addComponent(upstreamLabel)
                        .addGap(18, 18, 18)
                        .addComponent(upstreamSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(downstreamLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(downstreamSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(simpleTabLayout.createSequentialGroup()
                        .addComponent(geneIDtypeLabel)
                        .addGap(18, 18, 18)
                        .addComponent(identifierTypeCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, 164, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(organismLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(organismCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, 125, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(genomeBuildLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(genomeBuildCombobox, 0, 114, Short.MAX_VALUE))
                    .addGroup(simpleTabLayout.createSequentialGroup()
                        .addComponent(enteridslabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(browseButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 247, Short.MAX_VALUE)
                        .addComponent(helpIconButton)))
                .addContainerGap())
        );
        simpleTabLayout.setVerticalGroup(
            simpleTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(simpleTabLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(simpleTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(geneIDtypeLabel)
                    .addComponent(identifierTypeCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(organismLabel)
                    .addComponent(organismCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(genomeBuildLabel)
                    .addComponent(genomeBuildCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(23, 23, 23)
                .addGroup(simpleTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(enteridslabel)
                    .addComponent(browseButton)
                    .addComponent(helpIconButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 259, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(simpleTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(simpleTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(anchorLabel)
                        .addComponent(anchorPointComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(simpleTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(downstreamLabel)
                        .addComponent(upstreamLabel)
                        .addComponent(upstreamSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(downstreamSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );

        tabPanel.addTab(resourceMap.getString("simpleTab.TabConstraints.tabTitle"), simpleTab); // NOI18N

        advancedTab.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        advancedTab.setName("advancedTab"); // NOI18N
        advancedTab.setLayout(new java.awt.BorderLayout());

        jPanel3.setMinimumSize(new java.awt.Dimension(100, 40));
        jPanel3.setName("jPanel3"); // NOI18N
        jPanel3.setPreferredSize(new java.awt.Dimension(100, 40));

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        advancedExampleButton.setIcon(resourceMap.getIcon("advancedExampleButton.icon")); // NOI18N
        advancedExampleButton.setText(resourceMap.getString("advancedExampleButton.text")); // NOI18N
        advancedExampleButton.setName("advancedExampleButton"); // NOI18N
        advancedExampleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showAdvancedExample(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 110, Short.MAX_VALUE)
                .addComponent(advancedExampleButton)
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(advancedExampleButton))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        advancedTab.add(jPanel3, java.awt.BorderLayout.PAGE_START);

        jPanel4.setName("jPanel4"); // NOI18N
        jPanel4.setLayout(new java.awt.BorderLayout());

        tableScrollPane.setName("tableScrollPane"); // NOI18N
        jPanel4.add(tableScrollPane, java.awt.BorderLayout.CENTER);

        advancedTab.add(jPanel4, java.awt.BorderLayout.CENTER);

        tabPanel.addTab(resourceMap.getString("advancedTab.TabConstraints.tabTitle"), advancedTab); // NOI18N

        getContentPane().add(tabPanel, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void onOKClicked(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onOKClicked
    errorMessageLabel.setText("");
    if (loadDNAtrackCheckbox.isSelected()) {
        String targetName=dnaTrackNameTextfield.getText().trim();
        String nameError=gui.getEngine().checkNameValidity(targetName,false);
        if (nameError!=null) {
           JOptionPane.showMessageDialog(this, nameError, "Illegal DNA datatrack name: "+targetName, JOptionPane.ERROR_MESSAGE);
           return;
        }    
    }       
    AddSequencesTask task=null;
    if (tabPanel.getSelectedComponent()==simpleTab) { 
        String problem=checkSimpleTabEntry();
        if (problem!=null) {
            errorMessageLabel.setText(problem);
            return;
        }
        parseGeneIDEntries(); //
    }  else { // Advanced Dialog -> Manual entry of all information
        removeBlankRows();
        String problem=checkAdvancedTabEntry();
        if (problem!=null) {
            errorMessageLabel.setText(problem);
            return;
        }  
        Object[][] data=parseManualEntryTable(); // add data directly
        Sequence[] sequences=new Sequence[data.length];
        for (int i=0;i<sequences.length;i++) {
            sequences[i]=new Sequence((String)data[i][0], ((Integer)data[i][1]).intValue(), (String)data[i][2], (String)data[i][3], ((Integer)data[i][4]).intValue(), ((Integer)data[i][5]).intValue(), (String)data[i][6], ((Integer)data[i][7]), ((Integer)data[i][8]), ((Integer)data[i][9]).intValue());
        }
        task=new AddSequencesTask(gui.getEngine(),sequences);
        exitAndLoad(task);
    }      
}//GEN-LAST:event_onOKClicked



private void exitAndLoad(AddSequencesTask task) {
    this.setVisible(false);    
    gui.launchAddSequencesTask(task);
    if (loadDNAtrackCheckbox.isSelected()) launchLoadDNATrack();     
}

private void launchLoadDNATrack() {
        Object s=dnaSequenceSourceCombobox.getSelectedItem();
        String dnaTrackSource=null;
        if (s!=null) dnaTrackSource=s.toString();
        if (dnaTrackSource==null || dnaTrackSource.isEmpty()) return;
        Operation operation=gui.getEngine().getOperation("new");
        OperationTask newDNATrackTask=new OperationTask(operation.getName());
        String targetName=dnaTrackNameTextfield.getText().trim();
        newDNATrackTask.setParameter(OperationTask.OPERATION, operation);
        newDNATrackTask.setParameter(OperationTask.OPERATION_NAME, operation.getName());
        newDNATrackTask.setParameter(OperationTask.TARGET_NAME, targetName);
        newDNATrackTask.setParameter(OperationTask.SOURCE_NAME, targetName);
        newDNATrackTask.setParameter(Operation_new.DATA_TYPE, DNASequenceDataset.getType());    
        newDNATrackTask.setParameter(Operation_new.PARAMETERS,Operation_new.DATA_TRACK_PREFIX+dnaTrackSource);
        gui.launchOperationTask(newDNATrackTask, gui.isRecording());    
}






private void onCancelClicked(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onCancelClicked
      this.setVisible(false);
}//GEN-LAST:event_onCancelClicked

private void onBrowseButtonPressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_onBrowseButtonPressed

    JFileChooser filechooser = new JFileChooser(gui.getLastUsedDirectory());
    filechooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    int status=filechooser.showOpenDialog(null);
    if (status==JFileChooser.APPROVE_OPTION) {
        File selected=filechooser.getSelectedFile();
        gui.setLastUsedDirectory(selected);
        try {
           loadFileInBackground(selected);
        } catch (Exception e) {}
    }
}//GEN-LAST:event_onBrowseButtonPressed

private void idTextAreaFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_idTextAreaFocusGained
    // fromListRadioButton.setSelected(true);
}//GEN-LAST:event_idTextAreaFocusGained

private void showExample(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_showExample
    String example=//GEN-LAST:event_showExample
    ""; // auto-generated code prehibits deletion of first line...
    exampleShown++;
    example=exampleSequences[exampleShown%exampleSequences.length];
    geneIdentifiersTextarea.setText(example);
}

private void smdOKclicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_smdOKclicked
      selectMappingDialog.setVisible(false);
}//GEN-LAST:event_smdOKclicked

private void showAdvancedExample(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showAdvancedExample
// TODO add your handling code here:
         Object[][] list=new Object[][]{//GEN-LAST:event_showAdvancedExample
                    {"NTNG1",Organism.HUMAN,"hg18","1",new Integer(107482152),new Integer(107484352),"NTNG1",new Integer(107484152),new Integer(107827603),new Integer(1)},
                    {"RPRM",Organism.HUMAN,"hg18","2",new Integer(154043368),new Integer(154045568),"RPRM",new Integer(154043568),new Integer(154042098),new Integer(-1)},
                    {"CUX2",Organism.HUMAN,"hg18","12",new Integer(109954212),new Integer(109956412),"CUX2",new Integer(109956212),new Integer(110272739),new Integer(1)},
                    {"HAPLN4",Organism.HUMAN,"hg18","19",new Integer(19244978),new Integer(19247178),"HAPLN4",new Integer(19245178),new Integer(19226557),new Integer(-1)},
                    {"NTNG2",Organism.HUMAN,"hg18","9",new Integer(134025155),new Integer(134027355),"NTNG2",new Integer(134027155),new Integer(134109742),new Integer(1)},
                    };
         advancedInputTable.setModel(advancedTableModel);
         while (advancedTableModel.getRowCount()>0) {
             advancedTableModel.removeRow(0);
         }
         for (int i=0;i<list.length;i++) {
             advancedTableModel.addRow(list[i]);
         }         
}


private void removeBlankRows() {
    int rows=advancedTableModel.getRowCount();
    int columns=advancedTableModel.getColumnCount();
    for (int i=0;i<rows;i++) {
        boolean empty=true;
        for (int j=0;j<columns;j++) {
            Object val=advancedTableModel.getValueAt(i, j);
            if (val!=null && !val.toString().isEmpty()) {empty=false;break;}
        }
        if (empty) {advancedTableModel.removeRow(i);i--;rows--;}
    }    
}

/**
 * Checks if the input in the Advanced tab table is correctly formatted
 * If errors are encountered a message string describing the problem is returned.
 * If everything is OK a null value is returned
 * Note that some "errors" might be fixed, for instance of the sequence has 
 * no associated TSS or TES, these will be set to the start and end positions respectively 
 * If the sequence has no name but has a gene, the name of the gene will be used as the name
 * of the sequence, etc.
 */
private String checkAdvancedTabEntry() {
    int rows=advancedTableModel.getRowCount();
    HashSet<String> usedNames=new HashSet<String>();
    int nameColumn=0;
    int organismColumn=1;
    int buildColumn=2;
    int chromosomeColumn=3;
    int startColumn=4;
    int endColumn=5;
    int geneColumn=6;
    int TSSColumn=7;
    int TESColumn=8;
    int orientationColumn=9;

    for (int i=0;i<rows;i++) {
        Object nameValue=advancedTableModel.getValueAt(i, nameColumn);
        Object organismValue=advancedTableModel.getValueAt(i, organismColumn);
        Object buildValue=advancedTableModel.getValueAt(i, buildColumn);
        Object chrValue=advancedTableModel.getValueAt(i, chromosomeColumn);
        Object startValue=advancedTableModel.getValueAt(i, startColumn);
        Object endValue=advancedTableModel.getValueAt(i, endColumn);
        Object geneValue=advancedTableModel.getValueAt(i, geneColumn);
        Object TSSValue=advancedTableModel.getValueAt(i, TSSColumn);
        Object TESValue=advancedTableModel.getValueAt(i, TESColumn);
        Object orientationValue=advancedTableModel.getValueAt(i, orientationColumn);
        if ((nameValue==null || nameValue.toString().isEmpty()) && (geneValue==null || geneValue.toString().isEmpty())) return "Row "+(i+1)+": Missing both sequence and gene name";
        else if (usedNames.contains(nameValue.toString())) return "Row "+(i+1)+": Sequence name '"+nameValue.toString()+"' has been used before";
        else if (geneValue==null || geneValue.toString().isEmpty()) {} // {advancedTableModel.setValueAt(nameValue,i, geneColumn);geneValue=nameValue;}
        else if (nameValue==null || nameValue.toString().isEmpty()) {advancedTableModel.setValueAt(geneValue.toString(),i, nameColumn);nameValue=geneValue.toString();}
        String namecheck=gui.getEngine().checkSequenceNameValidity(nameValue.toString(), false);
        if (namecheck!=null) return ("Row "+(i+1)+": Invalid sequence name '"+nameValue.toString()+"': "+namecheck);
        usedNames.add(nameValue.toString());
        if (orientationValue==null || !(orientationValue instanceof Integer)) {orientationValue=new Integer(1);advancedTableModel.setValueAt(orientationValue,i, orientationColumn);}
        if (startValue==null || !(startValue instanceof Integer)) return "Row "+(i+1)+": Start is not an integer value";
        if (endValue==null || !(endValue instanceof Integer)) return "Row "+(i+1)+": End is not an integer value";
        if (TSSValue==null || TSSValue.toString().trim().isEmpty()) { // use start of sequence as TSS if not explicitly set=
//            Object newvalue=(((Integer)orientationValue).intValue()==1)?startValue:endValue;
//            advancedTableModel.setValueAt(newvalue,i, TSSColumn);
//            TSSValue=newvalue;
              TSSValue=null;
              advancedTableModel.setValueAt(TSSValue,i, TSSColumn);
        }
        if (TSSValue!=null && !(TSSValue instanceof Integer)) return "Row "+(i+1)+": TSS is not an integer value";
        if (TESValue==null || TESValue.toString().trim().isEmpty()) {
//            Object newvalue=(((Integer)orientationValue).intValue()==1)?endValue:startValue;
//            advancedTableModel.setValueAt(newvalue,i, TESColumn);
//            TESValue=newvalue;
              TESValue=null;
              advancedTableModel.setValueAt(TESValue,i, TESColumn);
        }
        if (TESValue!=null && !(TESValue instanceof Integer)) return "Row "+(i+1)+": TES is not an integer value";
        if (chrValue==null) return "Row "+(i+1)+": Missing chromosome specification";
        if (organismValue==null || !(organismValue instanceof Integer)) return "Row "+(i+1)+": Organism is not a valid Taxonomy ID value";
        if (buildValue==null  || buildValue.toString().isEmpty()) return "Row "+(i+1)+": Missing genome build specification";
        if ( ((Integer)startValue).intValue()>((Integer)endValue).intValue())  return "Row "+(i+1)+": Start coordinate should be smaller than End coordinate";
        if ( ((Integer)endValue).intValue()-((Integer)startValue).intValue()+1>MAX_SEQUENCE_LENGTH)  return "Row "+(i+1)+": Sequence exceeds (selfimposed) maximum length of "+MAX_SEQUENCE_LENGTH+" bp";
    }
    return null;
}


/**
 * Checks if the input in the Simple Tab dialog is correctly formatted
 * If errors are encountered a message string describing the problem is returned.
 * If everything is OK a null value is returned
 */
private String checkSimpleTabEntry() {
    String text=geneIdentifiersTextarea.getText().trim();
    text=text.replaceAll("\\s*,\\s*", "\n");
    geneIdentifiersTextarea.setText(text);
    String[] lines=text.split("\n");
    for (String line:lines) {
        if (line.startsWith("#")) continue; // regard as comment
        String[] elements=line.split("\t");
        if (elements.length>3) return "The number of columns in a line must be between 1 and 3";
        if (elements.length>=2) {
            String formatString=elements[1];
            if (!idResolver.isIDFormatSupported(formatString)) return "Unknown ID format: "+formatString;
        }
        if (elements.length>=3) {
//            String organismString=elements[2];
//            int organism=0;
//            try {organism=Integer.parseInt(organismString);} 
//            catch(NumberFormatException ne) {organism=Organism.getTaxonomyID(organismString);}
//            if (organism==0) return "Unknown organism: "+organismString;
//            if (!idResolver.isOrganismSupported(organism)) return "Unsupported organism: "+organismString;
            String buildString=elements[2];
            if (!idResolver.isGenomeBuildSupported(buildString)) return "Unsupported genome build: "+buildString;
        }
    }
    return null;
}

/** 
 * Parses the gene ID entries in the textarea under the "simple" tab,
 * obtains information about TSS, TES etc. for each Gene ID.
 * The method returns "right away" but spawns an asynchroneous background
 * task to connect to a web service which can resolve the Gene IDs.
 */
private void parseGeneIDEntries() {
    String organismName=(String)organismCombobox.getSelectedItem();
    int defaultOrganism=Organism.getTaxonomyID(organismName);
    String defaultFormat=(String)identifierTypeCombobox.getSelectedItem();
    String defaultBuild=(String)genomeBuildCombobox.getSelectedItem();
    String text=geneIdentifiersTextarea.getText();
    text=text.replaceAll(",", "\n");
    String[] lines=text.split("\n");
    ArrayList<GeneIdentifier> list=new ArrayList<GeneIdentifier>();
    for (String line:lines) {
        line=line.trim();
        if (line.isEmpty() || line.startsWith("#")) continue;
        String[] elements=line.split("\t");
        GeneIdentifier geneidentifier=new GeneIdentifier(elements[0], defaultFormat, defaultOrganism, defaultBuild);
        if (elements.length>=2) {
            geneidentifier.format=elements[1];
        } 
        if (elements.length==3) {
            String buildString=elements[2];
            int organism=Organism.getOrganismForGenomeBuild(buildString); // this should not fail (return 0) if checkSimpleTabEntry() returns OK
            geneidentifier.build=buildString;            
            geneidentifier.organism=organism;            
        } 
        list.add(geneidentifier);
        
    }
    resolveIDsInBackground(list);
}

/** 
 * This method is called on the EDT after gene IDs have been successfully resolved and a list
 * of GeneIDMappings is returned. This list must be processed to ensure that all genes are
 * present and the user must decide how to handle possible duplicate matches.
 * Once these issues have been dealt with an AddSequencesTask can be scheduled for execution
 */
private void processResolvedGeneIDs(ArrayList<GeneIdentifier> idlist, ArrayList<GeneIDmapping> resolvedlist) {
    // for (GeneIDmapping mapping:resolvedlist) {gui.debugMessage("Resolved:  "+mapping.geneID+" => "+mapping.geneName+"   "+mapping.chromosome+":"+mapping.TSS+"-"+mapping.TES+" ("+mapping.strand+")");} 
    boolean errors=false;
    ArrayList<Object[]> result=new ArrayList<Object[]>(idlist.size());
    for (GeneIdentifier geneid:idlist) {
       ArrayList<GeneIDmapping> list=getEntriesForID(resolvedlist,geneid.identifier);
       if ( list.isEmpty()) {
           gui.getEngine().logMessage("Unable to find information about "+Organism.getCommonName(geneid.organism).toLowerCase()+" "+geneid.format+" identifier: "+geneid.identifier);
           continue;
       } else if (list.size()>1) {
           list=selectCorrectMappings(list,geneid.identifier);
       } 
       //gui.debugMessage("Size of resolvedlist="+list.size());
       for (GeneIDmapping mapping:list) { // add all those mappings that are left for this ID
           //gui.debugMessage("AFTER:"+mapping.toString());
           Object[] sequenceInfo=new Object[10];  
           sequenceInfo[0]=mapping.geneID;
           sequenceInfo[1]=new Integer(geneid.organism);
           sequenceInfo[2]=geneid.build;
           sequenceInfo[3]=mapping.chromosome;
           // sequenceInfo[4]= sequence start // these will be filled in later based on the users region selections
           // sequenceInfo[5]= sequence end   // these will be filled in later based on the users region selections
           sequenceInfo[6]=mapping.geneName;
           sequenceInfo[7]=new Integer(mapping.TSS);
           sequenceInfo[8]=new Integer(mapping.TES);
           sequenceInfo[9]=new Integer(mapping.strand);
           boolean ok=fillInStartAndEndPositions(sequenceInfo);
           errors=errors || !ok;
           if (ok) result.add(sequenceInfo);
           //gui.debugMessage("SequenceInfo:   "+sequenceInfo[0].toString()+" "+sequenceInfo[1].toString()+" "+sequenceInfo[2].toString()+" "+sequenceInfo[3].toString()+" "+sequenceInfo[4].toString()+" "+sequenceInfo[5].toString()+" "+sequenceInfo[6].toString()+" "+sequenceInfo[7].toString()+" "+sequenceInfo[8].toString());                 
       }
    }
    //gui.debugMessage("Errors="+errors+"    sequences="+result.size());
    if (errors) {
        //errorMessageLabel.setText("Error!");
        return;
    }
    Object[][] data=new Object[result.size()][10];
    data=result.toArray(data);
    Sequence[] sequences=new Sequence[result.size()];
    for (int i=0;i<sequences.length;i++) {
        sequences[i]=new Sequence((String)data[i][0], ((Integer)data[i][1]).intValue(), (String)data[i][2], (String)data[i][3], ((Integer)data[i][4]).intValue(), ((Integer)data[i][5]).intValue(), (String)data[i][6], ((Integer)data[i][7]), ((Integer)data[i][8]), ((Integer)data[i][9]).intValue());
    }
    //AddSequencesTask task=new AddSequencesTask(gui.getEngine(),data);
    AddSequencesTask task=new AddSequencesTask(gui.getEngine(),sequences);
    exitAndLoad(task);    
}
        


/** Goes through a list of GeneIDmapping and returns only those entries that correspond to the given gene id */
private ArrayList<GeneIDmapping> getEntriesForID(ArrayList<GeneIDmapping> list, String id) {
    ArrayList<GeneIDmapping> result=new ArrayList<GeneIDmapping>();
    for (GeneIDmapping entry:list) {
        if (entry.geneID.equals(id)) result.add(entry);
    }
    return result;
}

/** Fills in upstream and downstream coordinates based on user selections and gene orientation */
private boolean fillInStartAndEndPositions(Object[] sequenceInfo) {
    int upstream=((Integer)upstreamSpinner.getValue()).intValue();
    int downstream=((Integer)downstreamSpinner.getValue()).intValue();
    if (upstream>0) upstream--; // to account for direct transition from -1 to +1 at TSS
    if (downstream>0) downstream--; // to account for direct transition from -1 to +1 at TSS
    String anchor=(String)anchorPointComboBox.getSelectedItem();
    int tss=((Integer)sequenceInfo[7]).intValue();
    int tes=((Integer)sequenceInfo[8]).intValue();
    if (anchor.equals("Transcription Start Site")) {
        if (((Integer)sequenceInfo[9]).intValue()==Sequence.DIRECT) {
           sequenceInfo[4]=tss+upstream;
           sequenceInfo[5]=tss+downstream;           
        } else { // Reverse Strand
           sequenceInfo[4]=tss-downstream;
           sequenceInfo[5]=tss-upstream;                
        }
    } else if (anchor.equals("Transcription End Site")) {
        if (((Integer)sequenceInfo[9]).intValue()==Sequence.DIRECT) {
           sequenceInfo[4]=tes+upstream;
           sequenceInfo[5]=tes+downstream;           
        } else { // Reverse Strand
           sequenceInfo[4]=tes-downstream;
           sequenceInfo[5]=tes-upstream;                
        }        
    } else {
        errorMessageLabel.setText("Unsupported anchor site: "+anchor);
        return false;
    }
    return true;
}

/** This method is used whenever multiple mappings are returned for a single gene ID
 * (I am not sure if this is possible but I will account for it anyway). A dialog is
 * displayed so that the user can select the mapping wanted (or even several mappings)
 */
private ArrayList<GeneIDmapping> selectCorrectMappings(ArrayList<GeneIDmapping> list, String id) {
    ArrayList<GeneIDmapping> result=new ArrayList<GeneIDmapping>();
    DefaultListModel model = new DefaultListModel();
    smdList.setModel(model);
    smdMessageLabel.setText("<html>Found several matches for identifier <b>"+id+"</b><br>Please select the correct one (or select several by holding down SHIFT or CRTL)</html>");
    for (int i=0;i<list.size();i++) {
        GeneIDmapping mapping = list.get(i);
        int start=(mapping.TSS<mapping.TES)?mapping.TSS:mapping.TES;
        int end  =(mapping.TSS<mapping.TES)?mapping.TES:mapping.TSS;
        String string=id+"_"+(i+1);
        string=padToLength(string, 20, false);
        string+=mapping.geneName;
        string=padToLength(string, 30, false);
        String chromosome="chr"+mapping.chromosome+":"+start+"-"+end;
        chromosome=padToLength(chromosome, 30, true);
        string+=chromosome+"       ";
        string+=((mapping.strand==Sequence.DIRECT)?"Direct          ":"Reverse          ");              
        model.addElement(string);
    }
    smdList.setSelectedIndex(0);    
    selectMappingDialog.pack();
    int height=selectMappingDialog.getHeight();
    int width=selectMappingDialog.getWidth();
    java.awt.Dimension size=gui.getFrame().getSize();
    selectMappingDialog.setLocation((int)((size.width-width)/2),(int)((size.height-height)/2));         
    selectMappingDialog.setVisible(true);
    // proceeds once the user clicks OK! (modal dialog)
    int[] indices=smdList.getSelectedIndices();
    for (int index:indices) {        
        GeneIDmapping mapping=list.get(index);
        mapping.geneID=mapping.geneID+"_"+(index+1);
        result.add(mapping);
    }
    return result;
}


private String padToLength(String string, int size, boolean front) {
    while (string.length()<size) {
        if (front) string=" "+string;
        else string+=" ";
    }
    return string;
}



/** 
 * Parses the manual entries in the table under the "advanced" tab and returns
 * a 2D Object matrix containing the data needed to specify sequences
 */
private Object[][] parseManualEntryTable() {
    int rows=advancedTableModel.getRowCount();
    int columns=advancedTableModel.getColumnCount();
    Object[][] result=new Object[rows][columns];
    for (int i=0;i<rows;i++) {
       for (int j=0;j<columns;j++) {
        result[i][j]=advancedTableModel.getValueAt(i, j);
       }
    }   
    return result;
}

/** Loads a file (hopefully containing gene IDs and displays it in the textarea */
private void loadFileInBackground(final File file) {
        SwingWorker worker=new SwingWorker<Boolean, Void>() {
            Exception ex=null;
            String text="";
            @Override 
            public Boolean doInBackground() {
                progressbar.setVisible(true);
                progressbar.setIndeterminate(true);            
                BufferedReader inputStream=null;
                try {
                    inputStream=new BufferedReader(new FileReader(file));
                    String line;
                    while((line=inputStream.readLine())!=null) {text+=line+"\n";}
                } catch (IOException e) { 
                    ex=e;
                    return Boolean.FALSE;
                } finally {
                    try {if (inputStream!=null) inputStream.close();} catch (IOException ioe) {System.err.println("SYSTEM ERROR: An error occurred when closing BufferedReader in SequenceInputDialog.loadFileInBackground: "+ioe.getMessage());}
                }
                return Boolean.TRUE;
            }
            @Override
            public void done() { // this method is invoked on the EDT!
                progressbar.setIndeterminate(false); 
                progressbar.setVisible(false);
                if (ex!=null) errorMessageLabel.setText(ex.getMessage());
                else geneIdentifiersTextarea.setText(text);                
            }
        }; // end of SwingWorker class
        worker.execute();
    } 


private void resolveIDsInBackground(final ArrayList<GeneIdentifier> list) {
        SwingWorker worker=new SwingWorker<Boolean, Void>() {
            Exception ex=null;
            ArrayList<GeneIDmapping> resolvedList=null;
            @Override 
            public Boolean doInBackground() {
                progressbar.setVisible(true);
                progressbar.setIndeterminate(true);            
                try {
                    resolvedList=idResolver.resolveIDs(list);
                } catch (Exception e) { 
                    ex=e;
                    return Boolean.FALSE;
                } 
                return Boolean.TRUE;
            }
            @Override
            public void done() { // this method is invoked on the EDT!
                progressbar.setIndeterminate(false); 
                progressbar.setVisible(false);
                if (ex!=null) {
                    errorMessageLabel.setText(ex.getMessage());
                }
                else if (resolvedList==null) errorMessageLabel.setText("Unable to resolve gene IDs");
                else {
                    processResolvedGeneIDs(list, resolvedList);
                }             
            }
        }; // end of SwingWorker class
        worker.execute();
    } 


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton advancedExampleButton;
    private javax.swing.JPanel advancedTab;
    private javax.swing.JLabel anchorLabel;
    private javax.swing.JComboBox anchorPointComboBox;
    private javax.swing.JButton browseButton;
    private javax.swing.JButton cancelButton;
    private javax.swing.JComboBox dnaSequenceSourceCombobox;
    private javax.swing.JTextField dnaTrackNameTextfield;
    private javax.swing.JLabel downstreamLabel;
    private javax.swing.JSpinner downstreamSpinner;
    private javax.swing.JLabel enteridslabel;
    private javax.swing.JLabel errorMessageLabel;
    private javax.swing.ButtonGroup fileOrInput;
    private javax.swing.JLabel fromLabel;
    private javax.swing.JLabel geneIDtypeLabel;
    private javax.swing.JTextArea geneIdentifiersTextarea;
    private javax.swing.JComboBox genomeBuildCombobox;
    private javax.swing.JLabel genomeBuildLabel;
    private javax.swing.JButton helpIconButton;
    private javax.swing.JLabel helpIconLabel;
    private javax.swing.JComboBox identifierTypeCombobox;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JCheckBox loadDNAtrackCheckbox;
    private javax.swing.JButton okButton;
    private javax.swing.JComboBox organismCombobox;
    private javax.swing.JLabel organismLabel;
    private javax.swing.JProgressBar progressbar;
    private javax.swing.JDialog selectMappingDialog;
    private javax.swing.JPanel simpleTab;
    private javax.swing.JPanel smdButtonsPanel;
    private javax.swing.JList smdList;
    private javax.swing.JLabel smdMessageLabel;
    private javax.swing.JPanel smdMiddlePanel;
    private javax.swing.JPanel smdTopPanel;
    private javax.swing.JTabbedPane tabPanel;
    private javax.swing.JScrollPane tableScrollPane;
    private javax.swing.JPanel topPanel;
    private javax.swing.JLabel upstreamLabel;
    private javax.swing.JSpinner upstreamSpinner;
    // End of variables declaration//GEN-END:variables



private class SequenceTableRenderer_Organism extends DefaultTableCellRenderer {

    public SequenceTableRenderer_Organism() {
           super();
           this.setHorizontalAlignment(this.LEFT);              
       }
    public void setValue(Object value) {
           if (value==null) {setText("");}
           else if (value instanceof Integer) {
               if (((Integer)value).intValue()==0) setForeground(java.awt.Color.RED);
               else setForeground(java.awt.Color.BLACK);
               setText(Organism.getCommonName(((Integer)value).intValue()));
           }  else {
               setForeground(java.awt.Color.RED);
               setText("* * *");
           }
       }
}// end class SequenceTableRenderer_Organism

private class SequenceTableRenderer_RightAlign extends DefaultTableCellRenderer {
    public SequenceTableRenderer_RightAlign() {
           super();
           this.setHorizontalAlignment(this.RIGHT);              
       }
    public void setValue(Object value) {
           if (value!=null)setText(value.toString());
           else setText("");
       }
}// end class SequenceTableRenderer_RightAlign

private class SequenceTableRenderer_number extends DefaultTableCellRenderer {
    public SequenceTableRenderer_number() {
           super();
           this.setHorizontalAlignment(this.RIGHT);              
       }
    public void setValue(Object value) {
           if (value==null) {setText("");}
           else if (value instanceof Integer) {
                    setForeground(java.awt.Color.BLACK);
                    setText(value.toString());
               }  else {
                    setForeground(java.awt.Color.RED);
                    setText("* * *");                   
               }
           
       }
}// end class SequenceTableRenderer_number

private class SequenceTableRenderer_Orientation extends DefaultTableCellRenderer {
    public SequenceTableRenderer_Orientation() {
           super();
           this.setHorizontalAlignment(this.LEFT);              
       }
    public void setValue(Object value) {
           if (value==null || value.toString().isEmpty()) {setText("");} 
           else if (value instanceof Integer) {
               setForeground(java.awt.Color.BLACK);
               if (((Integer)value).intValue()==Sequence.REVERSE) setText("Reverse");
               else setText("Direct");
           } else {
               setForeground(java.awt.Color.RED);
               setText("* * *");
           }
       }
}// end class SequenceTableRenderer_RightAlign


private class TableChangeListener implements TableModelListener {
        public void tableChanged(TableModelEvent e) {
            if (e.getType()==TableModelEvent.INSERT || e.getType()==TableModelEvent.DELETE) return;
            int col=e.getColumn();
            int row=e.getFirstRow();
            DefaultTableModel model=(DefaultTableModel)e.getSource();
            Object value=model.getValueAt(row, col);
            String colName=model.getColumnName(col);
            //System.err.println("Inserting value at "+row+":"+colName+" = "+value);
            if (colName.equals("Orientation")) {
                if (value==null) {}
                else if (value instanceof Integer) {
                   int intval=((Integer)value).intValue();
                   if (intval==1 || intval==-1) return;
                   else if (intval>=0) model.setValueAt(new Integer(1), row, col);
                   else model.setValueAt(new Integer(-1), row, col);
                } else if (value instanceof String) {
                    String strval=(String)value;
                    if (strval.equalsIgnoreCase("Reverse") || strval.equals("-") || strval.equals("-1")) model.setValueAt(new Integer(-1), row, col);
                    else model.setValueAt(new Integer(1), row, col);
                } else model.setValueAt(new Integer(1), row, col);
            } else if (colName.equals("Organism")) {
                if (value instanceof String) {
                    String name=(String)value;
                    if (name.startsWith("NCBI:")) name=name.substring(5);
                    int taxid=Organism.getTaxonomyID(name);
                    if (taxid!=0) model.setValueAt(new Integer(taxid), row, col);
                    else try {
                        int val=Integer.parseInt(name);
                        model.setValueAt(new Integer(val), row, col);
                    } catch (NumberFormatException ne) {model.setValueAt(new Integer(0), row, col);}
                }
            } else if (colName.equals("Chromosome")) {
                if (value!=null && !(value instanceof String)) {
                    model.setValueAt(value.toString(), row, col);
                }
            } else if (colName.equals("Gene name")) {
                if (value instanceof String && ((String)value).equals("null")) {
                    model.setValueAt(null, row, col);
                }
            } else if (colName.equals("Start") || colName.equals("End") || colName.equals("Gene TSS") || colName.equals("Gene TES")) {
                //if (value==null) System.err.println(colName+" : "+null);
                //else System.err.println(colName+" : "+value.getClass().toString()+" : "+value.toString());
                if (value instanceof String) {
                    String strval=((String)value).trim();
                    try {
                        int val=Integer.parseInt(strval);
                        model.setValueAt(new Integer(val), row, col);
                    } catch (NumberFormatException ne) {model.setValueAt(null, row, col);}
                } // else model.setValueAt("", row, col);
            }
        }

}

private class CellEditor_Orientation extends AbstractCellEditor implements TableCellEditor {
        JComboBox component=new JComboBox(new String[]{"Direct","Reverse"});
        public Object getCellEditorValue() {
            int selectedIndex=component.getSelectedIndex();
            if (selectedIndex==-1) return null;
            else if (selectedIndex==1) return new Integer(-1);
            else return new Integer(1);
        }

        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
            if (value!=null) {
                if (value.toString().equals("-1")) component.setSelectedIndex(1);
                else component.setSelectedIndex(0);
            }
            return component;
        }
    
}

private class CellEditor_Organism extends AbstractCellEditor implements TableCellEditor {   
        JComboBox component;
    
        public CellEditor_Organism() {
             component=new JComboBox(Organism.getSupportedOrganisms());
             component.setEditable(true);
        }
        
        public Object getCellEditorValue() {
            String name=(String)component.getSelectedItem();
            if (name.startsWith("NCBI:")) name=name.substring(5);
            int taxid=Organism.getTaxonomyID(name);
            if (taxid!=0) return new Integer(taxid);
            try {
                int val=Integer.parseInt(name);
                return new Integer(val);                
            } catch(NumberFormatException e) {}
            return new Integer(0);
        }

        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
            if (value==null) component.setSelectedIndex(0);
            else {
                if (value instanceof Integer) component.setSelectedItem(Organism.getCommonName(((Integer)value).intValue()));
                else {
                    try {
                        int organism=Integer.parseInt((value.toString()));                        
                        component.setSelectedItem(Organism.getCommonName(organism));
                    }
                    catch (NumberFormatException ne) {component.setSelectedIndex(0);}
                }
            }
            return component;
        }    
}

/*
 *
 */
private static String[]exampleSequences=new String[] {
    "NTNG1\tHGNC Symbol\thg18\n56475\tEntrez Gene\thg18\nENSG00000111249\tEnsembl Gene\thg18\nENSG00000187664\tEnsembl Gene\thg18\nENSG00000196358\tEnsembl Gene\thg18\n",
    "#Human smooth muscle genes markers\nENSG00000035403\nENSG00000100345\nENSG00000107796\nENSG00000130176\nENSG00000133026\nENSG00000149591\nENSG00000154553\nENSG00000163017\nENSG00000163431\nENSG00000175084",
    "#Human genes expressed in liver tissue\nENSG00000017427\nENSG00000115718\nENSG00000116833\nENSG00000118271\nENSG00000129965\nENSG00000131482\nENSG00000163631\nENSG00000167910\nENSG00000173531"
};
    
}



