
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Font;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * This class is an implementation of Icon that can draw different
 * simple icons to be used in various circumstances
 * 
 * @author kjetikl
 */
public class SimpleDataPanelIcon implements Icon {
    public static final int UNKNOWN_ICON=0;
    public static final int BLANK_ICON=1;
    public static final int SEQUENCE_ICON=2;
    public static final int SEQUENCE_ICON_BASES=3;
    public static final int REGION_ICON=4;
    public static final int REGION_MULTICOLOR_ICON=5;
    public static final int REGION_GRADIENT_ICON=6;
    public static final int NUMERIC_TRACK_GRAPH_ICON=7;
    public static final int NUMERIC_TRACK_GRADIENT_ICON=8;
    public static final int HIDDEN_ICON=9;
    public static final int COLOR_ICON=10;
    public static final int SEQUENCE_COLLECTION_ICON=11;
    public static final int MARKOV_MODEL_ICON=12;
    public static final int NUMERIC_VARIABLE_ICON=13;
    public static final int STRING_VARIABLE_ICON=14;
    public static final int STRING_COLLECTION_ICON=15;
    public static final int COLOR_ICON_SELECTED=16;
    public static final int NUMERIC_MAP_VARIABLE_ICON=17;


    
    
    public static final int NO_BORDER=0;
    public static final int SIMPLE_BORDER=1;
    public static final int BEVEL_BORDER=2;
    public static final Font largeFont=new Font(Font.SANS_SERIF,Font.PLAIN,18);
    public static final Font smallFont=new Font(Font.SERIF,Font.PLAIN,12);
    public static final Font numericVariableFont=new Font(Font.SERIF,Font.ITALIC+Font.BOLD,24);
    public static final Font markovFont=new Font(Font.SERIF,Font.ITALIC,16);
    public static final Font markovSubscriptFont=new Font(Font.SERIF,Font.ITALIC,10);

    public static final ImageIcon bgicon=new ImageIcon(Toolkit.getDefaultToolkit().getImage(SimpleDataPanelIcon.class.getResource("resources/icons/bgmodel.gif")));    
    
    private int height=0;
    private int width=0;
    private int type=0;
    private Color foregroundcolor=Color.red;
    private Color backgroundcolor=Color.white;
    private Color bordercolor=Color.black;
    private int drawborder=NO_BORDER;
    private VisualizationSettings settings;
    
    
    
    
    public SimpleDataPanelIcon(int height, int width, int type, VisualizationSettings settings) {
        this.height=height;
        this.width=width;
        this.type=type;
        this.settings=settings;
    }
    
    public SimpleDataPanelIcon(int height, int width, int type, int bordertype, VisualizationSettings settings) {
        this.height=height;
        this.width=width;
        this.type=type;
        this.drawborder=bordertype;
        this.settings=settings;
    }
    
    public SimpleDataPanelIcon(int height, int width, Color color) {
        this.height=height;
        this.width=width;
        this.type=COLOR_ICON;
        this.drawborder=SIMPLE_BORDER;
        this.settings=null;
        this.foregroundcolor=color;
        this.backgroundcolor=Color.BLACK;
    }
    
    public int getIconHeight() {
        return height;
    }

    public int getIconWidth() {
        return width;
    }
    
    public void setIconWidth(int width) {
        this.width=width;
    }
    
    public void setIconHeight(int height) {
        this.height=height;
    }
    
    public void setIconType(int type) {
        this.type=type;
    }
    
    public void setForegroundColor(Color color) {
        this.foregroundcolor=color;
    }
    
    public void setBackgroundColor(Color color) {
        this.backgroundcolor=color;
    }
    
    public void setBorderColor(Color color) {
        this.bordercolor=color;
    }

    public void drawBorder(int borderType) {
        this.drawborder=borderType;
    }
    
    public void drawBorder(boolean drawborder) {
        if (drawborder) this.drawborder=SIMPLE_BORDER;
        else this.drawborder=NO_BORDER;
    }
    
    public void paintIcon(Component c, Graphics g, int x, int y) {
           
             if (type==SEQUENCE_ICON) drawSequenceIcon(g, x, y, false);
        else if (type==SEQUENCE_ICON_BASES) drawSequenceIcon(g, x, y, true);
        else if (type==REGION_ICON) drawRegionIcon(g, x, y);
        else if (type==REGION_MULTICOLOR_ICON) drawRegionMulticolorIcon(g, x, y);
        else if (type==REGION_GRADIENT_ICON) drawRegionGradientIcon(g, x, y);
        else if (type==NUMERIC_TRACK_GRAPH_ICON) drawNumericTrackGraphIcon(g, x, y);  
        else if (type==NUMERIC_TRACK_GRADIENT_ICON) drawNumericTrackGradientIcon(g, x, y);  
        else if (type==HIDDEN_ICON) drawHiddenIcon(g, x, y);  
        else if (type==COLOR_ICON) drawColorIcon(g, x, y);  
        else if (type==COLOR_ICON_SELECTED) drawColorSelectedIcon(g, x, y);  
        else if (type==SEQUENCE_COLLECTION_ICON) drawSequenceCollectionIcon(g, x, y);  
        else if (type==MARKOV_MODEL_ICON) bgicon.paintIcon(c, g, x, y);//drawMarkovModelIcon(g, x, y);  
        else if (type==UNKNOWN_ICON) drawUnknownIcon(g, x, y);  
        else if (type==NUMERIC_VARIABLE_ICON) drawNumericVariableIcon(g, x, y);  
        else if (type==STRING_VARIABLE_ICON) drawStringVariableIcon(g, x, y);  
        else if (type==STRING_COLLECTION_ICON) drawStringCollectionIcon(g, x, y);  
        else if (type==NUMERIC_MAP_VARIABLE_ICON) drawNumericMapVariableIcon(g, x, y);  
             
        if (drawborder==SIMPLE_BORDER) drawSimpleIconBorder(g, x, y);
        else if (drawborder==BEVEL_BORDER) drawBevelIconBorder(g, x, y);
    }
    
    private void drawSequenceIcon(Graphics g, int x, int y, boolean colorBases) {
         int xoffset=x;
         int yoffset=y+5;
         g.setColor(foregroundcolor);
         //g.setFont(smallfont);
         //g.drawString("ACGT", x, y+14);
         // - A -
         g.drawLine(xoffset, yoffset+1, xoffset, yoffset+6);     // vertical left
         g.drawLine(xoffset+3, yoffset+1, xoffset+3, yoffset+6); // vertical right
         g.drawLine(xoffset+1, yoffset, xoffset+2, yoffset);     // top bar
         g.drawLine(xoffset+1, yoffset+3, xoffset+2, yoffset+3); // middle bar
             
         // - C -
         g.drawLine(xoffset+5, yoffset+1, xoffset+5, yoffset+5);  // vertical
         g.drawLine(xoffset+6, yoffset, xoffset+7, yoffset);      // top horizontal
         g.drawLine(xoffset+6, yoffset+6, xoffset+7, yoffset+6);  // bottom horizontal
         g.drawLine(xoffset+8, yoffset+1, xoffset+8, yoffset+1);  // top curve
         g.drawLine(xoffset+8, yoffset+5, xoffset+8, yoffset+5);  // bottom curve
         
         // - G -
         g.drawLine(xoffset+10, yoffset+1, xoffset+10, yoffset+5); // vertical
         g.drawLine(xoffset+11, yoffset, xoffset+12, yoffset);     // top horizontal
         g.drawLine(xoffset+11, yoffset+6, xoffset+12, yoffset+6); // bottom horizontal
         g.drawLine(xoffset+13, yoffset+1, xoffset+13, yoffset+1); // top curve
         g.drawLine(xoffset+13, yoffset+5, xoffset+13, yoffset+4); // bottom curve
         g.drawLine(xoffset+12, yoffset+3, xoffset+13, yoffset+3); // middle bar

         // - T -
         g.drawLine(xoffset+17, yoffset, xoffset+17, yoffset+7); // vertical
         g.drawLine(xoffset+15, yoffset, xoffset+19, yoffset);     // top horizontal
                  
         if (colorBases) {
             int yoff=yoffset+9;
             if (settings!=null) g.setColor(settings.getBaseColor('A')); else g.setColor(Color.GREEN);
             g.fillRect(xoffset, yoff, 5, 5);
             if (settings!=null) g.setColor(settings.getBaseColor('C')); else g.setColor(Color.BLUE);
             g.fillRect(xoffset+5, yoff, 5, 5);
             if (settings!=null) g.setColor(settings.getBaseColor('G')); else g.setColor(Color.YELLOW);
             g.fillRect(xoffset+10, yoff, 5, 5);
             if (settings!=null) g.setColor(settings.getBaseColor('T')); else g.setColor(Color.RED);
             g.fillRect(xoffset+15, yoff, 5, 5);
         }
    }
    
    private void drawRegionIcon(Graphics g, int x, int y) {
         if (backgroundcolor!=null) {
           g.setColor(backgroundcolor);
           g.fillRect(x, y, width, height);
           if (backgroundcolor.equals(Color.black)) g.setColor(Color.white);
           else g.setColor(Color.black);
         } else g.setColor(Color.black);
         int baseline=y+10;
         g.drawLine(x, baseline, x+width, baseline);
         g.setColor(foregroundcolor);
         g.fillRect(x+4, baseline-4, 5, 9);
         g.fillRect(x+11, baseline-4, 7, 9);
    }
    
    private void drawRegionMulticolorIcon(Graphics g, int x, int y) {
         if (backgroundcolor!=null) {
           g.setColor(backgroundcolor);
           g.fillRect(x, y, width, height);
           if (backgroundcolor.equals(Color.black)) g.setColor(Color.white);
           else g.setColor(Color.black);
         } else g.setColor(Color.black);
         int baseline=y+10;
         g.drawLine(x, baseline, x+width, baseline);
         g.fillRect(x+4, baseline-4, 5, 9);
         g.fillRect(x+11, baseline-4, 7, 9);
         g.setColor(settings.getColorFromIndex(0));
         g.drawLine(x+5, baseline-3, x+5, baseline+3);
         g.setColor(settings.getColorFromIndex(6));
         g.drawLine(x+6, baseline-3, x+6, baseline+3);
         g.setColor(settings.getColorFromIndex(1));
         g.drawLine(x+7, baseline-3, x+7, baseline+3);

         g.setColor(settings.getColorFromIndex(4));
         g.drawLine(x+12, baseline-3, x+12, baseline+3);
         g.setColor(settings.getColorFromIndex(3));
         g.drawLine(x+13, baseline-3, x+13, baseline+3);
         g.setColor(settings.getColorFromIndex(7));
         g.drawLine(x+14, baseline-3, x+14, baseline+3);
         g.setColor(settings.getColorFromIndex(2));
         g.drawLine(x+15, baseline-3, x+15, baseline+3);
         g.setColor(settings.getColorFromIndex(5));
         g.drawLine(x+16, baseline-3, x+16, baseline+3);

    }
    


    
        
    
    private void drawRegionGradientIcon(Graphics g, int x, int y) {
         g.setColor(backgroundcolor);
         g.fillRect(x, y, width, height);
         if (backgroundcolor.equals(Color.black)) g.setColor(Color.white);
         else g.setColor(Color.black);
         int baseline=y+10;
         g.drawLine(x, baseline, x+width, baseline);
         g.setColor(foregroundcolor);
         g.fillRect(x+4, baseline-4, 5, 9);
         Color gradientcolor=new Color(foregroundcolor.getRed(),foregroundcolor.getGreen(),foregroundcolor.getBlue(),100);
         g.setColor(gradientcolor);
         g.fillRect(x+11, baseline-4, 7, 9);
    }
    
    private void drawNumericTrackGraphIcon(Graphics g, int x, int y) {
         if (backgroundcolor!=null) {
           g.setColor(backgroundcolor);
           g.fillRect(x, y, width, height);
         }
         int baseline=y+14;
         g.setColor(foregroundcolor);
         int[] lineheights=new int[]{0,1,2,1,4,6,5,8,4,8,9,7,5,2,0,-2,-4,-3,0,2,4,1,2};
         for (int i=0;i<width;i++) {
             g.drawLine(x+i, baseline-1, x+i, baseline-lineheights[i]);
         }
         if (backgroundcolor!=null && backgroundcolor.equals(Color.black)) g.setColor(Color.white);
         else g.setColor(Color.black);
         g.drawLine(x, baseline, x+width, baseline);
    }
 
    private void drawNumericTrackGradientIcon(Graphics g, int x, int y) {
         g.setColor(backgroundcolor);
         g.fillRect(x, y, width, height);    
         int baseline=y+10;
         g.setColor(foregroundcolor);
         float[] rgb = foregroundcolor.getRGBComponents(null);
         Color gradients[]=new Color[6];
         gradients[0]=new Color(rgb[0],rgb[1],rgb[2],0.0f);  
         gradients[1]=new Color(rgb[0],rgb[1],rgb[2],0.2f);
         gradients[2]=new Color(rgb[0],rgb[1],rgb[2],0.4f);
         gradients[3]=new Color(rgb[0],rgb[1],rgb[2],0.6f);
         gradients[4]=new Color(rgb[0],rgb[1],rgb[2],0.8f);
         gradients[5]=foregroundcolor;
         int[] use=new int[]{0,3,2,1,3,5,3,0,1,0,5,4,3,2,4,0,3,0,1,2,4,2,3,5,0,0,0,0};
         for (int i=0;i<width;i++) {
             g.setColor(gradients[use[i]]);
             g.drawLine(x+i, baseline-6, x+i, baseline+6);
         }
         if (backgroundcolor.equals(Color.black)) g.setColor(Color.white);
         else g.setColor(Color.black);
         g.drawLine(x, baseline, x+width, baseline);
    }
    
    private void drawHiddenIcon(Graphics g, int x, int y) {
         g.setColor(new Color(0.8f,0.8f,0.8f));
         g.fillRect(x, y, width, height);  
         g.setColor(new Color(0.9f,0.9f,0.9f));
         g.setColor(Color.white);
         g.drawLine(x, y, x+width, y+height);
         g.drawLine(x, y+height, x+width, y);
         
         g.drawLine(x+1, y, x+width, y+height-1);
         g.drawLine(x+1, y+height, x+width, y+1);
         
         g.drawLine(x, y+1, x+width-1, y+height);
         g.drawLine(x, y+height-1, x+width-1, y);
    }
    
    
    private void drawSequenceCollectionIcon(Graphics g, int x, int y) {
         g.setColor(Color.BLACK);
         g.drawRect(x, y+2, 17, 2);  
         g.drawRect(x, y+6, 11, 2);  
         g.drawRect(x, y+10, 15, 2);  
         g.drawRect(x, y+14, 13, 2);  
         
         g.setColor(new Color(50,50,255)); // light Blue
         g.drawLine(x+1, y+3, x+16, y+3);
         g.drawLine(x+1, y+7, x+10, y+7);
         g.drawLine(x+1, y+11, x+14, y+11);
         g.drawLine(x+1, y+15, x+12, y+15);

         g.setColor(Color.RED);
         g.drawLine(x+1, y+3, x+2, y+3);
         g.drawLine(x+13, y+3, x+14, y+3);
         g.drawLine(x+5, y+7, x+6, y+7);         
         g.drawLine(x+3, y+11, x+4, y+11);
         g.drawLine(x+11, y+11, x+12, y+11);         
         g.drawLine(x+3, y+15, x+4, y+15);
         g.drawLine(x+9, y+15, x+10, y+15);
         
         g.setColor(Color.YELLOW);
         g.drawLine(x+3, y+3, x+4, y+3);
         g.drawLine(x+11, y+3, x+12, y+3);
         g.drawLine(x+1, y+7, x+2, y+7);         
         g.drawLine(x+9, y+7, x+10, y+7);
         g.drawLine(x+7, y+11, x+8, y+11);         
         g.drawLine(x+11, y+15, x+12, y+15);

         g.setColor(Color.GREEN);
         g.drawLine(x+7, y+3, x+8, y+3);
         g.drawLine(x+15, y+3, x+16, y+3);
         g.drawLine(x+3, y+7, x+4, y+7);         
         g.drawLine(x+5, y+11, x+6, y+11);
         g.drawLine(x+13, y+11, x+14, y+11);         
         g.drawLine(x+1, y+15, x+2, y+15);
         g.drawLine(x+7, y+15, x+8, y+15);

    }
    
    private void drawSequencePartitionIcon(Graphics g, int x, int y) {
         g.setColor(Color.BLACK);
         g.drawRect(x, y+2, 17, 2);  
         g.drawRect(x, y+6, 11, 2);  
         g.drawRect(x, y+10, 15, 2);  
         g.drawRect(x, y+14, 13, 2);  

         g.setColor(Color.RED);
         g.drawLine(x+1, y+3, x+16, y+3);
         g.drawLine(x+1, y+7, x+10, y+7);
         
         g.setColor(Color.GREEN);
         g.drawLine(x+1, y+11, x+14, y+11);
         g.drawLine(x+1, y+15, x+12, y+15);
  
    }    
    
    
    
    
    
    
    private void drawColorIcon(Graphics g, int x, int y) {
         g.setColor(foregroundcolor);
         g.fillRect(x, y, width, height);  
    }
    
    private void drawColorSelectedIcon(Graphics g, int x, int y) {
         g.setColor(foregroundcolor);
         g.fillRect(x, y, width, height);  
         g.setColor(Color.BLACK);
         g.drawLine(x, y, x+width, y+height);  
         g.drawLine(x+width,y, x, y+height);  
    }
    
    
    private void drawUnknownIcon(Graphics g, int x, int y) {
         Font f=g.getFont();
         g.setColor(Color.BLACK);
         g.setFont(largeFont);
         g.drawString("?", x+5, y+height-4); 
         g.setFont(f);
    }
   
    
    
    private void drawMarkovModelIcon(Graphics g, int x, int y) {
         Font f=g.getFont(); 
         g.setColor(Color.BLACK);
         Graphics2D g2=(Graphics2D)g;
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
         g.setFont(markovFont);
         g.drawString("M", x, y+height-5);
         g.setFont(markovSubscriptFont);
         g.drawString("n", x+13, y+height-3);
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
         g.setFont(f);
    }
    
    private void drawNumericVariableIcon(Graphics g, int x, int y) {
         Font f=g.getFont(); 
         g.setColor(Color.BLACK);
         g.setFont(numericVariableFont);
         Graphics2D g2=(Graphics2D)g;
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
         g.drawString("x", x+4, y+height-5);
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
         g.setFont(f);
    }
    private void drawNumericMapVariableIcon(Graphics g, int x, int y) {
         Font f=g.getFont(); 
         g.setColor(Color.BLACK);
         g.setFont(numericVariableFont);
         Graphics2D g2=(Graphics2D)g;
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
         g.drawString("x", x+2, y+height-5);
         g.setFont(markovSubscriptFont);
         g.drawString("i", x+14, y+height-3);
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
         g.setFont(f);
    }
    
   private void drawStringVariableIcon(Graphics g, int x, int y) {
         Font f=g.getFont();
         g.setColor(Color.BLACK);
         g.setFont(smallFont);
         Graphics2D g2=(Graphics2D)g;
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
         g.drawString("abc", x+2, y+height-5);
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
         g.setFont(f);
    }
    
   private void drawStringCollectionIcon(Graphics g, int x, int y) {
         Font f=g.getFont();
         g.setColor(Color.BLACK);
         g.setFont(smallFont);
         Graphics2D g2=(Graphics2D)g;
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
         g.drawString("abc", x+2, y+height-11);
         g.drawString("def", x+2, y+height-2);
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
         g.setFont(f);
    }
    
    
    
     
    
    private void drawBevelIconBorder(Graphics g, int x, int y) {
         g.setColor(Color.darkGray);                 
         g.drawLine(x, y+height, x+width-1, y+height);
         g.drawLine(x+width-1, y, x+width-1, y+height);
         
         g.setColor(Color.lightGray);     
         g.drawLine(x, y, x+width-1, y);
         g.drawLine(x, y, x, y+height);
    }
    
    private void drawSimpleIconBorder(Graphics g, int x, int y) {
         g.setColor(bordercolor);                 
         g.drawRect(x, y, width, height);
    }
  
    
}