/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import javax.swing.JLabel;
import javax.swing.JToolTip;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Motif;

/**
 * This tooltip is used when mouse is over Motifs in the MotifsPanel 
 * @author kjetikl
 */
public class SingleMotifTooltip extends JToolTip {
    private Font logofont;
    private char[] bases;
    private int fontheight=25; // if you change this size you must also change the ascentCorrection below accordingly (but I am not sure about the relationship...)
    private int ascentCorrection=6; // this is a needed hack because the ascent returned by the FontMetrics is not really the same height as that of a capital letter (even though that was stated in the documentation)
    private int lineheight=20;
    private int xoffset=4;
    private Color[] basecolors;
    private PriorsEditorEngine engine;
    
    
    public SingleMotifTooltip(VisualizationSettings settings) {
        logofont=new Font(Font.SANS_SERIF,Font.BOLD,fontheight);
        bases=new char[]{'A','C','G','T'};
        engine=settings.getEngine();
        basecolors=new Color[]{settings.getBaseColor('A'),settings.getBaseColor('C'),settings.getBaseColor('G'),settings.getBaseColor('T')};        
    }

    private String getDescription(Motif motif) {
        String longname=motif.getLongName();
        if (longname==null || longname.isEmpty()) longname=motif.getShortName(); 
        if (longname==null) longname="";
        if (!longname.equals("")) longname=" - "+longname;
        return "<html><nobr><b>"+motif.getName()+"</b>"+longname+"</nobr><br>"+motif.getLength()+"&nbsp;bp&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;IC:&nbsp;"+motif.getICcontent()+"</html>";           
    }   
            
    @Override
    @SuppressWarnings("unchecked")
    protected void paintComponent(Graphics g) {   
       Object data=engine.getDataItem(getTipText());
       if (data!=null && data instanceof Motif) {  
             String oldtext=getTipText();
             setTipText(getDescription((Motif)data));
             super.paintComponent(g);    
             Dimension dim=getPreferredSize();
             setTipText(oldtext);             
             paintSequenceLogo(g, (Motif)data, xoffset, dim.height, basecolors);
       } else super.paintComponent(g);                       
    }
    
       
    @Override
    public Dimension getPreferredSize() {
        Dimension dim=super.getPreferredSize();
        Object data=engine.getDataItem(getTipText());
        if (data==null || !(data instanceof Motif)) return dim;
        //String description=getDescription((Motif)data);
        //int width=getFontMetrics(getFont()).stringWidth(description)+4; // 4 is just a margin
        JLabel testlabel=new JLabel(getDescription((Motif)data));
        testlabel.setFont(getFont());
        Dimension labeldim=testlabel.getPreferredSize();        
        int width=labeldim.width+6;
        int charwidth=getFontMetrics(logofont).charWidth('G');
        int logowidth=charwidth*((Motif)data).getLength()+10; // 20 is just a margin
        return new Dimension((width>logowidth)?width:logowidth,labeldim.height+lineheight+14); // 12 is just a margin
    }

    
    private int paintSequenceLogo(Graphics graphics, Motif motif, int xoffset, int yoffset, Color[] basecolors) {
        Font oldfont=graphics.getFont();
        float[][] matrix=motif.getMatrix();
        if (motif.getMatrixType()==Motif.LOG_ODDS_MATRIX) matrix=motif.getMatrixAsFrequencyMatrix();        
        float[] letterXoffset=new float[4];                
        if (matrix==null) return 0;
        Graphics2D g = (Graphics2D)graphics;
        FontMetrics metrics=g.getFontMetrics(logofont);
        int ascent=metrics.getAscent()-ascentCorrection;
        int widthA=metrics.charWidth('A');     
        int widthC=metrics.charWidth('C');     
        int widthG=metrics.charWidth('G');     
        int widthT=metrics.charWidth('T');
        letterXoffset[0]=(widthG-widthA)/2;
        letterXoffset[1]=(widthG-widthC)/2;
        letterXoffset[3]=(widthG-widthT)/2;
        int logowidth=matrix.length*widthG;
        int fullwidth=motif.getLength()*widthG;
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g.setFont(logofont);
        g.setColor(java.awt.Color.RED);
        AffineTransform restore=g.getTransform();
        AffineTransform save=g.getTransform();           
        if (logowidth>getWidth()) save.scale((float)getWidth()/(float)logowidth, 1); // scale X-direction so that logo fits irrespective of size
        AffineTransform scaleY = new AffineTransform();
        g.setColor(Color.DARK_GRAY);
        g.fillRect(xoffset, yoffset, fullwidth, ascent);
        g.setColor(Color.WHITE);
        g.fillRect(xoffset, yoffset, logowidth, ascent);
        float xpos=xoffset;
        for (int i=0;i<matrix.length;i++) {
            float[] counts=new float[]{matrix[i][0],matrix[i][1],matrix[i][2],matrix[i][3]};
            float total=counts[0]+counts[1]+counts[2]+counts[3];
            float ic=Motif.calculateColumnIC(counts[0], counts[1], counts[2], counts[3], false);
            int[] sorted=new int[4]; // sorted in ascending order. Values are base-indices (i.e. 0=>A, 1=>C, 2=>G, 3=>T)
            int indexA=0, indexC=0, indexG=0, indexT=0;
            if (counts[0]>=counts[1]) indexA++;
            if (counts[0]>=counts[2]) indexA++;
            if (counts[0]>=counts[3]) indexA++;                
            if (counts[1]>counts[0]) indexC++;
            if (counts[1]>=counts[2]) indexC++;
            if (counts[1]>=counts[3]) indexC++;               
            if (counts[2]>counts[0]) indexG++;
            if (counts[2]>counts[1]) indexG++;
            if (counts[2]>=counts[3]) indexG++;                
            if (counts[3]>counts[0]) indexT++;
            if (counts[3]>counts[1]) indexT++;
            if (counts[3]>counts[2]) indexT++;
            sorted[indexA]=0;
            sorted[indexC]=1;
            sorted[indexG]=2;
            sorted[indexT]=3;
            float currentYoffset=yoffset+ascent-(ascent*ic/2f);
            for (int j=3;j>=0;j--) { // draws letters from top to bottom (most frequent first)
                int base=sorted[j];
                float fraction=counts[base]/total;
                scaleY.setTransform(save);
                scaleY.translate(letterXoffset[base],currentYoffset); // translated in 1-to-1 scale                    
                scaleY.scale(1,ic/2f); // scale by IC-content in position                   
                scaleY.scale(1, fraction);   
                g.setColor(basecolors[base]);               
                g.setTransform(scaleY);
                g.drawString(""+bases[base], xpos, ascent); // draw all letters at same position and use transform to place them correctly
                currentYoffset+=ascent*fraction*(ic/2f);
            } // end for each base  
            xpos+=widthG; // 
        } // end for each position
        g.setTransform(save);
        g.setColor(Color.BLACK);
        g.drawRect(xoffset,yoffset,fullwidth,ascent);
        g.setTransform(restore);
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
        g.setFont(oldfont);
        return xoffset+fullwidth+1;
    }     
    
}
