/*
 * UCSCtrackDialog.java
 *
 * Created on 5. november 2009, 10:29
 */

package priorseditor.gui;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;
import org.jdesktop.application.Action;
import priorseditor.engine.data.Organism;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.datatracks.DataTrack;
import priorseditor.engine.datatracks.DataSource;
import priorseditor.engine.datatracks.DataSource_http_GET;


/**
 *
 * @author  kjetikl
 */
public class UCSCtrackDialog extends javax.swing.JDialog {
    private String oldorganism="unknown";
    private String oldbuild="unknown";
    private String oldgroup="unknown";
    private String oldtrack="unknown";
    private String oldtable="unknown";
    private boolean OKpressed=false;
    private boolean everythingOK=true;
    private String baseURL="http://genome.ucsc.edu/cgi-bin/hgTables?";
    private static HashMap<String,String[][]> currentConfiguration=null; // this is static to enable 'next session' to start at same point as last time
    private SimpleDataPanelIcon regionIcon;
    private SimpleDataPanelIcon numericIcon;
    private SimpleDataPanelIcon unknownIcon;
    private DataTrack parentDataTrack=null;
    
    /** Creates new UCSCtrackDialog 
     * @parent A parent frame
     * @datatrack A parent datatrack (optional). If this is set then the the trackname will not be editable but be based on this track. Also if the selected source is incompatible with the track, a warning message will be displayed
     */
    public UCSCtrackDialog(java.awt.Frame parent, DataTrack datatrack) {
        super(parent, true);
        parentDataTrack=datatrack;
        numericIcon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.NUMERIC_TRACK_GRAPH_ICON,null);
        numericIcon.setForegroundColor(java.awt.Color.BLUE);
        numericIcon.setBackgroundColor(null);
        regionIcon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.REGION_ICON,null);
        regionIcon.setForegroundColor(java.awt.Color.GREEN);
        regionIcon.setBackgroundColor(null);
        unknownIcon=new SimpleDataPanelIcon(20, 20, SimpleDataPanelIcon.UNKNOWN_ICON,null);
        unknownIcon.setForegroundColor(java.awt.Color.BLACK);
        initComponents();       
        if (parentDataTrack==null) setTitle("Select Track from UCSC Genome Browser");
        else setTitle("Add source for "+parentDataTrack.getName()+" from UCSC Genome Browser");
        getRootPane().setDefaultButton(okButton);
        organismCombobox.setModel(new DefaultComboBoxModel(Organism.getSupportedOrganisms()));
        if (currentConfiguration==null) organismCombobox.setSelectedIndex(0);
        else {
            updateMenusWithNewData("all");
        }

        if (parentDataTrack!=null) {
            nameTextField.setText(parentDataTrack.getName());
            nameTextField.setEditable(false);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jPanel1 = new javax.swing.JPanel();
        cancelButton = new javax.swing.JButton();
        okButton = new javax.swing.JButton();
        progressbar = new javax.swing.JProgressBar();
        jLabel1 = new javax.swing.JLabel();
        jPanel3 = new javax.swing.JPanel();
        mainPanel = new javax.swing.JPanel();
        gridPanel = new javax.swing.JPanel();
        organismLabel = new javax.swing.JLabel();
        organismCombobox = new javax.swing.JComboBox();
        buildLabel = new javax.swing.JLabel();
        buildCombobox = new javax.swing.JComboBox();
        groupLabel = new javax.swing.JLabel();
        groupCombobox = new javax.swing.JComboBox();
        trackLabel = new javax.swing.JLabel();
        trackCombobox = new javax.swing.JComboBox();
        tableLabel = new javax.swing.JLabel();
        tableCombobox = new javax.swing.JComboBox();
        nameLabel = new javax.swing.JLabel();
        nameTextField = new javax.swing.JTextField();
        typeLabel = new javax.swing.JLabel();
        typeIconLabel = new javax.swing.JLabel();
        messagePanel = new javax.swing.JPanel();
        messageLabel = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(750, 340));
        setName("Form"); // NOI18N
        setResizable(false);

        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setPreferredSize(new java.awt.Dimension(100, 42));

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getActionMap(UCSCtrackDialog.class, this);
        cancelButton.setAction(actionMap.get("cancelAction")); // NOI18N
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(UCSCtrackDialog.class);
        cancelButton.setText(resourceMap.getString("cancelButton.text")); // NOI18N
        cancelButton.setMaximumSize(new java.awt.Dimension(75, 27));
        cancelButton.setMinimumSize(new java.awt.Dimension(75, 27));
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.setPreferredSize(new java.awt.Dimension(75, 27));

        okButton.setAction(actionMap.get("OKAction")); // NOI18N
        okButton.setText(resourceMap.getString("okButton.text")); // NOI18N
        okButton.setMaximumSize(new java.awt.Dimension(75, 27));
        okButton.setMinimumSize(new java.awt.Dimension(75, 27));
        okButton.setName("okButton"); // NOI18N
        okButton.setOpaque(false);
        okButton.setPreferredSize(new java.awt.Dimension(75, 27));

        progressbar.setMinimumSize(new java.awt.Dimension(40, 20));
        progressbar.setName("progressbar"); // NOI18N
        progressbar.setPreferredSize(new java.awt.Dimension(40, 20));

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(progressbar, javax.swing.GroupLayout.PREFERRED_SIZE, 128, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 88, Short.MAX_VALUE)
                .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jLabel1))
                    .addComponent(progressbar, javax.swing.GroupLayout.DEFAULT_SIZE, 27, Short.MAX_VALUE))
                .addContainerGap())
        );

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        jPanel3.setName("jPanel3"); // NOI18N
        jPanel3.setLayout(new java.awt.BorderLayout());

        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        gridPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(10, 30, 10, 20));
        gridPanel.setName("gridPanel"); // NOI18N
        gridPanel.setLayout(new java.awt.GridBagLayout());

        organismLabel.setText(resourceMap.getString("organismLabel.text")); // NOI18N
        organismLabel.setName("organismLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(8, 0, 8, 10);
        gridPanel.add(organismLabel, gridBagConstraints);

        organismCombobox.setName("organismCombobox"); // NOI18N
        organismCombobox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                organismSelectedAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridPanel.add(organismCombobox, gridBagConstraints);

        buildLabel.setText(resourceMap.getString("buildLabel.text")); // NOI18N
        buildLabel.setName("buildLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(8, 0, 8, 26);
        gridPanel.add(buildLabel, gridBagConstraints);

        buildCombobox.setName("buildCombobox"); // NOI18N
        buildCombobox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buildSelectedAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridPanel.add(buildCombobox, gridBagConstraints);

        groupLabel.setText(resourceMap.getString("groupLabel.text")); // NOI18N
        groupLabel.setName("groupLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(8, 0, 8, 10);
        gridPanel.add(groupLabel, gridBagConstraints);

        groupCombobox.setName("groupCombobox"); // NOI18N
        groupCombobox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                groupSelectedAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridPanel.add(groupCombobox, gridBagConstraints);

        trackLabel.setText(resourceMap.getString("trackLabel.text")); // NOI18N
        trackLabel.setName("trackLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(8, 0, 8, 10);
        gridPanel.add(trackLabel, gridBagConstraints);

        trackCombobox.setName("trackCombobox"); // NOI18N
        trackCombobox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                trackSelectedAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridPanel.add(trackCombobox, gridBagConstraints);

        tableLabel.setText(resourceMap.getString("tableLabel.text")); // NOI18N
        tableLabel.setName("tableLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(8, 0, 8, 10);
        gridPanel.add(tableLabel, gridBagConstraints);

        tableCombobox.setName("tableCombobox"); // NOI18N
        tableCombobox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tableSelectionAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridPanel.add(tableCombobox, gridBagConstraints);

        nameLabel.setText(resourceMap.getString("nameLabel.text")); // NOI18N
        nameLabel.setName("nameLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(8, 0, 8, 10);
        gridPanel.add(nameLabel, gridBagConstraints);

        nameTextField.setColumns(30);
        nameTextField.setText(resourceMap.getString("nameTextField.text")); // NOI18N
        nameTextField.setName("nameTextField"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridPanel.add(nameTextField, gridBagConstraints);

        typeLabel.setText(resourceMap.getString("typeLabel.text")); // NOI18N
        typeLabel.setName("typeLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(8, 0, 8, 10);
        gridPanel.add(typeLabel, gridBagConstraints);

        typeIconLabel.setText(resourceMap.getString("typeIconLabel.text")); // NOI18N
        typeIconLabel.setName("typeIconLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridPanel.add(typeIconLabel, gridBagConstraints);

        mainPanel.add(gridPanel);

        jPanel3.add(mainPanel, java.awt.BorderLayout.CENTER);

        messagePanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 10, 0, 0));
        messagePanel.setMinimumSize(new java.awt.Dimension(16, 30));
        messagePanel.setName("messagePanel"); // NOI18N
        messagePanel.setPreferredSize(new java.awt.Dimension(100, 30));
        messagePanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        messageLabel.setFont(resourceMap.getFont("messageLabel.font")); // NOI18N
        messageLabel.setForeground(resourceMap.getColor("messageLabel.foreground")); // NOI18N
        messageLabel.setText(resourceMap.getString("messageLabel.text")); // NOI18N
        messageLabel.setName("messageLabel"); // NOI18N
        messagePanel.add(messageLabel);

        jPanel3.add(messagePanel, java.awt.BorderLayout.SOUTH);

        getContentPane().add(jPanel3, java.awt.BorderLayout.CENTER);

        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setPreferredSize(new java.awt.Dimension(100, 20));

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 406, Short.MAX_VALUE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 20, Short.MAX_VALUE)
        );

        getContentPane().add(jPanel2, java.awt.BorderLayout.NORTH);

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void tableSelectionAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tableSelectionAction
    String newtable=(String)tableCombobox.getSelectedItem();
    // System.err.println("oldtable="+oldtable+"    newtable="+newtable);
    if (oldtable==null || !oldtable.equals(newtable)) {
        updateMenus("table");
    }
    oldtable=newtable;
}//GEN-LAST:event_tableSelectionAction

private void organismSelectedAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_organismSelectedAction
    String neworganism=(String)organismCombobox.getSelectedItem();
    if (oldorganism==null || !oldorganism.equals(neworganism)) { 
        int organismID=Organism.getTaxonomyID(neworganism);
        String[] builds=Organism.getSupportedGenomeBuilds(organismID);
        if (builds!=null) {
            buildCombobox.setModel(new DefaultComboBoxModel(builds));
            buildCombobox.setVisible(true);
            groupCombobox.setVisible(true);
            trackCombobox.setVisible(true);
            tableCombobox.setVisible(true);
            everythingOK=true;
            updateMenus("organism");
        }
        else {
            buildCombobox.setVisible(false);
            groupCombobox.setVisible(false);
            trackCombobox.setVisible(false);
            tableCombobox.setVisible(false);
            everythingOK=false;
        }
    }
    oldorganism=neworganism;
}//GEN-LAST:event_organismSelectedAction

private void buildSelectedAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buildSelectedAction
    String newbuild=(String)buildCombobox.getSelectedItem();
    if (oldbuild==null || !oldbuild.equals(newbuild)) {
       updateMenus("build");
    }
    oldbuild=newbuild;
}//GEN-LAST:event_buildSelectedAction

private void groupSelectedAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_groupSelectedAction
    String newgroup=(String)groupCombobox.getSelectedItem();
    //System.err.println("oldgroup="+oldgroup+"    newgroup="+newgroup);
    if (oldgroup==null || !oldgroup.equals(newgroup)) {
       updateMenus("group");
    }
    oldgroup=newgroup;
}//GEN-LAST:event_groupSelectedAction

private void trackSelectedAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_trackSelectedAction
    String newtrack=(String)trackCombobox.getSelectedItem();
    //System.err.println("oldtrack="+oldtrack+"    newtrack="+newtrack);
    if (oldtrack==null || !oldtrack.equals(newtrack)) {
       updateMenus("track");
    }
    oldtrack=newtrack;
}//GEN-LAST:event_trackSelectedAction



    @SuppressWarnings("unchecked")
    private void updateMenus(final String updatedMenu) {
        String organism=(String)organismCombobox.getSelectedItem();
        String build=(String)buildCombobox.getSelectedItem();
        String group=(String)groupCombobox.getSelectedItem();
        String table=(String)tableCombobox.getSelectedItem();
        String track=(String)trackCombobox.getSelectedItem();
        int organismID=Organism.getTaxonomyID(organism);
        String clade=Organism.getCladeForOrganism(organismID);
        if (organism.equals("Fruit fly")) organism="D.+melanogaster"; // this is a hack :(        
        String argument="clade="+clade+"&org="+organism+"&db="+build;
        if (group!=null && !group.isEmpty() && (updatedMenu.equals("group") || updatedMenu.equals("track") || updatedMenu.equals("table"))) argument+="&hgta_group="+getKeyForValue("group",group);
        if (track!=null && !track.isEmpty() && (updatedMenu.equals("track") || updatedMenu.equals("table"))) argument+="&hgta_track="+getKeyForValue("track",track);
        if (table!=null && !table.isEmpty() && (updatedMenu.equals("table"))) argument+="&hgta_table="+getKeyForValue("table",table);
        //System.err.println(argument);
        progressbar.setIndeterminate(true);
        organismCombobox.setEnabled(false);
        buildCombobox.setEnabled(false);
        groupCombobox.setEnabled(false);
        tableCombobox.setEnabled(false);
        trackCombobox.setEnabled(false);
        final String urlstring=baseURL+argument;
        SwingWorker worker=new SwingWorker<Void, Void>() {
            Exception ex=null;
            @Override 
            public Void doInBackground() {
                try {  
                        URL url=new URL(urlstring);
                        String page=getPage(url);        
                        HashMap<String,String[][]> map=parsePage(page);
                        currentConfiguration=map;
                } catch (Exception e) {ex=e;} 
                return null;
            }
            @Override
            public void done() { // this method is invoked on the EDT!
                progressbar.setIndeterminate(false);
                organismCombobox.setEnabled(true);
                buildCombobox.setEnabled(true);
                groupCombobox.setEnabled(true);
                tableCombobox.setEnabled(true);
                trackCombobox.setEnabled(true);
                if (ex!=null) {
                     messageLabel.setText(ex.getClass().getSimpleName()+" : "+ex.getMessage());
                     everythingOK=false;
                } else {
                    updateMenusWithNewData(updatedMenu);
                }

            }
        }; // end of SwingWorker class
        worker.execute();        
    }

    
    private void updateMenusWithNewData(String updatedMenu) {
        //System.err.println("Updating menu "+updatedMenu);
        if (updatedMenu.equals("all")) {          
            groupCombobox.setModel(getMenuModel("group"));
            trackCombobox.setModel(getMenuModel("track"));
            tableCombobox.setModel(getMenuModel("table"));  
            oldorganism=getCurrentSelectionForBuildAndOrganism("org");            
            oldbuild=getCurrentSelectionForBuildAndOrganism("db");  
            if (oldorganism.equals("D. melanogaster")) oldorganism="Fruit fly"; // this is a hack
            organismCombobox.setSelectedItem(oldorganism);   
            int organismID=Organism.getTaxonomyID(oldorganism);
            String[] builds=Organism.getSupportedGenomeBuilds(organismID);
            if (builds!=null) {
                buildCombobox.setModel(new DefaultComboBoxModel(builds));  
                buildCombobox.setSelectedItem(oldbuild);   
            }
        } else if (updatedMenu.equals("organism") || updatedMenu.equals("build")) { // this also updates build in the above menu
            groupCombobox.setModel(getMenuModel("group"));
            trackCombobox.setModel(getMenuModel("track"));
            tableCombobox.setModel(getMenuModel("table"));
        } else if (updatedMenu.equals("group")) {
            trackCombobox.setModel(getMenuModel("track"));
            tableCombobox.setModel(getMenuModel("table"));            
        } else if (updatedMenu.equals("track")) {
            tableCombobox.setModel(getMenuModel("table"));            
        } 
        oldgroup=getCurrentSelectionInMenu("group");
        oldtrack=getCurrentSelectionInMenu("track");
        oldtable=getCurrentSelectionInMenu("table");       
        groupCombobox.setSelectedItem(oldgroup);
        trackCombobox.setSelectedItem(oldtrack);
        tableCombobox.setSelectedItem(oldtable);
        if (nameTextField.isEditable()) {
            nameTextField.setText(getKeyForValue("table", oldtable));
            nameTextField.setCaretPosition(0);
        }

        Class type=getCurrentTrackType();
        if (type==null) {
            messageLabel.setText("Unable to parse data from this track");
            everythingOK=false;
            typeIconLabel.setIcon(unknownIcon);
            typeIconLabel.setText("Unknown");
        } else {
            messageLabel.setText("  ");
            everythingOK=true;
            if (type==NumericDataset.class) {
                typeIconLabel.setIcon(numericIcon);
                typeIconLabel.setText("Numeric");                
            } else if (type==RegionDataset.class) {
                typeIconLabel.setIcon(regionIcon);
                typeIconLabel.setText("Region");                
            } 
        }
        invalidate();
        repaint();
    }

    @Action
    public void OKAction() {
        if (parentDataTrack!=null) {
            Class type=getCurrentTrackType(); 
            if (type==null || !type.equals(parentDataTrack.getDataType())) {
                JOptionPane.showMessageDialog(this, "The selected source is not compatible with the current datatrack","Incompatible source",JOptionPane.ERROR_MESSAGE);                               
                return;
            }
        }        
        OKpressed=true;
        setVisible(false);
    }
    
    @Action
    public void cancelAction() {
        OKpressed=false;
        setVisible(false);
    }

    public boolean isOKPressed() {
        return OKpressed && everythingOK;
    }
    
    /** 
     * This method is used to retrieve a new DataTrack object based on the selections made in the dialog
     * the returned DataTrack object contains all necessary information to register a new track
     * @return DataTrack
     */
    public DataTrack getDataTrack() {
        if (!everythingOK) return null;
        Class type=getCurrentTrackType();
        if (type==null) return null;
        String dataformat;
        String outputType;
        if (type==NumericDataset.class) {
            dataformat="WIG";
            outputType="wigData";
        } else if (type==RegionDataset.class) {
            dataformat="GTF";
            outputType="gff";
        } else return null;
        String organism=(String)organismCombobox.getSelectedItem();
        String build=(String)buildCombobox.getSelectedItem();
        String group=(String)groupCombobox.getSelectedItem();
        String table=(String)tableCombobox.getSelectedItem();
        String track=(String)trackCombobox.getSelectedItem();
        String trackname=nameTextField.getText().trim();
        int organismID=Organism.getTaxonomyID(organism);        
        String clade=Organism.getCladeForOrganism(organismID);
        if (organism.equals("Fruit fly")) organism="D.+melanogaster"; // this is a hack :(        
        String argument="clade="+clade+"&org="+organism+"&db="+build;
        argument+="&hgta_group="+getKeyForValue("group",group);
        argument+="&hgta_track="+getKeyForValue("track",track);
        argument+="&hgta_table="+getKeyForValue("table",table);
        argument+="&hgta_regionType=range";        
        argument+="&hgta_outputType="+outputType;
        argument+="&hgta_doTopSubmit=1";                
        argument+="&position=chr$CHROMOSOME:$START-$END";                
        DataTrack datatrack=new DataTrack(trackname, type, "UCSC Genome Browser", table);
        DataSource source=new DataSource_http_GET(datatrack, organismID, build, baseURL.substring(0,baseURL.length()-1), dataformat, argument);
        datatrack.addDataSource(source);
        return datatrack;
    }

    /** 
     * This method is used to retrieve a new DataSource object based on the selections made in the dialog
     * The source should be added to a preexisting datatrack (it is not automatically added to the argument track)
     * @param datatrack The parent datatrack for this source
     * @return DataSource
     */
    public DataSource getDataSource(DataTrack datatrack) {
        if (!everythingOK) return null;
        Class type=getCurrentTrackType();
        if (type==null) return null;
        String dataformat;
        String outputType;
        if (type==NumericDataset.class) {
            dataformat="WIG";
            outputType="wigData";
        } else if (type==RegionDataset.class) {
            dataformat="GTF";
            outputType="gff";
        } else return null;
        String organism=(String)organismCombobox.getSelectedItem();
        String build=(String)buildCombobox.getSelectedItem();
        String group=(String)groupCombobox.getSelectedItem();
        String table=(String)tableCombobox.getSelectedItem();
        String track=(String)trackCombobox.getSelectedItem();
        int organismID=Organism.getTaxonomyID(organism);
        String clade=Organism.getCladeForOrganism(organismID);
        if (organism.equals("Fruit fly")) organism="D.+melanogaster"; // this is a hack :(        
        String argument="clade="+clade+"&org="+organism+"&db="+build;
        argument+="&hgta_group="+getKeyForValue("group",group);
        argument+="&hgta_track="+getKeyForValue("track",track);
        argument+="&hgta_table="+getKeyForValue("table",table);
        argument+="&hgta_regionType=range";        
        argument+="&hgta_outputType="+outputType;
        argument+="&hgta_doTopSubmit=1";                
        argument+="&position=chr$CHROMOSOME:$START-$END";                
        DataSource source=new DataSource_http_GET(datatrack, organismID, build, baseURL.substring(0,baseURL.length()-1), dataformat, argument);
        return source;
    }    
    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox buildCombobox;
    private javax.swing.JLabel buildLabel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JPanel gridPanel;
    private javax.swing.JComboBox groupCombobox;
    private javax.swing.JLabel groupLabel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JLabel messageLabel;
    private javax.swing.JPanel messagePanel;
    private javax.swing.JLabel nameLabel;
    private javax.swing.JTextField nameTextField;
    private javax.swing.JButton okButton;
    private javax.swing.JComboBox organismCombobox;
    private javax.swing.JLabel organismLabel;
    private javax.swing.JProgressBar progressbar;
    private javax.swing.JComboBox tableCombobox;
    private javax.swing.JLabel tableLabel;
    private javax.swing.JComboBox trackCombobox;
    private javax.swing.JLabel trackLabel;
    private javax.swing.JLabel typeIconLabel;
    private javax.swing.JLabel typeLabel;
    // End of variables declaration//GEN-END:variables


    
    /** Fetches a web page from the given URL and returns it as a String */
    private String getPage(URL url) throws Exception {
        String document="";
        InputStream inputStream = null;
        BufferedReader dataReader = null;
        URLConnection connection=url.openConnection();
        inputStream=connection.getInputStream();
        dataReader = new BufferedReader(new InputStreamReader(inputStream));
        String line;
        try {
           while ((line=dataReader.readLine())!=null){
              document+=line;
           }
        } catch (Exception e) {throw e;}
        finally {
            if (dataReader!=null) dataReader.close();
            if (inputStream!=null) inputStream.close();
        }    
        return document;
    }
    
    /**
     * Parses a page and returns a HashMap where the keys are names of drop-down menus
     * such as "clade","org,"db","hgta_group","hgta_track","hgta_table","hgta_outputType"
     * and the value is another structure with key-value options for that menu.
     */
    private HashMap<String,String[][]> parsePage(String page) {
        HashMap<String, String[][]> map=new HashMap<String,String[][]>();
        String[] sections=page.split("<SELECT NAME=");        
        if (sections.length<=1) return map;
        for (int i=1;i<sections.length;i++) { // start at i=1 because the first section contains no SELECT
            String section=sections[i].substring(0,sections[i].indexOf("</SELECT"));
            int pos=section.indexOf(" ");
            int pos2=section.indexOf(">");
            if (pos<0) pos=pos2;
            else if (pos2>0 && pos2<pos) pos=pos2;
            String menuname=section.substring(0,pos);
        
            String options[]=section.split("<OPTION");
            menuname=stripQuotes(menuname);
            if (options.length<=1) continue; // no options? try the next section
            String[][] keyvalue=new String[options.length-1][3];
            for (int j=1;j<options.length;j++) {
                  String option=options[j];
                  boolean isSelected=false;
                  if (option.indexOf(" SELECTED")>=0) {
                      isSelected=true;
                      option=option.replaceAll(" SELECTED","");   
                  }
                  option=option.replaceFirst(" VALUE=","");
                  if (option.indexOf("</OPTION")>0) option=option.substring(0,option.indexOf("</OPTION"));
                  int splitpos=option.indexOf('>');
                  String key=stripQuotes(option.substring(0,splitpos));
                  String value=option.substring(splitpos+1).trim();
                  keyvalue[j-1][0]=key;                
                  keyvalue[j-1][1]=value;     
                  keyvalue[j-1][2]=(isSelected)?"selected":null;
            }            
            map.put(menuname, keyvalue);
        }            
        return map;
    }
    
    
    
    private String stripQuotes(String string) {
        if (string!=null && string.startsWith("\"") && string.endsWith("\"")) return string.substring(1,string.length()-1);
        return string;
    }
    
    private DefaultComboBoxModel getMenuModel(String menu) {
        //debug("hgta_"+menu);
        if (currentConfiguration==null) return new DefaultComboBoxModel(new String[]{""});
        String[][] options=currentConfiguration.get("hgta_"+menu);
        if (options==null) return new DefaultComboBoxModel(new String[]{""});
        String[] names=new String[options.length];
        for (int i=0;i<options.length;i++) names[i]=options[i][1];
        return new DefaultComboBoxModel(names);
    }
    
    private Class getCurrentTrackType() {
        if (currentConfiguration==null) return null;
        String[][] options=currentConfiguration.get("hgta_outputType");
        if (options==null) return null;
        for (int i=0;i<options.length;i++) {
                 if (options[i][0].equals("wigData")) return NumericDataset.class;
            else if (options[i][0].equals("gff")) return RegionDataset.class;
        }
        return null;
    }
       
    
    /** Finds the currently selected item in currentConfiguration for the specified menu*/
    private String getCurrentSelectionInMenu(String menu) {
        if (currentConfiguration==null) return null;
        String[][] options=currentConfiguration.get("hgta_"+menu);
        if (options==null) return null;
        for (int i=0;i<options.length;i++) {
            if (options[i][2]!=null) return options[i][1];
        }
        return null;   
    }
    
    /** Finds the currently selected item in currentConfiguration for the specified menu which must be 'db' (build) or 'org' */
    private String getCurrentSelectionForBuildAndOrganism(String menu) {
        if (currentConfiguration==null) return null;
        String[][] options=currentConfiguration.get(menu);
        if (options==null) return null;
        for (int i=0;i<options.length;i++) {
            if (options[i][2]!=null) return options[i][1];
        }
        return null;   
    }
    
    
    
    
    
    
    private String getKeyForValue(String menu, String value) {
        if (currentConfiguration==null) return "";
        String[][] options=currentConfiguration.get("hgta_"+menu);
        if (options==null) return "";
        for (int i=0;i<options.length;i++) {
            if (options[i][1].equals(value)) return options[i][0];
        }
        return "";          
        
    }
    
    
    private void debug() {
        if (currentConfiguration==null) {
            System.err.println("currentConfiguration==null");
            return;
        }
        for (String sections:currentConfiguration.keySet()) {
            System.err.println("--- "+sections+" ---");
            String[][] pairs=currentConfiguration.get(sections);
            for (int i=0;i<pairs.length;i++) {
                System.err.print("    "+pairs[i][0]+" => "+pairs[i][1]);
                if (pairs[i][2]!=null) System.err.println("     SELECTED");
                else System.err.println();
            }
        }        
    }
    
    private void debug(String section) {
        if (currentConfiguration==null) {
            System.err.println("currentConfiguration==null");
            return;
        }
        String[][] pairs=currentConfiguration.get(section);
        if (pairs==null) {
            System.err.println("No such section: "+section+". Only found:");
            for (String string:currentConfiguration.keySet()) {
                System.err.println("    "+string);
            }
            return;
        }      
        System.err.println("--- "+section+" ---");        
        for (int i=0;i<pairs.length;i++) {
            System.err.print("    "+pairs[i][0]+" => "+pairs[i][1]);
            if (pairs[i][2]!=null) System.err.println("     SELECTED");
            else System.err.println();
        }
            
    }    
}
