/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui;

import javax.swing.event.ListDataEvent;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.data.*;

import java.awt.Color;
import java.io.Serializable;
import java.util.Set;
import java.util.HashMap;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import javax.swing.SwingUtilities;
import javax.swing.event.ListDataListener;
import org.jdesktop.application.ApplicationContext;
import priorseditor.engine.DataListener;



/**
 * This class is a placeholder for all settings applicable to sequence visualization,
 * and keeps information such as the color, type and height of all datatracks and the 
 * order of tracks (and whether they should be displayed at all). 
 * Relevant visualizers use this class to obtain the appropriate settings, which 
 * can be adjusted by the user in the GUI.
 * 
 * VisualizationSettings should be a "singleton" object within the GUI, and to
 * obtain the object one should use the getInstance() method rather than calling
 * the constructor directly (it is private anyway). 
 *
 * To set or get data one could use the getSetting() or storeSetting() methods respectively.
 * Alternatively, the methods getPersistantSetting() and storePersistantSetting() can be used
 * for settings that should be kept the same for later invokations of the application 
 * (these values are stored locally in the users home directory).
 * 
 * Many properties are provided with convenience accessor methods that should
 * preferably be used when available. For instance, in stead of using getSetting("height")
 * or storeSetting("height") directly, one should use getHeight() or setHeight(). 
 * These convenience methods take care of remembering the specific settings names (HashMap keys)
 * and also do conversions to appropriate data types. In addition, the methods can
 * supply sensible defaults for uninitialized settings and take care of any other
 * updates that should be made for consistency.
 * 
 * 
 * @author kjetikl
 */
public class VisualizationSettings implements ListDataListener, DataListener, Serializable {
    public static String SEQUENCE_WINDOW_SIZE="SequenceWindowSize";
    public static String SEQUENCE_SIZE="SequenceSize";
    private static int DEFAULT_SEQUENCE_WINDOW_SIZE=700;
    public static String TOTAL_TRACKS_HEIGHT="totalTracksHeight";
    public static String VIEWPORT_START="viewportStart";
    public static String MAINPANEL_BACKGROUND="mainpanelBackground";
    public static String FOREGROUND_COLOR="foregroundColor";
    public static String BACKGROUND_COLOR="backgroundColor";
    public static String COLORGRADIENT="colorgradient";
    public static String BASE_COLOR="baseColor";
    public static String SEQUENCE_ORIGO="sequenceOrigo";
    public static String TRACK_HEIGHT="trackHeight";
    private static int DEFAULT_TRACK_HEIGHT_DNA=10;
    private static int DEFAULT_TRACK_HEIGHT_NUMERIC=20;
    private static int DEFAULT_TRACK_HEIGHT_REGION=6;
    private static int DEFAULT_TRACK_HEIGHT_MOTIF=24;
    public static String SEQUENCE_MARGIN="sequenceMargin";
    private static int DEFAULT_SEQUENCE_MARGIN=20;
    public static String SEQUENCE_LABEL_FIXED_WIDTH="sequenceLabelFixedWidth";
    public static String SEQUENCE_LABEL_WIDTH="sequenceLabelWidth";
    public static String SEQUENCE_LABEL_COLOR="sequenceLabelColor";
    public static String SCALE_SEQUENCE_LABELS_TO_FIT="scaleSequenceLabelsToFit";
    private static int DEFAULT_SEQUENCE_LABEL_WIDTH=80;
    public static String CLUSTER_COLOR="clusterColor";
    public static String ZOOM_LEVEL="zoomLevel";
    private static double DEFAULT_ZOOM_LEVEL=100.0;
    public static String VISIBLE="visible";
    public static String DATASET_VISIBLE="datasetVisible";
    public static String GRAPH_TYPE="graphType";
    public static int BAR_GRAPH=0;
    public static int GRADIENT_GRAPH=1;
    public static int REGION_GRAPH=2;
    public static int REGION_MULTICOLOR_GRAPH=3;
    public static int GRADIENT_REGION_GRAPH=4;
    public static int DNA_GRAPH=5;
    public static String SEQUENCE_ORIENTATION="sequenceOrientation";
    public static int DIRECT=1;
    public static int REVERSE=-1;
    public static String CONSTRAINED="constrained";
    public static String SCROLL_LOCK="scroll lock";
    public static String SEQUENCE_WITH_SELECTION="sequenceWithSelection";
    public static String RULER="Ruler";
    public static String RULER_TSS="Origo At Gene TSS";
    public static String RULER_LEFT_END="Origo At Left End";
    public static String RULER_RIGHT_END="Origo At Right End";
    public static String RULER_CHROMOSOME="Origo At Chromosome Start";
    public static String RULER_FIXED="Fixed Ruler";
    public static String RULER_NONE="No Ruler";
    public static String SEQUENCE_ALIGNMENT="sequenceAlignment";
    public static String SEQUENCE_DEFAULT_ALIGNMENT="sequenceAlignment";
    public static String ALIGNMENT_LEFT="Align Left";
    public static String ALIGNMENT_RIGHT="Align Right";
    public static String ALIGNMENT_NONE="None";
    public static String ALIGNMENT_TSS="Align TSS";
    public static String ANTIALIAS_TEXT="Antialias text";
    public static String ANTIALIAS_MOTIFS="Antialias motifs";
    public static String VISUALIZE_REGION_STRAND="visualizeRegionStrand";
    public static String VISUALIZE_REGION_SCORE="visualizeRegionScore";
    public static String MULTICOLOR_PALETTE="multicolorpalette";
    public static String CLASS_LABEL_COLOR="classLabelColor";
    public static String COLOR_MOTIFS_BY_CLASS="colorByClass";
    public static final int DATATRACK_BORDER_SIZE=1;
    public static final int DATATRACK_MIN_HEIGHT=4; // 
    public static final int DATATRACK_MAX_HEIGHT=100; // 
    public static final int SEQUENCE_MARGIN_MIN_HEIGHT=4; // 
    public static final int SEQUENCE_MARGIN_MAX_HEIGHT=150; // 

   private static HashMap<String,Color> classColorsMap;
  static {
    //
    //  NOTE: Predefined colors are only defined for 2 levels and can not be extended without code modification!!!
    //
    classColorsMap=new HashMap<String,Color>();
    classColorsMap.put(MotifClassification.UNKNOWN_CLASS_LABEL,new Color(0,0,0));
    classColorsMap.put("1", new Color(255,0,0));
    classColorsMap.put("1.1", new Color(255,40,0));
    classColorsMap.put("1.2", new Color(220,20,60));
    classColorsMap.put("1.3", new Color(250,128,114));
    classColorsMap.put("1.6", new Color(192,32,32));
    classColorsMap.put("2", new Color(0,0,255));
    classColorsMap.put("2.1", new Color(0,0,180));
    classColorsMap.put("2.2", new Color(0,191,255));
    classColorsMap.put("2.3", new Color(30,144,255));
    classColorsMap.put("2.4", new Color(50,100,180));
    classColorsMap.put("2.5", new Color(135,206,250));
    classColorsMap.put("3", new Color(255,255,0));
    classColorsMap.put("3.1", new Color(255,240,0));
    classColorsMap.put("3.2", new Color(255,215,0));
    classColorsMap.put("3.3", new Color(255,255,48));
    classColorsMap.put("3.4", new Color(255,250,141));
    classColorsMap.put("3.5", new Color(255,215,80));
    classColorsMap.put("3.6", new Color(240,200,80));
    classColorsMap.put("4", new Color(0,255,0));
    classColorsMap.put("4.1", new Color(173,255,47));
    classColorsMap.put("4.2", new Color(124,252,0));
    classColorsMap.put("4.3", new Color(0,240,0));
    classColorsMap.put("4.4", new Color(50,205,50));
    classColorsMap.put("4.5", new Color(152,251,152));
    classColorsMap.put("4.6", new Color(144,238,144));
    classColorsMap.put("4.7", new Color(0,250,154));
    classColorsMap.put("4.8", new Color(60,179,113));
    classColorsMap.put("4.9", new Color(0,128,0));
    classColorsMap.put("4.10", new Color(107,142,35));
    classColorsMap.put("4.11", new Color(154,205,50));
    classColorsMap.put("4.12", new Color(0,100,0));
    classColorsMap.put("0", new Color(128,128,128));
    classColorsMap.put("0.1", new Color(64,64,64));
    classColorsMap.put("0.2", new Color(128,128,128));
    classColorsMap.put("0.3", new Color(169,169,169));
    classColorsMap.put("0.4", new Color(112,128,144));
    classColorsMap.put("0.5", new Color(226,226,226));
  }


    // Below are the default color palettes in order
    private Color[] bgColorList=new Color[]{Color.white,Color.white,Color.white, Color.white,Color.white,  Color.white,Color.white,Color.white,Color.white,Color.white,Color.white,Color.white,Color.white,Color.white};
    private Color[] fgColorList=new Color[]{Color.black,Color.red,  Color.green, Color.blue, Color.magenta,Color.cyan, Color.orange, new Color(128,0,0), new Color(0,128,0), new Color(0,0,128), new Color(128,0,128), new Color(0,128,128), new Color(128,128,0), new Color(128,64,0)};
   // private double[] zoomChoices=new double[]{5,10,25,50,75,90,100,150,200,500,800,1000,2000}; // these must be in increasing order!!!
    private double[] zoomChoices=new double[]{5,10,25,50,75,100,200,500,800,1000,2000}; // these must be in increasing order!!!
    
    private transient PriorsEditorGUI gui;
    private int colorpointer=0;
    private int clustercolorpointer=0;
    private HashMap<String,Object> settings;
    private static VisualizationSettings instance=null;
    private ArrayList<FeatureDataset> datatrackorder=null; // a list of the currently visible datatracks in order
    private ArrayList<String> datatrackorderMasterList=null; // the names of all datatracks that have been visualized
    private transient HashSet<VisualizationSettingsListener> listeners;
    private transient ArrayList<SelectionWindow> selections=new ArrayList<SelectionWindow>(); // this array stores selection windows
    private boolean enableNotifications=true;
    
    /**
     * Returns a singleton instance of VisualizationSettings
     * Use this method instead of the regular constructor
     */
    public static VisualizationSettings getInstance(PriorsEditorGUI gui) {
        if (instance==null) instance=new VisualizationSettings(gui);
        return instance;
    }
    
    /**
     * Creates a new instance of VisualizationSettings
     * Some settings are attempted read from persistant storage (local preference files)
     * @param gui
     */
    private VisualizationSettings(PriorsEditorGUI gui) {
        this.gui=gui;
        this.settings=new HashMap<String,Object>();
        this.listeners=new HashSet<VisualizationSettingsListener>();
        this.datatrackorderMasterList=new ArrayList<String>();
        if (getPersistantSetting(SEQUENCE_WINDOW_SIZE)==null) {storePersistantSetting(SEQUENCE_WINDOW_SIZE,new Integer(DEFAULT_SEQUENCE_WINDOW_SIZE));}
        setDefaultBaseColors(); // loads or otherwise initializes a palette of default colors for visualizing DNA bases
        gui.getEngine().addDataListener(this);
    }
    
    public void setGUI(PriorsEditorGUI gui) {
        this.gui=gui;
    }
    
    
    /**
     * Returns a visualization setting corresponding to the given key (if it exists)
     * Settings are usually stored in the format "datasetname.propertyname" (eg. Repeats.foregroundColor)
     * 
     * @param key The name of the visualization setting to be retrieved
     * @return An object representing the specified visualization setting
     */
    public Object getSetting(String key) {
        return settings.get(key);
    }
    
    /**
     * Stores a visualization setting (as an object) under a given name 
     * @param key The name of this setting
     * @param value The object representing the settings value
     */
    public void storeSetting(String key, Object value) {
        settings.put(key, value);
    }

    
    /**
     * Returns a visualization setting corresponding to the given key (if it exists)
     * The method works by first invoking getSetting() to see if a value currently
     * exists for this setting in working memory. If no value can be found at this
     * point, the method will look for a persistant value which has previously been
     * stored in the users home-directory from an earlier invokation of the application.
     * @param key The name of this setting
     * @param value The object representing the settings value
     */
    public Object getPersistantSetting(String key) {
        Object value=getSetting(key);
        if (value!=null) return value;
        ApplicationContext context=gui.getApplication().getContext();
        try {
            String filename=key+".xml";
            value=context.getLocalStorage().load(filename);
            storeSetting(key,value);
            return value;
        } catch (Exception e) {return null;}        
    }
      

    /**
     * Stores a visualization setting (an object) under a given name in a persistant way 
     * (so that the setting can restored when the PriorsEditor is invoked again at later time)
     * @param key The name of this setting
     * @param value The object representing the settings value
     */
    public void storePersistantSetting(String key, Object value) {
        storeSetting(key, value);
        ApplicationContext context=gui.getApplication().getContext();
        try {
            String filename=key+".xml";
            context.getLocalStorage().save(value,filename);
        } catch (Exception e) {}        
    }
      
    /** Return true if the visualizationSettings contain an explicitly 
     *  set value for the given dataset and setting 
     *  Use the static strings in this class to refer to settings
     */
    public boolean hasSetting(String settingname, String datasetname) {
        return (getSetting(datasetname+"."+settingname)!=null);
    }
    
    /**
     * This method renames all settings currently associated with the dataitem of the given name
     * Note: this renaming only works for non-persistant settings
     */
    public void renameDataItem(String oldname, String newname) {
        Set<String> keys=settings.keySet();
        ArrayList<String> oldkeys=new ArrayList<String>();
        HashMap<String, Object> newvalues=new HashMap<String, Object>();
        for (String key:keys) {
            if (key.equals(oldname)) {
                Object value=getSetting(key);
                oldkeys.add(key);
                newvalues.put(newname, value);
            }
            else if (key.startsWith(oldname+".")) {
                Object value=getSetting(key);
                oldkeys.add(key);
                String newkey=key.replaceFirst(oldname+".", newname+".");
                newvalues.put(newkey, value);
            }
            else if (key.endsWith("."+oldname)) {
                Object value=getSetting(key);
                oldkeys.add(key);
                String newkey=key.replace("."+oldname, "."+newname);
                newvalues.put(newkey, value);
            }
            else if (key.contains("."+oldname+".")) {
                Object value=getSetting(key);
                oldkeys.add(key);
                String newkey=key.replace("."+oldname+".", "."+newname+".");
                newvalues.put(newkey, value);
            }
        }
        for (String k:oldkeys) settings.remove(k);
        Set<String> newkeys=newvalues.keySet();
        for (String k:newkeys) settings.put(k,newvalues.get(k));
        // notifyListenersOfDataRenameEvent(oldname,newname);
    }
    
    /**
     * Returns a reference to the GUI itself
     * @return the GUI
     */
    public PriorsEditorGUI getGUI() {
        return gui;
    }
    
    /** Returns a reference to the underlying instance of PriorsEditorEngine used by the GUI
     * @return the engine
     */
    public PriorsEditorEngine getEngine() {
        return gui.getEngine();
    }
    
    
  /** Returns a new "unique" foreground/background color combination.
     * Tracks that do not have specified colors (yet) can 
     * let this function provide them with unique (to some extent) colors.
     * Each new call to this method returns a new fg/bg set of different colors
     * (at least up to a certain point, after which the colors are recycled)
     * This function is "synchronized" with getNewColorGradient() to make sure that unique colors are 
     * returned for each color request whether the user asks for gradients or fg/bg-color sets.
   * 
     * @return an array with two Color entries, the first is a foreground color and the second a background color
     */
    private Color[] getNewColorSet() {
        Color[] colors=new Color[]{fgColorList[colorpointer], bgColorList[colorpointer]};
        colorpointer++;
        if (colorpointer>=fgColorList.length) colorpointer=0; // cycle through the colors
        return colors;
    }
     
    
    /**
     * A convenience method to fetch the foreground color for a dataset
     * If no color is currently specified for this dataset, a new set of colors
     * will be provided (for both foreground and background)
     * @param datasetname The name of the dataset
     * @return The foreground color for this dataset 
     */
    public Color getForeGroundColor(String datasetname) {
        Object value=getSetting(datasetname+"."+FOREGROUND_COLOR);
        if (value!=null) return (Color)value;
        else {
            Color[] set=getNewColorSet();
            setForeGroundColor(datasetname,set[0]);
            setBackGroundColor(datasetname,set[1]); 
            return set[0];
        }
    }
    
    /**
     * A convenience method to set the foreground color for a dataset
     * @param datasetname The name of the dataset
     * @param color The foreground color for this dataset
     */
    public void setForeGroundColor(String datasetname, Color color) {
        storeSetting(datasetname+"."+FOREGROUND_COLOR,color);
        storeSetting(datasetname+"."+COLORGRADIENT,null); // invalidate gradients
        redraw();
    }
    
    /**
     * A convenience method to set the foreground color for multiple datasets
     * @param datasetnames The names of the datasets
     * @param color The foreground color for these datasets
     */
    public void setForeGroundColor(String datasetnames[], Color color) {
        for (String datasetname:datasetnames) {
            storeSetting(datasetname+"."+FOREGROUND_COLOR,color);
            storeSetting(datasetname+"."+COLORGRADIENT,null); // invalidate gradients
        }
        redraw();
    }
    
    
    
    
    /**
     * A convenience method to fetch the background color for a dataset
     * If no color is currently specified for this dataset, a new set of colors
     * will be provided 
     * @param datasetname The name of the dataset
     * @return The background color for this dataset 
     */
    public Color getBackGroundColor(String datasetname) {
        Object value=getSetting(datasetname+"."+BACKGROUND_COLOR);
        if (value!=null) return (Color)value;
        else {
            Color[] set=getNewColorSet();
            setForeGroundColor(datasetname,set[0]);
            setBackGroundColor(datasetname,set[1]);   
            return set[1];
        }
    }
    
    /**
     * A convenience method to set the background color for a dataset
     * @param datasetname The name of the dataset
     * @param background The background color for this dataset
     */
    public void setBackGroundColor(String datasetname, Color background) {
        storeSetting(datasetname+"."+BACKGROUND_COLOR,background);
        storeSetting(datasetname+"."+COLORGRADIENT,null); // invalidate gradients
        redraw();
    }
    

       
    /**
     * A convenience method to fetch the color gradient for a dataset
     * @param datasetname The name of the dataset
     * @return The highlight background color for this dataset
     */
    public ColorGradient getColorGradient(String datasetname) {
        Object value=getSetting(datasetname+"."+COLORGRADIENT);
        if (value!=null) return (ColorGradient)value; 
        else {
            Color fgcolor=getForeGroundColor(datasetname);
            Color bgcolor=getBackGroundColor(datasetname);
            ColorGradient gradient=new ColorGradient(fgcolor, bgcolor);
            storeSetting(datasetname+"."+COLORGRADIENT,gradient);
            return gradient;
        }    
    }
    
    /**
     * Given a foreground and background color corresponding to a color pair
     * from the predefined list of standard colors, this function will return
     * the next color pair in the list (if the supplied pair is not a standard
     * color pair, the first preset color pair will be returned).
     * @param foreground
     * @param background
     * @return Color[] a list containing the next foreground and background color pair (size=2);
     */
    public Color[] getNextPresetColorPair(Color foreground,Color background) {
        int size=bgColorList.length;
        if (foreground==null || background==null) return new Color[]{fgColorList[0],bgColorList[0]};
        int index=getIndexForColorPair(foreground, background);  
        if (index<0)  return new Color[]{fgColorList[0],bgColorList[0]};
        if (index==size-1) index=0; else index++;
        return new Color[]{fgColorList[index],bgColorList[index]};
    }
    
    /**
     * Given a foreground and background color corresponding to a color pair
     * from the predefined list of standard colors, this function will return
     * the previous color pair in the list (if the supplied pair is not a standard
     * color pair, the first preset color pair will be returned).
     * @param foreground
     * @param background
     * @return Color[] a list containing the next foreground and background color pair (size=2);
     */
    public Color[] getPreviousPresetColorPair(Color foreground,Color background) {
        int size=bgColorList.length;
        if (foreground==null || background==null) return new Color[]{fgColorList[0],bgColorList[0]};
        int index=getIndexForColorPair(foreground, background);   
        if (index<0)  return new Color[]{fgColorList[0],bgColorList[0]};
        if (index==0) index=size-1; else index--;
        return new Color[]{fgColorList[index],bgColorList[index]};
    }
    
    /** 
     * Returns the preset index corresponding to the given color pair 
     * or -1 if there is no such color pair
     */
    private int getIndexForColorPair(Color foreground,Color background) {
       int size=bgColorList.length;
       for (int i=0;i<size;i++) {
            if (foreground.equals(fgColorList[i]) && background.equals(bgColorList[i])) return i;
       }   
       return -1;
    }
    
     /**
     * A convenience method to get the color to be used for the specified base when visualizing DNA sequences
     * @param base The letter representing the base (A,C,G,T or X)
     * @return color The color to be used for this base
     */
    public Color getBaseColor(char base) {
        switch (base) {
            case 'A': break;
            case 'C': break;
            case 'G': break;
            case 'T': break;
            case 'a': base='A';break;
            case 'g': base='G';break;
            case 'c': base='C';break;
            case 't': base='T';break;
            default : base='X';
        }
        Object color=getPersistantSetting(BASE_COLOR+"."+base);
        if (color!=null) return (Color)color;
        else return Color.GRAY;
    }
    
    
    /**
     * A convenience method to set the color for specified base when visualizing DNA sequences
     * @param base The letter representing the base (A,C,G,T or X)
     * @param color The color to be used for this base
     */
    public void setBaseColor(char base, Color color) {
        storePersistantSetting(BASE_COLOR+"."+base,color);
        redraw();
    }

    /**
     * Sets a palette of default colors for visualizing DNA bases (if no settings are stored before)
     */
    private void setDefaultBaseColors() {
        if (getPersistantSetting(BASE_COLOR+".A")==null) storePersistantSetting(BASE_COLOR+".A", Color.green);
        if (getPersistantSetting(BASE_COLOR+".C")==null) storePersistantSetting(BASE_COLOR+".C", Color.blue);
        if (getPersistantSetting(BASE_COLOR+".G")==null) storePersistantSetting(BASE_COLOR+".G", Color.yellow);
        if (getPersistantSetting(BASE_COLOR+".T")==null) storePersistantSetting(BASE_COLOR+".T", Color.red);
        if (getPersistantSetting(BASE_COLOR+".X")==null) storePersistantSetting(BASE_COLOR+".X", Color.black);
        
    }
   
    /**
     * Returns a color to use for the given feature. If not color is currently
     * assigned to this feature a new color will be selected from a predefined palette.
     * @param feature The name of the feature
     * @return A color to use for this feature
     */
    private int paletteindex=0;
    public Color getFeatureColor(String feature) {
        Object value=getSetting(MULTICOLOR_PALETTE+"."+feature);
        if (value!=null) return (Color)value;
        else {
            Color featureColor;
            if (shouldColorMotifsByClass() && gui.getEngine().dataExists(feature, Motif.class)) {
                    String motifclass=((Motif)gui.getEngine().getDataItem(feature)).getClassification();
                    if (motifclass!=null) {
                        String[] path=MotifClassification.getClassPath(motifclass);
                        if (path.length>2) motifclass=path[1];
                    } else motifclass=MotifClassification.UNKNOWN_CLASS_LABEL;
                    featureColor=classColorsMap.get(motifclass);
                    if (featureColor==null) featureColor=classColorsMap.get(MotifClassification.UNKNOWN_CLASS_LABEL);
            } else {
                featureColor=getColorFromIndex(paletteindex);
                paletteindex++;
            }
            storeSetting(MULTICOLOR_PALETTE+"."+feature,featureColor);
            return featureColor;
        }
    }



    /**
     * Sets a color to use for the given feature.
     * @param feature The name of the feature
     * @param newcolor A color to use for this feature
     */
    public void setFeatureColor(String feature, Color newcolor, boolean doRedraw) {
         storeSetting(MULTICOLOR_PALETTE+"."+feature,newcolor);
         if (doRedraw) redraw();
    }

    /**
     * Returns true if the feature currently has an assigned color
     * @param feature The name of the feature
     */
    public boolean hasFeatureColor(String feature) {
         Object value=getSetting(MULTICOLOR_PALETTE+"."+feature);
         return (value!=null);
    }


 /** Goes through all motifs registered with engine and sets the color of each motif to a predefined class color
  *  It also updates the label colors for all registered classes
  */
 private void updateMotifColorsByClass() {
    ArrayList<Data> motifs=gui.getEngine().getAllDataItemsOfType(Motif.class);
    for (Data data:motifs) {
        String motifclass=((Motif)data).getClassification();
        if (motifclass!=null) {
            String[] path=MotifClassification.getClassPath(motifclass);
            if (path.length>2) motifclass=path[1];
        } else motifclass=MotifClassification.UNKNOWN_CLASS_LABEL;
        Color newcolor=classColorsMap.get(motifclass);
        if (newcolor==null) newcolor=classColorsMap.get(MotifClassification.UNKNOWN_CLASS_LABEL);
        setFeatureColor(data.getName(), newcolor, false);
    }
    String[] classlabels=MotifClassification.getClassStrings();
    for (String classlabel:classlabels) {
      String lookupkey=classlabel;
      String[] path=MotifClassification.getClassPath(classlabel);
      if (path.length>2) lookupkey=path[1];
      setClassLabelColor(classlabel, classColorsMap.get(lookupkey));
    }
    setClassLabelColor(MotifClassification.UNKNOWN_CLASS_LABEL, classColorsMap.get(MotifClassification.UNKNOWN_CLASS_LABEL));
    redraw();
 }

    /**
     * A convenience method to set the visibility of a feature (such as motifs)
     * @param datasetname The name of the dataset
     * @param show True if the feature should be shown and false if it should be hidden
     * @param redraw set to True if the gui should be automatically redrawn afterwards
     */
    public void setFeatureVisible(String featureName, boolean show, boolean redraw) {
            storeSetting(featureName+"."+VISIBLE,new Boolean(show));
            if (redraw) redraw();
    }

    /**
     * Sets a color to use for the given class label.
     * @param classlabel The name of the classlabel
     * @param newcolor A color to use for this classlabel
     */
    public void setClassLabelColor(String classlabel, Color newcolor) {
         storeSetting(CLASS_LABEL_COLOR+"."+classlabel,newcolor);
    }

    /**
     * Returns a color to use for the given class label.
     * Note that the returned value can be NULL. If no color is assigned for the label, it will not be assigned one automatically
     * @param classlabel The name of the classlabel
     */
    public Color getClassLabelColor(String classlabel) {
        return (Color)getSetting(CLASS_LABEL_COLOR+"."+classlabel);
    }

    /**
     * Specifies whether motifs should be assigned colors based on their classification
     * and not by random assignment.
     * If this setting was previously FALSE but is set to TRUE, all motifs will have their color updated
     * @param newsetting
     */
    public void setColorMotifsByClass(boolean newsetting) {
        boolean oldSetting=shouldColorMotifsByClass();
        storeSetting(COLOR_MOTIFS_BY_CLASS,new Boolean(newsetting));
        if (oldSetting==false && newsetting==true) updateMotifColorsByClass();
    }

    /**
     * Returns TRUE if the 'color motifs by class' setting is currently in effect
     * @return
     */
    public boolean shouldColorMotifsByClass() {
        Object value=getSetting(COLOR_MOTIFS_BY_CLASS);
        if (value!=null) return (Boolean)value;
        else {
            storeSetting(COLOR_MOTIFS_BY_CLASS,Boolean.FALSE);
            return Boolean.FALSE;
        }
    }

    /**
     * Sets a color to use for the label for the given sequence
     * @param classlabel The name of the classlabel
     * @param newcolor A color to use for this classlabel
     */
    public void setSequenceLabelColor(String sequencename, Color newcolor) {
         storeSetting(SEQUENCE_LABEL_COLOR+"."+sequencename,newcolor);
    }
    
    /**
     * Returns a color to use for the label for the given sequence
     * If no explicit color has been set the return value will default to BLACK
     * @param classlabel The name of the classlabel
     */
    public Color getSequenceLabelColor(String sequencename) {
        Color color=(Color)getSetting(SEQUENCE_LABEL_COLOR+"."+sequencename);
        if (color==null) return Color.BLACK;
        else return color;
    }
    
    /**
     * Sets a color to use for the cluster with the given name
     * @param clustername The name of the cluster
     * @param newcolor A color to use for this cluster
     */
    public void setClusterlColor(String clustername, Color newcolor) {
         storeSetting(CLUSTER_COLOR+"."+clustername,newcolor);
    }
    
    /**
     * Returns a color to use for the cluster with the given name
     * If no explicit color has been set a new color will be assigned
     * @param clustername The name of the cluster
     */
    public Color getClusterColor(String clustername) {
        Color color=(Color)getSetting(CLUSTER_COLOR+"."+clustername);
        if (color==null) {
            clustercolorpointer++;
            color=fgColorList[clustercolorpointer%fgColorList.length];
            setClusterlColor(clustername, color);
        }
        return color;
    }
    
    
    
    /**
     * Sets a background color to use for the main visualization panel
     * @param newcolor A color to use
     */
    public void setVisualizationPanelBackgroundColor(Color newcolor) {
         storePersistantSetting(MAINPANEL_BACKGROUND,newcolor);
         gui.getVisualizationPanel().getMainVisualizationPanel().setBackground(newcolor);
         redraw();
    }
    
    /**
     * Returns a background color to use for the main visualization panel
     */
    public Color getVisualizationPanelBackgroundColor() {
        Color color=(Color)getPersistantSetting(MAINPANEL_BACKGROUND);
        if (color!=null) {
            return color;
        } else {
            color=new javax.swing.JPanel().getBackground();
            storePersistantSetting(MAINPANEL_BACKGROUND,color);
            return color;
        }
    }

    
    
    /**
     * Gets the sequence margin, which is the number of extra pixels between sequences
     * in the visualize panelø
     */
    public int getSequenceMargin() {
        Object value=getPersistantSetting(SEQUENCE_MARGIN);
        if (value!=null) {
            return ((Integer)value).intValue();
        } else {
            storePersistantSetting(SEQUENCE_MARGIN,new Integer(DEFAULT_SEQUENCE_MARGIN));
            return DEFAULT_SEQUENCE_MARGIN;
        }
    }
    
    /**
     * Sets the sequence margin, which is the number of extra pixels between sequences
     * in the visualize panelø
     * @param margin The new margin
     */
    public void setSequenceMargin(int margin) {
            if (margin<SEQUENCE_MARGIN_MIN_HEIGHT) margin=SEQUENCE_MARGIN_MIN_HEIGHT;
            else if (margin>SEQUENCE_MARGIN_MAX_HEIGHT) margin=SEQUENCE_MARGIN_MAX_HEIGHT;
            storePersistantSetting(SEQUENCE_MARGIN,new Integer(margin));
            //rearrangeDataTracks();
            notifyListenersOfSequenceMarginChange(margin);
            redraw();
    }
    
    /**
     * A convenience method to fetch the height that should be used when drawing
     * the dataset (excluding borders)
     * @param datasetname The name of the dataset
     * @return The height of the dataset track
     */
    public int getDataTrackHeight(String datasetname) {
        Object value=getSetting(datasetname+"."+TRACK_HEIGHT);
        if (value!=null) return ((Integer)value).intValue(); 
        else {
            int newtrackHeight=6;
            Data dataset=gui.getEngine().getDataItem(datasetname);
                 if (dataset instanceof DNASequenceDataset) newtrackHeight=DEFAULT_TRACK_HEIGHT_DNA;
            else if (dataset instanceof NumericDataset) newtrackHeight=DEFAULT_TRACK_HEIGHT_NUMERIC;
            else if (dataset instanceof RegionDataset && ((RegionDataset)dataset).isMotifTrack()) newtrackHeight=DEFAULT_TRACK_HEIGHT_MOTIF;
            else if (dataset instanceof RegionDataset) newtrackHeight=DEFAULT_TRACK_HEIGHT_REGION;
            storeSetting(datasetname+"."+TRACK_HEIGHT,new Integer(newtrackHeight));
            return newtrackHeight;
        }
    }
    
    /**
     * A convenience method to set the height that should be used when drawing
     * the dataset (excluding borders)
     * @param datasetname The name of the dataset
     * @param height The height of the dataset track
     * @param redraw Set this to TRUE if visualization should be updated right away
     */
    public void setDataTrackHeight(String datasetname, int height, boolean redraw) {
            storeSetting(datasetname+"."+TRACK_HEIGHT,new Integer(height));
            if (redraw) {
                calculateTotalTracksHeight(); // recalculate total tracks height
                notifyListenersOfTrackOrderRearrangement(VisualizationSettingsListener.UPDATED,null);
                redraw();
            }
    }
     
   
    
    /**
     * A convenience method to decide the type of graph that should be used when drawing a datatrack 
     * @param datasetname The name of the dataset
     * @return The graph type for this dataset
     */
    public int getGraphType(String datasetname) {
        Object value=getSetting(datasetname+"."+GRAPH_TYPE);
        if (value!=null) return ((Integer)value).intValue(); 
        else {
            Data data=gui.getEngine().getDataItem(datasetname);
            if (data==null) return 0;
            int graphtype=BAR_GRAPH;
            if (data instanceof DNASequenceDataset) graphtype=DNA_GRAPH;
            else if (data instanceof NumericDataset) graphtype=BAR_GRAPH;
            else if (data instanceof RegionDataset && ((RegionDataset)data).isMotifTrack()) graphtype=REGION_MULTICOLOR_GRAPH;
            else if (data instanceof RegionDataset) graphtype=REGION_GRAPH;
            storeSetting(datasetname+"."+GRAPH_TYPE,new Integer(graphtype));
            return graphtype;
        }
    }
    
    /**
     * A convenience method to set the type of graph that should be used when drawing a datatrack
     * @param datasetname The name of the dataset
     * @param graphType The type of graph to use for this datattrack
     */
    public void setGraphType(String datasetname, int graphType) {
            storeSetting(datasetname+"."+GRAPH_TYPE,new Integer(graphType));
            redraw();
    }
    
    /**
     * A convenience method to decide whether or not the specified datatrack should be drawn at all
     * @param datasetname The name of the feature dataset
     * @return a boolean value which is true if the track should be displayed and false if it should be hidden
     */
    public boolean isVisible(String datasetname) {
        Object value=getSetting(datasetname+"."+VISIBLE);
        if (value!=null) return ((Boolean)value).booleanValue(); 
        else {
            storeSetting(datasetname+"."+VISIBLE,new Boolean(true));
            return true;
        }
    }
    
    /**
     * A convenience method to set the visibility for a datatrack or full
     * @param datasetname The name of the dataset
     * @param boolean True if the datatrack should be shown and false if it should be hidden
     */
    public void setVisible(String datasetname, boolean show) {
            storeSetting(datasetname+"."+VISIBLE,new Boolean(show));
            calculateTotalTracksHeight(); // recalculate total tracks height
            rearrangeDataTracks();
            redraw();
    }

    /**
     * A convenience method to set the visibility for several datatracks at once
     * @param datasetnames An array containing the names of the datasets
     * @param boolean True if the datatracks should be shown and false if they should be hidden
     */
    public void setVisible(String[] datasetnames, boolean show) {
            for (String datasetname:datasetnames) {
                storeSetting(datasetname+"."+VISIBLE,new Boolean(show));
            }
            calculateTotalTracksHeight(); // recalculate total tracks height
            rearrangeDataTracks();
            redraw();
    }

    /**
     * A convenience method to decide whether or not the specified sequence should be drawn at all
     * @param sequencename The name of the sequence
     * @return a boolean value which is true if the sequence should be displayed and false if it should be hidden
     */
    public boolean isSequenceVisible(String sequencename) {
        Object value=getSetting(sequencename+"."+VISIBLE);
        if (value!=null) return ((Boolean)value).booleanValue();
        else {
            storeSetting(sequencename+"."+VISIBLE,new Boolean(true));
            return true;
        }
    }

    /**
     * A convenience method to set the visibility for sequence
     * @param sequencename The name of the sequence
     * @param boolean True if the sequence should be shown and false if it should be hidden
     */
    public void setSequenceVisible(String sequencename, boolean show) {
            storeSetting(sequencename+"."+VISIBLE,new Boolean(show));
            notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.UPDATED, null);
            redraw();
    }

    /**
     * A convenience method to set the visibility for several sequences at once
     * @param sequencenames An array containing the names of the sequences
     * @param boolean True if the sequences should be shown and false if they should be hidden
     */
    public void setSequenceVisible(String[] sequencenames, boolean show) {
            for (String sequencename:sequencenames) {
                storeSetting(sequencename+"."+VISIBLE,new Boolean(show));
            }
            notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.UPDATED, null);
            redraw();
    }



    
    /**
     * Specifies that a subset of a sequence is marked as selected (this could for instance be a result of the user
     * selecting a region with the selection tool or zoom tool)
     * @param sequenceName The name of the sequence (format="datasetName.sequenceName")
     * @param start The genomic coordinate of the selection start
     * @param end The genomic coordinate of the selection end
     * @param clear If this is TRUE then all currently registered selection windows are cleared before registering the new selection
     * @param mergeoverlapping If this is TRUE then the current selection windows will be updated so
     *        that no two (or more) windows overlap (or are directly back-to-back) but rather are represented 
     *        by a single window that spans all the previous windows (and the newly submitted)
     */
    public void addSelectionWindow(String sequenceName, int start, int end, boolean clear, boolean mergeoverlapping) {
            if (clear) selections.clear();
            if (mergeoverlapping && !clear) {
                // remove windows that overlap and update bounds for newly submitted window
                Iterator itr = selections.iterator();
                while (itr.hasNext()) {
                  SelectionWindow window = (SelectionWindow) itr.next();
                  if (!window.sequenceName.equals(sequenceName)) continue;
                  if (!(window.end<start-1 || end<window.start-1)) { // not separate
                    itr.remove();
                    if (window.start<start) start=window.start;
                    if (window.end>end) end=window.end;                    
                  }
                }                
            } 
            selections.add(new SelectionWindow(sequenceName,start,end));            
            redraw(); // 
    }
  
    /** Removes all sequence selection windows*/
    public void clearSelectionWindows() {
            selections.clear();
            redraw(); // 
    }
    
    /**
     * Returns current set of sequence selections windows
     * or NULL if no selection windows are registered
     * Note that these might not be "live" updated while a user drags to select,
     * but are usually updated immediately after the user has finished with a selection
     */
    public ArrayList<SelectionWindow> getSelectionWindows() {
            if (selections.isEmpty()) return null;
            return selections;
    }
    
    /**
     * Returns the number of current active selection windows
     */
    public int getSelectionWindowCount() {
            return selections.size();
    }
    
    /**
     * Returns TRUE if the specified sequence has an active selection window
     */
    public boolean hasSelectionWindow(String sequenceName) {
            for (SelectionWindow window:selections) {
                if (window.sequenceName.equals(sequenceName)) return true;
            }
            return false;
    }

    /**
     * Sets the name of the sequence which has a currently selected subregion
     * Since only one sequence can have a current subregion selection at the time, this will "clear" any selections in other sequences
     * @param sequenceID The name of the sequence (format="datasetName.sequenceName")
     */
    public void setSequenceWithSelection(String sequenceID) {
            storeSetting(SEQUENCE_WITH_SELECTION,sequenceID);
            redraw(); // 
    }
  
    
    /**
     * Returns the name of the sequence which has a currently selected subregion
     * (selected by pressing the button somewhere inside a sequence and dragging to mark a region)
     * @return The name of the sequence (format="datasetName.sequenceName") which currently has a selected region, or null 
     */
    public String getSequenceWithSelection() {
            return (String)getSetting(SEQUENCE_WITH_SELECTION);
    }    
      
    
    /**
     * This method returns a list of the datatracks that should be visualized in the correct order
     * This list is based on the order of datasets in the "features" panel. 
     * Note that "hidden" datatracks are not included in this list! 
     * @return a list containing the datasets in order
     */
    public ArrayList<FeatureDataset> getDatatrackOrder() {
        if (datatrackorder==null) rearrangeDataTracks();
        return datatrackorder;
    }
    
    
    /**
     * Recalculates and stores the order of visible "feature" datatracks
     * This method should be called whenever the order or visibility of datatracks
     * is updated (such as when the user rearranges the order of datasets in the
     * "features" panel or selects hide or show on a datatrack).
     */
    public void rearrangeDataTracks() {
        FeaturesPanelListModel listmodel=gui.getFeaturesPanelListModel();
        if (listmodel==null) return;
        ArrayList<FeatureDataset> list=new ArrayList<FeatureDataset>();
        int size=listmodel.getSize();
        for (int i=0;i<size;i++) {
            FeatureDataset dataset=(FeatureDataset)listmodel.elementAt(i);
            if (isVisible(dataset.getName())) list.add(dataset);
        }
        datatrackorder=list;       
        //gui.debugMessage("rearrangeDataTracks");
        notifyListenersOfTrackOrderRearrangement(VisualizationSettingsListener.UPDATED,null);
    }
    
    /**
     * This method returns the total height (in pixels) of all visible
     * datatracks (including a 1px collapsed border!). For performance reasons, 
     * this value is recalculated and stored anew each time the visibility of 
     * a track is toggled or the height of a track is adjusted (rather than 
     * being calculated on the fly each time this method is called)
     * @return the total height of the tracks 
     */
    public int getTotalTracksHeight() {
        Object value=getSetting(TOTAL_TRACKS_HEIGHT);
        if (value!=null) return ((Integer)value).intValue(); 
        else {
            int height=calculateTotalTracksHeight();
            return height;
        }
    }
    
    /**
     * (Re)calculates and stores the total track height in pixels
     * This method should be called every time the visibility of a track is toggled
     * or when the height of a track is adjusted
     * @return the total height of the tracks (including 1px collapsable border) 
     */
    public int calculateTotalTracksHeight() {
        int height=0;
        FeaturesPanelListModel listmodel=gui.getFeaturesPanelListModel();
        if (listmodel!=null) {
          int size=listmodel.getSize();
          for (int i=0;i<size;i++) {
            FeatureDataset dataset=(FeatureDataset)listmodel.elementAt(i);
            String datasetname=dataset.getName();
            if (isVisible(datasetname)) height+=(getDataTrackHeight(datasetname)+1);
          }  
        }
        height+=1; // additional border
        return height;
    }
    
    
    
    
    /**
     * A convenience method to get the width of the window for dataset visualization (excluding borders)
     * @return The width of the sequence window
     */
    public int getSequenceWindowSize() {
        Object value=getPersistantSetting(SEQUENCE_WINDOW_SIZE);
        if (value!=null) {
            return ((Integer)value).intValue();
        } else {
            storePersistantSetting(SEQUENCE_WINDOW_SIZE,new Integer(DEFAULT_SEQUENCE_WINDOW_SIZE));
            return DEFAULT_SEQUENCE_WINDOW_SIZE;
        }
    }
    
    /**
     * A convenience method to set the width of the window for dataset visualization
     * @param width The new width of the sequence window
     */
    public void setSequenceWindowSize(int width) {
            storePersistantSetting(SEQUENCE_WINDOW_SIZE,new Integer(width));
            //rearrangeDataTracks();
            notifyListenersOfSequenceWindowSizeChange(width);
            redraw();
    }
    
    
    /**
     * This method returns the position of the start (lowest coordinate) of the view port in genomic coordinates. 
     * Note that the start of the viewport can (possibly) lie outside the sequence itself
     * (The datatrack visualizer should decide how to handle this)
     * @param sequenceName The name of the sequence itself 
     * @return The start of the viewport (genomic coordinates, direct strand)
     */
    public int getSequenceViewPortStart(String sequenceName) {
        Object value=getSetting(sequenceName+"."+VIEWPORT_START);
        if (value!=null) {
            return ((Integer)value).intValue();      
        } else {
            Sequence sequence=(Sequence)getEngine().getDataItem(sequenceName);
            int start=sequence.getRegionStart();
            storeSetting(sequenceName+"."+VIEWPORT_START, start);
            return start;
        }
    }
    
    /**
     * This method sets the position of the start (lowest coordinate) of the view port in genomic coordinates
     * Unless the sequence in question is "constrained", the start of the viewport can be set outside the 
     * region of the sequence itself. However, attempting to do this on a constrained sequence will result in
     * automatic repositioning according to the current settings.
     * Note also that a call to this function will not results in automatic update/redraw of sequences (for performance reasons)
     * 
     * @param sequenceName The name of the sequence itself 
     * @param start A position specifying the start of the viewport in genomic coordinates (direct strand)
     * */
    public void setSequenceViewPortStart(String sequenceName,int start) {
            if (isSequenceConstrained(sequenceName)) {
                double scale=getScale(sequenceName);
                Sequence sequence=(Sequence)getEngine().getDataItem(sequenceName);
                int sequenceSize=sequence.getSize();  
                int sequenceStart=sequence.getRegionStart();                  
                int sequenceEnd=sequence.getRegionEnd();  
                int end=start+(int)Math.ceil(getSequenceWindowSize()/scale)-1; // new end if VPstart is set to start
                boolean canFitInWindow=(sequenceSize<=getSequenceWindowSize()/scale);
                if (canFitInWindow) {
                    if (start>sequenceStart) start=sequenceStart;
                    else if (end<sequenceEnd) start=sequenceEnd-(int)Math.ceil(getSequenceWindowSize()/scale)+1;
                } else { // sequence is at current scale too large to fit within window
                    if (start<sequenceStart) start=sequenceStart;
                    else if (end>sequenceEnd) start=sequenceEnd-(int)Math.ceil(getSequenceWindowSize()/scale)+1;
                }
            }
            storeSetting(sequenceName+"."+VIEWPORT_START,new Integer(start));
            //redraw(); // calling redraw will update all sequences (which is a bit heavy when only one is changed)
    }
 
    /**
     * This method sets the position of the end (highest coordinate) of the view port in genomic coordinates
     * Note that the start of the viewport can be set outside the region of the sequence itself
     * (The datatrack visualizer should decide how to handle this)
     * Note also that a call to this function will not results in automatic update/redraw of sequences
     * (for performance reasons)
     * 
     * @param sequenceName The name of the sequence itself 
     * @param end A position specifying the end of the viewport in genomic coordinates (direct strand)
     * */
    public void setSequenceViewPortEnd(String sequenceName,int end) {
            // We do not set VP-end directly, since the VP-end is always calculated on the fly with respect to start and scale.
            // So instead we calculate and set a new VP-start
            int start=end-(int)Math.ceil(getSequenceWindowSize()/getScale(sequenceName))+1;                
            storeSetting(sequenceName+"."+VIEWPORT_START,new Integer(start));
            //redraw(); // calling redraw will update all sequences (which is a bit heavy when only one is changed)
    }
 
    
    
    
    
    /**
     * This method returns the position of the end (highest coordinate) of the view port in genomic coordinates
     * Note that the start of the viewport can (possibly) lie outside the sequence itself
     * (The datatrack visualizer should decide how to handle this)
     * @param sequenceName The name of the sequence itself 
     * @return The start of the viewport (genomic coordinate)
     */
    public int getSequenceViewPortEnd(String sequenceName) {
        // this value is inferred from viewPortStart in combination with scale and windowSize
        // note that the casting to (float) below is quite necessary, or else you will have problems once in a blue moon
        return getSequenceViewPortStart(sequenceName)+(int)Math.ceil((float)getSequenceWindowSize()/(float)getScale(sequenceName))-1;
    }
    

    
    /**
     * This method sets a new viewport to use for visualization of the given sequence (in genomic coordinates)
     * Note that the range of the viewport can be set outside the region of the underlying sequence
     * ViewPort should be specified relative to direct strand so that "start" is less than or equal to "end"
     * 
     * @param sequenceName The name of the sequence itself 
     * @param start A position specifying the start of the viewport (in genomic coordinates)
     * @param end A position specifying the end of the viewport (in genomic coordinates)
     * */
    public void setSequenceViewPort(String sequenceName,int start, int end) {
            storeSetting(sequenceName+"."+VIEWPORT_START,new Integer(start));
            // you must recalculate zoomFactor and set that also!
            double width=end-start+1;
            double scale=getSequenceWindowSize()/width;
            setSequenceZoomLevel(sequenceName,scale*100.0); // this will redraw!
            //redraw(); 
    }
    
    
    /**
     * This method sets a new viewport to use for visualization of a sequence so that
     * the given genomic coordinate aligns with the given screen position (=coordinate within a component)
     * The zoom-level will be kept as is, only the translation will be affected.
     * 
     * @param sequenceName The name of the sequence itself 
     * @param genomic A genomic coordinate
     * @param screen A screen position (offset within component)
     * @param alignment If alignment==ALIGNMENT_LEFT  the viewport will be set so that the genomic coordinate is the first within the screen pixel (eg. for very small scales)
     *                  If alignment==ALIGNMENT_RIGHT the viewport will be set so that the genomic coordinate is the last within the screen pixel (eg. for very small scales)
     *                  else the viewport will be set so that the genomic coordinate is in the middle of the pixel (round to the left)
     * */
    public void alignSequenceViewPort(String sequenceName,int genomic, int screen, String alignment) {
            int viewPortStart=getSequenceViewPortStart(sequenceName);
            double scale=getScale(sequenceName);
            int[] genomicScreen=getGenomicCoordinateFromScreenOffset(sequenceName, screen);
            int currentlyUnderScreenPixel;
            if (alignment.equals(ALIGNMENT_LEFT)) currentlyUnderScreenPixel=genomicScreen[0];
            else if (alignment.equals(ALIGNMENT_RIGHT)) currentlyUnderScreenPixel=genomicScreen[1];
            else if (alignment.equals(ALIGNMENT_TSS)) currentlyUnderScreenPixel=genomicScreen[0];
            else currentlyUnderScreenPixel=(int)((genomicScreen[0]+genomicScreen[1])/2);
            int screendiff=genomic-currentlyUnderScreenPixel; // number of genomic coordinates from start of window to screen pixel
            double genomicdiff=screendiff;//scale;
            int newVPstart=(int)(viewPortStart+genomicdiff);
            //gui.debugMessage(sequenceName+": genomic="+genomic+"  screen="+screen+"  scale="+scale+"  currentlyUnderScreenPixel="+currentlyUnderScreenPixel+" ["+genomicScreen[0]+","+genomicScreen[1]+"]   screendiff="+screendiff+"  genomicdiff="+genomicdiff+"  oldVPstart="+viewPortStart+"  newVPstart="+newVPstart+" alignment="+alignment);
            //storeSetting(sequenceName+"."+VIEWPORT_START,new Integer(newVPstart));
            setSequenceViewPortStart(sequenceName,newVPstart);
            //redraw(); // calling redraw will update all sequences (which is a bit heavy when only one is changed)
    }
    
    
    
    /**
     * Returns the zoom level (percentage value) for the specified sequence.
     * If this sequence has han individually specified zoom level, this value will be returned.
     * If not, the global zoom level will be used instead. 
     * A zoom percentage of 100 means that there is a 1-to-1 correspondence between bases and pixels 
     * (i.e. one base per pixel). A zoom value of 200 means one base takes up 2 pixels, while zoom=50 
     * means that the values of 2 neighboring bases should be represented by 1 pixel.
     * @param sequenceName The name of the sequence itself 
     * @return The zoom percentage 
     */
    public double getSequenceZoomLevel(String sequenceName) {
        Object value=getSetting(sequenceName+"."+ZOOM_LEVEL);
        if (value!=null) {
            return ((Double)value).doubleValue();
        } else {
            return getGlobalZoomLevel();
        }
    }
    
    /**
     * Sets the zoom level (percentage value) for the specified sequence.
     * A zoom percentage of 100 means that there is a 1-to-1 correspondence between bases and pixels 
     * (i.e. one base per pixel). A zoom value of 200 means one base takes up 2 pixels, while zoom=50 
     * means that the values of 2 neighboring bases should be represented by 1 pixel.
     * 
     * @param sequenceName The name of the sequence itself 
     * @param zoompercentage
     */
    public void setSequenceZoomLevel(String sequenceName, double zoompercentage) {
            storeSetting(sequenceName+"."+ZOOM_LEVEL,new Double(zoompercentage));
    }

    /**
     * Sets the zoom level (percentage value) for the specified sequence to the next zoom level in 
     * the preset list which is greater than the current zoom level. If the current alignment setting
     * for this sequence is something other than NONE the ViewPort will be repositioned according to
     * alignment settings
     * 
     * @param sequenceName The name of the sequence itself 
     */
    public void zoomInOnSequence(String sequenceName) {   
          double zoomLevel=getSequenceZoomLevel(sequenceName);
          int index;
          for (index=0;index<zoomChoices.length-1;index++) {
              if (zoomChoices[index]>zoomLevel) {break;}
          }
          zoomLevel=zoomChoices[index]; // new zoom level
          zoomToLevel(sequenceName,zoomLevel);
    }

    
     /**
     * Sets the zoom level (percentage value) for the specified sequence to the next zoom level in 
     * the preset list which is smaller than the current zoom level. If the current alignment setting
     * for this sequence is something other than NONE the ViewPort will be repositioned according to
     * alignment settings
     * 
     * @param sequenceName The name of the sequence itself 
      */
    public void zoomOutOnSequence(String sequenceName) {
          double zoomLevel=getSequenceZoomLevel(sequenceName);
          int index;
          for (index=zoomChoices.length-1;index>0;index--) {
              if (zoomChoices[index]<zoomLevel) {break;}
          }
          zoomLevel=zoomChoices[index]; // new zoom level
          zoomToLevel(sequenceName,zoomLevel);
    }

     /**
     * Sets the zoom level (percentage value) for the specified sequence to the next zoom level in 
     * the preset list which is smaller than the current zoom level. If the current alignment setting
     * for this sequence is something other than NONE the ViewPort will be repositioned according to
     * alignment settings
     * 
     * @param sequenceName The name of the sequence itself 
      */
    public void zoomToLevel(String sequenceName,double zoomLevel) {
          int fixedScreenPoint=1;
          int genomicAnchor=1;
          boolean cheatonrightalign=false;
          String alignment=getSequenceAlignment(sequenceName);
          Sequence datasequence=(Sequence)getEngine().getDataItem(sequenceName);
          int sequenceStart=datasequence.getRegionStart();
          int sequenceEnd=datasequence.getRegionEnd();
          int shownOrientation=getSequenceOrientation(sequenceName);
          if (alignment.equals(ALIGNMENT_RIGHT))  {
              if ((shownOrientation==DIRECT && getSequenceViewPortEnd(sequenceName)==sequenceEnd) || (shownOrientation==REVERSE && getSequenceViewPortStart(sequenceName)==sequenceStart) ) cheatonrightalign=true;
          }
          double scale=zoomLevel/100.0;
          if (alignment.equals(ALIGNMENT_TSS)) {
                 Integer TSSint=datasequence.getTSS();
                 if (TSSint!=null) genomicAnchor=TSSint.intValue();
                 else genomicAnchor=(datasequence.getStrandOrientation()==VisualizationSettings.DIRECT)?sequenceEnd:sequenceStart;                   
                 int[] range=getScreenCoordinateFromGenomic(sequenceName, genomicAnchor);
                 fixedScreenPoint=(int)(Math.floor((double)range[0]/scale)*scale);   
          } else if (alignment.equals(ALIGNMENT_LEFT)) {
                 genomicAnchor=(shownOrientation==VisualizationSettings.DIRECT)?sequenceStart:sequenceEnd;                   
                 int[] range=getScreenCoordinateFromGenomic(sequenceName, genomicAnchor);
                 fixedScreenPoint=(int)(Math.floor((double)range[0]/scale)*scale);     
                 //gui.debugMessage("ZoomOut "+sequenceName+" scale="+scale+", screen-range=["+range[0]+","+range[1]+"]  range0="+range[0]+" corrected to "+fixedScreenPoint);                 
          } else if (alignment.equals(ALIGNMENT_RIGHT)) {
                 genomicAnchor=(shownOrientation==VisualizationSettings.REVERSE)?sequenceStart:sequenceEnd;                   
                 int[] range=getScreenCoordinateFromGenomic(sequenceName, genomicAnchor);
                 fixedScreenPoint=(int)(Math.floor((double)range[1]/scale)*scale);                              
                 //gui.debugMessage("ZoomOut "+sequenceName+" scale="+scale+", screen-range=["+range[0]+","+range[1]+"]  range1="+range[1]+" corrected to "+fixedScreenPoint);                 
                 int overflow=(int)(Math.ceil(getSequenceWindowSize()/scale)-Math.floor(getSequenceWindowSize()/scale));
                 //gui.debugMessage("Overflow="+overflow);
                 fixedScreenPoint+=overflow;
          } else { // align to center point in window
                 fixedScreenPoint=(int)(getSequenceWindowSize()/2.0);
                 int[] gc=getGenomicCoordinateFromScreenOffset(sequenceName, fixedScreenPoint);
                 //genomicAnchor=(alignedToGene)?gc[0]:gc[1];                  
                 genomicAnchor=(int)((gc[0]+gc[1]))/2;                  
          }
          setSequenceZoomLevel(sequenceName,zoomLevel);
          if (cheatonrightalign) {
               if (shownOrientation==VisualizationSettings.DIRECT) setSequenceViewPortEnd(sequenceName, sequenceEnd);                
               else setSequenceViewPortStart(sequenceName, sequenceStart);
          } else {
            if (shownOrientation==REVERSE && alignment.equals(ALIGNMENT_RIGHT)) alignment=ALIGNMENT_LEFT;
            else if (shownOrientation==REVERSE && alignment.equals(ALIGNMENT_LEFT)) alignment=ALIGNMENT_RIGHT;
            alignSequenceViewPort(sequenceName, genomicAnchor, fixedScreenPoint,alignment); 
          }
          redraw();
    }    
    
    
    
    /**
     * Sets the zoom level (percentage value) for the specified sequence to the next zoom level in 
     * the preset list which is greater than the current zoom level and adjusts the viewPort so that
     * the genomic position under the given screen coordinate will remain fixed in the same position
     * after scaling (unless constraints on the sequence prohibits this. In which case the sequence 
     * will be repositioned automatically once more to conform with the constraints) 
     * 
     * @param sequenceName The name of the sequence itself 
     * @param x the screen coordinate which should be fixed (first position=1)
     */   
    public void zoomInOnSequence(String sequenceName,int x) {
        int gc[]=getGenomicCoordinateFromScreenOffset(sequenceName, x); // store genomic coordinate under cursor
        zoomInOnSequence(sequenceName); // zoom
        int gcoord=(int)((gc[0]+gc[1])/2);
        alignSequenceViewPort(sequenceName, gcoord, x, ALIGNMENT_NONE); // translate so that stored genomic coordinate aligns with cursor
        redraw();
    }
    
    /**
     * Sets the zoom level (percentage value) for the specified sequence to the next zoom level in 
     * the preset list which is smaller than the current zoom level and adjusts the viewPort so that
     * the genomic coordinate under the given screen coordinate will remain fixed in the same position 
     * after scaling (unless constraints on the sequence prohibits this. In which case the sequence 
     * will be repositioned automatically once more to conform with the constraints) 
     * 
     * @param sequenceName The name of the sequence itself 
     * @param x the screen coordinate which should be fixed (first position=1)
     */
    public void zoomOutOnSequence(String sequenceName,int x) {
        int gc[]=getGenomicCoordinateFromScreenOffset(sequenceName, x); // store genomic coordinate under cursor
        zoomOutOnSequence(sequenceName); // zoom
        int gcoord=(int)((gc[0]+gc[1])/2);
        alignSequenceViewPort(sequenceName, gcoord, x, ALIGNMENT_NONE); // translate so that stored genomic coordinate aligns with cursor
        redraw();
    }
    
    /**
     * Sets the zoom level and repositions the sequence so that the full sequence 
     * fits exactly within the display window
     *
     * @param sequenceName The name of the sequence itself 
     */
    public void zoomToFitSequence(String sequenceName) {
            double width=getSequenceWindowSize();
            Sequence sequence=(Sequence)getEngine().getDataItem(sequenceName);
            int start=sequence.getRegionStart();
            double sequenceLength=sequence.getSize();
            double scale=width/sequenceLength;
            setSequenceZoomLevel(sequenceName,scale*100.0);
            setSequenceViewPortStart(sequenceName, start);
            redraw();
    }
    
   
    
     /**
     * Returns the global zoom level (percentage value) for all sequences. 
     * A zoom percentage of 100 means that there is a 1-to-1 correspondence between bases and pixels 
     * (i.e. one base per pixel). A zoom value of 200 means one base takes up 2 pixels, while zoom=50 
     * means that the values of 2 neighboring bases should be represented by 1 pixel.
     * @return The zoom percentage 
     */
    public double getGlobalZoomLevel() {
        Object value=getSetting(ZOOM_LEVEL);
        if (value!=null) {
            return ((Double)value).doubleValue();
        } else {
            storeSetting(ZOOM_LEVEL,new Double(DEFAULT_ZOOM_LEVEL));
            return DEFAULT_ZOOM_LEVEL;
        }
    }
    
    /**
     * Sets the global zoom level (percentage value) for all sequences. 
     * (However, individual sequences can choose to override this setting using setSequenceZoomLevel() )
     * A zoom percentage of 100 means that there is a 1-to-1 correspondence between bases and pixels 
     * (i.e. one base per pixel). A zoom value of 200 means one base takes up 2 pixels, while zoom=50 
     * means that the values of 2 neighboring bases should be represented by 1 pixel.
     * @param zoompercentage
     */
    public void setGlobalZoomLevel(double zoompercentage) {
            //clearAllSettingsFor(ZOOM_LEVEL); // override all sequence specific settings if a new global zoom level is set!
            storeSetting(ZOOM_LEVEL,new Double(zoompercentage));
            redraw();
    }
    

    /**
     * Returns the scale level to be used for visualization of the given sequence.
     * Scale is related to zoomlevel in that the zoomlevel=scale*100
     * (Thus zoom=100 equals scale=1.0, zoom=25 equals scale=0.25 etc.)
     * @return scale level
     */
    public double getScale(String sequenceName){
        double zoomlevel=getSequenceZoomLevel(sequenceName);
        return zoomlevel/100.0;
    }
    
    /**
     * Returns the strand orientation to use for visualization of the specified sequence.
     * (The orientation to use for visualization can be selected by the user independent of
     * the original orientation of the sequence relative to its target gene).
     * This class provides two static constants which can be used to determine and compare orientation: DIRECT and REVERSE
     * @param sequenceName The name of the sequence itself 
     * @return An integer specifying orientation (DIRECT or REVERSE)
     */
    public int getSequenceOrientation(String sequenceName) {
        Object value=getSetting(sequenceName+"."+SEQUENCE_ORIENTATION);
        if (value!=null) {
            return ((Integer)value).intValue();
        } else {
            int orientation=((Sequence)gui.getEngine().getDataItem(sequenceName)).getStrandOrientation();
            storeSetting(sequenceName+"."+SEQUENCE_ORIENTATION,new Integer(orientation));
            return orientation;
        }
    }
    
    /**
     * Sets the strand orientation to use for visualization of the specified sequence.
     * (The orientation to use for visualization can be selected by the user independent of
     * the original orientation of the sequence relative to its target gene).
     * This class provides two static constants which can be used to determine and compare orientation: DIRECT and REVERSE

     * @param sequenceName The name of the sequence itself 
     * @param orientation An integer specifying orientation (DIRECT or REVERSE)
     */
    public void setSequenceOrientation(String sequenceName, int orientation) {
            storeSetting(sequenceName+"."+SEQUENCE_ORIENTATION,new Integer(orientation));
            redraw();
    }
    
    
    /**
     * Sets the zoom level (percentage value) for the specified sequence to the next zoom level in 
     * the preset list which is greater than the current zoom level and adjusts the viewPort so that
     * the genomic position under the given screen coordinate will remain fixed in the same position
     * after scaling
     * 
     * @param sequenceName The name of the sequence itself 
     * @param x The screen coordinate which should be fixed (first position=1)
     */  
    public void flipSequenceAroundFixedPoint(String sequenceName, int x) {
        int gc[]=getGenomicCoordinateFromScreenOffset(sequenceName, x); // store genomic coordinate under cursor
        int neworientation=(getSequenceOrientation(sequenceName)==DIRECT)?REVERSE:DIRECT;       
        storeSetting(sequenceName+"."+SEQUENCE_ORIENTATION,new Integer(neworientation));
        int gcoord=(int)((gc[0]+gc[1])/2);
        alignSequenceViewPort(sequenceName, gcoord, x, ALIGNMENT_NONE); // translate so that stored genomic coordinate aligns with cursor       
        redraw();
    }
                               
    
    /**
     * Returns the sequence alignment to use for visualization of the specified sequence.
     * This class provides two static constants which can be used to determine and compare alignments: LEFT and RIGHT
     * 
     * @param sequenceName The name of the sequence itself 
     * @return A String specifying alignment (LEFT or RIGHT)
     */
    public String getSequenceAlignment(String sequenceName) {
        Object value=getSetting(sequenceName+"."+SEQUENCE_ALIGNMENT);
        if (value!=null) {
            return (String)value;
        } else {// 
            String alignment=getDefaultSequenceAlignment();       
            storeSetting(sequenceName+"."+SEQUENCE_ALIGNMENT,alignment);
            return alignment;
        }
    }
    
    /**
     * Sets the sequence alignment to use for visualization of the specified sequence.
     * This class provides static constants which can be used to determine and compare alignments
     * (Note that calling this method does not results in any viewport updates to align sequences,
     * it only stores the settings for future references.)
     * If alignment is set to TSS the sequence will automatically be released from constraints
     * via call to setSequenceConstrained(...,...,false)
     * 
     * @param sequenceName The name of the sequence itself 
     * @param alignment A String specifying alignment (LEFT, RIGHT, TSS or NONE)
     */
    public void setSequenceAlignment(String sequenceName, String alignment) {
            storeSetting(sequenceName+"."+SEQUENCE_ALIGNMENT,alignment);
            if (alignment.equals(ALIGNMENT_TSS)) setSequenceConstrained(sequenceName, false);
    }    
    
    /**
     * Returns the default alignment to be used if nothing else is selected by the user
     * The default setting is persistant and can be set with setDefaultSequenceAlignment
     * @return The default alignment
     */
    public String getDefaultSequenceAlignment() {
        Object value=getPersistantSetting(SEQUENCE_DEFAULT_ALIGNMENT);
        if (value!=null) {
            return (String)value;
        } else {// if nothing else is specified align right. I think that is an OK default
            String alignment=ALIGNMENT_RIGHT;       
            storePersistantSetting(SEQUENCE_DEFAULT_ALIGNMENT,alignment);
            return alignment;
        }        
    }
    
    /**
     * Sets the default alignment to be used if nothing else is selected by the user
     * This setting is persistant
     * @param alignment The new default alignment
     */
    public void setDefaultSequenceAlignment(String alignment) {     
           storePersistantSetting(SEQUENCE_DEFAULT_ALIGNMENT,alignment);
    }

    /**
     * Gets the position of origo for this sequence in genomic coordinates
     * 
     * @param sequenceName The name of the sequence itself 
     * @return The position of origo in genomic coordinates
     */
    @Deprecated
    public int getSequenceOrigo(String sequenceName) {
        Object value=getSetting(sequenceName+"."+SEQUENCE_ORIGO);
        if (value!=null) {
            return ((Integer)value).intValue();
        } else {
            int origo=1; //
            storeSetting(sequenceName+"."+SEQUENCE_ORIGO,new Integer(origo));
            return origo;
        }
    }
    
    /**
     * Sets the position of origo for this sequence in genomic coordinates
     * 
     * @param sequenceName The name of the sequence itself 
     * @param origo The new position of origo (in genomic coordinates)
     */
    @Deprecated
    public void setSequenceOrigo(String sequenceName, int origo) {
            storeSetting(sequenceName+"."+RULER,new Integer(origo));
            redraw();
    }
    
    

    
    
    /**
     * Returns a string that describes the type of ruler to be used for the specified sequence
     * @param sequenceName The name of the sequence itself 
     * @return A string specifying the type of ruler to use
     */
    public String getRuler(String sequenceName) {
        String key=null;
        Object value=null;
        if (sequenceName!=null) {
            key=sequenceName+"."+RULER;
            value=getSetting(key);
        } 
        if (value==null) {
            key=RULER;
            value=getSetting(key);
        }
        if (value!=null) {
            return (String)value;
        } else {
            String ruler=RULER_TSS;
            storeSetting(key,ruler);
            return ruler;
        }
    }

    
    /**
     * Sets the constraint on the specified sequence 
     * If set to TRUE the sequence should be "constrained" meaning:
     * 1) if the zoom-level is such that the entire sequence can fit within the window, 
     *    no part of the sequence should be allowed to be moved outside the window 
     * 2) if the sequence is larger than the window, the sequence should not be allowed 
     *    to move such that it no longer fills the entire window.
     * 
     * @param sequenceName The name of the sequence itself 
     * @param constrained TRUE if the sequence should be constrained or FALSE if it should be allowed to move freely
     */
    public void setSequenceConstrained(String sequenceName, boolean constrained) {
            storeSetting(sequenceName+"."+CONSTRAINED, constrained);
    }
    
    /**
     * Returns TRUE if the specified sequence is "contrained", which means that:
     * 1) if the zoom-level is such that the entire sequence can fit within the window, 
     *    no part of the sequence should be allowed to be moved outside the window 
     * 2) if the sequence is larger than the window, the sequence should not be allowed 
     *    to move such that it no longer fills the entire window.
     * 
     * @param sequenceName The name of the sequence itself 
     */
    public boolean isSequenceConstrained(String sequenceName) {
        Object value=getSetting(sequenceName+"."+CONSTRAINED);
        if (value!=null) {
            return ((Boolean)value).booleanValue();
        } else {    
            storeSetting(sequenceName+"."+CONSTRAINED,Boolean.TRUE);
            return true;
        }
    }

    /**
     * Sets the scroll lock on the specified sequence 
     * If set to TRUE the sequence is not allowed to autoscroll when mouse cursor is dragged outside sequence window
     * 
     * @param sequenceName The name of the sequence itself 
     * @param lock TRUE if the sequence should not be allowed to autoscroll
     */
    public void setSequenceScrollLock(String sequenceName, boolean lock) {
            storeSetting(sequenceName+"."+SCROLL_LOCK, lock);
    }
    
    /**
     * Returns TRUE if the specified sequence should not be allowed to autoscroll when mouse cursor is dragged outside sequence window
     * 
     * @param sequenceName The name of the sequence itself 
     */
    public boolean isSequenceScrollLocked(String sequenceName) {
        Object value=getSetting(sequenceName+"."+SCROLL_LOCK);
        if (value!=null) {
            return ((Boolean)value).booleanValue();
        } else {    
            storeSetting(sequenceName+"."+SCROLL_LOCK,Boolean.TRUE);
            return true;
        }
    }


    /**
     * Sets the type of ruler to be used for this sequence
     * @param sequenceName The name of the sequence itself 
     * @param ruler A string specifying the type of ruler to use
     */
    public void setRuler(String sequenceName, String ruler) {
            String key=RULER;
            if (sequenceName!=null) key=sequenceName+"."+RULER;
            storeSetting(key,ruler);
            redraw();
    }

    /**
     * Returns the width to use for the sequence labels in the visualization panel
     * Depending on the current preferences settings, this could either be a fixed width
     * or be based on the width of the longest sequence name
     * @return The label width
     */
    public int getSequenceLabelWidth() {    
        if (getScaleSequenceLabelsToFit()) {
            Object value=getSetting(SEQUENCE_LABEL_WIDTH);
            if (value!=null) {
                return ((Integer)value).intValue();
            } else {
                return getSequenceLabelFixedWidth();
            }             
        } else return getSequenceLabelFixedWidth();
       
    }

    /**
     * Returns a fixed width to use for the sequence labels in the visualization panel
     * @return The label width
     */
    public int getSequenceLabelFixedWidth() {       
        Object value=getPersistantSetting(SEQUENCE_LABEL_FIXED_WIDTH);
        if (value!=null) {
            return ((Integer)value).intValue();
        } else {
            storePersistantSetting(SEQUENCE_LABEL_FIXED_WIDTH,new Integer(DEFAULT_SEQUENCE_LABEL_WIDTH));
            return DEFAULT_SEQUENCE_LABEL_WIDTH;
        }        
    }
    
    /**
     * Sets the maximum fixed width to use for the sequence labels in the visualization panel
     * @param width
     */
    public void setSequenceLabelFixedWidth(int width) {
            storeSetting(SEQUENCE_LABEL_FIXED_WIDTH,new Integer(width));
            rearrangeDataTracks();
            redraw();
    }
    
    /**
     * Sets the width of the widest label currently in use
     * @param width
     */
    public void setSequenceLabelMaxWidth(int width) {
            storePersistantSetting(SEQUENCE_LABEL_WIDTH,new Integer(width));
            rearrangeDataTracks();
            redraw();
    }
    
    
    /**
     * Returns a value that specifies whether the sequence labels should be scaled 
     * to the width of the longest sequence name (true) or use a fixed width (false)
     */
    public boolean getScaleSequenceLabelsToFit() {       
        Object value=getPersistantSetting(SCALE_SEQUENCE_LABELS_TO_FIT);
        if (value!=null) {
            return ((Boolean)value).booleanValue();
        } else {
            storePersistantSetting(SCALE_SEQUENCE_LABELS_TO_FIT,Boolean.TRUE);
            return Boolean.TRUE;
        }        
    }
    
    /**
     * Specifies whether the sequence labels should be scaled to the width
     * of the longest sequence name (true) or use a fixed width (false)
     * @param doScale
     */
    public void setScaleSequenceLabelsToFit(boolean doScale) {
            storePersistantSetting(SCALE_SEQUENCE_LABELS_TO_FIT,new Boolean(doScale));
            rearrangeDataTracks();
            redraw();
    }
    
    
    /**
     * Gets the antialiasing setting for text. If this is set to true, small text 
     * in the visualization panel (such as coordinates for the ruler and control panel)
     * will be drawn with an antialiased font
     */
    public boolean useTextAntialiasing() {       
        Object value=getPersistantSetting(ANTIALIAS_TEXT);
        if (value!=null) {
            return ((Boolean)value).booleanValue();
        } else {
            storePersistantSetting(ANTIALIAS_TEXT,Boolean.TRUE);
            return Boolean.TRUE;
        }        
    }
    
    /**
     * Sets the antialiasing setting for text. If this is set to true, small text 
     * in the visualization panel (such as coordinates for the ruler and control panel)
     * will be drawn with an antialiased font
     * @param antialias
     */
    public void setUseTextAntialiasing(boolean antialias) {
            storePersistantSetting(ANTIALIAS_TEXT,new Boolean(antialias));
            redraw();
    }
    
    /**
     * Gets the antialiasing setting for motif logos. If this is set to true,
     * the base letters in the motif sequence logos will be drawn with an antialiased font
     */
    public boolean useMotifAntialiasing() {       
        Object value=getPersistantSetting(ANTIALIAS_MOTIFS);
        if (value!=null) {
            return ((Boolean)value).booleanValue();
        } else {
            storePersistantSetting(ANTIALIAS_MOTIFS,Boolean.TRUE);
            return Boolean.TRUE;
        }        
    }
    
    /**
     * Sets the antialiasing setting for motif logos. If this is set to true,
     * the base letters in the motif sequence logos will be drawn with an antialiased font     
     * @param antialias
     */
    public void setUseMotifAntialiasing(boolean antialias) {
            storePersistantSetting(ANTIALIAS_MOTIFS,new Boolean(antialias));
            redraw();
    }
    
    
    
    
    /**
     * A convenience method to decide whether or not the score of regions should be visualized
     * @param datasetname The name of the dataset
     * @return a boolean value which is true if the the score of regions should be visualized
     */
    public boolean shouldVisualizeRegionScore(String datasetname) {
        Object value=getSetting(datasetname+"."+VISUALIZE_REGION_SCORE);
        if (value!=null) return ((Boolean)value).booleanValue(); 
        else {
            boolean defaultvalue=false;
            Data dataitem=gui.getEngine().getDataItem(datasetname);
            if (dataitem instanceof RegionDataset && ((RegionDataset)dataitem).isMotifTrack()) defaultvalue=true;
            storeSetting(datasetname+"."+VISUALIZE_REGION_SCORE,new Boolean(defaultvalue));
            return false;
        }
    }
    
    /**
     * A convenience method to select whether or not the score of regions should be visualized
     * by setting the height of regions relative to the score
     * @param datasetname The name of the dataset
     * @param boolean True if region score should be visualized
     */
    public void setVisualizeRegionScore(String datasetname, boolean show) {
            storeSetting(datasetname+"."+VISUALIZE_REGION_SCORE,new Boolean(show));
            redraw();
    }    

    /**
     * A convenience method to decide whether or not the strand of regions should be visualized
     * @param datasetname The name of the dataset
     * @return a boolean value which is true if the the strand of regions should be visualized
     */
    public boolean shouldVisualizeRegionStrand(String datasetname) {
        Object value=getSetting(datasetname+"."+VISUALIZE_REGION_STRAND);
        if (value!=null) return ((Boolean)value).booleanValue(); 
        else {
            boolean defaultvalue=false;
            Data dataitem=gui.getEngine().getDataItem(datasetname);
            if (dataitem instanceof RegionDataset && ((RegionDataset)dataitem).isMotifTrack()) defaultvalue=true;
            storeSetting(datasetname+"."+VISUALIZE_REGION_STRAND,new Boolean(defaultvalue));
            return false;
        }
    }
    
    /**
     * A convenience method to select whether or not the strand of regions should be visualized
     * by setting the height of regions relative to the strand
     * @param datasetname The name of the dataset
     * @param boolean True if region score should be visualized
     */
    public void setVisualizeRegionStrand(String datasetname, boolean show) {
            storeSetting(datasetname+"."+VISUALIZE_REGION_STRAND,new Boolean(show));
            redraw();
    }    

    
    
    
    /** Returns a list (or double[]) containing some preset zoom-level choices
     * @return a double[] with preset zoomlevels
     */
    public double[] getZoomChoices() {
        return zoomChoices;
    }
    
    
    
    
    /**
     * given a "screen coordinate" from a mouse event within a component, this function will calculate and return
     * the corresponding genomic coordinate (or range if zoom is less than 100.0)  
     * The function returns an int[] with two numbers. If scale is 1.0 or more, these two numbers will be 
     * the same and reflect the genomic coordinate corresponding to the screen coordinate given 
     * (orientation considered). 
     * 
     * @param sequenceName The name of the sequence itself 
     * @param x the offset within the component window (first window pixel should be 0 so exclude border!)
     * @return range of genomic coordinates corresponding to screen coordinates 
     */
    public int[] getGenomicCoordinateFromScreenOffset(String sequenceName, int x) {
        int viewPortStart=getSequenceViewPortStart(sequenceName);
        int viewPortEnd=getSequenceViewPortEnd(sequenceName);
        double scale=getScale(sequenceName);
        int orientation=getSequenceOrientation(sequenceName);
        /*if (scale>=1.0) {
            int baseoffset=(int)(x/scale); // the number of bases shown from start of window to position x
            int pos=0;
            if (orientation==FeatureSequenceData.DIRECT) {pos=viewPortStart+baseoffset;}
            else {pos=viewPortEnd-baseoffset;}
            return new int[]{pos,pos};
        *///} else {
            int firstoffset=(int)Math.floor(x/scale);
            int lastoffset=(int)Math.ceil((x+1)/scale)-1;
            int first=0,last=0;
            if (orientation==Sequence.DIRECT) {first=viewPortStart+firstoffset;last=viewPortStart+lastoffset;}
            else {first=viewPortEnd-lastoffset;last=viewPortEnd-firstoffset;}
            return new int[]{first,last};
        //}     
    } 
    
    /**
     * given a "genomic coordinate" from a mouse event, this function will calculate and return
     * the corresponding "screen coordinate" (offset within component) or range if zoom is more than 100.0  
     * The function returns an int[] with two numbers. If scale is 1.0 or less, these two numbers will be 
     * the same and reflect the screen coordinate corresponding to the genomic coordinate given 
     * (orientation considered). 
     * 
     * @param sequenceName The name of the sequence itself 
     * @param x the genomic coordinate
     * @return range of genomic coordinates corresponding to screen coordinates 
     */
    public int[] getScreenCoordinateFromGenomic(String sequenceName, int x) {
        int viewPortStart=getSequenceViewPortStart(sequenceName);
        int viewPortEnd=getSequenceViewPortEnd(sequenceName);
        double scale=getScale(sequenceName);
        int orientation=getSequenceOrientation(sequenceName);
        //gui.debugMessage(sequenceName+"  VPstart="+viewPortStart+" VPend="+viewPortEnd+" scale="+scale+" orientation="+orientation);
        /*
        if (scale<=1.0) { // the width of 1 base is equal to or less than 1 pixel. 
            int pos=0;    // Note that it could still be the case that the base spans 2 pixels!!!
            if (orientation==Sequence.DIRECT) {pos=(int)(((x-viewPortStart)*scale)+0.5);}
            else {pos=(int)(((viewPortEnd-x+1)*scale)+0.5)-1;}
            return new int[]{pos,pos};
        } else { // 1 base spans 1 or more pixels
         */
            int first=0,last=0;
            if (orientation==Sequence.DIRECT) {
                first=(int)Math.floor((x-viewPortStart)*scale);
                last=(int)Math.ceil((x+1-viewPortStart)*scale)-1;
            }
            else {
                first=(int)Math.floor((viewPortEnd-x)*scale);
                last=(int)Math.ceil((viewPortEnd-x+1)*scale)-1;
            }
            return new int[]{first,last};
       // }     
    } 
        
    
    /**
     * A call to this method forces the GUI to be redrawn after settings have been altered
     */
    public void redraw() {
       if (SwingUtilities.isEventDispatchThread()) {
           gui.getTopPanel().repaint();
       } else {
           Runnable runner = new Runnable() {
               public void run() {
                   gui.getTopPanel().repaint();
               }
           };
           SwingUtilities.invokeLater(runner);
       }
       
    }
 
    
    /** This is called from FeaturesPanelListModel when the order of tracks changes */
    public void setupMasterTrackorderList(FeaturesPanelListModel model) {
        ArrayList<String> neworder=new ArrayList<String>();
        for (int i=0;i<model.size();i++) { // populate with the current list
            neworder.add(((Data)model.getElementAt(i)).getName());
        }
        // now check if there are any more left in the master list and incorporate those also
        for (int i=0;i<datatrackorderMasterList.size();i++) {
            String name=datatrackorderMasterList.get(i);
            if (neworder.contains(name)) continue;
            if (i==0) { // first element in masterlist is not in new list. Place it on top!
                neworder.add(0, name);
            } else {
                String previous=datatrackorderMasterList.get(i-1);
                int newindex=neworder.indexOf(previous);
                //System.err.println("Datatrack '"+name+"' was previously after "+previous);
                neworder.add(newindex+1, name);
            }
        }
        datatrackorderMasterList=neworder;
        //System.err.println("MasterList\n==========");
        //for (String name:datatrackorderMasterList) System.err.println("  "+name);
    }
    
    /** 
     * Returns the correct position in the track order for the dataset with the given name
     * If a dataset with the given name has never been encountered before, the value -1 will be returned
     */
    public int getTrackOrderPosition(String name, FeaturesPanelListModel model) {
        //System.err.println("VisualizationSettings.getTrackOrderPosition");
        if (!datatrackorderMasterList.contains(name)) return -1;
        else {
            int i=datatrackorderMasterList.indexOf(name);
            if (i==0) {
                //System.err.println("Datatrack '"+name+"' was previously on top in:"+datatrackorderMasterList.toString());
                return 0;
            }
            int newindex=-1;
            while (newindex<0 && i>0){
               String previous=datatrackorderMasterList.get(i-1);
               newindex=getPositionInModel(previous,model);
               i--;
            }            
            //Object[] l=model.toArray(); String list="";
            //for (Object el:l) list+=el.toString()+",";
            //System.err.println("Datatrack '"+name+"' was previously after "+previous+" in:"+datatrackorderMasterList.toString());
            //System.err.println("Datatrack '"+name+"' should now be located after "+previous+" at "+newindex+" in:"+list);           
            return newindex+1; 
        }    
    }
    
    private int getPositionInModel(String name, FeaturesPanelListModel model) {
        int newindex=-1;
        for (int x=0;x<model.size();x++) {
           if (((Data)model.get(x)).getName().equals(name)) {newindex=x;break;}
        } 
        return newindex;
    }
     /**
     * This method goes through all settings and clears those that contain
     * the specified key string as a substring of their key.
     * (Eg: if subkey="zoom" then settings with keys such as "zoom", "sequencename.zoom" 
     * and "kazoom" will be removed.)
     * @param subkey
     */
    private void clearAllSettingsFor(String subkey) {
        Set<String> keys=settings.keySet();
        ArrayList<String> toDelete=new ArrayList<String>();
        for (String key:keys) {
            if (key.indexOf(subkey)>0) toDelete.add(key);
        }
        for (String key:toDelete) {
            settings.remove(key);
        }
    }   
    
    public void debug() { // prints all settings to log
        ArrayList<String> keys=new ArrayList<String>(settings.keySet());
        Collections.sort(keys);
        for (String k:keys) {
            Object val=settings.get(k);
            if (val==null) debugOutput("Viz:  "+k+" => NULL");
            else debugOutput("VisualizationSettings:  "+k+" => "+val.toString());
        }
        debugOutput("=== MasterList ===");
        for (String k:datatrackorderMasterList) {
            debugOutput("   "+k);
        }     
        debugOutput("-------------");
    }

    private void debugOutput(String msg) {
        if (gui!=null) gui.debugMessage(msg);
        else System.err.println(msg);
    }
    
    /* Listener callback method that the engine calls when Data is added (or datasets have sequences added to them) */
    public void dataAdded(Data data) {
        //gui.debugMessage("Data Added    data="+data.getName()+" class="+data.getClass().toString());
        if (data instanceof FeatureDataset || data instanceof Sequence) notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.ADDED,data);
    }
    
    /* Listener callback method that the engine calls when Data objects representing datasets have items added to them */
    public void dataAddedToSet(Data parent, Data child) {
        //gui.debugMessage("Data AddedToSet   parent="+parent.getName()+" class="+parent.getClass().toString()+",  child="+child.getName()+" class="+child.getClass().toString());
        //if (parent instanceof FeatureDataset || getEngine().isDefaultSequenceCollection(parent)) notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.ADDED,parent);
    }

    /* Listener callback method that the engine calls when Data is removed (or datasets have sequences removed from them) */
    public void dataRemoved(Data data) {
        if (data instanceof FeatureDataset || data instanceof Sequence) notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.REMOVED,data);
    }
    
    /* Listener callback method that the engine calls when Data objects representing datasets have items reomved from them */
    public void dataRemovedFromSet(Data parent, Data child) {
        //gui.debugMessage("Data AddedToSet parent="+parent.getName()+" class="+parent.getClass().toString()+",  child="+child.getName()+" class="+child.getClass().toString());
        if (parent instanceof FeatureDataset || getEngine().isDefaultSequenceCollection(parent)) notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.REMOVED,parent);
    }

    /* Listener callback method that the engine calls when a Data item has been updated */
    public void dataUpdated(Data data) {
        //gui.debugMessage("Data updated "+data.getName()+" class="+data.getClass().toString());
        if (data instanceof FeatureDataset || getEngine().isDefaultSequenceCollection(data)) notifyListenersOfSequenceOrderRearrangement(VisualizationSettingsListener.UPDATED,data);
    }

    /* Listener callback method that the engine calls just prior to data update */
    public void dataUpdate(Data oldvalue, Data newvalue) {
        // this event is not important for this class
    }

    
    
    
    
    
    /** Listener interface method called by DatasetPanels when contents has changed */
    public void contentsChanged(ListDataEvent e) {
        
    }

    /** Listener interface method called by the FeaturesPanel when order of items is changed (specifically when a new item is inserted) */
    public void intervalAdded(ListDataEvent e) {
        FeaturesPanelListModel listmodel=(FeaturesPanelListModel)e.getSource();
        if (listmodel==gui.getFeaturesPanelListModel()) rearrangeDataTracks(); // is the event coming from the "Features" panel
        
    }

    /** listener interface method called by the FeaturesPanel when order of items is changed (specifically when an item is removed from the list) */
    public void intervalRemoved(ListDataEvent e) {
        FeaturesPanelListModel listmodel=(FeaturesPanelListModel)e.getSource();
        if (listmodel==gui.getFeaturesPanelListModel()) rearrangeDataTracks(); // is the event coming from the "Features" panel
    }
    
    /**
     * Registers a new VisualizationSettingsListener that will be notified when important settings changes
     * @param listener
     */
    public void addVisualizationSettingsListener(VisualizationSettingsListener listener) {
        synchronized(listeners) {
            listeners.add(listener);
        }
    }
    
    
    /** This method functions as a sort of 'master switch' to turn on or off VisualizationSettingsListener notifications */
    public void enableVisualizationSettingsNotifications(boolean enable) {
        enableNotifications=enable;
    }
    
    /**
     * Removes the specified VisualizationSettingsListener
     * @param listener
     */
    public void removeVisualizationSettingsListener(VisualizationSettingsListener listener) {
        synchronized(listeners) {
          listeners.remove(listener);
        }
    }
    
    public void notifyListenersOfTrackOrderRearrangement(int type, Data affected) {
        //Since the original list of listeners might change as the result of a callback method invokation, it is necessary to synchronize and clone the listto avoid ConcurrentModificationExceptions 
        //HashSet listenerList=(HashSet)listeners.clone();
        if (!enableNotifications) return;
        HashSet listenerList;
        synchronized(listeners) {   
            listenerList=(HashSet)listeners.clone();
        }
        for (Object listener:listenerList) {
            ((VisualizationSettingsListener)listener).trackReorderEvent(type, affected);
        }
        
    }
    
    public void notifyListenersOfSequenceOrderRearrangement(int type, Data affected) {
        //Since the original list of listeners might change as the result of a callback method invokation, it is necessary to synchronize and clone the list to avoid ConcurrentModificationExceptions
        if (!enableNotifications && type!=VisualizationSettingsListener.REMOVED) return; // remove notifications should always be passed on!
        HashSet listenerList;
        synchronized(listeners) {
            listenerList=(HashSet)listeners.clone();
        }
        for (Object listener:listenerList) {
            ((VisualizationSettingsListener)listener).sequencesReorderEvent(type, affected);
        }
    }
    
    public void notifyListenersOfSequenceWindowSizeChange(int newsize) {
        if (!enableNotifications) return;
        for (Object listener:listeners) {
            ((VisualizationSettingsListener)listener).sequenceWindowSizeChangedEvent(newsize);
        }
    }
    
    public void notifyListenersOfSequenceMarginChange(int newsize) {
        if (!enableNotifications) return;
        for (Object listener:listeners) {
            ((VisualizationSettingsListener)listener).sequenceMarginSizeChangedEvent(newsize);
        }
    }
    
    public void notifyListenersOfDataRenameEvent(String oldname,String newname) {
        if (!enableNotifications) return;
        for (Object listener:listeners) {
            ((VisualizationSettingsListener)listener).dataRenamedEvent(oldname,newname);
        }
    }    
       
// --------------------------------------------------------------------------------------------
   //private float[] huemap=new float[]{0, 0.1f, 0.15f, 0.3f, 0.48f, 0.56f, 0.65f, 0.76f, 0.82f, 0.9f}; // size=7                               

    private float[] huemap=new float[]{0, 0.15f, 0.65f, 0.3f, 0.48f, 0.76f, 0.09f, 0.56f, 0.82f, 0.9f}; // size=7
    private float[] valmap=new float[]{
           1,    1,    1,    1,    1, 
        0.75f, 0.75f, 0.75f, 0.75f, 0.75f, 
        0.45f, 0.45f, 0.45f, 0.45f, 0.45f, 
        0.88f, 0.88f, 0.88f, 0.88f, 0.88f,
        0.60f, 0.60f, 0.60f, 0.60f, 0.60f, 
    }; 
    private float[] satmap=new float[]{
           1, 
        0.75f, 
        0.50f, 
        0.85f, 
        0.65f 
    }; 
            
    /** 
     * Given an integer number, this method returns a "unique" color (among a palette of 250)
     * corresponding to the index
     */
    public Color getColorFromIndex(int i) {
        i=i%250; // only 250 different colors
        //float[] map=new float[]{1, 0.7f, 0.4f, 0.9f, 0.5f, 0.8f, 0.6f}; // size=7

        int valline=i/10;
        int valindex=valline%valmap.length;
        float brightness=valmap[valindex];            
        int hueindex=(i%huemap.length);
        float hue=huemap[hueindex];

        int satindex=valline%satmap.length;
        float saturation=satmap[satindex];            
        //hue+=0.01*satindex;
        return new Color(Color.HSBtoRGB(hue, saturation, brightness));
    }    
    
    /** Imports VisualizationSettings from another object */
    public void importSettings(VisualizationSettings other) {
         this.colorpointer=other.colorpointer;
         this.settings=other.settings;
         this.datatrackorder=other.datatrackorder;
         this.datatrackorderMasterList=other.datatrackorderMasterList;
         //this.colorpointer=other.colorpointer;   
         //debug();
    }
    
    
    /**
     * Returns a hex string representing the given color. Eg "#FFFF00" for yellow
     * @param color
     * @return
     */
    public static String convertColorToHTMLrepresentation(Color color) {
        String red=Integer.toHexString(color.getRed());
        String green=Integer.toHexString(color.getGreen());
        String blue=Integer.toHexString(color.getBlue());
        if (red.length()==1) red="0"+red;
        if (green.length()==1) green="0"+green;
        if (blue.length()==1) blue="0"+blue;
        return ("#"+red+green+blue).toUpperCase();
    }
    
}
