/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.operationdialog;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.border.Border;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.OperationTask;
import priorseditor.engine.operations.Condition_within;
import priorseditor.engine.operations.Operation;
import priorseditor.engine.protocol.DataTypeTable;
import priorseditor.gui.PriorsEditorGUI;

/**
 * This class is the top class specifying GUIs (both renderers and editors) for
 * operations within the GUI interface.
 * 
 * @author kjetikl
 */
public abstract class OperationDialog extends JDialog {
    protected OperationTask parameters;
    protected PriorsEditorEngine engine;
    private boolean okPressed=false;
    protected JButton okButton;
    protected JButton cancelButton;
    private JPanel buttonsPanel;
    private JPanel buttonsInternalPanel;
    //protected Border commonBorder=BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(4,4,4,4), BorderFactory.createEtchedBorder());
    protected Border commonBorder=BorderFactory.createEmptyBorder(4,4,4,4);
    private DataTypeTable lookup;
    protected JComboBox sourceDataCombobox=null; 
    protected JTextField targetDataTextfield=null; 
    private JPanel sourceTargetPanel=null;
    private ConditionPanel_within windowSelectionPanel=null;
    protected ImageIcon informationIcon=null;
    protected PriorsEditorGUI gui=null;

    

    /** Creates new form TestExtendOperationDialog */
    public OperationDialog(java.awt.Frame parent) {
        super(parent);               
    }
    
    /** 
     * Creates a new OperationsDialog. 
     * The initialize() method should be called immediately after construction to set required parameters
     */
    public OperationDialog() {
        super();
    }
        
    /** This methtod should be called immediately after construction whenever an empty constructor is used */
    public void initialize(OperationTask parameters, DataTypeTable lookuptable, PriorsEditorGUI gui) {
        //System.err.println("Initialize["+parameters.getOperationName()+"]: lookuptable="+lookuptable+"   size="+lookuptable.size());
        this.setModal(true);
        this.parameters=parameters;
        this.engine=(PriorsEditorEngine)parameters.getParameter(OperationTask.ENGINE);
        this.gui=gui;
        if (lookuptable!=null) lookup=lookuptable;
        else {
           lookup=new DataTypeTable(engine);
           lookup.populateFromEngine();
        }
        String operationName=parameters.getOperation().getName();
        setTitle("  "+operationName);
        setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
        setResizable(false);
        java.net.URL iconURL=getClass().getResource("../resources/icons/help_icon.png");
        if (iconURL!=null) informationIcon=new ImageIcon(iconURL);  
        initComponents();   
    }
  
    public ImageIcon getInformationIcon() {return informationIcon;}
    
    public ExecutableTask getOperationTask() {
        return parameters;
    }
    
    public JPanel getOKCancelButtonsPanel() {
        return buttonsPanel;
    }
    
    public JPanel getRangeSelectionPanel() {
        return windowSelectionPanel;
    }
        
    public boolean okPressed() {return okPressed;}
    protected void setOK(boolean ok) {okPressed=ok;}
    
    protected void initComponents() {
        okButton=new JButton("  OK  ");
        cancelButton=new JButton("Cancel");
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setOK(true);
                setParameters();
                setVisible(false);
            }
        });
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okPressed=false;
                setVisible(false);
            }
        });
        buttonsPanel=new JPanel();
        buttonsInternalPanel=new JPanel();
        buttonsInternalPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        buttonsInternalPanel.add(okButton);
        buttonsInternalPanel.add(cancelButton);  
        buttonsPanel.setLayout(new BorderLayout());
        buttonsPanel.add(buttonsInternalPanel,BorderLayout.EAST);
        getRootPane().setDefaultButton(okButton);
        initSourceTargetPanel();
        Condition_within windowselection=(Condition_within)parameters.getParameter("within");
        Operation operation=parameters.getOperation();
        if (operation.isSubrangeApplicable() && windowselection!=null) {
            windowSelectionPanel=new ConditionPanel_within(windowselection, this);
            buttonsPanel.add(windowSelectionPanel,BorderLayout.WEST);
        }        
    }
    
    /**
     * In this method the parameters set by the user in the GUI dialog
     * should be parsed and transferred to the OperationTask object
     */
    protected void setParameters() {
        String sourceName=(String)sourceDataCombobox.getSelectedItem();
        parameters.setParameter(OperationTask.SOURCE_NAME, sourceName);
        String targetName=targetDataTextfield.getText();
        if (targetName!=null) targetName.trim();
        if (targetName==null || targetName.isEmpty()) targetName=sourceName;
        parameters.setParameter(OperationTask.TARGET_NAME, targetName);
        if (windowSelectionPanel!=null) {
            Condition_within within=windowSelectionPanel.getCondition();
            parameters.setParameter("within",within);
        }
    }
    
    /** Returns (as DefaultComboBoxModel) a list of names for all dataitems of the given class 
     * that will potentially be available at this point in the protocol 
     * either because a data item has been registered with the engine outside the protocol
     * or a data item has been defined earlier in the protocol
     */      
    public DefaultComboBoxModel getDataCandidates(Class candidateClass) {
         return getDataCandidates(new Class[]{candidateClass},false);
    }
    
    /** Returns (as DefaultComboBoxModel) a list of names for all dataitems of the given class 
     * that will potentially be available at this point in the protocol 
     * either because a data item has been registered with the engine outside the protocol
     * or a data item has been defined earlier in the protocol
     */      
    public DefaultComboBoxModel getDataCandidates(Class candidateClass, boolean topBlank) {
         return getDataCandidates(new Class[]{candidateClass},topBlank);
    }
    
    /** Returns (as DefaultComboBoxModel) a list of names for all dataitems of the given classes 
     * that will potentially be available at this point in the protocol
     * either because a data item has been registered with the engine outside the protocol
     * or a data item has been defined earlier in the protocol
     */    
    public DefaultComboBoxModel getDataCandidates(Class[] candidateClasses) {
         return getDataCandidates(candidateClasses,false);
    }
    
    public DefaultComboBoxModel getDataCandidates(Class[] candidateClasses, boolean topBlank) {
        ArrayList<String>candidateNames=lookup.getAllDataItemsOfType(candidateClasses);
        Collections.sort(candidateNames);
        int size=candidateNames.size();
        if (topBlank) size++;
        String[] entries=new String[size];
        int i=0;
        if (topBlank) {
            entries[i]="";
            i++;
        }
        for (String dataName:candidateNames) {
            entries[i]=dataName;
            i++;
        }
        DefaultComboBoxModel model=new DefaultComboBoxModel(entries);
        return model;       
    }  
    
    public DefaultListModel getDataCandidatesAsListModel(Class[] candidateClasses) {
        DefaultListModel model=new DefaultListModel();
        ArrayList<String>candidateNames=lookup.getAllDataItemsOfType(candidateClasses);
        Collections.sort(candidateNames);
        for (String dataName:candidateNames) {
            model.addElement(dataName);
        }        
        return model;       
    }  
   
    
    /** Returns true if a data item with the given will potentially exists
     *  either because it is already registered with the engine, or because
     *  the name has been defined earlier in the protocol script 
     */
    public boolean dataItemNameInUse(String name) {
        return lookup.contains(name);
    }
    
    public DataTypeTable getDataTypeTable() {
        return lookup;
    }
    
    /** Returns the class for the dataitem with the given name based on */
    public Class getClassForDataItem(String name) {
        if (lookup==null || name==null) return null;
        return lookup.getClassFor(name);
    }
    
    protected JPanel getSourceTargetPanel() {return sourceTargetPanel;} 

    private void initSourceTargetPanel() {
        sourceTargetPanel=new JPanel();
        sourceTargetPanel.setLayout(new BoxLayout(sourceTargetPanel, BoxLayout.LINE_AXIS));
        JPanel sPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        JPanel tPanel=new JPanel(new FlowLayout(FlowLayout.RIGHT));
        Class[] sourceCandidates=parameters.getOperation().getDataSourcePreferences();
        String sourceName=parameters.getSourceDataName();
        String targetName=parameters.getTargetDataName();
        if (targetName==null || targetName.isEmpty()) targetName=sourceName;
        DefaultComboBoxModel model=(DefaultComboBoxModel)getDataCandidates(sourceCandidates);
        sourceDataCombobox=new JComboBox(model);
        sourceDataCombobox.setEditable(true);
        if (sourceName!=null && !sourceName.isEmpty()) sourceDataCombobox.setSelectedItem(sourceName);
        sPanel.add(new JLabel("Source  "));
        sPanel.add(sourceDataCombobox);
        targetDataTextfield=new JTextField(targetName);
        targetDataTextfield.setColumns(16);
        tPanel.add(new JLabel("       Store results in  "));
        tPanel.add(targetDataTextfield);  
        sourceDataCombobox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                targetDataTextfield.setText(sourceDataCombobox.getSelectedItem().toString());
            }
        });        
        sourceTargetPanel.add(sPanel);
        sourceTargetPanel.add(tPanel);
    }
}
