/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.operationdialog;


import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import priorseditor.engine.data.*;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.operations.Operation_motifDiscovery;
import priorseditor.external.MotifDiscovery;
import priorseditor.gui.ParametersPanel;


/**
 *
 * @author kjetikl
 */
public class OperationDialog_motifDiscovery extends OperationDialog {
    private JComboBox sourceDataCombobox=null; 
    private JTextField targetDataTextfield=null; 
    private JTextField motifCollectionTextfield=null; 
    private JTextField motifprefixTextfield=null; 
    private JPanel sourceTargetPanel=null;
    private JPanel sequenceSubsetPanel;
    private JPanel algorithmPanel;
    private JPanel additionalParametersPanel;
    private JComboBox sequenceSubsetCombobox;
    private JComboBox algorithmComboBox; 
    private ParametersPanel parametersPanel=null;
    private ParameterSettings parameterSettings=null;
    private String[] initialEditValues=new String[3]; // if set to other than NULL, this array contains the original algorithm, trackname, and motifcollection name for a protocol line that is being edited

    public OperationDialog_motifDiscovery(JFrame parent) {
        super(parent); 
        //initComponents();
    }
    
    public OperationDialog_motifDiscovery() {
        super();
    }    
    
    public DefaultComboBoxModel getMotifDiscoveryAlgorithms() {
         String[] list=engine.getAvailableMotifDiscoveryAlgorithms();
         DefaultComboBoxModel newmodel=new DefaultComboBoxModel(list);
         return newmodel;
    } 
    
    
    
    @Override
    public void initComponents() {
        super.initComponents();  
        setResizable(false);
        setIconImage(gui.getFrame().getIconImage());        
        String algorithmName=(String)parameters.getParameter(Operation_motifDiscovery.ALGORITHM);
        String motifprefix=(String)parameters.getParameter(Operation_motifDiscovery.MOTIFPREFIX);
        String sourceName=parameters.getSourceDataName();
        parameterSettings=(ParameterSettings)parameters.getParameter(Operation_motifDiscovery.PARAMETERS);   
        if (parameterSettings==null) {
            parameterSettings=new ParameterSettings();
            parameters.setParameter(Operation_motifDiscovery.PARAMETERS,parameterSettings);    
        }
        initAlgorithmPanel(sourceName,algorithmName,motifprefix);
        initSourceTargetPanel();
        initSequenceSubsetPanel();
        algorithmPanel.setBorder(commonBorder);    
        sourceTargetPanel.setBorder(commonBorder);    
        sequenceSubsetPanel.setBorder(commonBorder);
        additionalParametersPanel=new JPanel();
        additionalParametersPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
        additionalParametersPanel.setBorder(commonBorder);
        add(sourceTargetPanel);   
        add(algorithmPanel);
        JScrollPane scrollPane=new JScrollPane(additionalParametersPanel);  
        scrollPane.setMaximumSize(new Dimension(600,400));        
        add(scrollPane);
        add(sequenceSubsetPanel);
        add(getOKCancelButtonsPanel());
        Object formatSelected=algorithmComboBox.getSelectedItem();
        algorithmComboBox.setSelectedItem(formatSelected); // programatic 'click' to show the initial panel (if any applicable)
        pack();
        if (sourceName!=null && sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedItem(sourceName);
        else {
           if (sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedIndex(0);
           sourceName=(String)sourceDataCombobox.getSelectedItem();
        }
        if (motifprefix!=null && motifprefixTextfield!=null) {
            motifprefixTextfield.setText(motifprefix);
            motifprefixTextfield.setCaretPosition(0);
        }

    }

    @Override
    protected void setParameters() {
        String sourceName=(String)sourceDataCombobox.getSelectedItem();       
        parameters.setParameter(OperationTask.SOURCE_NAME, sourceName);
        String sequenceCollection=(String)sequenceSubsetCombobox.getSelectedItem();
        if (!sequenceSubsetCombobox.isEnabled() || sequenceCollection==null || sequenceCollection.equals(engine.getDefaultSequenceCollectionName())) sequenceCollection=null;
        parameters.setParameter(OperationTask.SEQUENCE_COLLECTION_NAME, sequenceCollection);
        String algorithmName="motifDiscovery";
        String prefix=algorithmName;
        if (algorithmComboBox.getItemCount()==0) setOK(false); // if no algorithms exists the OK button will effectively just cancel the dialog
        else {
            algorithmName=(String)algorithmComboBox.getSelectedItem();
            if (motifprefixTextfield!=null) prefix=(String)motifprefixTextfield.getText().trim();
            if (prefix.isEmpty()) prefix=algorithmName;
            parameters.setParameter(Operation_motifDiscovery.ALGORITHM, algorithmName);
            parameters.setParameter(Operation_motifDiscovery.MOTIFPREFIX, prefix);
            if (parametersPanel!=null) {
                parametersPanel.setParameters();
                parameters.setParameter(Operation_motifDiscovery.PARAMETERS, parametersPanel.getParameterSettings());
            }            
        }
        String targetName=targetDataTextfield.getText();
        targetName.trim();
        if (targetName==null || targetName.isEmpty()) targetName=algorithmName+"_sites"; 
        String motifCollectionName=motifCollectionTextfield.getText();
        motifCollectionName.trim();
        if (motifCollectionName==null || targetName.isEmpty()) motifCollectionName=algorithmName+"_motifs";        
        parameters.setParameter(OperationTask.TARGET_NAME, targetName);        
        parameters.setParameter(Operation_motifDiscovery.MOTIFCOLLECTION, motifCollectionName);
        parameters.reserveDataName(motifCollectionName);
    }
    
    private void initSequenceSubsetPanel() {
        sequenceSubsetPanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        sequenceSubsetPanel.add(new JLabel("In sequence collection  "));
        String selectedName=(String)parameters.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
        if (selectedName==null) selectedName=engine.getDefaultSequenceCollectionName();
        DefaultComboBoxModel model=(DefaultComboBoxModel)getDataCandidates(new Class[]{SequenceCollection.class});
        sequenceSubsetCombobox=new JComboBox(model);
        sequenceSubsetCombobox.setSelectedItem(selectedName);
        sequenceSubsetPanel.add(sequenceSubsetCombobox);                
    } 
    
     private void initAlgorithmPanel(String sourceName, String algorithmName, String prefix) {
        algorithmPanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        algorithmPanel.add(new JLabel("Algorithm  "));
        algorithmComboBox=new JComboBox(getMotifDiscoveryAlgorithms());        
        if (algorithmName==null){
            if (algorithmComboBox.getItemCount()>0) algorithmComboBox.setSelectedIndex(0);
            algorithmName=(String)algorithmComboBox.getSelectedItem();
        }
        else algorithmComboBox.setSelectedItem(algorithmName);
        algorithmPanel.add(algorithmComboBox);   
        algorithmComboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object selected=algorithmComboBox.getSelectedItem();
                if (selected!=null) {                    
                    showParametersPanel(getAlgorithmSettingsPanel((String)selected));
                    if (motifprefixTextfield!=null) {
                        motifprefixTextfield.setText((String)selected);
                        motifprefixTextfield.setCaretPosition(0);
                    }
                    if (initialEditValues[0]!=null && ((String)selected).equals(initialEditValues[0])) {
                       targetDataTextfield.setText(initialEditValues[1]);
                       motifCollectionTextfield.setText(initialEditValues[2]);
                    } else {
                       targetDataTextfield.setText(gui.getGenericDataitemName("BindingSites_"+((String)selected),getDataTypeTable()));
                       motifCollectionTextfield.setText(gui.getGenericDataitemName("Motifs_"+((String)selected),getDataTypeTable()));
                    }
                }
                else additionalParametersPanel.removeAll();
                pack();
                //setLocation(gui.getFrame().getWidth()/2-getWidth()/2, gui.getFrame().getHeight()/2-getHeight()/2);               
            }
        });
        if (prefix==null) prefix=algorithmName;
        motifprefixTextfield=new JTextField(10);
        motifprefixTextfield.setText(prefix);
        motifprefixTextfield.setCaretPosition(0);
        algorithmPanel.add(new JLabel("        Motif prefix  "));
        algorithmPanel.add(motifprefixTextfield);
    }  
    

    private void initSourceTargetPanel() {
        sourceTargetPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        JPanel leftVertical=new JPanel();
        JPanel rightVertical=new JPanel();
        leftVertical.setLayout(new BoxLayout(leftVertical, BoxLayout.Y_AXIS));
        rightVertical.setLayout(new BoxLayout(rightVertical, BoxLayout.Y_AXIS));
        JPanel sourcePanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        JPanel trackPanel=new JPanel(new FlowLayout(FlowLayout.RIGHT));
        JPanel motifsPanel=new JPanel(new FlowLayout(FlowLayout.RIGHT));
        String algorithmName=(String)algorithmComboBox.getSelectedItem();
        Class[] sourceCandidates=parameters.getOperation().getDataSourcePreferences();
        String sourceName=parameters.getSourceDataName();
        String targetName=parameters.getTargetDataName();
        String motifCollectionName=(String)parameters.getParameter(Operation_motifDiscovery.MOTIFCOLLECTION);
        initialEditValues[0]=(String)parameters.getParameter(Operation_motifDiscovery.ALGORITHM);
        initialEditValues[1]=targetName;
        initialEditValues[2]=motifCollectionName;
        if (targetName==null || targetName.isEmpty()) targetName=gui.getGenericDataitemName("BindingSites_"+algorithmName,getDataTypeTable());
        if (motifCollectionName==null || motifCollectionName.isEmpty()) motifCollectionName=gui.getGenericDataitemName("Motifs_"+algorithmName,getDataTypeTable());
        DefaultComboBoxModel model=(DefaultComboBoxModel)getDataCandidates(sourceCandidates);
        sourceDataCombobox=new JComboBox(model);
        if (sourceName!=null && sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedItem(sourceName);
        else {
           if (sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedIndex(0);
           sourceName=(String)sourceDataCombobox.getSelectedItem();
        }
        JPanel tmp=new JPanel();
        sourcePanel.add(new JLabel("Source  "));
        sourcePanel.add(sourceDataCombobox);
        tmp.add(new JLabel("  "));
        leftVertical.add(sourcePanel);
        leftVertical.add(tmp);  
        motifCollectionTextfield=new JTextField(motifCollectionName);
        motifCollectionTextfield.setColumns(16);
        targetDataTextfield=new JTextField(targetName);
        targetDataTextfield.setColumns(16);
        trackPanel.add(new JLabel("        Store sites in  "));
        trackPanel.add(targetDataTextfield);  
        motifsPanel.add(new JLabel("       Store motifs in  "));
        motifsPanel.add(motifCollectionTextfield); 
        rightVertical.add(trackPanel);
        rightVertical.add(motifsPanel); 
        sourceTargetPanel.add(leftVertical);
        sourceTargetPanel.add(rightVertical);
    }  

    
    
    
    /** */
    private void showParametersPanel(JPanel panel) {
        additionalParametersPanel.removeAll();
        additionalParametersPanel.add(panel);        
    }
   
    /** */
    private JPanel getAlgorithmSettingsPanel(String algorithmName) {
        MotifDiscovery algorithm=(MotifDiscovery)engine.getExternalProgram(algorithmName);
        if (algorithm==null) {
            parametersPanel=null;
            return new JPanel();
        }
        ParametersPanel panel=new ParametersPanel(algorithm.getParameters(),parameterSettings, this);
        parametersPanel=panel;
        return panel;
    }
    

}
