/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.operationdialog;


import java.awt.Dimension;
import java.awt.FlowLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import priorseditor.engine.data.*;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.operations.Operation_motifScanning;
import priorseditor.external.MotifScanning;
import priorseditor.gui.ParametersPanel;


/**
 *
 * @author kjetikl
 */
public class OperationDialog_motifScanning extends OperationDialog {
    private JComboBox sourceDataCombobox=null; 
    private JTextField targetDataTextfield=null; 
    private JPanel sourceTargetPanel=null;
    private JPanel sequenceSubsetPanel;
    private JPanel algorithmPanel;
    private JPanel additionalParametersPanel;
    private JComboBox sequenceSubsetCombobox;
    private JComboBox algorithmComboBox; 
    private ParametersPanel parametersPanel=null;
    private ParameterSettings parameterSettings=null;
    private String[] initialEditValues=new String[2]; // if set to other than NULL, this array contains the original algorithm and trackname for a protocol line that is being edited
    
    public OperationDialog_motifScanning(JFrame parent) {
        super(parent); 
        //initComponents();
    }
    
    public OperationDialog_motifScanning() {
        super();
    }    
    
    public DefaultComboBoxModel getMotifScanningAlgorithms() {
         String[] list=engine.getAvailableMotifScanningAlgorithms();
         DefaultComboBoxModel newmodel=new DefaultComboBoxModel(list);
         return newmodel;
    } 
    
    
    
    @Override
    public void initComponents() {
        super.initComponents();  
        setResizable(false);
        setIconImage(gui.getFrame().getIconImage());        
        String algorithmName=(String)parameters.getParameter(Operation_motifScanning.ALGORITHM);
        String sourceName=parameters.getSourceDataName();
        parameterSettings=(ParameterSettings)parameters.getParameter(Operation_motifScanning.PARAMETERS);   
        if (parameterSettings==null) {
            parameterSettings=new ParameterSettings();
            parameters.setParameter(Operation_motifScanning.PARAMETERS,parameterSettings);    
        }
        initAlgorithmPanel(sourceName,algorithmName);
        initSourceTargetPanel();
        initSequenceSubsetPanel();
        algorithmPanel.setBorder(commonBorder);    
        sourceTargetPanel.setBorder(commonBorder);    
        sequenceSubsetPanel.setBorder(commonBorder);
        additionalParametersPanel=new JPanel();
        additionalParametersPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
        additionalParametersPanel.setBorder(commonBorder);
        add(sourceTargetPanel);   
        add(algorithmPanel);
        JScrollPane scrollPane=new JScrollPane(additionalParametersPanel);  
        scrollPane.setMaximumSize(new Dimension(500,420));
        add(scrollPane);
        add(sequenceSubsetPanel);
        add(getOKCancelButtonsPanel());
        Object formatSelected=algorithmComboBox.getSelectedItem();
        algorithmComboBox.setSelectedItem(formatSelected); // programatic 'click' to show the initial panel (if any applicable)
        pack();
        if (sourceName!=null && sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedItem(sourceName);
        else {
           if (sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedIndex(0);
           sourceName=(String)sourceDataCombobox.getSelectedItem();
        }
    }

    @Override
    protected void setParameters() {
        String sourceName=(String)sourceDataCombobox.getSelectedItem();       
        parameters.setParameter(OperationTask.SOURCE_NAME, sourceName);
        String targetName=targetDataTextfield.getText();
        targetName.trim();
        if (targetName==null || targetName.isEmpty()) targetName=sourceName;
        parameters.setParameter(OperationTask.TARGET_NAME, targetName);        
        String sequenceCollection=(String)sequenceSubsetCombobox.getSelectedItem();
        if (!sequenceSubsetCombobox.isEnabled() || sequenceCollection==null || sequenceCollection.equals(engine.getDefaultSequenceCollectionName())) sequenceCollection=null;
        parameters.setParameter(OperationTask.SEQUENCE_COLLECTION_NAME, sequenceCollection);
        if (algorithmComboBox.getItemCount()==0) setOK(false); // if no algorithms exists the OK button will effectively just cancel the dialog
        else {
            String algorithmName=(String)algorithmComboBox.getSelectedItem();
            parameters.setParameter(Operation_motifScanning.ALGORITHM, algorithmName);
            if (parametersPanel!=null) {
                parametersPanel.setParameters();
                parameters.setParameter(Operation_motifScanning.PARAMETERS, parametersPanel.getParameterSettings());
            }            
        } 
    }
    
    private void initSequenceSubsetPanel() {
        sequenceSubsetPanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        sequenceSubsetPanel.add(new JLabel("In sequence collection  "));
        String selectedName=(String)parameters.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
        if (selectedName==null) selectedName=engine.getDefaultSequenceCollectionName();
        DefaultComboBoxModel model=(DefaultComboBoxModel)getDataCandidates(new Class[]{SequenceCollection.class});
        sequenceSubsetCombobox=new JComboBox(model);
        sequenceSubsetCombobox.setSelectedItem(selectedName);
        sequenceSubsetPanel.add(sequenceSubsetCombobox);                
    } 
    
     private void initAlgorithmPanel(String sourceName, String algorithmName) {
        algorithmPanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        algorithmPanel.add(new JLabel("Algorithm  "));
        algorithmComboBox=new JComboBox(getMotifScanningAlgorithms());        
        if (algorithmName==null){
            if (algorithmComboBox.getItemCount()>0) algorithmComboBox.setSelectedIndex(0);
        }
        else algorithmComboBox.setSelectedItem(algorithmName);
        algorithmPanel.add(algorithmComboBox);   
        algorithmComboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object selected=algorithmComboBox.getSelectedItem();
                if (selected!=null) {
                    if (initialEditValues[0]!=null && ((String)selected).equals(initialEditValues[0])) {
                       targetDataTextfield.setText(initialEditValues[1]);
                    } else {
                        targetDataTextfield.setText(gui.getGenericDataitemName("BindingSites_"+((String)selected),getDataTypeTable()));
                    }
                    showParametersPanel(getAlgorithmSettingsPanel((String)selected));
                }
                else additionalParametersPanel.removeAll();
                pack();
                //setLocation(gui.getFrame().getWidth()/2-getWidth()/2, gui.getFrame().getHeight()/2-getHeight()/2);               
            }
        });
    }  
    

    private void initSourceTargetPanel() {
        sourceTargetPanel=new JPanel();
        sourceTargetPanel.setLayout(new BoxLayout(sourceTargetPanel, BoxLayout.LINE_AXIS));
        JPanel sPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        JPanel tPanel=new JPanel(new FlowLayout(FlowLayout.RIGHT));
        Class[] sourceCandidates=parameters.getOperation().getDataSourcePreferences();
        String sourceName=parameters.getSourceDataName();
        String targetName=parameters.getTargetDataName();
        String algorithmName=(String)algorithmComboBox.getSelectedItem();
        initialEditValues[0]=(String)parameters.getParameter(Operation_motifScanning.ALGORITHM);
        initialEditValues[1]=targetName;
        if (targetName==null || targetName.isEmpty()) targetName=gui.getGenericDataitemName("BindingSites_"+algorithmName,getDataTypeTable());
        DefaultComboBoxModel model=(DefaultComboBoxModel)getDataCandidates(sourceCandidates);
        sourceDataCombobox=new JComboBox(model);
        if (sourceName!=null && sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedItem(sourceName);
        else {
           if (sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedIndex(0);
           sourceName=(String)sourceDataCombobox.getSelectedItem();
        }
        sPanel.add(new JLabel("Source  "));
        sPanel.add(sourceDataCombobox);
        targetDataTextfield=new JTextField(targetName);
        targetDataTextfield.setColumns(16);
        tPanel.add(new JLabel("       Store results in  "));
        tPanel.add(targetDataTextfield);  
        sourceTargetPanel.add(sPanel);
        sourceTargetPanel.add(tPanel);
    }  
    
    /** */
    private void showParametersPanel(JPanel panel) {
        additionalParametersPanel.removeAll();
        additionalParametersPanel.add(panel);        
    }
   
    /** */
    private JPanel getAlgorithmSettingsPanel(String algorithmName) {
        MotifScanning algorithm=(MotifScanning)engine.getExternalProgram(algorithmName);
        if (algorithm==null) {
            parametersPanel=null;
            return new JPanel();
        }
        ParametersPanel panel=new ParametersPanel(algorithm.getParameters(),parameterSettings, this);
        parametersPanel=panel;
        return panel;
    }
    

}
