/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.operationdialog;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import java.util.List;

import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import priorseditor.engine.CompoundTask;
import priorseditor.engine.ExecutableTask;
import priorseditor.engine.OperationTask;
import priorseditor.engine.data.DNASequenceDataset;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.BackgroundModel;
import priorseditor.engine.data.MotifCollection;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.NumericVariable;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.SequenceCollection;
import priorseditor.engine.data.StringCollection;
import priorseditor.engine.data.StringVariable;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.operations.Operation_new;
import priorseditor.gui.prompt.*;


/**
 *
 * @author kjetikl
 */
public class OperationDialog_new extends OperationDialog {
    private JPanel nameAndTypePanel;
    private JPanel parametersPanel;
    private JTextField targetDataTextfield;
    private JTextArea parametersTextarea;
    private JComboBox dataTypeCombobox;
    private JButton configureButton;
    private Data dataobject=null;
    private OperationDialog_new thisdialog;
    
    private String currentdataname=null;
    private boolean userDefinedName=false; // true if the name of the data object has been typed by the user, false if it is a system generated default name

    
    public OperationDialog_new(JFrame parent) {
        super(parent);   
        thisdialog=this;
    }
    
    public OperationDialog_new() {
        super();   
        thisdialog=this;
    }
    
    @Override
    public void initComponents() {
        super.initComponents();
        nameAndTypePanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        nameAndTypePanel.setBorder(commonBorder);
        nameAndTypePanel.add(new JLabel("Name "));
        targetDataTextfield=new JTextField();
        nameAndTypePanel.add(targetDataTextfield);
        targetDataTextfield.setColumns(16);
        String currentdatatype=(String)parameters.getParameter(Operation_new.DATA_TYPE);
        String currentParametersString=(String)parameters.getParameter(Operation_new.PARAMETERS);
        String[] datatypes=Operation_new.getAvailableTypes();
        dataTypeCombobox = new JComboBox(datatypes);
        dataTypeCombobox.setEditable(false);
        nameAndTypePanel.add(new JLabel("      Type  "));
        nameAndTypePanel.add(dataTypeCombobox);                   
        parametersPanel=new JPanel(new BorderLayout());
        parametersPanel.setBorder(commonBorder);
        parametersPanel.add(new JLabel(" Parameters "),BorderLayout.NORTH);
        parametersTextarea=new JTextArea(8,30);
        parametersPanel.add(new JScrollPane(parametersTextarea));
        if (currentParametersString!=null) parametersTextarea.setText(currentParametersString);
        add(nameAndTypePanel);
        add(parametersPanel);
        JPanel buttonsPanel=new JPanel();
        buttonsPanel.setLayout(new BorderLayout());
        configureButton=new JButton("Configure");
        JPanel configbuttonpanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        configbuttonpanel.add(configureButton);
        buttonsPanel.add(configbuttonpanel,BorderLayout.WEST);
        buttonsPanel.add(getOKCancelButtonsPanel(),BorderLayout.EAST);
        add(buttonsPanel);
        pack();        
        configureButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                thisdialog.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                configureUsingPrompt();
                thisdialog.setCursor(Cursor.getDefaultCursor());
            }
        });
        dataTypeCombobox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String selected=(String)dataTypeCombobox.getSelectedItem();
                parametersTextarea.setText("");
                if (selected.equals(StringVariable.getType()) || selected.equals(NumericVariable.getType())) {
                   configureButton.setVisible(false);
                   parametersTextarea.setEditable(true);
                } else {
                   configureButton.setVisible(true);
                   parametersTextarea.setEditable(false);                    
                }
                updateDataName(selected);
            }
        });
        currentdataname=(String)parameters.getParameter(OperationTask.TARGET_NAME);
        if (currentdataname!=null && !currentdataname.isEmpty()) userDefinedName=true;
        targetDataTextfield.setText(currentdataname); 
        if (currentdatatype!=null) setSelectedDatatype(dataTypeCombobox,currentdatatype);
        else dataTypeCombobox.setSelectedIndex(0);
        targetDataTextfield.getDocument().addDocumentListener(new DocumentListener() {
            @Override public void insertUpdate(DocumentEvent e) {propertyChange();}
            @Override public void removeUpdate(DocumentEvent e) {propertyChange();}
            @Override public void changedUpdate(DocumentEvent e) {propertyChange();}                                        
            private void propertyChange() {
                String newName=targetDataTextfield.getText().trim();
                if (!currentdataname.equals(newName)) userDefinedName=true;
            }
        });     
    }

    private void updateDataName(String typename) {
        if (userDefinedName) return; // do not change user defined names
        Class type=engine.getDataClassForTypeName(typename);
        currentdataname=gui.getGenericDataitemName(type, getDataTypeTable());    
        targetDataTextfield.setText(currentdataname);
        userDefinedName=false; 
    }    
    
    /** This method circumvents issues with different case in data type names*/
    private void setSelectedDatatype(JComboBox box, String datatypename) {
        ComboBoxModel model=box.getModel();
        for (int i=0;i<model.getSize();i++) {
           if (((String)model.getElementAt(i)).equalsIgnoreCase(datatypename)) {
               box.setSelectedIndex(i); break;
           } 
        }       
    }
    
    /* This method is overridden here in order to return a compound task if necessary */
    @Override
    public ExecutableTask getOperationTask() {
        if (dataobject instanceof MotifCollection && ((MotifCollection)dataobject).isImportedCollection()) {
            boolean importAsCompound=false;
            if (importAsCompound) return importCollectionAsCompoundTask((MotifCollection)dataobject);
            else {
               return parameters;   
            }
        } else { // not imported MotifCollection
            return parameters; 
        }
    }
    
    /** When using this method the "import Motif Collection from file" */
    private CompoundTask importCollectionAsCompoundTask(MotifCollection collection) {
            CompoundTask compoundTask=new CompoundTask("new "+MotifCollection.getType());
            List<Motif> motifslist=collection.getPayload();
            String parameterString="";
            for (int i=0;i<motifslist.size();i++) {
                Motif motif=motifslist.get(i);
                String motifname=motif.getName();
                if (i==motifslist.size()-1) parameterString+=motifname;
                else parameterString+=motifname+",";
                OperationTask newmotiftask=new OperationTask("new");
                newmotiftask.setParameter(OperationTask.OPERATION_NAME, "new");
                newmotiftask.setParameter(Operation_new.DATA_TYPE, Motif.getType());
                newmotiftask.setParameter(OperationTask.TARGET_NAME,motifname);
                newmotiftask.setParameter(OperationTask.ENGINE,gui.getEngine());                    
                newmotiftask.setParameter(Operation_new.PARAMETERS, motif.getValueAsParameterString());
                compoundTask.addTask(newmotiftask);
            }
            parameters.setParameter(Operation_new.PARAMETERS, parameterString); // new MotifCollection task
            compoundTask.addTask(parameters);  
            return compoundTask;        
    }
    
    @Override
    protected void setParameters() {
        //super.setParameters(); // sets the common feature-transform parameters like 'source','target','where' and 'collection'
        String type=(String)dataTypeCombobox.getSelectedItem();
        parameters.setParameter(Operation_new.DATA_TYPE, type);
        String dataname=targetDataTextfield.getText().trim();
        if (dataname.isEmpty()) dataname="X";
        parameters.setParameter(OperationTask.TARGET_NAME, dataname);
        preprocessParametersTextarea(type);
        String parameterstext=parametersTextarea.getText();
        if (!parameterstext.isEmpty()) parameters.setParameter(Operation_new.PARAMETERS, parameterstext);
        else parameters.setParameter(Operation_new.PARAMETERS, null);
        if (dataobject!=null) dataobject.setAdditionalOperationNewTaskParameters(parameters);
    }
    
    private void configureUsingPrompt() {
        String dataname=targetDataTextfield.getText().trim();
        String type=(String)dataTypeCombobox.getSelectedItem();
        preprocessParametersTextarea(type);
        String parameterstring=parametersTextarea.getText();
        Prompt prompt=null;
        if (type.equals(NumericDataset.getType())) {
              prompt=new Prompt_NumericDataset(gui, null, dataname,parameterstring);
        }
        else if (type.equals(RegionDataset.getType())) {
              prompt=new Prompt_RegionDataset(gui, null, dataname,parameterstring);         
        }
        else if (type.equals(DNASequenceDataset.getType())) {
              prompt=new Prompt_DNASequenceDataset(gui, null, dataname,parameterstring,this);         
        }
        else if (type.equals(BackgroundModel.getType())) {
            if (dataobject!=null && dataobject instanceof BackgroundModel) 
              prompt=new Prompt_BackgroundModel(gui, null, (BackgroundModel)dataobject);
            else prompt=new Prompt_BackgroundModel(gui, null, null);           
        }
        else if (type.equals(SequenceCollection.getType())) {
            if (dataobject!=null && dataobject instanceof SequenceCollection) 
              prompt=new Prompt_SequenceCollection(gui, null, (SequenceCollection)dataobject);
            else prompt=new Prompt_SequenceCollection(gui, null, null);        
        }
        else if (type.equals(StringCollection.getType())) {
            if (dataobject!=null && dataobject instanceof StringCollection) 
              prompt=new Prompt_StringCollection(gui, null, (StringCollection)dataobject);
            else prompt=new Prompt_StringCollection(gui, null, null);
        }
        else if (type.equals(MotifCollection.getType())) {
            if (dataobject!=null && dataobject instanceof MotifCollection) 
              prompt=new Prompt_MotifCollection(gui, null, (MotifCollection)dataobject);
            else prompt=new Prompt_MotifCollection(gui, null, null);
        }
        else if (type.equals(Motif.getType())) {
            if (dataobject!=null && dataobject instanceof Motif) 
              prompt=new Prompt_Motif(gui, null, (Motif)dataobject,gui.getVisualizationSettings());
            else prompt=new Prompt_Motif(gui, null, null,gui.getVisualizationSettings());
        }
        if (prompt!=null) {
              prompt.setDataItemName(dataname);
              prompt.setLocation(this.getX()+this.getWidth()/2-prompt.getWidth()/2,this.getY()+this.getHeight()/2-prompt.getHeight()/2);
              prompt.disableNameEdit();
              prompt.setVisible(true);
              if (prompt.isOKPressed()) {
                 parameterstring=prompt.getValueAsParameterString();
                 parametersTextarea.setText(parameterstring);
                 dataobject=prompt.getData();
              }
              prompt.dispose();
        }
        
    }
    
    
    private void preprocessParametersTextarea(String type) {
        String val = parametersTextarea.getText().trim();
        val=val.replaceAll("\n\\s*\n", "\n"); // remove empty lines
             if (type.equals(StringCollection.getType()) || type.equals(SequenceCollection.getType()) || type.equals(MotifCollection.getType())) val=val.replaceAll(",?\n", ","); // replace newlines with commas
        else if (type.equals(Motif.getType())) val=val.replaceAll(";?\n", ";"); // replace newlines with commas
        else val=val.replaceAll("\n", " "); // replace newlines with spaces
        parametersTextarea.setText(val);
    }
    

}
