/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.operationdialog;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import priorseditor.engine.data.*;
import priorseditor.engine.OperationTask;
import priorseditor.engine.Parameter;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.operations.Operation_output;
import priorseditor.engine.dataformat.DataFormat;
import priorseditor.gui.ParametersPanel;


/**
 *
 * @author kjetikl
 */
public class OperationDialog_output extends OperationDialog {
    private JComboBox sourceDataCombobox=null; 
    private JTextField targetDataTextfield=null; 
    private JPanel sourceTargetPanel=null;
    private JPanel sequenceSubsetPanel;
    private JPanel formatPanel;
    private JPanel additionalParametersPanel;
    private JComboBox sequenceSubsetCombobox;
    private JComboBox formatComboBox; 
    private JLabel formatErrorLabel;
    private String formatErrorString=" *** No supported output formats for this data type ***";
    private ParametersPanel parametersPanel=null;
    private ParameterSettings parameterSettings=null;
    
    public OperationDialog_output(JFrame parent) {
        super(parent); 
        initComponents();        
    }
    
    public OperationDialog_output() {
        super();
    }    
    
    public DefaultComboBoxModel getFormatsForSource(String sourceName) {
         ArrayList<DataFormat> outputformatsList;
         Class sourceClass=getClassForDataItem(sourceName);
         if (sourceClass==null) outputformatsList=engine.getAllDataFormats();
         else outputformatsList=engine.getDataOutputFormats(sourceClass);
         String[] list=new String[outputformatsList.size()];
         int i=0;
         for (DataFormat formatter:outputformatsList) {
             list[i]=formatter.getName();
             i++;
         }
         DefaultComboBoxModel newmodel=new DefaultComboBoxModel(list);
         return newmodel;
    } 
    
    
    
    @Override
    public void initComponents() {
        super.initComponents();  
        setResizable(false);
        setIconImage(gui.getFrame().getIconImage());        
        String formatName=(String)parameters.getParameter(Operation_output.OUTPUT_FORMAT);     
        String sourceName=parameters.getSourceDataName();
        parameterSettings=(ParameterSettings)parameters.getParameter(Operation_output.OUTPUT_FORMAT_PARAMETERS);     
        if (parameterSettings==null) {
            parameterSettings=new ParameterSettings();
            parameters.setParameter(Operation_output.OUTPUT_FORMAT_PARAMETERS,parameterSettings);    
        }
        initSourceTargetPanel();
        initFormatPanel(sourceName,formatName);
        initSequenceSubsetPanel();
        formatPanel.setBorder(commonBorder);    
        sourceTargetPanel.setBorder(commonBorder);    
        sequenceSubsetPanel.setBorder(commonBorder);
        additionalParametersPanel=new JPanel();
        additionalParametersPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
        additionalParametersPanel.setBorder(commonBorder);
        add(sourceTargetPanel);   
        add(formatPanel);
        JScrollPane scrollPane=new JScrollPane(additionalParametersPanel);  
        scrollPane.setMaximumSize(new Dimension(500,320));        
        add(scrollPane);
        add(sequenceSubsetPanel);
        add(getOKCancelButtonsPanel());
        Object formatSelected=formatComboBox.getSelectedItem();
        formatComboBox.setSelectedItem(formatSelected); // programatic 'click' to show the initial panel (if any applicable)
        pack();
        if (sourceName!=null && sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedItem(sourceName);
        else {
           if (sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedIndex(0);
           sourceName=(String)sourceDataCombobox.getSelectedItem();
        }
    }

    @Override
    protected void setParameters() {
        String sourceName=(String)sourceDataCombobox.getSelectedItem();       
        parameters.setParameter(OperationTask.SOURCE_NAME, sourceName);
        String targetName=targetDataTextfield.getText();
        targetName.trim();
        if (targetName==null || targetName.isEmpty()) targetName=sourceName;
        parameters.setParameter(OperationTask.TARGET_NAME, targetName);        
        String sequenceCollection=(String)sequenceSubsetCombobox.getSelectedItem();
        if (!sequenceSubsetCombobox.isEnabled() || sequenceCollection==null || sequenceCollection.equals(engine.getDefaultSequenceCollectionName())) sequenceCollection=null;
        parameters.setParameter(OperationTask.SEQUENCE_COLLECTION_NAME, sequenceCollection);
        if (formatComboBox.getItemCount()==0) setOK(false); // if no applicable output formats exists the OK button will effectively just cancel the dialog
        else {
            String outputformatName=(String)formatComboBox.getSelectedItem();
            parameters.setParameter(Operation_output.OUTPUT_FORMAT, outputformatName);
            if (parametersPanel!=null) {
                parametersPanel.setParameters();
                parameters.setParameter(Operation_output.OUTPUT_FORMAT_PARAMETERS, parametersPanel.getParameterSettings());
            }            
        } // end valid output format
    }
    
    private void initSequenceSubsetPanel() {
        sequenceSubsetPanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        sequenceSubsetPanel.add(new JLabel("In sequence collection  "));
        String selectedName=(String)parameters.getParameter(OperationTask.SEQUENCE_COLLECTION_NAME);
        if (selectedName==null) selectedName=engine.getDefaultSequenceCollectionName();
        DefaultComboBoxModel model=(DefaultComboBoxModel)getDataCandidates(new Class[]{SequenceCollection.class});
        sequenceSubsetCombobox=new JComboBox(model);
        sequenceSubsetCombobox.setSelectedItem(selectedName);
        sequenceSubsetPanel.add(sequenceSubsetCombobox);                
    } 
    
     private void initFormatPanel(String sourceName, String formatName) {
        formatPanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        formatPanel.add(new JLabel("Output format  "));
        formatComboBox=new JComboBox(getFormatsForSource(sourceName));        
        if (formatName==null) {
            Class sourceClass=getClassForDataItem(sourceName);
            if (sourceClass!=null) {
                DataFormat of=engine.getDefaultDataFormat(sourceClass);
                if (of!=null) formatName=of.getName();
            }
        }
        if (formatName==null){
            if (formatComboBox.getItemCount()>0) formatComboBox.setSelectedIndex(0);
        }
        else formatComboBox.setSelectedItem(formatName);
        formatPanel.add(formatComboBox);   
        if (formatComboBox.getItemCount()>0) formatErrorLabel=new JLabel("");
        else formatErrorLabel=new JLabel(formatErrorString);
        formatErrorLabel.setForeground(java.awt.Color.RED);
        formatPanel.add(formatErrorLabel);
        formatComboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object selected=formatComboBox.getSelectedItem();
                String selectedSourceName=(String)sourceDataCombobox.getSelectedItem();   
                Class sourceClass=getClassForDataItem(selectedSourceName);
                if (selected!=null) showParametersPanel(getFormatSettingsPanel((String)selected,sourceClass)); 
                else additionalParametersPanel.removeAll();
                pack();
                //setLocation(gui.getFrame().getWidth()/2-getWidth()/2, gui.getFrame().getHeight()/2-getHeight()/2);
            }
        });
    }  
    

    private void initSourceTargetPanel() {
        sourceTargetPanel=new JPanel();
        sourceTargetPanel.setLayout(new BoxLayout(sourceTargetPanel, BoxLayout.LINE_AXIS));
        JPanel sPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        JPanel tPanel=new JPanel(new FlowLayout(FlowLayout.RIGHT));
        Class[] sourceCandidates=parameters.getOperation().getDataSourcePreferences();
        String sourceName=parameters.getSourceDataName();
        String targetName=parameters.getTargetDataName();
        //if (targetName==null || targetName.isEmpty()) targetName=engine.getDefaultOutputObjectName();
        if (targetName==null || targetName.isEmpty()) targetName=gui.getGenericDataitemName(OutputData.class,getDataTypeTable());
        DefaultComboBoxModel model=(DefaultComboBoxModel)getDataCandidates(sourceCandidates);
        sourceDataCombobox=new JComboBox(model);
        if (sourceName!=null && sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedItem(sourceName);
        else {
           if (sourceDataCombobox.getItemCount()>0) sourceDataCombobox.setSelectedIndex(0);
           sourceName=(String)sourceDataCombobox.getSelectedItem();
        }
        sPanel.add(new JLabel("Source  "));
        sPanel.add(sourceDataCombobox);
        targetDataTextfield=new JTextField(targetName);
        targetDataTextfield.setColumns(16);
        tPanel.add(new JLabel("       Output to  "));
        tPanel.add(targetDataTextfield);  
        sourceDataCombobox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                additionalParametersPanel.removeAll(); 
                String selectedSourceName=(String)sourceDataCombobox.getSelectedItem();                
                formatComboBox.setModel(getFormatsForSource(selectedSourceName));                
                if (formatComboBox.getItemCount()>0) {
                    String defaultFormat=null;
                    formatErrorLabel.setText("");
                    Class sourceClass=getClassForDataItem(selectedSourceName);
                    if (sourceClass!=null) {
                       DataFormat of=engine.getDefaultDataFormat(sourceClass);
                       if (of!=null) defaultFormat=of.getName();
                    }
                    if (defaultFormat!=null) formatComboBox.setSelectedItem(defaultFormat);
                    else formatComboBox.setSelectedIndex(0);
                    String selectedFormatName=(String)formatComboBox.getSelectedItem();
                    showParametersPanel(getFormatSettingsPanel((String)selectedFormatName,sourceClass));    
                }
                else {
                    formatErrorLabel.setText(formatErrorString);                            
                }
                String sel=(String)sourceDataCombobox.getSelectedItem();
                Data dataitem=engine.getDataItem(sel);                
                if (dataitem!=null && (dataitem instanceof SequenceCollection || dataitem instanceof FeatureDataset)) sequenceSubsetCombobox.setEnabled(true);
                else sequenceSubsetCombobox.setEnabled(false);
                pack();
            }
        });  
        sourceTargetPanel.add(sPanel);
        sourceTargetPanel.add(tPanel);
    }  
    
    /** */
    private void showParametersPanel(JPanel panel) {
        additionalParametersPanel.removeAll();
        additionalParametersPanel.add(panel);        
    }
   
    /** */
    private JPanel getFormatSettingsPanel(String outputformatName, Class sourceDataclass) {
        DataFormat outputFormat=engine.getDataFormat(outputformatName);
        if (outputFormat==null) {
            parametersPanel=null;
            return new JPanel();
        }
        Parameter[] formatparameters=outputFormat.getParameters();        
        ParametersPanel panel=new ParametersPanel(formatparameters,parameterSettings, this);
        parametersPanel=panel;
        return panel;
    }
    

}
