/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.operationdialog;

import java.awt.CardLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import priorseditor.engine.data.RegionDataset;
import priorseditor.engine.data.NumericDataset;
import priorseditor.engine.operations.Condition;
import priorseditor.engine.operations.Condition_region;
import priorseditor.engine.operations.Condition_position;
import priorseditor.engine.operations.Operation_statistic;

/**
 *
 * @author kjetikl
 */
public class OperationDialog_statistic extends FeatureTransformOperationDialog {
    private JComboBox statisticFunctionCombobox;
    private JComboBox whereCombobox;
    private JPanel whereclausePanel;
    private ConditionPanel_region regionConditionPanel;
    private ConditionPanel_position positionConditionPanel;
    private JPanel usedConditionPanel;    
    private JPanel valuePanel;
    
    public OperationDialog_statistic(JFrame parent) {
        super(parent);    
    }
    
    public OperationDialog_statistic() {
        super();    
    }
    
    @Override
    public void initComponents() {
        super.initComponents();
        JPanel sourceTargetPanel=getSourceTargetPanel();
        sourceTargetPanel.setBorder(commonBorder);
        initWhereClausePanels();
        valuePanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        valuePanel.setBorder(commonBorder);
        valuePanel.add(new JLabel("Statistic function   "));
        statisticFunctionCombobox=new JComboBox(Operation_statistic.getStatisticFunctions());
        statisticFunctionCombobox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                updateTargetDataName();
            }
        });
        ActionListener[] list=sourceDataCombobox.getActionListeners();
        for (ActionListener listener:list) { // remove old listeners. I will make a new one instead
            sourceDataCombobox.removeActionListener(listener);
        }               
        sourceDataCombobox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                String sourcename=(String)sourceDataCombobox.getSelectedItem();
                updatePanelsForSource(sourcename);
                updateTargetDataName();
            }
        });


        valuePanel.add(statisticFunctionCombobox);      
        whereclausePanel.setBorder(commonBorder);
        sourceDataCombobox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String dataitem=(String)sourceDataCombobox.getSelectedItem(); 
                if (getClassForDataItem(dataitem)==NumericDataset.class) {
                    usedConditionPanel.removeAll();
                    usedConditionPanel.add(positionConditionPanel);
                }
                else if (getClassForDataItem(dataitem)==RegionDataset.class) {
                    usedConditionPanel.removeAll();
                    usedConditionPanel.add(regionConditionPanel);
                } 
                updateTargetDataName();
                pack();
            }
        });
        add(sourceTargetPanel);
        add(valuePanel);
        add(whereclausePanel);
        add(getOKCancelButtonsPanel());
        updatePanelsForSource((String)sourceDataCombobox.getSelectedItem());
        String statisticFunction=(String)parameters.getParameter(Operation_statistic.STATISTIC_FUNCTION);
        if (statisticFunction==null) statisticFunctionCombobox.setSelectedIndex(0);
        else statisticFunctionCombobox.setSelectedItem(statisticFunction);
        String targetName=(String)parameters.getTargetDataName();
        if (targetName!=null && !targetName.isEmpty()) targetDataTextfield.setText(targetName);
    }
    
    /** Sets proper condition and statistical functions depending on the class type of the source data
     */
    private void updatePanelsForSource(String sourceName) {
        if (sourceName==null || sourceName.isEmpty()) return;
        Class sourcedataclass=getClassForDataItem(sourceName);
        if (sourcedataclass==NumericDataset.class) {
            usedConditionPanel.removeAll();
            usedConditionPanel.add(positionConditionPanel);
        } else if (sourcedataclass==RegionDataset.class) {
            usedConditionPanel.removeAll();
            usedConditionPanel.add(regionConditionPanel);
        } else { // this should not happen!
            usedConditionPanel.removeAll();
        }        
        pack();
    }

    private void updateTargetDataName() {
        String statistic=(String)statisticFunctionCombobox.getSelectedItem();
        statistic=statistic.replace("-", "_");
        statistic=statistic.replace(" ", "_");
        String sourceName=(String)sourceDataCombobox.getSelectedItem();
        if (sourceName==null) sourceName="statistic";
        targetDataTextfield.setText(sourceName+"_"+statistic);
        targetDataTextfield.setCaretPosition(0);
    }
    
    
    @Override
    protected void setParameters() {
        super.setParameters(); // sets the common feature-transform parameters like 'source','target','where' and 'collection'
        //parameters.setParameter(Operation_weight.WEIGHT_STRING, (String)weightbyCombobox.getSelectedItem());
        if (whereCombobox.getSelectedItem().equals("Where")) {
            Condition condition;
            if (isSourceNumeric()) condition=positionConditionPanel.getCondition();
            else condition=regionConditionPanel.getCondition();
            parameters.setParameter("where", condition);
        } else parameters.setParameter("where", null);
        parameters.setParameter(Operation_statistic.STATISTIC_FUNCTION, (String)statisticFunctionCombobox.getSelectedItem());
    }
    

    private void initWhereClausePanels() {
       Condition condition=(Condition)parameters.getParameter("where");       
       final JPanel cardsPanel=new JPanel(new CardLayout());
       JPanel card[]=new JPanel[2];     
       card[1]=new JPanel(new FlowLayout(FlowLayout.LEADING));
       whereclausePanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
       whereCombobox=new JComboBox(new String[]{"","Where"});
       whereCombobox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (whereCombobox.getSelectedItem().equals("Where")) ((CardLayout)cardsPanel.getLayout()).show(cardsPanel, "useCondition");
                else ((CardLayout)cardsPanel.getLayout()).show(cardsPanel, "noCondition");                       
            }
        });
        if (condition == null) {
            positionConditionPanel=new ConditionPanel_position(null, this);
            regionConditionPanel=new ConditionPanel_region(null, this);   
        } else if (condition instanceof Condition_position) {
            positionConditionPanel=new ConditionPanel_position((Condition_position)condition, this);
            regionConditionPanel=new ConditionPanel_region(null, this);   
        } else {
            positionConditionPanel=new ConditionPanel_position(null, this);
            regionConditionPanel=new ConditionPanel_region((Condition_region)condition, this);   
        }
        usedConditionPanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        if (isSourceNumeric()) usedConditionPanel.add(positionConditionPanel);
        else usedConditionPanel.add(regionConditionPanel);
        cardsPanel.add(new JPanel(),"noCondition");
        cardsPanel.add(usedConditionPanel,"useCondition");
        whereclausePanel.add(new JLabel("Condition  "));
        whereclausePanel.add(whereCombobox);
        whereclausePanel.add(cardsPanel);

       if (condition!=null) whereCombobox.setSelectedItem("Where");
       else whereCombobox.setSelectedItem("");

    }    
 
     
    
    private boolean isSourceNumeric() {
          String dataitem=(String)sourceDataCombobox.getSelectedItem(); 
          if (getClassForDataItem(dataitem)==NumericDataset.class) return true;   
          else return false;
    }
    
}
