/*
 * Prompt.java
 *
 * Created on 25. mars 2009, 16:45
 */

package priorseditor.gui.prompt;

import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import priorseditor.engine.OperationTask;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.PriorsEditorEngine;
import priorseditor.engine.data.Data;
import priorseditor.engine.operations.Operation_new;
import priorseditor.gui.PriorsEditorGUI;

/**
 *
 * @author  kjetikl
 */
public abstract class Prompt extends javax.swing.JDialog {
    protected PriorsEditorEngine engine;
    protected PriorsEditorGUI gui;
    private boolean okPressed=false;
    private String importFromFile_filename=null;
    private String importFromFile_dataformatname=null;
    private ParameterSettings importFromFile_dataformatsettings=null;
    
    /** Creates new form Prompt */
    public Prompt(PriorsEditorGUI gui, String prompt) {
        super(gui.getFrame(), true);
        this.engine=gui.getEngine();
        this.gui=gui;
        initComponents();
        if (prompt!=null) promptLabel.setText("<html><font color='red'><b>"+prompt+"</b></font><br>"+"&nbsp;</html>");
        else promptLabel.setText("");
        getRootPane().setDefaultButton(okButton);
        this.setMinimumSize(new Dimension(300,200));
    }

    protected void setMainPanel(JComponent mainpanel) {mainPanel.add(mainpanel,BorderLayout.CENTER);}
   
    public boolean isOKPressed() {
        return okPressed;
    }

    /**
     * If the prompt is used to import a data object from file, this method can be used to
     * store information about the file, fileformat and settings used (in the onOKPressed() method)
     * These settings can later be probed by getImportFromFileSettings or
     * @param filename
     * @param dataformatName
     * @param formatSettings
     */
    protected void setImportFromFileSettings(String filename, String dataformatName, ParameterSettings formatSettings) {
        importFromFile_filename=filename;
        importFromFile_dataformatname=dataformatName;
        importFromFile_dataformatsettings=formatSettings;
    }

    /**
     * This will return TRUE if the prompt was used to import a data object from file,
     * and setImportFromFileSettings() was later used to specify information about
     * filename, dataformat and settings
     * @return
     */
    public boolean isDataImportedFromFile() {
        return (importFromFile_filename!=null);
    }

    /**
     * If the prompt was used to import a data object from file (as specified by a call to setImportFromFileSettings())
     * this method can be used to retrieve the filename, dataformat and formatsettings used.
     *
     * @return an Object array containing (in order) filename (string), dataformatname (String) and dataformatsettings (ParameterSettings)
     */
    public Object[] getImportFromFileSettings() {
        return new Object[] {importFromFile_filename,importFromFile_dataformatname,importFromFile_dataformatsettings};
    }

    /**
     *
     * @param task
     * @return TRUE if the data object was imported from file and the settings have been set in the Task
     *         or FALSE if data object was not imported from file and no changes was made in the Task
     */
    public boolean setImportFromFileSettingsInTask(OperationTask task) {
        if (importFromFile_filename==null) return false;
        task.setParameter(Operation_new.PARAMETERS, Operation_new.FILE_PREFIX);  // required to signal import
        task.setParameter(Operation_new.FILENAME, importFromFile_filename);  //
        task.setParameter(Operation_new.DATA_FORMAT, importFromFile_dataformatname);
        task.setParameter(Operation_new.DATA_FORMAT_SETTINGS, importFromFile_dataformatsettings);
        return true;
    }

    /** Programatically click the OK button */
    public void clickOK() {
        okButton.doClick();
    }
    
    public void setDataItemName(String name) {
        dataItemNameTextField.setText(name);
    }
    
    public String getDataItemName() {
        return dataItemNameTextField.getText().trim();
    }
    
    public void setDataItemNameLabelText(String newText) {
        dataItemNameLabel.setText(newText);
    }
    
    public void disableNameEdit() {
        dataItemNameTextField.setEditable(false);   
    }
    
    /** 
     * This method is called automatically when the user presses OK
     * Subclasses should implement this method to perform necessary data creations
     * The returned boolean value can be used to control whether the dialog should
     * close when the OK button was pressed. If something wrong has happened
     * (for instance the user has entered an illegal value) then the function can
     * return false to keep the dialog open
     */
    public abstract boolean onOKPressed();
    
    /** Returns the Data item that was prompted */
    public abstract Data getData();
    
    /** Returns a value that could be used to initialize a new dataitem based on the users selections */
    public String getValueAsParameterString() {
        if (getData()!=null) return getData().getValueAsParameterString();
        else return "";
    }



    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        dataItemNameLabel = new javax.swing.JLabel();
        dataItemNameTextField = new javax.swing.JTextField();
        jPanel4 = new javax.swing.JPanel();
        promptLabel = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        cancelButton = new javax.swing.JButton();
        okButton = new javax.swing.JButton();
        mainPanel = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setName("Form"); // NOI18N

        jPanel1.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        jPanel1.setAlignmentX(0.0F);
        jPanel1.setMinimumSize(new java.awt.Dimension(110, 10));
        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setLayout(new java.awt.BorderLayout());

        jPanel3.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 5, 5, 5));
        jPanel3.setMaximumSize(new java.awt.Dimension(2147483647, 40));
        jPanel3.setMinimumSize(new java.awt.Dimension(50, 34));
        jPanel3.setName("jPanel3"); // NOI18N
        jPanel3.setPreferredSize(new java.awt.Dimension(100, 34));
        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.LINE_AXIS));

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(Prompt.class);
        dataItemNameLabel.setText(resourceMap.getString("dataItemNameLabel.text")); // NOI18N
        dataItemNameLabel.setName("dataItemNameLabel"); // NOI18N
        jPanel3.add(dataItemNameLabel);

        dataItemNameTextField.setColumns(30);
        dataItemNameTextField.setText(resourceMap.getString("dataItemNameTextField.text")); // NOI18N
        dataItemNameTextField.setMaximumSize(new java.awt.Dimension(500, 40));
        dataItemNameTextField.setMinimumSize(new java.awt.Dimension(150, 20));
        dataItemNameTextField.setName("dataItemNameTextField"); // NOI18N
        dataItemNameTextField.setPreferredSize(new java.awt.Dimension(250, 20));
        jPanel3.add(dataItemNameTextField);

        jPanel1.add(jPanel3, java.awt.BorderLayout.PAGE_END);

        jPanel4.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        jPanel4.setName("jPanel4"); // NOI18N
        jPanel4.setLayout(new javax.swing.BoxLayout(jPanel4, javax.swing.BoxLayout.LINE_AXIS));

        promptLabel.setText(resourceMap.getString("promptLabel.text")); // NOI18N
        promptLabel.setName("promptLabel"); // NOI18N
        jPanel4.add(promptLabel);

        jPanel1.add(jPanel4, java.awt.BorderLayout.PAGE_START);

        getContentPane().add(jPanel1, java.awt.BorderLayout.PAGE_START);

        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setPreferredSize(new java.awt.Dimension(100, 40));

        cancelButton.setText(resourceMap.getString("cancelButton.text")); // NOI18N
        cancelButton.setMaximumSize(new java.awt.Dimension(75, 27));
        cancelButton.setMinimumSize(new java.awt.Dimension(75, 27));
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.setPreferredSize(new java.awt.Dimension(75, 27));
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonPressed(evt);
            }
        });

        okButton.setText(resourceMap.getString("okButton.text")); // NOI18N
        okButton.setMaximumSize(new java.awt.Dimension(75, 27));
        okButton.setMinimumSize(new java.awt.Dimension(75, 27));
        okButton.setName("okButton"); // NOI18N
        okButton.setPreferredSize(new java.awt.Dimension(75, 27));
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonPressed(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap(230, Short.MAX_VALUE)
                .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        getContentPane().add(jPanel2, java.awt.BorderLayout.PAGE_END);

        mainPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setLayout(new java.awt.BorderLayout());
        getContentPane().add(mainPanel, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void cancelButtonPressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonPressed
     okPressed=false;
     setVisible(false);
}//GEN-LAST:event_cancelButtonPressed

private void okButtonPressed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonPressed
    String targetName=getDataItemName();
    String nameError=engine.checkNameValidity(targetName,false);
    if (nameError!=null) {
        JOptionPane.showMessageDialog(this, nameError+":\n"+targetName, "Illegal data name", JOptionPane.ERROR_MESSAGE);
        return;
    }    
    okPressed=true;    
    if (onOKPressed()) {
        setVisible(false);
    }
}//GEN-LAST:event_okButtonPressed

protected Dimension getDataNameLabelDimensions() {
    return dataItemNameLabel.getSize();
}
protected Dimension getDataNameFieldDimensions() {
    return dataItemNameTextField.getSize();
}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel dataItemNameLabel;
    private javax.swing.JTextField dataItemNameTextField;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel promptLabel;
    // End of variables declaration//GEN-END:variables

}
