/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.prompt;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.BevelBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.data.*;

import priorseditor.engine.SystemError;
import priorseditor.engine.dataformat.DataFormat;
import priorseditor.engine.dataformat.DataFormat_INCLUSive_Background;
import priorseditor.engine.protocol.ParseError;
import priorseditor.gui.ExcelAdapter;
import priorseditor.gui.LoadFromFilePanel;
import priorseditor.gui.PriorsEditorGUI;


/**
 *
 * @author kjetikl
 */
public class Prompt_BackgroundModel extends Prompt {

    private BackgroundModel data;

    private JPanel manualEntryPanel;
    private JPanel predefinedModelsPanel;
    private LoadFromFilePanel importModelPanel;
    private JTabbedPane tabbedPanel;
    private JComboBox orderCombobox;
    private JTable matrixTable;
    private JTable oligoTable;
    private JScrollPane scrollpane;    
    private JScrollPane oligoscrollpane;    
    private DefaultTableModel matrixTableModel;
    private DefaultTableModel oligoTableModel;
    private DefaultTableModel snfTableModel;
    private DefaultTableModel predefinedModelTableModel;
    private JTable predefinedModelTable;
    private JTable rowheaderView;
    private JTable oligoRowheaderView;
    private String[] bases = new String[]{"A","C","G","T"};
    private JLabel predefinedPanelErrorLabel;
    private JLabel manualPanelErrorLabel;
    private boolean showExisting=false;
    private int rowheadersize=60;
    
    public Prompt_BackgroundModel(PriorsEditorGUI gui, String prompt, BackgroundModel dataitem) {
        super(gui,prompt);
        showExisting=(dataitem!=null);
        if (dataitem!=null) data=dataitem;
        else data=new BackgroundModel(gui.getGenericDataitemName(BackgroundModel.class, null));
        setDataItemName(data.getName());
        setTitle("Background Model");
        setupPredefinedModelsPanel();
        setupManualEntryPanel();
        setupImportModelPanel();
        tabbedPanel=new JTabbedPane();
        tabbedPanel.addTab("Predefined Models", predefinedModelsPanel);
        tabbedPanel.addTab("Manual Entry", manualEntryPanel);
        tabbedPanel.addTab("Import Model", importModelPanel);
        JPanel internal=new JPanel(new BorderLayout());
        Dimension size=new Dimension(430,500);
        internal.setMinimumSize(size);
        internal.setPreferredSize(size);
        // internal.setMaximumSize(size);
        manualEntryPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        predefinedModelsPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        importModelPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        internal.add(tabbedPanel);
        this.setResizable(true);
        setMainPanel(internal);        
        pack();
        if (showExisting) {
            String predefinedModelName=dataitem.getPredefinedModel();
            if (predefinedModelName==null) tabbedPanel.setSelectedComponent(manualEntryPanel);
            else {
                tabbedPanel.setSelectedComponent(predefinedModelsPanel);
                ListSelectionModel selectionmodel=predefinedModelTable.getSelectionModel();
                for (int i=0;i<predefinedModelTableModel.getRowCount();i++) {
                    if (((String)predefinedModelTableModel.getValueAt(i,0)).equals(predefinedModelName)) {
                        selectionmodel.setSelectionInterval(i, i);
                        break;
                    }
                }                
            }
        }
    }
    
    private void setupPredefinedModelsPanel() {
       predefinedModelsPanel=new JPanel();
        String[] columns = new String[]{"Name","Order","Organism","Filename"};
        Object[][] predefinedmodelstable = BackgroundModel.predefinedModels;
        
        predefinedModelTableModel=new DefaultTableModel(predefinedmodelstable,columns);
        predefinedModelTable=new JTable(predefinedModelTableModel) {
            public boolean isCellEditable(int row,int col) {return false;}
        };
        predefinedModelTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        predefinedModelTable.getColumn("Order").setMinWidth(15);
        predefinedModelTable.getColumn("Order").setMaxWidth(50);
        predefinedModelTable.removeColumn(predefinedModelTable.getColumn("Filename"));
        predefinedModelsPanel.setLayout(new BorderLayout());
        scrollpane=new JScrollPane(predefinedModelTable);
        predefinedModelTable.setFillsViewportHeight(true);
        predefinedModelTable.setAutoCreateRowSorter(true);
        predefinedModelTable.getTableHeader().setReorderingAllowed(false);
        predefinedModelsPanel.add(scrollpane,BorderLayout.CENTER);
        JPanel statuspanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        predefinedPanelErrorLabel=new JLabel("  ");
        predefinedPanelErrorLabel.setFont(new Font(Font.SANS_SERIF,Font.BOLD,12));
        predefinedPanelErrorLabel.setForeground(java.awt.Color.RED);
        statuspanel.add(predefinedPanelErrorLabel);
        predefinedModelsPanel.add(statuspanel,BorderLayout.SOUTH);
        predefinedModelTable.getColumn("Order").setCellRenderer(new CellRenderer_RightAlign());
        predefinedModelTable.getColumn("Organism").setCellRenderer(new CellRenderer_Organism());
        predefinedModelTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (showExisting) return;
                int row=predefinedModelTable.getSelectedRow();
                if (row>=0) {
                  String trackname=(String)predefinedModelTable.getValueAt(row, predefinedModelTable.getColumn("Name").getModelIndex());
                  setDataItemName(trackname);
                } else setDataItemName(data.getName());
            }
        });
        predefinedModelTable.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if (e.getClickCount()==2) clickOK();
                 }
        });
    }
    
    private void setupManualEntryPanel() {
        manualEntryPanel=new JPanel();
        String[] bases1 = new String[]{"A","C","G","T"};        
        manualEntryPanel.setLayout(new BoxLayout(manualEntryPanel, BoxLayout.Y_AXIS));
               
        snfTableModel=new DefaultTableModel(new Float[][]{{new Float(data.getSNF('A')),new Float(data.getSNF('C')),new Float(data.getSNF('G')),new Float(data.getSNF('T'))}},bases1);
        JTable snfTable=new JTable(snfTableModel);
        ExcelAdapter adapter1=new ExcelAdapter(snfTable,false, ExcelAdapter.CONVERT_TO_FLOAT);
        snfTable.setFillsViewportHeight(true);
        matrixTableModel=new DefaultTableModel(data.getTransitionMatrix(),bases);
        matrixTable=new JTable(matrixTableModel);
        ExcelAdapter adapter2=new ExcelAdapter(matrixTable,false, ExcelAdapter.CONVERT_TO_FLOAT);
        scrollpane=new JScrollPane(matrixTable);
        rowheaderView=getRowHeaderView(data.getOrder());
        scrollpane.setRowHeaderView(rowheaderView);
        Dimension rowsize=new Dimension(rowheadersize,170);
        scrollpane.getRowHeader().setMaximumSize(rowsize);
        scrollpane.getRowHeader().setMinimumSize(rowsize);
        scrollpane.getRowHeader().setPreferredSize(rowsize);
        matrixTable.setFillsViewportHeight(true);
        snfTable.getTableHeader().setReorderingAllowed(false);
        matrixTable.getTableHeader().setReorderingAllowed(false);
        snfTable.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        snfTable.setCellSelectionEnabled(true);
        matrixTable.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        matrixTable.setCellSelectionEnabled(true);
        
        float[] oligoFrequencies=data.getOligoFrequencies();
        Float[][] oligovalues=new Float[oligoFrequencies.length][1];
        for (int i=0;i<oligovalues.length;i++) oligovalues[i][0]=oligoFrequencies[i];
        oligoTableModel=new DefaultTableModel(oligovalues,new String[]{"Frequency"});
        oligoTable=new JTable(oligoTableModel);
        ExcelAdapter adapter3=new ExcelAdapter(oligoTable,false, ExcelAdapter.CONVERT_TO_FLOAT);
        oligoTable.setFillsViewportHeight(true);
        oligoTable.getTableHeader().setReorderingAllowed(false);
        oligoTable.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        oligoTable.setCellSelectionEnabled(true);        
        oligoscrollpane=new JScrollPane(oligoTable);
        oligoRowheaderView=getOligoRowHeaderView(data.getOrder());
        oligoscrollpane.setRowHeaderView(oligoRowheaderView);
        oligoscrollpane.getRowHeader().setMaximumSize(rowsize);
        oligoscrollpane.getRowHeader().setMinimumSize(rowsize);
        oligoscrollpane.getRowHeader().setPreferredSize(rowsize);
        
        JPanel internal1=new JPanel(new FlowLayout(FlowLayout.LEADING));
        JPanel internal2label=new JPanel(new FlowLayout(FlowLayout.LEADING));
        JPanel internal3label=new JPanel(new FlowLayout(FlowLayout.LEADING));
        JPanel internal4label=new JPanel(new FlowLayout(FlowLayout.LEADING));
        internal1.add(new JLabel("Model Order   "));
        orderCombobox=new JComboBox(new Integer[]{0,1,2,3,4,5});
        internal1.add(orderCombobox);

        Dimension d=new Dimension(400,38);
        Dimension d2=new Dimension(400,120);

        JPanel internal2table=new JPanel(new BorderLayout());
        JPanel internal2=new JPanel(new FlowLayout(FlowLayout.LEADING));
        JPanel internal3=new JPanel(new FlowLayout(FlowLayout.LEADING));
        JPanel internal4=new JPanel(new FlowLayout(FlowLayout.LEADING));
        internal2table.add(snfTable.getTableHeader(),BorderLayout.NORTH);
        internal2table.add(snfTable,BorderLayout.CENTER);        
        internal3.add(oligoscrollpane);
        internal4.add(scrollpane);
        scrollpane.setCorner(JScrollPane.UPPER_LEFT_CORNER, rowheaderView.getTableHeader());
        oligoscrollpane.setCorner(JScrollPane.UPPER_LEFT_CORNER, oligoRowheaderView.getTableHeader());
        internal2table.setMinimumSize(d);
        internal2table.setMaximumSize(d);
        internal2table.setPreferredSize(d);
        scrollpane.setMinimumSize(d2);
        scrollpane.setMaximumSize(d2);
        scrollpane.setPreferredSize(d2);
        oligoscrollpane.setMinimumSize(d2);
        oligoscrollpane.setMaximumSize(d2);
        oligoscrollpane.setPreferredSize(d2);
        JPanel statusPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        statusPanel.setMinimumSize(d);
        statusPanel.setMaximumSize(d);
        statusPanel.setPreferredSize(d);
        JPanel statuspanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        manualPanelErrorLabel=new JLabel("  ");
        manualPanelErrorLabel.setFont(new Font(Font.SANS_SERIF,Font.BOLD,12));
        manualPanelErrorLabel.setForeground(java.awt.Color.RED);
        statuspanel.add(manualPanelErrorLabel);
        internal2.add(internal2table);
        internal2label.add(new JLabel("Single Nucleotide Frequencies"));
        internal3label.add(new JLabel("Oligo Frequencies"));
        internal4label.add(new JLabel("Transition Matrix"));
        statusPanel.add(manualPanelErrorLabel);
        manualEntryPanel.add(internal1);
        manualEntryPanel.add(internal2label);
        manualEntryPanel.add(internal2);
        manualEntryPanel.add(internal3label);
        manualEntryPanel.add(internal3);
        manualEntryPanel.add(internal4label);
        manualEntryPanel.add(internal4);
        manualEntryPanel.add(statusPanel);
        orderCombobox.setSelectedItem(new Integer(data.getOrder()));
        orderCombobox.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                int order=((Integer)orderCombobox.getSelectedItem()).intValue();
                matrixTableModel=new DefaultTableModel(getDefaultModel(order),bases);
                matrixTable.setModel(matrixTableModel);
                rowheaderView=getRowHeaderView(order);
                scrollpane.setRowHeaderView(rowheaderView);
                scrollpane.setCorner(JScrollPane.UPPER_LEFT_CORNER, rowheaderView.getTableHeader());
                Dimension rowsize=new Dimension(rowheadersize,170);
                scrollpane.getRowHeader().setMaximumSize(rowsize);
                scrollpane.getRowHeader().setMinimumSize(rowsize);
                scrollpane.getRowHeader().setPreferredSize(rowsize);

                oligoTableModel=new DefaultTableModel(getDefaultOligoModel(order),new String[]{"Frequency"});
                oligoTable.setModel(oligoTableModel);     
                oligoRowheaderView=getOligoRowHeaderView(order);
                oligoscrollpane.setRowHeaderView(oligoRowheaderView);
                oligoscrollpane.setCorner(JScrollPane.UPPER_LEFT_CORNER, oligoRowheaderView.getTableHeader());
                oligoscrollpane.getRowHeader().setMaximumSize(rowsize);
                oligoscrollpane.getRowHeader().setMinimumSize(rowsize);
                oligoscrollpane.getRowHeader().setPreferredSize(rowsize);            
                manualEntryPanel.repaint();            
            }
        });        
    }
    
    private void setupImportModelPanel() {  
        ArrayList<DataFormat> dataformats=engine.getDataInputFormats(BackgroundModel.class);
        //dataformats.remove(engine.getDataFormat("Plain"));
        importModelPanel=new LoadFromFilePanel(dataformats,gui,BackgroundModel.class);
    }
    
    
    
    private JTable getRowHeaderView(int order) {
        int size=(int)Math.pow(4,order);
        String[][] prefixes=new String[size][1];
        for (int i=0;i<size;i++) {
            prefixes[i]=new String[]{BackgroundModel.integerToPattern(i, order)};
        }        
        DefaultTableModel rowHeaders=new DefaultTableModel(prefixes,new String[]{"Prefix"});
        JTable header=new JTable(rowHeaders);
        header.setFillsViewportHeight(true);
        header.getColumn(header.getColumnName(0)).setMaxWidth(rowheadersize);
        header.getTableHeader().setReorderingAllowed(false);
        header.setEnabled(false);
        Dimension d=new Dimension(rowheadersize,170);
        header.setMinimumSize(d);     
        return header;
    }
    
    private JTable getOligoRowHeaderView(int order) {
        int size=4;
        if (order>1) size=(int)Math.pow(4,order);
        String[][] prefixes=new String[size][1];
        for (int i=0;i<size;i++) {
            prefixes[i]=new String[]{BackgroundModel.integerToPattern(i, (order==0)?1:order)};
        }        
        DefaultTableModel rowHeaders=new DefaultTableModel(prefixes,new String[]{"Oligo"});
        JTable header=new JTable(rowHeaders);
        header.setFillsViewportHeight(true);
        header.getColumn(header.getColumnName(0)).setMaxWidth(rowheadersize);
        header.getTableHeader().setReorderingAllowed(false);
        header.setEnabled(false);
        Dimension d=new Dimension(rowheadersize,170);
        header.setMinimumSize(d);
        return header;
    }
    
    
    private Float[][] getDefaultModel(int order) {
        int size=(int)Math.pow(4,order);
        Float[][] values=new Float[size][4];
        for (int i=0;i<size;i++) {
            for (int j=0;j<4;j++) values[i][j]=new Float(0.25f);
        }        
        return values;
    }
    
    private Float[][] getDefaultOligoModel(int order) {
        int size=4;
        if (order>1) size=(int)Math.pow(4,order);
        float freq=1.0f/size;
        Float[][] values=new Float[size][1];
        for (int i=0;i<size;i++) {
           values[i][0]=new Float(freq);
        }        
        return values;
    }
    
    
    @Override
    public boolean onOKPressed() {
        if (tabbedPanel.getSelectedComponent()==predefinedModelsPanel) {
             int row=predefinedModelTable.getSelectedRow();
             if (row<0) return false;
             String modelname=(String)predefinedModelTable.getValueAt(row, 0);
             data.setPredefinedModel(modelname);
             if (showExisting) {
                 try {
                     data=importPredefinedModel(data);
                 } catch (Exception e) {
                      //e.printStackTrace(System.err);
                      reportPredefinedModelError(e.getMessage());
                      return false;                    
                 }
             }
        } else if (tabbedPanel.getSelectedComponent()==importModelPanel) { 
            try {
                String filename=importModelPanel.getFilename();
                if (filename==null) throw new SystemError("Missing filename");
                data=(BackgroundModel)importModelPanel.loadData(data,BackgroundModel.getType());
            } catch (Exception e) {
                String exceptionText=e.getClass().getSimpleName();
                if (exceptionText.contains("ParseError") || exceptionText.contains("ExecutionError")) exceptionText="";
                else exceptionText+=":";
                JOptionPane.showMessageDialog(this, "An error occurred while importing background model from file:\n"+exceptionText+e.getMessage(),"Import Error",JOptionPane.ERROR_MESSAGE);
                return false;
            }
        } else if (tabbedPanel.getSelectedComponent()==manualEntryPanel) {
            int order=((Integer)orderCombobox.getSelectedItem()).intValue();
            float[] newSNF=new float[4];
            for (int col=0;col<4;col++) {
                String txt=snfTableModel.getValueAt(0, col).toString();
                try {
                    newSNF[col]=Float.parseFloat(txt);
                } catch (NumberFormatException e) {
                    reportManualEntryError("Value in SNF column "+(col+1)+" is not a valid number");
                    return false;
                }
            }
            float sumSNF=newSNF[0]+newSNF[1]+newSNF[2]+newSNF[3];
            if (sumSNF>1.02 || sumSNF<0.98) {
                reportManualEntryError("Values in SNF should sum to 1.0 (Currently "+sumSNF+")");
                return false;
            }    
            int oligorows=4;
            if (order>1) oligorows=((int)Math.pow(4, order));  
            int oligoorder=order;
            if (oligoorder==0) oligoorder=1;
            float[] newOligos=new float[oligorows];
            for (int row=0;row<oligorows;row++) {
                String txt=oligoTableModel.getValueAt(row, 0).toString();
                try {
                    newOligos[row]=Float.parseFloat(txt);
                } catch (NumberFormatException e) {
                    String prefix=BackgroundModel.integerToPattern(row, oligoorder);
                    reportManualEntryError("Oligo frequency value for "+(prefix)+" is not a valid number");
                    return false;
                }
            }
            float oligosum=0;
            for (int row=0;row<oligorows;row++) oligosum+=newOligos[row];
            if (oligosum>1.02 || oligosum<0.98) {
                reportManualEntryError("Sum of Oligo frequencies should be 1.0 (currently "+oligosum+")");
                return false;
            }                                   
            int rows=((int)Math.pow(4, order));            
            int size=rows*4;
            float[] newMatrix=new float[size];
            int i=0;
            for (int r=0;r<rows;r++) {
                for (int c=0;c<4;c++) {
                   String txt=matrixTableModel.getValueAt(r, c).toString();
                   try {
                       newMatrix[i]=Float.parseFloat(txt);
                       //System.err.println("value("+r+","+c+")[i="+i+"] set to=> "+Float.parseFloat(txt));
                   } catch (NumberFormatException e) {
                       String prefix=BackgroundModel.integerToPattern(r, order);
                       String nextbase=BackgroundModel.integerToPattern(c, 1);
                       reportManualEntryError("Value in transition matrix("+prefix+","+nextbase+") is not a valid number");
                       return false;
                   }                   
                i++;
                }
                float rowsum=newMatrix[r*4]+newMatrix[r*4+1]+newMatrix[r*4+2]+newMatrix[r*4+3];
                if (rowsum>1.02 || rowsum<0.98) {
                   String prefix=BackgroundModel.integerToPattern(r, order);
                   reportManualEntryError("Values in transition matrix row="+prefix+" should sum to 1.0 (Currently "+rowsum+")");
                   return false;
                }                
            }
            data.setValue(newSNF, newOligos,newMatrix);
        }
        String newName=getDataItemName();
        if (!data.getName().equals(newName)) data.rename(newName);
        return true;
    }
    
    @Override
    public Data getData() {
       return data; 
    }
    

    private void reportManualEntryError(String msg) {
        if (msg==null) msg="NULL";
        manualPanelErrorLabel.setText(msg);
    }
    
    private void reportPredefinedModelError(String msg) {
        if (msg==null) msg="NULL";
        predefinedPanelErrorLabel.setText(msg);
    }
    
    
    @Deprecated
    private BackgroundModel importModelFromFile(String filename, BackgroundModel model) throws IOException, ParseError, SystemError, InterruptedException {
        // this is executed on the EDT for now
        File file=new File(filename);
        BufferedReader inputStream=null;
        ArrayList<String> input=new ArrayList<String>();
        try {
            inputStream=new BufferedReader(new FileReader(file));
            String line;
            while((line=inputStream.readLine())!=null) {input.add(line);}
        } catch (IOException e) { 
            throw e;
        } finally {
            try {if (inputStream!=null) inputStream.close();} catch (IOException ioe) {System.err.println("SYSTEM ERROR: An error occurred when closing BufferedReader in Prompt_BackgroundModel.importModelFromFile: "+ioe.getMessage());}
        }        
        DataFormat format=engine.getDataFormat("INCLUSive_Background_Model");
        if (format==null) throw new SystemError("Unknown DataFormat: INCLUSive_Background_Model");
        model=(BackgroundModel)format.parseInput(input, model,null, null);
        return model;
    }

    private BackgroundModel importPredefinedModel(BackgroundModel data) throws ExecutionError {
            String modelName=data.getPredefinedModel();
            if (modelName==null) throw new ExecutionError("SYSTEM ERROR: No predefined model defined");
            String filename=BackgroundModel.getFilenameForModel(modelName);
            if (filename==null) throw new ExecutionError("Unknown Background Model: "+modelName);
            BufferedReader inputStream=null;
            ArrayList<String> input=new ArrayList<String>();
            try {
                inputStream=new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/priorseditor/engine/resources/"+filename)));
                String line;
                while((line=inputStream.readLine())!=null) {input.add(line);}
            } catch (IOException e) { 
                throw new ExecutionError("An error occurred when loading predefined Background Model:\n["+e.getClass().getSimpleName()+"] "+e.getMessage(),0);
            } finally {
                try {
                    if (inputStream!=null) inputStream.close();
                } catch (IOException ioe) {System.err.println("SYSTEM ERROR: An error occurred when closing BufferedReader Prompt_BackgroundModel.importPredefinedModel(): "+ioe.getMessage());}
            }           
            DataFormat_INCLUSive_Background format=new DataFormat_INCLUSive_Background();   
            try {data=(BackgroundModel)format.parseInput(input, data,null,null);}
            catch (Exception e) {throw new ExecutionError(e.getClass().getSimpleName()+":"+e.getMessage());} 
            data.setPredefinedModel(modelName);
            return data;        
    }
    
    
    
    
    
private class CellRenderer_Organism extends DefaultTableCellRenderer {

    public CellRenderer_Organism() {
           super();
           this.setHorizontalAlignment(this.LEFT);              
       }
    public void setValue(Object value) {
           if (value==null) {setText("");}
           else if (value instanceof Integer) {
               if (((Integer)value).intValue()==0) setForeground(java.awt.Color.RED);
               else setForeground(java.awt.Color.BLACK);
               setText(Organism.getCommonName(((Integer)value).intValue()));
           }  else {
               setForeground(java.awt.Color.RED);
               setText("* * *");
           }
       }
}// end class CellRenderer_Organism

private class CellRenderer_RightAlign extends DefaultTableCellRenderer {
    public CellRenderer_RightAlign() {
           super();
           this.setHorizontalAlignment(this.RIGHT);              
       }
    public void setValue(Object value) {
           if (value!=null)setText(value.toString());
           else setText("");
       }
}// end class CellRenderer_RightAlign    
    
}
