/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.prompt;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import priorseditor.engine.data.Data;
import priorseditor.engine.data.Motif;
import priorseditor.engine.data.MotifClassification;
import priorseditor.engine.protocol.ParseError;
import priorseditor.gui.ExcelAdapter;
import priorseditor.gui.MotifClassDialog;
import priorseditor.gui.PriorsEditorGUI;
import priorseditor.gui.VisualizationSettings;

/**
 *
 * @author kjetikl
 */
public class Prompt_Motif extends Prompt {
    private javax.swing.JLabel bindingFactorsLabel;
    private javax.swing.JScrollPane bindingFactorsScrollPane;
    private javax.swing.JTextArea bindingFactorsTextArea;
    private javax.swing.JTextArea classTextArea;
    private javax.swing.JLabel classLabel;
    private javax.swing.JLabel consensusLabel;
    private javax.swing.JTextField consensusTextField;
    private javax.swing.JLabel errorLabel;
    private javax.swing.JCheckBox halfsiteCheckbox;
    private javax.swing.JLabel icLabel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel logoLabel;
    private SequenceLogoPanel logoPanel;
    private javax.swing.JTextField longNameTextField;
    private javax.swing.JLabel matrixLabel;
    private javax.swing.JScrollPane matrixScrollPane;
    private javax.swing.JTable matrixTable;
    private javax.swing.JLabel motifLongNameLabel;
    private javax.swing.JLabel motifShortNameLabel;
    private javax.swing.JTextArea organismTextArea;
    private javax.swing.JScrollPane organismScrollPane;
    private javax.swing.JLabel organismsLabel;
    private javax.swing.JComboBox qualityCombobox;
    private javax.swing.JLabel qualityLabel;
    private javax.swing.JTextField shortNameTextField;
    private javax.swing.JLabel sizeLabel;
    private javax.swing.JPanel mainPanel;
    private VisualizationSettings settings=null;
    private String currentMotifClass=null;
    private TableModelListener tableModelListener=null;
    
    private Motif data;
    private final java.awt.Frame parenthack;
    private String oldConsensus=null;
    
    
    public Prompt_Motif(PriorsEditorGUI gui, String prompt, Motif dataitem, VisualizationSettings settings) {
        super(gui,prompt);
        parenthack=gui.getFrame();
        this.settings=settings;
        if (dataitem!=null) data=dataitem;
        else data=new Motif(gui.getGenericDataitemName(Motif.class, null));
        setDataItemName(data.getName());
        setDataItemNameLabelText("Motif ID  ");
        setTitle("Motif");
        mainPanel=new JPanel();
        JPanel insetPanel=new JPanel(new FlowLayout(FlowLayout.LEADING));
        insetPanel.add(mainPanel);
        initComponents();
        setMainPanel(new JScrollPane(insetPanel));
        DefaultTableModel model=null;
        if (data.getMatrix()==null) model=new DefaultTableModel(new Float[][]{{0.25f,0.25f,0.25f,0.25f},{0.25f,0.25f,0.25f,0.25f},{0.25f,0.25f,0.25f,0.25f},{0.25f,0.25f,0.25f,0.25f}},new String[]{"A","C","G","T"});
        else model=new DefaultTableModel(convertMatrix(data.getMatrix()),new String[]{"A","C","G","T"});
        matrixTable.setModel(model);
        matrixTable.setFillsViewportHeight(true);
        matrixTable.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        matrixTable.setCellSelectionEnabled(true);
        matrixTable.getTableHeader().setReorderingAllowed(false);
        matrixScrollPane.setMaximumSize(new Dimension(1000,220));
        ExcelAdapter adapter=new ExcelAdapter(matrixTable,true, ExcelAdapter.CONVERT_TO_FLOAT);
        sizeLabel.setText("Size: "+data.getLength()+" bp");
        icLabel.setText("IC: "+data.getICcontent());
        shortNameTextField.setText(data.getShortName());
        shortNameTextField.setCaretPosition(0);
        longNameTextField.setText(data.getLongName());
        longNameTextField.setCaretPosition(0);
        consensusTextField.setText(data.getConsensusMotif());
        consensusTextField.setCaretPosition(0);
        qualityCombobox.setSelectedItem(data.getQuality());
        currentMotifClass=data.getClassification();
        classTextArea.setText(MotifClassification.getFullLevelsString(currentMotifClass));
        String organismText=data.getOrganisms();
        if (organismText!=null) organismText=organismText.replace(',', '\n');
        organismTextArea.setText(organismText);
        organismTextArea.setCaretPosition(0);
        String bindingFactorsText=data.getBindingFactors();
        if (bindingFactorsText!=null) bindingFactorsText=bindingFactorsText.replace(',', '\n');        
        bindingFactorsTextArea.setText(bindingFactorsText);
        bindingFactorsTextArea.setCaretPosition(0);
        halfsiteCheckbox.setSelected(data.isHalfsite());
        oldConsensus=data.getConsensusMotif();        
        classTextArea.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {                 
                if (e.getClickCount()==2) {
                    MotifClassDialog dialog=new MotifClassDialog(parenthack,currentMotifClass);
                    dialog.setLocation(parenthack.getWidth()/2-dialog.getWidth()/2, parenthack.getHeight()/2-dialog.getHeight()/2);
                    dialog.setVisible(true);
                    if (dialog.isOKClicked()) {
                        currentMotifClass=dialog.getSelectedClass();
                        classTextArea.setText(MotifClassification.getFullLevelsString(currentMotifClass));
                    }
                }
            }
            }
        );
        tableModelListener=new TableModelListener() {
            @Override
            public void tableChanged(TableModelEvent e) {
                updateConsensusFromPWM();
                logoPanel.setClearOnNextPaint();
                logoPanel.repaint(); // updates sequence logo
            }
        };
        model.addTableModelListener(tableModelListener);
        consensusTextField.addFocusListener(new FocusAdapter() {
            @Override
            public void focusLost(FocusEvent e) {
                onConsensusUpdate();
            }            
        });
        consensusTextField.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                onConsensusUpdate();
            }
        });
        
        pack();  
        // check if the dialog window fits. If not resize it
        Dimension dim=getSize();
        Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
        if (dim.height>screen.height-100) {
            setSize(screen.height-100,dim.width);
        }
    }
    
    private void onConsensusUpdate() {
        String newconsensus=consensusTextField.getText().trim();
        if (!newconsensus.equalsIgnoreCase(oldConsensus)) {
            oldConsensus=newconsensus;
            updatePWMfromConsensus(newconsensus);
            logoPanel.setClearOnNextPaint();
            logoPanel.repaint(); // updates sequence logo
        }        
    }
    
    
    /** This method should be called if the user updates the consensus-string in the prompt
     *  it will update the PWM matrix to fit the consensus string
     */
    private void updatePWMfromConsensus(String newconsensus) {
        // check if the multiple sequences have been entered
        Float[][] matrix=null;
        String[] sequences=newconsensus.split("[^a-zA-Z]+");
        if (sequences.length>1) { // multiple aligned sequences. 
            int consensuslength=sequences[0].length();
            matrix=new Float[consensuslength][4];
            // initialize the matrix with zeros! 
            for (int i=0;i<consensuslength;i++) {matrix[i][0]=0f;matrix[i][1]=0f;matrix[i][2]=0f;matrix[i][3]=0f;}
            for (int i=0;i<sequences.length;i++) {
                 if (sequences[i].length()!=consensuslength) {reportError("Aligned sequences are not of equal length");return;}                  
                 for (int j=0;j<sequences[i].length();j++) {
                    char base=Character.toUpperCase(sequences[i].charAt(j));
                         if (base=='A' || base=='a') {matrix[j][0]+=1f;}
                    else if (base=='C' || base=='c') {matrix[j][1]+=1f;}
                    else if (base=='G' || base=='g') {matrix[j][2]+=1f;}
                    else if (base=='T' || base=='t') {matrix[j][3]+=1f;}
                 }                 
            }
        } else { // single consensus sequence
            matrix=new Float[newconsensus.length()][4];
            for (int i=0;i<newconsensus.length();i++) {
                char base=Character.toUpperCase(newconsensus.charAt(i));
                     if (base=='A') {matrix[i][0]=12f;matrix[i][1]=0f;matrix[i][2]=0f;matrix[i][3]=0f;}
                else if (base=='C') {matrix[i][0]=0f;matrix[i][1]=12f;matrix[i][2]=0f;matrix[i][3]=0f;}
                else if (base=='G') {matrix[i][0]=0f;matrix[i][1]=0f;matrix[i][2]=12f;matrix[i][3]=0f;}
                else if (base=='T') {matrix[i][0]=0f;matrix[i][1]=0f;matrix[i][2]=0f;matrix[i][3]=12f;}
                else if (base=='R') {matrix[i][0]=6f;matrix[i][1]=0f;matrix[i][2]=6f;matrix[i][3]=0f;}
                else if (base=='Y') {matrix[i][0]=0f;matrix[i][1]=6f;matrix[i][2]=0f;matrix[i][3]=6f;}
                else if (base=='M') {matrix[i][0]=6f;matrix[i][1]=6f;matrix[i][2]=0f;matrix[i][3]=0f;}
                else if (base=='K') {matrix[i][0]=0f;matrix[i][1]=0f;matrix[i][2]=6f;matrix[i][3]=6f;}
                else if (base=='W') {matrix[i][0]=6f;matrix[i][1]=0f;matrix[i][2]=0f;matrix[i][3]=6f;}
                else if (base=='S') {matrix[i][0]=0f;matrix[i][1]=6f;matrix[i][2]=6f;matrix[i][3]=0f;}
                else if (base=='B') {matrix[i][0]=0f;matrix[i][1]=4f;matrix[i][2]=4f;matrix[i][3]=4f;}
                else if (base=='D') {matrix[i][0]=4f;matrix[i][1]=0f;matrix[i][2]=4f;matrix[i][3]=4f;}
                else if (base=='H') {matrix[i][0]=4f;matrix[i][1]=4f;matrix[i][2]=0f;matrix[i][3]=4f;}
                else if (base=='V') {matrix[i][0]=4f;matrix[i][1]=4f;matrix[i][2]=4f;matrix[i][3]=0f;}
                else {matrix[i][0]=3f;matrix[i][1]=3f;matrix[i][2]=3f;matrix[i][3]=3f;}
            }
        }
        DefaultTableModel model=new DefaultTableModel(matrix,new String[]{"A","C","G","T"});
        matrixTable.setModel(model);
        model.addTableModelListener(tableModelListener);
        updateConsensusFromPWM();              
    }
    
    /** This method should be called if the user updates the PWM matrix in the prompt
     *  it will update the consensus string to fit the PWM matrix
     */
    private void updateConsensusFromPWM() {
        float[][] matrix=null;
        try {
           matrix=parseMatrixTable();
           data.setMatrix(matrix); // necessary for the sequence logo to be updated
           consensusTextField.setText(Motif.getConsensusForMatrix(matrix));
           oldConsensus=consensusTextField.getText();
           icLabel.setText("IC: "+Motif.calculateInformationContent(matrix, false));
           sizeLabel.setText("Size: "+matrix.length+" bp");
           mainPanel.revalidate();
           reportError(""); // clear the error message
        } catch (ParseError e) {
           reportError(e.getMessage());
        }
    }
    
    
    
    
    @Override
    public Data getData() {
        return data;
    }

    @Override
    public boolean onOKPressed() {  
        float[][] matrix=null;
        try {
           matrix=parseMatrixTable();
           data.setMatrix(matrix);
        } catch (ParseError e) {
           reportError(e.getMessage());
           return false;
        }
        String consensus=consensusTextField.getText();
        if (consensus!=null && !consensus.trim().isEmpty()) data.setConsensusMotif(consensus.trim()); 
        else data.setConsensusMotif(Motif.getConsensusForMatrix(matrix));  
        String shortName=shortNameTextField.getText();
        if (shortName!=null && !shortName.trim().isEmpty()) data.setShortName(shortName.trim()); else data.setShortName(data.getConsensusMotif());
        String longName=longNameTextField.getText();
        if (longName!=null && !longName.trim().isEmpty()) data.setLongName(longName.trim()); else data.setLongName(null);
        data.setClassification(currentMotifClass);
        data.setHalfsite(halfsiteCheckbox.isSelected());
        int quality=((Integer)qualityCombobox.getSelectedItem()).intValue();
        data.setQuality(quality);
        
        String organisms=organismTextArea.getText();
        if (organisms!=null) organisms=processString(organisms.trim());
        if (organisms!=null && !organisms.isEmpty()) data.setOrganisms(organisms);
        else data.setOrganisms(null);
        String factors=bindingFactorsTextArea.getText();
        if (factors!=null) factors=processString(factors.trim());
        if (factors!=null && !factors.isEmpty()) data.setBindingFactors(factors);
        else data.setBindingFactors(null);
        data.setICcontent(Motif.calculateInformationContent(matrix, false));      
        String newName=getDataItemName();
        if (!data.getName().equals(newName)) data.rename(newName);  
        return true;
    }

    /** 
     * Converts a float[][] matrix to a Object[][] matrix (actually Float[][])
     * so that it can be used as argument for a DefaultTableModel
     * (autoboxing can not convert arrays of primitives to arrays of objects)
     */
    private Float[][] convertMatrix(float[][] m) {
        if (m==null) return null;
        Float[][] res=new Float[m.length][4];
        for(int i=0;i<m.length;i++) {
            res[i][0]=(Float)m[i][0];
            res[i][1]=(Float)m[i][1];
            res[i][2]=(Float)m[i][2];
            res[i][3]=(Float)m[i][3];
        }
        return res;
    }
    
    
    /** Returns the values to use for the matrix */
    private float[][] parseMatrixTable() throws ParseError {
        int rowcount=matrixTable.getRowCount();
        int highestset=0;
        for (int i=0;i<rowcount;i++) {
            Object value1=matrixTable.getValueAt(i, 0);
            Object value2=matrixTable.getValueAt(i, 1);
            Object value3=matrixTable.getValueAt(i, 2);
            Object value4=matrixTable.getValueAt(i, 3);
            boolean Aset=false;boolean Cset=false;boolean Gset=false;boolean Tset=false;
            if (value1!=null && !(value1.toString()).isEmpty()) Aset=true;
            if (value2!=null && !(value2.toString()).isEmpty()) Cset=true;
            if (value3!=null && !(value3.toString()).isEmpty()) Gset=true;
            if (value4!=null && !(value4.toString()).isEmpty()) Tset=true;
            boolean someset=(Aset||Cset||Gset||Tset);
            if (someset && !Aset) throw new ParseError("Missing value for A in row "+(i+1));
            if (someset && !Cset) throw new ParseError("Missing value for C in row "+(i+1));
            if (someset && !Gset) throw new ParseError("Missing value for G in row "+(i+1));
            if (someset && !Tset) throw new ParseError("Missing value for T in row "+(i+1));
            if (someset) highestset=i; else break;
        }
        float[][] matrix=new float[highestset+1][4];
        for (int i=0;i<=highestset;i++) {
            try {matrix[i][0]=Float.parseFloat(matrixTable.getValueAt(i, 0).toString());} catch (Exception e) {throw new ParseError("Unable to parse expected numeric value for base A in row "+(i+1));}
            try {matrix[i][1]=Float.parseFloat(matrixTable.getValueAt(i, 1).toString());} catch (Exception e) {throw new ParseError("Unable to parse expected numeric value for base C in row "+(i+1));}
            try {matrix[i][2]=Float.parseFloat(matrixTable.getValueAt(i, 2).toString());} catch (Exception e) {throw new ParseError("Unable to parse expected numeric value for base G in row "+(i+1));}
            try {matrix[i][3]=Float.parseFloat(matrixTable.getValueAt(i, 3).toString());} catch (Exception e) {throw new ParseError("Unable to parse expected numeric value for base T in row "+(i+1));}
        }
        return matrix;
    }
    
    private String processString(String input) {
        input=input.replaceAll("\n", ",");
        return input;
    }
    
    private void reportError(String msg) {
        if (msg==null) msg="NULL";
        errorLabel.setText(msg);
    }
    
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jLabel1 = new javax.swing.JLabel();
        motifShortNameLabel = new javax.swing.JLabel();
        motifLongNameLabel = new javax.swing.JLabel();
        consensusLabel = new javax.swing.JLabel();
        shortNameTextField = new javax.swing.JTextField();
        longNameTextField = new javax.swing.JTextField(40);
        consensusTextField = new javax.swing.JTextField(40);
        matrixLabel = new javax.swing.JLabel();
        matrixScrollPane = new javax.swing.JScrollPane();
        matrixTable = new javax.swing.JTable();
        logoPanel = new SequenceLogoPanel();
        logoLabel = new javax.swing.JLabel();
        classLabel = new javax.swing.JLabel();
        qualityLabel = new javax.swing.JLabel();
        classTextArea = new javax.swing.JTextArea();
        qualityCombobox = new javax.swing.JComboBox();
        halfsiteCheckbox = new javax.swing.JCheckBox();
        sizeLabel = new javax.swing.JLabel();
        icLabel = new javax.swing.JLabel();
        bindingFactorsLabel = new javax.swing.JLabel();
        bindingFactorsScrollPane = new javax.swing.JScrollPane();
        bindingFactorsTextArea = new javax.swing.JTextArea();
        organismsLabel = new javax.swing.JLabel();
        errorLabel = new javax.swing.JLabel();
        organismScrollPane = new javax.swing.JScrollPane();
        organismTextArea = new javax.swing.JTextArea();

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(priorseditor.gui.PriorsEditorApp.class).getContext().getResourceMap(Prompt_Motif.class);
        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        mainPanel.setName("Form"); // NOI18N
        mainPanel.setLayout(new java.awt.GridBagLayout());

        motifShortNameLabel.setText(resourceMap.getString("motifShortNameLabel.text")); // NOI18N
        motifShortNameLabel.setName("motifShortNameLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        mainPanel.add(motifShortNameLabel, gridBagConstraints);

        motifLongNameLabel.setText(resourceMap.getString("motifLongNameLabel.text")); // NOI18N
        motifLongNameLabel.setName("motifLongNameLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        mainPanel.add(motifLongNameLabel, gridBagConstraints);

        consensusLabel.setText(resourceMap.getString("consensusLabel.text")); // NOI18N
        consensusLabel.setName("consensusLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 10);
        mainPanel.add(consensusLabel, gridBagConstraints);

        shortNameTextField.setColumns(16);
        shortNameTextField.setName("shortNameTextField"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        mainPanel.add(shortNameTextField, gridBagConstraints);

        longNameTextField.setName("longNameTextField"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        mainPanel.add(longNameTextField, gridBagConstraints);

        consensusTextField.setName("consensusTextField"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        mainPanel.add(consensusTextField, gridBagConstraints);

        matrixLabel.setText(resourceMap.getString("matrixLabel.text")); // NOI18N
        matrixLabel.setName("matrixLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        mainPanel.add(matrixLabel, gridBagConstraints);

        matrixScrollPane.setName("matrixScrollPane"); // NOI18N

        matrixTable.setName("matrixTable"); // NOI18N
        matrixScrollPane.setPreferredSize(new java.awt.Dimension(300, 160));
        matrixScrollPane.setViewportView(matrixTable);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        mainPanel.add(matrixScrollPane, gridBagConstraints);

        // logoPanel.setBackground(resourceMap.getColor("logoPanel.background")); // NOI18N
        logoPanel.setMinimumSize(new java.awt.Dimension(100, 60));
        logoPanel.setName("logoPanel"); // NOI18N
        logoPanel.setPreferredSize(new java.awt.Dimension(100, 60));

        javax.swing.GroupLayout logoPanelLayout = new javax.swing.GroupLayout(logoPanel);
        logoPanel.setLayout(logoPanelLayout);
        logoPanelLayout.setHorizontalGroup(
            logoPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 268, Short.MAX_VALUE)
        );
        logoPanelLayout.setVerticalGroup(
            logoPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 60, Short.MAX_VALUE)
        );

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        mainPanel.add(logoPanel, gridBagConstraints);

        logoLabel.setText(resourceMap.getString("logoLabel.text")); // NOI18N
        logoLabel.setName("logoLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        mainPanel.add(logoLabel, gridBagConstraints);

        classLabel.setText(resourceMap.getString("classLabel.text")); // NOI18N
        classLabel.setName("classLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        mainPanel.add(classLabel, gridBagConstraints);

        qualityLabel.setText(resourceMap.getString("qualityLabel.text")); // NOI18N
        qualityLabel.setName("qualityLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        mainPanel.add(qualityLabel, gridBagConstraints);

        classTextArea.setName("classTextArea"); // NOI18N
        classTextArea.setColumns(20);
        classTextArea.setRows(4);
        classTextArea.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        mainPanel.add(classTextArea, gridBagConstraints);

        qualityCombobox.setModel(new javax.swing.DefaultComboBoxModel(new Integer[] {1,2,3,4,5,6}));
        qualityCombobox.setName("qualityCombobox"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        mainPanel.add(qualityCombobox, gridBagConstraints);

        halfsiteCheckbox.setText(resourceMap.getString("halfsiteCheckbox.text")); // NOI18N
        halfsiteCheckbox.setName("halfsiteCheckbox"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        mainPanel.add(halfsiteCheckbox, gridBagConstraints);

        sizeLabel.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        sizeLabel.setText(resourceMap.getString("sizeLabel.text")); // NOI18N
        sizeLabel.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
        sizeLabel.setInheritsPopupMenu(false);
        sizeLabel.setMaximumSize(new java.awt.Dimension(120, 14));
        sizeLabel.setMinimumSize(new java.awt.Dimension(120, 14));
        sizeLabel.setName("sizeLabel"); // NOI18N
        sizeLabel.setPreferredSize(new java.awt.Dimension(120, 14));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        mainPanel.add(sizeLabel, gridBagConstraints);

        icLabel.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        icLabel.setText(resourceMap.getString("icLabel.text")); // NOI18N
        icLabel.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
        icLabel.setMaximumSize(new java.awt.Dimension(120, 14));
        icLabel.setMinimumSize(new java.awt.Dimension(120, 14));
        icLabel.setName("icLabel"); // NOI18N
        icLabel.setPreferredSize(new java.awt.Dimension(120, 14));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        mainPanel.add(icLabel, gridBagConstraints);

        bindingFactorsLabel.setText(resourceMap.getString("bindingFactorsLabel.text")); // NOI18N
        bindingFactorsLabel.setName("bindingFactorsLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        mainPanel.add(bindingFactorsLabel, gridBagConstraints);

        bindingFactorsScrollPane.setName("bindingFactorsScrollPane"); // NOI18N

        bindingFactorsTextArea.setColumns(20);
        bindingFactorsTextArea.setRows(3);
        bindingFactorsTextArea.setName("bindingFactorsTextArea"); // NOI18N
        bindingFactorsScrollPane.setViewportView(bindingFactorsTextArea);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        mainPanel.add(bindingFactorsScrollPane, gridBagConstraints);

        organismsLabel.setText(resourceMap.getString("organismsLabel.text")); // NOI18N
        organismsLabel.setName("organismsLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        mainPanel.add(organismsLabel, gridBagConstraints);

        errorLabel.setFont(resourceMap.getFont("errorLabel.font")); // NOI18N
        errorLabel.setForeground(resourceMap.getColor("errorLabel.foreground")); // NOI18N
        errorLabel.setText("   "); // NOI18N
        errorLabel.setName("errorLabel"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.gridwidth = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 10, 0);
        mainPanel.add(errorLabel, gridBagConstraints);

        organismScrollPane.setName("organismScrollPane"); // NOI18N

        organismTextArea.setName("organismList"); // NOI18N
        organismTextArea.setRows(3);
        organismScrollPane.setViewportView(organismTextArea);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        mainPanel.add(organismScrollPane, gridBagConstraints);
    }
        
    
    private class SequenceLogoPanel extends JPanel { 
        private Font font;
        private char[] bases;
        private Color[] basecolors;
        private float[] letterXoffset=new float[4];        
        private int fontheight=70; // if you change this size you must also change the ascentCorrection below accordingly (but I am not sure about the relationship...)
        private int ascentCorrection=20; // this is a needed hack because the ascent returned by the FontMetrics is not really the same height as that of a capital letter (even though that was stated in the documentation)
        private int xoffset=0;
        private int yoffset=0;
        private boolean clearOnNextPaint=false;
        
        public SequenceLogoPanel() {
            setOpaque(true);
            font=new Font(Font.SANS_SERIF,Font.BOLD,fontheight);
            bases=new char[]{'A','C','G','T'};
            basecolors=new Color[]{settings.getBaseColor('A'),settings.getBaseColor('C'),settings.getBaseColor('G'),settings.getBaseColor('T')};
        }
        
        @Override
        public void paintComponent(Graphics graphics) {
            //super.paintComponent(graphics);
            float[][] matrix=data.getMatrix();
            if (matrix==null) return;
            if (data.getMatrixType()==Motif.LOG_ODDS_MATRIX) matrix=data.getMatrixAsFrequencyMatrix();            
            Graphics2D g = (Graphics2D)graphics;
            if (clearOnNextPaint) {
                g.setColor(getBackground());
                g.fillRect(0, 0, getWidth(), getHeight());                
                clearOnNextPaint=false;
            }
            FontMetrics metrics=g.getFontMetrics(font);
            int ascent=metrics.getAscent()-ascentCorrection;
            int widthA=metrics.charWidth('A');     
            int widthC=metrics.charWidth('C');     
            int widthG=metrics.charWidth('G');     
            int widthT=metrics.charWidth('T');
            letterXoffset[0]=(widthG-widthA)/2;
            letterXoffset[1]=(widthG-widthC)/2;
            letterXoffset[3]=(widthG-widthT)/2;
            int logowidth=matrix.length*widthG;
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g.setFont(font);
            g.setColor(java.awt.Color.RED);
            AffineTransform restore=g.getTransform();
            AffineTransform save=g.getTransform();           
            if (logowidth>getWidth()) save.scale((float)getWidth()/(float)logowidth, 1); // scale X-direction so that logo fits irrespective of size
            AffineTransform scaleY = new AffineTransform();
            g.setColor(Color.WHITE);
            g.fillRect(xoffset, yoffset, logowidth, ascent);
            float xpos=xoffset;
            for (int i=0;i<matrix.length;i++) {
                float[] counts=new float[]{matrix[i][0],matrix[i][1],matrix[i][2],matrix[i][3]};
                float total=counts[0]+counts[1]+counts[2]+counts[3];
                float ic=Motif.calculateColumnIC(counts[0], counts[1], counts[2], counts[3], false);
                int[] sorted=new int[4]; // sorted in ascending order. Values are base-indices (i.e. 0=>A, 1=>C, 2=>G, 3=>T)
                int indexA=0, indexC=0, indexG=0, indexT=0;
                if (counts[0]>=counts[1]) indexA++;
                if (counts[0]>=counts[2]) indexA++;
                if (counts[0]>=counts[3]) indexA++;                
                if (counts[1]>counts[0]) indexC++;
                if (counts[1]>=counts[2]) indexC++;
                if (counts[1]>=counts[3]) indexC++;               
                if (counts[2]>counts[0]) indexG++;
                if (counts[2]>counts[1]) indexG++;
                if (counts[2]>=counts[3]) indexG++;                
                if (counts[3]>counts[0]) indexT++;
                if (counts[3]>counts[1]) indexT++;
                if (counts[3]>counts[2]) indexT++;
                sorted[indexA]=0;
                sorted[indexC]=1;
                sorted[indexG]=2;
                sorted[indexT]=3;
                float currentYoffset=yoffset+ascent-(ascent*ic/2f);
                for (int j=3;j>=0;j--) { // draws letters from top to bottom (most frequent first)
                    int base=sorted[j];
                    float fraction=counts[base]/total;
                    scaleY.setTransform(save);
                    scaleY.translate(letterXoffset[base],currentYoffset); // translated in 1-to-1 scale                    
                    scaleY.scale(1,ic/2f); // scale by IC-content in position                   
                    scaleY.scale(1, fraction);   
                    g.setColor(basecolors[base]);
                    g.setTransform(scaleY);
                    g.drawString(""+bases[base], xpos, ascent); // draw all letters at same position and use transform to place them correctly
                    currentYoffset+=ascent*fraction*(ic/2f);
                } // end for each base  
                xpos+=widthG; // 
            } // end for each position
            g.setTransform(save);
            g.setColor(Color.BLACK);
            g.drawLine(xoffset, yoffset, xoffset, ascent+yoffset); // draw vertical axis
            g.drawLine(xoffset, ascent+yoffset, (int)xpos, ascent+yoffset); // draw horizontal axis
            for (int i=0;i<=matrix.length;i++) { // draw axis ticks
               int x=xoffset+i*widthG;
               g.drawLine(x, ascent+yoffset, x, ascent+yoffset+3); 
            }
            g.setTransform(restore);
        }               

        
    public void setClearOnNextPaint() {clearOnNextPaint=true;}
    
    } // end class SequenceLogo

    
}
