/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.prompt;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Set;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.BevelBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import priorseditor.engine.ExecutionError;
import priorseditor.engine.ParameterSettings;
import priorseditor.engine.data.*;
import priorseditor.engine.SystemError;
import priorseditor.engine.dataformat.DataFormat;
import priorseditor.gui.LoadFromFilePanel;
import priorseditor.gui.MotifBrowserPanel;
import priorseditor.gui.PriorsEditorGUI;


/**
 *
 * @author kjetikl
 */
public class Prompt_MotifCollection extends Prompt {

    private MotifCollection data;

    private MotifBrowserPanel manualSelectionPanel;
    private JPanel predefinedCollectionsPanel;
    private LoadFromFilePanel importCollectionPanel;
    private JPanel parseListPanel;
    private JPanel fromTrackPanel;
    private JTabbedPane tabbedPanel;
    private DefaultTableModel predefinedCollectionsTableModel;
    private JTable predefinedCollectionsTable;
    private JLabel predefinedPanelErrorLabel;
    private JTextField sequencesupportnumber;
    private JComboBox fromTrackSelection;
    private JComboBox sequencesupportoperator;
    private JComboBox sequencesupportpercentageoperator;
    private JScrollPane scrollpane;
    private boolean showExisting=false;
    private boolean isimported=false;     

    private JTextArea parseListTextArea;


    private static final int UNION=0;
    private static final int INTERSECTION=1;
    private static final int SUBTRACTION=2;
    
    public Prompt_MotifCollection(PriorsEditorGUI gui, String prompt, MotifCollection dataitem) {
        super(gui,prompt);
        showExisting=(dataitem!=null);
        if (dataitem!=null) data=dataitem;
        else data=new MotifCollection(gui.getGenericDataitemName(MotifCollection.class, null));
        setDataItemName(data.getName());
        setTitle("Motif Collection");
        if (dataitem==null) setupPredefinedModelsPanel();
        manualSelectionPanel=new MotifBrowserPanel(gui,data);
        setupImportModelPanel();
        setupFromTrackPanel();
        setupParseListPanel();
        tabbedPanel=new JTabbedPane();
        if (dataitem==null) tabbedPanel.addTab("Predefined Collections", predefinedCollectionsPanel);
        tabbedPanel.addTab("Manual Selection", manualSelectionPanel);
        tabbedPanel.addTab("Parse List", parseListPanel);
        tabbedPanel.addTab("From Track", fromTrackPanel);
        if (dataitem==null) tabbedPanel.addTab("Import Collection", importCollectionPanel);
        JPanel internal=new JPanel(new BorderLayout());
        Dimension size=new Dimension(580,500);
        internal.setMinimumSize(size);
        internal.setPreferredSize(size);
        // internal.setMaximumSize(size);
        manualSelectionPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        if (dataitem==null)  predefinedCollectionsPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        if (dataitem==null) importCollectionPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        parseListPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        fromTrackPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        internal.add(tabbedPanel);
        this.setResizable(true);
        setMainPanel(internal);        
        pack();
        if (showExisting) tabbedPanel.setSelectedComponent(manualSelectionPanel);
    }
    
    private void setupPredefinedModelsPanel() {
        predefinedCollectionsPanel=new JPanel();
        String[] columns = new String[]{"Name","Motifs"};
        Set<String> collectionNames=engine.getPredefinedMotifCollections();        
        Object[][] predefinedcollectionstable = new Object[collectionNames.size()][2];
        int i=0;
        for (String name:collectionNames) {
            predefinedcollectionstable[i][0]=name;
            predefinedcollectionstable[i][1]=new Integer(engine.getSizeForMotifCollection(name));
            i++;
        }
        
        predefinedCollectionsTableModel=new DefaultTableModel(predefinedcollectionstable,columns){
            @Override
            public Class getColumnClass(int col) {
                if (col==1) return Integer.class;
                else  return String.class;
            }
        };
        predefinedCollectionsTable=new JTable(predefinedCollectionsTableModel) {
            @Override
            public boolean isCellEditable(int row,int col) {return false;}
        };
        predefinedCollectionsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        predefinedCollectionsTable.getColumn("Motifs").setMinWidth(65);
        predefinedCollectionsTable.getColumn("Motifs").setMaxWidth(65);
        predefinedCollectionsTable.getColumn("Motifs").setWidth(65);
        predefinedCollectionsPanel.setLayout(new BorderLayout());
        scrollpane=new JScrollPane(predefinedCollectionsTable);
        predefinedCollectionsTable.setFillsViewportHeight(true);
        predefinedCollectionsTable.setAutoCreateRowSorter(true);
        predefinedCollectionsTable.getTableHeader().setReorderingAllowed(false);
        predefinedCollectionsPanel.add(scrollpane,BorderLayout.CENTER);
        JPanel statuspanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
        predefinedPanelErrorLabel=new JLabel("  ");
        predefinedPanelErrorLabel.setFont(new Font(Font.SANS_SERIF,Font.BOLD,12));
        predefinedPanelErrorLabel.setForeground(java.awt.Color.RED);
        statuspanel.add(predefinedPanelErrorLabel);
        predefinedCollectionsPanel.add(statuspanel,BorderLayout.SOUTH);
        predefinedCollectionsTable.getColumn("Motifs").setCellRenderer(new CellRenderer_RightAlign());
        predefinedCollectionsTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (showExisting) return;
                int row=predefinedCollectionsTable.getSelectedRow();
                if (row>=0) {
                  String trackname=(String)predefinedCollectionsTable.getValueAt(row, predefinedCollectionsTable.getColumn("Name").getModelIndex());
                  setDataItemName(trackname.replace(" ", "_"));
                } else setDataItemName(data.getName());
            }
        });
        predefinedCollectionsTable.addMouseListener(new MouseAdapter() {
                @Override
                public void mouseClicked(MouseEvent e) {
                    if (e.getClickCount()==2) clickOK();
                 }
        });
        predefinedCollectionsTable.getRowSorter().toggleSortOrder(0);
    }
    
    
    private void setupImportModelPanel() {
        ArrayList<DataFormat> dataformats=engine.getDataInputFormats(MotifCollection.class);
        //dataformats.remove(engine.getDataFormat("Plain"));
        importCollectionPanel=new LoadFromFilePanel(dataformats,gui,MotifCollection.class);
    }
    
    private void setupParseListPanel() {
        parseListPanel=new JPanel();     
        parseListPanel.setLayout(new BoxLayout(parseListPanel, BoxLayout.Y_AXIS));  
        JPanel internal1=new JPanel(new FlowLayout(FlowLayout.LEADING));        
        JPanel internal2=new JPanel(new BorderLayout());  
        JLabel label=new JLabel("<html>Enter names of motifs or collections below (separated by commas, semicolons or whitespace)<br>Prefix name with <b>&mdash;</b> (minus) to subtract from selection or <b>&</b> to intersect</html>");
        parseListTextArea=new JTextArea();
        internal1.add(label);
        internal2.add(parseListTextArea);
        internal1.setBorder(BorderFactory.createEmptyBorder(10, 20, 5, 20));
        internal2.setBorder(BorderFactory.createEmptyBorder(0, 20, 20, 20));
        parseListPanel.add(internal1);
        parseListPanel.add(internal2);
    }  
    
    private void setupFromTrackPanel() {
        fromTrackPanel=new JPanel();  
        fromTrackPanel.setLayout(new FlowLayout(FlowLayout.LEADING));       
        JPanel internal=new JPanel();
        internal.setLayout(new GridBagLayout());
        internal.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
        GridBagConstraints constraints=new GridBagConstraints();  
        constraints.anchor=GridBagConstraints.WEST;
        constraints.fill=GridBagConstraints.NONE;
        constraints.gridheight=GridBagConstraints.RELATIVE; 
        constraints.ipadx=10;
        constraints.ipady=10;
        JLabel label1=new JLabel("Motif track");
        JLabel label2=new JLabel("Sequence support");
        DefaultComboBoxModel tracksmodel=new DefaultComboBoxModel();
        ArrayList<Data> regionsets=engine.getAllDataItemsOfType(RegionDataset.class);
        for (Data regiondata:regionsets) {
            if (((RegionDataset)regiondata).isMotifTrack()) tracksmodel.addElement(regiondata.getName());
        }
        fromTrackSelection=new JComboBox(tracksmodel);
        fromTrackSelection.setPreferredSize(new Dimension(16,20));
        sequencesupportoperator=new JComboBox(new String[]{"At least","Exactly"});        
        sequencesupportoperator.setPreferredSize(new Dimension(76,20));      
        sequencesupportpercentageoperator=new JComboBox(new String[]{"","%"});  
        sequencesupportpercentageoperator.setPreferredSize(new Dimension(40,20));
        sequencesupportnumber=new JTextField(3);
        sequencesupportnumber.setText("1");
        sequencesupportnumber.setPreferredSize(new Dimension(40,20));   
        sequencesupportnumber.setHorizontalAlignment(JTextField.RIGHT);     
        constraints.gridy=0;
        constraints.gridx=0;
        constraints.insets=new Insets(5, 0, 5, 0);
        internal.add(label1,constraints);
        constraints.gridx=1;
        constraints.fill=GridBagConstraints.HORIZONTAL;        
        constraints.gridwidth=GridBagConstraints.REMAINDER;        
        constraints.insets=new Insets(5, 0, 5, 0);
        internal.add(fromTrackSelection,constraints);
        constraints.fill=GridBagConstraints.NONE;      
        constraints.gridwidth=1;         
        constraints.gridy=1;
        constraints.gridx=0;
        constraints.insets=new Insets(5, 0, 5, 0);
        internal.add(label2,constraints);
        constraints.gridx=1;
        constraints.insets=new Insets(5, 0, 5, 0);
        internal.add(sequencesupportoperator,constraints);        
        constraints.gridx=2;
        constraints.insets=new Insets(5, 5, 5, 0);
        internal.add(sequencesupportnumber,constraints);           
        constraints.gridx=3;
        constraints.insets=new Insets(5, 0, 5, 0);
        internal.add(sequencesupportpercentageoperator,constraints);           
        fromTrackPanel.add(internal);
    }        
    
    @Override
    public boolean onOKPressed() {
        if (tabbedPanel.getSelectedComponent()==predefinedCollectionsPanel) {
             int row=predefinedCollectionsTable.getSelectedRow();
             if (row<0) return false;
             String collectionname=(String)predefinedCollectionsTable.getValueAt(row, 0);
             data.setPredefinedCollectionName(collectionname);
             if (showExisting) {
                 try {
                     data=importPredefinedCollection(data); // replace already existing collection. This will not result in "new" operation
                 } catch (Exception e) {
                      //e.printStackTrace(System.err);
                      reportPredefinedCollectionlError(e.getMessage());
                      return false;                    
                 }
             }
        } else if (tabbedPanel.getSelectedComponent()==importCollectionPanel) { 
            try {
                String filename=importCollectionPanel.getFilename();
                if (filename==null) throw new SystemError("Missing filename");
                DataFormat format=importCollectionPanel.getDataFormat();
                ParameterSettings settings=importCollectionPanel.getParameterSettings();
                data=(MotifCollection)importCollectionPanel.loadData(data,MotifCollection.getType());
                isimported=true;
                data.setPredefinedCollectionName(null);
                setImportFromFileSettings(filename, (format!=null)?format.getName():null, settings);
            } catch (Exception e) {
                JOptionPane.showMessageDialog(this, "An error occurred while importing collection from file:\n"+e.getClass().getSimpleName()+":"+e.getMessage(),"Import Error",JOptionPane.ERROR_MESSAGE);
                return false;
            }
        } else if (tabbedPanel.getSelectedComponent()==manualSelectionPanel) {
            manualSelectionPanel.updateMotifCollection();           
        } else if (tabbedPanel.getSelectedComponent()==fromTrackPanel) {
            String trackName=(String)fromTrackSelection.getSelectedItem();
            if (trackName!=null && !trackName.isEmpty()) {
                Data track=engine.getDataItem(trackName);
                boolean percentage=false;
                if (track instanceof RegionDataset) {
                   String selectionoperation=(String)sequencesupportoperator.getSelectedItem(); 
                   String percentagestring=(String)sequencesupportpercentageoperator.getSelectedItem(); 
                   if (percentagestring!=null && percentagestring.equals("%")) percentage=true;
                   boolean exactly=selectionoperation.equalsIgnoreCase("exactly");
                   int quorum=0;
                   try {quorum=Integer.parseInt(sequencesupportnumber.getText());} catch(NumberFormatException e) {}
                   data.createCollectionFromTrack((RegionDataset)track, quorum, exactly, percentage, engine);
                }                    
            }      
        } else if (tabbedPanel.getSelectedComponent()==parseListPanel) {
            data.setPredefinedCollectionName(null);            
            data.clearAll(engine);
            ArrayList<String> notfound=new ArrayList<String>();
            String text=parseListTextArea.getText();
            text=text.replace(",", " ");
            text=text.replace(";", " ");
            String[] elements=text.split("\\s+");
            for (int i=0;i<elements.length;i++) {  
               String motifname=elements[i];                
               int mode=UNION;
               if (motifname.startsWith("+") && motifname.length()>1) {mode=UNION;motifname=motifname.substring(1);}
               if (motifname.startsWith("-") && motifname.length()>1) {mode=SUBTRACTION;motifname=motifname.substring(1);}
               if (motifname.startsWith("&") && motifname.length()>1) {mode=INTERSECTION;motifname=motifname.substring(1);}
               Data dataobject=engine.getDataItem(motifname);
               if (dataobject==null) notfound.add(motifname);
               else if (dataobject instanceof Motif) {
                   if (mode==UNION) data.addMotif((Motif)dataobject);
                   else if (mode==SUBTRACTION) data.removeMotif((Motif)dataobject);
                   else if (mode==INTERSECTION) {
                       boolean hasAlready=data.contains((Motif)dataobject);
                       data.clearAll(engine);
                       if (hasAlready) data.addMotif((Motif)dataobject);
                   }
               } else if (dataobject instanceof MotifCollection) {
                   if (mode==UNION) data.merge((MotifCollection)dataobject);
                   else if (mode==SUBTRACTION) data.subtract((MotifCollection)dataobject);
                   else if (mode==INTERSECTION) data.intersect((MotifCollection)dataobject);                 
               } else notfound.add(motifname);
            }  
            if (notfound.size()>15) {
                String errormsg=notfound.size()+" motifs (or collections) were not found:\n\n";
                JOptionPane.showMessageDialog(this, errormsg, "Unknown motifs/collections", JOptionPane.WARNING_MESSAGE);                
            } else if (notfound.size()>0) {
                String errormsg="The following "+notfound.size()+" motif"+((notfound.size()==1)?"s":"")+" were not found:\n\n";
                for (int i=0;i<notfound.size()-1;i++) {                
                     errormsg+=notfound.get(i)+"\n";
                } 
                errormsg+=notfound.get(notfound.size()-1);
                JOptionPane.showMessageDialog(this, errormsg, "Unknown motifs", JOptionPane.WARNING_MESSAGE);
            }
        }
        String newName=getDataItemName();
        if (!data.getName().equals(newName)) data.rename(newName);
        return true;
    }
    
    @Override
    public Data getData() {
       return data; 
    }
    
    public boolean isImported() {
        return isimported;
    }
    

    private void reportPredefinedCollectionlError(String msg) {
        if (msg==null) msg="NULL";
        predefinedPanelErrorLabel.setText(msg);
    }
    

    
    private MotifCollection importPredefinedCollection(MotifCollection data) throws ExecutionError {
            String collectionName=data.getPredefinedCollectionName();
            if (collectionName==null) throw new ExecutionError("SYSTEM ERROR: No predefined collection defined");
            String filename=engine.getFilenameForMotifCollection(collectionName);
            String formatname="PriorsEditorMotif";
            if (filename==null) throw new ExecutionError("Unknown Motif Collection: "+collectionName);
            BufferedReader inputStream=null;
            ArrayList<String> input=new ArrayList<String>();
            try {
                //inputStream=new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream("/priorseditor/engine/resources/"+filename)));
                inputStream=new BufferedReader(new InputStreamReader(new FileInputStream(new File(filename))));
                String line;
                while((line=inputStream.readLine())!=null) {input.add(line);}
            } catch (IOException e) { 
                throw new ExecutionError("An error occurred when loading predefined Motif Collection:\n["+e.getClass().getSimpleName()+"] "+e.getMessage(),0);
            } finally {
                try {
                    if (inputStream!=null) inputStream.close();
                } catch (IOException ioe) {System.err.println("SYSTEM ERROR: An error occurred when closing BufferedReader in Prompt_MotifCollection.importPredefinedCollection(): "+ioe.getMessage());}
            }           
            DataFormat format = engine.getDataFormat(formatname);
            if (format==null) throw new ExecutionError("Unknown Dataformat: "+formatname);    
            try {data=(MotifCollection)format.parseInput(input, data,null,null);}
            catch (Exception e) {throw new ExecutionError(e.getClass().getSimpleName()+":"+e.getMessage());} 
            data.setPredefinedCollectionName(collectionName);
            return data;        
    }
    
private class CellRenderer_RightAlign extends DefaultTableCellRenderer {
    public CellRenderer_RightAlign() {
           super();
           this.setHorizontalAlignment(DefaultTableCellRenderer.RIGHT);              
    }
    @Override
    public void setValue(Object value) {
           if (value!=null)setText(value.toString());
           else setText("");
       }
}// end class CellRenderer_RightAlign     

}
