/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.prompt;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.HashMap;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import priorseditor.engine.data.*;

import priorseditor.gui.PriorsEditorGUI;


/**
 *
 * @author kjetikl
 */
public class Prompt_NumericMapVariable extends Prompt {
    
    private NumericMapVariable data;
    private JPanel mainpanel;
    private HashMap<String,JTextField> textfields; // the key is the sequence name (or *default*)
    private JLabel messageLabel;
      
    
    public Prompt_NumericMapVariable(PriorsEditorGUI gui, String prompt, NumericMapVariable dataitem) {
        super(gui,prompt);
        if (dataitem!=null) data=dataitem;
        else data=new NumericMapVariable(gui.getGenericDataitemName(NumericMapVariable.class, null), 0);
        setDataItemName(data.getName());
        setTitle("Numeric Map Variable");
        SequenceCollection dataset=engine.getDefaultSequenceCollection();
        int size=dataset.getNumberofSequences();
        textfields=new HashMap<String,JTextField>(size+1);
        mainpanel=new JPanel();
        mainpanel.setLayout(new GridBagLayout());        
        GridBagConstraints constraints = new GridBagConstraints();        
        constraints.anchor=GridBagConstraints.LINE_START;
        constraints.ipadx=10;
        constraints.ipady=5;      
        constraints.fill=GridBagConstraints.HORIZONTAL;
        for (int i=0;i<size;i++) {
                Sequence sequence=(Sequence)dataset.getSequenceByIndex(i,engine);
                String sequenceName=sequence.getName();
                JLabel sequenceNameLabel=new JLabel(sequenceName);
                if (dataitem.hasValueForSequence(sequenceName)) sequenceNameLabel.setForeground(java.awt.Color.BLACK);
                else sequenceNameLabel.setForeground(java.awt.Color.GRAY);
                constraints.gridx=0;
                constraints.gridy=i;
                mainpanel.add(sequenceNameLabel,constraints);
                JTextField textfield=new JTextField();
                textfields.put(sequenceName, textfield);
                textfield.setText(dataitem.getValue(sequenceName).toString());
                constraints.gridx=1;
                mainpanel.add(textfield,constraints);                
        }
        JLabel sequenceNameLabel=new JLabel("Default");
        sequenceNameLabel.setForeground(java.awt.Color.RED);
        constraints.gridx=0;
        constraints.gridy=size;
        mainpanel.add(sequenceNameLabel,constraints);
        JTextField textfield=new JTextField();
        textfields.put("*default*", textfield);
        textfield.setText(dataitem.getValue().toString());
        constraints.gridx=1;
        mainpanel.add(textfield,constraints);        
        JPanel internalPanel=new JPanel();
        internalPanel.setLayout(new BoxLayout(internalPanel, BoxLayout.Y_AXIS));
        messageLabel=new JLabel("  ");
        messageLabel.setFont(new Font(Font.SANS_SERIF,Font.BOLD,12));
        messageLabel.setForeground(java.awt.Color.RED);
        JScrollPane scrollPane=new JScrollPane(mainpanel); // just in case
        internalPanel.add(scrollPane);
        internalPanel.add(messageLabel);     
        Dimension dim=new Dimension(250,300);
        mainpanel.setMinimumSize(dim);
        mainpanel.setPreferredSize(dim);
        setMainPanel(internalPanel);    
        pack();
    }
    
    
    @Override
    public boolean onOKPressed() {
        try {
            for (String sequenceName:textfields.keySet()) {
                JTextField textfield=textfields.get(sequenceName);
                float value=Float.parseFloat(textfield.getText().trim());
                if (sequenceName.equals("*default*")) data.setDefaultValue(value);
                else data.setValue(sequenceName,value);
            }            
        } catch (NumberFormatException e) {
            messageLabel.setText("Please enter only numeric values");
            return false;
        }
        String newName=getDataItemName();
        if (!data.getName().equals(newName)) data.rename(newName);        
        return true;
    }
    
    @Override
    public Data getData() {
       return data; 
    }
    

}
