/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package priorseditor.gui.prompt;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import priorseditor.engine.data.*;
import priorseditor.engine.data.Sequence;
import priorseditor.gui.MiscIcons;
import priorseditor.gui.PriorsEditorGUI;

/**
 *
 * @author kjetikl
 */
public class Prompt_SequenceCollection extends Prompt {
    
    private JList sequenceList;
    private JList leftOutSequenceList;
    private DefaultListModel sequenceListModel;
    private DefaultListModel leftOutSequenceListModel;
    private SequenceCollection collection;
    private JPanel mainpanel;
    private JButton addButton;
    private JButton removeButton;
    private MiscIcons leftArrowIcon;
    private MiscIcons rightArrowIcon;
    private JLabel inCollectionLabel;
    private JLabel remainingLabel;
    
    
    
    
    public Prompt_SequenceCollection(PriorsEditorGUI gui, String prompt, SequenceCollection seqCollection) {
        super(gui,prompt);
        if (seqCollection!=null) collection=seqCollection;
        else collection=new SequenceCollection(gui.getGenericDataitemName(SequenceCollection.class, null));
        setDataItemName(collection.getName());
        setTitle("Sequence Collection");
        ArrayList<String> list=engine.getDefaultSequenceCollection().getAllSequenceNames();

        sequenceListModel = new DefaultListModel();
        sequenceList=new JList(sequenceListModel);
        leftOutSequenceListModel = new DefaultListModel();
        leftOutSequenceList=new JList(leftOutSequenceListModel);
        for (String name:list) {
            if (collection.contains(name)) sequenceListModel.addElement(name);
            else leftOutSequenceListModel.addElement(name);
        }
        
        JScrollPane scrollpaneLeft=new JScrollPane(sequenceList);
        Dimension size=new Dimension(220,300);
        scrollpaneLeft.setPreferredSize(size);
        JPanel leftPanel=new JPanel(new BorderLayout());
        inCollectionLabel=new JLabel("   In collection");
        leftPanel.add(inCollectionLabel,BorderLayout.NORTH);
        leftPanel.add(scrollpaneLeft,BorderLayout.CENTER);
        
        JScrollPane scrollpaneRight=new JScrollPane(leftOutSequenceList);
        scrollpaneRight.setPreferredSize(size);
        JPanel rightPanel=new JPanel(new BorderLayout());
        remainingLabel=new JLabel("   Remaining");
        rightPanel.add(remainingLabel,BorderLayout.NORTH);
        rightPanel.add(scrollpaneRight,BorderLayout.CENTER);
        
        JPanel buttonsPanel=new JPanel();
        buttonsPanel.setLayout(new BoxLayout(buttonsPanel, BoxLayout.Y_AXIS));
        leftArrowIcon=new MiscIcons(MiscIcons.LEFT_ARROW);
        rightArrowIcon=new MiscIcons(MiscIcons.RIGHT_ARROW);     
        addButton=new JButton(leftArrowIcon);
        removeButton=new JButton(rightArrowIcon);
        Dimension buttonssize=new Dimension(40,28);
        addButton.setMaximumSize(buttonssize);
        addButton.setMinimumSize(buttonssize);
        addButton.setPreferredSize(buttonssize);
        removeButton.setMaximumSize(buttonssize);
        removeButton.setMinimumSize(buttonssize);
        removeButton.setPreferredSize(buttonssize);
        addButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                addSequences();
            }
        });
        removeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                removeSequences();
            }
        });
        buttonsPanel.add(addButton);
        buttonsPanel.add(new JLabel(" "));
        buttonsPanel.add(removeButton);
        
        mainpanel=new JPanel();
        mainpanel.setLayout(new BoxLayout(mainpanel, BoxLayout.X_AXIS));
        mainpanel.add(leftPanel);
        mainpanel.add(buttonsPanel);
        mainpanel.add(rightPanel);
        if (collection.getName().equals(engine.getDefaultSequenceCollectionName())) {
            addButton.setEnabled(false);
            removeButton.setEnabled(false);
            leftArrowIcon.setForegroundColor(java.awt.Color.gray);
            rightArrowIcon.setForegroundColor(java.awt.Color.gray); 
        }
        setMainPanel(mainpanel);    
        updateLabels();
        pack();
    }
    
    
    @Override
    public boolean onOKPressed() {
        if (collection.getName().equals(engine.getDefaultSequenceCollectionName())) return true;
        ArrayList<String> list=engine.getDefaultSequenceCollection().getAllSequenceNames();
        for (int i=0;i<list.size();i++) {
            String sequenceName=list.get(i);
            if (sequenceListModel.contains(sequenceName) && !collection.contains(sequenceName)) {
                 Data seq=engine.getDataItem(sequenceName);
                 if (seq!=null && seq instanceof Sequence) collection.addSequence((Sequence)seq);                
            } else if (!sequenceListModel.contains(sequenceName) && collection.contains(sequenceName)) {
                 Data seq=engine.getDataItem(sequenceName);
                 if (seq!=null && seq instanceof Sequence) collection.removeSequence((Sequence)seq);                                 
            } 
        }
        String newName=getDataItemName();
        if (!collection.getName().equals(newName)) collection.rename(newName);        
        return true;
    }
    
    @Override
    public Data getData() {
       return collection; 
    }
    
    
    private void addSequences() {
        Object[] selection=leftOutSequenceList.getSelectedValues();
        for (Object selected:selection) {
            sequenceListModel.addElement(selected);
            leftOutSequenceListModel.removeElement(selected);
        }
        updateLabels();
    }
    
    private void removeSequences() {
        Object[] selection=sequenceList.getSelectedValues();
        for (Object selected:selection) {
            leftOutSequenceListModel.addElement(selected);
            sequenceListModel.removeElement(selected);
        }     
        updateLabels();
    }
    
    private void updateLabels() {
        int colSize=sequenceListModel.size();
        int remSize=leftOutSequenceListModel.size();
        inCollectionLabel.setText("   In collection ("+colSize+")");
        remainingLabel.setText("   Remaining ("+remSize+")");        
    }
}
